package com.ibm.appsure;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      AppSure
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.appsure.common.* ;
import com.ibm.appsure.db.* ;
/**
 * The AppSure object is the API to the AppSure system. This
 * API provides a mechanism for Application Clients to register
 * themselves with AppSure, notify AppSure of failures and
 * to clean up themselves in the AppSure database.<br><br>
 * 
 * This high level API was designed to provide a mechanism for
 * an application to Initialize, Process and Shutdown.
 *
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* AppSure
/*
/*----------------------------------------------------*/
public class AppSure {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    /**
      *  This is the connection to DB/2 that the AppSure will 
      *  use throughout the life of it's instance.
      **/
    private Connection dbConn     = null ;

    /**
      * This is the unique application that this object will
      * use throughout the life
      **/
    private Application application = null ;

    /**
      * This is the process Index for the current running application
      **/
    private int processIndex = 0 ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Public Members                                          */
    /*                                                           */
    /*-----------------------------------------------------------*/



    /**
      *  This method will initialize an application in the batch
      * system. This API does a check to make sure that this application
      * can run by looking up if other instances of this application
      * are running. If there are other instances of this application
      * running and the 'multipleInst' property of this application are
      * FALSE, then this method will return FALSE since the application
      * being initialized cannot have multiple instances running.
      *
      * @param conn     The JDBC Connection to the AppSure Database
      * @param appCode  This is the app code which is unique to an application.
      *
      *              
      **/
    public boolean startApp (Connection conn, String appCode) throws AppSureException {
        boolean rc = false ;

        this.dbConn  = conn ;

        /*--------------------------------------------*/
        /*                                            */
        /*  Only process this if we have a connection */
        /*                                            */
        /*--------------------------------------------*/
        if ( dbConn != null ) {

            /*-----------------------------------------------*/
            /*                                               */
            /*  Create the Application object and read it in */
            /*                                               */
            /*-----------------------------------------------*/
            application = new Application ( appCode ) ;
            if ( application.readFromDatabase ( conn ) ) {

                /*----------------------------------------------------*/
                /*                                                    */
                /*  If this application cannot run mutiple instances, */
                /* then make sure it is not currently running         */
                /*                                                    */
                /*----------------------------------------------------*/
                if ( !application.getMultipleInst() )
                    rc = !isAppRunning ( appCode ) ;
                else
                    rc = true ;


                /*---------------------------------------*/
                /*                                       */
                /*  Now, mark the application as running */
                /*                                       */
                /*---------------------------------------*/
                if ( rc ) {
                    processIndex = AppSureSQL.logApplicationRunning ( dbConn, application.getAppCode() ) ;
                    if ( processIndex == -1 ) {
                        throw new AppSureException ( "Cannot create application process" ) ;
                    }
                }

            } else
               throw new AppSureException ( "Could not read application ("+appCode+") from AppSure Database" ) ;
        } else
           throw new AppSureException ( "Could not connect to NULL database connection" ) ;

        return( rc ) ;
    }


    /**
      *  This method will determine if an application is running. This method
      * will return TRUE if the application passed is currently running, FALSE
      * if the application is not running.
      *
      * @param appCode The application code to check for
      **/
    public boolean isAppRunning ( String appCode ) throws AppSureException  {
        return( AppSureSQL.isAppRunning ( dbConn, appCode ) ) ;
    }

    /**
      *  This method will return TRUE if the application passed had a succesfull
      * run the previous time. A succesful run is determined by looking at the
      * failures from the last run. If the last failure recorded has not been
      * looked at by support yet then this method will return FALSE. Once support
      * has identified a problem and closed it, this method will then return TRUE
      *
      * @param appCode The AppCode of the application to check
      ***/
    public boolean didAppRun ( String appCode ) throws AppSureException {
        return( AppSureSQL.getLastFailure ( dbConn, appCode ).isSevZero() ) ;
    }


    /**
      *  This method will shutdown an application in the AppSure system. The
      *  shutdown procedure includes stopping the Application in the
      *  AppSure system and saving the log file into the AppSure system. If there
      *  has been an Application Failure, please be sure to use the 'setFailure'
      *  method prior to calling this function.<br><br>
      *
      *  The LogFilename and LogFileContents paramters can be used to send the entire
      *  log file to the AppSure system. If the LogFilename is given, then that log
      *  file will be used. If the LogFilename is NULL then the LogFileContents paramteter
      *  which holds the entire log file will be sent to the AppSure system. If both
      *  parameters are NULL, then no log file will be sent. It is more efficient to
      *  save the LogFilename to the local file system than to keep it in memory, especially
      *  when dealing with large log files.
      *
      *  @param appCode The Application Code unique to this application
      *  @param logFilename If a log file is saved to the local system, this parameter
      *                     can be used to send the log file
      *  @param logFileContents If a log file is not saved to the local system, this parameter
      *                     can be used to send the log file.
      **/
    public boolean shutDown (String logFilename, String logFileContents) throws AppSureException {
        boolean rc       = false ;
        int     logIndex = -1 ;                                  


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ok, now mark the application as not running             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        AppSureSQL.logApplicationStopped ( dbConn, processIndex, application.getAppCode() ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Now, create the 'Zero' Failure                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        rc = logFailure ( 0, logFilename, logFileContents ) ;

        return( rc ) ;
    }



    /**
      *  This method notifies the AppSure system of a failure. The Failure Type passed
      * to this method must be a known Failure Type in the AppSure system. <br><br>
      *
      * The LogFilename and LogFileContents paramters can be used to send the entire
      * log file to the AppSure system. If the LogFilename is given, then that log
      * file will be used. If the LogFilename is NULL then the LogFileContents paramteter
      * which holds the entire log file will be sent to the AppSure system. If both
      * parameters are NULL, then no log file will be sent. It is more efficient to
      * save the LogFilename to the local file system than to keep it in memory, especially
      * when dealing with large log files.
      *
      * @param appCode The Application Code unique to this application
      * @param failureType The FailureType as defined in the AppSure System.
      * @param logFilename If a log file is saved to the local system, this parameter
      *                    can be used to send the log file
      * @param logFileContents If a log file is not saved to the local system, this parameter
      *                    can be used to send the log file.
      **/
    public boolean logFailure ( int    failureType,
                                String logFilename,
                                String logFileContents ) throws AppSureException {
        int     logIndex = -1 ;
        boolean logged   = false ;


        logFileContents = obtainLogContents ( logFilename, logFileContents ) ;

        logIndex = AppSureSQL.createLog ( dbConn, application.getAppCode(), logFileContents ) ;                     

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Now, create the 'Zero' Failure                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( logIndex > 0 ) {
            logged = AppSureSQL.createFailure ( dbConn, application.getAppCode(), failureType, logIndex ) ;
        }

        return( logged ) ;

    } /* logFailure  */




    /**
      *  This method will try to obtain the log file contents based on the
      * input received. If the LogFIleName is null, then the logFileContents
      * are checked. This method will not return unless valid contents can
      * be obtained
      **/
    private String obtainLogContents ( String logFilename, String logFileContents ) throws AppSureException {
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First, try to read in the log file contents             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( logFilename != null ) {
            try {
                logFileContents = Util.readFile ( logFilename ) ;

            } catch ( Exception e ) {
                e.printStackTrace() ;
                throw new AppSureException ( "Log File Contents could not be read from file "+logFilename ) ;
            } /* catch  */
        } /* if  */

        if ( logFileContents == null )
            throw new AppSureException ( "Log File Contents could not be read from file "+logFilename ) ;
        return( logFileContents ) ;
    }
} /* AppSure */
