package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   MaskDocument                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MaskDocument                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import javax.swing.text.* ;
import javax.swing.event.* ;
import java.awt.event.* ;
import com.ibm.appsure.app.shared.util.* ;
/**
 * This Document subclasses PlainDocument so that Documents may
 * format characters being inserted to a format of the following:
 *
 * <pre>
 * MASKTYPE_UPPER    = Format all upper case<br>
 * MASKTYPE_LOWER    = Format all lower case<br>
 * MASKTYPE_NUMERIC  = Format all numeric only, do not allow alphas<br>
 * MASKTYPE_DATE     = Format for a date. You must specify the date format in<br>
 *                     the mask string. For example: 99/99/9999, or 9999/99/99 etc..<br>
 *                     This mask will auto-format 4 digit years.<br>
 * MASKTYPE_STRING   = Use a given mask to enter fields. valid mask characters<br>
 *                     are as follows:<br>
 *                     CHAR   DESCRIPT<br>
 *                     ------ ---------------------------------------<br>
 *                     9      Allow only a number in that character position<br>
 *                     X      Allow either number or alpha<br>
 *                     A      Allow only alpha.<br>
 *                     U      Allow only Upper Case alpha<br>
 *                     V      Allow either number of alpha, format for upper case<br>
 * <br>
 *                     An example of a MASKTYPE_STRING phone number would be:<br>
 *                     (999)999-9999<br>
 * <br>
 * </pre>
 * NOTE! All MaskDocuments are reusable except for the MASKTYPE_DATE and MASKTYPE_STRING.
 *       You must pass a valid TextComponent for a string or date mask to work.<br><br>
 *
 * NOTE! Please read the warnings when using setMinimumOffset
 *
 * @version 1.0, 12/7/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   MaskDocument                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class MaskDocument extends PlainDocument implements FocusListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int BLANKCHAR             = 250 ;
   public  static final int MASKTYPE_UNKNOWN      = 0 ;
   public  static final int MASKTYPE_UPPER        = 1 ;
   public  static final int MASKTYPE_LOWER        = 2 ;
   public  static final int MASKTYPE_NUMERIC      = 3 ;
   public  static final int MASKTYPE_STRING       = 4 ;
   public  static final int MASKTYPE_DATE         = 5 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextComponent textComponent       = null ;
   private int            maxTextLen          = 1024 ;
   private int            maskType            = MASKTYPE_UNKNOWN ;
   private String         mask                = null ;
   private char[]         maskArray           = null ;
   private char[]         maskDisplayArray    = null ;
   private int            dateType            = CDate.DATE_DATE ;
   private int            minimumOffset       = 0 ;


   /*-----------------------------------------------------------*/
   /* MaskDocument
    */
   /**
    *    This will construct a MaskDocument with
    *  a string mask.
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public MaskDocument ( int maskType, String mask,  JTextComponent textComponent ) throws NullPointerException {

      if ( ( mask != null ) && 
           ( mask.length() > 0 ) && 
           ( textComponent != null ) ) {

         this.maskType = maskType ;

         setMaxTextLength ( mask.length() ) ;
         setStringMask ( mask ) ;
         setTextComponent ( textComponent ) ;

      } /* if */

   }


   /*-----------------------------------------------------------*/
   /* MaskDocument
    */
   /**
    *    This will construct a mask with the specific
    *  mask type
    *  
    *             
    *  @param maskType The Mask Type ( MASKTYPE_UPPER, MASKTYPE_LOWER etc.. )
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public MaskDocument ( int maskType ) {
      this.maskType = maskType ;
   }



   /*-----------------------------------------------------------*/
   /* MaskDocument
    */
   /**
    *    This will construct a mask with the specific
    *  mask type and max text length
    *  
    *             
    *  @param maskType The Mask Type ( MASKTYPE_UPPER, MASKTYPE_LOWER etc.. )
    *  @param maximum allowed text length
    *  @param 
    *
    *  @return 
    */
   public MaskDocument ( int maskType, int maxTextLength ) {

      /* If they tried to fool our mask operations,
         set it to unknown */
      if ( ( maskType == MASKTYPE_STRING ) ||
           ( maskType == MASKTYPE_DATE ) )
         maskType = MASKTYPE_UNKNOWN ;

      this.maskType = maskType ;
      setMaxTextLength ( maxTextLength ) ;
   }



   /*------------------------------------------------------------*/
   /* setMinimumOffset
    */
   /**
    *    This method will set the minimum offset at
    *  which the user can start typing. Any attempt to enter
    *  data before that offset will immediatly jump the cursor to the minimum offset<br><br>
    *
    *  Please note! When using a minimum offset, please make sure there is
    *  data in the TextComponent that is at least the length of the minimum offset,
    *  otherwise when you try to insert the data, an exception will be thrown.
    *
    *  @param minimumOffset The minimum offset to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMinimumOffset ( int minimumOffset ) {
      this.minimumOffset = minimumOffset ;
   }




   /*------------------------------------------------------------*/
   /* setMaxTextLength
    */
   /**
    *    This method will set the max number of
    *  characters allowed to be entered in this
    *  document
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMaxTextLength ( int maxTextLen ) {
      this.maxTextLen = maxTextLen ;
   }




   /*-----------------------------------------------------------*/
   /* insertString
    */
   /**
    *    This method will be called when we need to
    *  insert a string into this document. We will use
    *  this method to force a string mask, and a normal format
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void insertString ( int offs, String str, AttributeSet a) {

      if ( str != null ) {
         try {
            int     strLength = str.length() ;
            boolean callSuper = true ;

            switch ( maskType ) {
               case MASKTYPE_UPPER :
                  str = str.toUpperCase() ;
                  break ;

               case MASKTYPE_LOWER :
                  str = str.toLowerCase() ;
                  break ;

               case MASKTYPE_NUMERIC :
                  try {
                     int number = new Integer ( str ).intValue() ;

                  } catch ( NumberFormatException e ) {
                     callSuper = false ;
                     str="" ;
                  }
                  break ;

               case MASKTYPE_DATE :
               case MASKTYPE_STRING :
                  formatCharForMask ( offs, str, a ) ;
                  callSuper = false ;
                  break ;

            }


            /* Ok, make sure that we are not passed the
               max text length */
            if ( callSuper ) {

               if ( (getLength()+strLength) > maxTextLen ) {

                  /* Ok, if the user pasted text into the
                     field, strip the extra text out */
                  if ( strLength > 1 )
                     str = str.substring ( 0, (maxTextLen-getLength()) ) ;
                  else
                     callSuper = false ;
               }
            }

            if ( callSuper ) {

               super.insertString ( offs, str, a ) ;
            }

         } catch ( BadLocationException e ) {
            System.out.println ( "Bad Location:" ) ;
            e.printStackTrace() ;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    This method will remove characters from
    *  our mask. If we do not have a string mask,
    *  we will default to the super.remove
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void remove ( int offSet, int len ) {
      try {

         /* If this is a masked string, we have
            to remove the characters carefully so
            we do not alter the mask */
         if ( ( isMaskString() ) && ( len != 0 ) ) {
            String maskDisplayString = null ;
            char   blankChars[]      = new char[len] ;
            int    i                 = 0 ;

            while ( i < len )
               blankChars[i++] = (char)BLANKCHAR ;

            formatCharForMask ( offSet, new String ( blankChars ), null ) ;

            /* Now, to move the cursor back to the beginning of offSet */
            textComponent.setCaretPosition ( offSet ) ;

         } else
            super.remove ( offSet, len ) ;

      } catch ( BadLocationException e ) {
      }

   } /* remove */

   /*-----------------------------------------------------------*/
   /* formatCharForMask
    */
   /**
    *    This method will format a character
    *  to fit in the current string mask
    *  
    *             
    *  @param offSet The off set of the character to insert
    *  @param str The String representation of the character
    *  @param 
    *
    *  @return 
    */
   private void formatCharForMask ( int offSet, String str, AttributeSet attr ) {
      char newChar    = ' ';
      int  length     = str.length() ;
      int  index      = 0 ;
      int  thisLength = getLength() ;

      /* Loop through each character in the
         string and add it to our mask */
      while ( ( index < length ) && 
              ( offSet >= 0 ) && 
              ( offSet < maskDisplayArray.length ) ) {

         /* get the next char */
         newChar = str.charAt(index) ;

         /* If this new char is the same as this
            char in the mask, allow it to overwrite. This
            is in case a programmer calls setText on the
            text component we are tied to */
         if ( newChar != maskDisplayArray[offSet] ) {

            /* Check if the character is
               going to overwrite a non-mask character. */
            offSet = getNextMaskInsertPoint(offSet) ;                                                                                     

            /* Now, check to see if this character will fit
               in the mask here */
            if ( ( offSet >= 0 ) && 
                 ( doesCharFitMask ( newChar, maskArray[offSet] ) ) ) {

               /* Ok, the character fits, set it and call the super */
               if ( newChar == BLANKCHAR )
                  maskDisplayArray[offSet] = ' ' ;
               else {
                  if ( ( ( ( maskArray[offSet] == 'U' ) ||
                           maskArray[offSet] == 'V' ) ) &&
                       ( isAlpha(newChar) ) )
                     newChar = new String (""+newChar ).toUpperCase().charAt(0) ;
                  maskDisplayArray[offSet] = newChar ;
               }

            }
         }
         if ( offSet >= 0 )
            offSet++ ;
         index++ ;
      }

      try {
         super.remove ( 0, thisLength ) ;
         super.insertString ( 0, new String ( maskDisplayArray ), attr ) ;

         if ( (offSet+1) > thisLength )
            offSet = thisLength-1 ;

         if ( offSet < 0 )
            offSet = 0 ;

         textComponent.setCaretPosition ( offSet ) ;
      } catch ( BadLocationException e ) {
      }

   }


   /*-----------------------------------------------------------*/
   /* getNextMaskInsertPoint
    */
   /**
    *    This method will determine the next available
    *  place for a character to over write a string mask.
    *  
    *             
    *  @param offSet where to start looking from
    *  @param 
    *  @param 
    *
    *  @return The offSet that can be over written. -1 If no place can be found
    */
   private int getNextMaskInsertPoint ( int startOffset ) {
      int returnOffset = startOffset ;

      if ( mask != null ) {
         int maskLength = mask.length() ;

         /* First, check to see if we
            have gone over the edge */
         if ( startOffset >= maskLength )
            returnOffset = -1 ;

         else {
            while ( ( !isMaskChar ( mask.charAt(returnOffset) ) ) &&
                    ( returnOffset >= 0 ) ) {
               returnOffset++ ;

               if ( returnOffset > maskLength )
                  returnOffset = -1 ;
            }
         }
      }

      return( returnOffset ) ;
   }



   /*-----------------------------------------------------------*/
   /* isMaskChar
    */
   /**
    *    This method will determine if a character is
    *  a mask character
    *  
    *             
    *  @param maskChar The Mask Character to check
    *  @param 
    *  @param 
    *
    *  @return TRUE - Is a mask character, FALSE if not
    */
   private boolean isMaskChar ( char maskChar ) {
      boolean format = false ;

      if ( (maskChar == 'A') || 
           (maskChar == 'X') || 
           (maskChar == 'V') || 
           (maskChar == 'U') || 
           (maskChar == '9') )
         format = true ;

      return format;
   }


   /*-----------------------------------------------------------*/
   /* doesCharFitMask
    */
   /**
    *    This method will determine if a character
    *  will fit a mask character
    *  
    *             
    *  @param extChar The external character to check to see if fits
    *  @param maskChar The mask character to check against
    *  @param 
    *
    *  @return TRUE = extChar does fit in maskChar
    */
   private boolean doesCharFitMask ( char extChar, char maskChar ) {
      boolean fit = false;

      if ( maskChar == '9' )
         fit = ( ( isNumeric ( extChar ) ) && ( !isSpace ( (int) extChar ) ) ) ;
      else if ( maskChar == 'X' )
         fit = ( ( isAlpha(extChar) ) || ( isNumeric ( extChar ) ) ) ;
      else if ( maskChar == 'A' )
         fit = isAlpha(extChar) ;
      else if ( maskChar == 'U' )
         fit = isAlpha(extChar) ;
      else if ( maskChar == 'V' )
         fit = ( ( isAlpha(extChar) ) || ( isNumeric ( extChar ) ) ) ;

      /* Always allow our internal blank character */
      if ( extChar == BLANKCHAR )
         fit = true ;

      return( fit ) ;
   }



   /*-----------------------------------------------------------*/
   /*   isAlpha                                                 */
   /*                                                           */
   /*   Description:                                            */
   /*      This function will determine if a key is an          */
   /*   alphabetic character.                                   */
   /*                                                           */
   /*   Parameters:                                             */
   /*      key - An integer representation of a character       */
   /*                                                           */
   /*   Returns:                                                */
   /*      TRUE - Character is an alpha, FALSE if not           */
   /*-----------------------------------------------------------*/
   private static boolean isAlpha ( char key ) {
      return( ( Character.isLetter ( Character.toUpperCase(key) ) ) || ( key == ' ' ) ) ;
   } /* isAlpha */


   /*-----------------------------------------------------------*/
   /*   isNumeric                                               */
   /*                                                           */
   /*   Description:                                            */
   /*      This function will determine if a key is a numeric   */
   /*   character                                               */
   /*                                                           */
   /*   Parameters:                                             */
   /*      key - An integer representation of a character       */
   /*                                                           */
   /*   Returns:                                                */
   /*      TRUE - Character is an number, FALSE if not          */
   /*-----------------------------------------------------------*/
   private static boolean isNumeric ( char key ) {
      return( Character.isDigit ( key ) ) ;
   }

   /*-----------------------------------------------------------*/
   /*   isSpace                                                 */
   /*                                                           */
   /*   Description:                                            */
   /*      This function will determine if a key is a spac      */
   /*   character                                               */
   /*                                                           */
   /*   Parameters:                                             */
   /*      key - An integer representation of a character       */
   /*                                                           */
   /*   Returns:                                                */
   /*      TRUE - Character is an space,  FALSE if not          */
   /*-----------------------------------------------------------*/
   private static boolean isSpace ( int key ) {
      return( key == 32 ) ;
   } /* isSpace */




   /*-----------------------------------------------------------*/
   /* setStringMask
    */
   /**
    *    This method will set the string mask
    *  
    *  
    *             
    *  @param mask The String mask
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void setStringMask ( String mask ) {
      int                  i        = 0 ;
      String               newStr   = "" ;

      if ( mask != null ) {
         this.mask             = mask ;
         this.maskArray        = mask.toCharArray() ;
         this.maskDisplayArray = mask.toCharArray() ;

         /* Clear our the masking characters */
         while ( i < maskDisplayArray.length ) {
            if ( isMaskChar ( maskDisplayArray[i] ) )
               maskDisplayArray[i]=' ' ;
            i++ ;
         }

         newStr = new String ( maskDisplayArray ) ;

      } else
         maskType = MASKTYPE_UNKNOWN ;

      /* Now, fire off the event so that whatever we are
         showing in gets updated */
      try {
         super.remove ( 0, getLength() ) ;
         super.insertString ( 0, newStr, null ) ;
      } catch ( BadLocationException e ) {
      }
   }






   /*------------------------------------------------------------*/
   /* focusGained
    */
   /**
    *    We need to format the year in a MASK_DATE
    *  to a 4 digit year. Since we do that operation in
    *  the focustLost method, we will do nothing.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusGained ( FocusEvent e ) {}




   /*------------------------------------------------------------*/
   /* focusLost
    */
   /**
    *    This method will be called only if we are 
    *  a MASK_DATE method. If we are it will
    *  format a 3 digit year
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusLost ( FocusEvent e ) {
      format4DigitYear() ;
   }






   /*------------------------------------------------------------*/
   /* setTextComponent
    */
   /**
    *    This method will set the text component that
    *  this document is altering. This should be only
    *  used for MASKTYPE_STRING and MASKTYPE_DATE fields.
    *
    *  @param textComponent The JTextComponent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTextComponent ( JTextComponent textComponent ) throws NullPointerException {
      if ( textComponent == null )
         throw ( new NullPointerException() ) ;
      else {

         /* If we already have a text component,
            remove us for it's list of listeners */
         if ( this.textComponent != null )
            this.textComponent.removeFocusListener ( this ) ;

         this.textComponent = textComponent ;

         if ( maskType == MASKTYPE_DATE )
            textComponent.addFocusListener ( this ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setDateType
    */
   /**
    *    This method will set what type of date that
    *  this document is using. This does not alter
    *  the physical piece of this document. We only use this to validate 4 digit years.
    *  The default date type for the MaskDocument is Date_DATE.
    *
    *  @param dateType The dal.DDate date type ( Date_DATE, Date_STAMP etc.. )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDateType ( int dateType ) {
      this.dateType = dateType ;
   }




   /*------------------------------------------------------------*/
   /* format4DigitYear
    */
   /**
    *    This method will take the text we currently
    *  have and format it for a 4 digit year. It will
    *  then replace the text in the TextComponent
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void format4DigitYear () {
      if ( textComponent != null ) {
         String dateStr     = textComponent.getText() ;

         dateStr = CDate.format4DigitYear ( dateStr, dateType ) ;

         formatCharForMask ( 0, dateStr, null ) ;
      }
   }







   /*------------------------------------------------------------*/
   /* isMaskString
    */
   /**
    *    This method will return TRUE if we 
    *  are showing a string mask
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean isMaskString () {
      return( ( maskType == MASKTYPE_STRING ) ||
              ( maskType == MASKTYPE_DATE ) ) ;
   }


} /* MaskDocument */
