package com.ibm.appsure.app.shared.gui.form ;
/**
 * The FormComponent is used to put different types of input fields on. The
 * FormComponent has a twistie that allows the user to roll up the fields into
 * a single line or roll down the fields to show all fields. The FormComponent
 * also has a static Title at the top of the form allowing the developer to create
 * many FormComponents (each with it's own name) and put them into
 * a JScrollPane. An example GUI using the FormComponent could look like this:
 *
 * <pre>
 * > Document Type
 * O SSD
 * O Service
 * O Parts
 * X Things
 * O other Things
 * 
 * > Document Classification
 * O Conf.
 * X Dealer Conf.
 * O Unclass
 *</pre><br><br><br>
 *
 * Please note that the FormComponent is designed to show only 1 Component in
 * it's content pane. If you wish to display several components, add those
 * components to a JPanel, and use that JPanel here.<br><br>
 *
 * Please note that the FormComponent extends DPanel, so you will be able to
 * precisely set the preferred width and height
 *
 * @version 1
 * @author David Lentz 
 */
import java.awt.* ;
import javax.swing.* ;
import com.ibm.appsure.app.shared.gui.* ;

public class FormComponent extends DPanel {

   /**
     * The Title is the text that is presented at the top
     * of the FormComponent. The control showing the title is
     * a HotLinkLabel.
     **/
   private HotLinkLabel title;

   /**
     * The 1 component that the FormComponent will display in it's content
     * area
     */
   private Component formComponent = null ;

   /**
     * The Order Number is simply a way of ordering many
     * FormComponents in a list. This property defaults
     * to zero.
     */
   private int orderNum = 0 ;

   /**
     *  This property is TRUE when the FormComponent is
     * showing expanded
     **/
   private boolean expanded = true ;


   /**
     * This default constructor creates the FormComponent initialized to
     * the title passed in
     **/
   public FormComponent ( String titleText ) {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      title = new HotLinkLabel ( titleText ) ;         
      title.setOpaque ( false ) ;
      title.showSelection ( false ) ;
      setOpaque ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our components                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( title, BorderLayout.NORTH ) ;
   }

   /**
     *  This constructor creates the FormComponent and automagically
     * sets the component passed in as the form component.
     **/
   public FormComponent ( String title, JComponent component ) {
      this ( title ) ;

      setFormComponent ( component ) ;
   }

   /**
     * This method will return the title
     */
   public String getTitle() {
      return(title.getText());
   }

   /**
     * This method will set the title
     */
   public void setTitle(String title) {
      if ( title != null )
         this.title.setText ( title ) ;
   }


   /**
     * This method will set the background color of
     * this component
     */
   public void setBackground ( Color color ) {
      if ( title != null )
         title.setBackground ( color ) ;
   }

   /**
     * This method will set the Foreground color that
     * the title of this component is displayed in
     */
   public void setForeground ( Color color ) {
      if ( title != null )
         title.setForeground ( color ) ;
   }

   /**
     * This method will set the Font that the title of
     * this FormComponent is displayed in
     */
   public void setFont ( Font font ) {
      if ( title != null ) {
         title.setFont ( font ) ;
      }
   }


   /**
     * This method will expand this FormComponent to show all
     * components inside of it.
     */
   public void expand () {
      if ( formComponent != null )
         add ( formComponent, BorderLayout.CENTER ) ;
      expanded = true ;
   }

   /**
     * This method will collapse this FormComponent so that all
     * that is seen will be the title
     */
   public void collapse() {
      if ( formComponent != null ) {
         remove ( formComponent ) ;
      }
      expanded = false ;
   }

   /**
     * This method will set the order number that this FormComponent
     * should be displayed in
     */
   public void setOrderNum ( int orderNum ) {
      this.orderNum = orderNum ;
   }

   /**
     * This method will return the order number that this FormComponent
     * should be displayed in
     */
   public int getOrderNum () {
      return( orderNum ) ;
   }

   /**
     * This method will set the 1 component that the FormComponent displays
     * in it's content area
     **/
   public void setFormComponent ( Component component ) {
      if ( formComponent != null )
         remove ( formComponent ) ;

      add ( component ) ;
      this.formComponent = component ;
   }

   /**
     *  Returns the current expanded/collapsed state. TRUE = exanded
     **/
   public boolean getExanded () {
      return( expanded ) ;
   }
}
