

/*

 * @(#)Properties.java  1.31 98/07/01

 *

 * Copyright 1995-1998 by Sun Microsystems, Inc.,

 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.

 * All rights reserved.

 * 

 * This software is the confidential and proprietary information

 * of Sun Microsystems, Inc. ("Confidential Information").  You

 * shall not disclose such Confidential Information and shall use

 * it only in accordance with the terms of the license agreement

 * you entered into with Sun.

 */



package com.ibm.appsure.app.shared.util;



import java.io.IOException;

import java.io.PrintStream;

import java.io.PrintWriter;

import java.io.InputStream;

import java.io.OutputStream;

import java.util.Hashtable;

import java.util.* ;

/**

 * The <code>Properties</code> class represents a persistent set of 

 * properties. The <code>Properties</code> can be saved to a stream 

 * or loaded from a stream. Each key and its corresponding value in 

 * the property list is a string. 

 * <p>

 * A property list can contain another property list as its 

 * "defaults"; this second property list is searched if 

 * the property key is not found in the original property list. 

 *

 * @author  Arthur van Hoff

 * @version 1.31, 07/01/98

 * @since   JDK1.0

 */

public

class AdvancedProperties extends Properties {

   boolean allowSpaceInKey = false ;

   /**

    * Creates an empty property list with no default values. 

    *

    * @since   JDK1.0

    */

   public AdvancedProperties() {

      this(null);

   }



   /**

    * Creates an empty property list with the specified defaults. 

    *

    * @param   defaults   the defaults.

    * @since   JDK1.0

    */

   public AdvancedProperties(Properties defaults) {

      this.defaults = defaults;

   }



   public void setAllowSpaceInKey( boolean allowSpaceInKey ) {

      this.allowSpaceInKey = allowSpaceInKey ;

   }

   /**

    * Reads a property list from an input stream. 

    *

    * @param      in   the input stream.

    * @exception  IOException  if an error occurred when reading from the

    *               input stream.

    * @since   JDK1.0

    */

   public synchronized void load(InputStream in) throws IOException {

      load ( in, allowSpaceInKey ) ;

   }



   /**

    * Reads a property list from an input stream. 

    *

    * @param      in   the input stream.

    * @param      allowSpaceInKey When set true keys may have spaces.  Must

    *             use a = or : to separate the key from the value.

    * @exception  IOException  if an error occurred when reading from the

    *               input stream.

    * @since   JDK1.0

    */

   public synchronized void load(InputStream in, boolean allowSpaceInKey) throws IOException {

      /**

       * Use char array to collect the key and val chars.  Use an initial

       * size of 80 chars and double the array during expansion.  

       */

      int buflen = 80;

      char[] buf = new char[buflen];

      int bufindx = 0;



      in = Runtime.getRuntime().getLocalizedInputStream(in);



      int ch = in.read();

      while (true) {

         switch (ch) {

         case -1:

            return;



         case '#':

         case '!':

            do {

               ch = in.read();

            } while ((ch >= 0) && (ch != '\n') && (ch != '\r'));

            continue;



         case '\n':

         case '\r':

         case ' ':

         case '\t':

            ch = in.read();

            continue;

         }



         /* Read the key into buf */

         bufindx = 0;

         /* Allow spaces in the keys */

         if (allowSpaceInKey) {

            while ((ch >= 0) && (ch != '=') && (ch != ':') && 

                   (ch != '\t') && (ch != '\n') && (ch != '\r')) {

               /* append ch to buf */

               if (bufindx >= buflen) {

                  /* expand buf */

                  buflen *= 2;

                  char[] nbuf = new char[buflen];

                  System.arraycopy(buf, 0, nbuf, 0, buf.length);

                  buf = nbuf;

               }

               buf[bufindx++] = (char)ch;

               ch = in.read();

            }

         }

         else {

            /* do not allow spaces in the keys */

            while ((ch >= 0) && (ch != '=') && (ch != ':') && 

                   (ch != ' ') && (ch != '\t') && (ch != '\n') && (ch != '\r')) {

               /* append ch to buf */

               if (bufindx >= buflen) {

                  /* expand buf */

                  buflen *= 2;

                  char[] nbuf = new char[buflen];

                  System.arraycopy(buf, 0, nbuf, 0, buf.length);

                  buf = nbuf;

               }

               buf[bufindx++] = (char)ch;

               ch = in.read();

            }

         }

         while ((ch == ' ') || (ch == '\t')) {

            ch = in.read();

         }

         if ((ch == '=') || (ch == ':')) {

            ch = in.read();

         }

         while ((ch == ' ') || (ch == '\t')) {

            ch = in.read();

         }

         /* create the key */

         String key = new String(buf, 0, bufindx);



         /* Read the value into buf, reuse buf */

         bufindx = 0;

         while ((ch >= 0) && (ch != '\n') && (ch != '\r')) {

            int next = 0;

            if (ch == '\\') {

               switch (ch = in.read()) {

               case '\r':

                  if (((ch = in.read()) == '\n') ||

                      (ch == ' ') || (ch == '\t')) {

                     // fall thru to '\n' case

                  }

                  else continue;

               case '\n': 

                  while (((ch = in.read()) == ' ') || (ch == '\t'));

                  continue;

               case 't': ch = '\t'; next = in.read(); break;

               case 'n': ch = '\n'; next = in.read(); break;

               case 'r': ch = '\r'; next = in.read(); break;

               case 'u': {

                     while ((ch = in.read()) == 'u');

                     int d = 0;

                     loop:

                     for (int i = 0 ; i < 4 ; i++) {

                        next = in.read();

                        switch (ch) {

                        case '0':

                        case '1':

                        case '2':

                        case '3':

                        case '4':

                        case '5':

                        case '6':

                        case '7':

                        case '8':

                        case '9':

                           d = (d << 4) + ch - '0';

                           break;

                        case 'a':

                        case 'b':

                        case 'c':

                        case 'd':

                        case 'e':

                        case 'f':

                           d = (d << 4) + 10 + ch - 'a';

                           break;

                        case 'A':

                        case 'B':

                        case 'C':

                        case 'D':

                        case 'E':

                        case 'F':

                           d = (d << 4) + 10 + ch - 'A';

                           break;

                        default:

                           break loop;

                        } 

                        ch = next;

                     }

                     ch = d;

                     break;

                  }

               default: next = in.read(); break;

               }

            }

            else {

               next = in.read();

            }

            /* append ch to buf */

            if (bufindx >= buflen) {

               /* expand buf */

               buflen *= 2;

               char[] nbuf = new char[buflen];

               System.arraycopy(buf, 0, nbuf, 0, buf.length);

               buf = nbuf;

            }

            buf[bufindx++] = (char)ch;

            ch = next;

         }

         /* create the val */

         String val = new String(buf, 0, bufindx);



         put(key, val);

      }

   }



}

