package com.ibm.appsure.app.shared.util ;
import java.util.* ;
import java.io.* ;

/*********************************************************
* The standard property loader loads properties using a
* class loader.  The class loader loads your property based
* on the class path structure.  The current default location
* for all properties is com.ibm.nzna.properties.  This
* loader will load the Global.properties file always by
* default and will then load your property file.  This results
* in global properties being overidden by your values in your
* property file.
* 
*
* Legal: Copyright (c) 2000 IBM. All Rights Reserved.
* Created: 4/5/100  2:09pm
* @author 
* @version
*********************************************************/
public class StandardPropertyLoader extends Object {
    static final String           fileSeparator       = System.getProperties().getProperty("file.separator") ;
    /** The location of the properties files to attempt to load **/
    static final String           defaultResourcePath = "com"+fileSeparator+"ibm"+fileSeparator+"nzna"+fileSeparator+"properties"+fileSeparator ;
    /** The global properties settings **/
    static private Properties     properties          = null ;
    /** The global properties file name **/
    static final public String    globalPropertyFile  = "global.properties" ;
    /** The site properties file name **/
    static final public String    sitePropertyKey     = "site.properties" ;
    /** The secure property file name **/
    static final public String    securePropertyFile  = "secure.properties" ;
    /** The secure property values **/
    static private Properties     secureProperties   = null ;

    /*********************************************************
    * gets an Input Stream for the property file using the
    * default Resource Path
    * 
    *
    * Author:  
    * Created: 10/30/100  3:43pm
    * @param
    * @return
    *********************************************************/
    public static InputStream getInputStream ( String propertyFile ) {
        InputStream io = null ;
        /* If the path is already contained in the propertyFile */
        if ( ( propertyFile.indexOf( defaultResourcePath )  > -1 ) ) {
            io = ResourceLoader.getInputStream ( propertyFile ) ;
        } else {
            io = ResourceLoader.getInputStream ( defaultResourcePath + propertyFile ) ;
        }
        return( io ) ;
    }
    /*********************************************************
    * Load the global properties file.
    * 
    *
    * Author:  
    * Created: 6/20/100  9:48pm
    * @param
    * @return
    *********************************************************/
    public static Properties getProperties() throws IOException {
        /* Are the secure properties loaded */
        if ( secureProperties == null ) {
            InputStream io = getInputStream (securePropertyFile) ;
            /* Set the Global property values */
            secureProperties = new SecureProperties() ;
            if ( io != null) {
                secureProperties.load( io ) ;
            }
        }
        /* Are the global properties loaded */
        if ( properties == null ) {
            InputStream io = getInputStream (globalPropertyFile) ;
            /* Set the Global property values */
            properties = new Properties() ;
            if ( io != null) {
                properties.load( io ) ;
                /* Perform substitutions on the properties */
                substitute ( properties, properties ) ;
            }
            /* Load the site properties file if available */
            String sitePropertyFile = (String) properties.get( sitePropertyKey ) ;
            if ( sitePropertyFile != null ) {
                properties = getSiteProperties ( properties, sitePropertyFile ) ;
            }
        }
        return( properties ) ;
    }
    /*********************************************************
    * Load the secure properties file.  Decrypts the entries
    * and places them in the secure properties object.
    * 
    *
    * Author:  
    * Created: 6/20/100  9:48pm
    * @param
    * @return
    *********************************************************/
    static Properties getSecureProperties() throws IOException {
        InputStream io = getInputStream ( securePropertyFile) ;
        SecureProperties secureProperties = null ;
        /* Load the secure properties file if available */
        if ( io != null) {
            secureProperties = new SecureProperties() ;
            secureProperties.load( io ) ;
        } else {
            System.out.println( securePropertyFile + " not loaded" ) ;
        }
        return( secureProperties ) ;
    }
    /*********************************************************
    * Load the site properties file.
    * 
    *
    * Author:  
    * Created: 6/20/100  9:48pm
    * @param
    * @return
    *********************************************************/
    private static Properties getSiteProperties( Properties properties, String sitePropertyFile ) throws IOException {
        System.out.println( "Loading site properties") ;
        InputStream io = getInputStream ( sitePropertyFile) ;
        /* Load the site properties file if available */
        if ( io != null) {
            properties.load( io ) ;
            /* Perform substitutions on the properties */
            substitute ( properties, properties ) ;
        } else {
            System.out.println( sitePropertyFile + " not loaded" ) ;
        }
        return( properties ) ;
    }
    /*********************************************************
    * Loads the property files for the application.  This method load the
    * Global.properties file first and will then update the properties with
    * the property info located in the PropertyFilename property file.
    * The PropertyFilename is without extension.
    * 
    *
    * Author:  
    * Created: 4/5/100  2:30pm
    * @param   properties        The property object for the application.
    * @param   PropertyFilename  The file name of the property file.
    * @return  boolean           True if file loaded.
    *********************************************************/
    public static Properties load ( String propertyFilename ) throws IOException {
        Properties properties = null ;
        /* Append extension of property file to name if missing */
        if ( propertyFilename.indexOf (".properties") == -1) {
            propertyFilename = propertyFilename + ".properties" ;
        }

        /* Get the global properties */
        properties = getProperties() ;
        /* Clone the global properties */
        properties = (Properties) properties.clone() ;

        /* Add the specific property values */
        update ( properties, propertyFilename ) ;

        return( properties ) ;
    }
    /*********************************************************
    * Adds the values to the properties object passed in.  Loads
    * the global.properties file first and then the propery file
    * which is specified.
    * 
    *
    * Author:  
    * Created: 6/21/100  4:47pm
    * @deprecated use load( String propertyFilename )
    * @param
    * @return true if the propertyFilename was updated in the properties.
    *********************************************************/
    public static boolean load ( Properties properties, String propertyFilename ) {
        try {
            /* Get the global properties */
            Properties globalProperties = getProperties() ;
            /* Add the values from the global properties */
            Enumeration enum = globalProperties.keys() ;
            while ( enum.hasMoreElements() ) {
                String key     = (String) enum.nextElement() ;
                String value   = (String) globalProperties.get( key ) ;
                properties.put( key, value ) ;
            }

        } catch ( Exception e ) {
            e.printStackTrace() ;
        }
        /* Add the values from the specific property file */
        boolean success = update( properties , propertyFilename ) ; 
        return( success ) ;
    }

    /*********************************************************
    * Loads the property files for the application.  This method only updates
    * the existing property object with the source identified in the
    * PropertyFilename properties file.
    * The PropertyFilename is without extension.
    *
    * Author:  
    * Created: 4/5/100  2:30pm
    * @param   properties        The property object for the application.
    * @param   PropertyFilename  The file name of the property file.
    * @return  boolean           True if file loaded.
    *********************************************************/
    public static boolean update ( Properties properties, String propertyFilename ) {
        /* Append extension of property file to name if missing */
        if ( propertyFilename.indexOf (".properties") == -1) {
            propertyFilename = propertyFilename + ".properties" ;
        }


        /* Set true when properties successfully loaded */
        boolean success = false ;
        try {
            /* Set the application specific property values */
            InputStream io = getInputStream ( propertyFilename ) ;
            if ( io != null) {
                /* This will overide any keys which are the same from the Global.properties */
                properties.load( io ) ;
                success = true ;
            }
        } catch (Exception e) {
            e.printStackTrace() ;   
        }
        if ( success ) {
            /* Substitute using local values */
            substitute( properties, properties ) ;
        }
        return( success ) ;
    }
    /*********************************************************
    * Get the properties resource root path from the classpath.
    * 
    *
    * Author:  
    * Created: 6/21/100  7:44pm
    * @param
    * @return The classpath entry which contains the properties files.
    *********************************************************/
    public static String getPath() {
        String path = null ;
        /* First search using the default resource path */
        path = ResourceLoader.getPath( defaultResourcePath ) ;
        return( path ) ;
    }
    /*********************************************************
    * Performs substitutions on the property values using the
    * global properties.
    * 
    *
    * Author:  
    * Created: 6/23/100  6:44pm
    * @param
    * @return
    *********************************************************/
    private static void substitute ( Properties source, Properties target ) {
        try {
            Enumeration enum = target.keys() ;
            while ( enum.hasMoreElements() ) {
                String         key      = (String) enum.nextElement() ;
                Object         objValue = target.get( key );
                /* The value may not be a property string */
                if ( objValue instanceof String ) {
                    String         value    = (String) objValue ;
                    /* Does this value contain a substitution */
                    substituteKeyValue ( source, target, key, value ) ;
                    /* Search and Replace for password substitutions */
                    substituteSecure ( target, key ) ;
                }
            }
        } catch ( Exception e ) {
            e.printStackTrace() ;
        }
    }
    /*********************************************************
    * Substitute the property value recursively with the meta
    * values defined.
    * 
    *
    * Author:  
    * Created: 7/25/100  6:00pm
    * @param
    * @return
    *********************************************************/
    private static void substituteKeyValue ( Properties source, Properties target, String key, String value ) {
        StringBuffer   newValue = new StringBuffer() ;
        /* Does this value contain a substitution */
        int lastIndex     = 0 ;
        int indexEnd      = 0 ;
        int indexBeginKey = 0 ;
        int index         = 0 ;
        while ( (index = value.indexOf("{subst:", index)) > -1 ) {
            /* add contents of value not in {} */
            newValue.append( value.substring(lastIndex, index ) );

            /* Set to the end of the subst command */
            indexEnd      = value.indexOf ( '}', index ) ;
            /* Process only if we have the termination character */
            if ( indexEnd > -1 ) {
                indexBeginKey = value.indexOf ( ':', index ) ;
                String metaKey    = value.substring( indexBeginKey + 1, indexEnd ) ;
                /* Get the value for this key from the global properties */
                String metaValue  = (String) source.getProperty( metaKey,"{undefined value}" ) ;
                /* Recursively replace if this metaKey has metaValues */
                if ( metaValue.indexOf("{subst:") > -1 ) {
                    substituteKeyValue ( source, target, metaKey, metaValue ) ;
                    /* Get the updated value for this key */
                    metaValue  = (String) source.getProperty( metaKey,"{undefined value}" ) ;
                }
                /* Append to the new value string */
                newValue.append( metaValue ) ;
                /* Set last position index */
                lastIndex = indexEnd + 1 ;
                /* Move index to new position to search from */
                index     = indexEnd ;
            }
        }
        /* If this contained a substitution reinsert new value */
        if ( lastIndex > 0 ) {
            /* add contents of value not in {} */
            newValue.append( value.substring(lastIndex, value.length() ) );
            /* Append last string to newValue and set in the properties*/
            target.put( key, newValue.toString()) ;
        }
    }
    /*********************************************************
    * Performs substitution of values in a Properties object
    * searching for the {secure} directive.  Substitutions
    * are performed using the {secure as a key token and the
    * following string as the password identifier.
    * 
    *
    * Author:  
    * Created: 10/27/100  6:31pm
    * @param
    * @return
    *********************************************************/
    public static void substituteSecure( Properties properties, String key ) {
        StringBuffer newValue   = new StringBuffer() ;
        String value            = (String) properties.get( key ) ;
        String secureValue      = null ;
        int    index            = 0 ;
        int    indexEnd         = 0 ;
        int    lastIndex        = 0 ;
        int    indexBeginKey    = 0 ;

        /* Do we need to do the subst */
        if (( index = value.indexOf("{secure:")) > -1 ) {
            /* add contents of value pre {} */
            newValue.append( value.substring(0, index ) );
            /* Set to the end of the command */
            indexEnd      = value.indexOf ( '}', index ) ;
            /* Process only if we have the termination character */
            if ( indexEnd > -1 ) {
                indexBeginKey     = value.indexOf ( ':', index ) ;
                String secureKey  = value.substring( indexBeginKey + 1, indexEnd ) ;
                /* Get value from the Secure Properties */
                if ( secureProperties != null ) {
                    secureValue = (String) secureProperties.get( secureKey ) ;
                    /* Append to the new value string */
                    newValue.append( secureValue ) ;
                    /* Set the end position of the substitution */
                    lastIndex = indexEnd + 1 ;
                }
            }
            /* If this contained a substitution reinsert new value */
            if ( lastIndex > 0 ) {
                /* add contents of value not in {} */
                newValue.append( value.substring(lastIndex, value.length() ) );
                /* Append last string to newValue and set in the properties*/
                properties.put( key, newValue.toString()) ;
            }
        }
    }
    /*********************************************************
    * Sets the property object used to lookup secure values.
    * 
    *
    * Author:  
    * Created: 10/27/100  7:24pm
    * @param
    * @return
    *********************************************************/
    public static void setSecureProperties ( Properties secureProperties ) {
        secureProperties = properties ;
    }


    /*------------------------------------------------------------*/
    /* load
     */
    /**
     *    This method loads the given property file and parses through the given parameters.  Any option/parameter
     *     listed in both will be overwriten with the value from the command line.
     *  
     *
     * @param propFile String containing the name of the property file.
     *  @param args Command line arguments as an array of String
     *  
     *  @return Properties The updated property file.
     *//*---------------------------------------------------------*/
    public StandardPropertyLoader ( String propFile, String[] args ) throws Exception {
        try {
            String     tempStr = null;
            String     key     = null;
            String     value   = null;

            load ( propFile ) ;

            for (int i=0; i < args.length; i++) {
                tempStr = args[i];
                key = tempStr.substring(1, tempStr.indexOf(":"));
                value = tempStr.substring(tempStr.indexOf(":")+1);
                properties.put(key, value);

            }
        } catch (Exception e) {
            System.out.println(e);
            throw e ;
        }
    } 
}
