package com.ibm.appsure.common;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      Failure
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.appsure.db.* ;
import com.ibm.appsure.* ;
/**
 * The Failure class holds information about a specific
 * Application Failure. The Failure contains the following information:<br><br>
 * 
 * <pre>
 * - The log file produced when the failure ocurred (through the logIndex property)<br>
 * - The Application Code of the application that failed.<br>
 * - The type of failure (through the failureType property)<br>
 * - The time that a notification was sent out for this failure<br>
 * - The time that this failure was closed out<br>
 * - The time that work began on this failure<br>
 * - The resolution to this failure<br>
 * </pre>
 *
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* Failure
/*
/*----------------------------------------------------*/
public class Failure extends DatabaseObject implements com.ibm.appsure.app.shared.gui.MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
    * This is the appCode of the application that this
    * log file belongs to. The appCode value is defined in
    * the APP.APPLICATION table 
    */
   private String appCode = null ;

   /**
    * This is the time that a notification went out to the 
    * Primary or Secondary contact for this failure 
    */
   private String notified = null ;

   /**
    * This is the time that work began on this failure. 
    */
   private String working = null ;

   /**
    * This is the time that work stopped and the failure
    * was fixed. 
    */
   private String closed = null ;

   /**
    * This is the resolution that lead to the fix for this failure 
    */
   private String resolution = null ;

   /**
    * This is what type of failure this is. The Type of failure
    * identifies the failure severity and wether or not to
    * eMail or Page the Primary/Secondary contact. 
    */
   private int failureType = 0 ;

   /**
    * This property holds all information regarding the log file 
    */
   private LogFile logFile;

   /**
     *  This is the description of the type of failure that this is
     **/
   private String failureDescription = null ;







   /**
     *  This constructor creates a new Failure suitable to be
     * INSERTED into the APP.FAILURE table
     **/
   public Failure () {
      updateRecStatus ( REC_INSERT ) ;
   }


   /**
     *  This method will return the time that the notification
     * went out for this failure
     *
     * @return String (DB/2 Timestamp)
     **/
   public String getNotified() {
      return notified;
   }

   /**
     *  This method will set the time that a notification went out
     * for this failure.
     *
     * @param notified This must be a DB/2 Timestamp
     **/
   public void setNotified(String notified) {
      this.notified = notified;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return NULL if this failure is not currently
     * being worked on. It will return a NON-NULL value if this
     * failure has been closed or is currently being worked on.
     *
     * @return String (DB/2 Timestamp)
     **/
   public String getWorking() {
      return working;
   }

   /**
     *  This method will set the date/time that work began on this
     * failure.
     *
     * @param working This must be a DB/2 Timestamp
     **/
   public void setWorking(String working) {
      this.working = working;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return the time that this failure was fixed.
     *
     * @return String (DB/2 Timestamp)
     **/
   public String getClosed() {
      return closed;
   }

   /**
     *  This method will set the date/time that this failure was fixed. Please
     * note that if the working date/time has not been set yet, it will be set
     * to this value as well.
     *
     * @param closed This value must be a DB/2 Timestamp
     **/
   public void setClosed(String closed) {
      this.closed = closed;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return the free-form text resolution for this
     * failure
     * 
     * @return String
     **/
   public String getResolution() {
      return resolution;
   }

   /**
     *  This method will set the resolution for this failure.
     *
     * @param resolution The resolution that lead to this failure's fix.
     **/
   public void setResolution(String resolution) {
      this.resolution = resolution;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return what type of failure this is. This
     * integer can be used to read the entire failure type from the
     * APP.FAILURETYPE table
     *
     * @return int
     **/
   public int getFailureType() {
      return failureType;
   }

   /**
     *  This method will set what type of failure this is. This failure type
     * integer must exist in the APP.FAILURETYPE table.
     *
     * @param failureType The failure type as defined in the APP.FAILURETYPE table
     **/
   public void setFailureType(int failureType) {
      this.failureType = failureType;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return the appCode to the application
     * that this log files belongs to
     *
     * @return String
     **/
   public String getAppCode() {
      return appCode;
   }


   /**
     *  This method will read this record in from the database. If the runIndex
     * property is not set this method will do nothing.
     *
     * @return 0 if read from the database, the SQLCode if the record failed reading
     **/
   public int readFromDatabase () {
      return( 0 ) ;
   }


   /**
     *  This method will save this information to the database. 
     *
     * @return 0 if the record was saved, the SQLCode if the record was not saved
     **/
   public int saveToDatabase () {
      return( 0 ) ;
   }

   /**
     * This method returns TRUE if this failure is the default Zero Severity
     * failure.
     **/
   public boolean isSevZero () {
      return( failureType == 0 ) ;
   }

   public LogFile getLogFile() {
      return logFile;
   }

   public void setLogFile(LogFile logFile) {
      this.logFile = logFile;
   }

   public void setFailureDescription ( String failureDescription ) {
      this.failureDescription = failureDescription ;
   }

   /**
     *  This method will do the following:<br><pre>
     * 
     *  1. Check the eMailable flag for the failure and eMail if necessary
     *  2. Check the page flag for the failure and page if necessary
     *  3. Update the notification time for the failure</pre>
     *
     * @param conn A connection to the database for reading/writing
     * @param appCode The application code to look up
     **/
   public void sendAlert ( Connection conn, String appCode ) throws AppSureException, Exception {
      try {
         FailureType failureTypeRec = AppSureSQL.readFailureType  ( conn, getFailureType() ) ;
         Contact      toUser         = AppSureSQL.getContact       ( conn, appCode, Contact.CONTACT_PRIMARY ) ;
         Contact      secondUser     = AppSureSQL.getContact       ( conn, appCode, Contact.CONTACT_SECONDARY ) ;

         if ( toUser != null ) {
            if ( toUser.getPerson() != null ) {
               AppSureEMailer.sendMail ( toUser.getPerson().getEmail(),
                                         appCode+" Failure. Reason:"+failureTypeRec.toString(),
                                         logFile.getLogContents() ) ;
            } else
               Log.log ( "Could not find Primary Contact eMail for appliction "+appCode ) ;
         } else
            Log.log ( "Could not find Primary Contact for appliction "+appCode ) ;

         if ( secondUser != null ) {
            if ( secondUser.getPerson() != null ) {
               AppSureEMailer.sendMail ( secondUser.getPerson().getEmail(),
                                         appCode+" Failure. Reason:"+failureTypeRec.toString(),
                                         logFile.getLogContents() ) ;
            } else
               Log.log ( "   Could not find Primary Contact eMail for appliction "+appCode ) ;
         } else
            Log.log ( "   Could not find Primary Contact for appliction "+appCode ) ;
      } catch ( Exception e ) {
         throw ( e ) ;
      } catch ( AppSureException e ) {
         throw ( e ) ;
      }
   }


   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = failureDescription ;
            break ;

         case 1 :
            retObj = notified ;
            break ;

         case 2 :
            retObj = closed ;
            break ;

         case 3 :
            retObj = working ;
            break ;

         default:
            retObj = "" ;
      }

      return ( retObj ) ;
   }

} /* Failure */
