package com.ibm.appsure.common;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      FailureType
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
/**
 * The FailureType defines what type of failure is what. The FailureType contains
 * information specific to a type of failure. This information includes:<br><br>
 * 
 * <pre>
 * Description of the failure.<br>
 * If this failure should be eMailed<br>
 * If this failure should be paged out<br>
 * What severity this failure is<br>
 * </pre>
 *
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* FailureType
/*
/*----------------------------------------------------*/
public class FailureType extends DatabaseObject {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
    * This is the primary key for this type of failure. This
    * primary key is stored in the APP.FAILURETYPE table 
    */
   private int failureType;

   /**
    * This is the human readable description of what type of failure this
    * is. The type of failure could be 'Publishing Error', or 'Fatal Error'. 
    */
   private String description;

   /**
    * This property tells if an eMail should be sent if this
    * Failure Type occurs. 
    */
   private boolean email;

   /**
    * This property tells if a page should go out if this
    * failure happens. The page will go out to the
    * Primary Contact for the application 
    */
   private boolean page;

   /**
    * This is the severity of this failure. A severity is simply
    * a number. The actual number should represent a department's
    * consideration of a severity. 
    */
   private int severity;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This constructor creates a failure type suitable for an
     * INSERT into the APP.FAILURETYPE table.
     **/
   public FailureType () {
      updateRecStatus ( REC_INSERT ) ;
   }

   /**
     *  This constructor creates a failure type that can be read
     * from the database. Please note that this constructor does
     * not read from the database. To read this object from the
     * database please use the 'readFromDatabase' method.
     *
     * @param failureType The Failure Type that exists in the APP.FAILURETYPE table
     **/
   public FailureType ( int failureType ) {
      this.failureType = failureType ;
      updateRecStatus ( REC_NOCHANGE ) ;
   }

   /**
     *  This method will return the primary key for the failure type which
     * is stored in the APP.FAILURETYPE table
     *
     * @return int
     **/
   public int getFailureType(){ 
      return failureType; 
   }


   /**
     *  This method will return the human readable description of this type of failure
     *
     * @return String
     **/
   public String getDescription(){ 
      return description;
   }

   /**
     *  This method will set the description for this failure type. This description
     * should be human readable
     *
     * @param description The new description
     **/
   public void setDescription(String description){ 
      this.description = description;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return the property eMail which tells
     * if this failure will be eMailed when it happens
     *
     * @returns boolean
     **/
   public boolean getEmail(){ 
      return email;
   }

   /**
     *  This method will set the property eMail which tells
     * if this failure will be eMailed when it happens
     *
     * @param eMail TRUE = This failure should be eMailed
     **/
   public void setEmail(boolean email){ 
      this.email = email;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return the Page property which
     * tells if this failure will be paged out when it happens
     *
     * @returns boolean
     **/
   public boolean getPage(){ 
      return page;
   }

   /**
     *  This method will set the page property which tells if
     * this failure will be paged out when it happens
     *
     * @param page TRUE = This failure will have to be paged out
     **/
   public void setPage(boolean page){ 
      this.page = page;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will return the severity of this failure
     *
     * @returns int
     **/
   public int getSeverity(){ 
      return severity;
   }

   /**
     *  This method will set the severity of this failure.
     *
     * @param severity A number for the severity
     **/
   public void setSeverity(int severity){ 
      this.severity = severity;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /**
     *  This method will read this object from the database only
     * if this object's failureType property has been set
     *
     * @returns 0 if the object was read, SQLCode for failure if it was not read
     **/
   public int readFromDatabase () {
      return( 0 ) ;
   }

   /**
     *  This method will save this object to the database.
     *
     * @returns 0 if the object was saved, SQLCode for failure if it was not saved.
     **/
   public int saveToDatabase () {
      return( 0 ) ;
   }

} /* FailureType */
