package com.ibm.appsure.common;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      LogFile
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.appsure.db.* ;
import com.ibm.appsure.* ;
/**
 * This object represents a single Log File in the AppSure system.
 * The information in this object is stored in the APP.LOGS table
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* LogFile
/*
/*----------------------------------------------------*/
public class LogFile extends DatabaseObject implements com.ibm.appsure.app.shared.gui.MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is the primary key for this log entry in the
    * AppSure system. The information in this
    * object is stored in the APP.LOGS table 
    */
   private int logIndex = 0 ;

   /**
    * This is the appCode of the application that this
    * log file belongs to. The appCode value is defined in
    * the APP.APPLICATION table 
    */
   private String appCode = null ;

   /**
    * This is the primary key for the application that this
    * log file belongs to. This primary key is defined in the
    * APP.APPLICATION table 
    */
   private int appIndex = 0 ;

   /**
    * This property holds the contents of the log file. 
    */
   private String logContents = null ;

   /**
    * This is the timestamp of when this log file was
    * generated 
    */
   private String logTimestamp = null ;

   /**
    * This property determines if this log file should be
    * kept when it's time expires. If this property is FALSE,
    * then this log file will be erased when the next
    * AppSure cleanup is performed. 
    */
   private boolean keep = false ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This constructor creates a new LogFile suitable for INSERT
     * into the AppSure system
     **/
   public LogFile () {
      updateRecStatus ( REC_INSERT ) ;
   }

   /**
     *  This constructor creates a new LogFile that can be read
     * from the APP.LOGS table. Please note that this constructor
     * does not read the information from the database. A call to
     * the 'readFromDatabase' method must be made to do that
     *
     * @param logIndex This is the primary key for this log file
     **/
   public LogFile ( int logIndex ) {
      this.logIndex = logIndex ;
   }

   /**
     *  This method will return the primary key for this log
     * in the APP.LOGS table.
     *
     * @return int This is the primary key for this log file
     **/
   public int getLogIndex() {
      return logIndex; 
   }

   /**
     *  This method will return the appCode to the application
     * that this log files belongs to
     *
     * @return String
     **/
   public String getAppCode() {
      return appCode;
   }

   /**
     *  This method will set the appCode to the application 
     * that this log file belongs to
     *
     * @param appCode The appCode to the application
     **/
   public void setAppCode(String appCode) {
      this.appCode = appCode;
   }

   /**
     *  This method will return the AppIndex. The AppIndex
     * is the primary key to the application that this log
     * file belongs to.
     *
     * @return int
     **/
   public int getAppIndex() {
      return appIndex;
   }

   /**
     *  This method will set the AppIndex. The AppIndex
     * is the primary key to the application that this log
     * file belongs to.
     *
     * @param appIndex The Primary Key to the application
     **/
   public void setAppIndex(int appIndex) {
      this.appIndex = appIndex;
   }

   /**
     *  This method will return the entire contents of the
     * log file.
     *
     * @return String
     **/
   public String getLogContents() {
      return logContents;
   }

   /**
     *  This method will set the log file contents
     *
     * @param logContents The new contents for this log file
     **/
   public void setLogContents(String logContents) {
      this.logContents = logContents;
   }

   /**
     *  This methodw ill return the date/time that the
     * log file was written
     *
     * @return String
     **/
   public String getLogTimestamp() {
      return logTimestamp;
   }

   /**
     *  This method will set the date/time that this log file
     * was written
     *
     * @param logTimestamp This must be in DB/2 Timestamp format
     **/
   public void setLogTimestamp(String logTimestamp) {
      this.logTimestamp = logTimestamp;
   }

   /**
     *  This method will return the 'keep' property which 
     * tells wether this log file is erased when it expires or not.
     *
     * @return boolean
     **/
   public boolean getKeep() {
      return keep;
   }

   /**
     *  This method will set the 'keep' property which
     * tells wether this log file is erased when it expires or not.
     *
     * @param keep TRUE = keep this log file forever, FALSE = erase after expiration
     **/
   public void setKeep(boolean keep) {
      this.keep = keep;
   }

   /**
     *  This method will read this record in from the database. If the runIndex
     * property is not set this method will do nothing.
     *
     * @return 0 if read from the database, the SQLCode if the record failed reading
     **/
   public int readFromDatabase () {
      return( 0 ) ;
   }


   /**
     *  This method will save this information to the database. 
     *
     * @return 0 if the record was saved, the SQLCode if the record was not saved
     **/
   public int writeToDatabase ( Connection conn ) throws AppSureException {
      try {
         PreparedStatement stmt = null ;

         switch ( getRecStatus() ) {
            case REC_DELETE :
               stmt = conn.prepareStatement ( "INSERT INTO APPSURE.logs "+
                                              "values ( "+logIndex+", '"+appCode+"', ?, CURRENT TIMESTAMP, 'Y' )" ) ;

               stmt = conn.prepareStatement ( "DELETE FROM APPSURE.FAILURE WHERE LOGINDEX = "+logIndex ) ;
               stmt.execute() ;
               stmt.close() ;
               stmt = conn.prepareStatement ( "DELETE FROM APPSURE.LOGS WHERE LOGINDEX = "+logIndex ) ;
               stmt.execute() ;
               stmt.close() ;
               break;

            case REC_UPDATE :
               stmt = conn.prepareStatement ( "UPDATE APPSURE.LOGS SET KEEP = '"+((keep ? "Y":"N"))+"', LOGTIMESTAMP = '"+logTimestamp+"', LOG = ? WHERE LOGINDEX = "+logIndex ) ;
               stmt.setBytes ( 1, getLogContents().getBytes() ) ;
               stmt.execute() ;
               stmt.close() ;
               break ;

            case REC_INSERT :
               stmt = conn.prepareStatement ( "INSERT INTO APPSURE.LOGS VALUES ( "+logIndex+", ?, '"+logTimestamp+"', '"+((keep ? "Y":"N"))+"' )" ) ;
               stmt.setBytes ( 1, getLogContents().getBytes() ) ;
               stmt.execute() ;
               stmt.close() ;
               break ;
         }
         stmt.close() ;
      } catch ( SQLException e ) {
         throw new AppSureException ( e ) ;
      }

      return( 0 ) ;
   }

   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = ""+logIndex ;
            break ;

         case 1 :
            retObj = logTimestamp ;
            break ;

         case 2 :
            retObj = keep ? "Y" : "N" ;
            break ;

         default:
            retObj = "" ;
      }
      return( retObj ) ;
   }

} /* LogFile */
