package com.ibm.appsure.common ;  
/*-----------------------------------------------------------*/
/*   RunTimeRec                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: RunTimeRec                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
/**
 * This class holds the information regarding the time an 
 * Application runs. This information includes:<br><br>
 *
 * <pre>
 * Start Time     - The time that the application should start
 * Day            - The Day that the application should start
 * </pre>
 *
 *
 * @version 1.0, 5/7/2001 
 * @author  David Lentz
 */
public class RunTimeRec extends DatabaseObject {

   /**
    * The startTime property is the time of day that this runTime
    * instance starts. 
    */
   private String startTime = null ;

   /**
    * This is the day of week that this runtime instance starts
    * on. This day of week must be all upper case and must
    * contain the the first 3 letters of the day. For example: mon, tue, wed, thu, fri, sat, sun
    */
   private String day = null ;

   /**
     *  This is the application code for this run
     **/
   private String appCode = null ;

   /**
     *  This is the primary key
     **/
   private int runTimeIndex = 0 ;

   /**
     *  This constructor creates a RunTimeRec suitable for saving as a new
     * run time record
     **/
   public RunTimeRec () {
   }

   /**
     *  This constructor creates a RunTimeRec that should already be in the
     * AppSure database
     **/
   public RunTimeRec ( int runTimeIndex ) {
      this.runTimeIndex = runTimeIndex ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /**
     *  This method will return the time of day that this run time instance
     * should start on. The time returned is in the format HH:MM:SS
     *
     * @return The start time
     **/
   public String getStartTime() {
      return startTime; 
   }

   /**
     *  This method will set the new time of day that this run time instance
     * should start.
     *
     * @param startTime The new start time which should be in the format HH:MM:SS
     **/
   public void setStartTime(String startTime) {
      this.startTime = startTime;
   }


   /**
     *  This method will return the 3 character day that this run time instance
     * runs on.
     *
     * @returns String (MON, TUE, WED, THU, FRI, SAT, SUN)
     **/
   public String getDay() {
      return day;
   }

   /**
     *  This method will set the day that this runtime instance starts
     * on. The day must be one of (MON, TUE, WED, THU, FRI, SAT, SUN).
     *
     * @param day The new day
     **/
   public void setDay(String day) {
      this.day = day;
   }


   /**
     *  This method will read this record in from the database. If the runIndex
     * property is not set this method will do nothing.
     *
     * @return 0 if read from the database, the SQLCode if the record failed reading
     **/
   public int readFromDatabase () {
      return( 0 ) ;
   }


   /**
     *  This method will save this information to the database. 
     *
     * @return 0 if the record was saved, the SQLCode if the record was not saved
     **/
   public int saveToDatabase () {
      return( 0 ) ;
   }


   /**
     *  This method will return the appCode that this run is associated with
     *
     * @return String
     **/
   public String getAppCode () {
      return ( appCode ) ;
   }

   /**
     *  This method will set the appCode that this run is associated with
     *
     * @param appCode The AppCode that should already be in the appSure.Application table
     **/
   public void setAppCode ( String appCode ) {
      this.appCode = appCode ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /**
     *  This method will determine if the application represented by this RunTimeRec
     * ran for this time. This is done by looking into the AppSure.AppProcess table. The
     * time indicated in this record must be present in the table. The time om tje AppProcess
     * table can be 5 minutes before or 5 minutes after the time represented in this record
     * for a succesful run.
     *
     * @param conn The Database Connection needed to read from the AppProcess table
     **/
   public boolean didAppRun ( Connection conn ) throws Exception {
      boolean ran        = false ;
      String  beforeTime = Util.getBeforeTime ( startTime ) ;
      String  afterTime  = Util.getAfterTime  ( startTime ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, make the beforeTime and afterTime into a timestamp  */                                             
      /*                                                           */
      /*-----------------------------------------------------------*/
      beforeTime = Util.createTimestampFromTime ( beforeTime ) ;
      afterTime  = Util.createTimestampFromTime ( afterTime ) ;
      
      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PROCESSINDEX "+
                                              "FROM APPSURE.APPPROCESS "+
                                              "WHERE STARTTIME > '"+beforeTime+"' AND "+
                                              "      STARTTIME < '"+afterTime+"'" ) ;

         ran = rs.next() ;
         
         rs.close() ;
         stmt.close() ;
      } catch ( Exception e ) {
         throw e ;
      }

      return ( ran ) ;
   }


   /**
     *  This method will return the run time index that this run time represents
     *
     * @return int
     **/
   public int getRunTimeIndex () {
      return ( runTimeIndex ) ;
   }
}


