/*
 *                
 * @(#)BatchInit.java	1.0 04/24/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch;

import java.util.*;
import java.io.*; 
import java.sql.*; 
import com.ibm.nzna.shared.util.*; 
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.appsure.*; 

/**
 * This class is used to get the command line arguments and the properties 
 * from property file. If there is a naming conflicating, the command line
 * name takes precedence.  The class StandardPropertyLoader will be loading
 * both with the given property file name and command args.  
 *  
 * The command params will be simply specified as: <br>
 *    -opt1 val -opt2 val -opt3 val    <br> 
 * 
 * The example command line options will be: <br> 
 *  -website  (default: esupport|bpartner|techtips) <br>  
 *  -db       (default: DbName) <br>  
 *  -database (default: db server name) <br>  
 *  -port     (default: db port number) <br>  
 *  -userid   (default: quest) <br>  
 *  -password (default: xxx)   <br> 
 *  -logdir   (default: $home/logs)  <br> 
 *  -outfile  (default: $logdir/navgen_$website.out)   <br> 
 *  -logfile  (default: navgen_$website.date, where $website is the website
 *                            name and date is the three letter date such as 'Fri') <br> 
 *  -loglevel (default: production)  <br> 
 *  -plugin   (default: none) <br>  
 *  -esupport.xmltemplate   xml template string for docgen <br>  
 *  -esupport.html.outpath  html doc out path for docgen esupport (etc) website <br>  
 *  -esupport.xml.outpath   html doc out path for docgen esupport website <br>  
 *  -esupport.ftp.host      ftp host <br>  
 * <br>
 * 
 *  As an example:  <br> 
 *    -website infotips -database quest -userid quest -password xxx <br> 
 *    -logfile log1|log2|log3   <br> 
 *    -plugin com.ibm.nzna.projects.batch.plugin1|com.ibm.nzna.projects.batch.plugin2  <br>
 * <br> 
 *  
 * @version 1.0, 04/24/2001 
 * @author  Steve Wang
 */

public class BatchInit {
    /**
     * The batch log system 
     * @see com.ibm.nzna.shared.batch.BatchLogSystem  
     */
    public static BatchLogSystem  logSystem = null; 

    /**
     * The appsure system 
     * @see com.ibm.appsure.AppSure   
     */
    public static AppSure appSure = null; 

    /**
     * The tigris.Constants table 
     */
    public static com.ibm.nzna.projects.common.quest.Constants tConstants = 
        new com.ibm.nzna.projects.common.quest.Constants(); 

    /**
     * Total Fatal Errors
     */
    public static long batchFatal = 0; 

    /**
     * Total Errors
     */
    public static long batchError = 0; 

    /**
     * Total Warnings
     */
    public static long batchWarning = 0; 

    /**
     * This function gets the plugins in the format of p1|p2|p3  etc.
     * @param none.  
     * @return String The plugins in the format of p1|p2|p3.
     */
    public static  String getPlugins() throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("plugins");
    } 

    /**
     * This function gets the database name
     * @param none
     * @return String Database name
     */
    public static  String getDbName () throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("database.dbname");
    }
    
    /**
     * This function gets the database server
     * @param none
     * @return String Database server name
     */
    public static  String getDbServerName () throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("database.server");
    }
    
    /**
     * This function gets the database port
     * @param none
     * @return String Db2 server port number
     */
    public static  String getDbPort () throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("database.port");
    }
    
    /**
     * This function gets the database user id
     * @param none
     * @return String User id for accessing the database
     */
    public static  String getDbUserId () throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("database.userid");
    }
    
    /**
     * This function gets the database user password
     * @param none
     * @return String User password for accessing the database
     */
    public static  String getDbUserPasswd () throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("database.password");
    }

    /**
     * This function gets the db2 jdbc driver (net or app) 
     * @param none
     * @return boolean true for app driver false for net driver 
     */
    public static boolean getDb2Driver () {
        try {
            return "app".equals(StandardPropertyLoader.getProperties().getProperty("database.driver"));
        } catch (Exception e) {
            // Default to net driver
            return false; 
        }
    }
    
    /**
     * Get the batch insert number for gdbm table
     */
    public static final int getBatchInserts() {
        try{
            String num = StandardPropertyLoader.getProperties().getProperty("batchInserts");
            return Integer.parseInt(num); 
        } catch (Exception e) {
        }
        // Default to one insert
        return 1; 
    }

    /** 
     * Debug or not
     */
    public final static boolean getDebug () {
        try {
            String debug = StandardPropertyLoader.getProperties().getProperty("debug");
            if (debug!=null && debug.compareToIgnoreCase("Y")==0) {
                return true;
            } 
        } catch (Exception e) {
        }
        return false; 
    }

    /**
     * Set which level (debug, informational, warning) to output
     */
    public final static int getLogLevel() {
        try {
            String loglevel = StandardPropertyLoader.getProperties().getProperty("logLevel");
            if (loglevel!=null && loglevel.compareToIgnoreCase("D")==0) {
                return Constants.DEBUG;
            } else if (loglevel!=null && loglevel.compareToIgnoreCase("I")==0) {
                return Constants.INFORMATIONAL;
            } else if (loglevel!=null && loglevel.compareToIgnoreCase("W")==0) {
                return Constants.WARNING;
            }
        } catch (Exception e) {
        }
        return Constants.DEBUG; 
    }

    /**
     * Log files directory
     */
    public static  String getLogdir () throws IOException {
        String logdir = StandardPropertyLoader.getProperties().getProperty("logdir");
        if (logdir==null) {
            logdir = "." + File.separator; 
        } 
        return BatchUtil.substitute(logdir, "/", File.separator); 
    }

    /**
     * This function gets the command to run the C++ program 
     *
     * @param none
     * @return String The command string with command line arguments 
     */
    public static  String getCreateDbCommand () throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("createDbCommand");
    }

    /**
     * This function gets the working directory of the createDbCommand
     * 
     * @param none
     * @return String The working directory of the command
     */
    public static  String getCommandDir () throws IOException {
        return StandardPropertyLoader.getProperties().getProperty("commandDir");
    }
    
    /**
     * This function gets website name
     * @param none
     * @return Vector The vector contanins website name in the format of: <br>
     * ESUPPORT: Esupport website <br>
     * BPARTNER: Business partner website <br>  
     * TECHTIPS: Infotips website <br>
     */
    public static  Vector getWebSiteNames () throws IOException {
        //The website param contains website names separated by '|'
        String sites   = StandardPropertyLoader.getProperties().getProperty("websites");
        Vector sitevec = new Vector(); 
        if (sites.length()>0) {
            StringTokenizer st = new StringTokenizer(sites, "|");
            while (st.hasMoreTokens()) {
                sitevec.add(st.nextToken()); 
            }
        } 
        return sitevec; 
    }

    /**
     * This function gets the xml template. We now store xml template in the
     * docgen property file instead of a file. 
     * @param site The website name
     * @return String xml tmeplate string
     */
    public static  String getXmlTemplate (String site) throws IOException {
        String template_dir = ""; 
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            template_dir = StandardPropertyLoader.getProperties().getProperty("infotips.xmltemplate");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            template_dir = StandardPropertyLoader.getProperties().getProperty("ipartner.xmltemplate");
        } else {
            template_dir = StandardPropertyLoader.getProperties().getProperty("esupport.xmltemplate");
        }

        byte[] buff = new byte[8192]; 
        //FileInputStream fin = new FileInputStream(template_dir + "docgen_template.xml");
        InputStream fin = ResourceLoader.getInputStream(template_dir + "docgen_template.xml");
        int n_read = fin.read (buff); 
        String template = new String(buff, 0, n_read); 
        fin.close();
        return template; 
    }
    
    /**
     * get html doc output path 
     * @param site The website name
     * @return String The output path for the html docs
     */
    public static  String getHtmlDocOutPath (String site) throws IOException {
        String out_path = ""; 
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("infotips.html.outpath");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("ipartner.html.outpath");
        } else {
            out_path = StandardPropertyLoader.getProperties().getProperty("esupport.html.outpath");
        }

        // Since the path separator is by default unix style in the property file, 
        // we replace it with system default
        return BatchUtil.substitute(out_path, "/", File.separator); 
    }

    /**
     * get the tmp files out path. This is where the temp  files located
     */
    public static String getTempFilePath () throws IOException {
        String out_path = StandardPropertyLoader.getProperties().getProperty("tempPath"); 
        
        // Since the path separator is by default unix style in the property file, 
        // we replace it with system default
        return BatchUtil.substitute(out_path, "/", File.separator); 
    }
    
    /**
     * get gdbm file output path 
     * @param site The website name
     * @return String The output path for the gdbm file 
     */
    public static  String getGdbmFileOutPath (String site) throws IOException {
        String out_path = ""; 
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("infotips.gdbm.outpath");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("ipartner.gdbm.outpath");
        } else if (site.compareTo(Constants.WEB_SITE_ESUPPORT)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("esupport.gdbm.outpath");
        } else if (site.compareTo(Constants.WEB_SITE_ESUPPORT_PROFILE)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("esupport.profile.gdbm.outpath");
        } else {
            out_path = StandardPropertyLoader.getProperties().getProperty("esupport.gdbm.outpath");
        }

        // Since the path separator is by default unix style in the property file, 
        // we replace it with system default
        return BatchUtil.substitute(out_path, "/", File.separator); 
    }

    /**
     * get gdbm file output path 
     * @param site The website name
     * @return String The output path for the gdbm file 
     */
    public static  String getGdbmFile(int site) throws IOException {
        String out_path = ""; 
        switch (site) {
        case Constants.TECHTIPS_TYPE:
            out_path = StandardPropertyLoader.getProperties().getProperty("techtips.gdbm.outpath");
            break;
        case Constants.BPARTNER_TYPE:
            out_path = StandardPropertyLoader.getProperties().getProperty("bpartner.gdbm.outpath");
            break;
        case Constants.INFOTIPS_TYPE:
            out_path = StandardPropertyLoader.getProperties().getProperty("infotips.gdbm.outpath");
            break;
        case Constants.IPARTNER_TYPE:
            out_path = StandardPropertyLoader.getProperties().getProperty("ipartner.gdbm.outpath");
            break;
        case Constants.ESUPPORT_PROFILE_TYPE:
            out_path = StandardPropertyLoader.getProperties().getProperty("esupport.profile.gdbm.outpath");
            break;
        default:
            // All site for marketing catalog
            out_path = StandardPropertyLoader.getProperties().getProperty("esupport.gdbm.outpath");
            break;
        }

        // Since the path separator is by default unix style in the property file, 
        // we replace it with system default
        return BatchUtil.substitute(out_path, "/", File.separator); 
    }
    
    /**
     * Get gdbm file name
     */
    public static  String getGdbmFileName () throws IOException {
        String name = StandardPropertyLoader.getProperties().getProperty("gdbmFile");
        if (name==null) {
            name = "db"; 
        }
        return name; 
    }

    /**
     * Get gdbm file name
     */
    public static  String getProfileDbFileName () throws IOException {
        String name = StandardPropertyLoader.getProperties().getProperty("profileDb");
        if (name==null) {
            name = "profile.db"; 
        }
        return name; 
    }

    /**
     * get xml doc output path 
     * @param site The website name
     * @return String The output path for the xml docs
     */
    public static  String getXmlDocOutPath (String site) throws IOException {
        String out_path = ""; 
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("infotips.xml.outpath");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            out_path = StandardPropertyLoader.getProperties().getProperty("ipartner.xml.outpath");
        } else {
            out_path = StandardPropertyLoader.getProperties().getProperty("esupport.xml.outpath");
        }

        // Since the path separator is by default unix style in the property file, 
        // we replace it with system default
        return BatchUtil.substitute(out_path, "/", File.separator); 
    }

    /**
     * get html ftp host 
     * @param site The website name
     * @return String The ftp host
     */
    public static  String getFtpHost (String site) throws IOException {
        try {
            if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
                return StandardPropertyLoader.getProperties().getProperty("infotips.ftp.host");
            } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
                return StandardPropertyLoader.getProperties().getProperty("ipartner.ftp.host");
            } else {
                return StandardPropertyLoader.getProperties().getProperty("esupport.ftp.host");
            }
        } catch (Exception e) {
            // When the host name does not exit, don't ftp
            return ""; 
        }
    }

    /**
     * get xml ftp host 
     * @param site The website name
     * @return String Host name
     */
    public static  String getXmlFtpHost (String site) throws IOException {
        //currently set it as the same as html host
        return getFtpHost(site); 
    }

    /**
     * get ftp host dir for html docs
     * @param site The website name
     * @retrun String ftp host directory
     */
    public static  String getFtpHostDir (String site) throws IOException {
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            return StandardPropertyLoader.getProperties().getProperty("infotips.ftp.hostdir");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            return StandardPropertyLoader.getProperties().getProperty("ipartner.ftp.hostdir");
        //} else if (site.compareTo(Constants.WEB_SITE_ESUPPORT_PROFILE)==0) {
        //    return StandardPropertyLoader.getProperties().getProperty("esupport.profile.ftp.hostdir");
        } else {
            return StandardPropertyLoader.getProperties().getProperty("esupport.ftp.hostdir");
        }
    }
    
    /**
     * get ftp host dir for xml docs
     * @param site The website name
     * @retrun String ftp host directory for xml
     */
    public static  String getXmlFtpHostDir (String site) throws IOException {
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            return StandardPropertyLoader.getProperties().getProperty("infotips.ftp.xmlhostdir");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            return StandardPropertyLoader.getProperties().getProperty("ipartner.ftp.xmlhostdir");
        } else {
            return StandardPropertyLoader.getProperties().getProperty("esupport.ftp.xmlhostdir");
        }
    }

    /**
     * ftp host user id
     * @param site The website name
     * @return String The ftp host user id
     */
    public static  String getFtpUserId (String site) throws IOException {
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            return StandardPropertyLoader.getProperties().getProperty("infotips.ftp.userid");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            return StandardPropertyLoader.getProperties().getProperty("ipartner.ftp.userid");
        //} else if (site.compareTo(Constants.WEB_SITE_ESUPPORT_PROFILE)==0) {
        //    return StandardPropertyLoader.getProperties().getProperty("esupport.profile.ftp.userid");
        } else {
            return StandardPropertyLoader.getProperties().getProperty("esupport.ftp.userid");
        }
    }
    
    /**
     * ftp host password for the user id
     * @param site The website name
     * @return String The ftp password
     */
    public static  String getFtpPassword (String site) throws IOException {
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            return StandardPropertyLoader.getProperties().getProperty("infotips.ftp.password");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            return StandardPropertyLoader.getProperties().getProperty("ipartner.ftp.password");
        //} else if (site.compareTo(Constants.WEB_SITE_ESUPPORT_PROFILE)==0) {
        //    return StandardPropertyLoader.getProperties().getProperty("esupport.profile.ftp.password");
        } else {
            return StandardPropertyLoader.getProperties().getProperty("esupport.ftp.password");
        }
    }
    
    /**
     * ftp host image dir 
     * @param site The website name
     * @return String The host image directory
     */
    public static  String getHostImageDir (String site) throws IOException {
        if (site.compareTo(Constants.WEB_SITE_INFOTIPS)==0) {
            return StandardPropertyLoader.getProperties().getProperty("infotips.ftp.imagedir");
        } else if (site.compareTo(Constants.WEB_SITE_IPARTNER)==0) {
            return StandardPropertyLoader.getProperties().getProperty("ipartner.ftp.imagedir");
        } else {
            return StandardPropertyLoader.getProperties().getProperty("esupport.ftp.imagedir");
        }
    }
    
    /**
     * local image dir 
     * @param site The website name
     * @return String The local image directory
     */
    public static  String getLocalImageDir (String site) throws IOException {
        //Need to find the media path from Constants table. Html path is fine
        return tConstants.getStrConst(Constants.IMAGELOC_PREFIX /*IMAGELOC_*/ + site); 
    }
    
    /**
     * This function gets website type
     * @param site The website name
     * @return int The site type
     */
    public static  int getSiteType (String sitename) throws IOException {
        //if (Constants.WEB_SITE_TECHTIPS.compareToIgnoreCase(sitename) == 0) {
        //    return Constants.TECHTIPS_TYPE;
        //} else if (Constants.WEB_SITE_BPARTNER.compareToIgnoreCase(sitename) == 0) {
        //    return Constants.BPARTNER_TYPE;
        if (Constants.WEB_SITE_ESUPPORT.compareToIgnoreCase(sitename) == 0) {
            return Constants.ESUPPORT_TYPE;
        } else if (Constants.WEB_SITE_INFOTIPS.compareToIgnoreCase(sitename) == 0) {
            return Constants.INFOTIPS_TYPE;
        } else if (Constants.WEB_SITE_IPARTNER.compareToIgnoreCase(sitename) == 0) {
            return Constants.IPARTNER_TYPE;
        } else if (Constants.WEB_SITE_ESUPPORT_PROFILE.compareToIgnoreCase(sitename) == 0) {
            return Constants.ESUPPORT_PROFILE_TYPE;
        } 
        return Constants.ESUPPORT_TYPE;
    }

    /**
     * This function gets website name from the website type
     * @param site The website name
     * @return int The site type
     */
    public static  String getSiteName (int sitetype) throws IOException {
        switch (sitetype) {
        //case Constants.TECHTIPS_TYPE:
        //    return Constants.WEB_SITE_TECHTIPS; 
        //case Constants.BPARTNER_TYPE:
        //    return Constants.WEB_SITE_BPARTNER;
        case Constants.INFOTIPS_TYPE:
            return Constants.WEB_SITE_INFOTIPS;
        case Constants.IPARTNER_TYPE:
            return Constants.WEB_SITE_IPARTNER;
        case Constants.ESUPPORT_TYPE:
            return Constants.WEB_SITE_ESUPPORT;
        case Constants.ESUPPORT_PROFILE_TYPE:
            return Constants.WEB_SITE_ESUPPORT_PROFILE;
        default:
            // All site for marketing catalog
            return "AllSite"; 
        }
    }

    /**
     * Get if docgen needs fullupdate
     *
     * @param none
     * @return String Returns string 'true' or 'false'
     */
    public static  String getDocgenFullUpdate() throws IOException {
        String full = StandardPropertyLoader.getProperties().getProperty("fullUpdate");
        if (full==null) {
            return "false"; 
        }
        return full; 
    }
    
    /**
     * Get docgen publish now. This parameter is set from command line
     * as    -publishNow:true
     *
     * @param none
     * @return String Returns string 'true' or 'false'
     */
    public static  String getDocgenPublishNow() throws IOException {
        String pubNow = StandardPropertyLoader.getProperties().getProperty("publishNow");
        if (pubNow==null) {
            return "false"; 
        }
        return pubNow; 
    }
    
    /**
     * This function gets website lang ind
     */
    public static  int getLangInd () {
        //Currently we have publish English only 
        return 1; 
    }
    
    /**
     * This function gets App name in a (property) file
     * @param appNameKey the key for the app name.  
     * @return String the app name.
     */
    public static  String getAppName(String appNameKey) throws IOException {
        String appName = StandardPropertyLoader.getProperties().getProperty(appNameKey+".AppName");
        if (appName==null) {
            // if not in property file, use the key name
            appName = appNameKey;
            BatchUtil.logit("Property appName not found, using appNameKey", Constants.WARNING, 0); 
        }
        return appName; 
    }      

    /**
     * This function gets sleep time for certain app 
     * @param appNameKey the key for the app name.  
     * @return String the app name.
     */
    public static  long getAppMaxSleepTime(String appNameKey) throws IOException {
        String max = StandardPropertyLoader.getProperties().getProperty(appNameKey+".maxsleep");
        
        // assign max sleep time to 1 hour if not found from property
        long sleep_max = 3600000;  
        try {
            sleep_max = Long.parseLong(max);
        } catch (NumberFormatException e) {
        }
        return sleep_max; 
    }  

    /**
     * This function gets app RunTimeRec for appSure 
     * @param appNameKey the key for the app name.  
     * @return int the runtimerec number defined in property somewhere.
     */
    public static  int getAppRunTimeRec(String appNameKey) throws IOException {
        // todo
        // needs to look up the RunTimeRec somewhere
        return 0; 
    }
}

