/*
 *                
 * @(#)BuildProfileDb.java	1.0 08/08/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch.plugin;

import java.util.*; 
import java.io.*; 
import java.sql.*; 
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.projects.batch.*;
import com.ibm.nzna.projects.batch.event.*;
import com.ibm.nzna.projects.batch.intf.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.util.*; 
import com.ibm.nzna.shared.ftp.FTP.*; 
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.quest.doc.*; 


/**
 *  The BuildProfileDB plugin main class. The following keys will be
 *  saved in the profile gdbm file.      
 *  <br>  
    1) The Brand display list gdbm keys:             <br> 
       Brand  (key)  Display on profiling  (value)   <br> 
       e.g., "IBM Aptiva|Aptiva PS/1",   <br> 
        "IBM PC|Business PCs",           <br> 
        "IBM ThinkPad|Notebooks and PC Companions", <br> 
        "IBM PC Server|Servers",                    <br> 
        "IBM IntelliStation|IntelliStation",        <br> 
        "IBM Network Station|Network Station thin client",  <br> 
        "IBM NetVista|NetVista",                            <br> 
        "IBM NetVista thin client|NetVista thin client",    <br> 
        "Ambra|Ispirati",                                   <br> 
        "IBM Monitors|Monitors",                            <br>  <br> 
    
    2) OPTIONS key  <br> 
       Create the "OPTIONS" key in the GDBM database. The data for this key 
       will be the JavaScript categories and subcat arrays.  For non-JavaScript 
       pages, these arrays will be parsed at runtime to create static HTML option 
       lists (see tcGDBM.C).  <br> <br> 

    3) "category|subcategory" keys in the GDBM database.            
       The data for each key will be HTML <option> tags for use in a select
       list.  This list will be usd by both JavaScript and non-JavaScript 
       pages. <br> <br> 

    4) "category|subcategory|option" keys in the GDBM database.
       The data for each key will be the description of the option. <br> <br> 

    5) "BRANDS" key in the GDBM database.            
       The data for this key will be the JavaScript brands and families arrays.
       For non-JavaScript pages, these arrays will be parsed at runtime to
       create static HTML option lists (see tcGDBM.C). <br> <br> 

    6) "brand" keys in the GDBM database.            
       The data will be the name to display to the user for the brand
       (i.e., "IBM Thinkpad" = "Notebook & PDA").  <br> <br> 

    7) "brand|family" keys in the GDBM database.            
       The data for these keys will be the JavaScript machines and models 
       arrays.  For non-JavaScript pages, these arrays will be parsed at
       runtime to create static HTML option lists (see tcGDBM.C). <br> <br> 

    8) "brand|family|machineType|model" keys in the GDBM database.
       The data for each key will be the description of the machine. <br> <br> 

 * @author  Steve Wang                                     
 */

public class BuildProfileDb extends Object {
    /** 
     * IBM Option brand
     */
    public final static String OPTION_BRAND = "IBM Options"; 

    public BuildProfileDb() {
    }
    /**
     * Constructor
     */
    public BuildProfileDb (Connection conn, int website, NavgenIntf nav) {
        this.website = website; 
        this.conn    = conn; 
        navgen       = nav; 
        try {
            dbFileName = BatchInit.getGdbmFile(Constants.ESUPPORT_PROFILE_TYPE)
                       + BatchInit.getProfileDbFileName() /*profile.db*/; 
        } catch (Exception e) {
            BatchUtil.logit("Could not find db file name for profile. Use ./db.tmp ", Constants.INFORMATIONAL, 0); 
            dbFileName = "./profile.db"; 
        }
        buildProfileBrandsTable(); 
    }

    /**
     * The website the profile db is saved to. Currently only esupport. We 
     * will use a integer for profiling in the navgen.gdbm table. 
     */
    private int website = 100; 

    /**
     * A database connection 
     */
    private Connection conn = null; 

    /**
     * A navgen interface
     */
    private NavgenIntf navgen = null; 

    /**
     * db file name
     */
    private String dbFileName = null; 
    
    /** 
     * A list of brands and displays in a 2 dimentional String array
     */
    private String brandNameDefs[][] = {
            {"IBM Aptiva", "Aptiva PS/1"},
            {"IBM PC", "Business PCs"},
            {"IBM ThinkPad", "Notebooks and PC Companions"},
            {"IBM PC Server", "Servers"},
            {"IBM IntelliStation", "IntelliStation"},
            {"IBM Network Station", "Network Station thin client"},
            {"IBM NetVista", "NetVista"},
            {"IBM NetVista thin client", "NetVista thin client"},
            {"Ambra", "Ispirati"},
            {"IBM Monitors", "Monitors"}
    }; 

    /**
     * A hashtable with profiling brand as key and its dispaly as value
     */
    private Hashtable hProfileBrands = new Hashtable(); 

    /**
     * Construct the profiling brand hashtable. Key is the brand name 
     * and display is the value 
     *
     * @param none
     * @return none
     */
    public void buildProfileBrandsTable () {
        BatchUtil.logit("Create profile Brands table for website " + website, Constants.INFORMATIONAL, 0); 
        for (int i=0; i<brandNameDefs.length; i++) {
            hProfileBrands.put(new String(brandNameDefs[i][0]), new String(brandNameDefs[i][1]));
        }
    }
    
    /**
     * Save the Brand display list gdbm keys:             
     * Brand  (key)  Display on profiling  (value)   
     * e.g., "IBM Aptiva|Aptiva PS/1"   
     *
     * @param none
     * @return none
     */
    public void saveBrandDisplay () throws WriteGdbmException {
        BatchUtil.logit("Save brand display for website " + website, Constants.INFORMATIONAL, 0); 
        for (int i=0; i<brandNameDefs.length; i++) {
            BuildDb2.writeGdbm(conn, brandNameDefs[i][0], brandNameDefs[i][1], dbFileName); 
        }
    }

    /**
     * Return a JavaScript array containing the brands.  This array will contain
     * the GDBM key of the brand & the name it is to be displayed as.
     * This array is built based on the contents of the brandList vector:
     * - Only brands in brandList are put in the array
     * - The brands are listed in the order they appear in brandList
     * - Each brand is written as "key value|display value" 
     *   (i.e., "IBM Intellistation|Workstations")
     *   The javascript code will know how to parse the key/display values.
     *
     * @param none
     * @return none
     */
    private String getBrandKeysAsArray() {
        String strout = "new Array(";
        for (int i=0; i<brandNameDefs.length; i++) {
            strout += "\"" + brandNameDefs[i][0] + '|' + brandNameDefs[i][1] + "\"";
            if (i<brandNameDefs.length-1) {
                strout += ","; 
            }
        }
        strout += ");"; 
        return strout; 
    }

    /**
     * Return the array string values in the sorted set as a Javascript array
     *
     * @param ts A sorted set of string values
     * @return String A javascript array
     */
    private String getSetAsArray(Collection ts) {
        return "new Array(" + BatchUtil.getLists(ts, "\"", "\"", ",") + ");";
    }

    /**
     * Format the options for a given category, subcategory as HTML <option>
     * tags for a select list.  Each <option> value will be the option name.
     * The <option> text will be the option name followed by the option
     * description.  If there is only one option for the category/subcategory,
     * then it will be pre-selected.
     *
     * @param bfmk The BFMachineKey
     * @param optionSet The options set for the option brand
     * @param prodMap The product map containing product description
     */                   //key = category|subcategory set=optionSubcatMap(key)
    private String getOptionSelectionList(BFMachineKey bfmk, TreeSet optionSet, TreeMap prodMap) {
        boolean preSelect = (optionSet.size()==1)? true:false; 
        String strout  = "";  

        // Iterate through each option
        Iterator it_option = optionSet.iterator();
        while (it_option.hasNext()) {
            CategoryRec option = (CategoryRec)it_option.next(); 
            BFMModelKey bfmmk = new BFMModelKey(bfmk.brandInd, bfmk.familyInd, bfmk.machine, option.getDescript()); 
            if (preSelect) {
                strout += "<option value=\"" + option.getDescript() + "\" selected>";
            } else {
                strout += "<option value=\"" + option.getDescript() + "\">";
            }
            strout += option + "&nbsp;&nbsp;&nbsp; " + (String)prodMap.get(bfmmk) + "</option>\n";  
        }//end of while it_option
        return strout; 
    }

    /**
     * Iterate through all products to get brand, family, machine and model data
     * 
     * @param brandSet The brandSet containing all brands in set 
     * @param brandChild The brandChildMap
     * @param familyChild The familyChildMap
     * @param machineChild The machineChildMap
     * @param prodMap The product map containing description
     * @return none
     */
    public void saveBrandKeys(TreeSet brandSet, TreeMap brandChild, TreeMap familyChild, 
                            TreeMap machineChild, TreeMap prodMap) throws WriteGdbmException {
        BatchUtil.logit("Save brand keys for website " + website, Constants.INFORMATIONAL, 0); 
        // Iterate through each brand
        String family_brand = ""; 
        Iterator it_b = brandSet.iterator();
        while (it_b.hasNext()) {
            CategoryRec brand = (CategoryRec)it_b.next(); 

            BatchUtil.logit("Saving for brand " + brand.getDescript(), Constants.DEBUG, 0); 
            // Process the options brand and jump to the next brand when finishing
            if (brand.getDescript().compareTo(OPTION_BRAND)==0) {
                saveOptionKeys(brand, brandChild, familyChild, machineChild, prodMap);
                continue; 
            }

            // Skip the brand not in brandNameDefs 
            if (hProfileBrands.get(brand.getDescript())==null) {
                continue;
            }

            BrandKey bk = new BrandKey(brand.getInd()); 
            if (brandChild.containsKey(bk)) {
                TreeSet familySet   = (TreeSet)brandChild.get(bk); 
                family_brand += "families[\"" + brand.getDescript() + "\"] = " + getSetAsArray(familySet) + "\n";
                
                String machine_set = "";
                // Iterate through each family
                Iterator it_f = familySet.iterator();
                while (it_f.hasNext()) {
                    CategoryRec family = (CategoryRec)it_f.next(); 
                    BrandFamilyKey bfk = new BrandFamilyKey(brand.getInd(), family.getInd()); 
                    if (familyChild.containsKey(bfk)) {
                        TreeSet machineSet = (TreeSet)familyChild.get(bfk);
                        machine_set += "var machines = " + getSetAsArray( machineSet ) + "\n" + "var models = new Array();\n";
                        
                        String model_set = ""; 
                        // Iterate through each machine
                        Iterator it_m = machineSet.iterator();
                        while (it_m.hasNext()) {
                            CategoryRec machine = (CategoryRec)it_m.next();
                            BFMachineKey bfmk = new BFMachineKey(brand.getInd(), family.getInd(), machine.getDescript());
                            if (machineChild.containsKey(bfmk)) {
                                TreeSet modelSet = (TreeSet)machineChild.get(bfmk);
                                model_set += "models[\"" + machine.getDescript() + "\"] = " + getSetAsArray(modelSet) + "\n";
                                
                                // Iterate through each model
                                Iterator it_mdl = modelSet.iterator();
                                while (it_mdl.hasNext()) {
                                    CategoryRec model = (CategoryRec)it_mdl.next();
                                    //------------------------------------------------------------------------
                                    // Create the "brand|family|machineType|model" keys in the GDBM database.
                                    // The data for each key will be the description of the machine.
                                    //------------------------------------------------------------------------

                                    String gkey = brand.getDescript()   + NavgenIntf.PIPESEP  
                                                + family.getDescript()  + NavgenIntf.PIPESEP  
                                                + machine.getDescript() + NavgenIntf.PIPESEP 
                                                + model.getDescript();
                                    String desc = (String)prodMap.get(new BFMModelKey(brand.getInd(), 
                                                                            family.getInd(),
                                                                            machine.getDescript(),
                                                                            model.getDescript()));
                                    BuildDb2.writeGdbm(conn, gkey, desc, dbFileName); 
                                }//end of while it_mdl
                            }
                        }//end of while it_m
                        //------------------------------------------------------------------------
                        // Create the "brand|family" keys in the GDBM database.            
                        // The data for these keys will be the JavaScript machines and models 
                        // arrays.  For non-JavaScript pages, these arrays will be parsed at
                        // runtime to create static HTML option lists (see tcGDBM.C).
                        //------------------------------------------------------------------------
                        String gkey = brand.getDescript() + NavgenIntf.PIPESEP + family.getDescript();
                        String gval = machine_set + model_set;
                        BuildDb2.writeGdbm(conn, gkey, gval, dbFileName); 
                    }
                }//end of while it_f
            }
        }//end of while it_b
        //------------------------------------------------------------------------
        // Create the "BRANDS" key in the GDBM database.            
        // The data for this key will be the JavaScript brands and families arrays.
        // For non-JavaScript pages, these arrays will be parsed at runtime to
        // create static HTML option lists (see tcGDBM.C).
        //------------------------------------------------------------------------
        String gkey = "BRANDS";
        String gval = "var brands = " + getBrandKeysAsArray() + "\n" + "var families = new Array();\n" + family_brand; 
        BuildDb2.writeGdbm(conn, gkey, gval, dbFileName); 
    }

    /** 
     * Save all option related gdbm key-val
     *
     * @param brand The option brand categoryRec
     * @param brandChild The brandChildMap
     * @param familyChild The familyChildMap
     * @param machineChild The machineChildMap
     * @param prodMap The product map containing description
     * @return none
     */
    // brand -- option brand
    // family  - category      - optionCatSet
    // machine - subcategory   - optionSubcatMap  BFMkey
    // model   - option num    - optionMap        BFMMKey
    public void saveOptionKeys (CategoryRec brand, TreeMap brandChild, TreeMap familyChild, 
                            TreeMap machineChild, TreeMap prodMap) throws WriteGdbmException {
        BatchUtil.logit("Save option keys for website " + website, Constants.DEBUG, 0); 
        // Get the option brand key 
        BrandKey bk = new BrandKey(brand.getInd()); 
        TreeSet optionCatSet   = (TreeSet)brandChild.get(bk); 

        String gkey = "";
        String gval = ""; 
        String subcat_set = "";
        // Iterate through each category
        Iterator it_cat = optionCatSet.iterator();
        while (it_cat.hasNext()) {
            CategoryRec cat = (CategoryRec)it_cat.next(); 
            BrandFamilyKey bfk = new BrandFamilyKey(brand.getInd(), cat.getInd()); 
            if (familyChild.containsKey(bfk)) {
                TreeSet subCatSet = (TreeSet)familyChild.get(bfk);

                subcat_set += "subcat[\"" + cat.getDescript() + "\"] = " + getSetAsArray(subCatSet) + "\n"; 
                // Iterate through each subcat
                Iterator it_subcat = subCatSet.iterator();
                while (it_subcat.hasNext()) {
                    CategoryRec subcat = (CategoryRec)it_subcat.next();
                    BFMachineKey bfmk = new BFMachineKey(brand.getInd(), cat.getInd(), subcat.getDescript());
                    if (machineChild.containsKey(bfmk)) {
                        TreeSet optionSet = (TreeSet)machineChild.get(bfmk);

                        //------------------------------------------------------------------------
                        // Create the "category|subcategory" keys in the GDBM database.            
                        // The data for each key will be HTML <option> tags for use in a select
                        // list.  This list will be usd by both JavaScript and non-JavaScript 
                        // pages.
                        //------------------------------------------------------------------------
                        gkey = cat.getDescript() + NavgenIntf.PIPESEP + subcat.getDescript();
                        gval = getOptionSelectionList(bfmk, optionSet, prodMap);  
                        BuildDb2.writeGdbm(conn, gkey, gval, dbFileName); 

                        // Iterate through each option
                        Iterator it_option = optionSet.iterator();
                        while (it_option.hasNext()) {
                            CategoryRec option = (CategoryRec)it_option.next();
                            //------------------------------------------------------------------------
                            // Create the "category|subcategory|option" keys in the GDBM database.
                            // The data for each key will be the description of the option.
                            //------------------------------------------------------------------------
                            gkey = cat.getDescript()    + NavgenIntf.PIPESEP  
                                        + subcat.getDescript() + NavgenIntf.PIPESEP  
                                        + option.getDescript(); 
                            String desc = (String)prodMap.get(new BFMModelKey(brand.getInd(),
                                                                       cat.getInd(), 
                                                                       subcat.getDescript(),
                                                                       option.getDescript())
                                                              ); 
                            BuildDb2.writeGdbm(conn, gkey, desc, dbFileName); 
                        }//end of while it_option
                    }
                }//end of while it_subcat
            }
        }//end of while it_cat
        //------------------------------------------------------------------------
        // Create the "OPTIONS" key in the GDBM database.            
        // The data for this key will be the JavaScript categories and subcat
        // arrays.  For non-JavaScript pages, these arrays will be parsed at 
        // runtime to create static HTML option lists (see tcGDBM.C).
        //------------------------------------------------------------------------
        gkey = "OPTIONS";
        gval = "var categories = " + getSetAsArray(optionCatSet) + "\n" + "var subcat = new Array();\n" + subcat_set; 
        BuildDb2.writeGdbm(conn, gkey, gval, dbFileName); 
    }
}

