package com.ibm.nzna.projects.batch.wwcathc;

import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.shared.ftp.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;
import com.ibm.appsure.*;
import java.io.* ;
import java.util.* ;
import java.sql.*;

      /**
	*<pre>
	*The wwcathc is a batch that generates URLs for Marketing Catalog Product Specifications
	*documents which reside on a Domino server.   Instead of connecting to a Lotus Notes database
	* to pull the data, the data will be exported into a flat text file and ftp'ed to a server of our choosing.
	* WWCatHC will have to parse this flat text file and create the URLs.  The URL for each document
	* is based on  the value of the PartNum_T field associated with that document.  Also, the URL
	* for each document is relative to the Notes database.  WWCatHC will write the key and the
	* value to the NAVGEN.GDBM table instead of writing directly to a gdbm file.
	*
	*The key string is a combination of "mk" + <productNum>, where <productNum> is the type and
	* model for a machine or the partNum for an option.  An example  is mk04H7122 where 04H7122 is
	* the part Number for a wireless modem.
	*
	*The URL or value string is formatted as follows:
	* "mkURL" + "\x0" +"/us/products.nsf/$wwwPartNumLookup/_" + "\x0" + <productNum> + "\x0"
	* where <productNum> is the type and model for a machine or the partNum for an option.  An example is
	* mkURL = /us/products.nsf/$wwwPartNumLookup/_04H7122 where 04H7122 is the part number
	* for a wireless modem.  Each key-value pairs are written to the NAVGEN.GDBM table where key is
	* the GKEY and value/URL is the GVAL.
	*
	* </pre>
	*
	* @version 1.0, 7/13/2001
	* @author  Tracey Watkins
        **/

public class WWCatHC extends Batch implements FailureTypes {

	/*********************************MEMBERS************************************/
	/**
	 * This flag is used to determine whether or not  the batch should continue.
	 */
	private boolean okToContinue = true;

	/**
	 * This holds all properties from a property file as well as the command line args.
	 */
	private Properties  props = new Properties() ;

	/**
	   *This is the log file for the batch.  All progress, errors and severe errors should be logged here.
	   */
	private BatchLogSystem logFile = null  ;

	/**
	   *This is the timestamp of the last successful run of the batch.
	   */
	private Timestamp lastRun = null  ;

	/**
	* This is the appCode used for logging to the AppSure database.
	*/
	private static final String APPCODE = "WWCATHC";

	/**
	* This is the run index used for logging to the AppSure database.
	*/
	private int runIndex = 0;

	/**
	* This is the maximum number of steps for this batch.
	*/
	final static int MAX_STEPS = 7;

	/**
	* This is the current step of the batch.
	*/
	private int step = 0;

	/**
	* This is the current step of the batch.
	*/
	private int debugLevel = 0;

	/*************************************METHODS****************************************/
	/**
	* main
	*
	 * This is the starting point of the batch.
	 *
	 *@param	String[] Command line parameters
	 *@return	Nothing
	 */
	public static void main( String[] args ) {
		System.out.println("In main");
		WWCatHC wwcathc = new WWCatHC( args );
		System.out.println("wwcathc created");
		if ( wwcathc.getOkToContinue () )
			wwcathc.startBatch() ;
		System.out.println("batch started");
		if ( wwcathc.getOkToContinue() )
			wwcathc.stopBatch();
		System.out.println("batch stopped");
		System.exit( wwcathc.getOkToContinue() ? 1 : 0 ) ;
	}	 //end of main method

	/**
	* Constructor
	*    	This method will create and initialize our batch.
	*  @param	None
	*  @return	Nothing
	*/
	public WWCatHC (String[] args) {
		logFile = new BatchLogSystem( "logs" + System.getProperty("file.separator") +"wwcathc", this );
		okToContinue = initialize( args );
	}  //end of constructor

	/**
	* getOkToContinue
	*    This method will return TRUE if it is ok to start the batch.
	*  @param None
	*  @return True if the batch can continue, otherwise false.
	*/
	public boolean getOkToContinue () {
		return( okToContinue ) ;
	}


	/**
	*This class is used to hold three string values:  a key, a value and a url.  The key
	* and value strings will be inserted into the NAVGEN.GDBM table.
	*/
	class GDBMString {
		/**
		*This holds the key to be used in the gdbm file.
		*/
		String keyStr = null;
		/**
		* This holds the value or the URL to the document to be used in the gdbm file.
		*/
		String valueStr = null;
		/**
		* This holds the url to be logged.  It is equivalent to the valueStr without the
		* null separator.
		*/
		String urlStr = null;
		public GDBMString() {
		}
	}	//end of class GDBMString

	/**
	 *initialize
	 *This method will initalize the batch.
	 *
	 *@param	None
	 *@return	  True if successful, otherwise, false.
	 */
	public boolean initialize( String[] args ) {
		boolean rc = true ;
		try {
			logFile.logProgress( "Initializing the batch.", MAX_STEPS, step++ );
                        String fileSep = System.getProperty("file.separator");
//                        String path = "d"+fileSep +"opicm"+fileSep+"com" +fileSep + "ibm"+fileSep +"nzna"+fileSep+"properties"+fileSep;
//			StandardPropertyLoader stdPropLdr = new StandardPropertyLoader("wwcathc.properties", args);
//                        props = stdPropLdr.getProperties();

			String path = StandardPropertyLoader.getPath();
//			System.out.println("path = " + path);
//			props = StandardPropertyLoader.load( propFile );

			props = this.load("wwcathc.properties", args);

			if ( props.isEmpty() ) {
				logFile.logSevereError( "Unable to read properties file or command line parameters.");
				return( rc = false );
			}
                        else {
//                            System.out.println("Props path : " + path);
//                            System.out.println("Properties are ... " );
//                            Enumeration e = props.propertyNames();
//                            while (e.hasMoreElements()) {
//                                System.out.println((String)e.nextElement());
//                            }
                        }
			logFile.log("props loaded");

			// These variables are used for connecting/loggins to the appSure database.
			String appSureDbName =props.getProperty( "db.appsure.name" );
			String appSureServerName = props.getProperty( "db.appsure.host" );
			String appSureId = props.getProperty( "db.appsure.uid" );
			String appSurePassword = props.getProperty( "db.appsure.pw" );
			String appSurePort = props.getProperty( "db.appsure.port" );
			boolean useAppDriver = new Boolean( props.getProperty( "db.appsure.appDriver" ) ).booleanValue();
			boolean DEBUG = new Boolean( props.getProperty( "wwcathc.DEBUG" ) ).booleanValue();
			runIndex=  new Integer(props.getProperty( "db.appsure.run.index" ) ).intValue();
			debugLevel = new Integer( props.getProperty( "wwcathc.debug.level" ) ).intValue();

			//Create a connection to the AppSure database and start the application.
			DatabaseSystem dbSys = new DatabaseSystem (DEBUG, logFile, useAppDriver );
//		if ( dbSys.createConnection(DatabaseSystem.APPSURE, appSureDbName, appSureServerName,
//										appSurePort, appSureId, appSurePassword) ) {
//				logFile.logProgress( "Connection to the AppSure database has been created successfully." );
//				rc = AppSure.startApp(dbSys.getConnection( DatabaseSystem.APPSURE ), APPCODE, runIndex);
//			} else {
//				logFile.logError( "Unable to connect to the AppSure database." );
//				rc = false ;
//			}

		//Get quest db properties from property file.
		String dbHost = props.getProperty("db.quest.host" );
		String dbName = props.getProperty("db.quest.name" );
		String dbPort = props.getProperty("db.quest.port" );
		String dbUser = props.getProperty("db.quest.uid" );
		String dbPassword = props.getProperty("db.quest.pwd" );
	        useAppDriver =  new Boolean( props.getProperty("db.quest.appDriver") ).booleanValue() ;

		logFile.log( "Host: " + dbHost + "\tName: " + dbName + "\tPort: " + dbPort + "\nUser: " +  dbUser +
					 "\tPwd: " + dbPassword + "\nappDriver: " + useAppDriver  );

		//Create a connection to the database.
		if ( DatabaseSystem.createConnection(DatabaseSystem.QUEST, dbName, dbHost, dbPort, dbUser, dbPassword) ) {
//                rc = dbSys.createConnection(dbSys.QUEST, dbName, dbHost, dbPort, dbUser, dbPassword);
//                System.out.println( "rc: " +rc);
//                if (rc) {
			logFile.log( "Connection successfully created to the quest database." );
		} else {
                    logFile.logError( "Unable to connect to the Quest database." );
//				AppSure.logFailure( failureType, logFile, logFile.getProgressFile() );
			return( rc = false );
		}

		//Get the last timestamp for the successful run.
		String qryStr = "SELECT strconst FROM tigris.constants WHERE constant='WWCATHC_TIMESTAMP'";
		SQLMethod sqlMethod = new SQLMethod(1, "WWCatHC",  2);
		Statement stmt = DatabaseSystem.getConnection(DatabaseSystem.QUEST).createStatement();
		ResultSet rs = stmt.executeQuery( qryStr );
		if (rs.next())
		    lastRun = rs.getTimestamp(1);
                sqlMethod.close();
	} //end of try
 	catch ( Exception e ) {
		rc = false;
		logFile.logSevereError("Exception caught during writeToDatabase(). " );
		logFile.logSevereError( e.getMessage() );
                e.printStackTrace();
//			AppSure.logFailure( failureType, logFile, logFile.getProgressFile() );
	}

		return rc;
	}  //end of method initialize


	/*------------------------------------------------------------*/
	/* load
	 */
	/**
	 *    This method should parse through the given parameters
	 *
	 *  @param args Command line arguments as an array of String
	 *  @return True if successful, otherwise false.
	 *//*---------------------------------------------------------*/
	public Properties  load ( String propFile, String[] args ) {
		logFile.log( "Loading the properties/command line args." );
		Properties properties = null;
		try {
			properties = StandardPropertyLoader.load( propFile );
			String tempStr = null;
			String key = null;
			String value = null;
			for ( int i=0; i < args.length; i++ ) {
				tempStr = args[i];
				key = tempStr.substring(1, tempStr.indexOf(":"));
				value = tempStr.substring(tempStr.indexOf(":")+1);
				properties.put(key, value);
				logFile.log( key + "  =  " + properties.get(key));
			}
		} catch ( Exception e ) {
                    logFile.log(e.getMessage());
			e.printStackTrace();
		}
		return properties;
	} //end of load method


	/**
	* startBatch
	*
	* This method should be called when the batch starts.
	*@param		None
	*@return	  Nothing
	 */
	public void startBatch() {
		logFile.logProgress( "Processing...." , MAX_STEPS, step++);
		try {
			okToContinue = parsePartNumFile();
		} //end of try
		catch ( Exception e ) {
                    logFile.logSevereError("Exception caught during batchStart(). " );
                    logFile.logSevereError( e );
//			AppSure.logFailure( failureType, logFile, logFile.getProgressFile() " );
		}
		logFile.logProgress( "WWCatHC completed successfully: " + getOkToContinue() , MAX_STEPS, step++ );
	} //end of method startBatch()


	/**
	* stopBatch
	* This method should be called when the batch stops.
	*
	*@param		None
	*@return	  Nothing
	 */
	public void stopBatch() {
		try {
			logFile.logProgress( "WWCatHC shutting down.", MAX_STEPS, step++ );
//			AppSure.shutDown( APPCODE, runIndex );
		} //end of try
		catch ( Exception e ) {
                    logFile.logSevereError("Exception caught while stopping the batch. " );
                    logFile.logSevereError( e );
//			AppSure.logFailure(failureType, logFile, logFile.getProgressFile() );
		}
	} //end of method stopBatch()


	/*------------------------------------------------------------*/
	/* ftpInFile
	 */
	/**
	 *    This method will attempt to ftp the input file.
	 *
	 *  @param String 	Name of the file to be ftp'ed.
	 *
	 *  @return TRUE if successful, otherwise false.
	 *//*---------------------------------------------------------*/
	private boolean ftpInFile ( String filename ) {
		logFile.logProgress("Ftping the input file.", MAX_STEPS, step++);
		boolean rc = false ;
		Ftp ftp = new Ftp();
		try {
			String   ftpHost = props.getProperty("wwcathc.ftp.host");
			String   ftpUser = props.getProperty("wwcathc.ftp.uid");
			String   ftpPwd = props.getProperty("wwcathc.ftp.pwd");
			String   ftpPath = props.getProperty("wwcathc.ftp.path");
			logFile.log("FTP Host = " + ftpHost + " User = " + ftpUser + " PW = " + ftpPwd + " Path = " + ftpPath);
			ftp.connect( ftpHost );
			logFile.log("Connected to : " + ftpHost);
			ftp.login( ftpUser, ftpPwd );
			logFile.log("     as user: " + ftpUser);
			logFile.log(ftp.getReply().toString());
			ftp.cd(ftpPath);
			logFile.log("Changed to directory: " + ftpPath);
			logFile.log(ftp.getReply().toString());
			ftp.get( filename);
			logFile.log(ftp.getReply().toString());
			logFile.log("FTPed file " +  filename + " from server " + ftpHost);
			ftp.logout();
			ftp.disconnect();
			logFile.log("Disconnected from ftp site.");
			rc = true ;
		} //end of try
		catch ( Exception e ) {
			logFile.log(debugLevel, e);
			rc = false;
		}  //end of catch
		System.out.println( "RC ftpInputFile: " + rc );
		return( rc ) ;
	} //end of method ftpInputFile


	/**
	* parsePartNumFile
	*
	*This method will create a Vector of part numbers retrieved from a flat text file.
	*
	*@param		None
	*@return	  Vector of GBDMString objects that are key - value pairs
	 */
	public boolean parsePartNumFile() {
		logFile.logProgress( "Parsing the part number input file.", MAX_STEPS, step++ );
		boolean rc = true;
		try {
			String NULLSEP = props.getProperty( "wwcathc.NULLSEP" );
			String KEYSTR = props.getProperty( "wwcathc.KEYSTR" );
			String VALUESTR = props.getProperty( "wwcathc.VALUESTR" );
			String URLSTR = props.getProperty( "wwcathc.URLSTR" );
			String FILESEP = System.getProperty( "file.separator" );
			String fileName = props.getProperty( "wwcathc.file.name" );

			//Use this if the input file is located on another machine.
//			rc = this.ftpInFile( props.getProperty( "wwcathc.file.name" ) );

			String partNumFile =FileUtil.readFile( fileName );

			//Use this if the input file is located on this machine.
			File inFile = new File( fileName );
			Timestamp lastModified = new Timestamp( inFile.lastModified() );
			if ( lastModified.before( lastRun) ) {
			    logFile.logSevereError("The input file " + fileName + " has not changed since the last run of WWCatHC. ");
			    logFile.logSevereError("It was last modified on " + lastModified );
//			AppSure.logFailure( failureType, logFile, logFile.getProgressFile() );

			    rc = false;
			    return rc;
			}
			logFile.log("The input file " +  fileName + " was last modified on " + lastModified);
			Vector gdbmVec = new Vector();
			String partNum_T = null;
			GDBMString gdbmStr = null;
			StringTokenizer strTok = new StringTokenizer( partNumFile );

			logFile.log( "There are " + strTok.countTokens() + " part numbers in the file." );
                        Vector parts = new Vector();
			while ( strTok.hasMoreTokens() ) {
				partNum_T = strTok.nextToken();
                                if (parts.contains(partNum_T)) {
                                    logFile.log(partNum_T + " is a duplicate.");
                                } else {
                                    parts.addElement(partNum_T);
                                    gdbmStr = new GDBMString();
                                    gdbmStr.keyStr = KEYSTR + partNum_T;
                                    gdbmStr.valueStr = VALUESTR + partNum_T + NULLSEP;
                                    gdbmStr.urlStr = URLSTR + partNum_T;
                                    gdbmVec.addElement(gdbmStr);
                                } //end of else
			}  //end of while

			//Used this block for debugging.
//                        for ( int counter=0; counter < gdbmVec.size(); counter++ ) {
//				GDBMString gStr = (GDBMString)gdbmVec.elementAt( counter );
//				logFile.log( "key: " + gStr.keyStr + "value: " + gStr.urlStr);
//			}

			//Trim the vector.
			gdbmVec.trimToSize();

			//Verify that the vector is not empty.  If is is, then return false with reason being problem with parsePartNumFile.
			if ( gdbmVec.size() == 0 ) {
				logFile.logSevereError( "There was no data in the file." );
				rc = false;
			} else {
				rc = this.writeToDatabase( gdbmVec );
			}  //end of if
		} //end of try
		catch ( Exception e ) {
                    logFile.logSevereError("Exception caught during parsePartNumFile(). " );
                    logFile.logSevereError( e );
//			AppSure.logFailure( failureType, logFile, logFile.getProgressFile() );
		}
		return rc;
	}  //end of method getPartNumbers

	/**
	   *  writeToDatabase(Vector)
	   *
	   *This method will connect to the quest database and insert a record into the NavGen.GDGBM table for each
	   *keyString-valueString value pairs in the given Vector.  It will return true if successful; otherwise it will
	   *return false.
	   *
	   *@param	Vector 		The vector of keyString-valueString value pairs to be inserted into quest.
	   *@return	True if successful, otherwise false.
	   */
	public boolean writeToDatabase(Vector gdbmVector) {
		logFile.logProgress( "Writing values to the database.", MAX_STEPS, step++ );
		boolean rc = true;
		try {

			//Get db properties from property file.
			String dbHost = props.getProperty("db.quest.host" );
			String dbName = props.getProperty("db.quest.name" );
			String dbPort = props.getProperty("db.quest.port" );
			String dbUser = props.getProperty("db.quest.uid" );
			String dbPassword = props.getProperty("db.quest.pwd" );
			boolean useAppDriver =  new Boolean( props.getProperty("db.quest.appDriver") ).booleanValue() ;
			int SITEIND = new Integer( props.getProperty( "wwcathc.SITEIND" ) ).intValue();
			Vector gdbmVec = gdbmVector;

			logFile.log( "Host: " + dbHost + "\tName: " + dbName + "\tPort: " + dbPort + "\nUser: " +  dbUser +
						 "\tPwd: " + dbPassword + "\nappDriver: " + useAppDriver + "\tSiteInd: " + SITEIND );

			//Create a connection to the database.
			if ( DatabaseSystem.createConnection(DatabaseSystem.QUEST, dbName, dbHost, dbPort, dbUser, dbPassword) ) {
				logFile.log( "Connection successfully created to the quest database." );
			} else {
                            logFile.logError( "Unable to connect to the Quest database." );
//				AppSure.logFailure( failureType, logFile, logFile.getProgressFile() );
				return( rc = false );
			}

			//Create the query used in the Prepared Statement.
			String deleteQryStr = "DELETE FROM navgen.gdbm where gkey like 'mk%' and gval like 'mkURL%'";
			String qryStr = "INSERT INTO navgen.gdbm (siteind, gkey, gval) VALUES (" + SITEIND + ", ?, ?) " ;
			String updateTimestampQry = "UPDATE tigris.constants SET STRCONST=CURRENT TIMESTAMP WHERE constant='WWCATHC_TIMESTAMP'";
			SQLMethod sqlMethod = new SQLMethod(1, "WWCatHC",  10);
			QuestStatement questDelStmt = new QuestStatement( sqlMethod.createStatement( ) );
			if ( questDelStmt.executeUpdate(deleteQryStr) != 1 ) {
				logFile.log( "Unable to delete old Marketing Catalog URLs." );
			} else {
				logFile.log( "Old Marketing Catalog URLS deleted from Navgen.gdbm" ) ;
			}
                        questDelStmt.close();

			QuestPreparedStatement questPrepStmt = new QuestPreparedStatement( sqlMethod.createPreparedStatement(qryStr) );
			GDBMString gdbmStr = new GDBMString();

			//Execute the query for each element in the Vector.
			for ( int counter=0;  counter< gdbmVec.size(); counter++ ) {
				gdbmStr = (GDBMString)gdbmVec.elementAt( counter );
				questPrepStmt.setString(1, gdbmStr.keyStr);
				questPrepStmt.setString(2, gdbmStr.valueStr);
				if ( questPrepStmt.executeUpdate() !=1 ) {
					logFile.logError("Unable to insert : " + gdbmStr.keyStr + " - " + gdbmStr.urlStr);
				} else
                                    logFile.log("Inserted : " + gdbmStr.keyStr +  " - " + gdbmStr.urlStr);
			} //end of for

			QuestPreparedStatement updateStmt = new QuestPreparedStatement( sqlMethod.createPreparedStatement( updateTimestampQry ) );
			if ( updateStmt.executeUpdate() != 1 ) {
				logFile.log( "Unable to update WWCATHC_TIMESTAMP." );
			} else {
				logFile.log( "WWCATHC_TIMESTAMP updated in tigris.constants." ) ;
			}

                        sqlMethod.close();
		} //end of try
		catch ( Exception e ) {
			rc = false;
			logFile.logSevereError("Exception caught during writeToDatabase(). " );
			logFile.logSevereError( e );
//			AppSure.logFailure( failureType, logFile, logFile.getProgressFile() );
		}

		return rc;
	}  //end of method writeToDatabase(Vector)

} //end of class WWCatHC
