package com.ibm.nzna.projects.common.quest ;
/*-----------------------------------------------------------*/
/*   LocaleRec                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LocaleRec                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This record holds information on a locale. This locale can be
 * a single geography, a list of geographies or countries or world wide. This
 * object also has usefull methods in determining if it represents the world,
 * or if a specific geography has all countries assigned to it etc..  
 *
 * @version 1.0, 7/25/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   LocaleRec                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class LocaleRec extends PersistentRec implements java.io.Serializable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean worldWide      = false ;            // If true, this locale is World Wide
   private Vector  geoVec         = null ;             // Holds all of our LocaleGeoRecs. One for each geography

   /**
     * The default constructor. Please note that to use this constructor
     * use must already have a TypeList object instantiated
     */
   public LocaleRec() {
      if ( geoVec == null )
         createGeoVec() ;
   }

   /**
     * This constructor will initialize itself by reading in the
     * geographies from TIGRIS.TYPEGEOGRAPHY with the Connection
     * object passed in
     */
   public LocaleRec ( Connection conn ) throws SQLException {
      createGeoVec ( conn ) ;
   }



   public LocaleRec(LocaleRec localeRec) {
      if ( localeRec != null ) {
         this.worldWide      = localeRec.worldWide ;      
         updateRecStatus ( REC_NOCHANGE ) ;
         updateRecStatus ( localeRec.getRecStatus() ) ;

         if ( localeRec.geoVec != null ) {
            int i    = 0 ;
            int size = localeRec.geoVec.size() ;

            geoVec = new Vector ( size ) ;
            while ( i < size )
               geoVec.addElement ( localeRec.geoVec.elementAt(i++) ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* createGeoVec
    */
   /**
    *    This method will create our LocaleGeoRec
    *  vector.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createGeoVec () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If there is a TypeList instance, use it                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( TypeList.getInstance() != null ) {
         Vector typeGeoVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;
         int    i          = 0 ;
         int    length     = 0 ;

         try {

            length = typeGeoVec.size() ;

            /* Create our new vector and populate it */
            geoVec = new Vector ( length ) ;
            while ( i < length ) {
               if ( ((TypeGeoRec)typeGeoVec.elementAt(i)).getInd() != TypeList.GEO_WORLD_WIDE )
                  geoVec.addElement ( new LocaleGeoRec ( (TypeGeoRec) typeGeoVec.elementAt(i++ ) ) ) ;
               else
                  i++ ;
            }

         } catch ( NullPointerException e ) {
         } catch ( Exception e ) {
            LogSystem.log ( 1, e) ;
         }
      }
   }

   /*------------------------------------------------------------*/
   /* createGeoVec
    */
   /**
    *    This method will create our LocaleGeoRec
    *  vector by reading the information from the database
    *  
    *
    *  @param conn The Connection to read the information with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createGeoVec ( Connection conn ) throws SQLException {
      TypeGeoRec geoRec = null ;
      Statement  stmt   = conn.createStatement() ;
      ResultSet  rs     = stmt.executeQuery ( "SELECT GEOIND, DESCRIPT "+
                                              "FROM TIGRIS.TYPEGEOGRAPHY FOR FETCH ONLY" ) ;//WHERE GEOIND <> "+TypeList.GEO_WORLD_WIDE+" FOR FETCH ONLY" ) ;

      geoVec = new Vector ( 5 ) ;

      while ( rs.next() ) {
         if ( rs.getInt(1) != TypeList.GEO_WORLD_WIDE ) {
            geoRec = new TypeGeoRec ( rs.getInt(1), rs.getString(2).trim() ) ;
            geoRec.getCountryVec ( conn ) ;
            geoVec.addElement ( new LocaleGeoRec ( geoRec ) ) ;
         }
      }

      rs.close() ;
      stmt.close() ;

   }


   /*-----------------------------------------------------------*/
   /* geoCount
    */
   /**
    *    This method will return the number of items
    *  in the geoVec
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return items The number of items in the geoVec
    */
   public int geoCount () {
      int rc = 0 ;

      try {
         LocaleGeoRec localeGeoRec = null ;
         int          i            = 0 ;
         int          length       = geoVec.size() ;

         while ( i < length ) {
            localeGeoRec = (LocaleGeoRec) geoVec.elementAt(i) ;

            if ( ( localeGeoRec.isAll() ) ||
                 ( localeGeoRec.countryCount() > 0 ) ) {
               rc++ ;
            }
            i++ ;
         }

      } catch ( NullPointerException e ) {
      } catch ( Exception e ) {
         LogSystem.log ( 1, e);
      }

      return( rc ) ;
   } /* geoCount */



   /*-----------------------------------------------------------*/
   /* countryCount
    */
   /**
    *    This method will return the number of countries
    *  in the countryVec.
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return item The number of items in the countryVec
    */
   public int countryCount () {
      int rc = 0 ;

      try {
         LocaleGeoRec localeGeoRec = null ;
         int          i            = 0 ;
         int          length       = geoVec.size() ;

         while ( i < length ) {
            localeGeoRec = (LocaleGeoRec) geoVec.elementAt(i) ;

            rc+=localeGeoRec.countryCount() ;
            i++ ;
         }

      } catch ( NullPointerException e ) {
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* localeGeoRecFromGeo
    */
   /**
    *    This method will return the correct LocaleGeoRec
    *  that represents the specified TypeGeoRec passed in
    *  
    *
    *  @param geoRec The TypeGeoRec to look for
    *  @param None
    *  @param None
    *
    *  @return LocaleGeoRec
    *//*---------------------------------------------------------*/
   private LocaleGeoRec localeGeoRecFromGeo ( TypeGeoRec geoRec ) {
      int          i       = 0 ;
      int          length  = geoVec.size() ;
      LocaleGeoRec retRec  = null ;

      while ( ( i < length ) && ( retRec == null ) ) {
         if ( ((LocaleGeoRec)geoVec.elementAt(i)).equals ( geoRec ) )
            retRec = (LocaleGeoRec)geoVec.elementAt(i) ;
         i++ ;
      }
      return( retRec ) ;
   }


   /*------------------------------------------------------------*/
   /* localeGeoRecFromGeo
    */
   /**
    *    This method will return the correct LocaleGeoRec
    *  that represents the specified TypeGeoRec passed in
    *  
    *
    *  @param geoRec The TypeGeoRec to look for
    *  @param None
    *  @param None
    *
    *  @return LocaleGeoRec
    *//*---------------------------------------------------------*/
   private LocaleGeoRec localeGeoRecFromGeo ( int geoInd ) {
      int          i       = 0 ;
      int          length  = geoVec.size() ;
      LocaleGeoRec retRec  = null ;

      while ( ( i < length ) && ( retRec == null ) ) {
         if ( ((LocaleGeoRec)geoVec.elementAt(i)).getTypeGeoRec().getInd() == geoInd )
            retRec = (LocaleGeoRec)geoVec.elementAt(i) ;
         i++ ;
      }
      return( retRec ) ;
   }





   /*------------------------------------------------------------*/
   /* validateWorldWide
    */
   /**
    *    This method will validate if this locale is
    *  world wide or not. It will set the worldWide
    *  property to TRUE if we are, and FALSE if we aren't
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void validateWorldWide () {
      int     i       = 0 ;
      int     length  = geoVec.size() ;
      boolean allGeo  = true ;

      /* First, walk through the geoVecs and validate
         that they are each 'all' */
      while ( ( i < length ) && ( allGeo ) )
         allGeo = ( ((LocaleGeoRec)geoVec.elementAt(i++)).isAll() ) ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we did not have all geos with the 'allGeo', then     */
      /* look for the geoInd of WORLD_WIDE                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !allGeo ) {
         i=0 ;

         while ( ( i < length ) && ( !allGeo ) )
            allGeo = ( ((LocaleGeoRec)geoVec.elementAt(i++)).getTypeGeoRec().getInd()==TypeList.GEO_WORLD_WIDE ) ;
      }

      worldWide = allGeo ;
   }


   /*------------------------------------------------------------*/
   /* addCountry
    */
   /**
    *    This method will add a country to the list
    *  of associated countries.
    *  
    *
    *  @param countryRec The TypeCountryCodeRec to add 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCountry ( TypeCountryCodeRec countryRec ) {
      if ( countryRec != null ) {
         LocaleGeoRec geoRec = localeGeoRecFromGeo ( countryRec.getGeoInd() ) ;

         if ( geoRec != null ) {
            updateRecStatus ( REC_UPDATE ) ;
            geoRec.addCountry ( countryRec ) ;
            if ( geoRec.isAll() )
               validateWorldWide() ;
         }
      }
   }






   /*------------------------------------------------------------*/
   /* removeCountry
    */
   /**
    *    This method will remove a country from
    *  our list
    *  
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCountry ( TypeCountryCodeRec countryRec ) {
      if ( countryRec != null ) {
         LocaleGeoRec geoRec = localeGeoRecFromGeo ( (TypeGeoRec) TypeList.getInstance().objectFromInd ( countryRec.getGeoInd(), TypeList.TYPE_GEOGRAPHY ) ) ;

         if ( geoRec != null ) {
            updateRecStatus ( REC_UPDATE ) ;
            geoRec.removeCountry ( countryRec ) ;
            worldWide = false ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* removeGeography
    */
   /**
    *    This method will remove all countries associated
    *  to the geography passed.
    *  
    *
    *  @param typeGeoRec The TypeGeoRec to remove all countries associated to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeGeography ( TypeGeoRec typeGeoRec ) {
      LocaleGeoRec localeGeoRec = localeGeoRecFromGeo ( typeGeoRec ) ;

      updateRecStatus ( REC_UPDATE ) ;
      localeGeoRec.removeAll() ;
      worldWide = false ;
   }



   /*------------------------------------------------------------*/
   /* addGeography
    */
   /**
    *    This method will add all counties associated
    *  to the geography passed.
    *  
    *
    *  @param typeGeoRec The TypeGeoRec to add all countries associated to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addGeography ( TypeGeoRec typeGeoRec ) {
      LocaleGeoRec localeGeoRec = localeGeoRecFromGeo ( typeGeoRec ) ;

      updateRecStatus ( REC_UPDATE ) ;
      localeGeoRec.associateAll() ;
      validateWorldWide() ;
   }





   /*------------------------------------------------------------*/
   /* addGeography
    */
   /**
    *    This method will add the geography indicator
    *  to this record, including all countries associated
    *  with the geo ind
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addGeography ( int geoInd ) {
      addGeography ( (TypeGeoRec)TypeList.getInstance().objectFromInd ( geoInd, TypeList.TYPE_GEOGRAPHY ) ) ;
   }





   /*------------------------------------------------------------*/
   /* addCountry
    */
   /**
    *    THis method will add a country to the list
    *  
    *  
    *
    *  @param countryCodeInd The COuntry Code Indicator to add as defined in TIGRIS.TYPECOUNTRYCODE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCountry ( int countryCodeInd ) {
      addCountry ( (TypeCountryCodeRec) TypeList.getInstance().objectFromInd ( countryCodeInd, TypeList.TYPE_COUNTRYCODE ) ) ;
   }

   /*------------------------------------------------------------*/
   /* isWorldWide
    */
   /**
    *    This method will return the world wide property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = This locale is world wide, associated to all geographies and all countries
    *//*---------------------------------------------------------*/
   public boolean isWorldWide() {
      validateWorldWide() ;
      return( worldWide ) ;
   }





   /*------------------------------------------------------------*/
   /* getCountryList
    */
   /**
    *    This method will return a vector containing
    *  all of the countries we have associated to us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getCountryList () {
      Vector retVec = null ;

      /* If this is world wide, just return the whole list */
      validateWorldWide() ;
      if ( worldWide )
         retVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;

      /* Otherwise we have to go through each LocaleGeoRec and
         add each country */
      else {
         LocaleGeoRec localeGeoRec  = null ;
         Vector       geoCountryVec = null ;
         int          i             = 0 ;
         int          length        = geoVec.size() ;
         int          geoIndex      = 0 ;
         int          geoLength     = 0 ;

         /* Create our return list */
         retVec = new Vector ( 100, 10 ) ;

         while ( i < length ) {
            localeGeoRec = (LocaleGeoRec) geoVec.elementAt(i) ;

            /* If this geography has countries associated to
               it, add them */
            if ( localeGeoRec.countryCount() > 0 ) {
               geoCountryVec = localeGeoRec.getCountryList() ;

               if ( geoCountryVec != null ) {
                  geoIndex = 0 ;
                  geoLength = geoCountryVec.size() ;

                  while ( geoIndex < geoLength )
                     retVec.addElement ( geoCountryVec.elementAt(geoIndex++ ) ) ;
               }
            }
            i++ ;
         }


      }

      return( retVec ) ;

   }





   /*------------------------------------------------------------*/
   /* getSingleCountryList
    */
   /**
    *    This method will return a country list that
    *  contains only countries that are not associated
    *  to a geography all.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getSingleCountryList () {
      Vector retVec = null ;
      LocaleGeoRec localeGeoRec  = null ;
      Vector       geoCountryVec = null ;
      int          i             = 0 ;
      int          length        = geoVec.size() ;
      int          geoIndex      = 0 ;
      int          geoLength     = 0 ;

      /* Create our return list */
      retVec = new Vector ( 100, 10 ) ;

      while ( i < length ) {
         localeGeoRec = (LocaleGeoRec) geoVec.elementAt(i) ;

         /* If this geography has countries associated to
            it, add them */
         if ( ( !localeGeoRec.isAll() ) && ( localeGeoRec.countryCount() > 0 ) ) {
            geoCountryVec = localeGeoRec.getCountryList() ;

            if ( geoCountryVec != null ) {
               geoIndex = 0 ;
               geoLength = geoCountryVec.size() ;

               while ( geoIndex < geoLength )
                  retVec.addElement ( geoCountryVec.elementAt(geoIndex++ ) ) ;
            }
         }
         i++ ;
      }

      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* setWorldWide
    */
   /**
    *    This method will set this locale rec to be
    *  a world wide locale
    *  
    *
    *  @param worldWide TRUE = Set World Wide, FALSE = Remove everything
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWorldWide ( boolean worldWide ) {
      if ( worldWide ) {
         int     i       = 0 ;
         int     length  = geoVec.size() ;
         boolean allGeo  = true ;

         /* First, walk through the geoVecs and validate
            that they are each 'all' */
         while ( ( i < length ) && ( allGeo ) )
            ((LocaleGeoRec)geoVec.elementAt(i++)).associateAll() ;

         updateRecStatus ( REC_UPDATE ) ;
         this.worldWide = true ;
      } else {
         int     i       = 0 ;
         int     length  = geoVec.size() ;
         boolean allGeo  = true ;

         /* First, walk through the geoVecs and validate
            that they are each 'all' */
         while ( ( i < length ) && ( allGeo ) )
            ((LocaleGeoRec)geoVec.elementAt(i++)).removeAll() ;

         i=0 ;
         while ( i < length ) {
            if ( ((LocaleGeoRec)geoVec.elementAt(i)).getTypeGeoRec().getInd() == TypeList.GEO_WORLD_WIDE ) {
               geoVec.removeElementAt(i) ;
               i=length ;
            }
            i++ ;

         }
         this.worldWide = false ;
      }
   }

   /*------------------------------------------------------------*/
   /* getGeoListContainingAll
    */
   /**
    *    This method will return a list of geographies
    *  that have all countries assigned to them
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getGeoListContainingAll () {
      Vector retVec = new Vector ( 6 ) ;

      try {
         int i      = 0 ;
         int length = geoVec.size() ;

         while ( i < length ) {
            if ( ((LocaleGeoRec)geoVec.elementAt(i)).isAll() )
               retVec.addElement ( ((LocaleGeoRec)geoVec.elementAt(i)).getTypeGeoRec() ) ;
            i++ ;
         }

      } catch ( NullPointerException ex ) {
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* clone
    */
   /**
    *    This method will return a clone of itself
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object clone () {
      LocaleRec localeRec = new LocaleRec ( this ) ;
      return( localeRec ) ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will determine if the object
    *  passed is equal to this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if they are equal
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      boolean rc = false ;

      if ( object instanceof LocaleRec ) {
         LocaleRec localeRec = (LocaleRec) object ;

         if ( ( worldWide ) && ( localeRec.isWorldWide() ) )
            rc = true ;
         else
            rc = ( localeRec.countryCount() == countryCount() ) ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getLocaleList
    */
   /**
    *    This method will return a string with a comma
    *  delimited list of countries
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getLocaleList () {
      String localeStr = "" ;

      if ( isWorldWide() )
         localeStr = "WorldWide" ;
      else {
         Vector countryVec = getCountryList () ;

         if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
            int i       = 0 ;
            int length  = countryVec.size() ;

            localeStr = "" ;

            while ( i < length )
               localeStr = localeStr+((TypeCountryCodeRec)countryVec.elementAt(i++)).toString()+", " ;

            localeStr = localeStr.substring ( 0, localeStr.length()-2 ) ;
         }
      }
      return( localeStr ) ;
   }


   /**
     *  This method will return a human readable ( Not Customer Readable)
     * string of all properties of this file
     **/
   public String toOut() {
      String retStr = "Locale Information\n" ;
      Vector country = getSingleCountryList() ;
      Vector geo     = getGeoListContainingAll() ;

      retStr+="Number of Countries:   "+countryCount()+"\n" ;
      retStr+="Number of Geographies: "+geoCount()+"\n" ;

      if ( ( country != null ) && ( country.size() > 0 ) ) {
         int                i          = 0 ;
         int                size       = country.size() ;
         TypeCountryCodeRec countryRec = null ;

         retStr+= "Countries:            " ;
         while ( i < size ) {
            countryRec = (TypeCountryCodeRec) country.elementAt(i) ;

            retStr+=countryRec.toString()+"("+countryRec.getInd()+")" ;
            if ( i < (size-1) )
               retStr+="," ;
            i++ ;
         }

         retStr+="\n" ;
      }

      if ( ( geo != null ) && ( geo.size() > 0 ) ) {
         int                i          = 0 ;
         int                size       = geo.size() ;
         TypeGeoRec geoRec = null ;

         retStr+= "Geos with ALL Assoc.: " ;
         while ( i < size ) {
            geoRec = (TypeGeoRec) geo.elementAt(i) ;

            retStr+=geoRec.toString()+"("+geoRec.getInd()+")" ;
            if ( i < (size-1) )
               retStr+="," ;
            i++ ;
         }

         retStr+="\n" ;
      }


      return( retStr ) ;
   }

} /* LocaleRec */






/*------------------------------------------------------------*/
/* LocaleGeoRec
 */
/**
 *    This class holds all countries that are currently
 *  associated to a geography. If all countries are
 *  associated, then the 'all' flag will be true and the country vec will be null
 */
class LocaleGeoRec extends Object implements java.io.Serializable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private TypeGeoRec       geoRec        = null ;       // The geography this represents
   private Vector           allCountryVec = null ;       // All of the countries that can be associated
   private Vector           countryVec    = null ;       // All countries that are associated
   private boolean          all           = false ;      // TRUE if all countries are associated

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create the default LocaleGeoRec
    *  
    *  
    *
    *  @param geoRec The Geography record that this record represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LocaleGeoRec ( TypeGeoRec geoRec ) {
      this.geoRec = geoRec ;
      allCountryVec = geoRec.getCountryVec() ;
   }


   public LocaleGeoRec ( LocaleGeoRec oldRec ) {
      geoRec = oldRec.geoRec ;
      allCountryVec = (Vector)oldRec.allCountryVec.clone() ;
      countryVec    = (Vector)oldRec.countryVec.clone() ;
      all           = oldRec.all ;
   }




   /*------------------------------------------------------------*/
   /* isAll
    */
   /**
    *    This method will return the 'Al' property.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE means that all countries have been associated, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean isAll() {
      return( all ) ;
   }




   /*------------------------------------------------------------*/
   /* addCountry
    */
   /**
    *    This method will add a country to our
    *  list
    *  
    *
    *  @param countryRec The TypeCountryCodeRec to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCountry ( TypeCountryCodeRec countryRec ) {

      /* Do not add the country if it doesn't belong to us or
         if we already have all of the countries associated! */
      if ( ( countryRec.getGeoInd() == geoRec.getInd() ) && ( !all ) ) {

         if ( countryVec == null )
            countryVec = new Vector ( 10 ) ;

         /* Do not add the country if it is already in our
            vector! We do not want duplicates! */
         if ( !countryVec.contains ( countryRec ) )
            countryVec.addElement ( countryRec ) ;

         if ( countryVec.size() == allCountryVec.size() ) {
            countryVec.removeAllElements() ;
            countryVec = null ;
            all = true ;
         }
      }
   }






   /*------------------------------------------------------------*/
   /* removeCountry
    */
   /**
    *    This method will remove a country from
    *  the associations
    *  
    *
    *  @param countryRec The country record to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCountry ( TypeCountryCodeRec countryRec ) {

      /* Do not even try to remove it if this country record
         doesn't really belong to us! */
      if ( countryRec.getGeoInd() == geoRec.getInd() ) {

         /* If we currently have all countries associated,
            we need to turn that off, and add all of the other
            countries to our vector */
         if ( all ) {
            int i       = 0 ;
            int length  = allCountryVec.size() ;

            all = false ;
            countryVec = new Vector ( length-1 ) ;

            while ( i < length ) {
               if ( ((TypeCountryCodeRec)allCountryVec.elementAt(i)) != countryRec )
                  countryVec.addElement ( allCountryVec.elementAt(i) ) ;
               i++ ;
            }
         } /* if all */

         /* Otherwise, just remove it */
         else {
            countryVec.removeElement ( countryRec ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* countryCount
    */
   /**
    *    This method will return the number of countries
    *  this geography currently has associated to it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int countryCount() {
      if ( all )
         return( allCountryVec.size() ) ;
      else if ( countryVec != null )
         return( countryVec.size() ) ;
      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* getCountryList
    */
   /**
    *    This method will return a list of countries that
    *  have been associated to this record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of TypeCountryCodeRecs
    *//*---------------------------------------------------------*/
   public Vector getCountryList () {
      if ( countryVec != null )
         return( countryVec ) ;
      else if ( all )
         return( allCountryVec ) ;
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will perform an equality check
    *  to ensure that the object passed is
    *  equal to this record
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if equal, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      boolean rc = false ;

      if ( object instanceof TypeGeoRec ) {
         rc = ( geoRec.getInd() == ((TypeGeoRec)object).getInd() ) ;
      } else if ( object instanceof LocaleGeoRec ) {
         rc = ( geoRec.getInd() == ((LocaleGeoRec)object).getTypeGeoRec().getInd() ) ;
      }


      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* associateAll
    */
   /**
    *    This method will associate all countries
    *  to this geography that this geography contains
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void associateAll () {
      all = true ;
      if ( countryVec != null ) {
         countryVec.removeAllElements() ;
         countryVec = null ;
      }

   }





   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all countries that
    *  are assigned
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll () {
      all = false ;
      if ( countryVec != null ) {
         countryVec.removeAllElements () ;
         countryVec = null ;
      }
   }





   /*------------------------------------------------------------*/
   /* getTypeGeoRec
    */
   /**
    *    This method will return the TypeGeoRec
    *  that this LocaleGeoRec represents.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TypeGeoRec geoRec
    *//*---------------------------------------------------------*/
   public TypeGeoRec getTypeGeoRec() {
      return( geoRec ) ;
   }



} /* LocaleGeoRec */