package com.ibm.nzna.projects.common.quest.doc ;     
/*-----------------------------------------------------------*/
/*   DocumentDraft                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentDraft                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
/**
 * A DocumentDraft is an extension to a Document object. This 
 * extension holds the data values that are part of a Draft. Document
 * drafts are not stored in the TIGRIS.DOCUMENTS table, but are stored
 * in the mirrored QUEST.DOCUMENTS table.<br><br>
 *
 * <b>Please Note!</b> Just like the Document class, if you add any
 * properties to this class you <b>MUST</b> put a line in the initializeProperties
 * method which copies that property properly.
 *
 * @version 1.0, 2/21/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentDraft                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentDraft extends Document implements java.io.Serializable {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /**
     *  This property is fired when the comments are updated
     **/
   public static final int COMMENTS = 100 ;

   /**
     *  This property is fired when the requestor is changed
     **/
   public static final int REQUESTOR = 101 ;

   /**
     *  This property is fired when the Work Required is changed
     **/
   public static final int WORK_REQUIRED = 102 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  The date that this draft was started
     **/
   private String startDate = null ;

   /**
     *  The date that this draft was stopped
     **/
   private String stopDate = null ;

   /**
     *  Requestor is the person who requested a change to a document
     **/
   private String requestor = null ;

   /**
     *  The comments made on this single draft
     **/
   private String comments = "" ;

   /**
     *  These are the original comments that the user saved with this draft
     **/
   private String originalComments = "" ;

   /**
     *  The Draft Document Indicator that this document draft
     *  points to in the TIGRIS.DOCUMENTS table
     **/
   private int existingDocInd    = 0 ;

   /**
     *  The type of work that this document draft represents. This indicator
     *  is defined in TIGRIS.TYPEWORKREQUIRED and can be one of the following:<br><br>
     *
     *  CREATE, EDIT, DELETE
     **/
   private int workRequired       = 0 ;


   /**
     *  If TRUE, this property identifies this document as a Document Template.
     **/
   private boolean template = false ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a new Draft for
    *  the document indicator passed in. If this 
    *  indicator is Zero, it is assumed this draft represents
    *  a new Document that has not been created yet and the work
    *  required is set to Create.
    *
    *  @param docInd The Document Indicator of the EXISTING DOCUMENT
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentDraft ( int docInd ) {
      super ( docInd ) ;
      if ( docInd == 0 ) {
         setWorkRequired ( TypeList.WORKREQ_CREATE ) ;
         setStatus ( new DocumentStatus ( docInd,
                                          Status.TYPESTATUS_NEWREQUEST,
                                          new CDate ( CDate.DATE_DATE ).today() ) ) ;
         setStartDate ( new CDate ( CDate.DATE_DATE ).today() ) ;
      }
   }

   /**
     *  This constructor will create a brand new Document Draft
     * for a new document. The information for this new document
     * will be copied from the DocumentDraft object passed in
     **/
   public DocumentDraft ( DocumentDraft draft ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, copy all of the information over                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      initializeProperties ( draft ) ;                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, adjust our indicators and status                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      docInd = 0 ;
      existingDocInd = 0 ;
      template=false;
      clearFilename() ;
      updateRecStatus ( REC_NOCHANGE ) ;
      updateRecStatus ( REC_INSERT ) ;
      setStartDate ( new CDate ( CDate.DATE_DATE ).today() ) ;

   }

   /*------------------------------------------------------------*/
   /* saveToDatabase
    */
   /**
    *    This method will save this document to the
    *  database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return sqlCode , 0 if Ok
    *//*---------------------------------------------------------*/
   public int writeToDatabase () {
      try {
         generateKeys() ;
         generateFilename() ;
         return( DocumentDraftWriter.writeToDatabase ( this ) ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( -1 ) ;
   }



   /*------------------------------------------------------------*/
   /* Default Constructor
    */
   /**
    *    
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentDraft () {
      setDocumentClass ( TypeList.DOCCLASS_IBM_CONFIDENTIAL ) ;
      setWorkRequired ( TypeList.WORKREQ_CREATE ) ;
      setStatus ( new DocumentStatus ( 0,
                                       Status.TYPESTATUS_NEWREQUEST,
                                       new CDate ( CDate.DATE_DATE ).today() ) ) ;
      setStartDate ( new CDate ( CDate.DATE_DATE ).today() ) ;
   }




   /*------------------------------------------------------------*/
   /* createDraft
    */
   /**
    *    This method will create a new draft
    *  from the existing document indicator passed in. This method
    *  will read in all of the information about the existingDocInd
    *  passed in
    *  
    *
    *  @param int existingDocInd
    *  @param None
    *  @param None
    *
    *  @return CreateDocumentDraft object for the new Draft, NULL if error
    *//*---------------------------------------------------------*/
   public static CreateDocumentDraft createDraft ( int existingDocInd ) {
      CreateDocumentDraft returnRec   = null ;
      DocumentDraft       draft       = null ;
      int                 newDraftInd = draftExists ( existingDocInd ) ;                                           


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we receive an error, report it. If we receive    */
      /* no doc indicator for the draft, create a new draft and    */
      /* if we receive a doc ind for the draft, read the draft in  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( newDraftInd ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   0 Means that no draft was found, create a new one       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            Document document = new Document ( existingDocInd ) ;
            int      rc       = document.readFromDatabase() ;

            if ( rc == 0 ) {
               draft = new DocumentDraft() ;
               draft.initializeProperties ( document ) ;
               returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_CREATED,
                                                     0,
                                                     draft ) ;
            } else {
               LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ): Could not read Existing Document. Return Code:"+rc ) ;

               returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_NONE,
                                                     0,
                                                     null ) ;            
            }
            break;                          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   -1 means an error, report the error                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case -1 :
            LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ): Could not find Draft Document." ) ;
            returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_NONE,
                                                  0,
                                                  null ) ;
            draft = null ;
            break ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, anything else that is returned is a draft, read it  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         default :
            int draftRC = 0 ;
            draft = new DocumentDraft ( newDraftInd ) ;

            draftRC = draft.readFromDatabase() ;
            switch ( draftRC ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   0 Means everything read fine, adjust the REC_STATUS     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               case 0 :
                  draft.forceRecStatus ( REC_NOCHANGE ) ;
                  if ( draft.getRecycled() ) {
                     returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_RECYCLED,
                                                           0,
                                                           draft ) ;
                  } else {
                     returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_ALREADY_EXISTS,
                                                           0,
                                                           draft ) ;
                  }
                  break ;                                 

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   This should never happen, but just in case              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               case 100 :
                  LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ): Wierd, found draft "+newDraftInd+" for existing document, but can't read it! rc 100" ) ;
                  draft = null ;
                  returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_NONE,
                                                        0,
                                                        null ) ;
                  break ;                        


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Anything else, report as an error                       */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               default:
                  LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ): Read Draft Document "+newDraftInd+". rc:"+draftRC ) ;
                  returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_SQL_ERROR,
                                                        draftRC,
                                                        null ) ;
                  draft = null ;
                  break ;

            }
            break ;

      } /* switch newDraftInd  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   To catch all errors                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ) could not figure out what to do" ) ;
         returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_NONE,
                                               0,
                                               null ) ;
      }

      return( returnRec ) ;

   } /* createDraft  */



   /**
     * This method will attempt to create a DocumentDraft object from
     * the Document object passed in.
     */
   public static CreateDocumentDraft createDraft (Document document ) {
      CreateDocumentDraft returnRec      = null ;
      DocumentDraft       draft          = null ;
      int                 existingDocInd = document.getDocInd() ;
      int                 newDraftInd    = draftExists ( existingDocInd ) ;                                           


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we receive an error, report it. If we receive    */
      /* no doc indicator for the draft, create a new draft and    */
      /* if we receive a doc ind for the draft, read the draft in  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( newDraftInd ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   0 Means that no draft was found, create a new one       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            draft = new DocumentDraft() ;
            draft.initializeProperties ( document ) ;
            draft.setStartDate ( new CDate ( CDate.DATE_DATE ).today() ) ;
            draft.existingDocInd = existingDocInd ;
            draft.docInd         = 0 ;
            draft.setWorkRequired ( TypeList.WORKREQ_EDIT ) ;
            draft.updateRecStatus ( REC_NOCHANGE ) ;
            draft.updateRecStatus ( REC_INSERT ) ;

            returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_CREATED,
                                                  0,
                                                  draft ) ;
            break;                          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   -1 means an error, report the error                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case -1 :
            LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ): Could not find Draft Document." ) ;
            returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_NONE,
                                                  0,
                                                  null ) ;
            draft = null ;
            break ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, anything else that is returned is a draft, read it  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         default :
            int draftRC = 0 ;
            draft = new DocumentDraft ( newDraftInd ) ;

            draftRC = draft.readFromDatabase() ;
            switch ( draftRC ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   0 Means everything read fine, adjust the REC_STATUS     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               case 0 :
                  draft.forceRecStatus ( REC_NOCHANGE ) ;
                  if ( draft.getRecycled() ) {
                     returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_RECYCLED,
                                                           0,
                                                           draft ) ;
                  } else {
                     returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_ALREADY_EXISTS,
                                                           0,
                                                           draft ) ;
                  }
                  break ;                                 

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   This should never happen, but just in case              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               case 100 :
                  LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ): Wierd, found draft "+newDraftInd+" for existing document, but can't read it! rc 100" ) ;
                  draft = null ;
                  returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_NONE,
                                                        0,
                                                        null ) ;
                  break ;                        


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Anything else, report as an error                       */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               default:
                  LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ): Read Draft Document "+newDraftInd+". rc:"+draftRC ) ;
                  returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_SQL_ERROR,
                                                        draftRC,
                                                        null ) ;
                  draft = null ;
                  break ;

            }
            break ;

      } /* switch newDraftInd  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   To catch all errors                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         LogSystem.log ( 1, "DocumentDraft.createDraft ( "+existingDocInd+" ) could not figure out what to do" ) ;
         returnRec = new CreateDocumentDraft ( CreateDocumentDraft.DRAFT_NONE,
                                               0,
                                               null ) ;
      }

      return( returnRec ) ;

   }



   /*------------------------------------------------------------*/
   /* draftExists
    */
   /**
    *    This static method will do a simple look up
    *  to find our if a draft exists for the existing
    *  document indicator passed in
    *
    *  @param existingDocInd The exisiting document indicator to look for
    *  @param None
    *  @param None
    *
    *  @return The indicator of the draft if it exists, -1 if an error, 0 if no draft exists
    *//*---------------------------------------------------------*/
   public static int draftExists ( int existingDocInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "draftExists ("+existingDocInd+")", 5 ) ;
      int       returnInd = -1 ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DOCIND "+
                                              "FROM QUEST.DOCUMENTS "+
                                              "WHERE STOPDATE IS NULL AND "+
                                              "      EXISTINGDOCIND = "+existingDocInd+" "+
                                              "FOR FETCH ONLY" ) ;

         if ( rs.next() ) {
            returnInd = rs.getInt(1) ;
         } else {
            returnInd = 0 ;
         }

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      return( returnInd ) ;
   }


   /*------------------------------------------------------------*/
   /* setWorkRequired
    */
   /**
    *    This method will set the work required
    *  indicator of this document. This indicator is
    *  defined in the TIGRIS.TYPEWORKREQ table
    *
    *  @param workRequired The Work Required Indicator 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWorkRequired ( int workRequired ) {
      super.firePropertyChanged ( WORK_REQUIRED ) ;
      addUndo ( WORK_REQUIRED, new Integer ( workRequired ) );
      this.workRequired = workRequired ;
   }




   /*------------------------------------------------------------*/
   /* getWorkRequired
    */
   /**
    *    This method will return the work required
    *  indicator that this document draft has
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getWorkRequired () {
      return( workRequired ) ;
   }


   /*------------------------------------------------------------*/
   /* generateKeys
    */
   /**
    *    This method will generate the primary keys
    *  for this document. Currently the primary keys
    *  includes only Document Indicator. If this document
    *  already has a document indicator assigned, nothing is done
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void generateKeys () {

      if ( docInd == 0 ) {
         docInd = Counter.getCounter ( LogSystem.getInstance(), "DOCREQIND" ) ;
      }

      if ( existingDocInd == 0 ) {
         existingDocInd = Counter.getCounter ( LogSystem.getInstance(), "DOCIND" ) ;
      }

      if ( ( getFilename() == null ) || ( getFilename().length() == 0 ) )
         setFilename ( "MIGR-"+existingDocInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setStartDate
    */
   /**
    *    This method will set the start date of this
    *  document.
    *  
    *
    *  @param date The Start Date to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setStartDate ( String date ) {
      if ( date != null ) {
         this.startDate = date.trim() ;
      } else {
         this.startDate = new CDate ( CDate.DATE_DATE ).today() ;
      }
   }




   /*------------------------------------------------------------*/
   /* getStartDate
    */
   /**
    *    This method will return the start date
    *  of this document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getStartDate () {
      return( startDate ) ;
   }





   /*------------------------------------------------------------*/
   /* setStopDate
    */
   /**
    *    This method will set our stop date
    *  
    *  
    *
    *  @param stopDate The new stop date
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStopDate ( String stopDate ) {
      if ( stopDate != null ) {
         this.stopDate = stopDate.trim() ;
      }
   }


   /*------------------------------------------------------------*/
   /* setRequestor
    */
   /**
    *    This method will set our requestor
    *  
    *  
    *
    *  @param requestor The new requestor
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRequestor ( String requestor ) {
      if ( requestor != null ) {
         this.requestor = requestor.trim() ;
      } else {
         this.requestor = "" ;
      }
      addUndo ( REQUESTOR, this.requestor ) ;
      firePropertyChanged ( REQUESTOR ) ;
   }




   /*------------------------------------------------------------*/
   /* getRequestor
    */
   /**
    *    This method will return the requestor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getRequestor () {
      return( requestor ) ;
   }


   /*------------------------------------------------------------*/
   /* getStopDate
    */
   /**
    *    This method will return the stop date of this
    *  Document Draft. This Stop Date is only
    *  set if this document draft has been closed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getStopDate () {
      return( stopDate ) ;
   }





   /*------------------------------------------------------------*/
   /* setComments
    */
   /**
    *    This method will set the comments of this
    *  document draft. Please note that if you call
    *  this method, the comments set are actually
    *  simply added to the 'comments'. This makes it so
    *  that once the comments property has been set through the
    *  setComments method, those comments can not be overwritten.<br><br>
    *
    *  When the comment is added to the original comments, two carriage
    *  returns will be inserted between the original comments and the
    *  new comments
    *
    *  @param comments
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setComments ( String comments ) {
      if ( comments != null ) {
         addUndo ( COMMENTS, this.comments ) ;

         this.comments = comments.trim() ;
         firePropertyChanged ( COMMENTS ) ;
      }
   }


   /**
     *  This method will return the original comments that were entered
     * by a user who previously saved this document draft. If a user had
     * never saved this document draft before then this method will simply
     * return blank
     **/
   public String getOriginalComments () {
      return ( originalComments ) ;
   }


   /**
     *  This method will set the original comments that were read from the
     * database when this draft was created
     **/
   protected void setOriginalComments ( String originalComments ) {
      this.originalComments = originalComments ;
   }


   /*------------------------------------------------------------*/
   /* getComments
    */
   /**
    *    This method will return the comments
    *  property. 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getComments () {
      return( comments ) ;
   }





   /*------------------------------------------------------------*/
   /* setExistingDocInd
    */
   /**
    *    This method will set the existing document
    *  indicator that this draft points to
    *  
    *
    *  @param existingDocInd The existing document indicator as defined in TIGRIS.DOCUMENTS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setExistingDocInd ( int existingDocInd ) {
      this.existingDocInd = existingDocInd ;
   }




   /*------------------------------------------------------------*/
   /* getExistingDocInd
    */
   /**
    *    This method will return the existing document
    *  indicator of this draft
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getExistingDocInd () {
      return( existingDocInd ) ;
   }


   /*------------------------------------------------------------*/
   /* readFromDatabase
    */
   /**
    *    This method will read this document from the
    *  database.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return 0 if Ok, an SQL Code if not Ok
    *//*---------------------------------------------------------*/
   public int readFromDatabase () {
      String    rtnName   = "readFromDatabase" ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      int       rc        = 0 ;

      if ( sqlMethod != null ) {
         try {
            CallableStatement stmt  = sqlMethod.createStoredProc ( "QUEST.DocDraftReader", 3 ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the call                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.setInt               ( 1, getDocInd() ) ;
            stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
            stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            LogSystem.log ( 1, "Reading Document:"+getDocInd() ) ;
            stmt.execute() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the StoredProcRec from the Lob                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Throw an error if need be                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( retRec != null ) && ( retRec.error ) ) {
               LogSystem.beginLogSection ( "DocumentReader Returned Error" ) ;
               LogSystem.log ( 1, "sqlCode:   "+retRec.sqlCode ) ;
               LogSystem.log ( 1, "userError: "+retRec.userError ) ;
               LogSystem.log ( 1, "errorStr:  "+retRec.errorStr ) ;
               LogSystem.log ( 1, "error:     "+retRec.error ) ;
               try {
                  rc = new Integer ( retRec.sqlCode ).intValue() ;
               } catch ( Exception e1 ) {
                  LogSystem.log ( 1, "Cannot translate retRec.sqlCode to Integer for RC" ) ;
               }
               LogSystem.endLogSection () ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, continue                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get our return object                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               int originalDocInd = getDocInd() ;
               initializeProperties ( (DocumentDraft) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ) ;
               docInd = originalDocInd ;

               rc = 0 ;

            } /* Else if we have no error  */

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            rc = -1 ;
         }
      } /* if sqlMethod is not null  */

      sqlMethod.close() ;

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* initializeProperties
    */
   /**
    *    This method will initialize our data to the
    *  data from the Document passed in. Please note that this
    *  method does not copy the following properties: <br><br>
    *
    *  template
    *  
    *
    *  @param document The Document to copy the data from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initializeProperties ( Document document ) {

      super.initializeProperties ( document ) ;

      if ( document instanceof DocumentDraft ) {
         DocumentDraft draft = (DocumentDraft) document ;

         this.startDate        = draft.startDate ;
         this.stopDate         = draft.stopDate  ;
         this.requestor        = draft.requestor ;
         this.comments         = draft.comments ;
         this.template         = draft.template ;
         this.existingDocInd   = draft.existingDocInd ;
         this.workRequired     = draft.workRequired ;
         this.originalComments = draft.originalComments ;
      } else
         this.existingDocInd = document.getDocInd() ;
   }

   /**
     *  This method will adjust the 'Template' Property
     **/
   public void setTemplate ( boolean template ) {
      this.template = template ;
   }

   /**
     *  This method will return the current Template property. If
     * this method returns TRUE then it should be considered that
     * this document represents a Template
     **/
   public boolean getTemplate () {
      return( template ) ;
   }

   /**
     *  This method will return an exact copy of this document instance
     **/
   public Object clone () {
      DocumentDraft document = new DocumentDraft ( getDocInd() ) ;

      document.initializeProperties ( this ) ;

      return( document ) ;
   }


   /**
     *  This method will set the properties in this document draft
     * so that they will create a new Active Document when published. The
     * properties in this draft will be published as is
     **/
   public void createNew () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign the indicators                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setDocInd ( 0 ) ;
      existingDocInd = 0 ;
      clearFilename() ;
      generateKeys() ;

      if ( getStatus() != null )
         getStatus().setDocInd ( getExistingDocInd() ) ;

      updateRecStatus ( REC_NOCHANGE ) ;
      updateRecStatus ( REC_INSERT ) ;
      workRequired = TypeList.WORKREQ_CREATE ;
   }

   /**
     *  This method will check out the document. This means that no QIT user
     * should be able to change this document when it is checked out
     *
     * @param userId The User Id to check out
     **/
   public void checkOut ( String userId ) {
      if ( existingDocInd != 0 )
         super.checkOut ( userId ) ;

      if ( docInd != 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "Document.checkOut ( "+userId+" )", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE QUEST.DOCUMENTS SET MAINTLOCK = 'Y' WHERE DOCIND = "+docInd ) ;
            stmt.executeUpdate ( "DELETE FROM QUEST.DOCPROPERTIES WHERE DOCIND = "+docInd+" AND "+
                                 "PROPERTYIND = "+DocumentProperty.CHECKED_OUT_BY ) ;
            stmt.executeUpdate ( "INSERT INTO QUEST.DOCPROPERTIES "+
                                 "(DOCIND, PROPERTYIND, ORDERNUM, VALUE, DBUSER, CHANGEDTIME) "+
                                 "values "+
                                 "("+docInd+", "+DocumentProperty.CHECKED_OUT_BY+", 0, '"+userId+"', 'QUEST', CURRENT TIMESTAMP)" ) ;
            stmt.close() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
         }
         sqlMethod.close() ;
      }
   }

   /**
     *  This method will check in the document. This means that QIT users
     * should be able to change this document when it is not checked out
     *
     **/
   public void checkIn (  ) {
      if ( existingDocInd != 0 ) {
         super.checkIn() ;
      }

      if ( docInd != 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "Document.checkIn ()", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE QUEST.DOCUMENTS SET MAINTLOCK = 'N' WHERE DOCIND = "+docInd ) ;
            stmt.executeUpdate ( "DELETE FROM QUEST.DOCPROPERTIES WHERE DOCIND = "+docInd+" AND "+
                                 "PROPERTYIND = "+DocumentProperty.CHECKED_OUT_BY ) ;
            stmt.close() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
         }
         sqlMethod.close() ;
      }
   }

} /* DocumentDraft */

