package com.ibm.nzna.projects.common.quest.doc ;        
/*-----------------------------------------------------------*/
/*   DocumentProperty                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentProperty                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;   
/**
 * A Document Property is an external, dynamic property that
 * is stored in the TIGRIS.DOCPROPERTY table.
 *
 * @version 1.0, 10/16/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentProperty                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentProperty extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Document Property Types                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int UNIVERSAL_NOTES      = 1 ;
   public static final int DESCRIPTION          = 2 ;
   public static final int RELEASE_DATE         = 3 ;
   public static final int COUNTRYCOMMENT       = 4 ;
   public static final int CREATE_DATE          = 5 ;
   public static final int LAST_UPDATE_CTS_TIPS = 6 ;
   public static final int URL                  = 7 ;
   public static final int VERSION              = 8 ;  
   public static final int CHECKED_OUT_BY       = 9 ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String value          = null ;
   private int    propertyInd    = 0 ;
   private int    orderNum       = 1 ;
   private int    docInd         = 0 ;
   private String dbUser         = null ;
   private String changedTime    = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor takes only the document
    *  indicator that this property is being created for
    *  
    *
    *  @param docInd The Document Indicator as defined in QUEST.DOCUMENTS and TIGRIS.DOCUMENTS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentProperty ( int docInd ) {
      setDocInd ( docInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocInd
    */
   /**
    *    This method will change the document
    *  indicator that this property is associated with
    *  
    *
    *  @param docInd The document indicator
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocInd ( int docInd ) {
      this.docInd = docInd ;
   }





   /*------------------------------------------------------------*/
   /* getDocInd
    */
   /**
    *    This method will return the document indicator
    *  that this property is associated with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return docInd
    *//*---------------------------------------------------------*/
   public int getDocInd () {
      return( docInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setValue
    */
   /**
    *    This method will set the value of a
    *  document's property. The value cannot be null, if it is
    *  a null or the length of value is greater than 254 characters
    *  than an illegal argument exception will be thrown
    *  
    *
    *  @param value A Non-Null value. 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setValue ( String value ) throws IllegalArgumentException {
      if ( ( value == null ) || ( value.length() > 254 ) )
         throw new IllegalArgumentException ( "PropertyInd:"+propertyInd+" Document Property Value cannot be NULL or greater than 254 characters!" ) ;
      else {
         this.value = value.trim() ;
      }
   } /* setValue  */





   /*------------------------------------------------------------*/
   /* setOrderNum
    */
   /**
    *    This method will set the order number
    *  of this property. The order number is used
    *  to order properties with large values
    *
    *  @param orderNum The number to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOrderNum ( int orderNum ) {
      this.orderNum = orderNum ;
   }





   /*------------------------------------------------------------*/
   /* setPropertyInd
    */
   /**
    *    This method will set the property indicator. The
    *  property indicator defines what this property
    *  represents and must be one of the following types:<br><br>
    *
    *   DocumentProperty.NOTESID<br>
    *   DocumentProperty.DESCRIPT<br>
    *   DocumentProperty.RELEASEDATE<br>
    *   DocumentProperty.COUNTRYCOMMENT<br>
    *   DocumentProperty.DATECREATED<br>
    *   DocumentProperty.LASTUPDATECTSTIPS<br>
    *   DocumentProperty.URL<br>
    *   DocumentProperty.VERSION<br>
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPropertyInd ( int propertyInd ) {
      this.propertyInd = propertyInd ;
   }





   /*------------------------------------------------------------*/
   /* getValue
    */
   /**
    *    This method will return the value that this
    *  property represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getValue () {
      return( value ) ;
   }





   /*------------------------------------------------------------*/
   /* getPropertyInd
    */
   /**
    *    This method will return the property
    *  indicator that this property represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getPropertyInd () {
      return( propertyInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getOrderNum
    */
   /**
    *    This method will return the order number
    *  for this property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getOrderNum () {
      return( orderNum ) ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will compare an object to this
    *  object. If the object passed is an Integer
    *  object, the property indicator is compared, if
    *  the object passed is a String, then the value is 
    *  compared with the string. 
    *
    *  @param object The object to compare with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      boolean equal = false ;

      if ( object instanceof Integer ) {
         equal = ((Integer)object).intValue() == propertyInd ;
      }
      else if ( object instanceof String )
         equal = ((String)object).equals ( value ) ;
      else
         equal = super.equals ( object ) ;

      return( equal ) ;
   }

} /* DocumentProperty */
