
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.db.*;
import java.sql.*;
import java.util.Vector;

/**
 * Online Assistant Action.
 * An Action is a terminal point in the problem diagnosis logic flow
 * of a Symptom.
 * <p>
 * An Action contains the following conditional objects:
 * <ul>
 * <li>A Vector of Title objects.
 * <li>A Vector of AddtlInfo objects.
 * <li>A Vector of DocLink objects.
 * </ul>
 *
 * @see Symptom
 * @see Title
 * @see AddtlInfo
 * @see DocLink
 *
 * @version 1.37
 * @author  Mark DeBiase
**/
public class Action extends OAObject {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   protected int           actionInd_  = 0;
   protected Vector        titles_     = null;  // titles 
   protected Vector        addtlInfo_  = null;  // additional information 
   protected Vector        docLinks_   = null;  // document links
   protected TypeActionRec actionType_ = null;  // action type

   public static final String OA_PREFIX = "A";

   /**
    * New Action.
    * recStatus is set to REC_INSERT.
   **/
   public Action() {
      this(0);
   }

   /**
    * Action to be read from the database.
    * recStatus is set to REC_NOCHANGE.
   **/
   public Action(int actionInd) {
      super();
      setInd(actionInd);
      if ( actionInd_ == 0 ) {
         updateRecStatus(REC_INSERT);  // new object
      }
   }

   /**
    * Initialize from an existing Action object.
    * Called by ActionDraft(Action) constructor.
    * Used by the OAReader stored procedure to create a new draft Action
    * from a published Action.
   **/
   Action(Action a) {
      super(a);
      setInd(0);

      initSubObjects(a);

      updateRecStatus(REC_INSERT);
   }

   /**
    * Initialize from an existing ActionDraft object.
    * Used when publishing a draft Action.
   **/
   Action(ActionDraft ad) {
      super(ad);
      setInd( ad.getActiveActionInd() );

      initSubObjects(ad);

      if ( actionInd_ == 0 ) {
         updateRecStatus(REC_INSERT);
      } else {
         updateRecStatus(REC_UPDATE);
      }
   }

   /**
    * Initialize the Title, AddtlInfo and DocLink objects contained in 
    * this object.
    * Calls the resetNew() method on each object.  This method will
    * set the indicator to 0, set the recStatus to REC_INSERT and
    * clear the dbuser, changedTime and creationTime values.
   **/
   protected void initSubObjects(Action otherAction) {
      // NOTE: this initialization is not safe in that Vectors are
      //       shared between the new object and the original object

      this.titles_     = otherAction.titles_;
      this.addtlInfo_  = otherAction.addtlInfo_;
      this.docLinks_   = otherAction.docLinks_;
      this.actionType_ = otherAction.actionType_;

      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title t = (Title) titles_.elementAt(i);
            t.resetNew();
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            a.resetNew();
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            d.resetNew();
         }
      }
   }

   /**
    * Assign database keys for this object and all objects (Title,
    * AddtlInfo, DocLink) contained therein.
    * <b>Note:</b> Database keys will not be assigned for Link or
    * LinkGroup objects contained within DocLink objects.
    * @return true if database keys were assigned, otherwise false
   **/
   // this method assigns database keys for both Action & ActionDraft objects
   public boolean assignDatabaseKeys() {
      boolean success;
      boolean draft = false;

      if ( this instanceof ActionDraft ) {
         draft = true;
      }

      success = assignMyDatabaseKey();


      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size() && success; i++ ) {
            Title t = (Title) titles_.elementAt(i);
            if ( draft ) {
               success = t.assignDatabaseKeyForActionDraft();
            } else {
               success = t.assignDatabaseKeyForAction();
            }
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size() && success; i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            if ( draft ) {
               success = a.assignDatabaseKeyForActionDraft();
            } else {
               success = a.assignDatabaseKeyForAction();
            }
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size() && success; i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            if ( draft ) {
               success = d.assignDatabaseKeyForActionDraft();
            } else {
               success = d.assignDatabaseKeyForAction();
            }
         }
      }

      return (success);
   }
   // version to be called by publish stored procedure:
   void assignDatabaseKeys(Connection conn) throws Exception {
      boolean draft = false;

      assignMyDatabaseKey(conn);


      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title t = (Title) titles_.elementAt(i);
            if ( draft ) {
               t.assignDatabaseKeyForActionDraft(conn);
            } else {
               t.assignDatabaseKeyForAction(conn);
            }
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            if ( draft ) {
               a.assignDatabaseKeyForActionDraft(conn);
            } else {
               a.assignDatabaseKeyForAction(conn);
            }
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            if ( draft ) {
               d.assignDatabaseKeyForActionDraft(conn);
            } else {
               d.assignDatabaseKeyForAction(conn);
            }
         }
      }

      return;
   }

   /**
    * Assign a database key for this Action.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignMyDatabaseKey() {
      boolean success = true;

      if ( actionInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for Action");
         actionInd_ = Counter.getCounter( LogSystem.getInstance(), "ACTIONIND" );
         if ( actionInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for Action");
            success = false;
         }
      }

      return (success);
   }
   // version to be called by publish stored procedure:
   void assignMyDatabaseKey(Connection conn) throws Exception {

      if ( actionInd_ == 0 ) {
         actionInd_ = Counter.getCounter( conn, "ACTIONIND" );
         if ( actionInd_ == 0 ) {
            throw new Exception("Failed to get database key for Action");
         }
      }

      return;
   }

   /**
    * Returns true if any part of this object needs to be saved to the
    * database.
    * As a side effect of calling this method, the sort order on the
    * Conditional objects (Titles, AddtlInfos, DocLinks) is updated.
   **/
   public boolean objectModified() {
      boolean modified = super.modified();

      if ( titles_ != null ) {
         int sortOrder = 1;
         for ( int i=0; i<titles_.size() && !modified; i++ ) {
            Title t = (Title) titles_.elementAt(i);
            if ( t.valid() ) {
               t.setSortOrder( sortOrder++ );
            }
            modified = modified || t.modified();
         }
      }
      if ( addtlInfo_ != null ) {
         int sortOrder = 1;
         for ( int i=0; i<addtlInfo_.size() && !modified; i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            if ( a.valid() ) {
               a.setSortOrder( sortOrder++ );
            }
            modified = modified || a.modified();
         }
      }
      if ( docLinks_ != null ) {
         int sortOrder = 1;
         for ( int i=0; i<docLinks_.size() && !modified; i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            if ( d.valid() ) {
               d.setSortOrder( sortOrder++ );
            }
            modified = modified || d.modified();
         }
      }

      return (modified);
   }

   /**
    * Returns this object's indicator.
   **/
   public int getInd() {
      return (actionInd_);
   }

   /**
    * Sets this object's indicator.
   **/
   void setInd(int actionInd) {
      actionInd_ = actionInd;
      setOAItemNum(actionInd_);
   }

   /**
    * Set the OAItemNumber
   **/
   void setOAItemNum(int ind) {
      super.setOAItemNum(OA_PREFIX + ind);
   }

   /**
    * Gets the conditional titles.
    * <p>
    * <b>Note:</b> The last Title in the Vector should have no conditions
    * on it (other than a document classification of external).  It will
    * be used as the Default Title.
    *
    * @return a Vector of Title objects or null if there are no titles.
    * @see Title
   **/
   public Vector getTitles() {
      return (titles_);
   }


   /**
    * Sets the conditional Titles.
    * <p>
    * <b>Note:</b> The last Title in the Vector should have no conditions
    * on it. It will be used as the Default Title. It will also govern the
    * document classification of this object
    *
    * @param titles a Vector of Title objects.
    * @see Title
   **/
   public void setTitles(Vector titles) {
      titles_ = titles;

      if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) {
         docClass_ = ((Title)titles_.elementAt(titles_.size()-1)).getDocClass() ;
      }
      // dont update recstatus
   }

   public TypeDocClassRec getDocClass() {

      if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) {
         return( ((Title)titles_.elementAt(titles_.size()-1)).getDocClass() ) ;
      }

      return (docClass_);
   }

   /**
    * Gets the list of conditional additional information texts.
    * @return a Vector of AddtlInfo objects or null if there is no
    *         additional information.
    * @see AddtlInfo
   **/
   public Vector getAdditionalInfo() {
      return (addtlInfo_);
   }

   /**
    * Sets the list of conditional additional information texts.
    * @param additionalInfo a Vector of AddtlInfo objects
    * @see AddtlInfo
   **/
   public void setAdditionalInfo(Vector additionalInfo) {
      addtlInfo_ = additionalInfo;
      // dont update recstatus
   }

   /**
    * Same as getDocLinks().
    * Method name changed to be consistent with Question object;
    * this is here for backward compatability.
   **/
   public Vector getDocumentLinks() {
      return (getDocLinks());
   }
   /**
    * Same as setDocLinks().
    * Method name changed to be consistent with Question object;
    * this is here for backward compatability.
   **/
   public void setDocumentLinks(Vector docLinks) {
      setDocLinks(docLinks);
   }

   /**
    * Gets the list of conditional document links.
    * @return a Vector of DocLink objects or null if there are no DocLinks.
    * @see DocLink
   **/
   public Vector getDocLinks() {
      return (docLinks_);
   }

   /**
    * Sets the list of conditional document links.
    * @param docLinks a Vector of DocLink objects
    * @see DocLink
   **/
   public void setDocLinks(Vector docLinks) {
      docLinks_ = docLinks;
      // dont update recstatus
   }

   /**
    * Gets the Action type.
    * @return the action type.
    * @see com.ibm.nzna.projects.common.quest.type.TypeActionRec
   **/
   public TypeActionRec getActionType() { 
      return (actionType_);
   }

   /**
    * Sets the Action type.
    * @param actionType the action type
    * @see com.ibm.nzna.projects.common.quest.type.TypeActionRec
   **/
   public void setActionType(TypeActionRec actionType) {
      actionType_ = actionType;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns a HashList whose keys are Integer objects 
    * representing the Questions used in conditions on this object.
   **/
   void getAllQuestions(HashList questionList) {

      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title t = (Title) titles_.elementAt(i);
            t.getAllQuestions(questionList);
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            a.getAllQuestions(questionList);
         }
      }

      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            d.getAllQuestions(questionList);
         }
      }

   }

   /**
    * Perform a shallow copy of this object.
   **/
   public Object clone() {
      Action cloned = (Action) super.clone();

      if ( titles_ != null ) {
         cloned.titles_    = (Vector) titles_.clone();
      }
      if ( addtlInfo_ != null ) {
         cloned.addtlInfo_ = (Vector) addtlInfo_.clone();
      }
      if ( docLinks_ != null ) {
         cloned.docLinks_  = (Vector) docLinks_.clone();
      }

      return (cloned);
   }

   /**
    * Return a String representation of this object.
    * Not fully implemented - returns "Action" + getInd();
   **/
   public String toString() {
      return ("Action " + getInd());
   }

   /**
    * Write the Action and all its contained objects (with the exception of
    * Link and LinkGroup objects contained in DocLink objects) to the
    * database.  This method assumes that a DatabaseSystem object for
    * the DatabaseSystem.QUEST database has already been connected.
    * 
    * @return 0 for success, nonzero for failure.
   **/
   public int writeToDatabase() {
      LogSystem.log(2, ">>> Starting Action.writeToDatabase()");

      /*--------------------------------------------------------------*/
      /*   Assign database keys for this Action and its subobjects.   */
      /*--------------------------------------------------------------*/
      if ( !assignDatabaseKeys() ) {
         return (-1);
      }

      // if current recstatus is REC_INSERT && the call to SqlRunner
      // is successful, the recstatus of this object should be updated 
      // regardless of the success of the addtl info text updates.
      boolean forceChangeRecStatus = false;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "Action.writeToDatabase()", 10 );
      int returnCode = 0 ;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc("QUEST.ExtSqlRunner", 3);
         String sqlToRun = ActionWriter.getSQL(this);
         if ( sqlToRun.length() == 0 ) {
            LogSystem.log(1, "No changes to be written");
            return (0);
         }
         String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                       SqlRunner.END_DELIM, "\n" );

         LogSystem.log ( 1, "Starting to send statements to EXTSQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      
         stmt.setInt   ( 2, 0 );  // commit interval

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            returnCode = retRec.sqlCode ;
            if ( returnCode == 0 ) returnCode = -1;
         } else {
            if ( this.getRecStatus() == REC_INSERT ) {
               forceChangeRecStatus = true;
            }

            // use a PreparedStatement to update the AddtlInfo text
            PreparedStatement pstmt = null;

            for ( int i=0; addtlInfo_ != null && i < addtlInfo_.size(); i++ ) {
               AddtlInfo ai = (AddtlInfo) addtlInfo_.elementAt(i);
               int recStatus = ai.getRecStatus();
               if ( recStatus == REC_INSERT || recStatus == REC_UPDATE ) {
                  LogSystem.log ( 1, "Updating additional information text for " +
                                  ai.getInd() ) ;
                  //LogSystem.log ( 1, "Additional information text: >>>" +
                  //                   ai.getAddtlInfo() + "<<<");
                  if ( pstmt == null ) {
                     // don't prepare the statement until we know it is needed.
                     pstmt = sqlm.createPreparedStatement(
                                                         "UPDATE oa.actionAddInfo " +
                                                         "SET addInfo=? " +
                                                         "WHERE actionAddInfoInd=? " +
                                                         "AND languageInd=?"
                                                         );
                  }
                  pstmt.setString( 1, ai.getAddtlInfo() );
                  pstmt.setInt( 2, ai.getInd() );
                  pstmt.setInt( 3, ai.getLanguage().getInd() );
                  pstmt.executeUpdate();
               }
            }

            if ( pstmt != null ) {
               pstmt.close();
            }
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         returnCode = -1 ;
      } finally {
         sqlm.close() ;
      }

      if ( returnCode == 0 || forceChangeRecStatus ) {
         setRecNoChange();      // set all to REC_NOCHANGE
      }

      LogSystem.log(2, ">>> Action.writeToDatabase() complete");

      return (returnCode);
   }

   /**
    * Set the recStatus of this object and 
    * all its contained objects to REC_NOCHANGE.
    * Also saves a copy of the current country list.
   **/
   void setRecNoChange() {

      // this object
      updateRecStatus(REC_NOCHANGE);
      super.setOriginalCountryListFromCurrent();

      // contained objects:
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title t = (Title) titles_.elementAt(i);
            t.setRecNoChange();
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            a.setRecNoChange();
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            d.setRecNoChange();
         }
      }
   }

   /**
    * Read an Action object from the database.
    * 
    * @param actionInd   the database indicator of the Action to be read.
    * @param languageInd the language to read the text items for 
    *                    (0 = read all languages).
    * @return the Action object or null if an error occured or
    *         the object was not found.
   **/
   public static Action readFromDatabase(int actionInd, int languageInd) {
      LogSystem.log(2, ">>> Starting Action.readFromDatabase()");
      long start = System.currentTimeMillis();
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "Action.readFromDatabase()", 5 );
      StoredProcRec retRec    = null;
      Action        retAction = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, actionInd );              // object ind
         stmt.setInt( 2, OAReader.ACTION );        // object type
         stmt.setInt( 3, OAReader.READ_ACTIVE );   // read active object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Action and the StoredProcReturnRec objects      */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retAction = (Action)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      long end = System.currentTimeMillis();
      LogSystem.log(1, "Elapsed time: " + (end-start) + " msec");
      LogSystem.log(2, ">>> Action.readFromDatabase() completed");                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Do this after reading the question so that we           */
      /* do not open multiple connections to the database reading  */
      /* the type lists.                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retAction != null ) {
         // performance: dont update TypeRec data if in web mode
         if ( OAMode.currentMode != OAMode.WEB ) {
            retAction.updateTypeRecData();  // update the Type*Rec objects
            retAction.setRecNoChange();     // sets the original country list
         }
      } else {
         LogSystem.log(2, "No object found for indicator " + actionInd);
      }

      return (retAction);
   }

   /**
    * Update the TypeRec objects in the Action's subobjects with 
    * the proper descriptions.
    * Called by readFromDatabase();
   **/
   void updateTypeRecData() {
      super.updateTypeRecData();

      TypeList t = TypeList.getInstance();

      // action type
      TypeActionRec newActionType;
      newActionType = (TypeActionRec) t.objectFromInd( actionType_.getInd(),
                                                       TypeList.TYPE_ACTION );
      if ( newActionType != null ) {
         actionType_ = newActionType;
      }

      // update sub-objects TypeRec data:
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title title = (Title) titles_.elementAt(i);
            title.updateTypeRecData();
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo ai = (AddtlInfo) addtlInfo_.elementAt(i);
            ai.updateTypeRecData();
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink dl = (DocLink) docLinks_.elementAt(i);
            dl.updateTypeRecData();
         }
      }

      return;
   }

   /**
    * Find an open draft for an Action if one exists.
    * This method requires that a connection to DatabaseSystem.QUEST 
    * is already established.
    * 
    * @param actionInd   the database indicator of the active (published)
    *                    Action to find a draft for.
    * @return the indicator of the open draft for the Action or 0 if there
    *         is no open draft or an error occurs.
   **/
   public static int findOpenDraft(int actionInd) {
      LogSystem.log(2, ">>> Starting Action.findOpenDraft()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "Action.findOpenDraft()", 1 );

      int draftActionInd = 0;

      try {
         Statement stmt = sql.createStatement();
         ResultSet rs = stmt.executeQuery(
                                         "SELECT actionInd " +
                                         "FROM oadraft.action " +
                                         "WHERE activeActionInd=" + actionInd + " " +
                                         "AND stopDate IS NULL " +
                                         "FOR FETCH ONLY"
                                         );

         if ( rs.next() ) {
            draftActionInd = rs.getInt(1);
         } else {
            LogSystem.log(1, "No open draft found for " + actionInd);
         }
         rs.close();
      } catch ( Exception e ) {
         LogSystem.log ( 3, e );
      } finally {
         sql.close();
      }

      LogSystem.log(2, ">>> Action.findOpenDraft() completed");

      return (draftActionInd);
   }



   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the default title
    *  to this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Title if there is a default title, NULL if no titles exist
    *//*---------------------------------------------------------*/
   public Title getDefaultTitle () {
      if ( ( titles_ != null ) && ( titles_.size() > 0 ) )
         return ( (Title)titles_.elementAt ( titles_.size()-1 ) ) ;
      return ( null ) ;
   }

}

