
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.projects.common.storedProc.oaPublish.*;
import com.ibm.nzna.projects.common.storedProc.oaValidate.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;



/** 
 * Online Assistant Action Draft.
 * 
 * @version 1.23
 * @author Vishnu Kumar
 *
**/

public class ActionDraft extends Action implements DraftObject {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   private int         activeActionInd_ = 0;
   private boolean     maintLock_;
   private String      startDate_  = "";
   private String      stopDate_   = null;
   private boolean     recycled_;
   private TypeWorkRec typeWork_   = null;
   private String      comments_   = "";

   public ActionDraft() {
      super(0);

      TypeList t = TypeList.getInstance();
      typeWork_  = (TypeWorkRec) t.objectFromInd( 1, TypeList.TYPE_WORKREQUIRED );
   }

   public ActionDraft(int actionDraftInd) {
      super(actionDraftInd);
   }

   /**
    * Create a new ActionDraft object from an active Action.
   **/
   ActionDraft(Action publishedAction) {
      super(publishedAction);

      // set the active action indicator
      activeActionInd_ = publishedAction.actionInd_;
   }

   public int getActiveActionInd() {
      return (activeActionInd_);
   }

   public void setActiveActionInd(int activeActionInd) {
      activeActionInd_ = activeActionInd;
   }

   public boolean getMaintLock() {
      return (maintLock_);
   }

   public void setMaintLock(boolean maintLock)
   {
      maintLock_ = maintLock;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns the date the draft was opened.
   **/
   public String getStartDate() {  
      return (startDate_);
   }

   /**
    * Set the date the draft was opened.
    * <b>Note:</b> This value may be set, but it will not
    * used when the object is written to the database
    * (CURRENT DATE will be used).
   **/
   public void setStartDate(String startDate) {
      startDate_ = startDate;
   }

   /**
    * Returns the date the draft was closed or null if the draft is still open.
   **/
   public String getStopDate() {
      return (stopDate_);
   }

   /**
    * Set the date the draft was closed.
    * <b>Note:</b> This value may be set, but it will not
    * used when the object is written to the database
    * (CURRENT DATE will be used).
   **/
   public void setStopDate(String stopDate) {
      stopDate_ = stopDate;
   }

   /**
    * Returns true if this object is recycled.
   **/
   public boolean isRecycled() {
      return (recycled_);
   }

   /**
    * Set recycled flag for this object.
   **/
   public void setRecycled(boolean recycled) {
      recycled_ = recycled;
      updateRecStatus(REC_UPDATE);
   }

   public TypeWorkRec getTypeWorkRequired() {
      return (typeWork_);
   }

   public void setTypeWorkRequired(TypeWorkRec typeWork) {
      typeWork_ = typeWork;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Validate this object for publishing.  
    * 
    * @return a ValidationRec with the results of the validation or
    *         null if an error occurs.  Check the ValidationRec to 
    *         determine if publishing can proceed.
    * @see com.ibm.nzna.projects.common.storedProc.oaValidate.ValidationRec
   **/
   public ValidationRec validate() {
      LogSystem.log(2, ">>> Starting ActionDraft.validate()");
      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "", 10 );
      ValidationRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAValidate", 3 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter( 3, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, this.getInd() );         // object ind
         stmt.setInt( 2, OAValidate.ACTION );     // object type

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (ValidationRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "ValidationRec was returned null");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            retRec = null;   // return error to the user
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         retRec = null;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> ActionDraft.validate() complete");

      return (retRec);
   }

   /**
    * Assign a database key for this ActionDraft.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignMyDatabaseKey() {
      boolean success = true;

      if ( actionInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for ActionDraft");
         actionInd_ = Counter.getCounter( LogSystem.getInstance(), "DACTNIND" );
         if ( actionInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for ActionDraft");
            success = false;
         }
      }

      return (success);
   }

   /**
    * Write the ActionDraft and all its contained objects (with the exception
    * of Link and LinkGroup objects contained in DocLink objects) to the
    * database.  This method assumes that a DatabaseSystem object for
    * the DatabaseSystem.QUEST database has already been connected.
    * 
    * @return 0 for success, nonzero for failure.
   **/
   public int writeToDatabase() {
      LogSystem.log(2, ">>> Starting ActionDraft.writeToDatabase()");

      /*--------------------------------------------------------------*/
      /*   Assign database keys for this Action and its subobjects.   */
      /*--------------------------------------------------------------*/
      if ( !assignDatabaseKeys() ) {
         return (-1);
      }

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "ActionDraft.writeToDatabase()", 10 );
      int returnCode = 0 ;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc("QUEST.ExtSqlRunner", 3);
         String sqlToRun = ActionDraftWriter.getSQL(this);
         if ( sqlToRun.length() == 0 ) {
            LogSystem.log(1, "No changes to be written");
            return (0);
         }
         String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                       SqlRunner.END_DELIM, "\n" );

         LogSystem.log ( 1, "Starting to send statements to EXTSQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      
         stmt.setInt   ( 2, 0 );  // commit interval

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            returnCode = retRec.sqlCode ;
            if ( returnCode == 0 ) returnCode = -1;
         } else {
            // use a PreparedStatement to update the AddtlInfo text
            PreparedStatement pstmt = null;

            Vector addtlInfo = super.getAdditionalInfo();
            for ( int i=0; addtlInfo != null && i < addtlInfo.size(); i++ ) {
               AddtlInfo ai = (AddtlInfo) addtlInfo.elementAt(i);
               int recStatus = ai.getRecStatus();
               if ( recStatus == REC_INSERT || recStatus == REC_UPDATE ) {
                  LogSystem.log ( 1, "Updating additional information text for " +
                                  ai.getInd() ) ;
                  if ( pstmt == null ) {
                     // don't prepare the statement until we know it is needed.
                     pstmt = sqlm.createPreparedStatement(
                                                         "UPDATE oadraft.actionAddInfo " +
                                                         "SET addInfo=? " +
                                                         "WHERE actionAddInfoInd=?" +
                                                         "AND languageInd=?"
                                                         );
                  }
                  pstmt.setString( 1, ai.getAddtlInfo() );
                  pstmt.setInt( 2, ai.getInd() );
                  pstmt.setInt( 3, ai.getLanguage().getInd() );
                  pstmt.executeUpdate();
               }
            }

            if ( pstmt != null ) {
               pstmt.close();
            }
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         returnCode = -1 ;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> ActionDraft.writeToDatabase() complete");

      return (returnCode);
   }

   /**
    * Publish the draft Action.
    * Any changes to the draft must be saved to the database by
    * calling writeToDatabase() before the draft is published (the
    * publish method will not save the current object).
    * 
    * @return true for success, false for failure.
   **/
   public boolean publish() {
      LogSystem.log(2, ">>> Starting ActionDraft.publish()");

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "ActionDraft.publish()", 10 );
      boolean success = true;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAPublish", 3 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, this.getInd() );      // object ind
         stmt.setInt( 2, OAPublish.ACTION );   // object type

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         } else {
            // the userError String will contain the SQL generated for
            // publishing if DEBUG=true in the OAPublish stored procedure.
            if ( retRec.userError.length() > 0 ) {
               LogSystem.log(1, "Publish SQL:");
               LogSystem.log(2, retRec.userError);
            }
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            success = false;
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         success = false;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> ActionDraft.publish() complete");

      return (success);
   }

   /**
    * Read an ActionDraft object from the database.
    * 
    * @param actionDraftInd the database indicator of the ActionDraft to be read.
    * @param languageInd    the language to read the text items for 
    *                       (0 = read all languages).
    * @return the ActionDraft object or null if an error occured or
    *         the object was not found. The ActionDraft object is 
    *         returned as an Action object -- it must be cast.
   **/
   public static Action readFromDatabase(int actionDraftInd, int languageInd) {
      LogSystem.log(2, ">>> Starting ActionDraft.readFromDatabase()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "ActionDraft.readFromDatabase()", 5 );
      StoredProcRec retRec    = null;
      ActionDraft   retAction = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, actionDraftInd );         // object ind
         stmt.setInt( 2, OAReader.ACTION );        // object type
         stmt.setInt( 3, OAReader.READ_DRAFT );    // read draft object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Action and the StoredProcReturnRec objects      */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retAction = (ActionDraft)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      LogSystem.log(2, ">>> ActionDraft.readFromDatabase() completed");

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Do this after reading the question so that we           */
      /* do not open multiple connections to the database reading  */
      /* the type lists.                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retAction != null ) {
         retAction.updateTypeRecData();  // update the Type*Rec objects
         retAction.setRecNoChange();     // sets the original country list
      } else {
         LogSystem.log(2, "No object found for indicator " + actionDraftInd);
      }

      return (retAction);
   }

   /**
    * Get a draft Action.  This method will return the open draft of an
    * Action if one exists.  If there is no open draft, a new one will
    * be created.
    * <b>Note:</b> If a new draft is created, the openDraft flag in the
    * oa.action table will be set to 'Y'.  This flag will be reset to
    * 'N' when the draft is published.  If the draft is not saved, the
    * openDraft flag on oa.action will have to be manually reset.
    * 
    * 
    * @param actionInd   the database indicator of the published Action 
    *                    whose draft is to be read.
    * @param languageInd the language to read the text items for 
    *                    (0 = read all languages).
    * @param typeWork    the TypeWorkRec to be set into the
    *                    returned draft object if it is a new draft
    * @return the ActionDraft object or null if an error occured or
    *         the object was not found.
    * @exception PendingDraftException if there is a pending draft
   **/
   public static ActionDraft getDraft(int actionInd, int languageInd,
                                      TypeWorkRec typeWork)
   throws PendingDraftException
   {
      LogSystem.log(2, ">>> Starting ActionDraft.getDraft()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "ActionDraft.getDraft()", 5 );
      StoredProcRec retRec    = null;
      ActionDraft   retAction = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, actionInd );              // object ind
         stmt.setInt( 2, OAReader.ACTION );        // object type
         stmt.setInt( 3, OAReader.GET_DRAFT );     // read draft object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Action and the StoredProcReturnRec objects      */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retAction = (ActionDraft)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         } else {
            if ( retAction != null ) {
               if ( retAction.getRecStatus() == REC_INSERT ) {
                  retAction.setTypeWorkRequired( typeWork );
               } else {
                  retAction.setRecNoChange();  // sets original country list
               }
               retAction.updateTypeRecData(); // update the Type*Rec objects
            } else {
               LogSystem.log(2, "No object found for indicator " + actionInd);
            }
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      LogSystem.log(2, ">>> ActionDraft.getDraft() completed");

      // throw exception if the return code indicates a pending draft
      if ( retRec != null && retRec.error && 
           retRec.sqlCode == OAReader.PENDING_DRAFT_RC ) {
         throw new PendingDraftException();
      }

      return (retAction);
   }

   /**
    * Update the TypeRec objects in the Action's subobjects with 
    * the proper descriptions.
    * Called by readFromDatabase();
   **/
   void updateTypeRecData() {
      super.updateTypeRecData();

      TypeList t = TypeList.getInstance();

      // type work required
      TypeWorkRec newTypeWork;
      newTypeWork = (TypeWorkRec) t.objectFromInd( typeWork_.getInd(),
                                                   TypeList.TYPE_WORKREQUIRED );
      if ( newTypeWork != null ) {
         typeWork_ = newTypeWork;
      }

      return;
   }

   /*------------------------------------------------------------*/
   /* setComments
    */
   /**
    *    This method will set the comments
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setComments ( String comments ) {
      if ( comments == null ) comments = "";
      comments_ = comments;
      updateRecStatus(REC_UPDATE);
   }




   /*------------------------------------------------------------*/
   /* getComments
    */
   /**
    *    this method will return the comments
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getComments () {
      return (comments_);
   }





   /*------------------------------------------------------------*/
   /* forceCloseDraft
    */
   /**
    *    This method will set the value for 'OPENDRAFT'
    * in the OA.ACTION table to 'N' if there are no
    * open drafts.
    *  
    *  
    *
    *  @param databaseInd The database indicator for the ACTION
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void forceCloseDraft ( int databaseInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "forceCloseDraft", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         // update openDraft only if there are no open drafts
         stmt.executeUpdate (
                            "UPDATE oa.action SET openDraft='N' " +
                            "WHERE actionInd=" + databaseInd + " " +
                            "AND (" +
                            "SELECT actionInd FROM oadraft.action " +
                            "WHERE activeActionInd=" + databaseInd + " " +
                            "AND stopDate IS NULL" +
                            ") IS NULL"
                            );

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }
      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* resetAsNew
    */
   /**
    *    This method will take the current draft and
    *  creates a completely new draft. This newly
    *  created draft will produce a new action as well.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void resetAsNew () {                                          
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, set this indicator and active indicator to 0     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeActionInd_ = 0 ;
      setInd(0) ;                                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Let all child objects know of our new condition         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      initSubObjects ( this ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, make sure a new Symptom is created                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setTypeWorkRequired ( (TypeWorkRec)TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;
   }

}  

