
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;

import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant Actions.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.7
 * @author  Mark DeBiase
**/
public class ActionDraftReader {

  /**
   * Read and populate the specified ActionDraft object.
   * The ActionDraft object's indicator should contain the actionInd to be read.
   * @param conn  the database Connection object
   * @param a     the ActionDraft object to populate
   * @param lang  the language to read the sub-objects for (0 = all languages)
   * @return true if the ActionDraft was successfully read
             false if the ActionDraft was not found in the database
  **/
  public static boolean readFromDB(Connection conn, ActionDraft a, 
                                   int languageInd) 
                        throws SQLException
  {
    boolean    foundAction;
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    // turn on DB2 explain:
    //stmt.executeUpdate("SET CURRENT EXPLAIN MODE YES");
    //stmt.executeUpdate("SET CURRENT EXPLAIN SNAPSHOT YES");

    rs = stmt.executeQuery( 
           "SELECT " +
             "activeActionInd, actionTypeInd, " +
             "brandInd, typeWorkInd, docClassInd, " +
             "maintLock, startDate, stopDate, recycled, " +
             "archived, created, dbUser, changedTime, comments " +
           "FROM oadraft.action " +
           "WHERE actionInd=" + a.getInd()  + " " +
           "FOR FETCH ONLY");

    if ( !rs.next() ) {
      foundAction = false;
      stmt.close();
    } else {
      foundAction = true;

      //---------------------------------------------------------------------
      // Read the base ActionDraft data
      //---------------------------------------------------------------------
      a.setActiveActionInd( rs.getInt(1) );
      a.setActionType( new TypeActionRec( rs.getShort(2), "" ) );
      a.setBrand( new TypeBrandRec( rs.getInt(3), "", 0 ) );
      a.setTypeWorkRequired( new TypeWorkRec( rs.getInt(4), "") );
      a.setDocClass( new TypeDocClassRec( rs.getShort(5), "" ) );
      a.setMaintLock( DatabaseUtil.booleanFromDB( rs.getString(6) ) );
      a.setStartDate( rs.getString(7) );
      a.setStopDate( rs.getString(8) );
      a.setRecycled( DatabaseUtil.booleanFromDB( rs.getString(9) ) );
      a.setArchived( DatabaseUtil.booleanFromDB( rs.getString(10) ) );
      a.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(11) ) );
      a.setDbUser( rs.getString(12).trim() );
      a.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(13) ) );
      a.setComments( rs.getString(14) );
      if ( rs.wasNull() ) {
        a.setComments("");
      }

      stmt.close();

      //---------------------------------------------------------------------
      // Read the Action Countries
      //---------------------------------------------------------------------
      a.setCountryList(
         CountryReader.readCountries( conn, "oadraft.actionCountry",
                                      "actionInd", a.getInd() )
      );

      //---------------------------------------------------------------------
      // Read the Action Titles/AddtlInfo based on the language passed in
      // DRAFT AddtlInfo is always read in DEFAULT mode - only used by QIT.
      //---------------------------------------------------------------------
      TitleReader.readActionDraftTitles( conn, a, languageInd );
      AddtlInfoReader.readActionDraftAddtlInfo( conn, a, languageInd,
                                                OAMode.DEFAULT );

      //---------------------------------------------------------------------
      // Read the Action DocLinks
      //---------------------------------------------------------------------
      DocLinkReader.readActionDraftDocLinks( conn, a );

      a.updateRecStatus( a.REC_NOCHANGE );
    }

    return foundAction;
  }

  /**
   * Get a draft for an Action.
   * Check to see if there is an open draft -- if there is, then return it.
   * If there is no open draft, then create a new one.
   *
   * @exception PendingDraftException if openDraft='Y' on the active action
   *            and there is no open draft with stopDate=NULL
  **/
  public static ActionDraft getDraft(Connection conn, int actionInd, 
                                     int languageInd) 
                            throws SQLException, PendingDraftException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    ActionDraft returnedDraft = null;

    // check to see if there is already an open draft....
    rs = stmt.executeQuery(
      "SELECT actionInd " +
      "FROM oadraft.action " +
      "WHERE stopDate IS NULL AND activeActionInd=" + actionInd + " " +
      "FOR FETCH ONLY"
    );

    if ( rs.next() ) {
      // there is an open draft -- get it.
      int draftActionInd = rs.getInt(1);
      rs.close();
      ActionDraft ad = new ActionDraft( draftActionInd );
      if ( ActionDraftReader.readFromDB( conn, ad, languageInd ) ) {
        returnedDraft = ad;
      }
    } else {
      // there is no open draft -- read the action & create a new draft.
      rs.close();
      Action a = new Action( actionInd );
      if ( ActionReader.readFromDB( conn, a, languageInd, OAMode.DEFAULT ) ) {
        if ( a.hasOpenDraft() ) {
          throw new PendingDraftException();
        }
        ActionDraft ad = new ActionDraft( a );
        returnedDraft = ad;
        // if we opened a new draft, then update openDraft in oa.action
        stmt.executeUpdate(
          "UPDATE oa.action " +
          "SET openDraft='Y' " + 
          "WHERE actionInd=" + actionInd
        );
      }
    }

    stmt.close();

    return returnedDraft;
  }
}
