
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;

/**
 * Holds the Additional Information text for an Action or Question.
 * <p>
 * A link to a Quest document or graphics can be embedded in the
 * additional information text.  The AddtlInfo object maintains
 * a list of the Quest document indicators and a list of the
 * graphics that are embedded in the text.
 *
 * @version 1.22
 * @author  Mark DeBiase
**/
public class AddtlInfo extends ConditionalObject {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   private int        addtlInfoInd_   = 0;     // database indicator
   private int        parentInd_      = 0;     // foreign key to parent table
   private String     addtlInfo_      = null;  // additional info text
   private TypeLanguageRec language_  = null;  // language
   private String     lastTranslated_ = null;  // last translation date
   private int        parentTitleInd_ = 0;     // title to inherit 
                                               // conditions from
   private boolean    redirect_       = false; // valid for Action AddtlInfo only
   private boolean    textUpdated_    = false; // set to true if addtlinfo text 
                                               // is updated
   private String     addtlInfoSubString = null ;

   // List of Quest Documents (just the docInds) and Graphics that may be
   // embedded in the additional information text.
   private Vector     embeddedDocInds_  = null;
   private Vector     embeddedGraphics_ = null;

   /**
    * New object.
   **/
   public AddtlInfo() {
      this("");
   }

   /**
    * New object, with specified additional information text.
   **/
   public AddtlInfo(String info) {
      super();
      addtlInfo_ = info;
      updateRecStatus(REC_INSERT);
   }

   /**
    * AddltInfo object read from the database.
   **/
   AddtlInfo(int addtlInfoInd) {
      super();
      addtlInfoInd_ = addtlInfoInd;
      updateRecStatus(REC_NOCHANGE);
   }

   /**
    * Assign a database key for this AddtlInfo.
    * This method can be called for additional information attached to
    * either an Action or a Question so the counter name in tigris.counter
    * to be used is passed as a parameter.
    * @param  counterName the counter name to use to get the next key
    * @return true if a database key was assigned, otherwise false
   **/
   private boolean assignDatabaseKey(String counterName) {
      boolean success = true;

      if ( addtlInfoInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for AddtlInfo: " + counterName);
         addtlInfoInd_ = Counter.getCounter(LogSystem.getInstance(), counterName);
         if ( addtlInfoInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for AddtlInfo");
            success = false;
         }
      }

      return success;
   }
   // version to be called by publish stored procedure:
   private void assignDatabaseKey(Connection conn, String counterName)
   throws Exception
   {

      if ( addtlInfoInd_ == 0 ) {
         addtlInfoInd_ = Counter.getCounter(conn, counterName);
         if ( addtlInfoInd_ == 0 ) {
            throw new Exception("Failed to get database key for AddtlInfo");
         }
      }

      return;
   }

   /**
    * Assign database key for Action additional information.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForAction() {
      return assignDatabaseKey("ACTIONINFO");
   }
   void assignDatabaseKeyForAction(Connection conn) throws Exception {
      assignDatabaseKey(conn, "ACTIONINFO");
   }
   /**
    * Assign database key for Question additional information.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForQuestion() {
      return assignDatabaseKey("QSTNINFO");
   }
   void assignDatabaseKeyForQuestion(Connection conn) throws Exception {
      assignDatabaseKey(conn, "QSTNINFO");
   }
   /**
    * Assign database key for draft Action additional information.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForActionDraft() {
      return assignDatabaseKey("DACTNINFO");
   }
   void assignDatabaseKeyForActionDraft(Connection conn) throws Exception {
      assignDatabaseKey(conn, "DACTNINFO");
   }
   /**
    * Assign database key for draft Question additional information.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForQuestionDraft() {
      return assignDatabaseKey("DQSTNINFO");
   }
   void assignDatabaseKeyForQuestionDraft(Connection conn) throws Exception {
      assignDatabaseKey(conn, "DQSTNINFO");
   }

   /**
    * Get the database indicator for this object.
   **/
   public int getInd() {
      return addtlInfoInd_;
   }

   /**
    * Set the database indicator for this object.
   **/
   void setInd(int addtlInfoInd) {
      addtlInfoInd_ = addtlInfoInd;
   }

   /**
    * Get the parent objects's indicator for this object.
   **/
   int getParentInd() {
      return parentInd_;
   }

   /**
    * Set the parent objects's indicator for this object.
   **/
   void setParentInd(int parentInd) {
      parentInd_ = parentInd;
   }

   /**
    * Get the additional information String.
   **/
   public String getAddtlInfo() {
      return addtlInfo_;
   }

   /**
    * Set the additional information String.
   **/
   public void setAddtlInfo(String addtlInfo) {
      addtlInfo_   = addtlInfo;
      textUpdated_ = true;
      updateRecStatus(REC_UPDATE);                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we must create our addtlInfoSubString              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //try {
      //   addtlInfoSubString = addtlInfo.substring( 0, 50 ) ;
      //} catch ( Exception e ) {
         addtlInfoSubString = addtlInfo ;
      //}

      addtlInfoSubString = Text.stripHTML ( addtlInfoSubString ) ;
      addtlInfoSubString = Text.stripLineBreaks ( addtlInfoSubString ) ;

   }

   /**
    * Called by Readers to reset the textUpdated flag after reading 
    * an AddtlInfo object from the database.
   **/
   void setTextUpdated(boolean textUpdated) {
      textUpdated_ = textUpdated;
   }

   /**
    * Returs true if the addtional info text has been updated.
   **/
   boolean wasTextUpdated() {
      return textUpdated_;
   }

   /**
    * Gets the language.
    * @return the language
   **/
   public TypeLanguageRec getLanguage() {
      return language_;
   }

   /**
    * Sets the language.
    * @parm language the language
   **/
   public void setLanguage(TypeLanguageRec language) {
      language_ = language;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Get the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @return a String representing a DB2 timestamp
   **/
   public String getLastTranslationDate() {
      return lastTranslated_;
   }

   /**
    * Set the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @param translationDate a String representing a DB2 timestamp
   **/
   public void setLastTranslationDate(String translationDate) {
      lastTranslated_ = translationDate;
   }

   /**
    * Get the indicator of the Title whose Conditions this object will inherit.
    * A return value of 0 means that no Conditions are inherited.
   **/
   public int getParentTitleInd() {
      return parentTitleInd_;
   }

   /**
    * Set the indicator of the Title whose Conditions this object will inherit.
    * A value of 0 means that no Conditions will be inherited.
   **/
   public void setParentTitleInd(int parentTitleInd) {
      parentTitleInd_ = parentTitleInd;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns the value of the redirect flag.  
    * If set to true, the user should be redirected to the first link
    * found in the additional information text.
    * <p>
    * <b>Note:</b>This method will always return false AddtlInfo objects
    * in a Question that is read from the database.  It only applies to
    * Action AddtlInfo objects.
   **/
   public boolean getRedirectFlag() {
      return redirect_;
   }

   /**
    * Set the redirect flag for this object.
    * If set to true, this indicates that the user should be redirected to
    * the first HTML link in the additional information text.
    * <b>Note</b>This value is only valid for Action additional information.
    * It will be ignored when writing Question additional information to the
    * database and it will always be set to false when Question additional
    * information is read from the database.
   **/
   public void setRedirectFlag(boolean redirect) {
      redirect_ = redirect;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Get the list of embedded Quest Documents.  
    * This is returned as a Vector of Integer objects, each of which
    * is a Quest document indicator for an embedded object.
    * @return a Vector of Integer objects or null if there are no 
    *         embedded documents.
   **/
   public Vector getEmbeddedDocInds() {
      return embeddedDocInds_;
   }

   /**
    * Set the list of embedded Quest Documents.  
    * This should be a Vector of Integer objects, each of which is the
    * Quest document indicator for an embedded document.
    * @param embeddedDocInds a Vector of Integer objects
   **/
   public void setEmbeddedDocInds(Vector embeddedDocInds) {
      embeddedDocInds_ = embeddedDocInds;
   }

   /**
    * Get the list of embedded graphics.
    * @return a Vector of AddtlInfoGraphic objects or null if there are
    *         no embedded graphhics.
    *
   **/
   public Vector getEmbeddedGraphics() {
      return embeddedGraphics_;
   }

   /**
    * Set the list of embedded graphics.
    * @param embeddedGraphics a Vector of AddtlInfoGraphic objects
   **/
   public void setEmbeddedGraphics(Vector embeddedGraphics) {
      embeddedGraphics_ = embeddedGraphics;
   }

   /**
    * Perform a shallow copy of this object.
   **/
   public Object clone() {
      return super.clone();
   }

   /**
    * Return a String representation of this object.
   **/
   public String toString() {
      return addtlInfoSubString ;
   }

   public String debugString() {
      return "AddtlInfo " + addtlInfoInd_ + ": \"" + addtlInfo_ + 
      "\" (lang=" + language_.getInd() + ")";
   }

   /**
    * Reset values in this object.
    * Sets the indicator to 0, sets the recStatus to REC_INSERT.
   **/
   public void resetNew() {
      setInd(0);
      setParentInd(0);
      // Force the insert
      updateRecStatus(REC_NOCHANGE);
      updateRecStatus(REC_INSERT);
   }

   /**
    * Update the TypeRec objects in the AddtlInfo object with 
    * the proper descriptions.
    * Called after an AddtlInfo is read from the database.
   **/
   void updateTypeRecData() {
      super.updateTypeRecData();

      TypeList t = TypeList.getInstance();

      // language
      TypeLanguageRec newLangRec;
      newLangRec = (TypeLanguageRec) t.objectFromInd( language_.getInd(),
                                                      TypeList.TYPE_LANGUAGE );
      if ( newLangRec != null ) {
         language_ = newLangRec;
      }

      // update the TypeRecs for the embedded graphics objects
      if ( embeddedGraphics_ != null ) {
         for ( int i=0; i<embeddedGraphics_.size(); i++ ) {
            AddtlInfoGraphic g = (AddtlInfoGraphic) embeddedGraphics_.elementAt(i);
            g.updateTypeRecData();
         }
      }

      return;
   }
}

