
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant AddtlInfo objects.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.9
 * @author Vishnukumar. E
**/

class AddtlInfoReader {

  private static final String ACTIVE_SCHEMA = "oa";
  private static final String DRAFT_SCHEMA  = "oadraft";

//=============================================================================

  /**
   * Read Action Additional Info for all languages (oa schema).
   * @param conn         the database connection
   * @param a            the Action whose AddtlInfo is to be read
   * @param mode         one of the values defined in the OAMode class
  **/
  static void readActionAddtlInfo(Connection conn, int mode, Action a) 
         throws SQLException
  {
    readActionAddtlInfo(conn, a , mode, ACTIVE_SCHEMA);
  }

  /**
   * Read Action Additional Info for all languages (oadraft schema).
   * @param conn         the database connection
   * @param a            the Action whose AddtlInfo is to be read
   * @param mode         one of the values defined in the OAMode class
  **/
  static void readActionDraftAddtlInfo(Connection conn, int mode, Action a) 
         throws SQLException
  {
    readActionAddtlInfo(conn, a , mode, DRAFT_SCHEMA);
  }

  /**
   * Read Action Additional Info for all languages (common function).
   * @param conn         the database connection
   * @param a            the Action whose AddtlInfo is to be read
   * @param mode         one of the values defined in the OAMode class
   * @param schema       the database schema (oa or oadraft)
  **/
  private static void readActionAddtlInfo(Connection conn, Action a,
                                          int mode, String schema) 
                 throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // select all additional info
    rs = stmt.executeQuery(
           "SELECT " +
              "actionAddInfoInd, sortOrder, addInfo, titleInd, " +
              "redirectFlag, languageInd, lastTranslated, docClassInd, " +
              "created, dbUser, changedTime " +
           "FROM " + schema + ".actionAddInfo " +
           "WHERE actionInd=" + a.getInd() + " " +
           "ORDER BY sortOrder, languageInd " +
           "FOR FETCH ONLY");

    Vector v = new Vector();
    while ( rs.next() ) 
    {
      AddtlInfo ai = new AddtlInfo( rs.getInt(1) );
      ai.setParentInd( a.getInd() );

      ai.setSortOrder( rs.getShort(2) );
      ai.setAddtlInfo( rs.getString(3) );
      ai.setParentTitleInd( rs.getInt(4) );
      ai.setRedirectFlag( DatabaseUtil.booleanFromDB( rs.getString(5) ));
      ai.setLanguage( new TypeLanguageRec( rs.getShort(6), "" ) );
      ai.setLastTranslationDate( 
         DatabaseUtil.timestampFromDB( rs.getString(7) ) );
      ai.setDocClass( new TypeDocClassRec( rs.getShort(8), "") );
      ai.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(9) ) );
      ai.setDbUser( rs.getString(10).trim() );
      ai.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(11) ) );

      // read the embedded documents/graphics -- but not if OAMode.WEB
      if (mode != OAMode.WEB) {
        ai.setEmbeddedDocInds(
          readEmbeddedDocs( conn, schema + ".actionInfoLinks",
                            "actionAddInfoInd", ai.getInd() )
        );
        ai.setEmbeddedGraphics(
          readEmbeddedGraphics( conn, schema + ".actionInfoGraphic",
                                "actionAddInfoInd", ai.getInd() )
        );
      }

      // read the conditions/countries
      // MAD performance: addtlinfo conditions are now read all at once below
      //ai.setConditions(
      //   ConditionReader.readConditions( conn, schema + ".actionInfoCond",
      //                                   "actionAddInfoInd", ai )
      //);
      // MAD performance: addtlinfo countries are now read all at once below
      //ai.setCountryList(
      //   CountryReader.readCountries( conn, schema + ".actionInfoCountry",
      //                                "actionAddInfoInd", ai.getInd() )
      //);

      ai.setTextUpdated( false );
      ai.updateRecStatus( ai.REC_NOCHANGE );
      v.addElement(ai);
    }

    rs.close();
    stmt.close();

    a.setAdditionalInfo(v);

    // if we read any addtlinfo objects, then get their condtions/countries
    // addtlinfo is optional, so it is possible that there is none
    if (v.size() > 0) {
      ConditionReader.readAllActionAddtlInfoConditions(conn, a, schema, true);
      CountryReader.readAllActionAddtlInfoCountries(conn, a, schema, true);
    }
  }

//=============================================================================

  /**
   * Read Action Additional Info for a specific language (oa schema).
   * If languageInd=0 (read all languages), 
   * then readActionAddtlInfo(conn, q) will be called.
   * @param conn         the database connection
   * @param a            the Action whose AddtlInfo is to be read
   * @param languageInd  the language to be read
   * @param mode         one of the values defined in the OAMode class
  **/
  static void readActionAddtlInfo(Connection conn, Action a, 
                                  int languageInd, int mode)
         throws SQLException 
  {
    if (languageInd == 0) {
      readActionAddtlInfo(conn, a, mode, ACTIVE_SCHEMA);
    } else {
      readActionAddtlInfo(conn, a, languageInd, mode, ACTIVE_SCHEMA);
    }
  }

  /**
   * Read Action Additional Info for a specific language (oadraft schema).
   * If languageInd=0 (read all languages), 
   * then readActionAddtlInfo(conn, q) will be called.
   * @param conn         the database connection
   * @param a            the Action whose AddtlInfo is to be read
   * @param languageInd  the language to be read
   * @param mode         one of the values defined in the OAMode class
  **/
  static void readActionDraftAddtlInfo(Connection conn, Action a,
                                       int languageInd, int mode)
         throws SQLException 
  {
    if (languageInd == 0) {
      readActionAddtlInfo(conn, a, mode, DRAFT_SCHEMA);
    } else {
      readActionAddtlInfo(conn, a, languageInd, mode, DRAFT_SCHEMA);
    }
  }

  /**
   * Read Action Additional Info for a specific language (common function).
   * This method will attempt to read the addtl info in the specified language.
   * If an addtl info is not found in the requested language, the English 
   * addtl info will be read.
   * @param conn         the database connection
   * @param a            the Action whose AddtlInfo is to be read
   * @param languageInd  the language to be read
   * @param mode         one of the values defined in the OAMode class
   * @param schema       the database schema (oa or oadraft)
  **/
  private static void readActionAddtlInfo(Connection conn, Action a, 
                                          int languageInd, int mode, 
                                          String schema)
                      throws SQLException
  {
    // NOTE: can't use UNION here as we do in TitleReader because the
    //       actionAddtlInfo table has a LONG VARCHAR column --
    //       can't UNION on a LONG VARCHAR.

    PreparedStatement pstmt = null;
    Statement   stmt = conn.createStatement();
    ResultSet   rs1,    // result set for all unique addtl info indicators
                rs2;    // result set for specific addtl info/language

    // select all the addtl info indicators for the action
    rs1 = stmt.executeQuery( "SELECT DISTINCT actionAddInfoInd, sortOrder " +
                             "FROM " + schema + ".actionAddInfo " +
                             "WHERE actionInd=" + a.getInd() + " " +
                             "ORDER BY sortOrder " +
                             "FOR FETCH ONLY" );

    Vector v = new Vector();         
    while( rs1.next() ) {

      if (pstmt == null) {  
        // delay preparing the statment until we see if we really need it
        // because there may be no addtl infos for the action
        pstmt = conn.prepareStatement( 
           "SELECT " +
              "actionAddInfoInd, sortOrder, addInfo, titleInd, " +
              "redirectFlag, languageInd, lastTranslated, docClassInd, " +
              "created, dbUser, changedTime " +
           "FROM " + schema + ".actionAddInfo " +
           "WHERE actionAddInfoInd=? " +
           "AND languageInd=? " +
           "FOR FETCH ONLY" );
      }

      // try to read the addtl info in the requested language
      int actionAddInfoInd = rs1.getInt(1);
      pstmt.setInt(1, actionAddInfoInd);
      pstmt.setInt(2, languageInd);
      rs2 = pstmt.executeQuery();
  
      if(rs2.next() == false)
      {
        // could not find the requested language -- read English addtl info
        pstmt.setInt(1, actionAddInfoInd);
        pstmt.setInt(2, 1);
        rs2 = pstmt.executeQuery();
        rs2.next();
      }            

      AddtlInfo ai = new AddtlInfo( rs2.getInt(1) );
      ai.setParentInd( a.getInd() );

      ai.setSortOrder( rs2.getShort(2) );
      ai.setAddtlInfo( rs2.getString(3) );
      ai.setParentTitleInd( rs2.getInt(4) );
      ai.setRedirectFlag( DatabaseUtil.booleanFromDB( rs2.getString(5) ));
      ai.setLanguage( new TypeLanguageRec( rs2.getShort(6), "" ) );
      ai.setLastTranslationDate( 
         DatabaseUtil.timestampFromDB( rs2.getString(7) ) );
      ai.setDocClass( new TypeDocClassRec( rs2.getShort(8), "") );
      ai.setCreationTime( DatabaseUtil.timestampFromDB( rs2.getString(9) ) );
      ai.setDbUser( rs2.getString(10).trim() );
      ai.setChangedTime( DatabaseUtil.timestampFromDB( rs2.getString(11) ) );
      rs2.close();

      // read the embedded documents/graphics -- but not if OAMode.WEB
      if (mode != OAMode.WEB) {
        ai.setEmbeddedDocInds(
          readEmbeddedDocs( conn, schema + ".actionInfoLinks",
                            "actionAddInfoInd", ai.getInd() )
        );
        ai.setEmbeddedGraphics(
          readEmbeddedGraphics( conn, schema + ".actionInfoGraphic",
                                "actionAddInfoInd", ai.getInd() )
        );
      }

      // read the conditions/countries
      // MAD performance: addtlinfo conditions are now read all at once below
      //ai.setConditions(
      //   ConditionReader.readConditions( conn, schema + ".actionInfoCond",
      //                                   "actionAddInfoInd", ai )
      //);
      // MAD performance: addtlinfo countries are now read all at once below
      //ai.setCountryList(
      //   CountryReader.readCountries( conn, schema + ".actionInfoCountry",
      //                                "actionAddInfoInd", ai.getInd() )
      //);

      ai.setTextUpdated( false );
      ai.updateRecStatus( ai.REC_NOCHANGE );
      v.addElement(ai);
    }  

    rs1.close();
    stmt.close();
    if (pstmt != null) {
      pstmt.close();
    }

    a.setAdditionalInfo(v);

    // if we read any addtlinfo objects, then get their condtions/countries
    // addtlinfo is optional, so it is possible that there is none
    if (v.size() > 0) {
      ConditionReader.readAllActionAddtlInfoConditions(conn, a, schema, false);
      CountryReader.readAllActionAddtlInfoCountries(conn, a, schema, false);
    }
  }  

//=============================================================================

  /**
   * Read Question Additional Info for all languages (oa schema).
   * @param conn  the database connection
   * @param q     the Question whose AddtlInfo is to be read
   * @param mode  one of the values defined in the OAMode class
  **/
  static void readQuestionAddtlInfo(Connection conn, Question q, int mode) 
         throws SQLException
  {
    readQuestionAddtlInfo(conn, q, mode, ACTIVE_SCHEMA);
  }

  /**
   * Read Question Additional Info for all languages (oadraft schema).
   * @param conn  the database connection
   * @param q     the Question whose AddtlInfo is to be read
   * @param mode  one of the values defined in the OAMode class
  **/
  static void readQuestionDraftAddtlInfo(Connection conn, Question q, int mode) 
         throws SQLException
  {
    readQuestionAddtlInfo(conn, q, mode, DRAFT_SCHEMA);
  }

  /**
   * Read Question Additional Info for all languages (common function).
   * @param conn    the database connection
   * @param q       the Question whose AddtlInfo is to be read
   * @param mode    one of the values defined in the OAMode class
   * @param schema  the database schema (oa or oadraft)
  **/
  private static void readQuestionAddtlInfo(Connection conn, Question q,
                                            int mode, String schema) 
                 throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // select all additional info
    rs = stmt.executeQuery(
           "SELECT " +
              "questionAddInfoInd, sortOrder, addInfo, titleInd, " +
              "languageInd, lastTranslated, docClassInd, " +
              "created, dbUser, changedTime " +
           "FROM " + schema + ".questionAddInfo " +
           "WHERE questionInd=" + q.getInd() + " " +
           "ORDER BY sortOrder, languageInd " +
           "FOR FETCH ONLY");

    Vector v = new Vector();
    while ( rs.next() ) 
    {
      AddtlInfo ai = new AddtlInfo( rs.getInt(1) );
      ai.setParentInd( q.getInd() );

      ai.setSortOrder( rs.getShort(2) );
      ai.setAddtlInfo( rs.getString(3) );
      ai.setParentTitleInd( rs.getInt(4) );
      ai.setLanguage( new TypeLanguageRec( rs.getShort(5), "" ) );
      ai.setLastTranslationDate( 
         DatabaseUtil.timestampFromDB( rs.getString(6) ) );
      ai.setDocClass( new TypeDocClassRec( rs.getShort(7), "") );
      ai.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(8) ) );
      ai.setDbUser( rs.getString(9).trim() );
      ai.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(10) ) );

      // read the embedded documents/graphics -- but not if OAMode.WEB
      if (mode != OAMode.WEB) {
        ai.setEmbeddedDocInds(
          readEmbeddedDocs( conn, schema + ".questionInfoLinks",
                            "questionAddInfoInd", ai.getInd() )
        );
        ai.setEmbeddedGraphics(
          readEmbeddedGraphics( conn, schema + ".questionInfoGrphic",
                                "questionAddInfoInd", ai.getInd() )
        );
      }

      // read the conditions/countries
      // MAD performance: addtlinfo conditions are now read all at once below
      //ai.setConditions(
      //   ConditionReader.readConditions( conn, schema + ".questionInfoCond",
      //                                   "questionAddInfoInd", ai )
      //);
      // MAD performance: addtlinfo conditions are now read all at once below
      //ai.setCountryList(
      //   CountryReader.readCountries( conn, schema + ".questionInfoCntry",
      //                                "questionAddInfoInd", ai.getInd() )
      //);

      ai.setTextUpdated( false );
      ai.updateRecStatus( ai.REC_NOCHANGE );
      v.addElement(ai);
    }

    rs.close();
    stmt.close();

    q.setAdditionalInfo(v);

    // if we read any addtlinfo objects, then get their condtions/countries
    // addtlinfo is optional, so it is possible that there is none
    if (v.size() > 0) {
      ConditionReader.readAllQuestionAddtlInfoConditions(conn, q, schema, true);
      CountryReader.readAllQuestionAddtlInfoCountries(conn, q, schema, true);
    }
  }

//=============================================================================

  /**
   * Read Question Additional Info for a specific language (oa schema).
   * If languageInd=0 (read all languages), 
   * then readQuestionAddtlInfo(conn, q) will be called.
   * @param conn         the database connection
   * @param q            the Question whose AddtlInfo is to be read
   * @param languageInd  the language to be read
   * @param mode         one of the values defined in the OAMode class
  **/
  static void readQuestionAddtlInfo(Connection conn, Question q, 
                                    int languageInd, int mode) 
              throws SQLException
  {
    if (languageInd == 0) {
      readQuestionAddtlInfo(conn, q, mode, ACTIVE_SCHEMA);
    } else {
      readQuestionAddtlInfo(conn, q, languageInd, mode, ACTIVE_SCHEMA);
    }
  }

  /**
   * Read Question Additional Info for a specific language (oadraft schema).
   * If languageInd=0 (read all languages), 
   * then readQuestionAddtlInfo(conn, q) will be called.
   * @param conn         the database connection
   * @param q            the Question whose AddtlInfo is to be read
   * @param languageInd  the language to be read
   * @param mode         one of the values defined in the OAMode class
  **/
  static void readQuestionDraftAddtlInfo(Connection conn, Question q, 
                                         int languageInd, int mode) 
              throws SQLException
  {
    if (languageInd == 0) {
      readQuestionAddtlInfo(conn, q, mode, DRAFT_SCHEMA);
    } else {
      readQuestionAddtlInfo(conn, q, languageInd, mode, DRAFT_SCHEMA);
    }
  }

  /**
   * Read Question Additional Info for a specific language (common function).
   * This method will attempt to read the addtl info in the specified language.
   * If an addtl info is not found in the requested language, the English 
   * addtl info will be read.
   * @param conn         the database connection
   * @param q            the Question whose AddtlInfo is to be read
   * @param languageInd  the language to be read
   * @param mode         one of the values defined in the OAMode class
   * @param schema       the database schema (oa or oadraft)
  **/
  private static void readQuestionAddtlInfo(Connection conn, Question q, 
                                            int languageInd, int mode, 
                                            String schema)
         throws SQLException
  {
    // NOTE: can't use UNION here as we do in TitleReader because the
    //       questionAddtlInfo table has a LONG VARCHAR column --
    //       can't UNION on a LONG VARCHAR.

    PreparedStatement pstmt = null;
    Statement   stmt = conn.createStatement();
    ResultSet   rs1,    // result set for all unique addtl info indicators
                rs2;    // result set for specific addtl info/language
        
    // select all the addtl info indicators for the action
    rs1 = stmt.executeQuery( "SELECT DISTINCT questionAddInfoInd, sortOrder " +
                             "FROM " + schema + ".questionAddInfo " +
                             "WHERE questionInd=" + q.getInd() + " " +
                             "ORDER BY sortOrder " +
                             "FOR FETCH ONLY" );


    Vector v = new Vector();         
    while( rs1.next() ) {
      if (pstmt == null) {
        // delay preparing the statment until we see if we really need it
        // because there may be no addtl infos for the action
        pstmt = conn.prepareStatement( 
           "SELECT " +
              "questionAddInfoInd, sortOrder, addInfo, titleInd, " +
              "languageInd, lastTranslated, docClassInd, " +
              "created, dbUser, changedTime " +
           "FROM " + schema + ".questionAddInfo " +
           "WHERE questionAddInfoInd=? " +
           "AND languageInd=? " +
           "FOR FETCH ONLY" );
      }

      // try to read the addtl info in the requested language
      int questionAddInfoInd = rs1.getInt(1);
      pstmt.setInt(1, questionAddInfoInd);
      pstmt.setInt(2, languageInd);
      rs2 = pstmt.executeQuery();
  
      if(rs2.next() == false)
      {
        // could not find the requested language -- read English addtl info
        pstmt.setInt(1, questionAddInfoInd);
        pstmt.setInt(2, 1);
        rs2 = pstmt.executeQuery();
        rs2.next();
      }

      AddtlInfo ai = new AddtlInfo( rs2.getInt(1) );
      ai.setParentInd( q.getInd() );

      ai.setSortOrder( rs2.getShort(2) );
      ai.setAddtlInfo( rs2.getString(3) );
      ai.setParentTitleInd( rs2.getInt(4) );
      ai.setLanguage( new TypeLanguageRec( rs2.getShort(5), "" ) );
      ai.setLastTranslationDate( 
         DatabaseUtil.timestampFromDB( rs2.getString(6) ) );
      ai.setDocClass( new TypeDocClassRec( rs2.getShort(7), "") );
      ai.setCreationTime( DatabaseUtil.timestampFromDB( rs2.getString(8) ) );
      ai.setDbUser( rs2.getString(9).trim() );
      ai.setChangedTime( DatabaseUtil.timestampFromDB( rs2.getString(10) ) );
      rs2.close();

      // read the embedded documents/graphics -- but not if OAMode.WEB
      if (mode != OAMode.WEB) {
        ai.setEmbeddedDocInds(
          readEmbeddedDocs( conn, schema + ".questionInfoLinks",
                            "questionAddInfoInd", ai.getInd() )
        );
        ai.setEmbeddedGraphics(
          readEmbeddedGraphics( conn, schema + ".questionInfoGrphic",
                                "questionAddInfoInd", ai.getInd() )
        );
      }

      // read the conditions/countries
      // MAD performance: addtlinfo conditions are now read all at once below
      //ai.setConditions(
      //   ConditionReader.readConditions( conn, schema + ".questionInfoCond",
      //                                   "questionAddInfoInd", ai )
      //);
      // MAD performance: addtlinfo conditions are now read all at once below
      //ai.setCountryList(
      //   CountryReader.readCountries( conn, schema + ".questionInfoCntry",
      //                                "questionAddInfoInd", ai.getInd() )
      //);

      ai.setTextUpdated( false );
      ai.updateRecStatus( ai.REC_NOCHANGE );
      v.addElement(ai);
    }
    
    rs1.close();
    stmt.close();
    if (pstmt != null) {
      pstmt.close();
    }
     
    q.setAdditionalInfo(v);

    // if we read any addtlinfo objects, then get their condtions/countries
    // addtlinfo is optional, so it is possible that there is none
    if (v.size() > 0) {
      ConditionReader.readAllQuestionAddtlInfoConditions(conn, q, schema,false);
      CountryReader.readAllQuestionAddtlInfoCountries(conn, q, schema, false);
    }
  }

//=============================================================================

  /**
   * Read the list of embedded quest documents (docInds).
  **/
  // NOTE: this method should be replaced with new methods to read the
  //       embedded docs for all addtlinfo in a action/question at once.
  private static Vector readEmbeddedDocs(Connection conn, String tableName,
                                         String indName, int indValue)
                        throws SQLException 
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // select all embedded graphics
    rs = stmt.executeQuery(
           "SELECT docInd " +
           "FROM " + tableName + " " +
           "WHERE " + indName + "=" + indValue
    );

    Vector v = new Vector(1, 3);
    while ( rs.next() ) {
      int docInd = rs.getInt(1);
      v.addElement( new Integer(docInd) );
    }
    stmt.close();

    return v;
  }

  /**
   * Read the list of embedded graphics (all languages).
  **/
  // NOTE: this method should be replaced with new methods to read the
  //       embedded graphics for all addtlinfo in a action/question at once.
  private static Vector readEmbeddedGraphics(Connection conn, String tableName,
                                             String indName, int indValue)
                        throws SQLException 
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // select all embedded graphics
    rs = stmt.executeQuery(
           "SELECT a.graphicInd, a.languageInd, a.lastTranslated, a.docClassInd, b.filename " +
           "FROM    "+tableName+" a, TIGRIS.GRAPHICS B " +
           "WHERE A." + indName + "=" + indValue+" AND "+
           "      A.GRAPHICIND = B.GRAPHICIND FOR FETCH ONLY"
    );

    Vector v = new Vector(1, 3);
    while ( rs.next() ) {

      AddtlInfoGraphic graphic = new AddtlInfoGraphic( rs.getInt(1) );
      graphic.setLanguage( new TypeLanguageRec( rs.getShort(2), "" ) );
      graphic.setLastTranslationDate(
        DatabaseUtil.timestampFromDB( rs.getString(3) )
      );
      graphic.setDocClass ( new TypeDocClassRec( rs.getShort(4), "" ) );
      graphic.setFilename ( rs.getString ( 5 ).trim() ) ;

      v.addElement( graphic );
    }
    stmt.close();

    return v;
  }
}  
