
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;

/**
 * Represents one Answer to a Question.
 * An Answer is uniquely identified by the answer indicator.
 * <p>
 * There may be Conditions on an Answer to determine whether the
 * Answer will be displayed to the user in a specific context
 * (i.e., an Answer may not be displayed if it is redundant based
 * on the answer given to a previously asked question).
 * <p>
 * An Answer may have Conclusions associated with it.  The Conclusions
 * represent Answers to other Questions that may be inferred based
 * on the current Answer.  The Conclusions themsleves have Conditions
 * on them which determine whether or not the Conclusion applies
 * based on previously answered Questions.
 * <p>
 * An Answer has Conditional Titles to allow different titles to be
 * displayed based on User Set, Country or based on any other Question
 * in the Online Assistant.
 *
 * @see Question
 * @see Conditions
 * @see AnswerConcl
 *
 * @version 1.27
 * @author  Mark DeBiase
**/
public class Answer extends ConditionalObject {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  int     answerInd_    = 0;
  int     questionInd_  = 0;      
  Vector  titles_       = null;
  Vector  conclusions_  = null;

  boolean draft_           = false;   // is this Answer object in draft?
  int     activeAnswerInd_ = 0;       // for draft object, the active Answer
                                      // (will be 0 if this is a new Answer)
  int     draftAnswerInd_  = 0;       // used to save the draft answer indicator
                                      // when publishing

  /**
   * New Answer object.
  **/
  public Answer() {
    super();
    updateRecStatus(REC_INSERT);
  }

  /**
   * Answer object read from the database.
  **/
  Answer(int answerInd) {
    super();
    answerInd_ = answerInd;
    updateRecStatus(REC_NOCHANGE);
  }

  /**
   * Set the draft status of this Answer.
  **/
  void setDraft(boolean draft) {
    draft_ = draft;
  }

  /**
   * Returns true if this is a draft Answer.
  **/
  public boolean isDraft() {
    return draft_;
  }

  /**
   * Returns the active (published) Answer indicator.
   * If this Answer object represents a draft Answer, this method will
   * return the indicator of its corresponding active Answer.
   * If this Answer object is not a draft Answer, or it is a new draft
   * Answer which has never been published, this method will return 0.
  **/
  public int getActiveAnswerInd() {
    return activeAnswerInd_;
  }
  
  /**
   * Sets the active Answer indicator.
   * This method calls setDraft(true).
  **/
  void setActiveAnswerInd(int activeAnswerInd) {
    activeAnswerInd_ = activeAnswerInd;
    setDraft(true);
  }

  int getDraftAnswerInd() {
    return draftAnswerInd_;
  }

  void setDraftAnswerInd(int draftAnswerInd) {
    draftAnswerInd_ = draftAnswerInd;
  }

  /**
   * Assign database keys for this object and all objects contained therein.
   * @return true if database keys were assigned, otherwise false
  **/
  boolean assignDatabaseKeys() {
    boolean success;

    success = assignMyDatabaseKey();

    if (titles_ != null) {
      for (int i=0; i<titles_.size() && success; i++) {
        Title t = (Title) titles_.elementAt(i);
        if ( this.isDraft() ) {
          success = t.assignDatabaseKeyForAnswerDraft();
        } else {
          success = t.assignDatabaseKeyForAnswer();
        }
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  void assignDatabaseKeys(Connection conn) throws Exception {

    assignMyDatabaseKey(conn);

    if (titles_ != null) {
      for (int i=0; i<titles_.size(); i++) {
        Title t = (Title) titles_.elementAt(i);
        if ( this.isDraft() ) {
          t.assignDatabaseKeyForAnswerDraft(conn);
        } else {
          t.assignDatabaseKeyForAnswer(conn);
        }
      }
    }

    return;
  }
  /**
   * Assign a database key for this Answer.
   * @return true if a database key was assigned, otherwise false
  **/
  private boolean assignMyDatabaseKey() {
    boolean success = true;

    if (answerInd_ == 0 && valid()) {
      if ( isDraft() ) {
        LogSystem.log(1, "Getting database key for draft Answer");
        answerInd_ = Counter.getCounter(LogSystem.getInstance(), "DANSRIND");
      } else {
        LogSystem.log(1, "Getting database key for Answer");
        answerInd_ = Counter.getCounter(LogSystem.getInstance(), "ANSWERIND");
      }
      if (answerInd_ == 0) {
        LogSystem.log(2, "Failed to get database key for Answer");
        success = false;
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  private void assignMyDatabaseKey(Connection conn) throws Exception {

    if (answerInd_ == 0) {
      if ( isDraft() ) {
        answerInd_ = Counter.getCounter(conn, "DANSRIND");
      } else {
        answerInd_ = Counter.getCounter(conn, "ANSWERIND");
      }
      if (answerInd_ == 0) {
        throw new Exception("Failed to get database key for Answer");
      }
    }

    return;
  }

  /**
   * Returns true if any part of this object needs to be saved to the
   * database.
   * As a side effect of calling this method, the sort order on the
   * Conditional objects (Titles, AddtlInfos, DocLinks) is updated.
  **/
  public boolean objectModified() {
    boolean modified = super.modified();

    if (titles_ != null) {
      int sortOrder = 1;
      for (int i=0; i<titles_.size() && !modified; i++) {
        Title t = (Title) titles_.elementAt(i);
        if ( t.valid() ) { t.setSortOrder( sortOrder++ ); }
        modified = modified || t.modified();
      }
    }
    if (conclusions_ != null) {
      int sortOrder = 1;
      for (int i=0; i<conclusions_.size() && !modified; i++) {
        AnswerConcl c = (AnswerConcl) conclusions_.elementAt(i);
        // sortOrder does not matter for answer conclusions
        modified = modified || c.modified();
      }
    }

    return modified;
  }

  /**
   * Dummy function, needed for database compatability.
   * Always returns false.
  **/
  public boolean getArchived() {
    return false;
  }

  /**
   * Returns this object's database indicator.
  **/
  public int getInd() {
    return answerInd_;
  }

  /**
   * Sets this object's database indicator.
  **/
  void setInd(int answerInd) {
    answerInd_ = answerInd;
  }

  /**
   * Get the parent Quesiton's indicator for this Answer.
  **/
  int getParentInd() {
    return questionInd_;
  }

  /**
   * Set the parent Question's indicator for this Answer.
  **/
  void setParentInd(int questionInd) {
    questionInd_ = questionInd;
  }

  /**
   * Returns the list of conditional titles.
   * <p>
   * <b>Note:</b>The last Title in the Vector should have no conditions
   * on it (other than a document classification of external).  It will
   * be used as the Default Title.
   *
   * @return a Vector of Title objects or null if there are no titles.
  **/
  public Vector getTitles() {
    return titles_;
  }

  /**
   * Sets the list of conditional titles.
   * <p>
   * <b>Note:</b>The last Title in the Vector should have no conditions
   * on it (other than a document classification of external).  It will
   * be used as the Default Title.
   *
   * @param conditionalTitles  a Vector of Title objects
  **/
  public void setTitles(Vector titles) {
    titles_ = titles;

    if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) 
       docClass_ = ((Title)titles_.elementAt(titles_.size()-1)).getDocClass() ;

  }

  public TypeDocClassRec getDocClass() {

     if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) 
        return ( ((Title)titles_.elementAt(titles_.size()-1)).getDocClass() ) ;

    return docClass_;
  }


  /**
   * Returns the default title.
   * The default title is the last Title in the Vector of Title objects
   * (and should not have any conditions on it).
   * 
   * @return null if there are no titles associated with this object;
   *              otherwise the last Title object is returned.
  **/
  public Title getDefaultTitle() {
    if (titles_ != null && titles_.size() > 0) {
      return (Title) titles_.lastElement();
    } else {
      return null;
    }
  }

  /**
   * Returns the list of conclusions.
   * This method was modified to return an empty Vector instead of null
   * if there are no conclusions.
   * @return a Vector of AnswerConcl objects or an empty Vector if
   *         there are no conclusions.
  **/
  public Vector getConclusions() {
    if (conclusions_ == null) {
      conclusions_ = new Vector();
    }
    return conclusions_;
  }

  /**
   * Sets the list of conclusions.
   * @param answerConclusions  a Vector of AnswerConcl objects
  **/
  public void setConclusions(Vector answerConclusions) {
    conclusions_ = answerConclusions;
  }

  /**
   * Perform a shallow copy of this object.
   * Clones the vectors this object contains (titles, conclusions),
   * but not the objects in the vectors.
  **/
  public Object clone() {
    Answer cloned = (Answer) super.clone();

    cloned.titles_      = (Vector) titles_.clone();
    cloned.conclusions_ = (Vector) conclusions_.clone();

    return cloned;
  }

  /**
   * Returns true if this Answer object is equal to another Answer 
   * Answer object.  
   * Equality is based on the database keys of the two Answer objects.
   * Therefore, this method is only valid for objects that have valid
   * database indicators.
   * @exception IllegalStateException if the Answer object does not have
   *            a valid indicator (i.e., indicator is 0).
  **/
  public boolean equals(Object o) {
    if ( this.getInd() == 0 ) { 
      throw new IllegalStateException("equals(): Answer is new");
    }
    return ( o instanceof Answer && 
             ((Answer) o).getInd() == this.getInd() );
  }

  /**
   * Returns the hashCode for this Answer.
   * The hashCode for an Answer object is the database indicator so this 
   * method is only valid for objects that have valid indicators.
   * @exception IllegalStateException if the Answer object does not have
   *                                  a valid indicator.
  **/
  public int hashCode() {
    if (getInd() == 0) {
      throw new IllegalStateException("hashCode(): Answer is new");
    }
    return getInd();
  }

  // get all the questions that are used by this answer (questions used
  // either in the title conditions or the anser conclusions) and add
  // them to the specified HashList.
  /**
   * Returns a HashList containing the indicators of all the Questions
   * used by this Answer.
  **/
  public HashList getAllQuestions() {
    HashList questionList = new HashList();
    getAllQuestions(questionList);
    return questionList;
  }

  /**
   * Add the indicators of all the Questions that are used by this
   * Answer and add them to the specified HashList.
   * @param questionList  the HashList to add the Question indicators to
  **/
  public void getAllQuestions(HashList questionList) {

    // get questions used by the conditions on this answer
    super.getAllQuestions(questionList);

    // get questions used by the conditional titles
    if (titles_ != null) {
      for (int i=0; i<titles_.size(); i++) {
        Title t = (Title) titles_.elementAt(i);
        t.getAllQuestions(questionList);
      }
    }

    // get questions used by the conclusions
    if (conclusions_ != null) {
      for (int i=0; i<conclusions_.size(); i++) {
        AnswerConcl thisConclusion = (AnswerConcl) conclusions_.elementAt(i);
        int toQuestionInd = thisConclusion.getToQuestionInd();
        questionList.put( new Integer(toQuestionInd) );
      }
    }
  }

  /**
   * Returns a String representation of this object.
  **/
  public String toString() {
    if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) {
      return( titles_.elementAt( titles_.size()-1 ).toString() ) ;
    }
    return "Answer " + getInd();
  }

  /**
   * Reset values in this Answer and its sub-objects.
   * <p>
   * If this is a draft Answer, the recStatus will be set to REC_INSERT
   * (for creating a new draft of a published Answer).
   * If this is not a draft Answer, the recStatus will be set to either
   * REC_INSERT or REC_UPDATE based on whether the Answer is already published
   * (for publishing a draft answer).
  **/
  void resetNew() {

    //----------------------------------------------------------------------
    // Ok, we got here either from constructing a QuestionDraft object
    // from a Question object (creating a new draft from a published object)
    // or from constructing a Question object from a QuestionDraft object
    // (publishing a draft object).
    // In the first case, isDraft() will return true and we want to save the
    // current indicator as the active Answer indicator, reset the Answer
    // indicator to 0 and set the recStatus to insert for the new draft.
    // In the second case, we want to set the recStatus based on whether 
    // this Answer is already published -- this is determined by the
    // activeAnswerInd.  We also need to save the current draft indicator.
    //----------------------------------------------------------------------
    // MAD 7/16:
    // The modification state is now being tracked in the Conditions object
    // via the modified_ flag so that we don't delete/insert the Conditions
    // on an update unless it is necessary.  Because of this, we need to
    // call Conditions.setModified(true) if we are publishing an Answer
    // which already exists (it will have a status of REC_UPDATE).  This is
    // because existing Answers are not deleted on publish -- they are
    // just updated and we have to make sure the Conditions are updated
    // as well.  
    // There is also an origCountries_ Vector that was added to the 
    // ConditionalObject class which is used to prevent unnecessary updates
    // (also delete/insert) of the country list.  We shouldn't have to do
    // anything about this Vector because it is initialized to null which
    // will force an update of the country list.
    //----------------------------------------------------------------------

    if ( isDraft() ) {
      setActiveAnswerInd( getInd() );
      setInd(0);
         // Force the insert
        updateRecStatus(REC_NOCHANGE);
      updateRecStatus(REC_INSERT);
    } else {
      setDraftAnswerInd( getInd() );    // save current draft indicator
      setInd( getActiveAnswerInd() );   // set the indicator of the active obj
      if ( getInd() == 0 ) {

         // Force the insert
        updateRecStatus(REC_NOCHANGE);
        updateRecStatus(REC_INSERT);
      } else {
        updateRecStatus(REC_UPDATE);
        getConditions().setModified(true);  // see 7/16 comments above...
      }
    }

    // reset the sub-objects
    if (titles_ != null) {
      for (int i=0; i<titles_.size(); i++) {
        Title title = (Title) titles_.elementAt(i);
        title.resetNew();
      }
    }
    if (conclusions_ != null) {
      for (int i=0; i<conclusions_.size(); i++) {
        AnswerConcl concl = (AnswerConcl) conclusions_.elementAt(i);
        concl.resetNew();
      }
    }
  }

  /**
   * Set the recStatus of this object and 
   * all its contained objects to REC_NOCHANGE.
  **/
  void setRecNoChange() {

    // this object handled by superclass
    super.setRecNoChange();

    // contained objects:
    if (titles_ != null) {
      for (int i=0; i<titles_.size(); i++) {
        Title t = (Title) titles_.elementAt(i);
        t.setRecNoChange();
      }
    }
    if (conclusions_ != null) {
      for (int i=0; i<conclusions_.size(); i++) {
        AnswerConcl concl = (AnswerConcl) conclusions_.elementAt(i);
        concl.setRecNoChange();
      }
    }
  }

  /**
   * Update the TypeRec objects in the Answers's subobjects with 
   * the proper descriptions.
   * Called by readFromDatabase();
  **/
  void updateTypeRecData() {
    super.updateTypeRecData();

    // update sub-objects TypeRec data:
    if (titles_ != null) {
      for (int i=0; i<titles_.size(); i++) {
        Title title = (Title) titles_.elementAt(i);
        title.updateTypeRecData();
      }
    }
    if (conclusions_ != null) {
      for (int i=0; i<conclusions_.size(); i++) {
        AnswerConcl concl = (AnswerConcl) conclusions_.elementAt(i);
        concl.updateTypeRecData();
      }
    }

    return;
  }
}


