
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.type.TypeDocClassRec;
import java.sql.SQLException;
import java.util.Vector;

/**
 * Represents a single conclusion to an Answer.
 * A Conclusion is an Answer to another Question that can be 
 * inferred from the Answer to the current Question.
 * For example, a Conclusion on the Answer "2645-3EA"
 * to the Question "What model Thinkpad do you have?" might
 * be the Answer for "13.3 in. active matrix" to the Question
 * "What type of display do you have?".
 *
 * @version 1.14
 * @author  Mark DeBiase
**/

public class AnswerConcl extends ConditionalObject {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  private int fromAnswerInd_  = 0;
  private int toAnswerInd_    = 0;
  private int toQuestionInd_  = 0;

  /**
   * New AnswerConcl using Question/Answer objects.
   * @param toQuestion  the Question the Conclusion infers.
   * @param toAnswer    the Answer the Conclusion infers.
   *                    toAnswer must be a published Answer.
   * @exception IllegalArgumentException if either the Question's indicator
   *                                     or the Answer's indicator is 0.
  **/
  public AnswerConcl(Question toQuestion, Answer toAnswer) {
    this( toQuestion.getInd(), toAnswer.getInd() );
  }

  /**
   * New AnswerConcl using database indicators.
   * @param toQuestionInd  the database indicator of the Question
   *                       the Conclusion infers.
   * @param toAnswerInd    the database indicator of the Answer
   *                       the Conclusion infers.
   *                       toAnswerInd must be the indicator of 
   *                       a published Answer.
   * @exception IllegalArgumentException if toQuestionInd or toAnswerInd is 0.
  **/
  public AnswerConcl(int toQuestionInd, int toAnswerInd) {
    super();       // recStatus will be set to REC_INSERT by superclass
    fromAnswerInd_ = 0;
    toAnswerInd_   = toAnswerInd;
    toQuestionInd_ = toQuestionInd;

    if (toQuestionInd_ == 0 || toAnswerInd_ == 0) {
      throw new IllegalArgumentException("toQuestionInd or toAnswerInd is 0");
    }
    updateRecStatus(REC_INSERT);
  }

  /**
   * Object read from the database.
  **/
  AnswerConcl(int fromAnswerInd, int toQuestionInd, int toAnswerInd) {
    fromAnswerInd_ = fromAnswerInd;
    toAnswerInd_   = toAnswerInd;
    toQuestionInd_ = toQuestionInd;
    updateRecStatus(REC_NOCHANGE);
  }

  /**
   * Return the "from" answer indicator.
   * This is the indicator of the answer the conclusion is attached to.
  **/
  public int getFromAnswerInd() {
    return fromAnswerInd_;
  }

  /**
   * Set the "from" answer indicator.
  **/
  void setFromAnswerInd(int fromAnswerInd) {
    fromAnswerInd_ = fromAnswerInd;
  }

  /**
   * Return the "to" answer indicator.
   * This is the answer inferred by the conclusion.
  **/
  public int getToAnswerInd() {
    return toAnswerInd_;
  }

  /**
   * Return the "to" question indicator.
   * This is the question inferred by the conclusion.
  **/
  public int getToQuestionInd() {
    return toQuestionInd_;
  }

  /**
   * Perform a shallow copy of this object.
  **/
  public Object clone() {
    return super.clone();
  }

  /**
   * Return a String representation of this object.
   * Returns "AnswerConcl: [fromAnswerInd] -&gt; [toQuestionInd]:[toAnswerInd]"
  **/
  public String toString() {
    return "AnswerConcl: " + fromAnswerInd_ + " -> " + 
           toQuestionInd_ + ":" + toAnswerInd_;
  }

  /**
   * Reset values in this object.
   * Sets recStatus to REC_INSERT.
  **/
  void resetNew() {
    setFromAnswerInd(0);
    updateRecStatus(REC_INSERT);
  }

  /**
   * This method will throw an IllegalStateException.
   * The getInd() method is required by the ConditionalObject superclass.
   * However, AnswerConcl objects do not have a single value for an 
   * indicator -- the database key is comprised of the fromAnswerInd and
   * toAnswerInd values.  Therefore an IllegalStateException will be thrown
   * if this method is called.
   * 
   * @exception IllegalStateException if this method is called.
  **/
  public int getInd() {
    throw new IllegalStateException("getInd() is invalid for this object");
  }

  /**
   * Find the AnswerConcl object with the specified fromAnswerInd and
   * toAnswerInd in a Vector of AnswerConcl objects.
   * @param  fromAnswerInd  the AnswerConcl fromAnswerInd to be found
   * @param  toAnswerInd    the AnswerConcl toAnswerInd to be found
   * @param  v              the Vector of AnswerConcl objects
   * @return the AnswerConcl object with the specified fromAnswerInd/toAnswerInd
   *         or null if it was not found in the Vector.
  **/
  public static AnswerConcl findAnswerConclInVector(int fromAnswerInd, 
                                                    int toAnswerInd,
                                                    Vector v)
  {
    AnswerConcl foundObject = null;

    if (v != null) {
      for (int i=0; foundObject == null && i < v.size(); i++) {
        AnswerConcl tmp = (AnswerConcl) v.elementAt(i);
        if ( tmp.getFromAnswerInd() == fromAnswerInd &&
             tmp.getToAnswerInd() == toAnswerInd ) 
        {
          foundObject = tmp;
        }
      }
    }

    return foundObject;
  }
}

