
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;



/**
 * Reader for Online Assistant Answer Conclusions.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.9
 * @author VishnuKumar.E
**/ 
class AnswerConclReader {

  public static final String ACTIVE_SCHEMA = "oa";
  public static final String DRAFT_SCHEMA  = "oadraft";

  /**
   * Read the Conclusions for all Answers to the specified Question
   * (oa schema).
  **/
  static void readAllAnswerConcl(Connection conn, Question q)
              throws SQLException
  {
    readAllAnswerConcl(conn, q, ACTIVE_SCHEMA);
  }

  /**
   * Read the Conclusions for all Answers to the specified Question
   * (oadraft schema).
  **/
  static void readAllDraftAnswerConcl(Connection conn, Question q) 
              throws SQLException
  {
    readAllAnswerConcl(conn, q, DRAFT_SCHEMA);
  }

  /**
   * Read the Conclusions for all Answers to the specified Question
   * (common function).
   * @param conn  the database Connection object
   * @param q     the Question whose Answer Conclusions are to be read
   * @return true if any AnswerConcl objects were read;
   *         false if no AnswerConcl objects were read.
  **/
  private static void readAllAnswerConcl(Connection conn, Question q, 
                                         String schema) 
             throws SQLException
  {
  
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // Select all Answer Conclusions for the Question
    // NOTE: the answer table is used twice as a1 and a2:
    //       a1 is used to select all conclusions for the questionInd
    //       a2 is used to select the questionInd for the toAnswerInd value
    // NOTE: it is not a typo that the oa.answer a2 table is used below 
    //       without the schema parm -- this is because answer conclusions
    //       can only be based on published answers.
    rs = stmt.executeQuery( 
                "SELECT c.fromAnswerInd, c.toAnswerInd, a2.questionInd, " +
                       "c.docClassInd, c.dbUser, c.changedTime " +
                "FROM " + schema + ".answerConcl c, " +
                          schema + ".answer a1, " +
                       "oa.answer a2 " +
                "WHERE  c.fromAnswerInd=a1.answerInd" +
                " AND   a1.questionInd=" + q.getInd() +
                " AND   c.toAnswerInd=a2.answerInd " +
                "ORDER BY c.fromAnswerInd, c.toAnswerInd " +
                "FOR FETCH ONLY"
         );

    // values read from the database
    int fromAnswerInd = 0,
        toAnswerInd   = 0;

    // values read from database on previous iteration of the while loop
    int previousFromAnswerInd = -99;

    Vector conclusionVec = null;   // vector of conclusions
  
    // read each AnswerConcl object selected from database
    while( rs.next() ) {
 
      fromAnswerInd = rs.getInt(1);
      toAnswerInd   = rs.getInt(2);

      // when we get a new fromAnswerInd, set the conclusionVec in
      // the appropriate Answer object.
      if (fromAnswerInd != previousFromAnswerInd) {
        if (conclusionVec != null) {    // if not first time through
          Answer ans = q.getAnswerByInd(previousFromAnswerInd);
          ans.setConclusions( conclusionVec );
        }
        conclusionVec = new Vector();
        previousFromAnswerInd = fromAnswerInd;
      }

      int questionInd = rs.getInt(3);
      AnswerConcl co = new AnswerConcl(fromAnswerInd, questionInd, toAnswerInd);
      co.setDocClass( new TypeDocClassRec( rs.getShort(4), "" ) );
      co.setDbUser( rs.getString(5).trim() );  
      co.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(6) ) );

      co.updateRecStatus(PersistentRec.REC_NOCHANGE);
      conclusionVec.addElement(co);
    }  

    rs.close();
    stmt.close();

    // set the last set of conclusions into the appropriate Answer object
    // conclusionVec will be null if there were no conclusions
    if (conclusionVec != null) {
      Answer ans = q.getAnswerByInd(fromAnswerInd);
      ans.setConclusions(conclusionVec);
    }

    // read the Conditions/Countries for all AnswerConcls
    // -- only if we found Conclusions
    if (conclusionVec != null) {
      ConditionReader.readAllAnswerConclConditions(conn, q, schema);
      CountryReader.readAllAnswerConclCountries(conn, q, schema);
    }

  }

//=============================================================================

  /**
   * Read the Conclusions for the specified Answer (oa schema).
   * Note: this method is no longer used -- readAllAnswerConcl() is now used.
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readConclForAnswer(Connection conn, Answer a) throws SQLException
  {
    readConclForAnswer(conn, a, ACTIVE_SCHEMA);
  }
------------------------- END COMMENTED OUT ----------------------------- */

  /**
   * Read the Conclusions for the specified Answer (oadraft schema).
   * Note: this method is no longer used -- readAllAnswerConcl() is now used.
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readConclForAnswerDraft(Connection conn, Answer a) 
              throws SQLException
  {
    readConclForAnswer(conn, a, DRAFT_SCHEMA);
  }
------------------------- END COMMENTED OUT ----------------------------- */

  /**
   * Read the Conclusions for the specified Answer (common function).
   * Note: this method is no longer used -- readAllAnswerConcl() is now used.
   * @param conn  the database Connection object
   * @param a     the Answer whose Conclusions are to be read
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  private static void readConclForAnswer(Connection conn, Answer a, 
                                         String schema) 
             throws SQLException
  {
  
    Statement stmt1 = conn.createStatement();
    ResultSet rs1;

    // NOTE: it is not a typo that the oa.answer table is used below
    //       instead of using the schema parm -- this is because 
    //       answer conclusions can only be based on published answers.
    rs1 = stmt1.executeQuery( 
          "SELECT " +
             "c.fromAnswerInd, c.toAnswerInd, a.questionInd, " +
             "c.docClassInd, c.dbUser, c.changedTime " +
          "FROM " + schema + ".answerConcl c, oa.answer a " +
          "WHERE fromAnswerInd=" + a.getInd() + " " +
          "AND c.toAnswerInd=a.answerInd " +
          "FOR FETCH ONLY"
    );

  
    // read each AnswerConcl object
    Vector v = new Vector();
    while( rs1.next() ) {
 
      int fromAnswerInd = rs1.getInt(1);
      int toAnswerInd = rs1.getInt(2);
      int questionInd = rs1.getInt(3);

      AnswerConcl co = new AnswerConcl(fromAnswerInd, questionInd, toAnswerInd);
      co.setDocClass( new TypeDocClassRec( rs1.getShort(4), "" ) );
      co.setDbUser( rs1.getString(5).trim() );  
      co.setChangedTime( DatabaseUtil.timestampFromDB( rs1.getString(6) ) );

      // read the conditions/countries for the conclusion
      readConditionsForAnswerConcl( conn, co, schema );
      readCountriesForAnswerConcl( conn, co, schema );

      co.updateRecStatus(PersistentRec.REC_NOCHANGE);
      v.addElement(co);
    }  

    a.setConclusions(v);
  
    rs1.close();
    stmt1.close();
  }
------------------------- END COMMENTED OUT ----------------------------- */

//=============================================================================

  /**
   * Read and populate the Conditions for the specified Answer Conclusion.
   * Note: this method is no longer used -- 
   * ConditionReader.readAllAnswerConclConditions() is used instead.
   * @param conn  the database Connection object
   * @param a     the AnswerConl objects whose Conditions are to be read
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  private static void readConditionsForAnswerConcl(Connection conn, 
                                             AnswerConcl concl, String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;
    Vector    v = new Vector(1, 3);

    // we don't select dbUser/changedTime since they are the same
    // as the parent object.
    String sql = "SELECT questionInd, answerInd, operators" +
                 " FROM " + schema + ".answerConclCond" +
                 " WHERE fromAnswerInd=" + concl.getFromAnswerInd() +
                 " AND toAnswerInd= " + concl.getToAnswerInd() +
                 " ORDER BY condSortOrder" +
                 " FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    while ( rs.next() ) {
      int    questionInd = rs.getInt( 1 );
      int    answerInd   = rs.getInt( 2 );
      String operators   = rs.getString( 3 );
      
      ConditionTerm term = new ConditionTerm(questionInd, answerInd, operators);
      v.addElement( term );
    }

    Conditions cond = new Conditions( v, concl );

    rs.close();
    stmt.close();

    concl.setConditions( cond );
  }
------------------------- END COMMENTED OUT ----------------------------- */

//=============================================================================

  /**
   * Read and populate the Countries for the specified Answer Conclusion.
   * Note: this method is no longer used -- 
   * CountryReader.readAllAnswerConclCountries() is used instead.
   * @param conn  the database Connection object
   * @param a     the AnswerConl objects whose Conditions are to be read
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  private static void readCountriesForAnswerConcl(Connection conn, 
                                              AnswerConcl concl, String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;
    Vector    v = new Vector();

    // we don't select dbUser/changedTime - they are the same
    // as the parent object.
    String sql = "SELECT countryInd" +
                 " FROM " + schema + ".answerConclCountry" + 
                 " WHERE fromAnswerInd=" + concl.getFromAnswerInd() + 
                 " AND toAnswerInd=" + concl.getToAnswerInd() + 
                 " FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    while ( rs.next() ) {
      v.addElement( CountryReader.makeCountryCodeRec( rs.getInt(1) ) );
    }

    rs.close();
    stmt.close();

    concl.setCountryList( v );
  }
------------------------- END COMMENTED OUT ----------------------------- */
}
