
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.util.Vector;

/**
 * @version 1.5
 * @author  Mark DeBiase
**/
class AnswerConclWriter {

  private static final String SCHEMA = "oa";

  /**
   * Returns the SQL String to write the AnswerConcl object to the database.
  **/
  static String getSQL(AnswerConcl a) {
    String sql;

    switch ( a.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(a, SCHEMA);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(a, SCHEMA);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(a, SCHEMA);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(AnswerConcl a, String schema) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // insert the conclusion itself
    sb.append(
      "INSERT INTO " + schema + ".answerConcl " +
        "(fromAnswerInd, toAnswerInd, docClassInd, dbUser, changedTime) " +
      "VALUES ("
    );
    sb.append( a.getFromAnswerInd() )                    .append(',');
    sb.append( a.getToAnswerInd() )                      .append(',');
    sb.append( a.getDocClass().getInd() )                .append(',');
    sb.append( DatabaseUtil.stringToDB( a.getDbUser() ) ).append(',');
    sb.append( "CURRENT TIMESTAMP)")
      .append( SqlRunner.END_DELIM );

    // insert any conditions/countries
    sb.append( getConditionsInsertSQL(a, schema) );
    sb.append( getCountriesInsertSQL(a, schema) );

    return sb.toString();
  }

  static String getUpdateSQL(AnswerConcl a, String schema) {
    // Note: oa.answer.questionInd is not updated

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);
    // the only column in the answerconcl table that can be updated
    // is the docclassind -- dbUser & changedTime are just along 
    // for the ride.

    sb.append("UPDATE " + schema + ".answerConcl SET");
    sb.append( " docClassInd=" )
      .append( a.getDocClass().getInd() );
    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( a.getDbUser() ) );
    sb.append( ",changedTime=CURRENT TIMESTAMP");
    sb.append(" WHERE fromAnswerInd=").append( a.getFromAnswerInd() )
      .append("   AND toAnswerInd=").append( a.getToAnswerInd() )
      .append(SqlRunner.END_DELIM);

    // update any conditions/countries
    sb.append( getConditionsUpdateSQL(a, schema) );
    sb.append( getCountriesUpdateSQL(a, schema) );

    return sb.toString();
  }

  static String getDeleteSQL(AnswerConcl a, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete any conditions/countries
    sb.append( getConditionsDeleteSQL(a, schema) );
    sb.append( getCountriesDeleteSQL(a, schema) );

    sb.append( "DELETE FROM " + schema + ".answerConcl")
      .append( " WHERE fromAnswerInd=").append( a.getFromAnswerInd() )
      .append( " AND toAnswerInd=").append( a.getToAnswerInd() )
      .append( SqlRunner.END_DELIM );

    return sb.toString();
  }

  private static String getConditionsInsertSQL(AnswerConcl a, String schema) {
    String insertSql = "";

    Conditions cond = a.getConditions();
    if ( cond != null && cond.size() > 0 ) {
      StringBuffer sb = new StringBuffer( DatabaseUtil.DEFAULT_SB_SIZE );

      Vector v = cond.getTerms();
      for (int i=0; i<v.size(); i++) {
        ConditionTerm term = (ConditionTerm) v.elementAt(i);

        //*****************************************************************
        // Note: this code assumes the following column order on the 
        // condition table:
        //   fromAnswerInd, toAnswerInd, condSortOrder, questionInd,
        //   condAnswerInd, operators, dbUser, changedTime
        //*****************************************************************
        sb.append("INSERT INTO " + schema + ".answerConclCond VALUES (");
        sb.append( a.getFromAnswerInd()                           ).append(',');
        sb.append( a.getToAnswerInd()                             ).append(',');
        sb.append( (i + 1)                                        ).append(',');
        sb.append( term.getQuestionInd()                          ).append(',');
        // if answerInd is 0, then term applies to all answers - insert null
        if (term.getAnswerInd() == 0) {
          sb.append( "null,");
        } else {
          sb.append( term.getAnswerInd()                          ).append(',');
        }
        sb.append( DatabaseUtil.stringToDB( term.getOperators() ) ).append(',');
        sb.append( DatabaseUtil.stringToDB( a.getDbUser() )       ).append(',');
        sb.append("CURRENT TIMESTAMP)")
          .append(SqlRunner.END_DELIM);

      }
      insertSql = sb.toString();
    }

    return insertSql;
  }

  private static String getConditionsUpdateSQL(AnswerConcl a, String schema) {

    // only update the conditions if they have been modified
    if (a.getConditions().isModified()) {
      return getConditionsDeleteSQL(a, schema) + 
             getConditionsInsertSQL(a, schema);
    } else {
      return "";
    }
  }

  private static String getConditionsDeleteSQL(AnswerConcl a, String schema) {
    return "DELETE FROM " + schema + ".answerConclCond" +
           " WHERE fromAnswerInd=" + a.getFromAnswerInd() +
           " AND toAnswerInd=" + a.getToAnswerInd() + SqlRunner.END_DELIM;
  }

  private static String getCountriesInsertSQL(AnswerConcl a, String schema) {
    String insertSql = "";
    Vector countryList = a.getCountryList();

    if (countryList != null && countryList.size() > 0) {
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);
      TypeCountryCodeRec country;

      for (int i=0; i<countryList.size(); i++) {
        country = (TypeCountryCodeRec) countryList.elementAt(i);

        // insert columns = fromAnswerInd, toAnswerInd, countryInd,
        //                  dbUser, changedTime
        sb.append( "INSERT INTO " + schema + ".answerConclCountry VALUES (" )
          .append( a.getFromAnswerInd()                   ).append(',')
          .append( a.getToAnswerInd()                     ).append(',')
          .append( country.getInd()                       ).append(',')
          .append( DatabaseUtil.stringToDB(a.getDbUser()) ).append(',')
          .append( "CURRENT TIMESTAMP)")
          .append(SqlRunner.END_DELIM);
      }
      insertSql = sb.toString();
    }

    return insertSql;
  }

  private static String getCountriesUpdateSQL(AnswerConcl a, String schema) {
    // only update the countries if they have changed
    if ( !VectorUtil.vectorsEqual(a.getCountryList(), 
                                  a.getOriginalCountryList()) )
    {
      return getCountriesDeleteSQL(a, schema) + 
             getCountriesInsertSQL(a, schema);
    } else {
      return "";
    }
  }

  private static String getCountriesDeleteSQL(AnswerConcl a, String schema) {
    return "DELETE FROM " + schema + ".answerConclCountry" +
           " WHERE fromAnswerInd=" + a.getFromAnswerInd() +
           " AND toAnswerInd=" + a.getToAnswerInd() + SqlRunner.END_DELIM;
  }

  /**
   * Return SQL to delete all the Conclusions for ALL Answers to a Question.
  **/
  static String getDeleteAllForQuestionSQL(int questionInd) {
    return getDeleteAllForQuestionSQL(questionInd, SCHEMA);
  }

  static String getDeleteAllForQuestionSQL(int questionInd, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from " + schema + ".answerconclcond where fromanswerind in " +
        "(select distinct answerind " + 
         "from " + schema + ".answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".answerconclcountry where fromanswerind in  " +
        "(select distinct answerind " + 
         "from " + schema + ".answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".answerconcl where fromanswerind in  " +
        "(select distinct answerind " + 
         "from " + schema + ".answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the Conclusions for a single Answer.
  **/
  static String getDeleteAllForAnswerSQL(int answerInd) {
    return getDeleteAllForAnswerSQL(answerInd, SCHEMA);
  }

  static String getDeleteAllForAnswerSQL(int answerInd, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from " + schema + ".answerconclcond where fromanswerind="
    ).append( answerInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".answerconclcountry where fromanswerind="
    ).append( answerInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".answerconcl where fromanswerind="
    ).append( answerInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
