
package com.ibm.nzna.projects.common.quest.oa;

import java.sql.*;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Reader for Online Assistant Conditions.
 * This class is used by the OA stored procedures.
 *
 * @version 1.8
 * @author  Mark DeBiase
**/
class ConditionReader {

  /**
   * Read Conditions from the database.
   * @param conn       database Connection object
   * @param tableName  the conditions table to read
   * @param indName    the name of the indicator column in the table
   * @param co         the ConditionalObject the Conditions belong to
   * @return a Conditions object containing the data from the specified table
  **/
  static Conditions readConditions(Connection conn, String tableName,
                                   String indName, ConditionalObject co)
                    throws SQLException
  {

    Statement stmt = conn.createStatement();
    ResultSet rs;
    Vector    v = new Vector(1, 3);

    // we don't select dbUser/changedTime since they are the same
    // as the parent object.
    String sql = "SELECT questionInd, answerInd, operators" +
                 " FROM " + tableName +
                 " WHERE " + indName + "=" + co.getInd() +
                 " ORDER BY condSortOrder" +
                 " FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    while ( rs.next() ) {
      int    questionInd = rs.getInt( 1 );
      int    answerInd   = rs.getInt( 2 );
      String operators   = rs.getString( 3 );
      
      ConditionTerm term = new ConditionTerm(questionInd, answerInd, operators);
      v.addElement( term );
    }

    Conditions cond = new Conditions( v, co );

    rs.close();
    stmt.close();

    return cond;
  }

  /**
   * Read the Action Title conditions from the database for all
   * Action Titles associated with the Action and populate the
   * Conditions in each Action Title object.
   * If the Action Titles have been read for all languages, the
   * multiple parms should be set to true; otherwise it should
   * be set to false.  The multiple parm indicates that there can
   * be multiple occurences of a given Action Title indicator in the
   * Vector of Titles (one for each langauge read).
   * @param conn    database Connection object
   * @param a       the Action object whose Title Conditions are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 action title indicator in the action title vector.
  **/
  static void readAllActionTitleConditions(Connection conn, Action a, 
                                           String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple titles
    // with the same title indicator but different language indicators.
    String sql =
         "SELECT DISTINCT atc.actionTitleInd, " + 
                "atc.questionInd, atc.answerInd, atc.operators, " +
                "atc.condSortOrder " +    // req'd for query, but not retrieved
         "FROM " + schema + ".actionTitleCond atc, " + 
                   schema + ".actionTitle at " +
         "WHERE  atc.actionTitleInd = at.actionTitleInd " +
         " AND   at.actionInd=" + a.getInd() + " " + 
         "ORDER  BY atc.actionTitleInd, atc.condSortOrder " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int actionTitleInd = 0,           // action title that the term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousActionTitleInd = -99;    // last actionTitleInd processed
    Vector conditionTermVec = null;      // Vector of ConditionTerm objects

    while ( rs.next() ) {
      actionTitleInd    = rs.getInt(1);
      termQuestionInd   = rs.getInt(2);
      termAnswerInd     = rs.getInt(3);
      termOperators     = rs.getString(4);

      // when we get to a new actionTitleInd, put the current Conditions
      // into the appropriate Action Title object.
      if (actionTitleInd != previousActionTitleInd) {
        if (conditionTermVec != null) {      // if not first time through
          Title title = (Title) ConditionalObject.findObjectInVector(
                                   previousActionTitleInd, a.getTitles() );
          Conditions cond = new Conditions( conditionTermVec, title );
          title.setConditions(cond);

          // handle multiple occurences of the title...
          if (multiple) {
            for (int i=2; title != null; i++) {
              title = (Title) ConditionalObject.findObjectInVector(
                                 previousActionTitleInd, a.getTitles(), i );
              cond = new Conditions( conditionTermVec, title );
              if (title != null) { title.setConditions(cond); }
            }
          }
        }

        conditionTermVec = new Vector();
        previousActionTitleInd = actionTitleInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Action Title object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      Title title = (Title) ConditionalObject.findObjectInVector(
                                 actionTitleInd, a.getTitles() );
      Conditions cond = new Conditions( conditionTermVec, title );
      title.setConditions(cond);

      // handle multiple occurences of the title...
      if (multiple) {
        for (int i=2; title != null; i++) {
          title = (Title) ConditionalObject.findObjectInVector(
                               actionTitleInd, a.getTitles(), i );
          cond = new Conditions( conditionTermVec, title );
          if (title != null) { title.setConditions(cond); }
        }
      }
    }

  }

  /**
   * Read the Action AddtlInfo conditions from the database for all
   * Action AddtlInfo associated with the Action and populate the
   * Conditions in each Action AddtlInfo object.
   * If the Action AddtlInfo have been read for all languages, the
   * multiple parms should be set to true; otherwise it should
   * be set to false.  The multiple parm indicates that there can
   * be multiple occurences of a given Action AddtlInfo indicator in the
   * Vector of AddtlInfo (one for each langauge read).
   * @param conn    database Connection object
   * @param a       the Action object whose AddtlInfo Conditions 
   *                are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 action addtlInfo indicator in the addtlInfo vector.
  **/
  static void readAllActionAddtlInfoConditions(Connection conn, Action a, 
                                              String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple addtlinfo
    // with the same addtlinfo indicator but different language indicators.
    String sql =
         "SELECT DISTINCT aic.actionAddInfoInd, " + 
                "aic.questionInd, aic.answerInd, aic.operators, " +
                "aic.condSortOrder " +    // req'd for query, but not retrieved
         "FROM " + schema + ".actionInfoCond aic, " + 
                   schema + ".actionAddInfo ai " +
         "WHERE  aic.actionAddInfoInd = ai.actionAddInfoInd " +
         " AND   ai.actionInd=" + a.getInd() + " " + 
         "ORDER  BY aic.actionAddInfoInd, aic.condSortOrder " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int actionInfoInd = 0,            // action info that the term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousActionInfoInd = -99;  // last actionInfoInd processed
    Vector conditionTermVec = null;   // Vector of ConditionTerm objects

    while ( rs.next() ) {
      actionInfoInd     = rs.getInt(1);
      termQuestionInd   = rs.getInt(2);
      termAnswerInd     = rs.getInt(3);
      termOperators     = rs.getString(4);

      // when we get to a new actionInfoInd, put the current Conditions
      // into the appropriate Action AddtlInfo object.
      if (actionInfoInd != previousActionInfoInd) {
        if (conditionTermVec != null) {      // if not first time through
          AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                                  previousActionInfoInd, 
                                                  a.getAdditionalInfo() );
          Conditions cond = new Conditions( conditionTermVec, info );
          info.setConditions(cond);

          // handle multiple occurences of the addtlInfo...
          if (multiple) {
            for (int i=2; info != null; i++) {
              info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                            previousActionInfoInd, 
                                            a.getAdditionalInfo(), i );
              cond = new Conditions( conditionTermVec, info );
              if (info != null) { info.setConditions(cond); }
            }
          }
        }

        conditionTermVec = new Vector();
        previousActionInfoInd = actionInfoInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Action AdttlInfo object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                              actionInfoInd, 
                                              a.getAdditionalInfo() );
      Conditions cond = new Conditions( conditionTermVec, info );
      info.setConditions(cond);

      // handle multiple occurences of the addtlInfo...
      if (multiple) {
        for (int i=2; info != null; i++) {
          info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                        actionInfoInd, 
                                        a.getAdditionalInfo(), i );
          cond = new Conditions( conditionTermVec, info );
          if (info != null) { info.setConditions(cond); }
        }
      }
    }

  }

  /**
   * Read the Action DocLink conditions from the database for all
   * Action DocLinks associated with the Action and populate the
   * Conditions in each Action DocLink object.
   * @param conn    database Connection object
   * @param a       the Action object whose DocLink Conditions are to be read
   * @param schema  either "oa" or "oadraft"
  **/
  static void readAllActionDocLinkConditions(Connection conn, Action a, 
                                             String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql =
         "SELECT alc.actionLinkInd, " + 
                "alc.questionInd, alc.answerInd, alc.operators, " +
                "alc.condSortOrder " +
         "FROM " + schema + ".actionLinkCond alc, " + 
                   schema + ".actionLink al " +
         "WHERE  alc.actionLinkInd = al.actionLinkInd " +
         " AND   al.actionInd=" + a.getInd() + " " + 
         "ORDER  BY alc.actionLinkInd, alc.condSortOrder " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int actionLinkInd = 0,            // action link that the term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousActionLinkInd = -99;     // last actionLinkInd processed
    Vector conditionTermVec = null;      // Vector of ConditionTerm objects

    while ( rs.next() ) {
      actionLinkInd     = rs.getInt(1);
      termQuestionInd   = rs.getInt(2);
      termAnswerInd     = rs.getInt(3);
      termOperators     = rs.getString(4);

      // when we get to a new actionLinkInd, put the current Conditions
      // into the appropriate Action DocLink object.
      if (actionLinkInd != previousActionLinkInd) {
        if (conditionTermVec != null) {      // if not first time through
          DocLink dl = (DocLink) ConditionalObject.findObjectInVector(
                                   previousActionLinkInd, a.getDocLinks() );
          Conditions cond = new Conditions( conditionTermVec, dl );
          dl.setConditions(cond);
        }

        conditionTermVec = new Vector();
        previousActionLinkInd = actionLinkInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Action DocLink object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      DocLink dl = (DocLink) ConditionalObject.findObjectInVector(
                                 actionLinkInd, a.getDocLinks() );
      Conditions cond = new Conditions( conditionTermVec, dl );
      dl.setConditions(cond);
    }

  }

  /**
   * Read the Question Title conditions from the database for all
   * Question Titles associated with the Question and populate the
   * Conditions in each Question Title object.
   * If the Question Titles have been read for all languages, the
   * multiple parms should be set to true; otherwise it should
   * be set to false.  The multiple parm indicates that there can
   * be multiple occurences of a given Question Title indicator in the
   * Vector of Titles (one for each langauge read).
   * @param conn    database Connection object
   * @param q       the Question object whose Title Conditions are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 question title indicator in the question title vector.
  **/
  static void readAllQuestionTitleConditions(Connection conn, Question q, 
                                             String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple titles
    // with the same title indicator but different language indicators.
    String sql =
         "SELECT DISTINCT qtc.questionTitleInd, " + 
                "qtc.questionInd, qtc.answerInd, qtc.operators, " +
                "qtc.condSortOrder " +    // req'd for query, but not retrieved
         "FROM " + schema + ".questionTitleCond qtc, " + 
                   schema + ".questionTitle qt " +
         "WHERE  qtc.questionTitleInd = qt.questionTitleInd " +
         " AND   qt.questionInd=" + q.getInd() + " " + 
         "ORDER  BY qtc.questionTitleInd, qtc.condSortOrder " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int questionTitleInd = 0,         // question title that the term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousQuestionTitleInd = -99;  // last questionTitleInd processed
    Vector conditionTermVec = null;      // Vector of ConditionTerm objects
                                         // for the current questionTitleInd

    while ( rs.next() ) {
      questionTitleInd  = rs.getInt(1);
      termQuestionInd   = rs.getInt(2);
      termAnswerInd     = rs.getInt(3);
      termOperators     = rs.getString(4);

      // when we get to a new questionTitleInd, put the current Conditions
      // into the appropriate Question Title object.
      if (questionTitleInd != previousQuestionTitleInd) {
        if (conditionTermVec != null) {      // if not first time through
          Title title = (Title) ConditionalObject.findObjectInVector(
                                   previousQuestionTitleInd, q.getTitles() );
          Conditions cond = new Conditions( conditionTermVec, title );
          title.setConditions(cond);

          // handle multiple occurences of the title...
          if (multiple) {
            for (int i=2; title != null; i++) {
              title = (Title) ConditionalObject.findObjectInVector(
                                 previousQuestionTitleInd, q.getTitles(), i );
              cond = new Conditions( conditionTermVec, title );
              if (title != null) { title.setConditions(cond); }
            }
          }
        }

        conditionTermVec = new Vector();
        previousQuestionTitleInd = questionTitleInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Question Title object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      Title title = (Title) ConditionalObject.findObjectInVector(
                                 questionTitleInd, q.getTitles() );
      Conditions cond = new Conditions( conditionTermVec, title );
      title.setConditions(cond);

      // handle multiple occurences of the title...
      if (multiple) {
        for (int i=2; title != null; i++) {
          title = (Title) ConditionalObject.findObjectInVector(
                               questionTitleInd, q.getTitles(), i );
          cond = new Conditions( conditionTermVec, title );
          if (title != null) { title.setConditions(cond); }
        }
      }
    }

  }

  /**
   * Read the Question AddtlInfo conditions from the database for all
   * Question AddtlInfo associated with the Question and populate the
   * Conditions in each Question AddtlInfo object.
   * If the Question AddtlInfo have been read for all languages, the
   * multiple parms should be set to true; otherwise it should
   * be set to false.  The multiple parm indicates that there can
   * be multiple occurences of a given Question AddtlInfo indicator in the
   * Vector of AddtlInfo (one for each langauge read).
   * @param conn    database Connection object
   * @param q       the Question object whose AddtlInfo Conditions 
   *                are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 question addtlInfo indicator in the addtlInfo vector.
  **/
  static void readAllQuestionAddtlInfoConditions(Connection conn, Question q, 
                                              String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple addtlinfo
    // with the same addtlinfo indicator but different language indicators.
    String sql =
         "SELECT DISTINCT qic.questionAddInfoInd, " + 
                "qic.questionInd, qic.answerInd, qic.operators, " +
                "qic.condSortOrder " +    // req'd for query, but not retrieved
         "FROM " + schema + ".questionInfoCond qic, " + 
                   schema + ".questionAddInfo qi " +
         "WHERE  qic.questionAddInfoInd = qi.questionAddInfoInd " +
         " AND   qi.questionInd=" + q.getInd() + " " + 
         "ORDER  BY qic.questionAddInfoInd, qic.condSortOrder " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int questionInfoInd = 0,          // question info that the term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousQuestionInfoInd = -99;   // last questionInfoInd processed
    Vector conditionTermVec = null;      // Vector of ConditionTerm objects

    while ( rs.next() ) {
      questionInfoInd   = rs.getInt(1);
      termQuestionInd   = rs.getInt(2);
      termAnswerInd     = rs.getInt(3);
      termOperators     = rs.getString(4);

      // when we get to a new questionInfoInd, put the current Conditions
      // into the appropriate Question AddtlInfo object.
      if (questionInfoInd != previousQuestionInfoInd) {
        if (conditionTermVec != null) {      // if not first time through
          AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                                  previousQuestionInfoInd, 
                                                  q.getAdditionalInfo() );
          Conditions cond = new Conditions( conditionTermVec, info );
          info.setConditions(cond);

          // handle multiple occurences of the addtlInfo...
          if (multiple) {
            for (int i=2; info != null; i++) {
              info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                            previousQuestionInfoInd, 
                                            q.getAdditionalInfo(), i );
              cond = new Conditions( conditionTermVec, info );
              if (info != null) { info.setConditions(cond); }
            }
          }
        }

        conditionTermVec = new Vector();
        previousQuestionInfoInd = questionInfoInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Question AdttlInfo object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                              questionInfoInd, 
                                              q.getAdditionalInfo() );
      Conditions cond = new Conditions( conditionTermVec, info );
      info.setConditions(cond);

      // handle multiple occurences of the addtlInfo...
      if (multiple) {
        for (int i=2; info != null; i++) {
          info = (AddtlInfo) ConditionalObject.findObjectInVector(
                                        questionInfoInd, 
                                        q.getAdditionalInfo(), i );
          cond = new Conditions( conditionTermVec, info );
          if (info != null) { info.setConditions(cond); }
        }
      }
    }

  }

  /**
   * Read the Question DocLink conditions from the database for all
   * Question DocLinks associated with the Question and populate the
   * Conditions in each Question DocLink object.
   * @param conn    database Connection object
   * @param q       the Question object whose DocLink Conditions are to be read
   * @param schema  either "oa" or "oadraft"
  **/
  static void readAllQuestionDocLinkConditions(Connection conn, Question q, 
                                               String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql =
         "SELECT qlc.questionLinkInd, " + 
                "qlc.questionInd, qlc.answerInd, qlc.operators, " +
                "qlc.condSortOrder " +
         "FROM " + schema + ".questionLinkCond qlc, " + 
                   schema + ".questionLink ql " +
         "WHERE  qlc.questionLinkInd = ql.questionLinkInd " +
         " AND   ql.questionInd=" + q.getInd() + " " + 
         "ORDER  BY qlc.questionLinkInd, qlc.condSortOrder " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int questionLinkInd = 0,          // question link that the term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousQuestionLinkInd = -99;   // last questionLinkInd processed
    Vector conditionTermVec = null;      // Vector of ConditionTerm objects

    while ( rs.next() ) {
      questionLinkInd   = rs.getInt(1);
      termQuestionInd   = rs.getInt(2);
      termAnswerInd     = rs.getInt(3);
      termOperators     = rs.getString(4);

      // when we get to a new questionLinkInd, put the current Conditions
      // into the appropriate Question DocLink object.
      if (questionLinkInd != previousQuestionLinkInd) {
        if (conditionTermVec != null) {      // if not first time through
          DocLink dl = (DocLink) ConditionalObject.findObjectInVector(
                                   previousQuestionLinkInd, q.getDocLinks() );
          Conditions cond = new Conditions( conditionTermVec, dl );
          dl.setConditions(cond);
        }

        conditionTermVec = new Vector();
        previousQuestionLinkInd = questionLinkInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Question DocLink object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      DocLink dl = (DocLink) ConditionalObject.findObjectInVector(
                                 questionLinkInd, q.getDocLinks() );
      Conditions cond = new Conditions( conditionTermVec, dl );
      dl.setConditions(cond);
    }

  }

  /**
   * Read the Answer conditions from the database for all 
   * Answers associated with the Question and populate the
   * conditions in each Answer object.
   * @param conn    database Connection object
   * @param q       the Question object whose Answer conditions are to be read
   * @param schema  either "oa" or "oadraft"
  **/
  static void readAllAnswerConditions(Connection conn, Question q, 
                                      String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql = "SELECT parentAnswerInd, questionInd, answerInd, operators " +
                 "FROM " + schema + ".answerCond " +
                 "WHERE  parentAnswerInd IN (" +
                          "SELECT answerInd " +
                          "FROM   "+schema+".answer " +
                          "WHERE  questionInd=" + q.getInd() +
                        ") " +
                 "ORDER BY parentAnswerInd, condSortOrder " +
                 "FOR READ ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int parentAnswerInd = 0,          // answer the condition term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousParentAnswerInd=-99;  // last answerInd processed
    Vector conditionTermVec = null;   // Vector of ConditionTerm objects
                                      // for the current answerInd

    while ( rs.next() ) {
      parentAnswerInd = rs.getInt(1);
      termQuestionInd = rs.getInt(2);
      termAnswerInd   = rs.getInt(3);
      termOperators   = rs.getString(4);

      // when we get to a new parentAnswerInd, put the current Conditions
      // into the appropriate Answer object.
      if (parentAnswerInd != previousParentAnswerInd) {
        if (conditionTermVec != null) {      // if not first time through
          Answer ans = q.getAnswerByInd(previousParentAnswerInd);
          Conditions cond = new Conditions( conditionTermVec, ans );
          ans.setConditions(cond);
        }
        conditionTermVec = new Vector();
        previousParentAnswerInd = parentAnswerInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Answer object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      Answer ans = q.getAnswerByInd(parentAnswerInd);
      Conditions cond = new Conditions( conditionTermVec, ans );
      ans.setConditions(cond);
    }
  }

  /**
   * Read the Answer Title conditions from the database for all
   * Answer Titles associated with the question and populate the
   * Conditions in each Answer Title object.
   * If the Answer Titles have been read for all languages, the
   * multiple parms should be set to true; otherwise it should
   * be set to false.  The multiple parm indicates that there can
   * be multiple occurences of a given Answer Title indicator in the
   * Vector of Titles (one for each langauge read).
   * @param conn    database Connection object
   * @param q       the Question object whose Answer Conditions are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 answer title indicator in the answer title vector.
  **/
  static void readAllAnswerTitleConditions(Connection conn, Question q, 
                                           String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple titles
    // with the same title indicator but different language indicators.
    String sql =
         "SELECT DISTINCT at.answerInd, atc.answerTitleInd, " +
                "atc.questionInd, atc.answerInd, atc.operators, " + 
                "atc.condSortOrder " +    // req'd for query, but not retrieved
         "FROM " + schema + ".answer a, " +
                   schema + ".answerTitle at, " +
                   schema + ".answerTitleCond atc " +
         "WHERE  at.answerTitleInd = atc.answerTitleInd " +
         "AND    a.answerInd = at.answerInd " +
         "AND    a.questionInd = " + q.getInd() + " " +
         "ORDER BY at.answerInd, atc.answerTitleInd, atc.condSortOrder " +
         "FOR READ ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int answerInd = 0,                // answer that the answer title belongs to
        answerTitleInd = 0,           // answer title that the term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousAnswerInd = -99;      // last answerInd processed
    int previousAnswerTitleInd = -99; // last answerTitleInd processed
    Vector conditionTermVec = null;   // Vector of ConditionTerm objects
                                      // for the current answerTitleInd

    while ( rs.next() ) {
      answerInd       = rs.getInt(1);
      answerTitleInd  = rs.getInt(2);
      termQuestionInd = rs.getInt(3);
      termAnswerInd   = rs.getInt(4);
      termOperators   = rs.getString(5);

      // when we get to a new answerTitleInd, put the current Conditions
      // into the appropriate Answer Title object.
      if (answerTitleInd != previousAnswerTitleInd) {
        if (conditionTermVec != null) {      // if not first time through
          Answer ans = q.getAnswerByInd(previousAnswerInd);
          Title title = (Title) ConditionalObject.findObjectInVector(
                                     previousAnswerTitleInd, ans.getTitles() );
          Conditions cond = new Conditions( conditionTermVec, title );
          title.setConditions(cond);

          // handle multiple occurences of the title...
          if (multiple) {
            for (int i=2; title != null; i++) {
              title = (Title) ConditionalObject.findObjectInVector(
                                   previousAnswerTitleInd, ans.getTitles(), i );
              cond = new Conditions( conditionTermVec, title );
              if (title != null) { title.setConditions(cond); }
            }
          }
        }

        conditionTermVec = new Vector();
        previousAnswerTitleInd = answerTitleInd;
        previousAnswerInd = answerInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Answer Title object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      Answer ans = q.getAnswerByInd(answerInd);
      Title title = (Title) ConditionalObject.findObjectInVector(
                                 answerTitleInd, ans.getTitles() );
      Conditions cond = new Conditions( conditionTermVec, title );
      title.setConditions(cond);

      // handle multiple occurences of the title...
      if (multiple) {
        for (int i=2; title != null; i++) {
          title = (Title) ConditionalObject.findObjectInVector(
                               answerTitleInd, ans.getTitles(), i );
          cond = new Conditions( conditionTermVec, title );
          if (title != null) { title.setConditions(cond); }
        }
      }
    }

  }

  /**
   * Read the AnswerConcl conditions from the database for all
   * answer conclusions associated with the question and populate
   * the Conditions in each AnswerConcl object.
   * @param conn    database Connection object
   * @param q       the Question object whose Answer conditions are to be read
   * @param schema  either "oa" or "oadraft"
  **/
  static void readAllAnswerConclConditions(Connection conn, Question q, 
                                           String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql = "SELECT acc.fromAnswerInd, acc.toAnswerInd, " +
                        "acc.questionInd, acc.answerInd, acc.operators " +
                 "FROM " + schema + ".answer a, " +
                           schema + ".answerConcl ac, " + 
                           schema + ".answerConclCond acc " +
                 "WHERE  acc.fromAnswerInd = ac.fromAnswerInd " +
                 "AND    acc.toAnswerInd = ac.toAnswerInd " +
                 "AND    ac.fromAnswerInd = a.answerInd " +
                 "AND    a.questionInd = " + q.getInd() + " " +
                 "ORDER BY acc.fromAnswerInd, acc.toAnswerInd, " +
                          "acc.condSortOrder " +
                 "FOR FETCH ONLY ";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int fromAnswerInd = 0,            // fromAnswerInd of the conclusion
        toAnswerInd   = 0,            // toAnswerInd of the conclusion
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    // values read from database on previous iteration of the while loop
    int previousFromAnswerInd = -99,
        previousToAnswerInd   = -99;

    Vector conditionTermVec = null;   // Vector of ConditionTerm objects
                                      // for the conclusion

    while ( rs.next() ) {
      fromAnswerInd   = rs.getInt(1);
      toAnswerInd     = rs.getInt(2);
      termQuestionInd = rs.getInt(3);
      termAnswerInd   = rs.getInt(4);
      termOperators   = rs.getString(5);

      // when we get to a new AnswerConcl (based on fromAnswerInd/toAnswerInd),
      // put the current conditions into the appropriate AnswerConcl object
      if ( fromAnswerInd != previousFromAnswerInd ||
           toAnswerInd != previousToAnswerInd )
      {
        if (conditionTermVec != null) {      // if not first time through
          Answer ans = q.getAnswerByInd(previousFromAnswerInd);
          Conditions cond = new Conditions( conditionTermVec, ans );
          AnswerConcl concl = AnswerConcl.findAnswerConclInVector(
                                                previousFromAnswerInd, 
                                                previousToAnswerInd, 
                                                ans.getConclusions() );
          concl.setConditions(cond);
        }
        conditionTermVec = new Vector();
        previousFromAnswerInd = fromAnswerInd;
        previousToAnswerInd = toAnswerInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate AnswerConcl object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      Answer ans = q.getAnswerByInd(fromAnswerInd);
      AnswerConcl concl = AnswerConcl.findAnswerConclInVector(
                                                     fromAnswerInd, 
                                                     toAnswerInd, 
                                                     ans.getConclusions() );
      Conditions cond = new Conditions( conditionTermVec, concl );
      concl.setConditions(cond);
    }
  }

  /**
   * Read the Edge conditions from the database for all 
   * Edges associated with the Symptom and populate the
   * conditions in each Edge object.
   * @param conn       database Connection object
   * @param s          the Symptom object whose Edge conditions are to be read
   * @param schema     either "oa" or "oadraft"
  **/
  static void readAllEdgeConditions(Connection conn, Symptom s, String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql = 
            "SELECT ec.edgeInd, ec.questionInd, ec.answerInd, ec.operators " +
            "FROM " + schema + ".edge e, " +
                      schema + ".edgeCond ec, " +
                      schema + ".edgeGroup eg " +
            "WHERE  ec.edgeInd = e.edgeInd " +
            "AND    eg.edgeGroupInd = e.edgeGroupInd " +
            "AND    eg.symptomInd = " + s.getInd() + " " +
            "ORDER BY ec.edgeInd, ec.condSortOrder " +
            "FOR READ ONLY";

    rs = stmt.executeQuery( sql );

                                      // each condition term read from db:
    int edgeInd = 0,                  // edge the condition term belongs to
        termQuestionInd,              // question the condition term is based on
        termAnswerInd;                // answer the condition term is based on
    String termOperators;             // condition term operators

    int previousEdgeInd = -99;        // last edgeInd processed
    Vector conditionTermVec = null;   // Vector of ConditionTerm objects

    while ( rs.next() ) {
      edgeInd         = rs.getInt(1);
      termQuestionInd = rs.getInt(2);
      termAnswerInd   = rs.getInt(3);
      termOperators   = rs.getString(4);

      // when we get to a new edgeInd, put the current Conditions
      // into the appropriate Edge object.
      if (edgeInd != previousEdgeInd) {
        if (conditionTermVec != null) {      // if not first time through
          Edge edge = s.getEdgeByInd(previousEdgeInd);
          Conditions cond = new Conditions( conditionTermVec, edge );
          edge.setConditions(cond);
        }
        conditionTermVec = new Vector();
        previousEdgeInd = edgeInd;
      }

      ConditionTerm term = new ConditionTerm(termQuestionInd, termAnswerInd, 
                                             termOperators);
      conditionTermVec.addElement( term );
    }

    rs.close();
    stmt.close();

    // put the last Conditions into the appropriate Edge object
    // conditionTermVec will null if no rows were selected
    if (conditionTermVec != null) {
      Edge edge = s.getEdgeByInd(edgeInd);
      Conditions cond = new Conditions( conditionTermVec, edge );
      edge.setConditions(cond);
    }
  }
}
