
package com.ibm.nzna.projects.common.quest.oa;

/**
 * This class represents a single term in the Conditions on an object.
 * A ConditionTerm is represented by the Question indicator, the Answer
 * indicator and the operators that apply to the term.
 * <p>
 * Most ConditionTerms will be based on a Question/Answer pair.
 * However there is one special case that must be supported: a Condition
 * that applies to any Answer of the Question.
 * This type of ConditionTerm can be created by passing a null value
 * for the Answer object (or indicator 0).  The termAppliesToAnyAnswer() 
 * method may be used to check for this special case.  If the ConditionTerm 
 * represents this special case, the getAnswerInd() method will return 0 
 * (which is not a valid answerInd).
 * <p>
 * The Conditions class holds a Vector of ConditionTerm objects
 * in postfix notation.
 *
 * @see Conditions
 *
 * @version 1.8
 * @author  Mark DeBiase
**/

public class ConditionTerm implements java.io.Serializable {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  private int    questionInd_ = 0;
  private int    answerInd_   = 0;
  private String operators_   = null;

  private static final String VALID_OPERATORS = " &|!";

  /**
   * Create a simple condition term with no operators.
   * @param q  the Question object for the term
   * @param a  the Answer object for the term or null to represent all answsers
  **/
  public ConditionTerm(Question q, Answer a) {
    this(q, a, "");
  }

  /**
   * Create a new condition term from Question and Answer objects.
   * @param q          the Question object for the term
   * @param a          the Answer object for the term or null to
   *                   represent all answsers
   * @param operators  the operators for the term
   * @exception IllegalArgumentException  if operators are not valid
   *                                      or the Question's indicator is 0.
  **/
  public ConditionTerm(Question q, Answer a, String operators) {
    questionInd_ = q.getInd();
    if ( questionInd_ == 0 ) {
      throw new IllegalArgumentException("questionInd is 0");
    }

    if (a != null) {
      answerInd_ = a.getInd();
    } else {
      answerInd_ = 0;
    }

    setOperators(operators);     // setOperators() does validity checking
  }

  /**
   * Create a new condition term from Question and Answer indicators.
   * @param questionInd  the Question indicator for the term
   * @param answerInd    the Answer indicator for the term or 0 
   *                     to represent all answers
   * @param operators    the operators for the term
   * @exception IllegalArgumentException  if operators are not valid
  **/
  public ConditionTerm(int questionInd, int answerInd, String operators) {
    questionInd_ = questionInd;
    answerInd_   = answerInd;
    setOperators(operators);     // setOperators() does validity checking
  }

  /**
   *  Returns true if this term applies to any answer to the question.
   *  If true, getAnswerInd() will return 0.
  **/
  public boolean termAppliesToAnyAnswer() {
    return (answerInd_ == 0);
  }

  /**
   * Gets the question indicator for this term.
   * @return the question indicator
  **/
  public int getQuestionInd() {
    return questionInd_;
  }

  /**
   * Gets the answer indicator for this term.
   * @return the answer indicator
  **/
  public int getAnswerInd() {
    return answerInd_;
  }

  /**
   * Gets the operators for this term.
   * @return the operators
  **/
  public String getOperators() {
    return operators_;
  }

  /**
   * Sets the operators for this term.
   * Checks the operators to make sure they are valid.
   * @exception IllegalArgumentException  if operators are not valid
  **/
  private void setOperators(String operators) {

    if (operators.length() > 2) {
      throw new IllegalArgumentException("Invalid operators: " + operators);
    }

    for (int i=0; i<operators.length(); i++) {
      char ch = operators.charAt(i);
      if (VALID_OPERATORS.indexOf(ch) < 0) {
        throw new IllegalArgumentException("Invalid operator: " + ch);
      }
    }

    operators_ = operators;
  }

  /**
   * Returns a String representation of this object.
  **/
  public String toString() {
    return questionInd_ + ":" + answerInd_ + " " + operators_;
  }
}
