
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.type.*;
import java.util.Vector;

/**
 * Base class for objects which can have conditions.
 * Maintains the "static" conditions for an object (document classification,
 * countries) as well as the Conditions object (conditions based on Online
 * Assistant Questions).
 *
 * @version 1.20
 * @author  Mark DeBiase
**/
public abstract class ConditionalObject extends PersistentRec 
implements Cloneable {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000001 ;

   private   int             sortOrder_;     // The sort order of this object
   private   Conditions      conditions_;    // The conditions on this object
   protected TypeDocClassRec docClass_;      // quest document classification
   private   Vector          countries_;     // list of applicable countries
   private   Vector          origCountries_; // original list of countries
   private   String          creationTime_;  // date object was created
   private   String          changedTime_;   // date object was updated
   private   String          dbuser_;        // last user to update this object

   /** Return the ConditionalObject's indicator **/
   public abstract int getInd();

   /**
    * New object.
   **/
   public ConditionalObject() {
      super();
      sortOrder_     = 0;
      conditions_    = null;
      docClass_      = null;
      countries_     = null;
      origCountries_ = null;
      creationTime_  = "";
      changedTime_   = "";
      dbuser_        = "";
   }

   /**
    * Returns current sort order.
    * Used by Writer classes.
   **/
   int getSortOrder() {
      return sortOrder_;
   }

   /**
    * Set new sort order.
    * Used by Writer classes to set the sortOrder based on this object's
    * current position in a Vector of ConditionalObjects.
   **/
   void setSortOrder(int sortOrder) {
      if ( sortOrder_ != sortOrder ) {
         sortOrder_ = sortOrder;
         updateRecStatus(REC_UPDATE);
      }
   }

   /**
    * Get the document classification for this object.
    * @return the document classification
   **/
   public TypeDocClassRec getDocClass() {
      return docClass_;
   }

   /**
    * Set the document classification for this object.
    * @param docClass the document classification
   **/
   public void setDocClass(TypeDocClassRec docClass) {
      docClass_ = docClass;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Gets the list of country indicators for this object.
    * @return a Vector of TypeCountryCodeRec objects.
    * @see com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec
   **/
   public Vector getCountryList() {
      if ( countries_ == null ) {
         countries_ = new Vector();
      }
      return countries_;
   }

   /**
    * Sets the list of country indicators for this object.
    * @param countries a Vector of TypeCountryCodeRec objects.
    * @see com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec
   **/
   public void setCountryList(Vector countries) {
      countries_ = countries;
      updateRecStatus(REC_UPDATE);
   }




   /*------------------------------------------------------------*/
   /* removeCountry
    */
   /**
    *    This method will remove a single country from
    *  the object
    *  
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCountry ( TypeCountryCodeRec countryRec ) {
      if ( countries_ != null ) {
         countries_.removeElement ( countryRec ) ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   }

   /**
    * Get the original country list.
    * The original country list is the list of countries that the object
    * contained when it was read from the database (or when it was last
    * saved to the database).  It is used to compare against the current 
    * country list to determine if any countries have been added/deleted.
   **/
   Vector getOriginalCountryList() {
      return origCountries_;
   }

   /**
    * Set the original country list to the current country list.
    * The original country list is the list of countries that the object
    * contained when it was read from the database (or when it was last
    * saved to the database).  It is used to compare against the current 
    * country list to determine if any countries have been added/deleted.
   **/
   void setOriginalCountryListFromCurrent() {
      if ( countries_ != null ) {
         origCountries_ = (Vector) countries_.clone();
      } else {
         origCountries_ = null;
      }
   }


   /**
    * Get the Conditions object for this object.
   **/
   // The conditions object should never be null; a new Conditions object
   // is automatically created when a ConditionalObject is instantiated and
   // the database methods should set the value to non-null when a 
   // ConditoinalObject is read from the database.
   public Conditions getConditions() {
      if ( conditions_ == null ) {
         conditions_ = new Conditions(this);
      }
      return conditions_;
   }

   /**
    * Set the Conditions object for this object.
    * Used by Reader classes to set the Conditions object.
   **/
   // user outside this package cannot set the conditions directly
   void setConditions(Conditions conditions) {
      if ( conditions == null ) {
         throw new IllegalArgumentException("Conditions object cannot be null");
      }

      conditions_ = conditions;
      //if (conditions_.isModified()) {
      //updateRecStatus(REC_UPDATE);
      //}
   }

   /**
    * Gets the creation date/time for this object.
   **/
   public String getCreationTime() {
      return creationTime_;
   }

   /**
    * Sets the creation date/time for this object.
    * Called by database routines.
   **/
   void setCreationTime(String creationTime) {
      creationTime_ = creationTime;
   }

   /**
    * Gets the last changed date/time for this object.
   **/
   public String getChangedTime() {
      return changedTime_;
   }

   /**
    * Sets the last changed date/time for this object.
    * <b>Note:</b> The changedTime value may be set, but it is not
    * used when the object is written to the database.
    * @param changedTime a String in DB2 timestamp format.
   **/
   public void setChangedTime(String changedTime) {
      changedTime_ = changedTime;
   }

   /**
    * Gets the user who last changed this object.
   **/
   public String getDbUser() {
      return dbuser_;
   }

   /**
    * Sets the user who last changed this object.
    * Used by Writer classes - the value from the parent object is used
    * when writing to the database.
   **/
   public void setDbUser(String dbuser) {
      dbuser_ = dbuser;
   }

   /**
    * Add the indicators of all the Questions used in the conditions
    * for this object to the specified HashList.
   **/
   public void getAllQuestions(HashList questionList) {
      if ( conditions_ != null ) {
         conditions_.getAllQuestions(questionList); 
      }
   }

   /**
    * Perform a shallow copy of this object.
   **/
   public Object clone() {
      try {
         ConditionalObject cloned = (ConditionalObject) super.clone();
         if ( conditions_ != null ) {
            cloned.conditions_ = (Conditions) conditions_.clone();
         }
         return cloned;
      } catch ( CloneNotSupportedException e ) {
         throw new InternalError(e.toString());
      }
   }

   /**
    * Return a String representation of this object.
   **/
   public String toString() {
      return ">>> ConditionalObject <<<";
   }

   /**
    * Update the TypeRec objects in this object with the proper descriptions.
    * Called after a ConditionalObject is read from the database.
   **/
   void updateTypeRecData() {
      TypeList t = TypeList.getInstance();

      // docClass
      TypeDocClassRec newDocClass;
      newDocClass = (TypeDocClassRec) t.objectFromInd( docClass_.getInd(),
                                                       TypeList.TYPE_DOCCLASS );
      if ( newDocClass != null ) {
         docClass_ = newDocClass;
      }

      // country list
      if ( countries_ != null ) {
         Vector newCountries = new Vector();
         for ( int i=0; i<countries_.size(); i++ ) {
            TypeCountryCodeRec country, newCountry;
            country = (TypeCountryCodeRec) countries_.elementAt(i);
            newCountry = (TypeCountryCodeRec) t.objectFromInd( country.getInd(), 
                                                               TypeList.TYPE_COUNTRYCODE );
            if ( newCountry != null ) {
               newCountries.addElement( newCountry );
            } else {
               newCountries.addElement( country );
            }
         }
         countries_ = newCountries;
      }
   }

   /**
    * Set the recStatus to REC_NOCHANGE.
    * Also sets the modified flag on the Conditions object to false and
    * saves a copy of the current country list.
   **/
   void setRecNoChange() {
      updateRecStatus(REC_NOCHANGE);
      // reset the modification flag on the conditions
      if ( conditions_ != null ) {
         conditions_.setModified(false);
      }
      // save the country list to compare for future modifications
      setOriginalCountryListFromCurrent();
   }

   /**
    * Find the ConditionalObject with the specified indicator in a
    * Vector of ConditionalObjects.
    * This method will find the first occurence of the 
    * specified ConditionalObject.
    * @param  ind  the ConditionalObject indicator to search for
    * @param  v    the Vector of ConditionalObjects to search
    * @return the ConditionalObject with the specified indicator or 
    *         null if it was not found in the Vector.
   **/
   public static ConditionalObject findObjectInVector(int ind, Vector v) {
      return findObjectInVector(ind, v, 1);
   }

   /**
    * Find the nth occurence of the ConditionalObject with the specified
    * indicator in a Vector of ConditionalObjects.
    * @param  ind  the ConditionalObject indicator to search for
    * @param  v    the Vector of ConditionalObjects to search
    * @param  n    the occurence to find -- must be &gt;= 1
    * @return the ConditionalObject with the specified indicator or 
    *         null if it was not found in the Vector at the specified
    *         occurence.
   **/
   public static ConditionalObject findObjectInVector(int ind, Vector v, int n) {
      ConditionalObject foundObject = null;
      int occurence = 1;  // number of times we found the specified object

      if ( v != null ) {
         for ( int i=0; foundObject == null && i < v.size(); i++ ) {
            ConditionalObject tmp = (ConditionalObject) v.elementAt(i);
            if ( tmp.getInd() == ind ) {
               if ( occurence == n ) {
                  foundObject = tmp;
               } else {
                  occurence++;
               } 
            }
         }
      }

      return foundObject;
   }

}
