
package com.ibm.nzna.projects.common.quest.oa;

import java.util.Vector;

/**
 * Represents the conditions on an object (Title, AddtlInfo, etc).
 * <p>
 * A Condition is a boolean expression whose terms are Question/Answer
 * pairs.  Each term of the Condition is stored in a Vector, along
 * with the operator(s) that apply to the term, in postfix notation
 * (reverse Polish notation).
 * For example, the Condition 
 * <xmp>
 *         (Q1:A3 & Q2:A1) | (Q1:A4 & Q5:A2)
 * </xmp>
 * would be represented in postfix as
 * <xmp>
 *         Q1:A3 Q2:A1 & Q1:A4 Q5:A2 & |
 * </xmp>
 * and would be represented by the following ConditionTerm objects:
 * <xmp>
 *         ConditionTerm(Q1, A3);
 *         ConditionTerm(Q2, A1, "&");
 *         ConditionTerm(Q1, A4);
 *         ConditionTerm(Q5, A2, "&|");
 * </xmp>
 * There is also a special type of ConditionTerm that indicates the 
 * Condition can apply to any Answer of a given Question.  See the
 * documentation for the ConditionTerm object for details on this.
 * <p>
 * <b>Note:</b> It is not possible for classes outside this package
 * to create a Conditions object.  Those objects (Title, Answer, etc.)
 * which have conditions already have a valid Conditions object 
 * which should be used.
 * <p>
 * Classes outside this package can not modify the Vector of ConditionTerm 
 * objects directly (a copy can be obtained via getConditionTerms(), but 
 * cannot be set).  Use the clear() addTerm() methods to change the 
 * conditions.
 *
 * @see ConditionTerm
 *
 * @version 1.11
 * @author  Mark DeBiase
**/
public class Conditions implements Cloneable, java.io.Serializable {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000002 ;

  private Vector            terms_  = null;
  private ConditionalObject parent_ = null;  // object that this Conditions
                                             // object belongs to

  // modified_ flag will be used to determine whether conditions need to 
  // be updated when the parent object is updated.
  private boolean modified_ = false;

  /**
   * Create an empty Conditions object.
  **/
  Conditions(ConditionalObject parent) {
    this(null, parent);
  }

  /**
   * Conditions read from the database.
   * @param terms  a Vector of ConditionTerm objects
  **/
  Conditions(Vector terms, ConditionalObject parent) {
    super();
    terms_    = terms;
    parent_   = parent;
    modified_ = false;

    if (terms_ == null) {
      terms_ = new Vector(1, 3);  // initialize to an empty vector
    }
  }

  /**
   * Add a condition term from Question and Answer objects.
   * @param q          the Question object for the term
   * @param a          the Answer object for the term
   * @param operators  the operators for the term
  **/
  public void addTerm(Question q, Answer a, String operators) {
    ConditionTerm term = new ConditionTerm(q, a, operators);
    this.addTerm(term);
  }

  /**
   * Add a condition term from Question and Answer indicators.
   * @param questionInd the Question indicator for the term
   * @param answerInd   the Answer indicator for the term
   * @param operators   the operators for the term
  **/
  public void addTerm(int questionInd, int answerInd, String operators) {
    ConditionTerm term = new ConditionTerm(questionInd, answerInd, operators);
    this.addTerm(term);
  }

  /**
   * Add a condition term from a ConditionTerm object.
   * @param term  a ConditionTerm object representing the term
  **/
  public void addTerm(ConditionTerm term) {
    terms_.addElement(term);
    parent_.updateRecStatus(parent_.REC_UPDATE);
    modified_ = true;
  }

  /**
   * Get a copy of the list of ConditionTerm objects.
   * @return  the Vector of ConditionTerms
  **/
  public Vector getConditionTerms() {
    return (Vector) terms_.clone();
  }

  /**
   * Get the list of ConditionTerm objects.
   * Used by ConditionWriter class.
   * @return  the Vector of ConditionTerms
  **/
  Vector getTerms() {
    // avoid overhead of clone'ing the vector when used by the package
    return terms_;
  }

  /**
   * Clear the list of ConditionsTerm objects.
  **/
  public void clear() {
    terms_.removeAllElements();
    parent_.updateRecStatus(parent_.REC_UPDATE);
    modified_ = true;
  }

  /**
   * Returns the modification state of the Conditions.
  **/
  boolean isModified() {
    return modified_;
  }

  /**
   * Sets the modification state of the Conditions.
  **/
  void setModified(boolean modified) {
    modified_ = modified;
  }

  /**
   * Returns a HashList containing indicators of all the Questions used
   * in the condition terms.
  **/
  public HashList getAllQuestions() {
    HashList questionList = new HashList();
    getAllQuestions(questionList);
    return questionList;
  }

  /**
   * Add the indicators of all the Questions used in the condition terms
   * to the specified HashList.
   * @param questionList  the HashList to add the Question indicators to
  **/
  public void getAllQuestions(HashList questionList) {
    for (int i=0; i<terms_.size(); i++) {
      ConditionTerm term = (ConditionTerm) terms_.elementAt(i);
      int qid = term.getQuestionInd();
      questionList.put( new Integer(qid) );
    }
  }

  /**
   * Return the number of terms in this Condition.
  **/
  public int size() {
    return terms_.size();
  }

  /**
   * Perform a shallow copy of this object.
  **/
  public Object clone() {
    try {
      Conditions cloned = (Conditions) super.clone();
      cloned.terms_     = (Vector) terms_.clone();
      return cloned;
    }
    catch (CloneNotSupportedException e) {
      throw new InternalError(e.toString());
    }
  }

  /**
   * Return a String representation of this object.
  **/
  public String toString() {
    StringBuffer sb = new StringBuffer();
    int size = terms_.size();

    sb.append("Conditions={");
    if (size > 0) {
      for (int i=0; i<size; i++) {
        ConditionTerm term = (ConditionTerm) terms_.elementAt(i);
        sb.append( term );
        if (i < size-1) { sb.append(", "); }
      }
    }

    sb.append('}');

    return sb.toString();
  }
}
