
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Reader for Online Assistant Countries.
 * This class is used by the OA stored procedures.
 *
 * @version 1.8
 * @author  Mark DeBiase
**/
class CountryReader {

  /**
   * Read Countries from the database.
   * @param conn       database Connection object
   * @param tableName  the country table to read
   * @param indName    the name of the indicator column in the table
   * @param indValue   the indicator value for the countries to be read
   * @return a Vector of TypeCountryCodeRec objects
  **/
  static Vector readCountries(Connection conn, String tableName,
                              String indName, int indValue)
                throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;
    Vector    v = new Vector();

    // we don't select dbUser/changedTime - they are the same
    // as the parent object.
    String sql = "SELECT countryInd" +
                 " FROM " + tableName +
                 " WHERE " + indName + "=" + indValue +
                 " FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    while ( rs.next() ) {
      v.addElement( makeCountryCodeRec( rs.getInt(1) ) );
    }

    rs.close();
    stmt.close();

    return v;
  }

  /**
   * Read the Action Title countries from the database for all
   * Action Titles associated with the Action and populate
   * the country list in each Action Title object.
   * If the Action Titles have been read for all languages, the
   * multiple parm should be set to true; otherwise it should be
   * set to false.  The multiple parm indicates that there can be
   * multiple occurences of a given Action Title indicator in the
   * Vector of Titles (one for each language read).
   * @param conn    database Connection object
   * @param a       the Action for which Title countries are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 action title indicator in the action title vector
  **/
  static void readAllActionTitleCountries(Connection conn, Action a, 
                                          String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple titles
    // with the same title indicator but different language indicators.
    String sql =
         "SELECT DISTINCT atc.actionTitleInd, atc.countryInd " +
         "FROM " + schema + ".actionTitleCountry atc, " +
                   schema + ".actionTitle at " +
         "WHERE  atc.actionTitleInd = at.actionTitleInd " +
         " AND   at.actionInd = " + a.getInd() + " " +
         "ORDER BY atc.actionTitleInd " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int actionTitleInd = 0,                // action title indicator
        countryInd;                        // country indicator

    int previousActionTitleInd = -99;      // last actionTitleInd processed
    Vector countryVec = null;              // Vector of TypeCountryCodeRecs

    while ( rs.next() ) {
      actionTitleInd   = rs.getInt(1);
      countryInd       = rs.getInt(2);

      // when we get to a new actionTitleInd, put the current country list
      // into the appropriate Action Title object.
      if (actionTitleInd != previousActionTitleInd) {
        if (countryVec != null) {      // if not first time through
          Title title = (Title) ConditionalObject.findObjectInVector( 
                                     previousActionTitleInd, a.getTitles() );
          title.setCountryList(countryVec);

          // handle multiple ocurences of the title....
          if (multiple) {
            for (int i=2; title != null; i++) {
              title = (Title) ConditionalObject.findObjectInVector( 
                                   previousActionTitleInd, a.getTitles(), i );
              if (title != null) { title.setCountryList(countryVec); }
            }
          }
        }
        countryVec = new Vector();
        previousActionTitleInd = actionTitleInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Action Title object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      Title title = (Title) ConditionalObject.findObjectInVector( 
                                 actionTitleInd, a.getTitles() );
      title.setCountryList(countryVec);

      // handle multiple ocurences of the title....
      if (multiple) {
        for (int i=2; title != null; i++) {
          title = (Title) ConditionalObject.findObjectInVector( 
                               actionTitleInd, a.getTitles(), i );
          if (title != null) { title.setCountryList(countryVec); }
        }
      }
    }

  }

  /**
   * Read the Action AddtlInfo countries from the database for all
   * Action AddtlInfo associated with the Action and populate
   * the country list in each Action AddtlInfo object.
   * If the Action AddtlInfo have been read for all languages, the
   * multiple parm should be set to true; otherwise it should be
   * set to false.  The multiple parm indicates that there can be
   * multiple occurences of a given Action AddtlInfo indicator in the
   * Vector of AddtlInfo (one for each language read).
   * @param conn    database Connection object
   * @param a       the Action for which AddtlInfo countries are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *              action addtlinfo indicator in the action addltinfo vector
  **/
  static void readAllActionAddtlInfoCountries(Connection conn, Action a, 
                                              String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple addtlInfo
    // with the same addtlInfo indicator but different language indicators.
    String sql =
         "SELECT DISTINCT aic.actionAddInfoInd, aic.countryInd " +
         "FROM " + schema + ".actionInfoCountry aic, " +
                   schema + ".actionAddInfo ai " +
         "WHERE  aic.actionAddInfoInd = ai.actionAddInfoInd " +
         " AND   ai.actionInd = " + a.getInd() + " " +
         "ORDER BY aic.actionAddInfoInd " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int actionInfoInd = 0,                 // action addtlinfo indicator
        countryInd;                        // country indicator

    int previousActionInfoInd = -99;       // last actionInfoInd processed
    Vector countryVec = null;              // Vector of TypeCountryCodeRecs

    while ( rs.next() ) {
      actionInfoInd    = rs.getInt(1);
      countryInd       = rs.getInt(2);

      // when we get to a new actionInfoInd, put the current country list
      // into the appropriate Action AddtlInfo object.
      if (actionInfoInd != previousActionInfoInd) {
        if (countryVec != null) {      // if not first time through
          AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                                                  previousActionInfoInd, 
                                                  a.getAdditionalInfo() );
          info.setCountryList(countryVec);

          // handle multiple ocurences of the addtlInfo....
          if (multiple) {
            for (int i=2; info != null; i++) {
              info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                                            previousActionInfoInd, 
                                            a.getAdditionalInfo(), i );
              if (info != null) { info.setCountryList(countryVec); }
            }
          }
        }
        countryVec = new Vector();
        previousActionInfoInd = actionInfoInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Action AddltInfo object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                                 actionInfoInd, a.getAdditionalInfo() );
      info.setCountryList(countryVec);

      // handle multiple ocurences of the addtlInfo....
      if (multiple) {
        for (int i=2; info != null; i++) {
          info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                               actionInfoInd, a.getAdditionalInfo(), i );
          if (info != null) { info.setCountryList(countryVec); }
        }
      }
    }

  }

  /**
   * Read the Action DocLink countries from the database for all
   * Action DocLinks associated with the Action and populate
   * the country list in each Action DocLink object.
   * @param conn    database Connection object
   * @param a       the Action for which DocLink countries are to be read
   * @param schema  either "oa" or "oadraft"
  **/
  static void readAllActionDocLinkCountries(Connection conn, Action a, 
                                            String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql =
         "SELECT alc.actionLinkInd, alc.countryInd " +
         "FROM " + schema + ".actionLinkCountry alc, " +
                   schema + ".actionLink al " +
         "WHERE  alc.actionLinkInd = al.actionLinkInd " +
         " AND   al.actionInd = " + a.getInd() + " " +
         "ORDER BY alc.actionLinkInd " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int actionLinkInd = 0,                 // action doclink indicator
        countryInd;                        // country indicator

    int previousActionLinkInd = -99;       // last actionLinkInd processed
    Vector countryVec = null;              // Vector of TypeCountryCodeRecs

    while ( rs.next() ) {
      actionLinkInd    = rs.getInt(1);
      countryInd       = rs.getInt(2);

      // when we get to a new actionLinkInd, put the current country list
      // into the appropriate Action DocLink object.
      if (actionLinkInd != previousActionLinkInd) {
        if (countryVec != null) {      // if not first time through
          DocLink d = (DocLink) ConditionalObject.findObjectInVector( 
                                                  previousActionLinkInd, 
                                                  a.getDocLinks() );
          d.setCountryList(countryVec);
        }
        countryVec = new Vector();
        previousActionLinkInd = actionLinkInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Action DocLink object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      DocLink d = (DocLink) ConditionalObject.findObjectInVector( 
                                 actionLinkInd, a.getDocLinks() );
      d.setCountryList(countryVec);
    }

  }

  /**
   * Read the Question Title countries from the database for all
   * Question Titles associated with the Question and populate
   * the country list in each Question Title object.
   * If the Question Titles have been read for all languages, the
   * multiple parm should be set to true; otherwise it should be
   * set to false.  The multiple parm indicates that there can be
   * multiple occurences of a given Question Title indicator in the
   * Vector of Titles (one for each language read).
   * @param conn    database Connection object
   * @param q       the Question for which Title countries are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 question title indicator in the question title vector
  **/
  static void readAllQuestionTitleCountries(Connection conn, Question q, 
                                            String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple titles
    // with the same title indicator but different language indicators.
    String sql =
         "SELECT DISTINCT qtc.questionTitleInd, qtc.countryInd " +
         "FROM " + schema + ".questionTitleCntry qtc, " +
                   schema + ".questionTitle qt " +
         "WHERE  qtc.questionTitleInd = qt.questionTitleInd " +
         " AND   qt.questionInd = " + q.getInd() + " " +
         "ORDER BY qtc.questionTitleInd " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int questionTitleInd = 0,              // question title indicator
        countryInd;                        // country indicator

    int previousQuestionTitleInd = -99;    // last questionTitleInd processed
    Vector countryVec = null;              // Vector of TypeCountryCodeRecs
                                           // for current questionTitleInd

    while ( rs.next() ) {
      questionTitleInd = rs.getInt(1);
      countryInd       = rs.getInt(2);

      // when we get to a new questionTitleInd, put the current country list
      // into the appropriate Question Title object.
      if (questionTitleInd != previousQuestionTitleInd) {
        if (countryVec != null) {      // if not first time through
          Title title = (Title) ConditionalObject.findObjectInVector( 
                                     previousQuestionTitleInd, q.getTitles() );
          title.setCountryList(countryVec);

          // handle multiple ocurences of the title....
          if (multiple) {
            for (int i=2; title != null; i++) {
              title = (Title) ConditionalObject.findObjectInVector( 
                                   previousQuestionTitleInd, q.getTitles(), i );
              if (title != null) { title.setCountryList(countryVec); }
            }
          }
        }
        countryVec = new Vector();
        previousQuestionTitleInd = questionTitleInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Question Title object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      Title title = (Title) ConditionalObject.findObjectInVector( 
                                 questionTitleInd, q.getTitles() );
      title.setCountryList(countryVec);

      // handle multiple ocurences of the title....
      if (multiple) {
        for (int i=2; title != null; i++) {
          title = (Title) ConditionalObject.findObjectInVector( 
                               questionTitleInd, q.getTitles(), i );
          if (title != null) { title.setCountryList(countryVec); }
        }
      }
    }

  }

  /**
   * Read the Question AddtlInfo countries from the database for all
   * Question AddtlInfo associated with the Question and populate
   * the country list in each Question AddtlInfo object.
   * If the Question AddtlInfo have been read for all languages, the
   * multiple parm should be set to true; otherwise it should be
   * set to false.  The multiple parm indicates that there can be
   * multiple occurences of a given Question AddtlInfo indicator in the
   * Vector of AddtlInfo (one for each language read).
   * @param conn    database Connection object
   * @param q       the Question for which AddtlInfo countries are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *              question addtlinfo indicator in the question addltinfo vector
  **/
  static void readAllQuestionAddtlInfoCountries(Connection conn, Question q, 
                                                String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple addtlInfo
    // with the same addtlInfo indicator but different language indicators.
    String sql =
         "SELECT DISTINCT qic.questionAddInfoInd, qic.countryInd " +
         "FROM " + schema + ".questionInfoCntry qic, " +
                   schema + ".questionAddInfo qi " +
         "WHERE  qic.questionAddInfoInd = qi.questionAddInfoInd " +
         " AND   qi.questionInd = " + q.getInd() + " " +
         "ORDER BY qic.questionAddInfoInd " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int questionInfoInd = 0,               // question addtlinfo indicator
        countryInd;                        // country indicator

    int previousQuestionInfoInd = -99;     // last questionInfoInd processed
    Vector countryVec = null;              // Vector of TypeCountryCodeRecs

    while ( rs.next() ) {
      questionInfoInd  = rs.getInt(1);
      countryInd       = rs.getInt(2);

      // when we get to a new questionInfoInd, put the current country list
      // into the appropriate Question AddtlInfo object.
      if (questionInfoInd != previousQuestionInfoInd) {
        if (countryVec != null) {      // if not first time through
          AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                                                  previousQuestionInfoInd, 
                                                  q.getAdditionalInfo() );
          info.setCountryList(countryVec);

          // handle multiple ocurences of the addtlInfo....
          if (multiple) {
            for (int i=2; info != null; i++) {
              info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                                            previousQuestionInfoInd, 
                                            q.getAdditionalInfo(), i );
              if (info != null) { info.setCountryList(countryVec); }
            }
          }
        }
        countryVec = new Vector();
        previousQuestionInfoInd = questionInfoInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Question AddltInfo object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      AddtlInfo info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                                 questionInfoInd, q.getAdditionalInfo() );
      info.setCountryList(countryVec);

      // handle multiple ocurences of the addtlInfo....
      if (multiple) {
        for (int i=2; info != null; i++) {
          info = (AddtlInfo) ConditionalObject.findObjectInVector( 
                               questionInfoInd, q.getAdditionalInfo(), i );
          if (info != null) { info.setCountryList(countryVec); }
        }
      }
    }

  }

  /**
   * Read the Question DocLink countries from the database for all
   * Question DocLinks associated with the Question and populate
   * the country list in each Question DocLink object.
   * @param conn    database Connection object
   * @param q       the Question for which DocLink countries are to be read
   * @param schema  either "oa" or "oadraft"
  **/
  static void readAllQuestionDocLinkCountries(Connection conn, Question q, 
                                              String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql =
         "SELECT qlc.questionLinkInd, qlc.countryInd " +
         "FROM " + schema + ".questionLinkCntry qlc, " +
                   schema + ".questionLink ql " +
         "WHERE  qlc.questionLinkInd = ql.questionLinkInd " +
         " AND   ql.questionInd = " + q.getInd() + " " +
         "ORDER BY qlc.questionLinkInd " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int questionLinkInd = 0,               // question doclink indicator
        countryInd;                        // country indicator

    int previousQuestionLinkInd = -99;     // last questionLinkInd processed
    Vector countryVec = null;              // Vector of TypeCountryCodeRecs

    while ( rs.next() ) {
      questionLinkInd  = rs.getInt(1);
      countryInd       = rs.getInt(2);

      // when we get to a new questionLinkInd, put the current country list
      // into the appropriate Question DocLink object.
      if (questionLinkInd != previousQuestionLinkInd) {
        if (countryVec != null) {      // if not first time through
          DocLink d = (DocLink) ConditionalObject.findObjectInVector( 
                                     previousQuestionLinkInd, q.getDocLinks() );
          d.setCountryList(countryVec);
        }
        countryVec = new Vector();
        previousQuestionLinkInd = questionLinkInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Question Title object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      DocLink d = (DocLink) ConditionalObject.findObjectInVector( 
                                 questionLinkInd, q.getDocLinks() );
      d.setCountryList(countryVec);
    }

  }

  /**
   * Read the Answer countries from the database for all
   * Answers associated with the Question and populate
   * the country list in each Answer object.
   * @param conn    database Connection object
   * @param q       the Question object whose Answer countries are to be read
   * @param schema  either "oa" or "oadraft"
  **/
  static void readAllAnswerCountries(Connection conn, Question q, String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql = "SELECT answerInd, countryInd " +
                 "FROM " + schema + ".answerCountry " +
                 "WHERE answerInd IN (" +
                           "SELECT answerind " +
                           "FROM " + schema + ".answer " +
                           "WHERE questionInd=" + q.getInd() +
                       ") " +
                 "ORDER BY answerInd " +
                 "FOR READ ONLY";

    rs = stmt.executeQuery( sql );

    int answerInd = 0,                // answer indicator from database
        countryInd;                   // country indicator from database

    int previousAnswerInd = -99;      // last answerInd processed
    Vector countryVec = null;         // the Vector of TypeCountryCodeRecs
                                      // for the current answerInd

    while ( rs.next() ) {
      answerInd  = rs.getInt(1);
      countryInd = rs.getInt(2);

      // when we get to a new answerInd, put the current country list
      // into the appropriate Answer object.
      if (answerInd != previousAnswerInd) {
        if (countryVec != null) {      // if not first time through
          Answer a = q.getAnswerByInd(previousAnswerInd);
          a.setCountryList(countryVec);
        }
        countryVec = new Vector();
        previousAnswerInd = answerInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Answer object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      Answer a = q.getAnswerByInd(answerInd);
      a.setCountryList(countryVec);
    }
  }

  /**
   * Read the Answer Title countries from the database for all
   * Answer Titles associated with the Question and populate
   * the country list in each Answer Title object.
   * If the Answers Titles have been read for all languages, the
   * multiple parm should be set to true; otherwise it should be
   * set to false.  The multiple parm indicates that there can be
   * multiple occurences of a given Answer Title indicator in the
   * Vector of Titles (one for each language read).
   * @param conn    database Connection object
   * @param q       the Question object whose Answer Title 
   *                countries are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 answer title indicator in the answer title vector
  **/
  static void readAllAnswerTitleCountries(Connection conn, Question q, 
                                          String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple titles
    // with the same title indicator but different language indicators.
    String sql =
         "SELECT DISTINCT at.answerInd, atc.answerTitleInd, atc.countryInd " +
         "FROM " + schema + ".answer a, " + 
                   schema + ".answerTitle at, " +
                   schema + ".answerTitleCountry atc " +
         "WHERE  at.answerTitleInd = atc.answerTitleInd " +
         "AND    a.answerInd = at.answerInd " +
         "AND    a.questionInd = " + q.getInd() + " " +
         "ORDER BY at.answerInd, atc.answerTitleInd " +
         "FOR READ ONLY";

    rs = stmt.executeQuery( sql );

    int answerInd = 0,                // answer indicator from database
        answerTitleInd = 0,           // answer title indicator from database
        countryInd;                   // country indicator from database

    int previousAnswerInd = -99;      // last answerInd processed
    int previousAnswerTitleInd = -99; // last answerTitleInd processed
    Vector countryVec = null;         // the Vector of TypeCountryCodeRecs
                                      // for the current answerTitleInd

    while ( rs.next() ) {
      answerInd      = rs.getInt(1);
      answerTitleInd = rs.getInt(2);
      countryInd     = rs.getInt(3);

      // when we get to a new answerTitleInd, put the current country list
      // into the appropriate Answer Title object.
      if (answerTitleInd != previousAnswerTitleInd) {
        if (countryVec != null) {      // if not first time through
          Answer a = q.getAnswerByInd(previousAnswerInd);
          Title title = (Title) ConditionalObject.findObjectInVector( 
                                     previousAnswerTitleInd, a.getTitles() );
          title.setCountryList(countryVec);

          // handle multiple ocurences of the title....
          if (multiple) {
            for (int i=2; title != null; i++) {
              title = (Title) ConditionalObject.findObjectInVector( 
                                   previousAnswerTitleInd, a.getTitles(), i );
              if (title != null) { title.setCountryList(countryVec); }
            }
          }
        }
        countryVec = new Vector();
        previousAnswerTitleInd = answerTitleInd;
        previousAnswerInd = answerInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Answer Title object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      Answer a = q.getAnswerByInd(answerInd);
      Title title = (Title) ConditionalObject.findObjectInVector( 
                                 answerTitleInd, a.getTitles() );
      title.setCountryList(countryVec);

      // handle multiple ocurences of the title....
      if (multiple) {
        for (int i=2; title != null; i++) {
          title = (Title) ConditionalObject.findObjectInVector( 
                               answerTitleInd, a.getTitles(), i );
          if (title != null) { title.setCountryList(countryVec); }
        }
      }
    }

  }

  /**
   * Read and populate the Countries for all AnswerConcl objects for
   * the specified Question.
   * @param conn  the database Connection object
   * @param q     the Question whose AnswerConcl countries are to be read
  **/
  static void readAllAnswerConclCountries(Connection conn, 
                                          Question q, String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;
    Vector    v = new Vector();

    String sql = 
        "SELECT acc.fromAnswerInd, acc.toAnswerInd, acc.countryInd " +
        "FROM " + schema + ".answer a, " +
                  schema + ".answerConcl ac, " + 
                  schema + ".answerConclCountry acc " +
        "WHERE  acc.fromAnswerInd = ac.fromAnswerInd " +
        "AND    acc.toAnswerInd = ac.toAnswerInd " +
        "AND    ac.fromAnswerInd = a.answerInd " +
        "AND    a.questionInd = " + q.getInd() + " " +
        "ORDER BY acc.fromAnswerInd, acc.toAnswerInd " +
        "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    // values read from the database
    int fromAnswerInd = 0,
        toAnswerInd   = 0,
        countryInd;

    // values read from database on previous iteration of the while loop
    int previousFromAnswerInd = -99,
        previousToAnswerInd = -99;

    Vector countryVec = null;

    while ( rs.next() ) {
      fromAnswerInd = rs.getInt(1);
      toAnswerInd   = rs.getInt(2);
      countryInd    = rs.getInt(3);

      // when we get to a new AnswerConcl (based on fromAnswerInd/toAnswerInd),
      // put the current country list into the appropriate AnswerConcl object.
      if ( fromAnswerInd != previousFromAnswerInd ||
           toAnswerInd != previousToAnswerInd ) 
      {
        if (countryVec != null) {      // if not first time through
          Answer a = q.getAnswerByInd(previousFromAnswerInd);
          AnswerConcl concl = AnswerConcl.findAnswerConclInVector(
                                                previousFromAnswerInd, 
                                                previousToAnswerInd, 
                                                a.getConclusions() );
          concl.setCountryList(countryVec);
        }

        countryVec = new Vector();
        previousFromAnswerInd = fromAnswerInd;
        previousToAnswerInd = toAnswerInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate AnswerConcl object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      Answer a = q.getAnswerByInd(fromAnswerInd);
      AnswerConcl concl = AnswerConcl.findAnswerConclInVector(
                                                     fromAnswerInd, 
                                                     toAnswerInd, 
                                                     a.getConclusions() );
      concl.setCountryList(countryVec);
    }

  }

  /**
   * Read the Symptom Title countries from the database for all
   * Symptom Titles associated with the Symptom and populate
   * the country list in each Symptom Title object.
   * If the Symptom Titles have been read for all languages, the
   * multiple parm should be set to true; otherwise it should be
   * set to false.  The multiple parm indicates that there can be
   * multiple occurences of a given Symptom Title indicator in the
   * Vector of Titles (one for each language read).
   * @param conn    database Connection object
   * @param s       the Symptom for which Title countries are to be read
   * @param schema  either "oa" or "oadraft"
   * @param multiple if true, there can be multiple occurences of a given
   *                 symptom title indicator in the symptom title vector
  **/
  static void readAllSymptomTitleCountries(Connection conn, Symptom s, 
                                           String schema, boolean multiple)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // need to SELECT DISTINCT because there may be multiple titles
    // with the same title indicator but different language indicators.
    String sql =
         "SELECT DISTINCT stc.symptomTitleInd, stc.countryInd " +
         "FROM " + schema + ".symptomTitleCntry stc, " +
                   schema + ".symptomTitle st " +
         "WHERE  stc.symptomTitleInd = st.symptomTitleInd " +
         " AND   st.symptomInd = " + s.getInd() + " " +
         "ORDER BY stc.symptomTitleInd " +
         "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int symptomTitleInd = 0,               // symptom title indicator
        countryInd;                        // country indicator

    int previousSymptomTitleInd = -99;     // last symptomTitleInd processed
    Vector countryVec = null;              // Vector of TypeCountryCodeRecs

    while ( rs.next() ) {
      symptomTitleInd  = rs.getInt(1);
      countryInd       = rs.getInt(2);

      // when we get to a new symptomTitleInd, put the current country list
      // into the appropriate Symptom Title object.
      if (symptomTitleInd != previousSymptomTitleInd) {
        if (countryVec != null) {      // if not first time through
          SymptomTitle title = SymptomTitle.findTitleInVector( 
                                                previousSymptomTitleInd, 
                                                s.getSymptomTitles() );
          title.setCountryList(countryVec);

          // handle multiple ocurences of the title....
          if (multiple) {
            for (int i=2; title != null; i++) {
              title = SymptomTitle.findTitleInVector( previousSymptomTitleInd, 
                                                      s.getSymptomTitles(), i );
              if (title != null) { title.setCountryList(countryVec); }
            }
          }
        }
        countryVec = new Vector();
        previousSymptomTitleInd = symptomTitleInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Symptom Title object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      SymptomTitle title = SymptomTitle.findTitleInVector( symptomTitleInd, 
                                                         s.getSymptomTitles() );
      title.setCountryList(countryVec);

      // handle multiple ocurences of the title....
      if (multiple) {
        for (int i=2; title != null; i++) {
          title = SymptomTitle.findTitleInVector( symptomTitleInd, 
                                                  s.getSymptomTitles(), i );
          if (title != null) { title.setCountryList(countryVec); }
        }
      }
    }

  }

  /**
   * Read the Edge countries from the database for all
   * Edges associated with the Symptom and populate
   * the country list in each Edge object.
   * @param conn       database Connection object
   * @param s          the Symptom object whose Edge countries are to be read
   * @param schema     either "oa" or "oadraft"
  **/
  static void readAllEdgeCountries(Connection conn, Symptom s, String schema)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String sql = "SELECT ec.edgeInd, ec.countryInd " +
                 "FROM " + schema + ".edge e, " +
                           schema + ".edgeCountry ec, " +
                           schema + ".edgeGroup eg " +
                 "WHERE  ec.edgeInd = e.edgeInd " +
                 "AND    eg.edgeGroupInd = e.edgeGroupInd " +
                 "AND    eg.symptomInd = " + s.getInd() + " " +
                 "ORDER BY ec.edgeInd " +
                 "FOR FETCH ONLY";

    rs = stmt.executeQuery( sql );

    int edgeInd = 0,                  // edge indicator from database
        countryInd;                   // country indicator from database

    int previousEdgeInd = -99;        // last edgeInd read from database
    Vector countryVec = null;         // the Vector of TypeCountryCodeRecs

    while ( rs.next() ) {
      edgeInd    = rs.getInt(1);
      countryInd = rs.getInt(2);

      // when we get to a new edgeInd, put the current country list
      // into the appropriate Edge object.
      if (edgeInd != previousEdgeInd) {
        if (countryVec != null) {      // if not first time through
          Edge edge = s.getEdgeByInd(previousEdgeInd);
          edge.setCountryList(countryVec);
        }
        countryVec = new Vector();
        previousEdgeInd = edgeInd;
      }

      countryVec.addElement( makeCountryCodeRec( countryInd ) );
    }

    rs.close();
    stmt.close();

    // put the last country list into the appropriate Edge object
    // countryVec will null if no rows were selected
    if (countryVec != null) {
      Edge edge = s.getEdgeByInd(edgeInd);
      edge.setCountryList(countryVec);
    }
  }


  // once a TypeCountryCodeRec is created for a given country
  // indicator, cache it to use again
  private static final int MAXCOUNTRYCODE=1000;

  /** Cache for TypeCountryCodeRec objects */
  // TypeCountryCodeRecs with indicators 1 to MAXCOUNTRYCODE will be cached.
  private static TypeCountryCodeRec tccrCache[] = 
                                       new TypeCountryCodeRec[MAXCOUNTRYCODE];

  /**
   * Construct a TypeCountryCodeRec object with the specified indicator.
   * The returned object will be initialized with 0 for all short values
   * and the empty string for all string values.
  **/
  static TypeCountryCodeRec makeCountryCodeRec(int ind) {
    TypeCountryCodeRec rec = null ;
    short z = 0;

    if (ind >= 0 && ind < MAXCOUNTRYCODE && tccrCache[ind] != null) {
      // return a cached object
      rec = tccrCache[ind];
    } else {
      // create a new object with the specified indicator

      //FIX WHEN REIMPLEMENTrec = new TypeCountryCodeRec( ind,   // the indicator
      //FIX WHEN REIMPLEMENT                              "",    // phoneCountryCode
      //FIX WHEN REIMPLEMENT                              "",    // description
      //FIX WHEN REIMPLEMENT                              "",    // zipMask
      //FIX WHEN REIMPLEMENT                              "",    // phoneMask 
      //FIX WHEN REIMPLEMENT                              z,     // zipDigits
      //FIX WHEN REIMPLEMENT                              "",    // timeDiff
      //FIX WHEN REIMPLEMENT                              "",    // multiZone
      //FIX WHEN REIMPLEMENT                              "",    // useDST
      //FIX WHEN REIMPLEMENT                              "",    // currency
      //FIX WHEN REIMPLEMENT                              "",    // currencyMask
      //FIX WHEN REIMPLEMENT                              z,     // lang1
      //FIX WHEN REIMPLEMENT                              z,     // lang2 
      //FIX WHEN REIMPLEMENT                              z,     // lang3
      //FIX WHEN REIMPLEMENT                              z      // geoInd
      //FIX WHEN REIMPLEMENT                            );

      // cache the object we just created if ind is in valid range
      if (ind >= 0 && ind < MAXCOUNTRYCODE) {
        tccrCache[ind] = rec;
      }
    }

    return rec;
  }
}
