
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.type.TypeDocClassRec;
import com.ibm.nzna.shared.util.*;
import java.sql.*;

/**
 * Online Assistant Document Link.
 * A Document Link is a Conditional Link or LinkGroup in an Action
 * or Question.
 *
 * @see Link
 * @see LinkGroup
 *
 * @version 1.18
 * @author  Mark DeBiase
**/
public class DocLink extends ConditionalObject {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  private int  docLinkInd_   = 0;   // database indicator
  private int  parentInd_    = 0;   // foreign key to parent table
  private int  linkInd_      = 0;   // Link object or null
  private int  linkGroupInd_ = 0;   // LinkGroup object or null

  /**
   * Create a new DocLink object containing a Link object.
  **/
  public DocLink(Link link) {
    super();
    linkInd_      = link.getInd();
    linkGroupInd_ = 0;
    updateRecStatus(REC_INSERT);
  }

  /**
   * Create a new DocLink object containing a LinkGroup object.
  **/
  public DocLink(LinkGroup linkGroup) {
    super();
    linkInd_      = 0;
    linkGroupInd_ = linkGroup.getInd();
    updateRecStatus(REC_INSERT);
  }

  /**
   * DocLink object read from the database.
  **/
  DocLink(int docLinkInd) {
    super();
    docLinkInd_ = docLinkInd;
    updateRecStatus(REC_NOCHANGE);
  }

  /**
   * Assign a database key for this DocLink.
   * This method can be called for DocLinks attached to
   * either an Action or a Question so the counter name in tigris.counter
   * to be used is passed as a parameter.
   * @param  counterName the counter name to use to get the next key
   * @return true if a database key was assigned, otherwise false
  **/
  private boolean assignDatabaseKey(String counterName) {
    boolean success = true;

    if (docLinkInd_ == 0 && valid()) {
      LogSystem.log(1, "Getting database key for DocLink: " + counterName);
      docLinkInd_ = Counter.getCounter( LogSystem.getInstance(), counterName );
      if (docLinkInd_ == 0) {
        LogSystem.log(2, "Failed to get database key for DocLink");
        success = false;
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  private void assignDatabaseKey(Connection conn, String counterName)
               throws Exception 
  {

    if (docLinkInd_ == 0) {
      docLinkInd_ = Counter.getCounter( conn, counterName );
      if (docLinkInd_ == 0) {
        throw new Exception("Failed to get database key for DocLink");
      }
    }

    return;
  }

  /**
   * Assign database key for Action DocLink.
   * @return true if a database key was assigned, otherwise false
  **/
  boolean assignDatabaseKeyForAction() {
    return assignDatabaseKey("ACTIONLINK");
  }
  void assignDatabaseKeyForAction(Connection conn) throws Exception {
    assignDatabaseKey(conn, "ACTIONLINK");
  }
  /**
   * Assign database key for Question DocLink.
   * @return true if a database key was assigned, otherwise false
  **/
  boolean assignDatabaseKeyForQuestion() {
    return assignDatabaseKey("QSTNLINK");
  }
  void assignDatabaseKeyForQuestion(Connection conn) throws Exception {
    assignDatabaseKey(conn, "QSTNLINK");
  }
  /**
   * Assign database key for draft Action DocLink.
   * @return true if a database key was assigned, otherwise false
  **/
  boolean assignDatabaseKeyForActionDraft() {
    return assignDatabaseKey("DACTNLINK");
  }
  void assignDatabaseKeyForActionDraft(Connection conn) throws Exception {
    assignDatabaseKey(conn, "DACTNLINK");
  }
  /**
   * Assign database key for draft Question DocLink.
   * @return true if a database key was assigned, otherwise false
  **/
  boolean assignDatabaseKeyForQuestionDraft() {
    return assignDatabaseKey("DQSTNLINK");
  }
  void assignDatabaseKeyForQuestionDraft(Connection conn) throws Exception
  {
    assignDatabaseKey(conn, "DQSTNLINK");
  }

  /**
   * Get the indicator for this DocLink.
  **/
  public int getInd() {
    return docLinkInd_;
  }

  /**
   * Set the indicator for this DocLink.
  **/
  void setInd(int docLinkInd) {
    docLinkInd_ = docLinkInd;
  }

  /**
   * Get the parent objects's indicator.
  **/
  int getParentInd() {
    return parentInd_;
  }

  /**
   * Set the parent objects's indicator.
  **/
  void setParentInd(int parentInd) {
    parentInd_ = parentInd;
  }

  /**
   * Returns true if this DocLink contains a LinkGroup instead of a Link.
  **/
  public boolean hasLinkGroup() {
    return ( linkGroupInd_ != 0 );
  }

  /**
   * Get the Link this DocLink object contains.
   * Returns 0 if this DocLink contains a LinkGroup instead of a Link.
   * @see #isLinkGroup()
  **/
  public int getLinkInd() {
    return linkInd_;
  }

  /**
   * Set the Link this DocLink object contains.
  **/
  void setLinkInd(int linkInd) {
    linkInd_ = linkInd;
  }

  /**
   * Get the LinkGroup this DocLink object contains.
   * Returns 0 if this DocLink contains a Link instead of a LinkGroup.
   * @see #isLinkGroup()
  **/
  public int getLinkGroupInd() {
    return linkGroupInd_;
  }

  /**
   * Set the LinkGroup this DocLink object contains.
  **/
  void setLinkGroupInd(int linkGroupInd) {
    linkGroupInd_ = linkGroupInd;
  }

  /**
   * Reset values in this object.
   * Sets the indicator to 0, sets the recStatus to REC_INSERT.
  **/
  void resetNew() {
    setInd(0);
    setParentInd(0);
    updateRecStatus ( REC_NOCHANGE ) ;
    updateRecStatus(REC_INSERT);
  }

  /**
   * Return a String representation of this object.
  **/
  public String toString() {
    return "DocLink " + docLinkInd_ + ": " +
              "linkInd=" + linkInd_ + " " +
              "linkGroupInd=" + linkGroupInd_;
  }
}
