
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.db.*;
import java.sql.*;
import java.util.Vector;

/**
 * Represents an Edge in a Symptom.
 * An represents an Answer to a Question in a Symptom.
 * An Edge can have Conditions (base class is ConditionalObject).
 * <p>
 * There are two types of Edges: "Answer" Edges which represent
 * a single Answer to a Question and "Other" Edges which represent
 * all Answers to the Question which were not already represented by 
 * specific Edges.
 * <p>
 * There can be multiple Edges for the same Answer originating from 
 * a single node.  The Conditions on the edge determine which path
 * will be taken in the Symptom logic flow.
 * <p>
 * There are four additional constructs that can be attached to
 * an Edge, represented by the EdgePreanswer, EdgeAlist, EdgeUnanswer
 * and EdgeReturn classes.  See the documentation on each class
 * for details.
 *
 * @see EdgePreanswer
 * @see EdgeAlist
 * @see EdgeUnanswer
 * @see EdgeReturn
 *
 * @version 1.24
 * @author  Mark DeBiase
**/

public class Edge extends ConditionalObject {
   // note: ConditionalObject.sortOrder_ does not apply to Edges
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   private int        edgeInd_      = 0;   // the edge's indicator
   private int        edgeGroupInd_ = 0;   // the edge group this edge belongs to
   private short      edgeType_     = 0;   // the edge type
   private Vector     preanswers_  = null; // list of EdgePreanswer objects
   private Vector     actionList_  = null; // list of EdgeAlist objects
   private Vector     unanswers_   = null; // list of EdgeUnanswer objects
   private EdgeReturn edgeReturn_  = null; // symptom subroutine return node 

   // need to save the original return node that was read from the database --
   // this is so we can tell if the return node was changed and the original
   // should be deleted from the database.
   EdgeReturn originalReturnNode = null;


   // activeQuestionInd_/activeAnswerInd_ are the indicators of the active
   // question/answer this edge represents.
   // - for edges based on active Questions, this will be the indicator of
   //   the Question/Answer objects returned by the getInd() method.
   // - for edges based on draft Questions, this will be the active question
   //   indicator returned by the QuestionDraft.getActiveQuestionInd() and
   //   Answer.getActiveAnswerInd() methods.
   // activeQuestionInd_/activeAnswerInd_ will be 0 if the edge is in a
   // draft symptom and the edge refers to an question or answer which has
   // never been published.
   //
   // draftQuestionInd_/draftAnswerInd_ are only valid for edges in a draft
   // symptom -- they hold the indicators of the draft question/answer that
   // the edge refers to.  they will be 0 if there is no draft object.
   // 
   // activeAnswerInd_ and draftAnswerInd_ will always be 0 for 
   // EDGE_TYPE_OTHER edges.
   //
   private int    activeQuestionInd_  = 0;
   private int    activeAnswerInd_    = 0;
   private int    draftQuestionInd_   = 0;
   private int    draftAnswerInd_     = 0;
   private String defaultAnswerTitle_ = null ;

   // edge types corresponding to the database
   public static final short EDGE_TYPE_ANSWER = 0;
   public static final short EDGE_TYPE_OTHER  = 1;

   /**
    * New object.  
    * This is a EDGE_TYPE_ANSWER edge based on a specific Answer to a Question.
    * @exception IllegalArgumentException if the Question's indicator is 0.
   **/
   public Edge(Question q, Answer a) {
      edgeType_ = EDGE_TYPE_ANSWER;

      if ( q instanceof QuestionDraft ) {
         QuestionDraft qd = (QuestionDraft) q;
         activeQuestionInd_ = qd.getActiveQuestionInd();
         draftQuestionInd_  = qd.getInd();
         activeAnswerInd_   = a.getActiveAnswerInd();
         draftAnswerInd_    = a.getInd();
      } else {
         activeQuestionInd_ = q.getInd();
         activeAnswerInd_   = a.getInd();
      }

      if ( activeQuestionInd_ == 0 && draftQuestionInd_ == 0 ) {
         throw new IllegalArgumentException("questionInd is 0");
      }
      if ( activeAnswerInd_ == 0 && draftAnswerInd_ == 0 ) {
         throw new IllegalArgumentException("answerInd is 0");
      }

      updateRecStatus(REC_INSERT);
   }

   /**
    * New object.
    * This is a EDGE_TYPE_OTHER edge representing any Answer to the Question.
    * @exception IllegalArgumentException if the Question's indicator is 0.
   **/
   public Edge(Question q) {
      edgeType_ = EDGE_TYPE_OTHER;

      if ( q instanceof QuestionDraft ) {
         QuestionDraft qd = (QuestionDraft) q;
         activeQuestionInd_ = qd.getActiveQuestionInd();
         draftQuestionInd_  = qd.getInd();
      } else {
         activeQuestionInd_ = q.getInd();
      }

      if ( activeQuestionInd_ == 0 && draftQuestionInd_ == 0 ) {
         throw new IllegalArgumentException("questionInd is 0");
      }

      updateRecStatus(REC_INSERT);
   }

   /**
    * Create an edge based on a <b><i>published</i></b> Answer to a
    * Question.  
    * This is an EDGE_TYPE_ANSWER edge based on a specific Answer to a Question.
    * 
    * @param questionInd  the indicator of a <i>published</i> question
    * @param answerInd    the indicator of a <i>published</i> answer
    * @exception IllegalArgumentException if questionInd or answerInd is 0.
    *
    * @deprecated use the Edge(Question, Answer) constructor instead
   **/
   public Edge(int questionInd, int answerInd) {
      super();

      edgeGroupInd_      = 0;
      edgeType_          = EDGE_TYPE_ANSWER;
      activeQuestionInd_ = questionInd;
      activeAnswerInd_   = answerInd;

      if ( activeQuestionInd_ == 0 ) {
         throw new IllegalArgumentException("questionInd is 0");
      }
      if ( activeAnswerInd_ == 0 ) {
         throw new IllegalArgumentException("answerInd is 0");
      }

      updateRecStatus(REC_INSERT);
   }

   /**
    * Create a edge based on a <b><i>published</i></b> Question
    * by database indicator.
    * This is an EDGE_TYPE_OTHER edge representing any Answer to the Question.
    *
    * @param questionInd  the indicator of a <i>published</i> question
    * @exception IllegalArgumentException if questionInd is 0.
    *
    * @deprecated use the Edge(Question) constructor instead
   **/
   public Edge(int questionInd) {
      super();

      edgeGroupInd_      = 0;
      edgeType_          = EDGE_TYPE_OTHER;
      activeQuestionInd_ = questionInd;
      activeAnswerInd_   = 0;

      if ( activeQuestionInd_ == 0 ) {
         throw new IllegalArgumentException("questionInd is 0");
      }

      updateRecStatus(REC_INSERT);
   }

   /**
    * Edge object read from the database
   **/
   Edge() { 
      // nothing to do -- Readers should call set methods.
   }

   /**
    * Gets the Edge indicator.
   **/
   public int getInd() {
      return edgeInd_;
   }

   /**
    * Sets the Edge indicator.
   **/
   void setInd(int edgeInd) {
      edgeInd_ = edgeInd;
   }


   /**
    * Assign database keys for this object and all objects contained therein.
    * @return true if database keys were assigned, otherwise false
   **/
   boolean assignDatabaseKeys(boolean draft) {
      boolean success;

      success = assignDatabaseKey(draft);

      if ( preanswers_ != null ) {
         for ( int i=0; i<preanswers_.size() && success; i++ ) {
            EdgePreanswer preans = (EdgePreanswer) preanswers_.elementAt(i);
            success = preans.assignDatabaseKey(draft);
         }
      }

      if ( actionList_ != null ) {
         for ( int i=0; i<actionList_.size() && success; i++ ) {
            EdgeAlist alist = (EdgeAlist) actionList_.elementAt(i);
            success = alist.assignDatabaseKey(draft);
         }
      }

      if ( unanswers_ != null ) {
         for ( int i=0; i<unanswers_.size() && success; i++ ) {
            EdgeUnanswer unans = (EdgeUnanswer) unanswers_.elementAt(i);
            success = unans.assignDatabaseKey(draft);
         }
      }

      if ( edgeReturn_ != null ) {
         success = edgeReturn_.assignDatabaseKey(draft);
      }

      return success;
   }
   // version to be called by publish stored procedure:
   void assignDatabaseKeys(Connection conn, boolean draft) throws Exception {

      assignDatabaseKey(conn, draft);

      if ( preanswers_ != null ) {
         for ( int i=0; i<preanswers_.size(); i++ ) {
            EdgePreanswer preans = (EdgePreanswer) preanswers_.elementAt(i);
            preans.assignDatabaseKey(conn, draft);
         }
      }

      if ( actionList_ != null ) {
         for ( int i=0; i<actionList_.size(); i++ ) {
            EdgeAlist alist = (EdgeAlist) actionList_.elementAt(i);
            alist.assignDatabaseKey(conn, draft);
         }
      }

      if ( unanswers_ != null ) {
         for ( int i=0; i<unanswers_.size(); i++ ) {
            EdgeUnanswer unans = (EdgeUnanswer) unanswers_.elementAt(i);
            unans.assignDatabaseKey(conn, draft);
         }
      }

      if ( edgeReturn_ != null ) {
         edgeReturn_.assignDatabaseKey(conn, draft);
      }

   }
   /**
    * Assign a database key for this Edge.
    * @return true if a database key was assigned, otherwise false
   **/
   private boolean assignDatabaseKey(boolean isDraft) {
      boolean success = true;
      String counterName;

      if ( isDraft ) {
         counterName = "DEDGEIND";
      } else {
         counterName = "EDGEIND";
      }

      if ( edgeInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for Edge: " + counterName);
         edgeInd_ = Counter.getCounter( LogSystem.getInstance(), counterName );
         if ( edgeInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for Edge");
            success = false;
         }
      }

      return success;
   }
   // version to be called by publish stored procedure:
   private void assignDatabaseKey(Connection conn, boolean isDraft) 
   throws Exception
   {
      String counterName;

      if ( isDraft ) {
         counterName = "DEDGEIND";
      } else {
         counterName = "EDGEIND";
      }

      if ( edgeInd_ == 0 ) {
         edgeInd_ = Counter.getCounter( conn, counterName );
         if ( edgeInd_ == 0 ) {
            throw new Exception("Failed to get database key for Edge");
         }
      }

   }

   /**
    * Reset values in this Edge and its sub-objects.
    * Sets the indicator to 0, sets the recStatus to REC_INSERT.
   **/
   public void resetNew() {
      setInd(0);
      updateRecStatus(REC_NOCHANGE);
      updateRecStatus(REC_INSERT);

      //reset sub-objects
      if ( preanswers_ != null ) {
         for ( int i=0; i<preanswers_.size(); i++ ) {
            EdgePreanswer preans = (EdgePreanswer) preanswers_.elementAt(i);
            preans.resetNew();
         }
      }
      if ( actionList_ != null ) {
         for ( int i=0; i<actionList_.size(); i++ ) {
            EdgeAlist alist = (EdgeAlist) actionList_.elementAt(i);
            alist.resetNew();
         }
      }
      if ( unanswers_ != null ) {
         for ( int i=0; i<unanswers_.size(); i++ ) {
            EdgeUnanswer unans = (EdgeUnanswer) unanswers_.elementAt(i);
            unans.resetNew();
         }
      }
      if ( edgeReturn_ != null ) {
         edgeReturn_.resetNew();
      }
   }

   /**
    * Gets the EdgeGroup indicator.
   **/
   int getEdgeGroupInd() {
      return edgeGroupInd_;
   }

   /**
    * Sets the EdgeGroup indicator.
   **/
   void setEdgeGroupInd(int edgeGroupInd) {
      edgeGroupInd_ = edgeGroupInd;
   }

   /**
    * Get the edge type.
    * Returns one of the following values:
    * <ul>
    * <li>EDGE_TYPE_ANSWER
    * <li>EDGE_TYPE_OTHER
    * </ul>
   **/
   public short getEdgeType() {
      return edgeType_;
   }

   /**
    * Set the edge type.
    * Valid values for edgeType:
    * <ul>
    * <li>EDGE_TYPE_ANSWER
    * <li>EDGE_TYPE_OTHER
    * </ul>
   **/
   void setEdgeType(short edgeType) {
      edgeType_ = edgeType;
   }

   /**
    * Returns true if this Edge represents a single Answer.
   **/
   public boolean isAnswerEdge() {
      return(edgeType_ == EDGE_TYPE_ANSWER);
   }

   //-----------------------------------------------------------------------
   // get/set active indicators
   //-----------------------------------------------------------------------

   /**
    * Gets the <b>active</b> Answer indicator.
    * Will return 0 if this Edge is EDGE_TYPE_OTHER or the Answer is a draft 
    * that has not been published yet (draft Symptoms only).
    * Same as getActiveAnswerInd().
   **/
   public int getAnswerInd() {
      return activeAnswerInd_;
   }

   /**
    * Sets the <i>active</i> Answer indicator.
    * Same as setActiveAnswerInd().
   **/
   void setAnswerInd(int answerInd) {
      activeAnswerInd_ = answerInd;
   }

   /**
    * Gets the <b>active</b> Question indicator.
    * Will return 0 if the Question is a draft that has not been
    * published yet (draft Symptoms only).
    * <br>
    * Same as getActiveQuestionInd().
   **/
   public int getQuestionInd() {
      return activeQuestionInd_;
   }

   /**
    * Sets the <i>active</i> Question indicator.
    * Same as setActiveQuestionInd().
   **/
   void setQuestionInd(int questionInd) {
      activeQuestionInd_ = questionInd;
   }

   /**
    * Get the active Answer indciator.
    * Same as getAnswerInd().
   **/
   public int getActiveAnswerInd() {
      return activeAnswerInd_;
   }

   /**
    * Set the active Answer indciator.
   **/
   void setActiveAnswerInd(int activeAnswerInd) {
      activeAnswerInd_ = activeAnswerInd;
   }

   /**
    * Get the active Question indicator.
    * Same as getQuestionInd().
   **/
   public int getActiveQuestionInd() {
      return activeQuestionInd_;
   }

   /**
    * Set the active Question indicator.
   **/
   void setActiveQuestionInd(int activeQuestionInd) {
      activeQuestionInd_ = activeQuestionInd;
   }

   //-----------------------------------------------------------------------
   // get/set draft indicators
   //-----------------------------------------------------------------------

   /**
    * Get the draft Answer indciator.
   **/
   public int getDraftAnswerInd() {
      return draftAnswerInd_;
   }

   /**
    * Set the draft Answer indciator.
   **/
   public void setDraftAnswerInd(int draftAnswerInd) {
      draftAnswerInd_ = draftAnswerInd;
   }

   /**
    * Get the draft Question indicator.
   **/
   public int getDraftQuestionInd() {
      return draftQuestionInd_;
   }

   /**
    * Set the draft Question indicator.
   **/
   public void setDraftQuestionInd(int draftQuestionInd) {
      draftQuestionInd_ = draftQuestionInd;
   }


   /**
    * Returns the list of preanswers.
    * @return a Vector, possibly empty, of EdgePreanswer objects.
   **/
   public Vector getPreanswers() {
      if (preanswers_ == null) {             // API backwards compatability
        preanswers_ = new Vector(1, 3);
      }
      return preanswers_;
   }

   /**
    * Sets the list of preanswers.
    * @param preanswers a Vector of EdgePreanswer objects.
   **/
   public void setPreanswers(Vector preanswers) {
      preanswers_ = preanswers;
   }

   /**
    * Add an EdgePreanswer object to the list of preanswers.
    * @param preans  the EdgePreanswer object to be added to the list.
   **/
   public void addPreanswer(EdgePreanswer preans) {
      Vector v = getPreanswers();
      v.addElement(preans);
   }

   /**
    * Returns the list of Actions to be added to the action list.
    * @return a Vector, possibly empty, of EdgeAlist objects.
   **/
   public Vector getActionList() {
      if (actionList_ == null) {             // API backwards compatability.
        actionList_ = new Vector(1, 3);
      }
      return actionList_;
   }

   /**
    * Sets the list of Actions to be added to the action list.
    * @param actionList a Vector, possibly empty, of EdgeAlist objects.
   **/
   public void setActionList(Vector actionList) {
      actionList_ = actionList;
   }

   /**
    * Add an EdgeAlist object to the action list.
    * @param alist  the EdgeAlist object to be added to the list.
   **/
   public void addActionList(EdgeAlist alist) {
      Vector v = getActionList();
      v.addElement(alist);
   }

   /**
    * Returns the list of unanswers.
    * @return a Vector of EdgeUnanswer objects.
   **/
   public Vector getUnanswers() {
      if (unanswers_ == null) {              // API backwards compatability
        unanswers_ = new Vector(1, 3);
      }
      return unanswers_;
   }

   /**
    * Sets the list of unanswers.
    * @param unanswers a Vector of EdgeUnanswer objects.
   **/
   public void setUnanswers(Vector unanswers) {
      unanswers_ = unanswers;
   }

   /**
    * Add an EdgeUnanswer object to the list of unanswers.
    * @param unanswer  the EdgeUnanswer object to be added to the list.
   **/
   public void addUnanswer(EdgeUnanswer unanswer) {
      Vector v = getUnanswers();
      v.addElement(unanswer);
   }

   /**
    * Returns the edge return node.
    * @return the edge return node or null if there is no return node
   **/
   public EdgeReturn getReturnNode() {
      return edgeReturn_;
   }

   /**
    * Sets the edge return node.
    * @param edgeReturn the edge return node
   **/
   public void setReturnNode(EdgeReturn edgeReturn) {
      edgeReturn_ = edgeReturn;
   }

   /**
    * Return a String representation of this object.
   **/
   public String toString() {
      //StringBuffer sb = new StringBuffer();
      //sb.append("Edge ").append(edgeInd_).append(": ");
      //if ( edgeType_ == EDGE_TYPE_ANSWER ) {
      //   sb.append("type=ANSWER ");
      //} else {
      //   sb.append("type=OTHER ");
      //}
      //sb.append("questionInd=").append(activeQuestionInd_)
      //.append("/").append(draftQuestionInd_);
      //sb.append(", answerInd=").append(activeAnswerInd_)
      //.append("/").append(draftAnswerInd_);
      //return sb.toString();
      return ( getDefaultAnswerTitle() ) ;
   }


  /**
   * Set the recStatus of this object and 
   * all its contained objects to REC_NOCHANGE.
  **/
  void setRecNoChange() {

    // this object handled by superclass
    super.setRecNoChange();

    // contained objects:
    if ( preanswers_ != null ) {
       for ( int i=0; i<preanswers_.size(); i++ ) {
          EdgePreanswer preans = (EdgePreanswer) preanswers_.elementAt(i);
          preans.updateRecStatus(REC_NOCHANGE);
       }
    }
    if ( actionList_ != null ) {
       for ( int i=0; i<actionList_.size(); i++ ) {
          EdgeAlist alist = (EdgeAlist) actionList_.elementAt(i);
          alist.updateRecStatus(REC_NOCHANGE);
       }
    }
    if ( unanswers_ != null ) {
       for ( int i=0; i<unanswers_.size(); i++ ) {
          EdgeUnanswer unans = (EdgeUnanswer) unanswers_.elementAt(i);
          unans.updateRecStatus(REC_NOCHANGE);
       }
    }
    if ( edgeReturn_ != null ) {
       edgeReturn_.updateRecStatus(REC_NOCHANGE);
    }
  }

   /*------------------------------------------------------------*/
   /* setDefaultAnswerTitle
    */
   /**
    *    This method will set the default answer
    *  title that is represented by this edge.
    *  
    *
    *  @param title the default title
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDefaultAnswerTitle(String defaultTitle) {
     defaultAnswerTitle_ = defaultTitle;
   }


   /*------------------------------------------------------------*/
   /* getDefaultAnswerTitle
    */
   /**
    *    This method will return the default answer
    *  title that is represented by this edge.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getDefaultAnswerTitle () {
      if ( defaultAnswerTitle_ == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "Edge.getDefaultAnswerTitle", 5 ) ;
         int       answerInd = 0 ;
         String    qualifier = null ;                                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Figure out what answer to use, the draft or active      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( draftAnswerInd_ == 0 ) {
            answerInd = activeAnswerInd_ ;
            qualifier = "OA" ;
         } else {
            answerInd = draftAnswerInd_ ;
            qualifier = "OADRAFT" ;
         }

         try {
            Statement stmt = sqlMethod.createStatement() ;
            ResultSet rs   = stmt.executeQuery ( "SELECT TITLE "+
                                                 "FROM "+qualifier+".ANSWERTITLE "+
                                                 "WHERE ANSWERIND = "+answerInd+" AND "+
                                                 "      SORTORDER = (SELECT MAX(SORTORDER) FROM "+qualifier+".ANSWERTITLE WHERE ANSWERIND = "+answerInd+") "+
                                                 "FOR FETCH ONLY" ) ;

            if ( rs.next() )
               defaultAnswerTitle_ = rs.getString(1).trim() ;


         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }

         sqlMethod.close() ;
      } /* if title is null  */

      return( defaultAnswerTitle_ ) ;
   }
}

