
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.*;

/**
 * Reader for Online Assistant Edges.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.8
 * @author  Mark DeBiase
**/


class EdgeDraftReader {

  /**
   * Read all Edges in the Symptom.
  **/
  static void readAllEdgesForSymptom(Connection conn, Symptom s) 
              throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all Edges
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "e.edgeInd, e.edgeType, e.activeAnswerInd, e.draftAnswerInd, " +
         "e.docClassInd, e.created, e.dbUser, e.changedTime, g.edgeGroupInd " +
       "FROM   oadraft.edge e, oadraft.edgeGroup g " +
       "WHERE  e.edgeGroupInd = g.edgeGroupInd " +
       "AND    g.symptomInd =" + s.getInd() + " " +
       "ORDER BY g.edgeGroupInd " +
       "FOR FETCH ONLY"
    );

    int edgeGroupInd = 0;
    int previousEdgeGroupInd = -99;

    Vector edgeVec = null;
    EdgeGroup group = null;

    while( rs.next() ) {

      edgeGroupInd = rs.getInt(9);

      // when we get to a new edgeGroupInd, put the current edgeVec into
      // the appropriate EdgeGroup object.
      if (edgeGroupInd != previousEdgeGroupInd) {
        if (edgeVec != null) {    // if not first time through
          // group still has the EdgeGroup from previous loop iteration
          group.setEdges( edgeVec );
        }
        edgeVec = new Vector(2, 5);
        previousEdgeGroupInd = edgeGroupInd;
        // get the EdgeGroup for the new edgeGroupInd
        group = EdgeGroup.findEdgeGroupInVector( edgeGroupInd, 
                                                 s.getEdgeGroups() );
      }

      Edge edge = new Edge();
      edge.setInd( rs.getInt(1) );
      edge.setEdgeType( rs.getShort(2) );
      edge.setActiveAnswerInd( rs.getInt(3) );
      edge.setDraftAnswerInd( rs.getInt(4) );
      // assume the head node is an oaobject node for a question:
      edge.setActiveQuestionInd( group.getHeadNode().getActiveObjectInd() );
      edge.setDraftQuestionInd(  group.getHeadNode().getDraftObjectInd() );
      edge.setEdgeGroupInd( group.getInd() );

      edge.setDocClass( new TypeDocClassRec( rs.getShort(5), "") );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(6) ) );
      edge.setDbUser( rs.getString(7).trim() );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(8) ) );

      //---------------------------------------------------------------------
      // Read the default Answer title for the Edge
      // Note: this method should be optimized to read answer titles for
      //       all edges at once, similar to conditions/countries below
      //---------------------------------------------------------------------
      readDefaultAnswerTitle(conn, edge);

      //---------------------------------------------------------------------
      // Read the Edge conditions/countries
      //---------------------------------------------------------------------
      // MAD performance: edge conditions are now read all at once below
      //edge.setConditions(
      //   ConditionReader.readConditions( conn, "oadraft.edgeCond",
      //                                   "edgeInd", edge )
      //);
      // MAD performance: edge countries are now read all at once below
      //edge.setCountryList(
      //   CountryReader.readCountries( conn, "oadraft.edgeCountry",
      //                                "edgeInd", edge.getInd() )
      //);

      //---------------------------------------------------------------------
      // Read the edgePreanser table (contains EdgePreanswer, EdgeAlist, 
      // EdgeUnanswer and EdgeReturn objects).
      //---------------------------------------------------------------------
      // MAD performance: edge preanswers are now read all at once below
      //readPreanswerTableForEdge(conn, edge, s);

      edge.updateRecStatus(edge.REC_NOCHANGE);
      edgeVec.addElement( edge );
    }
    
    rs.close();
    stmt.close();
  
    // put the last edgeVec into the appropriate EdgeGroup object
    // edgeVec will be null if no records were selected -- shouldnt happen!
    if (edgeVec != null) {
      // group still has the EdgeGroup from the last loop iteration
      group.setEdges( edgeVec );
    }

    //-----------------------------------------------------------------------
    // Read the Edge conditions/countries for all Edges in the Symptom.
    //-----------------------------------------------------------------------
    ConditionReader.readAllEdgeConditions(conn, s, "oadraft");
    CountryReader.readAllEdgeCountries(conn, s, "oadraft");

    //-----------------------------------------------------------------------
    // Read the edgePreanser table (contains EdgePreanswer, EdgeAlist, 
    // EdgeUnanswer and EdgeReturn objects) all Edges in the Symptom.
    //-----------------------------------------------------------------------
    readPreanswerTableForAllEdges(conn, s);
  }

  /**
   * Read SymptomDraft Edges.
   * Note this method is obsolete -- readAllEdgesForSymptom is used instead.
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readEdgesForEdgeGroup(Connection conn, EdgeGroup eg, Symptom s) 
              throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all Edges
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "edgeInd, edgeType, activeAnswerInd, draftAnswerInd, docClassInd, " +
         "created, dbUser, changedTime " +
       "FROM oadraft.edge " +
       "WHERE edgeGroupInd=" + eg.getInd() + " " +
       "FOR FETCH ONLY"
    );

    Vector v = new Vector();
    while( rs.next() )
    {
      //int edgeInd     = rs.getInt(1);
      //int answerInd   = rs.getInt(2);
      //int questionInd = eg.getHeadNode().getOAObjectInd();
      //Edge edge = new Edge(edgeInd, questionInd, answerInd);

      Edge edge = new Edge();
      edge.setInd( rs.getInt(1) );
      edge.setEdgeType( rs.getShort(2) );
      edge.setActiveAnswerInd( rs.getInt(3) );
      edge.setDraftAnswerInd( rs.getInt(4) );
      // assume the head node is an oaobject node for a question:
      edge.setActiveQuestionInd( eg.getHeadNode().getActiveObjectInd() );
      edge.setDraftQuestionInd(  eg.getHeadNode().getDraftObjectInd() );
      edge.setEdgeGroupInd( eg.getInd() );

      edge.setDocClass( new TypeDocClassRec( rs.getShort(5), "") );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(6) ) );
      edge.setDbUser( rs.getString(7).trim() );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(8) ) );

      //---------------------------------------------------------------------
      // Read the default Answer title for the Edge
      //---------------------------------------------------------------------
      readDefaultAnswerTitle(conn, edge);

      //---------------------------------------------------------------------
      // Read the Edge conditions/countries
      //---------------------------------------------------------------------
      edge.setConditions(
         ConditionReader.readConditions( conn, "oadraft.edgeCond",
                                         "edgeInd", edge )
      );
      edge.setCountryList(
         CountryReader.readCountries( conn, "oadraft.edgeCountry",
                                      "edgeInd", edge.getInd() )
      );

      //---------------------------------------------------------------------
      // Read the edgePreanser table (contains EdgePreanswer, EdgeAlist, 
      // EdgeUnasnwer and EdgeReturn objects).
      //---------------------------------------------------------------------
      readPreanswerTableForEdge(conn, edge, s);

      edge.updateRecStatus(edge.REC_NOCHANGE);
      v.addElement( edge );
    }
    
    rs.close();
    stmt.close();
  
    eg.setEdges(v);
  }
------------------------- END COMMENTED OUT ----------------------------- */

  /**
   * Read the default title for the Answer represented by this edge.
   * Will read the default answer title from the oadraft schema if
   * getDraftAnswerInd() returns non-zero.  Otherwise the default
   * answer title will be read from the oa schema.
  **/
  // Note: this method should be optimized to read the answer titles
  //       for all edges at once
  private static void readDefaultAnswerTitle(Connection conn, Edge edge)
                      throws SQLException
  {
    if (edge.getEdgeType() == Edge.EDGE_TYPE_OTHER) {
      // the edge represents multiple answers
      edge.setDefaultAnswerTitle("OTHER");
    } else {
      Statement stmt = conn.createStatement();
      ResultSet rs;

      int    answerInd;
      String schema;

      // determine whether we will read answer title from active or draft:
      if ( edge.getDraftAnswerInd() == 0 ) {
        answerInd = edge.getActiveAnswerInd();
        schema    = "oa";
      } else {
        answerInd = edge.getDraftAnswerInd();
        schema    = "oadraft";
      }

      rs = stmt.executeQuery(
           "SELECT title " +
           "FROM " + schema + ".answerTitle " +
           "WHERE answerInd=" + answerInd + " " +
           "AND sortOrder=(" +
                   "SELECT MAX(sortOrder) " +
                   "FROM " + schema + ".answerTitle " +
                   "WHERE answerInd=" + answerInd + 
           ") " +
           "FOR FETCH ONLY"
      );

      if ( rs.next() ) {
        edge.setDefaultAnswerTitle( rs.getString(1) );
      } else {
        edge.setDefaultAnswerTitle( "N/A" );
      }

      rs.close();
      stmt.close();
    }
  }

  /**
   * Read EdgePreanswer, EdgeAlist, EdgeUnasnswer and EdgeReturn objects
   * from the edgePreanswer table for all Edges in the Symptom.
  **/
  private static void readPreanswerTableForAllEdges(Connection conn, Symptom s)
          throws SQLException
  {
    Statement  stmt  = conn.createStatement();
    ResultSet  rs;
    PreparedStatement pstmtActive = null,
                      pstmtDraft  = null;

    //---------------------------------------------------------------------
    // Read all Edge preanswers, unanswers, alist and return objects
    // from the edgePreanswer table for this Symptom.
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "ep.edgePreanswerInd, ep.preanswerType, " +
         "ep.activeAnswerInd, ep.draftAnswerInd, " +
         "ep.activeActionInd, ep.draftActionInd, " + 
         "ep.activeQuestionInd, ep.draftQuestionInd, " + 
         "ep.nodeInd, ep.edgeInd " +
       "FROM oadraft.edgePreanswer ep, oadraft.edge e, oadraft.edgeGroup eg " +
       "WHERE ep.edgeInd = e.edgeInd " +
       " AND  e.edgeGroupInd = eg.edgeGroupInd " +
       " AND  eg.symptomInd = " + s.getInd() + " " +
       "ORDER BY ep.edgeInd " +
       "FOR FETCH ONLY"
    );

    while( rs.next() )
    {
      // get the edge this database row belongs to:
      Edge edge = s.getEdgeByInd( rs.getInt(10) );

      // read the common values....
      int    edgePreanswerInd = rs.getInt(1);
      short  preanswerType    = rs.getShort(2);

      switch (preanswerType) {

        case EdgeObject.EDGE_PREANSWER:
          {
            // need to get the question this answer belongs to
            int activeAnswerInd   = rs.getInt(3);
            int draftAnswerInd    = rs.getInt(4);
            int activeQuestionInd = 0;
            int draftQuestionInd  = 0;

            if ( activeAnswerInd != 0 ) {
              if (pstmtActive == null) {
                // dont prepare the stmt until we actully need it.
                pstmtActive = conn.prepareStatement(
                                    "SELECT questionInd " + 
                                    "FROM oa.answer " +
                                    "WHERE answerInd=? " +
                                    "FOR FETCH ONLY"
                              );
              }
              pstmtActive.setInt(1, activeAnswerInd);
              ResultSet rs2 = pstmtActive.executeQuery();
              rs2.next();
              activeQuestionInd = rs2.getInt(1);
              rs2.close();
            }

            if ( draftAnswerInd != 0 ) {
              if (pstmtDraft == null) {
                // dont prepare the stmt until we actully need it.
                pstmtDraft = conn.prepareStatement(
                                    "SELECT questionInd " + 
                                    "FROM oadraft.answer " +
                                    "WHERE answerInd=? " +
                                    "FOR FETCH ONLY"
                             );
              }
              pstmtDraft.setInt(1, draftAnswerInd);
              ResultSet rs2 = pstmtDraft.executeQuery();
              rs2.next();
              draftQuestionInd = rs2.getInt(1);
              rs2.close();
            }

            EdgePreanswer preans;
            preans = new EdgePreanswer( activeQuestionInd, activeAnswerInd,
                                        draftQuestionInd,  draftAnswerInd  );
            preans.setInd( edgePreanswerInd );
            preans.setEdgeInd( edge.getInd() );
            preans.updateRecStatus( preans.REC_NOCHANGE );

            edge.addPreanswer( preans );
          }
          break;

        case EdgeObject.EDGE_ALIST:
          {
            int activeActionInd = rs.getInt(5);
            int draftActionInd  = rs.getInt(6);

            EdgeAlist alist = new EdgeAlist( activeActionInd, draftActionInd );
            alist.setInd( edgePreanswerInd );
            alist.setEdgeInd( edge.getInd() );
            alist.updateRecStatus( alist.REC_NOCHANGE );

            edge.addActionList( alist );
          }
          break;

        case EdgeObject.EDGE_UNANSWER:
          {
            int activeQuestionInd = rs.getInt(7);
            int draftQuestionInd  = rs.getInt(8);

            EdgeUnanswer unans = new EdgeUnanswer( activeQuestionInd,
                                                   draftQuestionInd );
            unans.setInd( edgePreanswerInd );
            unans.setEdgeInd( edge.getInd() );
            unans.updateRecStatus( unans.REC_NOCHANGE );
 
            edge.addUnanswer( unans );
          }
          break;

        case EdgeObject.EDGE_RETURN:
          {
            int nodeInd = rs.getInt(9);

            EdgeReturn ereturn = new EdgeReturn( s.getNodeByInd(nodeInd) );
            ereturn.setInd( edgePreanswerInd );
            ereturn.setEdgeInd( edge.getInd() );
            ereturn.updateRecStatus( ereturn.REC_NOCHANGE );

            // we also have to save the return node that was read from 
            // the database so we can tell if it has been changed so 
            // we can delete it from the database.
            edge.setReturnNode( ereturn );
            edge.originalReturnNode = ereturn;
          }
          break;

        default:
          // unknown preanswerType
          break;
      }
    }

    rs.close();
    stmt.close();
    if (pstmtActive != null) {
      pstmtActive.close();
    }
    if (pstmtDraft != null) {
      pstmtDraft.close();
    }
  }

  /**
   * Read EdgePreanswer, EdgeAlist, EdgeUnasnswer and EdgeReturn objects
   * from the edgePreanswer table.
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  private static void readPreanswerTableForEdge(Connection conn, Edge edge,
                                                Symptom s)
          throws SQLException
  {
    Statement  stmt  = conn.createStatement();
    Statement  stmt2 = null;
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all Edge preanswers, unanswers, alist and return objects
    // from the edgePreanswer table.
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "edgePreanswerInd, preanswerType, " +
         "activeAnswerInd, draftAnswerInd, " +
         "activeActionInd, draftActionInd, " + 
         "activeQuestionInd, draftQuestionInd, " + 
         "nodeInd, " +
         "dbUser, changedTime " +
       "FROM oadraft.edgePreanswer " +
       "WHERE edgeInd=" + edge.getInd() + " " +
       "FOR FETCH ONLY"
    );

    Vector preanswerVec = new Vector(1,3);
    Vector alistVec     = new Vector(1,3);
    Vector unanswerVec  = new Vector(1,3);
    EdgeReturn ereturn  = null;

    while( rs.next() )
    {
      // read the common values....
      int    edgePreanswerInd = rs.getInt(1);
      short  preanswerType    = rs.getShort(2);
      String dbUser           = rs.getString(10).trim();
      String changedTime      = DatabaseUtil.timestampFromDB(rs.getString(11));

      switch (preanswerType) {

        case EdgeObject.EDGE_PREANSWER:
          {
            int activeAnswerInd   = rs.getInt(3);
            int draftAnswerInd    = rs.getInt(4);
            int activeQuestionInd = 0;
            int draftQuestionInd  = 0;

            if (stmt2 == null) {
              // delay creating the statement until it is needed...
              stmt2 = conn.createStatement();
            }

            // need to get the question this answer belongs to
            ResultSet rs2 = null;

            if ( activeAnswerInd != 0 ) {
              rs2 = stmt2.executeQuery( 
                          "SELECT questionInd " + 
                          "FROM oa.answer " +
                          "WHERE answerInd=" + activeAnswerInd + " " +
                          "FOR FETCH ONLY"
              );
              rs2.next();
              activeQuestionInd = rs2.getInt(1);
              rs2.close();
            }
            if ( draftAnswerInd != 0 ) {
              rs2 = stmt2.executeQuery( 
                          "SELECT questionInd " + 
                          "FROM oadraft.answer " +
                          "WHERE answerInd=" + draftAnswerInd + " " +
                          "FOR FETCH ONLY"
              );
              rs2.next();
              draftQuestionInd = rs2.getInt(1);
              rs2.close();
            }

            EdgePreanswer preans;
            preans = new EdgePreanswer( activeQuestionInd, activeAnswerInd,
                                        draftQuestionInd,  draftAnswerInd  );
            preans.setInd( edgePreanswerInd );
            preans.setEdgeInd( edge.getInd() );
            preans.updateRecStatus( preans.REC_NOCHANGE );

            preanswerVec.addElement( preans );
          }
          break;

        case EdgeObject.EDGE_ALIST:
          {
            int activeActionInd = rs.getInt(5);
            int draftActionInd  = rs.getInt(6);

            EdgeAlist alist = new EdgeAlist( activeActionInd, draftActionInd );
            alist.setInd( edgePreanswerInd );
            alist.setEdgeInd( edge.getInd() );
            alist.updateRecStatus( alist.REC_NOCHANGE );

            alistVec.addElement( alist );
          }
          break;

        case EdgeObject.EDGE_UNANSWER:
          {
            int activeQuestionInd = rs.getInt(7);
            int draftQuestionInd  = rs.getInt(8);

            EdgeUnanswer unans = new EdgeUnanswer( activeQuestionInd,
                                                   draftQuestionInd );
            unans.setInd( edgePreanswerInd );
            unans.setEdgeInd( edge.getInd() );
            unans.updateRecStatus( unans.REC_NOCHANGE );
 
            unanswerVec.addElement( unans );
          }
          break;

        case EdgeObject.EDGE_RETURN:
          {
            int nodeInd = rs.getInt(9);

            ereturn = new EdgeReturn( s.getNodeByInd(nodeInd) );
            ereturn.setInd( edgePreanswerInd );
            ereturn.setEdgeInd( edge.getInd() );
            ereturn.updateRecStatus( ereturn.REC_NOCHANGE );
          }
          break;

        default:
          // unknown preanswerType
          break;
      }
    }

    edge.setPreanswers( preanswerVec );
    edge.setActionList( alistVec );
    edge.setUnanswers( unanswerVec );

    edge.setReturnNode( ereturn );        // set to null if there is none
    edge.originalReturnNode = ereturn;    // need to save the return node
                                          // read from the db so it can be
                                          // deleted if it is changed
    rs.close();
    stmt.close();
    if (stmt2 != null) {
      stmt2.close();
    }
  }
------------------------- END COMMENTED OUT ----------------------------- */
}
