
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import java.util.Vector;

/**
 * @version 1.6
 * @author  Mark DeBiase
**/
class EdgeDraftWriter {

  /**
   * Returns the SQL String to write the Edge and its associated sub-objects
   * to the database.
  **/
  static String getSQL(Edge e) {

    StringBuffer sb = new StringBuffer();

    sb.append( getEdgeSQL(e) );

    // if this edge wasn't deleted, get the SQL for the preanswer table
    // (EdgePreanswer, EdgeUnanswer, EdgeReturn and EdgeAlist objects)
    if ( e.getRecStatus() != PersistentRec.REC_DELETE ) {
      sb.append( getPreanswerTableSQL(e) );
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Edge to the database.
  **/
  static String getEdgeSQL(Edge e) {
    String sql;

    switch ( e.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(e);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(e);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(e);
        break;

      default:
        sql = "";
    }

    return sql;
  }

  /**
   * Returns the SQL String to insert the Edge in the database.
  **/
  static String getInsertSQL(Edge e) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oadraft.edge " +
         "(edgeInd, edgeGroupInd, edgeType, activeAnswerInd, draftAnswerInd, " +
         "docClassInd, dbUser, changedTime) " +
      "VALUES ("
    );

    // edgeInd:
    sb.append( e.getInd()                                  ).append(',');

    // edgeGroupInd:
    sb.append( e.getEdgeGroupInd()                         ).append(',');

    // edgeType:
    sb.append( e.getEdgeType()                             ).append(',');

    // activeAnswerInd/draftAnswerInd:
    if ( e.getEdgeType() == Edge.EDGE_TYPE_ANSWER ) {
      if ( e.getActiveAnswerInd() != 0 ) {
        sb.append( e.getActiveAnswerInd() ).append(',');
      } else {
        sb.append("NULL,");
      }
      if ( e.getDraftAnswerInd() != 0 ) {
        sb.append( e.getDraftAnswerInd() ).append(',');
      } else {
        sb.append("NULL,");
      }
    } else {
      sb.append( "NULL,NULL,");
    }

    // docClassInd:
    sb.append( e.getDocClass().getInd()                   ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( e.getDbUser() )   ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // insert the countries
    sb.append( CountryWriter.getInsertSQL( "oadraft.edgeCountry",
                                           e.getInd(),
                                           e.getCountryList(),
                                           e.getDbUser() ) );

    // insert the conditions
    sb.append( ConditionWriter.getInsertSQL("oadraft.edgeCond",
                                            e.getInd(), 
                                            e.getConditions(), 
                                            e.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to update the Edge in the database.
  **/
  static String getUpdateSQL(Edge e) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oadraft.edge SET");

    sb.append(" docClassInd=")
      .append( e.getDocClass().getInd() );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( e.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE edgeInd=").append( e.getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oadraft.edgeCountry",
                                           "edgeInd",
                                            e.getInd(),
                                            e.getCountryList(),
                                            e.getOriginalCountryList(),
                                            e.getDbUser() ) );

    // update the conditions (updated via delete/insert)
    sb.append( ConditionWriter.getUpdateSQL("oadraft.edgeCond",
                                            "edgeInd",
                                            e.getInd(),
                                            e.getConditions(),
                                            e.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to delete the Edge from the database.
  **/
  static String getDeleteSQL(Edge e) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);


    // delete the countries
    sb.append( CountryWriter.getDeleteSQL( "oadraft.edgeCountry",
                                           "edgeInd",
                                            e.getInd() ) );

    // delete the conditions
    sb.append( ConditionWriter.getDeleteSQL( "oadraft.edgeCond",
                                             "edgeInd",
                                             e.getInd() ) );

    // delete the preanswers
    sb.append("DELETE FROM oadraft.edgePreanswer where edgeInd=")
      .append( e.getInd() ).append(SqlRunner.END_DELIM);

    // finally, delete the edge itself
    sb.append("DELETE FROM oadraft.edge WHERE edgeInd=")
      .append( e.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Generate SQL String to write EdgePreanswer, EdgeUnanswer, EdgeAlist
   * and EdgeReturn objects to the edgePreanswer table.  These four objects
   * can only be inserted or deleted - they cannot be updated.
  **/
  static String getPreanswerTableSQL(Edge e) {
    StringBuffer sb = new StringBuffer();
    Vector v;

    //---------------------------------------------------------------------
    // preanswer types:
    // 0 - "normal" preanswer (conclusion)
    //     if this edge is followed, then preanswer another answer
    //     in the answer history
    // 1 - action to be added to the action list
    // 2 - unanswer (NOANSWER)
    //     if this edge is followed, then remove the previous answer
    //     from the answer history
    // 3 - return node from symptom jump (symptom subroutine)
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // preanswer type 0:
    //---------------------------------------------------------------------
    v = e.getPreanswers();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        EdgePreanswer preans = (EdgePreanswer) v.elementAt(i);
        int recStatus = preans.getRecStatus();

        if ( recStatus == PersistentRec.REC_DELETE ) {

          sb.append( deleteFromPreanswerTable( preans.getInd() ) );

        } else if ( recStatus == PersistentRec.REC_INSERT ) {

          String sql = createPreanswerSQL( EdgeObject.EDGE_PREANSWER,
                                           preans.getActiveAnswerInd(),
                                           preans.getDraftAnswerInd() );

          sb.append( insertIntoPreanswerTable(e, preans.getInd(), sql) );

        }
      }
    }

    //---------------------------------------------------------------------
    // preanswer type 1:
    //---------------------------------------------------------------------
    v = e.getActionList();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        EdgeAlist alist = (EdgeAlist) v.elementAt(i);
        int recStatus = alist.getRecStatus();

        if ( recStatus == PersistentRec.REC_DELETE ) {

          sb.append( deleteFromPreanswerTable( alist.getInd() ) );

        } else if ( recStatus == PersistentRec.REC_INSERT ) {

          String sql = createPreanswerSQL( EdgeObject.EDGE_ALIST,
                                           alist.getActiveActionInd(),
                                           alist.getDraftActionInd() );

          sb.append( insertIntoPreanswerTable(e, alist.getInd(), sql) );

        }
      }
    }

    //---------------------------------------------------------------------
    // preanswer type 2:
    //---------------------------------------------------------------------
    v = e.getUnanswers();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        EdgeUnanswer unans = (EdgeUnanswer) v.elementAt(i);
        int recStatus = unans.getRecStatus();

        if ( recStatus == PersistentRec.REC_DELETE ) {

          sb.append( deleteFromPreanswerTable( unans.getInd() ) );

        } else if ( recStatus == PersistentRec.REC_INSERT ) {

          String sql = createPreanswerSQL( EdgeObject.EDGE_UNANSWER,
                                           unans.getActiveQuestionInd(),
                                           unans.getDraftQuestionInd() );

          sb.append( insertIntoPreanswerTable(e, unans.getInd(), sql) );

        }
      }
    }

    //---------------------------------------------------------------------
    // preanswer type 3:
    //---------------------------------------------------------------------
    EdgeReturn eret = e.getReturnNode();
    // check the original return node against the current return node to
    // see if the original has been changed and needs to be deleted:
    if (e.originalReturnNode != null && e.originalReturnNode != eret) {
      sb.append( deleteFromPreanswerTable( e.originalReturnNode.getInd() ) );
    }
    if (eret != null) {
      int recStatus = eret.getRecStatus();

      if ( recStatus == PersistentRec.REC_DELETE ) {

        sb.append( deleteFromPreanswerTable( eret.getInd() ) );

      } else if ( recStatus == PersistentRec.REC_INSERT ) {

        String sql = createPreanswerSQL( EdgeObject.EDGE_RETURN,
                                         eret.getNodeInd(), 0 );

        sb.append( insertIntoPreanswerTable(e, eret.getInd(), sql) );

      }
    }

    return sb.toString();
  }

  /**
   * Create the SQL string to be passed to the insertIntoPreanswerTable()
   * method.  Will generate the proper values for the active/draft indicators
   * for the given preanswer type.
  **/
  private static String createPreanswerSQL(short type, 
                                           int activeInd, int draftInd)
  {
    String retSQL = type + ",";

    // if indicator is 0, then insert NULL
    String activeStr = (activeInd != 0) ? activeInd + "," : "NULL,";
    String draftStr  =  (draftInd != 0) ? draftInd  + "," : "NULL,";

    // generate the sql for the activeAnswerInd, draftAnswerInd, 
    // activeActionInd, draftActionInd, activeQuestionInd, draftQuestionInd
    // and nodeInd columns.
    switch (type) {
      case EdgeObject.EDGE_PREANSWER:
        retSQL += activeStr + draftStr + "NULL,NULL,NULL,NULL,NULL,";
        break;
      case EdgeObject.EDGE_ALIST:
        retSQL += "NULL,NULL," + activeStr + draftStr + "NULL,NULL,NULL,";
        break;
      case EdgeObject.EDGE_UNANSWER:
        retSQL += "NULL,NULL,NULL,NULL," + activeStr + draftStr + "NULL,";
        break;
      case EdgeObject.EDGE_RETURN:
        // there is no draftNodeInd....
        retSQL += "NULL,NULL,NULL,NULL,NULL,NULL," + activeStr;
        break;
    }

    // returned SQL string will have a trailing comma
    return retSQL;
  }

  /**
   * Generate SQL to insert an entry into the edgePreanswer table.
   * @param e    the parent Edge
   * @param ind  the database indicator for the new entry
   * @param sql  a String containing the values to be inserted into the
   *             preanswerType, activeAnswerInd, draftAnswerInd,
   *             activeActionInd, draftActionInd, activeQuestionInd,
   *             draftQuestionInd and nodeInd columns.
  **/
  private static String insertIntoPreanswerTable(Edge e, int ind, String sql)
  {
    String dbUser = DatabaseUtil.stringToDB( e.getDbUser() );

    return "INSERT INTO oadraft.edgePreanswer VALUES (" +
            ind                 + "," +     // edgePreanswerInd
            e.getInd()          + "," +     // edgeInd
            sql                 +           // active/draft indicators
            dbUser              + "," +     // dbuser
            "CURRENT TIMESTAMP" + ")" +     // changedTime
            SqlRunner.END_DELIM;
           
  }

  /**
   * Generate SQL to delete an entry from the edgePreanswer table.
   * @param edgePreanswerInd  database key for the entry to be deleted
  **/
  private static String deleteFromPreanswerTable(int edgePreanswerInd) {
    return "DELETE FROM oadraft.edgePreanswer WHERE edgePreanswerInd=" +
           edgePreanswerInd + SqlRunner.END_DELIM;
  }

  /**
   * Return SQL to delete all the Edges and Preanswers for a Symptom.
  **/
  static String getDeleteAllForSymptomSQL(int symptomInd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from oadraft.edgeCond where edgeind in " +
        "(select distinct edgeind from oadraft.edge e, oadraft.edgegroup g " +
           "where e.edgegroupind = g.edgegroupind " +
           "and   g.symptomind = "
    ).append(symptomInd).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from oadraft.edgeCountry where edgeind in " +
        "(select distinct edgeind from oadraft.edge e, oadraft.edgegroup g " +
           "where e.edgegroupind = g.edgegroupind " +
           "and   g.symptomind = "
    ).append(symptomInd).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from oadraft.edgePreanswer where edgeind in " +
        "(select distinct edgeind from oadraft.edge e, oadraft.edgegroup g " +
           "where e.edgegroupind = g.edgegroupind " +
           "and   g.symptomind = "
    ).append(symptomInd).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from oadraft.edge where edgegroupind in " +
        "(select distinct edgegroupind from oadraft.edgegroup where symptomind="
    ).append(symptomInd).append(')').append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the Edges and Preanswers for an EdgeGroup.
  **/
  static String getDeleteAllForEdgeGroupSQL(int edgeGroupInd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from oadraft.edgeCond where edgeind in " +
        "(select distinct edgeind from oadraft.edgegroup where edgegroupind="
    ).append(edgeGroupInd).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from oadraft.edgeCountry where edgeind in " +
        "(select distinct edgeind from oadraft.edgegroup where edgegroupind="
    ).append(edgeGroupInd).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from oadraft.edgePreanswer where edgeind in " +
        "(select distinct edgeind from oadraft.edgegroup where edgegroupind="
    ).append(edgeGroupInd).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from oadraft.edge where edgegroupind = "
    ).append(edgeGroupInd).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
