
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;


/**
 * An EdgeGroup represents one or more Edges between two Nodes in a
 * Symptom logic flow.  
 * The EdgeGroup contains the list of Edges in the EdgeGroup and
 * the head and tail Nodes for the Edges in the group.
 * <p>
 * EdgeGroups exist for the benefit of the Online Assistant Symptom
 * Editor which allows graphical editing of the problem diagnosis
 * logic flow.  It allows the position of multiple Edges between
 * two Nodes to be stored as one unit.
 *
 * @version 1.20
 * @author  Mark DeBiase
**/
public class EdgeGroup extends PersistentRec {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  private int     edgeGroupInd_;
  private int     symptomInd_;
  private Node    headNode_;
  private Node    tailNode_;
  private Vector  edges_;
  // GUI data
  private int     xPos_   = 0;
  private int     yPos_   = 0;
  private int     cxPos_  = 0;
  private int     cyPos_  = 0;
  private int     zOrder_ = 0;

  private String  dbUser_      = null;
  private String  changedTime_ = null;

  /**
   * New EdgeGroup.
   * @param headNode the head Node -- this Node must point to a Question
   * @param tailNode the tail Node
   * @exception IllegalArgumentException if headNode is not a Question Node.
  **/
  public EdgeGroup(Node headNode, Node tailNode) {
    super();

    if (headNode.getNodeType() != Node.NODE_TYPE_QUESTION) {
      throw new IllegalArgumentException("Invalid headNode -- " +
                                         "it is not a Question Node");
    }

    headNode_     = headNode;
    tailNode_     = tailNode;

    updateRecStatus(REC_INSERT);
  }

  /**
   * EdgeGroup read from the database.
  **/
  EdgeGroup(int edgeGroupInd, Node headNode, Node tailNode) {
    edgeGroupInd_ = edgeGroupInd;
    headNode_     = headNode;
    tailNode_     = tailNode;
  }

  /**
   * Gets the EdgeGroup indicator.
  **/
  public int getInd() {
    return edgeGroupInd_;
  }

  /**
   * Sets the EdgeGroup indicator.
  **/
  void setInd(int edgeGroupInd) {
    edgeGroupInd_ = edgeGroupInd;
  }

  /**
   * Assign database keys for this object and all objects contained therein.
   * @return true if database keys were assigned, otherwise false
  **/
  boolean assignDatabaseKeys(boolean isDraft) {
    boolean success;

    success = assignDatabaseKey(isDraft);

    if (edges_ != null) {
      for (int i=0; i<edges_.size() && success; i++) {
        Edge e = (Edge) edges_.elementAt(i);
        success = e.assignDatabaseKeys(isDraft);
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  void assignDatabaseKeys(Connection conn, boolean isDraft) throws Exception {

    assignDatabaseKey(conn, isDraft);

    if (edges_ != null) {
      for (int i=0; i<edges_.size(); i++) {
        Edge e = (Edge) edges_.elementAt(i);
        e.assignDatabaseKeys(conn, isDraft);
      }
    }

  }

  /**
   * Assign a database key for this EdgeGroup.
   * @return true if a database key was assigned, otherwise false
  **/
  private boolean assignDatabaseKey(boolean isDraft) {
    boolean success = true;
    String counterName;

    if (isDraft) {
      counterName = "DEDGGRPIND";
    } else {
      counterName = "EDGEGRPIND";
    }

    if (edgeGroupInd_ == 0 && valid()) {
      LogSystem.log(1, "Getting database key for EdgeGroup: " + counterName);
      edgeGroupInd_ = Counter.getCounter(LogSystem.getInstance(), counterName);
      if (edgeGroupInd_ == 0) {
        LogSystem.log(2, "Failed to get database key for EdgeGroup");
        success = false;
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  private void assignDatabaseKey(Connection conn, boolean isDraft) 
               throws Exception 
  {
    String counterName;

    if (isDraft) {
      counterName = "DEDGGRPIND";
    } else {
      counterName = "EDGEGRPIND";
    }

    if (edgeGroupInd_ == 0) {
      edgeGroupInd_ = Counter.getCounter( conn, counterName );
      if (edgeGroupInd_ == 0) {
        throw new Exception("Failed to get database key for EdgeGroup");
      }
    }

  }

  /**
   * Reset values in this object.
   * Sets the indicator to 0, sets the recStatus to REC_INSERT.
  **/
  public void resetNew() {
    setInd(0);
    setSymptomInd(0);
    updateRecStatus(REC_NOCHANGE) ;
    updateRecStatus(REC_INSERT);

    // reset the edges
    if (edges_ != null) {
      for (int i=0; i<edges_.size(); i++) {
        Edge edge = (Edge) edges_.elementAt(i);
        edge.resetNew();
      }
    }
  }

  /**
   * Gets the parent Symptom indicator.
  **/
  int getSymptomInd() {
    return symptomInd_;
  }

  /**
   * Sets the parent Symptom indicator.
  **/
  void setSymptomInd(int symptomInd) {
    symptomInd_ = symptomInd;
  }

  /**
   * Gets the head Node indicator.
  **/
  public int getHeadNodeInd() {
    return headNode_.getInd();
  }

  /**
   * Gets the head Node.
  **/
  public Node getHeadNode() {
    return headNode_;
  }

  /**
   * Gets the tail Node indicator.
  **/
  public int getTailNodeInd() {
    return tailNode_.getInd();
  }

  /**
   * Gets the tail Node.
  **/
  public Node getTailNode() {
    return tailNode_;
  }

  /**
   * Gets the Vector of Edge objects or null if there are no Edges assigned.
  **/
  public Vector getEdges() {
    return edges_;
  }

  /**
   * Sets the Vector of Edge objects or null if there are no Edges assigned.
  **/
  public void setEdges(Vector edges) {
    edges_ = edges;
  }

  /**
   * Returns true if xPos, yPos, cxPos, cyPos and zOrder are not 0.
  **/
  public boolean isGuiDataValid() {
    return (xPos_ != 0 && yPos_ != 0 && 
            cxPos_ != 0 && cyPos_ != 0 && 
            zOrder_ != 0
           );
  }

  /**
   * Get the X coordinate where this EdgeGroup should start.
  **/
  public int getXPos() {
    return xPos_;
  }

  /**
   * Get the Y coordinate where this EdgeGroup should start.
  **/
  public int getYPos() {
    return yPos_;
  }

  /**
   * Get the X coordinate where this EdgeGroup should end.
  **/
  public int getcXPos() {
    return cxPos_;
  }

  /**
   * Get the Y coordinate where this EdgeGroup should end.
  **/
  public int getcYPos() {
    return cyPos_;
  }

  /**
   * Get the Z-order in which this EdgeGroup should be drawn.
  **/
  public int getZOrder() {
    return zOrder_;
  }

  /**
   * Set the GUI data for this node.
  **/
  public void setGuiData(int xPos, int yPos, int cxPos, int cyPos, int zOrder)
  {

    // avoid calling setModified(true) if data has not been changed
    if ( xPos_ != xPos || yPos_ != yPos || cxPos_ != cxPos ||
         cyPos_ != cyPos || zOrder_ != zOrder )
    {
      updateRecStatus(REC_UPDATE);

      xPos_   = xPos;
      yPos_   = yPos;
      cxPos_  = cxPos;
      cyPos_  = cyPos;  
      zOrder_ = zOrder;
    }
  }

  /**
   * Gets the last changed date/time for this object.
  **/
  public String getChangedTime() {
    return changedTime_;
  }

  /**
   * Sets the last changed date/time for this object.
   * Called by Reader classes.
  **/
  void setChangedTime(String changedTime) {
    changedTime_ = changedTime;
  }

  /**
   * Gets the user who last changed this object.
  **/
  public String getDbUser() {
    return dbUser_;
  }

  /**
   * Sets the user who last changed this object.
   * Used by Writer classes - the value from the parent object is used
   * when writing to the database.
  **/
  void setDbUser(String dbUser) {
    dbUser_ = dbUser;
  }

  /**
   * Get the hashCode for this EdgeGroup.  The hashCode is the database key,
   * so this is not valid for EdgeGrups which have not been saved to the
   * database yet.
  **/
  public int hashCode() {
    if ( edgeGroupInd_ == 0 ) {
      throw new IllegalStateException("hashCode(): EdgeGroup is new");
    }
    return this.getInd();
  }

  /**
   * Returns true if this EdgeGroup is the same as another EdgeGroup, based on 
   * database key.  Not valid for EdgeGroup which have not been saved to
   * the database.
  **/
  public boolean equals(Object o) {
    if ( edgeGroupInd_ == 0 ) {
      throw new IllegalStateException("equals(): EdgeGroup is new");
    }
    return ( (o instanceof EdgeGroup) && 
             (((EdgeGroup) o).getInd() == this.getInd()) );
  }

  /**
   * Return a String representation of this object.
  **/
  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("EdgeGroup ").append(edgeGroupInd_).append(": ");
    sb.append(Constants.NL);
    sb.append("\tHead Node = ").append(headNode_.getInd());
    sb.append(Constants.NL);
    sb.append("\tTail Node = ").append(tailNode_.getInd());
    sb.append(Constants.NL);
    sb.append("\tEdge Count = ");
    if ( edges_ != null ) {
      sb.append( edges_.size() );
    } else {
      sb.append( 0 );
    }

    return sb.toString();
  }

  /**
   * Set the recStatus of this object and 
   * all its contained objects to REC_NOCHANGE.
  **/
  void setRecNoChange() {

    // this object
    updateRecStatus(REC_NOCHANGE);

    // contained objects:
    if (edges_ != null) {
      for (int i=0; i<edges_.size(); i++) {
        Edge e = (Edge) edges_.elementAt(i);
        e.setRecNoChange();
      }
    }
  }

  /**
   * Find the EdgeGroup with the specified indicator in a Vector of EdgeGroups.
   * @param  ind  the edgeGroupIndicator to search for
   * @param  v    the Vector of EdgeGroups
   * @return the EdgeGroup object with the specified indicator
   *         or null if it was not found.
  **/
  public static EdgeGroup findEdgeGroupInVector(int ind, Vector v) {
    EdgeGroup foundGroup = null;

    for (int i=0; foundGroup == null && i < v.size(); i++) {
      EdgeGroup group = (EdgeGroup) v.elementAt(i);
      if (group.getInd() == ind) {
        foundGroup = group;
      }
    }

    return foundGroup;
  }
}

