
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;

/**
 * Represents a Preanswer for an Edge.
 * A Preanswer represents a conclusion that can be drawn 
 * based on traversing the Edge in the Symptom logic diagram.
 * A preanswer is similar to an Answer Conclusion --
 * see the AnswerConcl object for an example.
 * 
 * @version 1.10
 * @author  Mark DeBiase
**/

public class EdgePreanswer extends EdgeObject {

  private int activeQuestionInd_ = 0;
  private int activeAnswerInd_   = 0;
  private int draftQuestionInd_  = 0;
  private int draftAnswerInd_    = 0;

  /**
   * New EdgePreanswer object.
   * @param q the Question the preanswer infers.
   * @param a the Answer the preanswer infers.
   * @exception IllegalArgumentException if either the Question's indicator
   *                                     or the Answer's indicator is 0.
  **/
  public EdgePreanswer(Question q, Answer a) {
    if (q instanceof QuestionDraft) {
      QuestionDraft qd   = (QuestionDraft) q;
      activeQuestionInd_ = qd.getActiveQuestionInd();
      draftQuestionInd_  = qd.getInd();
      activeAnswerInd_   = a.getActiveAnswerInd();
      draftAnswerInd_    = a.getInd();
    } else {
      activeQuestionInd_ = q.getInd();
      activeAnswerInd_   = a.getInd();
    }

    if (activeQuestionInd_ == 0 && draftQuestionInd_ == 0) {
      throw new IllegalArgumentException("questionInd is 0");
    }
    if (activeAnswerInd_ == 0 && draftAnswerInd_ == 0) {
      throw new IllegalArgumentException("answerInd is 0");
    }

    updateRecStatus(REC_INSERT);
  }

  /**
   * New EdgePreanswer object based on a <b><i>published</i></b> Question.
   * @param questionInd the indicator of the <i>published</i> Question 
                        the preanswer infers.
   * @param answerInd   the indicator of the <i>published</i> Answer 
                        the preanswer infers.
   * @exception IllegalArgumentException if either questionInd or 
   *                                     answerInd is 0.
   * @deprecated use EdgePreanswer(Question, Answer) constructor instead
  **/
  public EdgePreanswer(int questionInd, int answerInd) {
    super();

    activeAnswerInd_   = answerInd;
    activeQuestionInd_ = questionInd;

    if (activeQuestionInd_ == 0 || activeAnswerInd_ == 0) {
      throw new IllegalArgumentException("questionInd or answerInd is 0");
    }

    updateRecStatus(REC_INSERT);
  }

  /**
   * Constructor used by draft Reader.
  **/
  EdgePreanswer(int activeQuestionInd, int activeAnswerInd,
                int draftQuestionInd,  int draftAnswerInd)
  {
    activeQuestionInd_ = activeQuestionInd;
    activeAnswerInd_   = activeAnswerInd;
    draftQuestionInd_  = draftQuestionInd;
    draftAnswerInd_    = draftAnswerInd;
  }

  /**
   * Gets the Question indicator.
  **/
  public int getQuestionInd() {
    return activeQuestionInd_;
  }

  /**
   * Gets the Answer indicator.
  **/
  public int getAnswerInd() {
    return activeAnswerInd_;
  }

  /**
   * Gets the published Question indicator.
   * Same as getQuestionInd()
  **/
  public int getActiveQuestionInd() {
    return activeQuestionInd_;
  }

  /**
   * Gets the published Answer indicator.
   * Same as getAnswerInd()
  **/
  public int getActiveAnswerInd() {
    return activeAnswerInd_;
  }

  /**
   * Gets the draft Question indicator.
  **/
  public int getDraftQuestionInd() {
    return draftQuestionInd_;
  }

  /**
   * Gets the draft Answer indicator.
  **/
  public int getDraftAnswerInd() {
    return draftAnswerInd_;
  }

  /**
   * Sets the draft Question indicator.
  **/
  // For use by Avalon.
  public void setDraftQuestionInd(int draftQuestionInd) {
    draftQuestionInd_ = draftQuestionInd;
  }

  /**
   * Sets the draft Answer indicator.
  **/
  // For use by Avalon.
  public void setDraftAnswerInd(int draftAnswerInd) {
    draftAnswerInd_ = draftAnswerInd;
  }


  /**
   * Returns a String representation of this object.
  **/
  public String toString() {
    return "EdgePreanswer " + getInd() + ": preanswer=" + 
           activeQuestionInd_ + "/" + draftQuestionInd_ + 
           ":" + activeAnswerInd_ + "/" + draftAnswerInd_;
  }
}

