
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.*;

/**
 * Reader for Online Assistant Edges.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.9
 * @author  Mark DeBiase
**/


class EdgeReader {

  /**
   * Read all Edges in the Symptom.
  **/
  static void readAllEdgesForSymptom(Connection conn, Symptom s, int mode) 
              throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all Edges
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
            "SELECT e.edgeInd, e.edgeType, e.answerInd, e.docClassInd, " +
                   "e.created, e.dbUser, e.changedTime, g.edgeGroupInd " +
            "FROM   oa.edge e, oa.edgeGroup g " +
            "WHERE  e.edgeGroupInd = g.edgeGroupInd " +
            "AND    g.symptomInd =" + s.getInd() + " " +
            "ORDER BY g.edgeGroupInd " +
            "FOR FETCH ONLY"
    );

    int edgeGroupInd = 0;
    int previousEdgeGroupInd = -99;

    Vector edgeVec = null;
    EdgeGroup group = null;

    while( rs.next() ) {

      edgeGroupInd = rs.getInt(8);

      // when we get to a new edgeGroupInd, put the current edgeVec into
      // the appropriate EdgeGroup object.
      if (edgeGroupInd != previousEdgeGroupInd) {
        if (edgeVec != null) {    // if not first time through
          // group still has the EdgeGroup from previous loop iteration
          group.setEdges( edgeVec );
        }
        edgeVec = new Vector(2, 5);
        previousEdgeGroupInd = edgeGroupInd;
        // get the EdgeGroup for the new edgeGroupInd
        group = EdgeGroup.findEdgeGroupInVector( edgeGroupInd, 
                                                 s.getEdgeGroups() );
      }

      Edge edge = new Edge();
      edge.setInd( rs.getInt(1) );
      edge.setEdgeType( rs.getShort(2) );
      edge.setAnswerInd( rs.getInt(3) );
      // assume the head node is an oaobject node for a question:

      edge.setQuestionInd( group.getHeadNode().getOAObjectInd() );
      edge.setEdgeGroupInd( group.getInd() );

      edge.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
      edge.setDbUser( rs.getString(6).trim() );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );

      //---------------------------------------------------------------------
      // Read the default Answer title for the Edge, if not in WEB mode.
      // Note: this method should be optimized to read answer titles for
      //       all edges at once, similar to conditions/countries below
      //---------------------------------------------------------------------
      if (mode != OAMode.WEB) {
        readDefaultAnswerTitle(conn, edge);
      }

      //---------------------------------------------------------------------
      // Read the Edge conditions/countries
      //---------------------------------------------------------------------
      // MAD performance: edge conditions are now read all at once below
      //edge.setConditions(
      //   ConditionReader.readConditions( conn, "oa.edgeCond",
      //                                   "edgeInd", edge )
      //);
      // MAD performance: edge countries are now read all at once below
      //edge.setCountryList(
      //   CountryReader.readCountries( conn, "oa.edgeCountry",
      //                                "edgeInd", edge.getInd() )
      //);

      //---------------------------------------------------------------------
      // Read the edgePreanser table (contains EdgePreanswer, EdgeAlist, 
      // EdgeUnanswer and EdgeReturn objects).
      //---------------------------------------------------------------------
      // MAD performance: edge preanswers are now read all at once below
      //readPreanswerTableForEdge(conn, edge, s);

      edge.updateRecStatus(edge.REC_NOCHANGE);
      edgeVec.addElement( edge );
    }
    
    rs.close();
    stmt.close();
  
    // put the last edgeVec into the appropriate EdgeGroup object
    // edgeVec will be null if no records were selected -- shouldnt happen!
    if (edgeVec != null) {
      // group still has the EdgeGroup from the last loop iteration
      group.setEdges( edgeVec );
    }

    //-----------------------------------------------------------------------
    // Read the Edge conditions/countries for all Edges in the Symptom.
    //-----------------------------------------------------------------------
    ConditionReader.readAllEdgeConditions(conn, s, "oa");
    CountryReader.readAllEdgeCountries(conn, s, "oa");

    //-----------------------------------------------------------------------
    // Read the edgePreanser table (contains EdgePreanswer, EdgeAlist, 
    // EdgeUnanswer and EdgeReturn objects) all Edges in the Symptom.
    //-----------------------------------------------------------------------
    readPreanswerTableForAllEdges(conn, s);
  }

  /**
   * Read Symptom Edges.
   * Note this method is obsolete -- readAllEdgesForSymptom() is used instead.
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readEdgesForEdgeGroup(Connection conn, EdgeGroup eg, Symptom s) 
              throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all Edges
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "edgeInd, edgeType, answerInd, docClassInd, " +
         "created, dbUser, changedTime " +
       "FROM oa.edge " +
       "WHERE edgeGroupInd=" + eg.getInd() + " " +
       "FOR FETCH ONLY"
    );

    Vector v = new Vector();
    while( rs.next() )
    {
      //int edgeInd     = rs.getInt(1);
      //int answerInd   = rs.getInt(2);
      //int questionInd = eg.getHeadNode().getOAObjectInd();
      //Edge edge = new Edge(edgeInd, questionInd, answerInd);

      Edge edge = new Edge();
      edge.setInd( rs.getInt(1) );
      edge.setEdgeType( rs.getShort(2) );
      edge.setAnswerInd( rs.getInt(3) );
      // assume the head node is an oaobject node for a question:
      edge.setQuestionInd( eg.getHeadNode().getOAObjectInd() );
      edge.setEdgeGroupInd( eg.getInd() );

      edge.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
      edge.setDbUser( rs.getString(6).trim() );
      edge.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );

      //---------------------------------------------------------------------
      // Read the default Answer title for the Edge
      //---------------------------------------------------------------------
      readDefaultAnswerTitle(conn, edge);

      //---------------------------------------------------------------------
      // Read the Edge conditions/countries
      //---------------------------------------------------------------------
      edge.setConditions(
         ConditionReader.readConditions( conn, "oa.edgeCond",
                                         "edgeInd", edge )
      );
      edge.setCountryList(
         CountryReader.readCountries( conn, "oa.edgeCountry",
                                      "edgeInd", edge.getInd() )
      );

      //---------------------------------------------------------------------
      // Read the edgePreanser table (contains EdgePreanswer, EdgeAlist, 
      // EdgeUnasnwer and EdgeReturn objects).
      //---------------------------------------------------------------------
      readPreanswerTableForEdge(conn, edge, s);

      edge.updateRecStatus(edge.REC_NOCHANGE);
      v.addElement( edge );
    }
    
    rs.close();
    stmt.close();
  
    eg.setEdges(v);
  }
------------------------- END COMMENTED OUT ----------------------------- */

  /**
   * Read the default title for the Answer represented by this edge.
  **/
  // Note: this method should be optimized to read the answer titles
  //       for all edges at once.  but since it is not called when
  //       OAMode == WEB, it only impacts batch jobs & QIT.
  private static void readDefaultAnswerTitle(Connection conn, Edge edge)
                      throws SQLException
  {
    if (edge.getEdgeType() == Edge.EDGE_TYPE_OTHER) {
      // the edge represents multiple answers....
      edge.setDefaultAnswerTitle("OTHER");
    } else {
      Statement stmt = conn.createStatement();
      ResultSet rs;

      int answerInd = edge.getAnswerInd();

      rs = stmt.executeQuery(
           "SELECT title " +
           "FROM oa.answerTitle " +
           "WHERE answerInd=" + answerInd + " " +
           "AND sortOrder=(" +
                   "SELECT MAX(sortOrder) " +
                   "FROM oa.answerTitle " +
                   "WHERE answerInd=" + answerInd + 
           ") " +
           "FOR FETCH ONLY"
      );

      if ( rs.next() ) {
        edge.setDefaultAnswerTitle( rs.getString(1) );
      } else {
        edge.setDefaultAnswerTitle( "N/A" );
      }
  
      rs.close();
      stmt.close();
    }
  }

  /**
   * Read EdgePreanswer, EdgeAlist, EdgeUnasnswer and EdgeReturn objects
   * from the edgePreanswer table for all Edges in the Symptom.
  **/
  private static void readPreanswerTableForAllEdges(Connection conn, Symptom s)
          throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;
    PreparedStatement pstmt = null;

    //---------------------------------------------------------------------
    // Read all Edge preanswers, unanswers, alist and return objects
    // from the edgePreanswer table for this Symptom.
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "ep.edgePreanswerInd, ep.preanswerType, " +
         "ep.answerInd, ep.actionInd, ep.questionInd, ep.nodeInd, " +
         "ep.edgeInd " +
       "FROM oa.edgePreanswer ep, oa.edge e, oa.edgeGroup eg " +
       "WHERE ep.edgeInd = e.edgeInd " +
       " AND  e.edgeGroupInd = eg.edgeGroupInd " +
       " AND  eg.symptomInd = " + s.getInd() + " " +
       "ORDER BY ep.edgeInd " +
       "FOR FETCH ONLY"
    );

    while( rs.next() )
    {
      // get the edge this database row belongs to:
      Edge edge = s.getEdgeByInd( rs.getInt(7) );

      // read the common values....
      int    edgePreanswerInd = rs.getInt(1);
      short  preanswerType    = rs.getShort(2);

      switch (preanswerType) {

        case EdgeObject.EDGE_PREANSWER:
          {
            // need to get the question this answer belongs to
            if (pstmt == null) {
              // dont prepare the stmt until we actully need it.
              pstmt = conn.prepareStatement(
                                  "SELECT questionInd " + 
                                  "FROM oa.answer " +
                                  "WHERE answerInd=? " +
                                  "FOR FETCH ONLY"
                      );
            }
            int answerInd = rs.getInt(3);
            pstmt.setInt(1, answerInd);

            ResultSet rs2 = pstmt.executeQuery();
            rs2.next();
            int questionInd = rs2.getInt(1);
            rs2.close();

            EdgePreanswer preans = new EdgePreanswer(questionInd, answerInd);
            preans.setInd( edgePreanswerInd );
            preans.setEdgeInd( edge.getInd() );
            preans.updateRecStatus( preans.REC_NOCHANGE );

            edge.addPreanswer( preans );
          }
          break;

        case EdgeObject.EDGE_ALIST:
          {
            int actionInd = rs.getInt(4);

            EdgeAlist alist = new EdgeAlist( actionInd );
            alist.setInd( edgePreanswerInd );
            alist.setEdgeInd( edge.getInd() );
            alist.updateRecStatus( alist.REC_NOCHANGE );

            edge.addActionList( alist );
          }
          break;

        case EdgeObject.EDGE_UNANSWER:
          {
            int questionInd = rs.getInt(5);

            EdgeUnanswer unans = new EdgeUnanswer( questionInd );
            unans.setInd( edgePreanswerInd );
            unans.setEdgeInd( edge.getInd() );
            unans.updateRecStatus( unans.REC_NOCHANGE );
 
            edge.addUnanswer( unans );
          }
          break;

        case EdgeObject.EDGE_RETURN:
          {
            int nodeInd = rs.getInt(6);

            EdgeReturn ereturn = new EdgeReturn( s.getNodeByInd(nodeInd) );
            ereturn.setInd( edgePreanswerInd );
            ereturn.setEdgeInd( edge.getInd() );
            ereturn.updateRecStatus( ereturn.REC_NOCHANGE );

            // we also have to save the return node that was read from 
            // the database so we can tell if it has been changed so 
            // we can delete it from the database.
            edge.setReturnNode( ereturn );
            edge.originalReturnNode = ereturn;
          }
          break;

        default:
          // unknown preanswerType
          break;
      }
    }

    rs.close();
    stmt.close();
    if (pstmt != null) {
      pstmt.close();
    }
  }

  /**
   * Read EdgePreanswer, EdgeAlist, EdgeUnasnswer and EdgeReturn objects
   * from the edgePreanswer table.
   * Note: this method is obsolete -- readPreanaswerTableForAllEdges() is
   * now used instead.
  **/
/* ---------------------- START COMMENTED OUT ------------------------------
  private static void readPreanswerTableForEdge(Connection conn, Edge edge,
                                                Symptom s)
          throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all Edge preanswers, unanswers, alist and return objects
    // from the edgePreanswer table.
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "edgePreanswerInd, preanswerType, " +
         "answerInd, actionInd, questionInd, nodeInd, " +
         "dbUser, changedTime " +
       "FROM oa.edgePreanswer " +
       "WHERE edgeInd=" + edge.getInd() + " " +
       "FOR FETCH ONLY"
    );

    Vector preanswerVec = new Vector(1,3);
    Vector alistVec     = new Vector(1,3);
    Vector unanswerVec  = new Vector(1,3);
    EdgeReturn ereturn  = null;

    while( rs.next() )
    {
      // read the common values....
      int    edgePreanswerInd = rs.getInt(1);
      short  preanswerType    = rs.getShort(2);
      String dbUser           = rs.getString(7).trim();
      String changedTime      = DatabaseUtil.timestampFromDB( rs.getString(8) );

      switch (preanswerType) {

        case EdgeObject.EDGE_PREANSWER:
          {
            int answerInd = rs.getInt(3);
            // need to get the question this answer belongs to
            Statement stmt2 = conn.createStatement();
            ResultSet rs2;
            rs2 = stmt2.executeQuery( "SELECT questionInd " + 
                                      "FROM oa.answer " +
                                      "WHERE answerInd=" + answerInd + " " +
                                      "FOR FETCH ONLY"  );
            rs2.next();
            int questionInd = rs2.getInt(1);
            rs2.close();
            stmt2.close();

            EdgePreanswer preans = new EdgePreanswer(questionInd, answerInd);
            preans.setInd( edgePreanswerInd );
            preans.setEdgeInd( edge.getInd() );
            preans.updateRecStatus( preans.REC_NOCHANGE );

            preanswerVec.addElement( preans );
          }
          break;

        case EdgeObject.EDGE_ALIST:
          {
            int actionInd = rs.getInt(4);

            EdgeAlist alist = new EdgeAlist( actionInd );
            alist.setInd( edgePreanswerInd );
            alist.setEdgeInd( edge.getInd() );
            alist.updateRecStatus( alist.REC_NOCHANGE );

            alistVec.addElement( alist );
          }
          break;

        case EdgeObject.EDGE_UNANSWER:
          {
            int questionInd = rs.getInt(5);

            EdgeUnanswer unans = new EdgeUnanswer( questionInd );
            unans.setInd( edgePreanswerInd );
            unans.setEdgeInd( edge.getInd() );
            unans.updateRecStatus( unans.REC_NOCHANGE );
 
            unanswerVec.addElement( unans );
          }
          break;

        case EdgeObject.EDGE_RETURN:
          {
            int nodeInd = rs.getInt(6);

            ereturn = new EdgeReturn( s.getNodeByInd(nodeInd) );
            ereturn.setInd( edgePreanswerInd );
            ereturn.setEdgeInd( edge.getInd() );
            ereturn.updateRecStatus( ereturn.REC_NOCHANGE );
          }
          break;

        default:
          // unknown preanswerType
          break;
      }
    }

    edge.setPreanswers( preanswerVec );
    edge.setActionList( alistVec );
    edge.setUnanswers( unanswerVec );

    edge.setReturnNode( ereturn );        // set to null if there is none
    edge.originalReturnNode = ereturn;    // need to save the return node
                                          // read from the db so it can be
                                          // deleted if it is changed
    rs.close();
    stmt.close();
  }
------------------------- END COMMENTED OUT ----------------------------- */
}
