
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.db.*;
import java.sql.*;
import java.util.Vector;

/**
 * Online Assistant Document Link.
 * A Link can refer to a Quest document or to an external URL.
 *
 * @see QuestLink
 * @see ExternalLink
 *
 * @version 1.23
 * @author  Mark DeBiase
**/
public abstract class Link extends PersistentRec implements OAInterface {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /* If you add fields to this object, the serialVersionUID    */
  /* of all subclasses MUST BE UPDATED.                        */
  /*                                                           */
  /*-----------------------------------------------------------*/

  protected int      linkInd_        = 0;
  protected Vector   titles_         = null;  // Vector of LanguageText objects
  protected boolean  archived_       = false;
  protected String   creationTime_   = "";
  protected String   dbuser_         = "";
  protected String   changedTime_    = "";
  private boolean    openDraft_;              // true if there is an open draft

  // these values correspond to the link types in the database
  static final short LINK_TYPE_QUEST    = 0;
  static final short LINK_TYPE_EXTERNAL = 1;

  /**
   * Return the URL string for this Link.
   * For an ExternalLlink this will be the complete URL.
   * For a QuestLink this will include a substution pattern that must
   * be replaced with the path appropriate to the web site it will be
   * displayed on.
   * @see ExternalLink#getUrlString
   * @see QuestLink#getUrlString
  **/
  public abstract String getUrlString();

  /**
   * New Link object.
  **/
  public Link() {
    updateRecStatus(REC_INSERT);
  }

  /**
   * New Link object with specified title.
   * <p>
   * This constructor will create a LanguageText object with
   * a TypeLanguageRec of English for the title and add it
   * to the Vector of LanguageText objects.
  **/
  public Link(String title) {
    linkInd_   = 0;
    if (titles_ == null) {
      titles_ = new Vector();
    }
    titles_.addElement( new LanguageText(title) );
    updateRecStatus(REC_INSERT);
  }

  /**
   * Link to be read from the database.
  **/
  public Link(int linkInd) {
    linkInd_ = linkInd;
  }

  /**
   * Initialize from an existing Link object.
   * Called by LinkDraft(Link) constructor [where LinkDraft is an
   * instance of either QuestLinkDraft or ExternalLinkDraft].
   * Used by the OAReader stored procedure to create a new draft Link
   * from a published Link.
  **/
  Link(Link l) {
    setInd(0);

    initFromOtherLink(l);

    updateRecStatus(REC_INSERT);
  }

  /**
   * Initialize from an existing LinkDraft object.
   * Used when publishing a draft Link.
  **/
  Link(LinkDraft ld) {
    setInd( ld.getActiveLinkInd() );

    initFromOtherLink( ld.asLink() );
    
    if (linkInd_ == 0) {
      updateRecStatus(REC_INSERT);
    } else {
      updateRecStatus(REC_UPDATE);
    }
  }

  /**
   * Initialize this Link object from another Link object.
   * Sets the recStatus on all Link titles to REC_INSERT.
  **/
  private void initFromOtherLink(Link otherLink) {
    // NOTE: this initialization is not safe in that Vectors are
    //       shared between the new object and the original object

    this.titles_       = otherLink.titles_;
    this.archived_     = otherLink.archived_;
    this.dbuser_       = otherLink.dbuser_;
    // these values are reset for the new object:
    this.creationTime_ = "";
    this.changedTime_  = "";

    if (titles_ != null) {
      for (int i=0; i<titles_.size(); i++) {
        LanguageText txt = (LanguageText) titles_.elementAt(i);
        txt.updateRecStatus(REC_INSERT);
      }
    }
  }

  /**
   * Assign a database key for this Link.
   * @return true if a database key was assigned, otherwise false
  **/
  public boolean assignDatabaseKeys() {
    boolean success = true;
    boolean draft   = false;

    if (this instanceof LinkDraft) {
      draft = true;
    }

    if (linkInd_ == 0 && valid()) {
      if (draft) {
        LogSystem.log(1, "Getting database key for LinkDraft");
        linkInd_ = Counter.getCounter( LogSystem.getInstance(), "DLINKIND" );
      } else {
        LogSystem.log(1, "Getting database key for Link");
        linkInd_ = Counter.getCounter( LogSystem.getInstance(), "LINKIND" );
      }
      if (linkInd_ == 0) {
        LogSystem.log(2, "Failed to get database key for Link");
        success = false;
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  void assignDatabaseKeys(Connection conn) throws Exception {
    boolean draft   = false;

    if (this instanceof LinkDraft) {
      draft = true;
    }

    if (linkInd_ == 0) {
      if (draft) {
        linkInd_ = Counter.getCounter( conn, "DLINKIND" );
      } else {
        linkInd_ = Counter.getCounter( conn, "LINKIND" );
      }
      if (linkInd_ == 0) {
        throw new Exception("Failed to get database key for Link");
      }
    }
  }

  /**
   * Get this Link's database indicator.
  **/
  public int getInd() {
    return linkInd_;
  }
  /**
   * Set this Link's database indicator.
  **/
  void setInd(int linkInd) {
    linkInd_ = linkInd;
  }

  /**
   * Return the list of titles for this Link.
   * @return a Vector of LanguageText objects
  **/
  public Vector getLinkTitles() {
    return titles_;
  }
  /**
   * Set the list of titles for this Link.
   * @param titles a Vector of LanguageText objects
  **/
  public void setLinkTitles(Vector titles) {
    titles_ = titles;
  }

  /**
   * Returns true if this Link is archived.
  **/
  public boolean isArchived() {
    return archived_;
  }
  /**
   * Set the Link's archive state.
  **/
  public void setArchived(boolean archived) {
    archived_ = archived;
    updateRecStatus(REC_UPDATE);
  }

  /**
   * Returns the creation time in DB2 timestamp format.
  **/
  public String getCreationTime() {
    return creationTime_;
  }
  /**
   * Sets the creation time.
   * <b>Note:</b> The creationTime value may be set, but it is not
   * used when the object is written to the database.
   * @param creationTime a String in DB2 timestamp format.
  **/
  public void setCreationTime(String creationTime) {
    creationTime_ = creationTime;
  }

  /**
   * Gets the user who last updated this object.
  **/
  public String getDbUser() {
    return dbuser_;
  }
  /**
   * Sets the user who last updated this object.
  **/
  public void setDbUser(String dbuser) {
    dbuser_ = dbuser.toUpperCase();
    //updateRecStatus(REC_UPDATE);
  }

  /**
   * Returns the changed time in DB2 timestamp format.
  **/
  public String getChangedTime() {
    return changedTime_;
  }
  /**
   * Sets the changed time.
   * <b>Note:</b> The changedTime value may be set, but it is not
   * used when the object is written to the database.
   * @param creationTime a String in DB2 timestamp format.
  **/
  public void setChangedTime(String changedTime) {
    changedTime_ = changedTime;
  }

  /**
   * Returns true if there is an open draft of this object.
   * This method is only valid for Link objects, not
   * valid for LinkDraft objects.
  **/
  public boolean hasOpenDraft() {
    return openDraft_;
  }

  /**
   * Set the open draft flag for this object.
  **/
  void setOpenDraft(boolean openDraft) {
    openDraft_ = openDraft;
  }

  /**
   * Write the Link object to the database.
   * This method assumes that a DatabaseSystem object for
   * the DatabaseSystem.QUEST database has already been connected.
   * 
   * @return 0 for success, nonzero for failure.
  **/
  public int writeToDatabase() {
    LogSystem.log(2, ">>> Starting Link.writeToDatabase()");

    /*--------------------------------------------------------------*/
    /*   Assign database keys for this Link and its subobjects.     */
    /*--------------------------------------------------------------*/
    if ( !assignDatabaseKeys() ) {
      return -1;
    }

    SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                    "Link.writeToDatabase()", 5 );
    int returnCode = 0 ;
    StoredProcRec retRec = null;

    try {
      CallableStatement stmt = sqlm.createStoredProc ( "QUEST.sqlRunner", 2 );
      String sqlToRun = LinkWriter.getSQL(this);
      if (sqlToRun.length() == 0) {
        LogSystem.log(1, "No changes to be written");
        return 0;
      }
      String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                    SqlRunner.END_DELIM, "\n" );

      LogSystem.log ( 1, "Starting to send statements to SQLRUNNER" ) ;
      LogSystem.log ( 1, debugString ) ;

      /*-----------------------------------------------------------*/
      /*   Set up the Callable Stmt                                */
      /*-----------------------------------------------------------*/
      stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
      stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
      stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      

      /*-----------------------------------------------------------*/
      /*   Execute the SQL                                         */
      /*-----------------------------------------------------------*/
      stmt.execute() ;

      /*-----------------------------------------------------------*/
      /*   Get the StoredProcRec from the Lob                      */
      /*-----------------------------------------------------------*/
      retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 2 );
      stmt.close() ;

      if (retRec == null) {
        LogSystem.log( 2, "StoredProcRec is null - error status unknown");
      }

      if ( ( retRec != null ) && ( retRec.error ) ) {
        LogSystem.log( 2, retRec.errorStr ) ;
        returnCode = retRec.sqlCode ;
        if (returnCode == 0) returnCode = -1;
      }
    } 
    catch ( Exception e ) {
       sqlm.rollBack() ;
       LogSystem.log ( 3, e ) ;
       returnCode = -1 ;
    }
    finally {
       sqlm.close() ;
    }

    LogSystem.log(2, ">>> Link.writeToDatabase() complete");

    return returnCode;
  }

  /**
   * Read a Link object from the database.
   * This method is called from the QuestLink and ExternalLink objects.
   * 
   * @param linkInd     the database indicator of the Link to be read.
   * @param languageInd the language to read the text items for 
   *                    (0 = read all languages).
   * @return the Link object or null if an error occured or
   *         the object was not found.
  **/
  public static Link readFromDatabase(int linkInd, int languageInd) {
    LogSystem.log(2, ">>> Starting Link.readFromDatabase()");
    SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                   "Link.readFromDatabase()", 5 );
    StoredProcRec retRec    = null;
    Link          retLink   = null;

    try {
      CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

      /*-----------------------------------------------------------*/
      /*   Set up the Callable Stmt                                */
      /*-----------------------------------------------------------*/
      stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
      stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
      stmt.setInt( 1, linkInd );                // object ind
      stmt.setInt( 2, OAReader.LINK );          // object type
      stmt.setInt( 3, OAReader.READ_ACTIVE );   // read active object
      stmt.setInt( 4, languageInd );            // language ind
      stmt.setInt( 5, OAMode.getMode() );       // current mode

      /*-----------------------------------------------------------*/
      /*   Execute the SQL                                         */
      /*-----------------------------------------------------------*/
      stmt.execute() ;

      /*-----------------------------------------------------------*/
      /*   Get the Link and the StoredProcReturnRec objects        */
      /*   from the returned LOBs                                  */
      /*-----------------------------------------------------------*/
      retLink = (Link)StoredProcUtil.getObjectFromStatement( stmt, 6 );
      retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
      stmt.close() ;

      if (retRec == null) {
        LogSystem.log( 2, "StoredProcRec is null - error status unknown");
      }

      if ( ( retRec != null ) && ( retRec.error ) ) {
        LogSystem.log( 2, retRec.errorStr ) ;
      } else {
        if ( retLink != null ) {
          // update the Type*Rec objects
          retLink.updateTypeRecData();
        } else {
          LogSystem.log(2, "No object found for indicator " + linkInd);
        }
      }

    } catch ( Exception e ) {
       sql.rollBack() ;
       LogSystem.log ( 3, e ) ;
    }

    sql.close() ;
    LogSystem.log(2, ">>> Link.readFromDatabase() completed");

    return retLink;
  }

  /**
   * Update the TypeRec objects in the Links's LanguageText objects with 
   * the proper descriptions.
   * Called by readFromDatabase();
  **/
  void updateTypeRecData() {

    // update sub-objects TypeRec data:
    if (titles_ != null) {
      for (int i=0; i<titles_.size(); i++) {
        LanguageText title = (LanguageText) titles_.elementAt(i);
        title.updateTypeRecData();
      }
    }

    return;
  }
}
