
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant draft Link objects.
 * 
 * @version 1.4
 * @author  Mark DeBiase
**/


public class LinkDraftReader {

  /**
   * Read Links.
   * 
   * @param conn    the database Connection object
   * @param linkInd the indicator of the Link to be read
   * @param lang    the language to read the titles for (0 = all languages)
   * 
   * @return a LinkDraft object if the Link was successfully read or 
   *         null if the Link was not found in the database.
  **/
  public static LinkDraft readFromDB(Connection conn, int linkInd, 
                                     int languageInd) 
                          throws SQLException
  {
    LinkDraft  retLinkDraft;
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read the Link
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "l.linkType, l.docInd, d.lndocid, l.url, l.urlValid, " +
         "l.activeLinkInd, l.maintLock, l.startDate, l.stopDate, l.recycled, " +
         "l.archived, l.created, l.dbUser, l.changedTime, l.typeWorkInd, " +
         "l.comments " +
       "FROM " +
         "oadraft.link l LEFT OUTER JOIN tigris.documents d " +
                        "ON l.docInd=d.docInd " +
       "WHERE linkInd=" + linkInd + " " +
       "FOR FETCH ONLY"
    );

    if ( !rs.next() ) {
      retLinkDraft = null;     // linkInd not found
      stmt.close();
    } else {

      //---------------------------------------------------------------------
      // Read info specific to a QuestLink or ExternalLink
      //---------------------------------------------------------------------
      Link lnk;
      short linkType = rs.getShort(1);
      if (linkType == Link.LINK_TYPE_QUEST) {
        QuestLinkDraft qld = new QuestLinkDraft( linkInd );
        qld.setDocInd( rs.getInt(2) );

        String lnDocId = rs.getString(3);
        if (lnDocId != null) {
          qld.setLNDocId( lnDocId.trim() );
        }

        // read draft info: activeLinkInd, maintLock, start/stopDate, recycled
        qld.setActiveLinkInd( rs.getInt(6) );
        qld.setMaintLock( DatabaseUtil.booleanFromDB( rs.getString(7) ) );
        qld.setStartDate( rs.getString(8) );
        qld.setStopDate( rs.getString(9) );
        qld.setRecycled( DatabaseUtil.booleanFromDB( rs.getString(10) ) );

        retLinkDraft = qld;
        lnk = qld.asLink();
      } else {
        ExternalLinkDraft eld = new ExternalLinkDraft( linkInd );
        eld.setUrl( rs.getString(4) );
        eld.setUrlValid( DatabaseUtil.booleanFromDB( rs.getString(5) ) );

        // read draft info: activeLinkInd, maintLock, start/stopDate, recycled
        eld.setActiveLinkInd( rs.getInt(6) );
        eld.setMaintLock( DatabaseUtil.booleanFromDB( rs.getString(7) ) );
        eld.setStartDate( rs.getString(8) );
        eld.setStopDate( rs.getString(9) );
        eld.setRecycled( DatabaseUtil.booleanFromDB( rs.getString(10) ) );

        retLinkDraft = eld;
        lnk = eld.asLink();
      }

      //---------------------------------------------------------------------
      // Read info common to both types of Link
      //---------------------------------------------------------------------
      lnk.setArchived( DatabaseUtil.booleanFromDB( rs.getString(11) ) );
      lnk.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(12) ) );
      lnk.setDbUser( rs.getString(13).trim() );
      lnk.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(14) ) );
      retLinkDraft.setTypeWorkRequired(new TypeWorkRec( rs.getShort(15), "" ));
      retLinkDraft.setComments( rs.getString(16) );
      if ( rs.wasNull() ) {
        retLinkDraft.setComments("");
      }

      stmt.close();

      //---------------------------------------------------------------------
      // Read Link Titles
      //---------------------------------------------------------------------
      if (languageInd == 0) {
        readLinkTitles(conn, lnk);
      } else {
        readLinkTitles(conn, lnk, languageInd);
      }

      lnk.updateRecStatus(lnk.REC_NOCHANGE);
    }

    return retLinkDraft;
  }

  /**
   * Read Link Title for specific language.
   * Returns the English title if requested language is not found.
  **/
  static void readLinkTitles(Connection conn, Link lnk, int languageInd)
         throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String selectStmt = "SELECT " + 
                          "languageInd, lastTranslated, title, " +
                          "dbUser, changedTime " +
                        "FROM oadraft.linkTitle " + 
                        "WHERE linkInd=" + lnk.getInd();

    String titleSql = selectStmt;
    titleSql += " AND languageInd=" + languageInd + " FOR FETCH ONLY";

    // try to get title in requested language
    rs = stmt.executeQuery( titleSql );

    if ( !rs.next() ) {
      // title was not found in requested language -- get English title
      titleSql = selectStmt;
      titleSql += " AND languageInd=1 FOR FETCH ONLY";
      rs = stmt.executeQuery( titleSql );
      rs.next();    // advance cursor to first row
    }

    TypeLanguageRec langRec = new TypeLanguageRec( rs.getShort(1), "" );
    String title = rs.getString(3);
 
    LanguageText langText = new LanguageText( title, langRec );
    langText.setLastTranslationDate( 
                         DatabaseUtil.timestampFromDB(rs.getString(2)) ); 
    langText.setDbUser( rs.getString(4) );
    langText.setChangedTime( DatabaseUtil.timestampFromDB(rs.getString(5)) );

    langText.updateRecStatus(langText.REC_NOCHANGE);
    // return a vector that contains just the requested title
    Vector v = new Vector(1);
    v.addElement(langText);

    rs.close();
    stmt.close();

    lnk.setLinkTitles( v );
  }

  /**
   * Read Link Titles for all languages.
  **/
  static void readLinkTitles(Connection conn, Link lnk)
         throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String selectStmt = "SELECT " + 
                          "languageInd, lastTranslated, title, " +
                          "dbUser, changedTime " +
                        "FROM oadraft.linkTitle " + 
                        "WHERE linkInd=" + lnk.getInd() + " " +
                        "ORDER BY languageInd " +
                        "FOR FETCH ONLY";

    rs = stmt.executeQuery( selectStmt );

    Vector v = new Vector();
    while ( rs.next() ) {
      TypeLanguageRec langRec = new TypeLanguageRec( rs.getShort(1), "" );
      String title = rs.getString(3);
 
      LanguageText langText = new LanguageText( title, langRec );
      langText.setLastTranslationDate( 
                         DatabaseUtil.timestampFromDB(rs.getString(2)) ); 
      langText.setDbUser( rs.getString(4) );
      langText.setChangedTime( 
                         DatabaseUtil.timestampFromDB(rs.getString(5)) );

      langText.updateRecStatus(langText.REC_NOCHANGE);
      v.addElement(langText);
    }
    rs.close();
    stmt.close();

    lnk.setLinkTitles( v );
  }

  /**
   * Get a draft for a Link.
   * Check to see if there is an open draft -- if there is, then return it.
   * If there is no open draft, then create a new one.
   *
   * @exception PendingDraftException if openDraft='Y' on the active link
   *            and there is no open draft with stopDate=NULL
  **/
  public static LinkDraft getDraft(Connection conn, int linkInd, 
                                   int languageInd) 
                          throws SQLException, PendingDraftException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    LinkDraft returnedDraft = null;

    // check to see if there is already an open draft....
    rs = stmt.executeQuery(
      "SELECT linkInd " +
      "FROM oadraft.link " +
      "WHERE stopDate IS NULL AND activeLinkInd=" + linkInd + " " +
      "FOR FETCH ONLY"
    );

    if ( rs.next() ) {
      // there is an open draft -- get it.
      int draftLinkInd = rs.getInt(1);
      rs.close();
      returnedDraft = LinkDraftReader.readFromDB( conn, draftLinkInd, 
                                                  languageInd );
    } else {
      // there is no open draft -- read the active link & create a new draft.
      rs.close();
      Link link = LinkReader.readFromDB( conn, linkInd, languageInd );
      if ( link != null ) {
        if ( link.hasOpenDraft() ) {
          throw new PendingDraftException();
        }
        if (link instanceof QuestLink) {
          QuestLinkDraft qld = new QuestLinkDraft( (QuestLink) link );
          returnedDraft = qld;
        } else {
          ExternalLinkDraft eld = new ExternalLinkDraft( (ExternalLink) link );
          returnedDraft = eld;
        }

        // if we opened a new draft, then update openDraft in oa.action
        stmt.executeUpdate(
          "UPDATE oa.link " +
          "SET openDraft='Y' " + 
          "WHERE linkInd=" + linkInd
        );
      }
    }

    stmt.close();

    return returnedDraft;
  }
}
