
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;

/**
 * Online Assistant Link Group.
 * A Link Group is a collection of Link objects.
 *
 * @see Link
 *
 * @version 1.15
 * @author  Mark DeBiase
**/
public class LinkGroup extends PersistentRec implements OAInterface {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;     

   protected int       linkGroupInd_   = 0;
   protected Vector    links_          = null;  // The Links in this LinkGroup
   protected Vector    titles_         = null;  // Vector of LanguageText objects
   protected boolean   archived_       = false;
   protected String    creationTime_   = "";
   protected String    dbuser_         = "";
   protected String    changedTime_    = "";
   protected boolean   openDraft_      = false; // true if there is an open draft
                                                // of this object

   /**
    * New LinkGroup object with the specified English title.
   **/
   public LinkGroup(String title) {
      linkGroupInd_ = 0;
      if ( titles_ == null ) {
         titles_ = new Vector();
      }
      titles_.addElement( new LanguageText(title) );
      updateRecStatus(REC_INSERT);
   }

   /**
    * LinkGroup to be read from the database.
    *
    * @param linkGroupInd the linkGroup database indicator
   **/
   public LinkGroup(int linkGroupInd) {
      linkGroupInd_ = linkGroupInd;
   }

   /**
    * Initialize from an existing LinkGroup object.
    * Called by LinkGroupDraft(LinkGroup) constructor.
    * Used by the OAReader stored procedure to create a new draft LinkGroup
    * from a published LinkGroup.
   **/
   LinkGroup(LinkGroup lg) {
      setInd(0);

      initFromOtherLinkGroup(lg);

      updateRecStatus(REC_INSERT);
   }

   /**
    * Initialize from an existing LinkGroupDraft object.
    * Used when publishing a draft LinkGroup.
   **/
   LinkGroup(LinkGroupDraft lgd) {
      setInd( lgd.getActiveLinkGroupInd() );

      initFromOtherLinkGroup(lgd);

      if ( linkGroupInd_ == 0 ) {
         updateRecStatus(REC_INSERT);
      } else {
         updateRecStatus(REC_UPDATE);
      }
   }

   /**
    * Initialize this LinkGroup object from another LinkGroup object.
    * Sets the recStatus on all LinkGroup titles to REC_INSERT.
   **/
   private void initFromOtherLinkGroup(LinkGroup otherLinkGroup) {
      // NOTE: this initialization is not safe in that Vectors are
      //       shared between the new object and the original object

      this.links_        = otherLinkGroup.links_;
      this.titles_       = otherLinkGroup.titles_;
      this.archived_     = otherLinkGroup.archived_;
      this.dbuser_       = otherLinkGroup.dbuser_;
      // these values are reset for the new object:
      this.creationTime_ = "";
      this.changedTime_  = "";

      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            LanguageText txt = (LanguageText) titles_.elementAt(i);
            txt.updateRecStatus(REC_INSERT);
         }
      }
   }

   /**
    * Return the list of Links for this LinkGroup.
    * @return a Vector of Link objects
   **/
   public Vector getLinks() {
      return(links_);
   }
   /**
    * Set the list of Links for this LinkGroup.
    * @param links a Vector of Link objects
   **/
   public void setLinks(Vector links) {
      links_ = links;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Return the list of titles for this LinkGroup.
    * @return a Vector of LanguageText objects
   **/
   public Vector getLinkGroupTitles() {
      return(titles_);
   }
   /**
    * Set the list of titles for this LinkGroup.
    * @param titles a Vector of LanguageText objects
   **/
   public void setLinkGroupTitles(Vector titles) {
      titles_ = titles;
   }

   /**
    * Assign a database key for this LinkGroup
    * (but <b>NOT</b> the Links it contains).
    * @return true if a database keys were assigned, otherwise false
   **/
   public boolean assignDatabaseKeys() {
      boolean success = true;

      if ( linkGroupInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for LinkGroup");
         linkGroupInd_ = Counter.getCounter(LogSystem.getInstance(), "LINKGRPIND");
         if ( linkGroupInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for LinkGroup");
            success = false;
         }
      }

      return(success);
   }
   // version to be called by publish stored procedure:
   void assignDatabaseKeys(Connection conn) throws Exception {

      if ( linkGroupInd_ == 0 ) {
         linkGroupInd_ = Counter.getCounter(conn, "LINKGRPIND");
         if ( linkGroupInd_ == 0 ) {
            throw new Exception("Failed to get database key for LinkGroup");
         }
      }
   }

   /**
    * Get this LinkGroup's database indicator.
   **/
   public int getInd() {
      return(linkGroupInd_);
   }
   /**
    * Set this LinkGroup's database indicator.
   **/
   void setInd(int linkGroupInd) {
      linkGroupInd_ = linkGroupInd;
   }

   /**
    * Returns true if this Link is archived.
   **/
   public boolean isArchived() {
      return(archived_);
   }
   /**
    * Set the Link's archive state.
   **/
   public void setArchived(boolean archived) {
      archived_ = archived;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns the creation time in DB2 timestamp format.
   **/
   public String getCreationTime() {
      return(creationTime_);
   }
   /**
    * Sets the creation time.
    * <b>Note:</b> The creationTime value may be set, but it is not
    * used when the object is written to the database.
    * @param creationTime a String in DB2 timestamp format.
   **/
   public void setCreationTime(String creationTime) {
      creationTime_ = creationTime;
   }

   /**
    * Gets the user who last updated this object.
   **/
   public String getDbUser() {
      return(dbuser_);
   }
   /**
    * Sets the user who last updated this object.
   **/
   public void setDbUser(String dbuser) {
      dbuser_ = dbuser.toUpperCase();
      //updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns the changed time in DB2 timestamp format.
   **/
   public String getChangedTime() {
      return(changedTime_);
   }
   /**
    * Sets the changed time.
    * <b>Note:</b> The changedTime value may be set, but it is not
    * used when the object is written to the database.
    * @param creationTime a String in DB2 timestamp format.
   **/
   public void setChangedTime(String changedTime) {
      changedTime_ = changedTime;
   }

   /**
    * Returns true if there is an open draft of this object.
    * This method is only valid for active objects (Action,
    * Question, Symptom), not for draft objects (ActionDraft,
    * QuestionDraft, SymptomDrat).
   **/
   public boolean hasOpenDraft() {
      return(openDraft_);
   }

   /**
    * Set the open draft flag for this object.
   **/
   void setOpenDraft(boolean openDraft) {
      openDraft_ = openDraft;
   }

   /**
    * Write the LinkGroup object to the database.
    * This method will NOT write the Link objects contained in the LinkGroup.
    * This method assumes that a DatabaseSystem object for
    * the DatabaseSystem.QUEST database has already been connected.
    * 
    * @return 0 for success, nonzero for failure.
   **/
   public int writeToDatabase() {
      LogSystem.log(2, ">>> Starting LinkGroup.writeToDatabase()");

      /*--------------------------------------------------------------*/
      /*   Assign database keys for this Link and its subobjects.     */
      /*--------------------------------------------------------------*/
      if ( !assignDatabaseKeys() ) {
         return(-1);
      }

      /*--------------------------------------------------------------*/
      /*   Make sure the Links have valid (non-zero) indicators.      */
      /*--------------------------------------------------------------*/
      for ( int i=0; links_ != null && i<links_.size(); i++ ) {
         Link lnk = (Link) links_.elementAt(i);
         if ( lnk.getInd() == 0 ) {
            LogSystem.log(2, "Link " + i + " does not have a valid indicator");
            return(-1);
         }
      }

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "LinkGroup.writeToDatabase()", 5 );
      int returnCode = 0 ;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.sqlRunner", 2 );
         String sqlToRun = LinkGroupWriter.getSQL(this);
         if ( sqlToRun.length() == 0 ) {
            LogSystem.log(1, "No changes to be written");
            return(0);
         }
         String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                       SqlRunner.END_DELIM, "\n" );

         LogSystem.log ( 1, "Starting to send statements to SQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 2 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            returnCode = retRec.sqlCode ;
            if ( returnCode == 0 ) returnCode = -1;
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         returnCode = -1 ;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> LinkGroup.writeToDatabase() complete");

      return(returnCode);
   }

   /**
    * Read a LinkGroup object from the database.
    * 
    * @param linkGroupInd  the database indicator of the LinkGroup to be read.
    * @param languageInd   the language to read the text items for 
    *                      (0 = read all languages).
    * @return the LinkGroup object or null if an error occured or
    *         the object was not found.
   **/
   public static LinkGroup readFromDatabase(int linkGroupInd, int languageInd) {
      LogSystem.log(2, ">>> Starting LinkGroup.readFromDatabase()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "LinkGroup.readFromDatabase()", 5 );
      StoredProcRec retRec     = null;
      LinkGroup     retLinkGrp = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, linkGroupInd );           // object ind
         stmt.setInt( 2, OAReader.LINKGROUP );     // object type
         stmt.setInt( 3, OAReader.READ_ACTIVE );   // read active object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Link and the StoredProcReturnRec objects        */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retLinkGrp = (LinkGroup)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      LogSystem.log(2, ">>> LinkGroup.readFromDatabase() completed");

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Do this after reading the question so that we           */
      /* do not open multiple connections to the database reading  */
      /* the type lists.                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retLinkGrp != null ) {
         // update the Type*Rec objects
         retLinkGrp.updateTypeRecData();
      } else {
         LogSystem.log(2, "No object found for indicator " + linkGroupInd);
      }

      return(retLinkGrp);
   }

   /**
    * Update the TypeRec objects in the LinksGroup's LanguageText objects with 
    * the proper descriptions.
    * Called by readFromDatabase();
   **/
   void updateTypeRecData() {

      // update title TypeRec data:
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            LanguageText title = (LanguageText) titles_.elementAt(i);
            title.updateTypeRecData();
         }
      }

      return;
   }


}
