
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.projects.common.storedProc.oaPublish.*;
import com.ibm.nzna.projects.common.storedProc.oaValidate.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;

/**
 * Online Assistant draft LinkGroup.
 * A Link Group is a collection of Link objects.
 *
 * @see Link
 *
 * @version 1.16
 * @author  Mark DeBiase
**/
public class LinkGroupDraft extends LinkGroup implements DraftObject {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   private int         activeLinkGroupInd_ = 0;
   private boolean     maintLock_          = false;
   private String      startDate_          = "";
   private String      stopDate_           = null;
   private boolean     recycled_           = false;
   private TypeWorkRec typeWork_           = null;
   private String      comments_           = "";

   /**
    * New LinkGroupDraft object with the specified English title.
   **/
   public LinkGroupDraft(String title) {
      super(title);

      TypeList t = TypeList.getInstance();
      typeWork_  = (TypeWorkRec) t.objectFromInd( 1, TypeList.TYPE_WORKREQUIRED );
   }

   /**
    * LinkGroup to be read from the database.
    *
    * @param linkGroupInd the linkGroup database indicator
   **/
   public LinkGroupDraft(int linkGroupInd) {
      super(linkGroupInd);
   }

   /**
    * Create a new LinkGroupDraft object from an active LinkGroup.
   **/
   LinkGroupDraft(LinkGroup publishedLinkGroup) {
      super(publishedLinkGroup);

      // set the active linkgroup indicator
      activeLinkGroupInd_ = publishedLinkGroup.linkGroupInd_;
   }

   public int getActiveLinkGroupInd() {
      return (activeLinkGroupInd_);
   }

   public void setActiveLinkGroupInd(int activeLinkGroupInd) {
      activeLinkGroupInd_ = activeLinkGroupInd;
   }

   public boolean getMaintLock() {
      return (maintLock_);
   }

   public void setMaintLock(boolean maintLock)
   {
      maintLock_ = maintLock;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns the date the draft was opened.
   **/
   public String getStartDate() {  
      return (startDate_);
   }

   /**
    * Set the date the draft was opened.
    * <b>Note:</b> This value may be set, but it will not
    * used when the object is written to the database
    * (CURRENT DATE will be used).
   **/
   public void setStartDate(String startDate) {
      startDate_ = startDate;
   }

   /**
    * Returns the date the draft was closed or null if the draft is still open.
   **/
   public String getStopDate() {
      return (stopDate_);
   }

   /**
    * Set the date the draft was closed.
    * <b>Note:</b> This value may be set, but it will not
    * used when the object is written to the database
    * (CURRENT DATE will be used).
   **/
   public void setStopDate(String stopDate) {
      stopDate_ = stopDate;
   }

   /**
    * Returns true if this object is recycled.
   **/
   public boolean isRecycled() {
      return (recycled_);
   }

   /**
    * Set recycled flag for this object.
   **/
   public void setRecycled(boolean recycled) {
      recycled_ = recycled;
      updateRecStatus(REC_UPDATE);
   }

   public TypeWorkRec getTypeWorkRequired() {
      return (typeWork_);
   }

   public void setTypeWorkRequired(TypeWorkRec typeWork) {
      typeWork_ = typeWork;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Assign a database key for this LinkGroup
    * (but <b>NOT</b> the Links it contains).
    * @return true if a database keys were assigned, otherwise false
   **/
   public boolean assignDatabaseKeys() {
      boolean success = true;

      if ( linkGroupInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for LinkGroupDraft");
         linkGroupInd_ = Counter.getCounter(LogSystem.getInstance(), "DLNKGRPIND");
         if ( linkGroupInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for LinkGroupDraft");
            success = false;
         }
      }

      return (success);
   }
   // version to be called by publish stored procedure:
   void assignDatabaseKeys(Connection conn) throws Exception {

      if ( linkGroupInd_ == 0 ) {
         linkGroupInd_ = Counter.getCounter(conn, "DLNKGRPIND");
         if ( linkGroupInd_ == 0 ) {
            throw new Exception("Failed to get database key for LinkGroupDraft");
         }
      }
   }

   /**
    * Write the draft LinkGroup object to the database.
    * This method will NOT write the Link objects contained in the LinkGroup.
    * This method assumes that a DatabaseSystem object for
    * the DatabaseSystem.QUEST database has already been connected.
    * 
    * @return 0 for success, nonzero for failure.
   **/
   public int writeToDatabase() {
      LogSystem.log(2, ">>> Starting LinkGroupDraft.writeToDatabase()");

      /*--------------------------------------------------------------*/
      /*   Assign database keys for this Link and its subobjects.     */
      /*--------------------------------------------------------------*/
      if ( !assignDatabaseKeys() ) {
         return (-1);
      }

      /*--------------------------------------------------------------*/
      /*   Make sure the Links have valid (non-zero) indicators.      */
      /*--------------------------------------------------------------*/
      for ( int i=0; links_ != null && i<links_.size(); i++ ) {
         Link lnk = (Link) links_.elementAt(i);
         if ( lnk.getInd() == 0 ) {
            LogSystem.log(2, "Link " + i + " does not have a valid indicator");
            return (-1);
         }
      }

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "LinkGroupDraft.writeToDatabase()", 5 );
      int returnCode = 0 ;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.sqlRunner", 2 );
         String sqlToRun = LinkGroupDraftWriter.getSQL(this);
         if ( sqlToRun.length() == 0 ) {
            LogSystem.log(1, "No changes to be written");
            return (0);
         }
         String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                       SqlRunner.END_DELIM, "\n" );

         LogSystem.log ( 1, "Starting to send statements to SQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 2 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            returnCode = retRec.sqlCode ;
            if ( returnCode == 0 ) returnCode = -1;
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         returnCode = -1 ;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> LinkGroupDraft.writeToDatabase() complete");

      return (returnCode);
   }

   /**
    * Validate this object for publishing.  
    * 
    * @return a ValidationRec with the results of the validation or
    *         null if an error occurs.  Check the ValidationRec to 
    *         determine if publishing can proceed.
    * @see com.ibm.nzna.projects.common.storedProc.oaValidate.ValidationRec
   **/
   public ValidationRec validate() {
      LogSystem.log(2, ">>> Starting LinkDraft.validate()");
      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "", 10 );
      ValidationRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAValidate", 3 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter( 3, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, this.getInd() );         // object ind
         stmt.setInt( 2, OAValidate.LINKGROUP );  // object type

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (ValidationRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "ValidationRec was returned null");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            retRec = null;   // return error to the user
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         retRec = null;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> LinkDraft.validate() complete");

      return (retRec);
   }

   /**
    * Publish the LinkGroupDraft.
    * Any changes to the draft must be saved to the database by
    * calling writeToDatabase() before the draft is published (the
    * publish method will not save the current object).
    * 
    * @return true for success, false for failure.
   **/
   public boolean publish() {
      LogSystem.log(2, ">>> Starting LinkGroupDraft.publish()");

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "LinkGroupDraft.publish()", 10 );
      boolean success = true;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAPublish", 3 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, this.getInd() );         // object ind
         stmt.setInt( 2, OAPublish.LINKGROUP );   // object type

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         } else {
            // the userError String will contain the SQL generated for
            // publishing if DEBUG=true in the OAPublish stored procedure.
            if ( retRec.userError.length() > 0 ) {
               LogSystem.log(1, "Publish SQL:");
               LogSystem.log(2, retRec.userError);
            }
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            success = false;
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         success = false;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> LinkGroupDraft.publish() complete");

      return (success);
   }

   /**
    * Read a LinkGroupDraft object from the database.
    * 
    * @param linkGroupInd  the database indicator of the draft LinkGroup.
    * @param languageInd   the language to read the text items for 
    *                      (0 = read all languages).
    * @return the LinkGroupDraft object or null if an error occured or
    *         the object was not found.  The LinkGroupDraft object is 
    *         returned as a LinkGroup ojbect -- it must be cast.
   **/
   public static LinkGroup readFromDatabase(int linkGroupInd, int languageInd) {
      LogSystem.log(2, ">>> Starting LinkGroupDraft.readFromDatabase()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "LinkGroupDraft.readFromDatabase()", 5 );
      StoredProcRec  retRec     = null;
      LinkGroupDraft retLinkGrp = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, linkGroupInd );           // object ind
         stmt.setInt( 2, OAReader.LINKGROUP );     // object type
         stmt.setInt( 3, OAReader.READ_DRAFT );    // read draft object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Link and the StoredProcReturnRec objects        */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retLinkGrp = 
         (LinkGroupDraft)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      LogSystem.log(2, ">>> LinkGroupDraft.readFromDatabase() completed");

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Do this after reading the question so that we           */
      /* do not open multiple connections to the database reading  */
      /* the type lists.                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retLinkGrp != null ) {
         // update the Type*Rec objects
         retLinkGrp.updateTypeRecData();
      } else {
         LogSystem.log(2, "No object found for indicator " + linkGroupInd);
      }

      return (retLinkGrp);
   }

   /**
    * Get a draft LinkGroup.  This method will return the open draft of an
    * LinkGroup if one exists.  If there is no open draft, a new one will
    * be created.
    * <b>Note:</b> If a new draft is created, the openDraft flag in the
    * oa.linkGroup table will be set to 'Y'.  This flag will be reset to
    * 'N' when the draft is published.  If the draft is not saved, the
    * openDraft flag on oa.linkGroup will have to be manually reset.
    * 
    * 
    * @param linkGroupInd the database indicator of the published LinkGroup 
    *                     whose draft is to be read.
    * @param languageInd  the language to read the text items for 
    *                     (0 = read all languages).
    * @param typeWork    the TypeWorkRec to be set into the
    *                    returned draft object if it is a new draft
    * @return the LinkGroupDraft object or null if an error occured or
    *         the object was not found.
    * @exception PendingDraftException if there is a pending draft
   **/
   public static LinkGroupDraft getDraft(int linkGroupInd, int languageInd,
                                         TypeWorkRec typeWork)
   throws PendingDraftException
   {
      LogSystem.log(2, ">>> Starting LinkGroupDraft.getDraft()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "LinkGroupDraft.getDraft()", 5 );
      StoredProcRec  retRec       = null;
      LinkGroupDraft retLinkGroup = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, linkGroupInd );           // object ind
         stmt.setInt( 2, OAReader.LINKGROUP );     // object type
         stmt.setInt( 3, OAReader.GET_DRAFT );     // get draft object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Action and the StoredProcReturnRec objects      */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retLinkGroup = 
         (LinkGroupDraft)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         } else {
            if ( retLinkGroup != null ) {
               if ( retLinkGroup.getRecStatus() == REC_INSERT ) {
                  retLinkGroup.setTypeWorkRequired( typeWork );
               }
               // update the Type*Rec objects
               retLinkGroup.updateTypeRecData();
            } else {
               LogSystem.log(2, "No object found for indicator " + linkGroupInd);
            }
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      LogSystem.log(2, ">>> LinkGroupDraft.getDraft() completed");

      // throw exception if the return code indicates a pending draft
      if ( retRec != null && retRec.error && 
           retRec.sqlCode == OAReader.PENDING_DRAFT_RC ) {
         throw new PendingDraftException();
      }

      return (retLinkGroup);
   }

   /*------------------------------------------------------------*/
   /* setComments
    */
   /**
    *    This method will set the comments
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setComments ( String comments ) {
      comments_ = comments;
      updateRecStatus(REC_UPDATE);
   }




   /*------------------------------------------------------------*/
   /* getComments
    */
   /**
    *    this method will return the comments
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getComments () {
      return (comments_);
   }



   /*------------------------------------------------------------*/
   /* forceCloseDraft
    */
   /**
    *    This method will set the value for 'OPENDRAFT'
    * in the OA.LINKGROUP table to 'N' if there are no
    * open drafts.
    *  
    *  
    *
    *  @param databaseInd The database indicator for the LINKGROUP
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void forceCloseDraft ( int databaseInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "forceCloseDraft", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         // update openDraft only if there are no open drafts
         stmt.executeUpdate (
                            "UPDATE oa.linkGroup SET openDraft='N' " +
                            "WHERE linkGroupInd=" + databaseInd + " " +
                            "AND (" +
                            "SELECT linkGroupInd FROM oadraft.linkGroup " +
                            "WHERE activeLinkGroupInd=" + databaseInd + " " +
                            "AND stopDate IS NULL" +
                            ") IS NULL"
                            );

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }
      sqlMethod.close() ;
   }

}
