package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.*;

/**
 * Reader for Online Assistant LinkGroup objects.
 * 
 * @version 1.2
 * @author Vishnu Kumar. E
**/


public class LinkGroupReader {

  /**
   * Read LinkGroup.
   * The LinkGroup object's indicator should contain the linkInd to be read.
   * @param conn  the database Connection object
   * @param l     the LinkGroup object to populate
   * @param lang  the language to read the titles for (0 = all languages)
   * @return true if the LinkGroup was successfully read
             false if the LinkGroup was not found in the database
  **/
  public static boolean readFromDB(Connection conn, LinkGroup lg, 
                                   int languageInd) 
                throws SQLException
  {
    boolean    foundLinkGroup;
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    rs = stmt.executeQuery( 
       "SELECT " +
         "archived, created, dbUser, changedTime, openDraft " +
       "FROM oa.linkGroup " +
       "WHERE linkGroupInd=" + lg.getInd() + " " +
       "FOR FETCH ONLY"
    );

    if ( !rs.next() ) {
      foundLinkGroup = false;
      rs.close();
      stmt.close();
    } else {
      foundLinkGroup = true;

      //---------------------------------------------------------------------
      // Read LinkGroup information
      //---------------------------------------------------------------------
      lg.setArchived( DatabaseUtil.booleanFromDB( rs.getString(1) ) );
      lg.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(2) ) );
      lg.setDbUser( rs.getString(3).trim() );
      lg.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(4) ) );
      lg.setOpenDraft( DatabaseUtil.booleanFromDB( rs.getString(5) ) );

      rs.close();
      stmt.close();

      //---------------------------------------------------------------------
      // Read the Links that are in the LinkGroup
      //---------------------------------------------------------------------
      readLinkGroupLinks(conn, lg, languageInd);

      //---------------------------------------------------------------------
      // Read Link Titles
      //---------------------------------------------------------------------
      if (languageInd == 0) {
        readLinkGroupTitles(conn, lg);
      } else {
        readLinkGroupTitles(conn, lg, languageInd);
      }
      
      lg.updateRecStatus(lg.REC_NOCHANGE);
    }

    return foundLinkGroup;
 }

  /**
   * Read Link Title for specific language.
   * Returns the English title if requested language is not found.
  **/
  static void readLinkGroupTitles(Connection conn, LinkGroup lg, 
                                  int languageInd)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String selectStmt = "SELECT " + 
                          "languageInd, lastTranslated, title, " +
                          "dbUser, changedTime " +
                        "FROM oa.linkGroupTitle " + 
                        "WHERE linkGroupInd=" + lg.getInd();

    String titleSql = selectStmt;
    titleSql += " AND languageInd=" + languageInd + " FOR FETCH ONLY";

    // try to get title in requested language
    rs = stmt.executeQuery( titleSql );

    if ( !rs.next() ) {
      // title was not found in requested language -- get English title
      titleSql = selectStmt;
      titleSql += " AND languageInd=1 FOR FETCH ONLY";
      rs = stmt.executeQuery( titleSql );
      rs.next();    // advance cursor to first row
    }

    TypeLanguageRec langRec = new TypeLanguageRec( rs.getShort(1), "" );
    String title = rs.getString(3);
 
    LanguageText langText = new LanguageText( title, langRec );
    langText.setLastTranslationDate( 
                         DatabaseUtil.timestampFromDB(rs.getString(2)) ); 
    langText.setDbUser( rs.getString(4) );
    langText.setChangedTime( DatabaseUtil.timestampFromDB(rs.getString(5)) );

    langText.updateRecStatus(langText.REC_NOCHANGE);
    // return a vector that contains just the requested title
    Vector v = new Vector(1);
    v.addElement(langText);

    rs.close();
    stmt.close();

    lg.setLinkGroupTitles( v );
  }

  /**
   * Read Link Titles for all languages.
  **/
  static void readLinkGroupTitles(Connection conn, LinkGroup lg)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String selectStmt = "SELECT " + 
                          "languageInd, lastTranslated, title, " +
                          "dbUser, changedTime " +
                        "FROM oa.linkGroupTitle " + 
                        "WHERE linkGroupInd=" + lg.getInd() + " " +
                        "ORDER BY languageInd " +
                        "FOR FETCH ONLY";

    rs = stmt.executeQuery( selectStmt );

    Vector v = new Vector();
    while ( rs.next() ) {
      TypeLanguageRec langRec = new TypeLanguageRec( rs.getShort(1), "" );
      String title = rs.getString(3);
 
      LanguageText langText = new LanguageText( title, langRec );
      langText.setLastTranslationDate( 
                         DatabaseUtil.timestampFromDB(rs.getString(2)) ); 
      langText.setDbUser( rs.getString(4) );
      langText.setChangedTime( 
                         DatabaseUtil.timestampFromDB(rs.getString(5)) );

      langText.updateRecStatus(langText.REC_NOCHANGE);
      v.addElement(langText);
    }

    rs.close();
    stmt.close();

    lg.setLinkGroupTitles( v );
  }

  /**
   * Read the Links that are in the LinkGroup.
  **/
  static void readLinkGroupLinks(Connection conn, LinkGroup lg, int languageInd)
              throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    rs = stmt.executeQuery(
      "SELECT linkInd " +
      "FROM oa.linkGroupLinks " +
      "WHERE linkGroupInd=" + lg.getInd() + " " +
      "ORDER BY sortOrder " +
      "FOR FETCH ONLY"
    );

    Vector linksVec = new Vector();
    while ( rs.next() ) {
      int linkInd = rs.getInt(1);

      Link lnk = LinkReader.readFromDB(conn, linkInd, languageInd);
      linksVec.addElement( lnk );
    }

    rs.close();
    stmt.close();

    lg.setLinks( linksVec );
  }

}
