
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant Link objects.
 * 
 * @version 1.7
 * @author  Mark DeBiase
**/


public class LinkReader {

  /**
   * Read Links.
   * 
   * @param conn    the database Connection object
   * @param linkInd the indicator of the Link to be read
   * @param lang    the language to read the titles for (0 = all languages)
   * 
   * @return a Link object if the Link was successfully read or 
   *         null if the Link was not found in the database.
  **/
  public static Link readFromDB(Connection conn, int linkInd, int languageInd) 
                     throws SQLException
  {
    Link       retLink;
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read the Link
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "l.linkType, l.docInd, d.lndocid, l.url, l.urlValid, " +
         "l.archived, l.created, l.dbUser, l.changedTime, l.openDraft " +
       "FROM " +
         "oa.link l LEFT OUTER JOIN tigris.documents d ON l.docInd=d.docInd " +
       "WHERE linkInd=" + linkInd + " " +
       "FOR FETCH ONLY"
    );

    if ( !rs.next() ) {
      retLink = null;     // linkInd not found
      stmt.close();
    } else {

      //---------------------------------------------------------------------
      // Read info specific to a QuestLink or ExternalLink
      //---------------------------------------------------------------------
      Link lnk;
      short linkType = rs.getShort(1);
      if (linkType == Link.LINK_TYPE_QUEST) {
        QuestLink ql = new QuestLink( linkInd );
        ql.setDocInd( rs.getInt(2) );

        String lnDocId = rs.getString(3);
        if (lnDocId != null) {
          ql.setLNDocId( lnDocId.trim() );
        }
        lnk = ql;
      } else {
        ExternalLink el = new ExternalLink( linkInd );
        el.setUrl( rs.getString(4) );
        el.setUrlValid( DatabaseUtil.booleanFromDB( rs.getString(5) ) );
        lnk = el;
      }

      //---------------------------------------------------------------------
      // Read info common to both types of Link
      //---------------------------------------------------------------------
      lnk.setArchived( DatabaseUtil.booleanFromDB( rs.getString(6) ) );
      lnk.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
      lnk.setDbUser( rs.getString(8).trim() );
      lnk.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(9) ) );
      lnk.setOpenDraft( DatabaseUtil.booleanFromDB( rs.getString(10) ) );

      stmt.close();

      //---------------------------------------------------------------------
      // Read Link Titles
      //---------------------------------------------------------------------
      if (languageInd == 0) {
        readLinkTitles(conn, lnk);
      } else {
        readLinkTitles(conn, lnk, languageInd);
      }

      lnk.updateRecStatus(lnk.REC_NOCHANGE);
      retLink = lnk;
    }

    return retLink;
  }

  /**
   * Read Link Title for specific language.
   * Returns the English title if requested language is not found.
  **/
  static void readLinkTitles(Connection conn, Link lnk, int languageInd)
         throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String selectStmt = "SELECT " + 
                          "languageInd, lastTranslated, title, " +
                          "dbUser, changedTime " +
                        "FROM oa.linkTitle " + 
                        "WHERE linkInd=" + lnk.getInd();

    String titleSql = selectStmt;
    titleSql += " AND languageInd=" + languageInd + " FOR FETCH ONLY";

    // try to get title in requested language
    rs = stmt.executeQuery( titleSql );

    if ( !rs.next() ) {
      // title was not found in requested language -- get English title
      titleSql = selectStmt;
      titleSql += " AND languageInd=1 FOR FETCH ONLY";
      rs = stmt.executeQuery( titleSql );
      rs.next();    // advance cursor to first row
    }

    TypeLanguageRec langRec = new TypeLanguageRec( rs.getShort(1), "" );
    String title = rs.getString(3);
 
    LanguageText langText = new LanguageText( title, langRec );
    langText.setLastTranslationDate( 
                         DatabaseUtil.timestampFromDB(rs.getString(2)) ); 
    langText.setDbUser( rs.getString(4) );
    langText.setChangedTime( DatabaseUtil.timestampFromDB(rs.getString(5)) );

    langText.updateRecStatus(langText.REC_NOCHANGE);
    // return a vector that contains just the requested title
    Vector v = new Vector(1);
    v.addElement(langText);

    rs.close();
    stmt.close();

    lnk.setLinkTitles( v );
  }

  /**
   * Read Link Titles for all languages.
  **/
  static void readLinkTitles(Connection conn, Link lnk)
         throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    String selectStmt = "SELECT " + 
                          "languageInd, lastTranslated, title, " +
                          "dbUser, changedTime " +
                        "FROM oa.linkTitle " + 
                        "WHERE linkInd=" + lnk.getInd() + " " +
                        "ORDER BY languageInd " +
                        "FOR FETCH ONLY";

    rs = stmt.executeQuery( selectStmt );

    Vector v = new Vector();
    while ( rs.next() ) {
      TypeLanguageRec langRec = new TypeLanguageRec( rs.getShort(1), "" );
      String title = rs.getString(3);
 
      LanguageText langText = new LanguageText( title, langRec );
      langText.setLastTranslationDate( 
                         DatabaseUtil.timestampFromDB(rs.getString(2)) ); 
      langText.setDbUser( rs.getString(4) );
      langText.setChangedTime( 
                         DatabaseUtil.timestampFromDB(rs.getString(5)) );

      langText.updateRecStatus(langText.REC_NOCHANGE);
      v.addElement(langText);
    }
    rs.close();
    stmt.close();

    lnk.setLinkTitles( v );
  }
}
