
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.shared.util.*;
import java.sql.*;

/**
 * Represents a Node in a Symptom.
 * A Node is either an instance of an OAObject (a Question, an
 * Action or another Symptom) or a special state (RESOLVED, UNRESOLVED,
 * ALIST, RETURN) in the problem diagnosis logic flow of a Symptom.
 * <p>
 * The isOAObjectNode() and getNodeType() methods can be used to 
 * determine what type of Node the object represents.
 *
 * @version 1.20
 * @author  Mark DeBiase
**/
public class Node extends PersistentRec {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  private int    nodeInd_     = 0;   // this node's database key
  private int    symptomInd_  = 0;   // the symptom the node belongs to
  private short  nodeType_    = 0;   // the node type (see types below)

  // activeObjectInd_ is the indicator of the active OAObject this node
  // refers to.
  // - for nodes based on active objects,  this will be the indicator of the
  //   corresponding Action, Question or Symptom object returned by 
  //   the getInd() method.
  // - for nodes based on draft objects, this will be the active object 
  //   indicator in the corresponding ActionDraft, QuestionDraft or 
  //   SymptomDraft object returned by the getActiveXXXInd() method.
  // activeObjectInd_ will be 0 if the node is in a draft symptom and the
  // node refers to a draft object which has never been published.  it
  // should always be non-zero for nodes in an active symptom.
  //
  // draftObjectInd_ is only valid for nodes in a draft symptom -- it holds
  // the indicator of the corresponding draft object that the node refers
  // to.  it will be 0 if there is no draft object.
  //
  // both activeObjectInd_ and draftObjectInd_ will be 0 if this is a
  // special node (RESOLVED, UNRESOLVED, RETURN, ALIST).
  //
  private int  activeObjectInd_ = 0;
  private int  draftObjectInd_  = 0; 
                                       

  // GUI data - saved to database
  private int  xPos_   = 0;
  private int  yPos_   = 0;
  private int  width_  = 75;
  private int  height_ = 75;
  private int  zOrder_ = 0;

  private String   dbUser_      = null;
  private String   changedTime_ = null;

  // node types - correspond to node types in database:
  public  static final short NODE_TYPE_ACTION     =  0;
  public  static final short NODE_TYPE_QUESTION   =  1;
  public  static final short NODE_TYPE_SYMPTOM    =  2;
  public  static final short NODE_TYPE_RESOLVED   =  3;
  public  static final short NODE_TYPE_UNRESOLVED =  4;
  public  static final short NODE_TYPE_RETURN     =  5;
  public  static final short NODE_TYPE_ALIST      =  6;

  /** 
   * Create a node which refers to an Action, Question or Symptom.
   * @param oaobj the OAObject the node refers to.
   * @exception IllegalArgumentException if the OAObject's indicator is 0
   *            (or both the indicator and the active indicator are zero
   *             for a draft OAObject).
  **/
  public Node(OAObject oaobj) {
    if (oaobj instanceof ActionDraft) {
      ActionDraft ad   = (ActionDraft) oaobj;
      nodeType_        = NODE_TYPE_ACTION;
      activeObjectInd_ = ad.getActiveActionInd();
      draftObjectInd_  = ad.getInd();
    } else if (oaobj instanceof QuestionDraft) {
      QuestionDraft qd = (QuestionDraft) oaobj;
      nodeType_        = NODE_TYPE_QUESTION;
      activeObjectInd_ = qd.getActiveQuestionInd();
      draftObjectInd_  = qd.getInd();
    } else if (oaobj instanceof SymptomDraft) {
      SymptomDraft sd  = (SymptomDraft) oaobj;
      nodeType_        = NODE_TYPE_SYMPTOM;
      activeObjectInd_ = sd.getActiveSymptomInd();
      draftObjectInd_  = sd.getInd();
    } else if (oaobj instanceof Action) {
      nodeType_        = NODE_TYPE_ACTION;
      activeObjectInd_ = oaobj.getInd();
    } else if (oaobj instanceof Question) {
      nodeType_        = NODE_TYPE_QUESTION;
      activeObjectInd_ = oaobj.getInd();
    } else if (oaobj instanceof Symptom) {
      nodeType_        = NODE_TYPE_SYMPTOM;
      activeObjectInd_ = oaobj.getInd();
    }

    if (activeObjectInd_ == 0 && draftObjectInd_ == 0) {
      throw new IllegalArgumentException("oaObjectInd is 0");
    }

    updateRecStatus(REC_INSERT);
  }

  /** 
   * Create a special state node.  nodeType should be one of the 
   * following values:
   * <ul>
   * <li> NODE_TYPE_RESOLVED
   * <li> NODE_TYPE_UNRESOLVED
   * <li> NODE_TYPE_RETURN
   * <li> NODE_TYPE_ALIST
   * </ul>
   * @param nodeType  the node type
   * @exception IllegalArgumentException if the nodeType is invalid.
  **/
  public Node(short nodeType) {
    if ( nodeType != NODE_TYPE_RESOLVED    &&
         nodeType != NODE_TYPE_UNRESOLVED  &&
         nodeType != NODE_TYPE_RETURN      &&
         nodeType != NODE_TYPE_ALIST  )
    {
      throw new IllegalArgumentException("Invalid nodeType " + nodeType);
    }
 
    nodeType_ = nodeType;

    updateRecStatus(REC_INSERT);
  }

  /**
   * Create a node which referes to a <b><i>published</i></b>
   * Action, Question or Symptom by database indicator.
   * nodeType should be one of the following values:
   * <ul>
   * <li> NODE_TYPE_ACTION
   * <li> NODE_TYPE_QUESTION
   * <li> NODE_TYPE_SYMPTOM
   * </ul>
   * @param nodeType  the node type.
   * @param oaobjInd  the OAObject's (Action, Question or Symptom) indicator.
   *                  oaobjInd should be the indicator of a 
   *                  <i>published</i> object.
   * @exception IllegalArgumentException if the nodeType is invalid or the
   *                                     oaobjInd is 0.
   *
   * @deprecated use the Node(OAObject) constructor instead
  **/
  public Node(short nodeType, int oaobjInd) {
    if ( nodeType != NODE_TYPE_ACTION   &&
         nodeType != NODE_TYPE_QUESTION &&
         nodeType != NODE_TYPE_SYMPTOM  )
    {
      throw new IllegalArgumentException("Invalid nodeType " + nodeType);
    }
    if ( oaobjInd == 0 ) {
      throw new IllegalArgumentException("oaObjectInd is 0");
    }

    nodeType_        = nodeType;
    activeObjectInd_ = oaobjInd;
    draftObjectInd_  = 0;

    updateRecStatus(REC_INSERT);
  }

  /**
   * Constructor used by Readers.
  **/
  Node() {
    super();
  }

  /**
   * Gets the Node database indicator.
  **/
  public int getInd() {
    return nodeInd_;
  }

  /**
   * Sets the Node database indicator.
  **/
  void setInd(int nodeInd) {
    nodeInd_ = nodeInd;
  }

  /**
   * Assign a database key for this Node.
   * @return true if a database key was assigned, otherwise false
  **/
  public boolean assignDatabaseKey(boolean isDraft) {
    boolean success = true;
    String  counterName;

    if (isDraft) {
      counterName = "DNODEIND";
    } else {
      counterName = "NODEIND";
    }

    if (nodeInd_ == 0 && valid()) {
      LogSystem.log(1, "Getting database key for Node: " + counterName);
      nodeInd_ = Counter.getCounter( LogSystem.getInstance(), counterName );
      if (nodeInd_ == 0) {
        LogSystem.log(2, "Failed to get database key for Node");
        success = false;
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  void assignDatabaseKey(Connection conn, boolean isDraft) throws Exception {
    String  counterName;

    if (isDraft) {
      counterName = "DNODEIND";
    } else {
      counterName = "NODEIND";
    }

    if (nodeInd_ == 0) {
      nodeInd_ = Counter.getCounter( conn, counterName );
      if (nodeInd_ == 0) {
        throw new Exception("Failed to get database key for Node");
      }
    }
  }

  /**
   * Reset values in this object.
   * Sets the indicator to 0, sets the recStatus to REC_INSERT.
  **/
  public void resetNew() {
    setInd(0);
    setSymptomInd(0);
    updateRecStatus ( REC_NOCHANGE ) ;
    updateRecStatus(REC_INSERT);
  }

  /**
   * Gets the parent symptom indicator.
  **/
  int getSymptomInd() {
    return symptomInd_;
  }

  /**
   * Sets the parent symptom indicator.
  **/
  void setSymptomInd(int symptomInd) {
    symptomInd_ = symptomInd;
  }

  /**
   * Gets the node type.
  **/
  public short getNodeType() {
    return nodeType_;
  }

  /**
   * Sets the node type.
  **/
  void setNodeType(short nodeType) {
    nodeType_ = nodeType;
  }

  /**
   * Returns true if this node represents a OAObject (i.e., not a 
   * special node).
  **/
  public boolean isOAObjectNode() {
    return ( nodeType_ == NODE_TYPE_ACTION   ||
             nodeType_ == NODE_TYPE_QUESTION ||
             nodeType_ == NODE_TYPE_SYMPTOM
           );
  }

  /**
   * Gets the <b>active</b> OAObject indicator.
   * Returns 0 if the Node does not represent an OAObject <I>or</I>
   * the Node is in a draft Symptom and represents an OAObject
   * which has never been published.
   * <br>
   * Same as getActiveObjectInd().
  **/
  public int getOAObjectInd() {
    return activeObjectInd_;
  }

  /**
   * Set the <i>active</i> OAObject indicator.
   * Same as setActiveObjectInd().
  **/
  void setOAObjectInd(int oaObjectInd) {
    activeObjectInd_ = oaObjectInd;
  }

  /**
   * Gets the active OAObject indicator.
   * Same as getOAObjectInd().
   * @see getOAObjectInd()
  **/
  public int getActiveObjectInd() {
    return activeObjectInd_;
  }

  /**
   * Set the OAObject indicator.
   * Same as setOAObjectInd().
   * @see setOAObjectInd()
  **/
  void setActiveObjectInd(int oaObjectInd) {
    activeObjectInd_ = oaObjectInd;
  }

  /**
   * Gets the draft OAObject indicator.
   * Valid only for Nodes in a draft Symptom.  Returns the draft indicator
   * of the OAObject the node represents or 0 if there is no draft for
   * the OAObject or the Node is a special node.
  **/
  public int getDraftObjectInd() {
    return draftObjectInd_;
  }

  /**
   * Set the draft OAObject indicator.
  **/
  public void setDraftObjectInd(int draftObjectInd) {
    draftObjectInd_ = draftObjectInd;
  }

  /**
   * Get the X coordinate where this node should be drawn at.
  **/
  public int getXPos() {
    return xPos_;
  }

  /**
   * Get the Y coordinate where this node should be drawn at.
  **/
  public int getYPos() {
    return yPos_;
  }

  /**
   * Get the pixel width of this node.
  **/
  public int getWidth() {
    return width_;
  }

  /**
   * Get the pixel height of this node.
  **/
  public int getHeight() {
    return height_;
  }

  /**
   * Get the Z-order in which this node should be drawn.
  **/
  public int getZOrder() {
    return zOrder_;
  }

  /**
   * Set the GUI data for this node.
  **/
  public void setGuiData(int xPos, int yPos, int width, int height, int zOrder)
  {
    // avoid calling setModified(true) if data has not been changed
    if ( xPos_ != xPos || yPos_ != yPos || width_ != width ||
         height_ != height || zOrder_ != zOrder )
    {
      updateRecStatus(REC_UPDATE);

      xPos_   = xPos;
      yPos_   = yPos;
      width_  = width;
      height_ = height;
      zOrder_ = zOrder;
    }
  }

  /**
   * Get the hashCode for this Node.  The hashCode is the database key,
   * so this is not valid for Nodes which have not been assigned a
   * database key.
  **/
  public int hashCode() {
    if ( getInd() == 0 ) {
      throw new IllegalStateException("hashCode(): Node is new");
    }
    return this.getInd();
  }

  /**
   * Returns true if this Node is the same as another Node, based on 
   * database key.  
   * Not valid for Node which have not been assigned a database indicator.
  **/
  public boolean equals(Object o) {
    if ( getInd() == 0 ) {
      throw new IllegalStateException("equals(): Node is new");
    }
    return ( (o instanceof Node) && 
             (((Node) o).getInd() == this.getInd()) );
  }

  /**
   * Gets the last changed date/time for this object.
  **/
  public String getChangedTime() {
    return changedTime_;
  }

  /**
   * Sets the last changed date/time for this object.
   * Called by Reader classes.
  **/
  void setChangedTime(String changedTime) {
    changedTime_ = changedTime;
  }

  /**
   * Gets the user who last changed this object.
  **/
  public String getDbUser() {
    return dbUser_;
  }

  /**
   * Sets the user who last changed this object.
   * Used by Writer classes - the value from the parent object is used
   * when writing to the database.
  **/
  void setDbUser(String dbUser) {
    dbUser_ = dbUser;
  }

  /**
   * Returns true if xPos, yPos, width, height and zOrder are non-zero.
  **/
  public boolean isGuiDataValid() {
    return ( xPos_ != 0 && yPos_ != 0 && 
             width_ != 0 && height_ != 0 &&
             zOrder_ != 0 
           );
  }


  /**
   * Return a String representation of this object.
  **/
  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("Node " ).append(nodeInd_).append(": ");
    switch(nodeType_) {
      case NODE_TYPE_ACTION:
        sb.append("Action ").append(activeObjectInd_ + "/" + draftObjectInd_);
        break;
      case NODE_TYPE_QUESTION:
        sb.append("Question ").append(activeObjectInd_ + "/" + draftObjectInd_);
        break;
      case NODE_TYPE_SYMPTOM:
        sb.append("Symptom ").append(activeObjectInd_ + "/" + draftObjectInd_);
        break;
      case NODE_TYPE_RESOLVED:
        sb.append("RESOLVED");
        break;
      case NODE_TYPE_UNRESOLVED:
        sb.append("UNRESOLVED");
        break;
      case NODE_TYPE_RETURN:
        sb.append("RETURN");
        break;
      case NODE_TYPE_ALIST:
        sb.append("ALIST");
        break;
      default:
        sb.append("Unknown Type");
        break;
    }
    return sb.toString();
  }


  /*------------------------------------------------------------*/
  /* setXPos
   */
  /**
   *    This method will set the X Position of the
   *  node
   *  
   *
   *  @param x The X Position in Pixels
   *  @param None
   *  @param None
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
  public void setXPos ( int xPos ) {
     xPos_ = xPos ;
  }


  /*------------------------------------------------------------*/
  /* setYPos
   */
  /**
   *    This method will set the Y Position of the
   *  node
   *  
   *
   *  @param y The Y Position in Pixels
   *  @param None
   *  @param None
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
  public void setYPos ( int yPos ) {
     yPos_ = yPos ;
  }





  /*------------------------------------------------------------*/
  /* setWidth
   */
  /**
   *    This method will set the width of the
   *  node
   *  
   *
   *  @param width The width in pixels
   *  @param None
   *  @param None
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
  public void setWidth ( int width ) {
     width_ = width ;
  }


  /*------------------------------------------------------------*/
  /* setHeight
   */
  /**
   *    This method will set the height of the
   *  node
   *  
   *
   *  @param height The height in pixels
   *  @param None
   *  @param None
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
  public void setHeight ( int height ) {
     height_ = height ;
  }

}
