
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.type.*;
import java.util.Vector;

/**
 * Base class for Online Assistant Action, Question and Symptom objects.
 * Maintains the information common to all three objects: document 
 * classification, list of countries, brand, archived state, creation and
 * changed times of the base object and the user who last updated the
 * base object.
 *
 * @version 1.24
 * @author  Mark DeBiase
**/
public abstract class OAObject extends PersistentRec
                               implements OAInterface, Cloneable
{
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000001 ;

  private   String          OAItemNum_;
  protected TypeDocClassRec docClass_;       // document classification
  private   Vector          countries_;      // countries this object applies to
  private   Vector          origCountries_;  // original list of countries
  private   TypeBrandRec    brand_;          // the brand
  private   boolean         archived_;       // if true, object is archived
  private   String          creationTime_;   // date object was created
  private   String          changedTime_;    // data object was updated
  private   String          dbuser_;         // last user to update this object
  private   boolean         openDraft_;      // true if there is an open draft
                                             // of this object

  public abstract boolean assignDatabaseKeys();
  public abstract int     getInd();

  /**
   * New object.
  **/
  public OAObject() {
 
    OAItemNum_     = "";
    docClass_      = null;
    countries_     = null;
    origCountries_ = null;
    brand_         = null;
    archived_      = false;
    creationTime_  = "";
    changedTime_   = "";
    dbuser_        = "";
  }

  /**
   * Create a new OAObject with the attributes of an existing OAObject.
  **/
  OAObject(OAObject anotherObject) {
    // NOTE: this initialization is not safe in that Vectors are
    //       shared between the new object and the original object
    OAItemNum_     = "";
    docClass_      = anotherObject.docClass_;
    countries_     = anotherObject.countries_;
    origCountries_ = null;   // will cause countries to be updated in DB unless
                             // setOriginalCountryListFromCurrent() is called
    brand_         = anotherObject.brand_;
    archived_      = anotherObject.archived_;
    dbuser_        = anotherObject.dbuser_;
    // these values should be reset for the new object:
    creationTime_  = "";
    changedTime_   = "";
  }

  /**
   * Gets the OAItemNum.
  **/
  public String getOAItemNum() {
    return OAItemNum_;
  }

  /**
   * Sets the OAItemNum.
  **/
  void setOAItemNum(String OAItemNum) {
    OAItemNum_ = OAItemNum;
  }

  /**
   * Get the document classification for this object.
   * @return the document classification
   * @see com.ibm.nzna.projects.common.quest.type.TypeDocClassRec
  **/
  public TypeDocClassRec getDocClass() {
    return docClass_;
  }

  /**
   * Set the document classification for this object.<br><Br>
   * <b>Obsolete. This method now does nothing since the Doc Classification
   * is governed by the classification of the default title
   * @param docClass the document classification
   * @see com.ibm.nzna.projects.common.quest.type.TypeDocClassRec
  **/
  public void setDocClass(TypeDocClassRec docClass) {
    //docClass_ = docClass;
    updateRecStatus(PersistentRec.REC_UPDATE);
  }

  /**
   * Gets the list of countries for this object.
   * @return a Vector of TypeCountryCodeRec objects.
   * @see com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec
  **/
  public Vector getCountryList() {
    if (countries_ == null) {
      countries_ = new Vector();
    }
    return countries_;
  }

  /**
   * Sets the list of countries for this object.
   * @param countries a Vector of TypeCountryCodeRec objects.
   * @see com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec
  **/
  public void setCountryList(Vector countries) {
    countries_ = countries;
    updateRecStatus(PersistentRec.REC_UPDATE);
  }

  /**
   * Get the original country list.
   * The original country list is the list of countries that the object
   * contained when it was read from the database (or when it was last
   * saved to the database).  It is used to compare against the current 
   * country list to determine if any countries have been added/deleted.
  **/
  Vector getOriginalCountryList() {
    return origCountries_;
  }

  /**
   * Set the original country list to the current country list.
   * The original country list is the list of countries that the object
   * contained when it was read from the database (or when it was last
   * saved to the database).  It is used to compare against the current 
   * country list to determine if any countries have been added/deleted.
  **/
  void setOriginalCountryListFromCurrent() {
    if (countries_ != null) {
      origCountries_ = (Vector) countries_.clone();
    } else {
      origCountries_ = null;
    }
  }

  /**
   * Gets the Brand for this object.
   * @return the brand.
   * @see com.ibm.nzna.projects.common.quest.type.TypeBrandRec
  **/
  public TypeBrandRec getBrand() {
    return brand_;
  }

  /**
   * Sets the Brand for this object
   * @param brand the brand
   * @see com.ibm.nzna.projects.common.quest.type.TypeBrandRec
  **/
  public void setBrand(TypeBrandRec brand) {
    brand_ = brand;
    updateRecStatus(PersistentRec.REC_UPDATE);
  }

  /**
   * Returns true if this object is archived.
  **/
  public boolean isArchived() {
    return archived_;
  }

  /**
   * Set the archive state of this object.
  **/
  public void setArchived(boolean archived) {
    archived_ = archived;
    updateRecStatus(PersistentRec.REC_UPDATE);
  }

  /**
   * Gets the creation date/time for this object.
   * The creation time is a String in DB2 timestamp format.
  **/
  public String getCreationTime() {
    return creationTime_;
  }

  /**
   * Sets the creation date/time for this object.
   * <b>Note:</b> The creationTime value may be set, but it is not
   * used when the object is written to the database.
   * @param creationTime a String in DB2 timestamp format.
  **/
  public void setCreationTime(String creationTime) {
    creationTime_ = creationTime;
  }

  /**
   * Gets the last changed date/time for this object.
   * The changed time is a String in DB2 timestamp format.
  **/
  public String getChangedTime() {
    return changedTime_;
  }

  /**
   * Sets the last changed date/time for this object.
   * <b>Note:</b> The changedTime value may be set, but it is not
   * used when the object is written to the database.
   * @param changedTime a String in DB2 timestamp format.
  **/
  public void setChangedTime(String changedTime) {
    changedTime_ = changedTime;
  }

  /**
   * Gets the user who last changed this object.
  **/
  public String getDbUser() {
    return dbuser_;
  }

  /**
   * Sets the user who last changed this object.
   * Does not affect the modification state of the object.
  **/
  public void setDbUser(String dbuser) {
    dbuser_ = dbuser.toUpperCase();
  }

  /**
   * Returns true if there is an open draft of this object.
   * This method is only valid for active objects (Action,
   * Question, Symptom), not for draft objects (ActionDraft,
   * QuestionDraft, SymptomDrat).
  **/
  public boolean hasOpenDraft() {
    return openDraft_;
  }

  /**
   * Set the open draft flag for this object.
  **/
  void setOpenDraft(boolean openDraft) {
    openDraft_ = openDraft;
  }

  /**
   * Perform a shallow copy of this object.
  **/
  public Object clone() {
    try {
      OAObject cloned = (OAObject) super.clone();
      return cloned;
    } 
    catch (CloneNotSupportedException e) { 
      throw new InternalError(e.toString());
    }
  }

  /*
   * Perform a deep copy on this object.
   * First calls clone() to create a shallow copy.  Then a deep copy is 
   * made of each vector that this object maintains (all the objects in
   * the vector are clone()'d).
  **/
/*
  public Object deepClone() {
    OAObject cloned = (OAObject) this.clone();

    // nothing in this object needs deep copy

    return cloned;
  }
*/

  /**
   * Returns the hashCode for this object.
   * The hashCode for an OAObject is only valid for objects that
   * have been assigned database indicators.
   * @exception IllegalStateException if object does not have valid indicator
   *                                  (i.e., getInd() returns 0)
  **/
  public int hashCode() {
    if (getInd() == 0) {
      throw new IllegalStateException("hashCode(): OAObject is new");
    }
    return OAItemNum_.hashCode();
  }

  /**
   * Return true if this OAObject is the same as another OAObject 
   * based on their database indicators.  This method is only 
   * valid for objects that have been assigned database indicators.
   * @exception IllegalStateException if object does not have valid indicator
   *                                  (i.e., getInd() returns 0)
  **/
  public boolean equals(Object o) {
    if (getInd() == 0) {
      throw new IllegalStateException("equals(): OAObject is new");
    }
    return ( o instanceof OAObject && 
             ((OAObject) o).getInd() == this.getInd() );
  }

  /**
   * Return a String representation of this object.
   * Not really implemented - returns "OAObject".
  **/
  public String toString() {
    return "OAObject";
  }

  /**
   * Update the TypeRec objects in this object with the proper descriptions.
   * Called after an OAObject is read from the database.
  **/
  void updateTypeRecData() {
    TypeList t = TypeList.getInstance();

    // brand
    TypeBrandRec newBrand;
    newBrand = (TypeBrandRec) t.objectFromInd( brand_.getInd(), 
                                               TypeList.TYPE_BRAND );
    if (newBrand != null) { brand_ = newBrand; }

    // docClass
    TypeDocClassRec newDocClass;
    newDocClass = (TypeDocClassRec) t.objectFromInd( docClass_.getInd(),
                                                     TypeList.TYPE_DOCCLASS );
    if (newDocClass != null) { docClass_ = newDocClass; }

    // country list
    if (countries_ != null) {
      Vector newCountries = new Vector();
      for (int i=0; i<countries_.size(); i++) {
        TypeCountryCodeRec country, newCountry;
        country = (TypeCountryCodeRec) countries_.elementAt(i);
        newCountry = (TypeCountryCodeRec) t.objectFromInd( country.getInd(), 
                                                   TypeList.TYPE_COUNTRYCODE );
        if (newCountry != null) {
          newCountries.addElement( newCountry );
        } else {
          newCountries.addElement( country );
        }
      }
      countries_ = newCountries;
    }
  }
}

