
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.projects.common.storedProc.oaPublish.*;
import com.ibm.nzna.projects.common.storedProc.oaValidate.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.db.*;
import java.sql.*;

/**
 * Draft QuestLink object.
 *
 * @version 1.15
 * @author  Mark DeBiase
**/
public class QuestLinkDraft extends QuestLink implements LinkDraft {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  private int         activeLinkInd_ = 0;
  private boolean     maintLock_     = false;
  private String      startDate_     = "";
  private String      stopDate_      = null;
  private boolean     recycled_      = false;
  private TypeWorkRec typeWork_      = null;
  private String      comments_      = "";

  /**
   * Create a new QuestLink with the specified title.
   * The title contains the text to be displayed on the web -
   * used in the HTML anchor tag.
   * <p>
   * This constructor will create a LanguageText object with
   * a TypeLanguageRec of English for the title and add it
   * to the Vector of LanguageText objects in the Link object
   * superclass.
   * 
   * @see Link
   *
   * @param docInd the Quest document indicator
   * @param title  the link title
  **/
  public QuestLinkDraft(int docInd, String title) {
    super(docInd, title);

    TypeList t = TypeList.getInstance();
    typeWork_  = (TypeWorkRec) t.objectFromInd( 1, TypeList.TYPE_WORKREQUIRED );
  }

  /**
   * QuestLink to be read from the database.
   * @param linkInd the link indicator
  **/
  public QuestLinkDraft(int linkInd) {
    super(linkInd);
  }

  /**
   * Initialize from an active Link.
  **/
  QuestLinkDraft(QuestLink publishedLink) {
    super(publishedLink);

    activeLinkInd_ = publishedLink.linkInd_;
  }

  public int getActiveLinkInd() {
    return activeLinkInd_;
  }
  
  void setActiveLinkInd(int activeLinkInd) {
    activeLinkInd_ = activeLinkInd;
  }

  public boolean getMaintLock() {
    return maintLock_;
  }

  public void setMaintLock(boolean maintLock)
  {
    maintLock_ = maintLock;
    updateRecStatus(REC_UPDATE);
  }

  /**
   * Returns the date the draft was opened.
  **/
  public String getStartDate() {  
    return startDate_;
  }

  /**
   * Set the date the draft was opened.
   * <b>Note:</b> This value may be set, but it will not
   * used when the object is written to the database
   * (CURRENT DATE will be used).
  **/
  public void setStartDate(String startDate) {
    startDate_ = startDate;
  }

  /**
   * Returns the date the draft was closed or null if the draft is still open.
  **/
  public String getStopDate() {
    return stopDate_;
  }

  /**
   * Set the date the draft was closed.
   * <b>Note:</b> This value may be set, but it will not
   * used when the object is written to the database
   * (CURRENT DATE will be used).
  **/
  public void setStopDate(String stopDate) {
    stopDate_ = stopDate;
  }

  /**
   * Returns true if this object is recycled.
  **/
  public boolean isRecycled() {
    return recycled_;
  }

  /**
   * Set recycled flag for this object.
  **/
  public void setRecycled(boolean recycled) {
    recycled_ = recycled;
    updateRecStatus(REC_UPDATE);
  }

  public TypeWorkRec getTypeWorkRequired() {
    return typeWork_;
  }

  public void setTypeWorkRequired(TypeWorkRec typeWork) {
    typeWork_ = typeWork;
    updateRecStatus(REC_UPDATE);
  }

  /**
   * Return this QuestLinkDraft object as a Link object.
  **/
  public Link asLink() {
    return (Link) this;
  }

  /**
   * Write the QuestLinkDraft object to the database.
   * This method assumes that a DatabaseSystem object for
   * the DatabaseSystem.QUEST database has already been connected.
   * 
   * @return 0 for success, nonzero for failure.
  **/
  public int writeToDatabase() {
    LogSystem.log(2, ">>> Starting QuestLinkDraft.writeToDatabase()");

    /*--------------------------------------------------------------*/
    /*   Assign database keys for this Link and its subobjects.     */
    /*--------------------------------------------------------------*/
    if ( !assignDatabaseKeys() ) {
      return -1;
    }

    SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                    "QuestLinkDraft.writeToDatabase()", 5 );
    int returnCode = 0 ;
    StoredProcRec retRec = null;

    try {
      CallableStatement stmt = sqlm.createStoredProc ( "QUEST.sqlRunner", 2 );
      String sqlToRun = LinkDraftWriter.getSQL(this);
      if (sqlToRun.length() == 0) {
        LogSystem.log(1, "No changes to be written");
        return 0;
      }
      String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                    SqlRunner.END_DELIM, "\n" );

      LogSystem.log ( 1, "Starting to send statements to SQLRUNNER" ) ;
      LogSystem.log ( 1, debugString ) ;

      /*-----------------------------------------------------------*/
      /*   Set up the Callable Stmt                                */
      /*-----------------------------------------------------------*/
      stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
      stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
      stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      

      /*-----------------------------------------------------------*/
      /*   Execute the SQL                                         */
      /*-----------------------------------------------------------*/
      stmt.execute() ;

      /*-----------------------------------------------------------*/
      /*   Get the StoredProcRec from the Lob                      */
      /*-----------------------------------------------------------*/
      retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 2 );
      stmt.close() ;

      if (retRec == null) {
        LogSystem.log( 2, "StoredProcRec is null - error status unknown");
      }

      if ( ( retRec != null ) && ( retRec.error ) ) {
        LogSystem.log( 2, retRec.errorStr ) ;
        returnCode = retRec.sqlCode ;
        if (returnCode == 0) returnCode = -1;
      }
    } 
    catch ( Exception e ) {
       sqlm.rollBack() ;
       LogSystem.log ( 3, e ) ;
       returnCode = -1 ;
    }
    finally {
       sqlm.close() ;
    }

    LogSystem.log(2, ">>> QuestLinkDraft.writeToDatabase() complete");

    return returnCode;
  }

  /**
   * Validate this object for publishing.  
   * 
   * @return a ValidationRec with the results of the validation or
   *         null if an error occurs.  Check the ValidationRec to 
   *         determine if publishing can proceed.
   * @see com.ibm.nzna.projects.common.storedProc.oaValidate.ValidationRec
  **/
  public ValidationRec validate() {
    LogSystem.log(2, ">>> Starting QuestLinkDraft.validate()");
    SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "", 10 );
    ValidationRec retRec = null;

    try {
      CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAValidate", 3 );

      /*-----------------------------------------------------------*/
      /*   Set up the Callable Stmt                                */
      /*-----------------------------------------------------------*/
      stmt.registerOutParameter( 3, Types.LONGVARBINARY ) ;
      stmt.setInt( 1, this.getInd() );         // object ind
      stmt.setInt( 2, OAValidate.LINK );       // object type

      /*-----------------------------------------------------------*/
      /*   Execute the SQL                                         */
      /*-----------------------------------------------------------*/
      stmt.execute() ;

      /*-----------------------------------------------------------*/
      /*   Get the StoredProcRec from the Lob                      */
      /*-----------------------------------------------------------*/
      retRec = (ValidationRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
      stmt.close() ;

      if (retRec == null) {
        LogSystem.log( 2, "ValidationRec was returned null");
      }

      if ( ( retRec != null ) && ( retRec.error ) ) {
        LogSystem.log( 2, retRec.errorStr ) ;
        retRec = null;   // return error to the user
      }
    } 
    catch ( Exception e ) {
      sqlm.rollBack() ;
      LogSystem.log ( 3, e ) ;
      retRec = null;
    }
    finally {
      sqlm.close() ;
    }

    LogSystem.log(2, ">>> QuestLinkDraft.validate() complete");

    return retRec;
  }

  /**
   * Publish the draft QuestLink.
   * Any changes to the draft must be saved to the database by
   * calling writeToDatabase() before the draft is published (the
   * publish method will not save the current object).
   * 
   * @return true for success, false for failure.
  **/
  public boolean publish() {
    LogSystem.log(2, ">>> Starting QuestLinkDraft.publish()");

    SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                    "QuestLinkDraft.publish()", 10 );
    boolean success = true;
    StoredProcRec retRec = null;

    try {
      CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAPublish", 3 );

      /*-----------------------------------------------------------*/
      /*   Set up the Callable Stmt                                */
      /*-----------------------------------------------------------*/
      stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
      stmt.setInt( 1, this.getInd() );      // object ind
      stmt.setInt( 2, OAPublish.LINK );     // object type

      /*-----------------------------------------------------------*/
      /*   Execute the SQL                                         */
      /*-----------------------------------------------------------*/
      stmt.execute() ;

      /*-----------------------------------------------------------*/
      /*   Get the StoredProcRec from the Lob                      */
      /*-----------------------------------------------------------*/
      retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
      stmt.close() ;

      if (retRec == null) {
        LogSystem.log( 2, "StoredProcRec is null - error status unknown");
      } else {
        // the userError String will contain the SQL generated for
        // publishing if DEBUG=true in the OAPublish stored procedure.
        if (retRec.userError.length() > 0) {
          LogSystem.log(1, "Publish SQL:");
          LogSystem.log(2, retRec.userError);
        }
      }

      if ( ( retRec != null ) && ( retRec.error ) ) {
        LogSystem.log( 2, retRec.errorStr ) ;
        success = false;
      }
    } 
    catch ( Exception e ) {
      sqlm.rollBack() ;
      LogSystem.log ( 3, e ) ;
      success = false;
    }
    finally {
      sqlm.close() ;
    }

    LogSystem.log(2, ">>> QuestLinkDraft.publish() complete");

    return success;
  }

  /**
   * Read a QuestLinkDraft object from the database.
   * 
   * @param linkDraftInd the database indicator of the draft to be read.
   * @param languageInd  the language to read the text items for 
   *                     (0 = read all languages).
   * @return the QuestLinkDraft object or null if an error occured or
   *         the object was not found.  The QuestLinkDraft object is
   *         returned as a Link object -- it must be cast.
  **/
  public static Link readFromDatabase(int linkDraftInd, int languageInd) {
    LogSystem.log(2, ">>> Starting QuestLinkDraft.readFromDatabase()");
    SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                   "QuestLinkDraft.readFromDatabase()", 5 );
    StoredProcRec retRec    = null;
    LinkDraft     retLink   = null;

    try {
      CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

      /*-----------------------------------------------------------*/
      /*   Set up the Callable Stmt                                */
      /*-----------------------------------------------------------*/
      stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
      stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
      stmt.setInt( 1, linkDraftInd );           // object ind
      stmt.setInt( 2, OAReader.LINK );          // object type
      stmt.setInt( 3, OAReader.READ_DRAFT );    // read draft object
      stmt.setInt( 4, languageInd );            // language ind
      stmt.setInt( 5, OAMode.getMode() );       // current mode

      /*-----------------------------------------------------------*/
      /*   Execute the SQL                                         */
      /*-----------------------------------------------------------*/
      stmt.execute() ;

      /*-----------------------------------------------------------*/
      /*   Get the Link and the StoredProcReturnRec objects        */
      /*   from the returned LOBs                                  */
      /*-----------------------------------------------------------*/
      retLink = (LinkDraft)StoredProcUtil.getObjectFromStatement( stmt, 6 );
      retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
      stmt.close() ;

      if (retRec == null) {
        LogSystem.log( 2, "StoredProcRec is null - error status unknown");
      }

      if ( ( retRec != null ) && ( retRec.error ) ) {
        LogSystem.log( 2, retRec.errorStr ) ;
      } else {
        if ( retLink != null ) {
          // update the Type*Rec objects
          retLink.asLink().updateTypeRecData();
        } else {
          LogSystem.log(2, "No object found for indicator " + linkDraftInd);
        }
      }

    } catch ( Exception e ) {
       sql.rollBack() ;
       LogSystem.log ( 3, e ) ;
    }

    sql.close() ;
    LogSystem.log(2, ">>> QuestLinkDraft.readFromDatabase() completed");

    return ( (retLink != null) ? retLink.asLink() : null );
  }

  /**
   * Get a draft QuestLink.  This method will return the open draft of an
   * QuestLink if one exists.  If there is no open draft, a new one will
   * be created.
   * <b>Note:</b> If a new draft is created, the openDraft flag in the
   * oa.link table will be set to 'Y'.  This flag will be reset to
   * 'N' when the draft is published.  If the draft is not saved, the
   * openDraft flag on oa.link will have to be manually reset.
   * 
   * @see ExternalLinkDraft#getDraft()
   * 
   * @param linkInd     the database indicator of the published Link 
   *                    whose draft is to be read.
   * @param languageInd the language to read the text items for 
   *                    (0 = read all languages).
   * @param typeWork    the TypeWorkRec to be set into the
   *                    returned draft object if it is a new draft
   * @return the LinkDraft object or null if an error occured or
   *         the object was not found.
   * @exception PendingDraftException if there is a pending draft
  **/
  public static LinkDraft getDraft(int linkInd, int languageInd,
                                   TypeWorkRec typeWork)
                throws PendingDraftException
  {
    LogSystem.log(2, ">>> Starting QuestLinkDraft.getDraft()");
    SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                   "QuestLinkDraft.getDraft()", 5 );
    StoredProcRec retRec    = null;
    LinkDraft     retLink   = null;

    try {
      CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

      /*-----------------------------------------------------------*/
      /*   Set up the Callable Stmt                                */
      /*-----------------------------------------------------------*/
      stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
      stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
      stmt.setInt( 1, linkInd );                // object ind
      stmt.setInt( 2, OAReader.LINK );          // object type
      stmt.setInt( 3, OAReader.GET_DRAFT );     // read draft object
      stmt.setInt( 4, languageInd );            // language ind
      stmt.setInt( 5, OAMode.getMode() );       // current mode

      /*-----------------------------------------------------------*/
      /*   Execute the SQL                                         */
      /*-----------------------------------------------------------*/
      stmt.execute() ;

      /*-----------------------------------------------------------*/
      /*   Get the LinkDraft and the StoredProcReturnRec objects   */
      /*   from the returned LOBs                                  */
      /*-----------------------------------------------------------*/
      retLink = (LinkDraft)StoredProcUtil.getObjectFromStatement( stmt, 6 );
      retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
      stmt.close() ;

      if (retRec == null) {
        LogSystem.log( 2, "StoredProcRec is null - error status unknown");
      }

      if ( ( retRec != null ) && ( retRec.error ) ) {
        LogSystem.log( 2, retRec.errorStr ) ;
      } else {
        if ( retLink != null ) {
          if (retLink.asLink().getRecStatus() == REC_INSERT) {
            retLink.setTypeWorkRequired( typeWork );
          }
          // update the Type*Rec objects
          retLink.asLink().updateTypeRecData();
        } else {
          LogSystem.log(2, "No object found for indicator " + linkInd);
        }
      }

    } catch ( Exception e ) {
       sql.rollBack() ;
       LogSystem.log ( 3, e ) ;
    }

    sql.close() ;
    LogSystem.log(2, ">>> QuestLinkDraft.getDraft() completed");

    // throw exception if the return code indicates a pending draft
    if ( retRec != null && retRec.error && 
         retRec.sqlCode == OAReader.PENDING_DRAFT_RC )
    {
      throw new PendingDraftException();
    }

    return retLink;
  }

  /*------------------------------------------------------------*/
  /* setComments
   */
  /**
   *    This method will set the comments
   *  
   *  
   *
   *  @param None
   *  @param None
   *  @param None
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
  public void setComments ( String comments ) {
      comments_ = comments;
      updateRecStatus(REC_UPDATE);
  }




  /*------------------------------------------------------------*/
  /* getComments
   */
  /**
   *    this method will return the comments
   *  
   *  
   *
   *  @param None
   *  @param None
   *  @param None
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
  public String getComments () {
     return comments_;
  }



   /*------------------------------------------------------------*/
   /* forceCloseDraft
    */
   /**
    *    This method will set the value for 'OPENDRAFT'
    * in the OA.LINK table to 'N' if there are no
    * open drafts.
    *  
    *  
    *
    *  @param databaseInd The database indicator for the LINK
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
  public static void forceCloseDraft ( int databaseInd ) {
     SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "forceCloseDraft", 5 ) ;

     try {
        Statement stmt = sqlMethod.createStatement() ;

        // update openDraft only if there are no open drafts
        stmt.executeUpdate ( 
          "UPDATE oa.link SET openDraft='N' " +
          "WHERE linkInd=" + databaseInd + " " +
          "AND (" +
             "SELECT linkInd FROM oadraft.link " +
             "WHERE activeLinkInd=" + databaseInd + " " +
             "AND stopDate IS NULL" +
          ") IS NULL"
        );

     } catch ( Exception e ) {
        sqlMethod.rollBack() ;
        LogSystem.log ( 1, e ) ;
     }
     sqlMethod.close() ;
  }


}
