
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.db.*;
import java.sql.*;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Online Assistant Question.
 * A Question is a decision point in the problem diagnosis logic
 * flow of a Symptom.
 * <p>
 * A Question contains the following conditional objects:
 * <ul>
 * <li>A Vector of Answer objects.
 * <li>A Vector of Title objects.
 * <li>A Vector of AddtlInfo objects.
 * <li>A Vector of DocLink objects.
 * </ul>
 *
 * @version 1.38
 * @author  Mark DeBiase
**/
public class Question extends OAObject {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   protected int              questionInd_    = 0;
   protected Vector           answers_        = null;
   protected Vector           titles_         = null;
   protected Vector           addtlInfo_      = null;
   protected Vector           docLinks_       = null;
   protected Hashtable        answersByInd_   = null;
   protected TypeQuestionRec  questionType_   = null;
   protected boolean          hiddenAnswers_  = false;    // do we need this?
   protected boolean          autoFill_       = false;

   public static final String OA_PREFIX = "Q";

   /**
    * New Question.
    * recStatus is set to REC_INSERT.
   **/
   public Question() {
      this(0);
   }

   /**
    * Question to be read from database.
    * recStatus is set to REC_NOCHANGE.
   **/
   public Question(int questionInd) {
      super();
      setInd(questionInd);
      if ( questionInd_ == 0 ) {
         updateRecStatus(REC_INSERT);  // new object
      }
   }

   /**
    * Initialize from an existing Question object.
    * Called by QuestionDraft(Question) constructor.
    * Used by the OAReader stored procedure to create a new draft Question
    * from a published Question.
   **/
   Question(Question q) {
      super(q);
      setInd(0);

      initSubObjects(q);

      updateRecStatus(REC_INSERT);
   }

   /**
    * Initialize from an existing QuestionDraft object.
    * Used when publishing a draft Question.
   **/
   Question(QuestionDraft qd) {
      super(qd);
      setInd( qd.getActiveQuestionInd() );

      initSubObjects(qd);

      if ( questionInd_ == 0 ) {
         updateRecStatus(REC_INSERT);
      } else {
         updateRecStatus(REC_UPDATE);
      }
   }

   /**
    * Initialize the Answer, Title, AddtlInfo and DocLink objects 
    * contained in this object.
    * Calls the resetNew() method on each object.  This method will
    * set the indicator to 0, set the recStatus to REC_INSERT and
    * clear the dbuser, changedTime and creationTime values.
   **/
   protected void initSubObjects(Question otherQuestion) {
      // NOTE: this initialization is not safe in that Vectors are
      //       shared between the new object and the original object

      this.answers_       = otherQuestion.answers_;
      this.titles_        = otherQuestion.titles_;
      this.addtlInfo_     = otherQuestion.addtlInfo_;
      this.docLinks_      = otherQuestion.docLinks_;
      this.questionType_  = otherQuestion.questionType_;
      this.hiddenAnswers_ = otherQuestion.hiddenAnswers_;
      this.autoFill_      = otherQuestion.autoFill_;

      boolean draft = false;
      if ( this instanceof QuestionDraft ) {
         draft = true;
      }

      if ( answers_ != null ) {
         for ( int i=0; i<answers_.size(); i++ ) {
            Answer ans = (Answer) answers_.elementAt(i);
            ans.setDraft(draft);
            ans.resetNew();
         }
      }
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title t = (Title) titles_.elementAt(i);
            t.resetNew();
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            a.resetNew();
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            d.resetNew();
         }
      }
   }

   /**
    * Assign database keys for this object and all objects (Answer, 
    * Title, AddtlInfo, DocLink) contained therein.
    * <b>Note:</b> Database keys will not be assigned for Link or
    * LinkGroup objects contained within DocLink objects.
    * @return true if database keys were assigned, otherwise false
   **/
   // this method assigns database keys for both Question & QuestionDraft objs
   public boolean assignDatabaseKeys() {
      boolean success;
      boolean draft = false;

      if ( this instanceof QuestionDraft ) {
         draft = true;
      }

      success = assignMyDatabaseKey();

      if ( answers_ != null ) {
         for ( int i=0; i<answers_.size() && success; i++ ) {
            Answer a = (Answer) answers_.elementAt(i);
            if ( draft ) {
               a.setDraft(true);  // in case its not already set for this Answer....
            }
            success = a.assignDatabaseKeys();
         }
      }
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size() && success; i++ ) {
            Title t = (Title) titles_.elementAt(i);
            if ( draft ) {
               success = t.assignDatabaseKeyForQuestionDraft();
            } else {
               success = t.assignDatabaseKeyForQuestion();
            }
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size() && success; i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            if ( draft ) {
               success = a.assignDatabaseKeyForQuestionDraft();
            } else {
               success = a.assignDatabaseKeyForQuestion();
            }
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size() && success; i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            if ( draft ) {
               success = d.assignDatabaseKeyForQuestionDraft();
            } else {
               success = d.assignDatabaseKeyForQuestion();
            }
         }
      }

      return(success);
   }
   // version to be called by publish stored procedure:
   void assignDatabaseKeys(Connection conn) throws Exception {
      boolean draft = false;

      if ( this instanceof QuestionDraft ) {
         draft = true;
      }

      assignMyDatabaseKey(conn);

      if ( answers_ != null ) {
         for ( int i=0; i<answers_.size(); i++ ) {
            Answer a = (Answer) answers_.elementAt(i);
            if ( draft ) {
               a.setDraft(true);  // in case its not already set for this Answer....
            }
            a.assignDatabaseKeys(conn);
         }
      }
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title t = (Title) titles_.elementAt(i);
            if ( draft ) {
               t.assignDatabaseKeyForQuestionDraft(conn);
            } else {
               t.assignDatabaseKeyForQuestion(conn);
            }
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            if ( draft ) {
               a.assignDatabaseKeyForQuestionDraft(conn);
            } else {
               a.assignDatabaseKeyForQuestion(conn);
            }
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            if ( draft ) {
               d.assignDatabaseKeyForQuestionDraft(conn);
            } else {
               d.assignDatabaseKeyForQuestion(conn);
            }
         }
      }

      return;
   }

   /**
    * Assign a database key for this Question.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignMyDatabaseKey() {
      boolean success = true;

      if ( questionInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for Question");
         questionInd_ = Counter.getCounter( LogSystem.getInstance(), "QSTNIND" );
         if ( questionInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for Question");
            success = false;
         }
      }

      return(success);
   }
   // version to be called by publish stored procedure:
   void assignMyDatabaseKey(Connection conn) throws Exception {

      if ( questionInd_ == 0 ) {
         questionInd_ = Counter.getCounter( conn, "QSTNIND" );
         if ( questionInd_ == 0 ) {
            throw new Exception("Failed to get database key for Question");
         }
      }

      return;
   }

   /**
    * Returns true if any part of this object needs to be saved to the
    * database.
    * As a side effect of calling this method, the sort order on the
    * Conditional objects (Answers, Titles, AddtlInfos, DocLinks) is updated.
   **/
   public boolean objectModified() {
      boolean modified = super.modified();

      if ( answers_ != null ) {
         int sortOrder = 1;
         for ( int i=0; i<answers_.size() && !modified; i++ ) {
            Answer a = (Answer) answers_.elementAt(i);
            if ( a.valid() ) {
               a.setSortOrder( sortOrder++ );
            }
            modified = modified || a.objectModified();
         }
      }
      if ( titles_ != null ) {
         int sortOrder = 1;
         for ( int i=0; i<titles_.size() && !modified; i++ ) {
            Title t = (Title) titles_.elementAt(i);
            if ( t.valid() ) {
               t.setSortOrder( sortOrder++ );
            }
            modified = modified || t.modified();
         }
      }
      if ( addtlInfo_ != null ) {
         int sortOrder = 1;
         for ( int i=0; i<addtlInfo_.size() && !modified; i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            if ( a.valid() ) {
               a.setSortOrder( sortOrder++ );
            }
            modified = modified || a.modified();
         }
      }
      if ( docLinks_ != null ) {
         int sortOrder = 1;
         for ( int i=0; i<docLinks_.size() && !modified; i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            if ( d.valid() ) {
               d.setSortOrder( sortOrder++ );
            }
            modified = modified || d.modified();
         }
      }

      return(modified);
   }

   /**
    * Returns this object's indicator.
   **/
   public int getInd() {
      return(questionInd_);
   }

   /**
    * Sets this object's indicator.
   **/
   void setInd(int questionInd) {
      questionInd_ = questionInd;
      setOAItemNum(questionInd_);
   }

   /**
    * Set the OAItemNumber
   **/
   void setOAItemNum(int itemNum) {
      super.setOAItemNum(OA_PREFIX + itemNum);
   }

   /**
    * Gets the list of conditional Answers.
    * <b>Note:</b>
    * Conditions on Answers are only used to determine whether or not
    * the Answer is displayed.
    * @return a Vector of Answer objects
    * @see Answer
   **/
   public Vector getAnswers() {
      if ( answers_ == null ) {
         answers_ = new Vector();
      }
      return(answers_);
   }

   /**
    * Sets the list of conditional Answers.
    * <b>Note:</b>
    * Conditions on Answers are only used to determine whether or not
    * the Answer is displayed.
    * @param a Vector of Answer objects
    * @see Answer
   **/
   public void setAnswers(Vector answers) {
      answers_ = answers;
      // dont update recstatus
   }

   /**
    * Return the Answer with the specified indicator.
    * <b>Note:</b> This method uses a Hashtable that indexes
    * each Answer by its database indicator to look up the
    * specified Answer object.  Therefore this method is only
    * useful for Answers that have been assigned database indicators.
   **/
   public Answer getAnswerByInd(int answerInd) {
      if ( answersByInd_ == null ) {
         buildAnswerTable();
      }
      return(Answer) answersByInd_.get( new Integer(answerInd) );
   }

   /**
    * Creates a Hashtable that indexes the Answers to this Question
    * by database indicator.  Use the getAnswerByInd() method to
    * retrieve the Answer object for a given indicator.
   **/
   private void buildAnswerTable() {
      answersByInd_ = new Hashtable();
      for ( int i=0; i<answers_.size(); i++ ) {
         Answer a = (Answer) answers_.elementAt(i);
         answersByInd_.put( new Integer(a.getInd()), a );
      }
   }

   /**
    * Gets the conditional Titles.
    * <p>
    * <b>Note:</b> The last Title in the Vector should have no conditions
    * on it (other than a document classification of external).  It will
    * be used as the Default Title.
    *
    * @return a Vector of Title objects or null if there are no titles.
    * @see Title
   **/
   public Vector getTitles() {
      return(titles_);
   }

   /**
    * Sets the conditional Titles.
    * <p>
    * <b>Note:</b> The last Title in the Vector should have no conditions
    * on it. It will be used as the Default Title. It will also govern the
    * document classification of this object
    *
    * @param titles a Vector of Title objects.
    * @see Title
   **/
   public void setTitles(Vector titles) {
      titles_ = titles;

      if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) {
         docClass_ = ((Title)titles_.elementAt(titles_.size()-1)).getDocClass() ;
      }
      // dont update recstatus
   }

   public TypeDocClassRec getDocClass() {

      if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) {
         return( ((Title)titles_.elementAt(titles_.size()-1)).getDocClass() ) ;
      }

      return(docClass_);
   }


   /**
    * Gets the list of conditional additional information texts.
    * @return a Vector of AddtlInfo objects or null if there is 
    *         no additional information.
    * @see AddtlInfo
   **/
   public Vector getAdditionalInfo() {
      return(addtlInfo_);
   }

   /**
    * Sets the list of conditional additional information texts.
    * @param addtlInfo a Vector of AddtlInfo objects
    * @see AddtlInfo
   **/
   public void setAdditionalInfo(Vector addtlInfo) {
      addtlInfo_ = addtlInfo;
      // dont update recstatus
   }

   /**
    * Gets the list of conditional document links.
    * @return a Vector of DocLink objects or null if there are no DocLinks.
    * @see DocLink
   **/
   public Vector getDocLinks() {
      return(docLinks_);
   }

   /**
    * Sets the list of conditional document links.
    * @param docLinks a Vector of DocLink objects
    * @see DocLink
   **/
   public void setDocLinks(Vector docLinks) {
      docLinks_ = docLinks;
      // dont update recstatus
   }

   /**
    * Gets the Question type.
    * @return the question type.
    * @see com.ibm.nzna.projects.common.quest.type.TypeQuestionRec
   **/
   public TypeQuestionRec getQuestionType() { 
      return(questionType_);
   }

   /**
    * Sets the Question type.
    * @param questionType the question type.
    * @see com.ibm.nzna.projects.common.quest.type.TypeQuestionRec
   **/
   public void setQuestionType(TypeQuestionRec questionType) {
      questionType_ = questionType;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns true if the hidden answers flag is set.
    * <b>Is this needed?</b>
   **/
   public boolean getHiddenAnswers() {
      return(hiddenAnswers_);
   }

   /**
    * Sets the hidden answers flag.
    * <b>Is this needed?</b>
   **/
   public void setHiddenAnswers(boolean hiddenAnswers) {
      hiddenAnswers_ = hiddenAnswers;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns true if the autofill flag is set.
    * If this method returns true and there is only one Answer to be
    * displayed for the Question (based on the conditions on the Answers),
    * then the Question should be automatically answered and not displayed
    * to the user.
   **/
   public boolean getAutoFill() {
      return(autoFill_);
   }

   /**
    * Sets the autofill flag.
    * @see #getAutoFill
   **/
   public void setAutoFill(boolean autoFill) {
      autoFill_ = autoFill;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Perform a shallow copy of this object.
   **/
   public Object clone() {

      Question cloned = (Question) super.clone();

      // Question data:
      if ( answers_ != null ) {
         cloned.answers_  = (Vector) answers_.clone();
      }
      if ( answersByInd_ != null ) {
         cloned.answersByInd_ = (Hashtable) answersByInd_.clone();
      }
      if ( addtlInfo_ != null ) {
         cloned.addtlInfo_  = (Vector) addtlInfo_.clone();
      }
      if ( docLinks_ != null ) {
         cloned.docLinks_ = (Vector) docLinks_.clone();
      }

      return(cloned);
   }

   /**
    * Return a String representation of this object.
   **/
   public String toString() {
      return("Question " + getInd());
   }

   /**
    * Write the Question and all its contained objects (with the exception of
    * Link and LinkGroup objects contained in DocLink objects) to the
    * database.  This method assumes that a DatabaseSystem object for
    * the DatabaseSystem.QUEST database has already been connected.
    * 
    * @return 0 for success, nonzero for failure.
   **/
   public int writeToDatabase() {
      LogSystem.log(2, ">>> Starting Question.writeToDatabase()");

      /*--------------------------------------------------------------*/
      /*   Assign database keys for this Question and its subobjects. */
      /*--------------------------------------------------------------*/
      if ( !assignDatabaseKeys() ) {
         return(-1);
      }

      // if current recstatus is REC_INSERT && the call to SqlRunner
      // is successful, the recstatus of this object should be updated 
      // regardless of the success of the addtl info text updates.
      boolean forceChangeRecStatus = false;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "Question.writeToDatabase()", 10 );
      int returnCode = 0 ;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ("QUEST.ExtSqlRunner", 3);
         String sqlToRun = QuestionWriter.getSQL(this);
         if ( sqlToRun.length() == 0 ) {
            LogSystem.log(1, "No changes to be written");
            return(0);
         }
         String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                       SqlRunner.END_DELIM, "\n" );

         LogSystem.log ( 1, "Starting to send statements to EXTSQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      
         stmt.setInt   ( 2, 0 );  // commit interval

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            returnCode = retRec.sqlCode ;
            if ( returnCode == 0 ) returnCode = -1;
         } else {
            if ( this.getRecStatus() == REC_INSERT ) {
               forceChangeRecStatus = true;
            }

            // use a PreparedStatement to update the AddtlInfo text
            PreparedStatement pstmt = null;

            for ( int i=0; addtlInfo_ != null && i < addtlInfo_.size(); i++ ) {
               AddtlInfo ai = (AddtlInfo) addtlInfo_.elementAt(i);
               int recStatus = ai.getRecStatus();
               if ( recStatus == REC_INSERT || recStatus == REC_UPDATE ) {
                  LogSystem.log ( 1, "Updating additional information text for " +
                                  ai.getInd() ) ;
                  //LogSystem.log ( 1, "Additional information text: >>>" +
                  //                   ai.getAddtlInfo() + "<<<");
                  if ( pstmt == null ) {
                     // don't prepare the statement until we know it is needed.
                     pstmt = sqlm.createPreparedStatement(
                                                         "UPDATE oa.questionAddInfo " +
                                                         "SET addInfo=? " +
                                                         "WHERE questionAddInfoInd=?" +
                                                         "AND languageInd=?"
                                                         );
                  }
                  pstmt.setString( 1, ai.getAddtlInfo() );
                  pstmt.setInt( 2, ai.getInd() );
                  pstmt.setInt( 3, ai.getLanguage().getInd() );
                  pstmt.executeUpdate();
               }
            }

            if ( pstmt != null ) {
               pstmt.close();
            }
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         returnCode = -1 ;
      } finally {
         sqlm.close() ;
      }

      if ( returnCode == 0 || forceChangeRecStatus ) {
         setRecNoChange();      // set all to REC_NOCHANGE
      }

      LogSystem.log(2, ">>> Question.writeToDatabase() complete");

      return(returnCode);
   }

   /**
    * Set the recStatus of this object and 
    * all its contained objects to REC_NOCHANGE.
   **/
   void setRecNoChange() {

      // this object:
      updateRecStatus(REC_NOCHANGE);
      super.setOriginalCountryListFromCurrent();

      // contained objects:
      if ( answers_ != null ) {
         for ( int i=0; i<answers_.size(); i++ ) {
            Answer a = (Answer) answers_.elementAt(i);
            a.setRecNoChange();
         }
      }
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title t = (Title) titles_.elementAt(i);
            t.setRecNoChange();
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo a = (AddtlInfo) addtlInfo_.elementAt(i);
            a.setRecNoChange();
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink d = (DocLink) docLinks_.elementAt(i);
            d.setRecNoChange();
         }
      }
   }

   /**
    * Read a Question object from the database.
    * 
    * @param questionInd the database indicator of the Question to be read.
    * @param languageInd the language to read the text items for 
    *                    (0 = read all languages).
    * @return the Question object or null if an error occured or
    *         the object was not found.
   **/
   public static Question readFromDatabase(int questionInd, int languageInd) {
      LogSystem.log(2, ">>> Starting Question.readFromDatabase()");
      long start = System.currentTimeMillis();
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "Question.readFromDatabase()", 5 );
      StoredProcRec retRec      = null;
      Question      retQuestion = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, questionInd );            // object ind
         stmt.setInt( 2, OAReader.QUESTION );      // object type
         stmt.setInt( 3, OAReader.READ_ACTIVE );   // read active object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Question and the StoredProcReturnRec objects    */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retQuestion = (Question)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      long end = System.currentTimeMillis();
      LogSystem.log(1, "Elapsed time: " + (end-start) + " msec");
      LogSystem.log(2, ">>> Question.readFromDatabase() completed");

      if ( retQuestion != null ) {
         // performance: dont update TypeRec data if in web mode
         if ( OAMode.currentMode != OAMode.WEB ) {
            retQuestion.updateTypeRecData();  // update the Type*Rec objects
            retQuestion.setRecNoChange();     // sets the original country list
         }
      } else {
         LogSystem.log(2, "No object found for indicator " + questionInd);
      }

      return(retQuestion);
   }

   /**
    * Update the TypeRec objects in the Question's subobjects with 
    * the proper descriptions.
    * Called by readFromDatabase();
   **/
   void updateTypeRecData() {
      super.updateTypeRecData();

      TypeList t = TypeList.getInstance();

      // question type
      TypeQuestionRec newQuestionType;
      newQuestionType = (TypeQuestionRec) t.objectFromInd( questionType_.getInd(),
                                                           TypeList.TYPE_QUESTION );
      if ( newQuestionType != null ) {
         questionType_ = newQuestionType;
      }

      // update sub-objects TypeRec data:
      if ( answers_ != null ) {
         for ( int i=0; i<answers_.size(); i++ ) {
            Answer ans = (Answer) answers_.elementAt(i);
            ans.updateTypeRecData();
         }
      }
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            Title title = (Title) titles_.elementAt(i);
            title.updateTypeRecData();
         }
      }
      if ( addtlInfo_ != null ) {
         for ( int i=0; i<addtlInfo_.size(); i++ ) {
            AddtlInfo ai = (AddtlInfo) addtlInfo_.elementAt(i);
            ai.updateTypeRecData();
         }
      }
      if ( docLinks_ != null ) {
         for ( int i=0; i<docLinks_.size(); i++ ) {
            DocLink dl = (DocLink) docLinks_.elementAt(i);
            dl.updateTypeRecData();
         }
      }

      return;
   }

   /**
    * Find an open draft for a Question if one exists.
    * This method requires that a connection to DatabaseSystem.QUEST 
    * is already established.
    * 
    * @param questionInd the database indicator of the active (published)
    *                    Question to find a draft for.
    * @return the indicator of the open draft for the Question or 0 if there
    *         is no open draft or an error occurs.
   **/
   public static int findOpenDraft(int questionInd) {
      LogSystem.log(2, ">>> Starting Question.findOpenDraft()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "Question.findOpenDraft()", 1 );

      int draftQuestionInd = 0;

      try {
         Statement stmt = sql.createStatement();
         ResultSet rs = stmt.executeQuery(
                                         "SELECT questionInd " +
                                         "FROM oadraft.question " +
                                         "WHERE activeQuestionInd=" + questionInd + " " +
                                         "AND stopDate IS NULL " +
                                         "FOR FETCH ONLY"
                                         );

         if ( rs.next() ) {
            draftQuestionInd = rs.getInt(1);
         } else {
            LogSystem.log(1, "No open draft found for " + questionInd);
         }
         rs.close();
      } catch ( Exception e ) {
         LogSystem.log ( 3, e );
      } finally {
         sql.close();
      }

      LogSystem.log(2, ">>> Question.findOpenDraft() completed");

      return(draftQuestionInd);
   }





   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the default title
    *  to this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Title if there is a default title, NULL if no titles exist
    *//*---------------------------------------------------------*/
   public Title getDefaultTitle () {
      if ( ( titles_ != null ) && ( titles_.size() > 0 ) )
         return ( (Title)titles_.elementAt ( titles_.size()-1 ) ) ;
      return ( null ) ;
   }
}

