
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.util.* ;
import java.sql.*;


/**
 * Reader for Online Assistant Draft Questions.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.8
 * @author VishnuKumar.E
**/
public class QuestionDraftReader {

  /**
   * Read and populate the specified QuestionDraft object.
   * The Question object's indicator should contain the questionInd to be read.
   * @param conn  the database Connection object
   * @param q     the Question to populate
   * @param lang  the language to read the sub-objects for (0 = all languages)
   * @return true if the Question was successfully read
             false if the Question was not found in the database
  **/
  public static boolean readFromDB(Connection conn, QuestionDraft qd, 
                                   int languageInd)
                        throws SQLException
  {
    boolean foundQuestion;
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // turn on DB2 explain:
    //stmt.executeUpdate("SET CURRENT EXPLAIN MODE YES");
    //stmt.executeUpdate("SET CURRENT EXPLAIN SNAPSHOT YES");

    rs = stmt.executeQuery( 
        "SELECT " + 
           "activeQuestionInd, questionTypeInd, hiddenFlag, autoFillFlag, " +
           "brandInd, typeWorkInd, docClassInd, " +
           "maintLock, startDate, stopDate, recycled, " +
	   "archived, created, dbUser, changedTime, comments " +
        "FROM oadraft.question " + 
        "WHERE questionInd=" + qd.getInd() + " " +
        "FOR FETCH ONLY");

    if (!rs.next()) {
      foundQuestion = false;
      stmt.close();
    } else {
      foundQuestion = true;
  
      //---------------------------------------------------------------------
      // Read the base Questiondraft data
      //---------------------------------------------------------------------
      qd.setActiveQuestionInd( rs.getInt(1) );
      qd.setQuestionType( new TypeQuestionRec( rs.getShort(2), "" ) );
      qd.setHiddenAnswers( DatabaseUtil.booleanFromDB( rs.getString(3) ) );
      qd.setAutoFill( DatabaseUtil.booleanFromDB( rs.getString(4) ) ); 
      qd.setBrand( new TypeBrandRec( rs.getInt(5), "", 0 ) );
      qd.setTypeWorkRequired( new TypeWorkRec( rs.getInt(6), "") );
      qd.setDocClass( new TypeDocClassRec( rs.getShort(7), "" ) );
      qd.setMaintLock( DatabaseUtil.booleanFromDB(rs.getString(8) ) );
      qd.setStartDate( rs.getString(9) );
      qd.setStopDate( rs.getString(10) );
      qd.setRecycled( DatabaseUtil.booleanFromDB( rs.getString(11) ) );
      qd.setArchived( DatabaseUtil.booleanFromDB( rs.getString(12) ) );
      qd.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(13) ) );
      qd.setDbUser( rs.getString(14).trim() );
      qd.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(15) ) );
      qd.setComments( rs.getString(16) );
      if ( rs.wasNull() ) {
        qd.setComments("");
      }


      stmt.close();

      //---------------------------------------------------------------------
      // Read the Question Countries
      //---------------------------------------------------------------------
       qd.setCountryList(
          CountryReader.readCountries( conn, "oadraft.questionCountry",
                                       "questionInd", qd.getInd() )
       );

      //---------------------------------------------------------------------
      // Read the Question Answers
      //---------------------------------------------------------------------
      AnswerDraftReader.readAnswersForQuestion( conn, qd, languageInd );

      //---------------------------------------------------------------------
      // Read the Question Titles/AddtlInfo based on the language passed in
      // DRAFT AddtlInfo is always read in DEFAULT mode - only used by QIT.
      //---------------------------------------------------------------------
      TitleReader.readQuestionDraftTitles( conn, qd, languageInd );
      AddtlInfoReader.readQuestionDraftAddtlInfo( conn, qd, languageInd,
                                                  OAMode.DEFAULT );

      //---------------------------------------------------------------------
      // Read the Question DocLinks
      //---------------------------------------------------------------------
      DocLinkReader.readQuestionDraftDocLinks( conn, qd );

      qd.updateRecStatus( qd.REC_NOCHANGE );
      qd.answersByInd_ = null;   // dont send Hashtable back from stored proc
    }

    return foundQuestion;
  }

  /**
   * Get a draft for a Question.
   * Check to see if there is an open draft -- if there is, then return it.
   * If there is no open draft, then create a new one.
   *
   * @exception PendingDraftException if openDraft='Y' on the active question
   *            and there is no open draft with stopDate=NULL
  **/
  public static QuestionDraft getDraft(Connection conn, int questionInd, 
                                       int languageInd) 
                            throws SQLException, PendingDraftException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    QuestionDraft returnedDraft = null;
    

    // check to see if there is already an open draft....
    rs = stmt.executeQuery(
      "SELECT questionInd " +
      "FROM oadraft.question " +
      "WHERE stopDate IS NULL AND activeQuestionInd=" + questionInd + " " +
      "FOR FETCH ONLY"
    );

    if ( rs.next() ) {
      // there is an open draft -- get it.
      int draftQuestionInd = rs.getInt(1);
      rs.close();
      QuestionDraft qd = new QuestionDraft( draftQuestionInd );
      if ( QuestionDraftReader.readFromDB( conn, qd, languageInd ) ) {
        returnedDraft = qd;
      }
    } else {
      // there is no open draft -- read the question & create a new draft.
      rs.close();
      Question q = new Question( questionInd );
      if ( QuestionReader.readFromDB( conn, q, languageInd, OAMode.DEFAULT ) ) {
        if ( q.hasOpenDraft() ) {
          throw new PendingDraftException();
        }
        QuestionDraft qd = new QuestionDraft( q );
        returnedDraft = qd;
        // if we opened a new draft, then update openDraft in oa.question
        stmt.executeUpdate(
          "UPDATE oa.question " +
          "SET openDraft='Y' " + 
          "WHERE questionInd=" + questionInd
        );
      }
    }

    stmt.close();

    return returnedDraft;
  }
}
