
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.db.*;
import java.sql.*;
import java.util.Vector;
import java.util.Hashtable;

/**
 * Online Assistant Symptom.
 * A Symptom represents a problem diagnosis logic flow.  It consists of
 * Nodes and Edges.  A Node represents an Action, Question or another 
 * Symptom or a special action (end the Symtom as resolved or unresolved,
 * display the action list or return from a Symptom subroutine).  An Edge
 * represents an Answer to a Question used in the logic flow and is used
 * to determine the next Node.
 * <p>
 * There is one Node designated as the Root Node which is the first
 * Node in the Symptom.
 * The Root Node of every Symptom should point to Question 0 which 
 * will be set up as a dummy Question with one answer.  The purpose of
 * the dummy Question will be to allow conditions to be placed on the
 * Answer to make it possible to have a conditional root node
 * (the conditions on the answer to Question 0 will determine which
 * node should be processed first).
 * When navigating the Online Assistant on the web, Question 0 should
 * never be displayed to the user.
 *
 * @see Node
 * @see Edge
 * @version 1.40
 * @author  Mark DeBiase
**/
public class Symptom extends OAObject {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   protected int            symptomInd_     = 0;
   protected Node           rootNode_       = null;
   protected Vector         titles_         = null;
   protected Vector         nodes_          = null;
   protected Hashtable      nodesByInd_     = null;  // index Nodes by nodeInd
   protected Vector         edgeGroups_     = null;
   protected TypeSymptomRec symptomType_    = null;
   protected TypeStatusRec  symptomStatus_  = null;
   protected Hashtable      edgesByInd_     = null;  // index Edges by edgeInd


   public static final String OA_PREFIX = "S";

   /**
    * New Symptom.
    * recStatus is set to REC_INSERT.
   **/
   public Symptom() {
      this(0);
   }

   /**
    * Symptom to be read from database.
    * recStatus is set to REC_NOCHANGE.
   **/
   public Symptom(int symptomInd) {
      super();
      setInd(symptomInd);
      if ( symptomInd_ == 0 ) {
         updateRecStatus(REC_INSERT);  // new object
      }
   }

   /**
    * Initialize from an existing Symptom object.
    * Called by SymptomDraft(Symptom) constructor.
    * Used by the OAReader stored procedure to create a new draft Symptom
    * from a published Symptom.
   **/
   Symptom(Symptom s) {
      super(s);
      setInd(0);

      initSubObjects(s);

      updateRecStatus(REC_INSERT);
   }

   /**
    * Initialize from an existing SymptomDraft object.
    * Used when publishing a draft Symptom.
   **/
   Symptom(SymptomDraft sd) {
      super(sd);
      setInd( sd.getActiveSymptomInd() );

      initSubObjects(sd);

      if ( symptomInd_ == 0 ) {
         updateRecStatus(REC_INSERT);
      } else {
         updateRecStatus(REC_UPDATE);
      }
   }

   /**
    * Initialize the Title, AddtlInfo and DocLink objects contained in 
    * this object.
    * Calls the resetNew() method on each object.  This method will
    * set the indicator to 0, set the recStatus to REC_INSERT and
    * clear the dbuser, changedTime and creationTime values.
   **/
   protected void initSubObjects(Symptom otherSymptom) {
      // NOTE: this initialization is not safe in that Vectors are
      //       shared between the new object and the original object

      this.rootNode_      = otherSymptom.rootNode_;
      this.titles_        = otherSymptom.titles_;
      this.nodes_         = otherSymptom.nodes_;
      this.edgeGroups_    = otherSymptom.edgeGroups_;
      this.symptomType_   = otherSymptom.symptomType_;
      this.symptomStatus_ = otherSymptom.symptomStatus_;

      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            SymptomTitle t = (SymptomTitle) titles_.elementAt(i);
            t.resetNew();
         }
      }
      if ( nodes_ != null ) {
         for ( int i=0; i<nodes_.size(); i++ ) {
            Node n = (Node) nodes_.elementAt(i);
            n.resetNew();
         }
      }
      if ( edgeGroups_ != null ) {
         for ( int i=0; i<edgeGroups_.size(); i++ ) {
            EdgeGroup g = (EdgeGroup) edgeGroups_.elementAt(i);
            g.resetNew();
         }
      }
   }

   /**
    * Assign database keys for this object and all objects contained therein.
   **/
   public boolean assignDatabaseKeys() {
      boolean success;
      boolean draft = false;

      if ( this instanceof SymptomDraft ) {
         draft = true;
      }

      success = assignMyDatabaseKey();

      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size() && success; i++ ) {
            SymptomTitle t = (SymptomTitle) titles_.elementAt(i);
            success = t.assignDatabaseKey(draft);
         }
      }
      if ( nodes_ != null ) {
         for ( int i=0; i<nodes_.size() && success; i++ ) {
            Node n = (Node) nodes_.elementAt(i);
            success = n.assignDatabaseKey(draft);
         }
      }
      if ( edgeGroups_ != null ) {
         for ( int i=0; i<edgeGroups_.size() && success; i++ ) {
            EdgeGroup eg = (EdgeGroup) edgeGroups_.elementAt(i);
            success = eg.assignDatabaseKeys(draft);
         }
      }

      return(success);
   }
   // version to be called by publish stored procedure:
   void assignDatabaseKeys(Connection conn) throws Exception {
      boolean draft = false;

      if ( this instanceof SymptomDraft ) {
         draft = true;
      }

      assignMyDatabaseKey(conn);

      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            SymptomTitle t = (SymptomTitle) titles_.elementAt(i);
            t.assignDatabaseKey(conn, draft);
         }
      }
      if ( nodes_ != null ) {
         for ( int i=0; i<nodes_.size(); i++ ) {
            Node n = (Node) nodes_.elementAt(i);
            n.assignDatabaseKey(conn, draft);
         }
      }
      if ( edgeGroups_ != null ) {
         for ( int i=0; i<edgeGroups_.size(); i++ ) {
            EdgeGroup eg = (EdgeGroup) edgeGroups_.elementAt(i);
            eg.assignDatabaseKeys(conn, draft);
         }
      }

      return;
   }

   /**
    * Assign a database key for this Symptom.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignMyDatabaseKey() {
      boolean success = true;

      if ( symptomInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for Symptom");
         symptomInd_ = Counter.getCounter( LogSystem.getInstance(), "SYMPTMIND" );
         if ( symptomInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for Symptom");
            success = false;
         }
      }

      return(success);
   }
   // version to be called by publish stored procedure:
   void assignMyDatabaseKey(Connection conn) throws Exception {

      if ( symptomInd_ == 0 ) {
         symptomInd_ = Counter.getCounter( conn, "SYMPTMIND" );
         if ( symptomInd_ == 0 ) {
            throw new Exception("Failed to get database key for Symptom");
         }
      }

   }

   /**
    * Returns this object's indicator.
   **/
   public int getInd() {
      return(symptomInd_);
   }

   /**
    * Sets this object's indicator.
   **/
   void setInd(int symptomInd) {
      symptomInd_ = symptomInd;
      setOAItemNum(symptomInd_);
   }

   /**
    * Set the OAItemNumber
   **/
   void setOAItemNum(int itemNum) {
      super.setOAItemNum(OA_PREFIX + itemNum);
   }

   /**
    * Gets the conditional Titles.
    * @return a Vector of SymptomTitle objects or null if there are no titles.
   **/
   public Vector getSymptomTitles() {
      return(titles_);
   }

   /**
    * Sets the conditional Titles. This in turn sets the document classification of the
    * symptom to the classification of the title that is default
    * @param titles a Vector of SymptomTitle objects.
   **/
   public void setSymptomTitles(Vector titles) {
      titles_ = titles;

      if ( ( titles_ != null ) && ( titles_.size() > 0 ) ) {
         docClass_ = ((SymptomTitle)titles_.elementAt(titles_.size()-1)).getDocClass() ;
      }

      // dont update recstatus
   }

   /**
    * Get the Root Node indicator.
   **/
   public int getRootNodeInd() {
      return(rootNode_.getInd());
   }

   /**
    * Get the RootNode.
   **/
   public Node getRootNode() {
      return(rootNode_);
   }

   /**
    * Set the RootNode.
   **/
   public void setRootNode(Node rootNode) {
      rootNode_ = rootNode;
   }

   /**
    * Returns a Vector that constains all the Nodes in the Symptom
    * or null if no Nodes have been added yet.
   **/
   public Vector getNodes() {
      return(nodes_);
   }

   /**
    * Sets the list of Nodes.
   **/
   public void setNodes(Vector nodes) {
      nodes_ = nodes;
   }

   /**
    * Return the Node with the specified indicator.
    * <b>Note:</b> This method uses a Hashtable that indexes
    * each Node by its database indicator to look up the
    * specified Node object.  Therefore this method is only
    * useful for Node that have been assigned database indicators.
   **/
   public Node getNodeByInd(int nodeInd) {
      if ( nodesByInd_ == null ) {
         buildNodeTable();
      }
      return(Node) nodesByInd_.get( new Integer(nodeInd) );
   }

   /**
    * Creates a Hashtable that indexes the Nodes in this Symptom
    * by database indicator.  Use the getNodeByInd() method to
    * retrieve the Node object for a given indicator.
   **/
   private void buildNodeTable() {
      nodesByInd_ = new Hashtable();
      for ( int i=0; i<nodes_.size(); i++ ) {
         Node n = (Node) nodes_.elementAt(i);
         nodesByInd_.put( new Integer(n.getInd()), n );
      }
   }

   /**
    * Overloaded version of getEdgesForNodeAndAnswer() which defaults the
    * answerIndIsDraft parameter to false.
   **/
   public Hashtable getEdgesForNodeAndAnswer(int headNodeInd, int answerInd) {
      return(getEdgesForNodeAndAnswer(headNodeInd, answerInd, false));
   }

   /**
    * Get the set of Edges and their associated tail Nodes for the given
    * head node indicator and answer indicator.
    *
    * @param headNodeInd      the head node indicator
    * @param answerInd        the answer indicator
    * @param answerIndIsDraft if true, the answerInd is for a draft answer
    *                         and Edge.getDraftAnswerInd() will be used when
    *                         comparing the answer indicator.  if false,
    *                         the answerInd is for an active answer and 
    *                         Edge.getActiveAnswerInd() will be used when
    *                         comparing the answer indicator.
    * 
    * @return A Hashtable that contains all the Edges and tail Nodes for
    *         the specified headNodeInd/answerInd.  The Hashtable key
    *         will be the matching Edge object and the value will
    *         be the associated tail Node object.
   **/
   public Hashtable getEdgesForNodeAndAnswer(int headNodeInd, int answerInd, 
                                             boolean answerIndIsDraft) 
   {
      Hashtable results = new Hashtable();

      // go through the list of EdgeGroups checking for a match on headNodeInd
      for ( int i=0; i<edgeGroups_.size(); i++ ) {
         EdgeGroup group = (EdgeGroup) edgeGroups_.elementAt(i);

         if ( group.getHeadNode().getInd() == headNodeInd ) {
            Vector edgeVec = group.getEdges();

            // go through the list of Edges checking for a match on answerInd
            for ( int j=0; j<edgeVec.size(); j++ ) {
               Edge edge = (Edge) edgeVec.elementAt(j);
               boolean answersMatch;
               if ( answerIndIsDraft ) {
                  answersMatch = ( edge.getDraftAnswerInd() == answerInd );
               } else {
                  answersMatch = ( edge.getActiveAnswerInd() == answerInd );
               }
               if ( answersMatch ) {
                  results.put( edge, group.getTailNode() );
               }
            }
         }

      }

      return(results);
   }

   /**
    * Get the set of EDGE_TYPE_OTHER edges and their associated tail
    * Nodes for the given head node indicator.  There can be multiple
    * EDGE_TYPE_OTHER edges for a given head node because each edge
    * can have different conditions.
    *
    * @param headNodeInd  the head node indicator
    * 
    * @return A Hashtable that contains all the EDGE_TYPE_OTHER edges
    *         and tail Node for the specified headNodeInd.  The Hashtable
    *         key will be the Edge object and the value will be the 
    *         associated tail Node object.
    *         If there are are no EDGE_TYPE_OTHER edges, then the Hastable
    *         will be empty.
   **/
   public Hashtable getTypeOtherEdgesForNode(int headNodeInd)
   {
      Hashtable results = new Hashtable();

      // go through the list of EdgeGroups checking for a match on headNodeInd
      for ( int i=0; i<edgeGroups_.size(); i++ ) {
         EdgeGroup group = (EdgeGroup) edgeGroups_.elementAt(i);

         if ( group.getHeadNode().getInd() == headNodeInd ) {
            Vector edgeVec = group.getEdges();

            // go through the list of Edges looking for #OTHER edges
            for ( int j=0; j<edgeVec.size(); j++ ) {
               Edge edge = (Edge) edgeVec.elementAt(j);
               if ( edge.getEdgeType() == Edge.EDGE_TYPE_OTHER ) {
                  results.put( edge, group.getTailNode() );
               }
            }
         }

      }

      return(results);
   }

   /**
    * Returns a Vector containing all the EdgeGroups in the Symptom
    * or null if no EdgeGroups have been added yet.
   **/
   public Vector getEdgeGroups() {
      return(edgeGroups_);
   }

   /**
    * Sets the list of EdgeGropus.
    * @param edgeGroups a Vector of EdgeGroups objects
   **/
   public void setEdgeGroups(Vector  edgeGroups) {
      edgeGroups_ = edgeGroups;
   }

   /**
    * Gets the Symptom type.
    * @return TypeSymptomRec the symptom type.
    * @see com.ibm.nzna.projects.common.quest.type.TypeSymptomRec
   **/
   public TypeSymptomRec getSymptomType() {
      return(symptomType_);
   }

   /**
    * Sets the Symptom type.
    * @param symptomType the symptom type.
    * @see com.ibm.nzna.projects.common.quest.type.TypeSymptomRec
   **/
   public void setSymptomType(TypeSymptomRec symptomType) {
      symptomType_ = symptomType;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Gets the Symptom status.
    * @return TypeStatusRec the symptom type.
    * @see com.ibm.nzna.projects.common.quest.type.TypeStatusRec
   **/
   public TypeStatusRec getSymptomStatus() {
      return(symptomStatus_);
   }

   /**
    * Sets the Symptom status.
    * @param symptomType the symptom type.
    * @see com.ibm.nzna.projects.common.quest.type.TypeSymptomRec
   **/
   public void setSymptomStatus(TypeStatusRec symptomStatus) {
      symptomStatus_ = symptomStatus;
      updateRecStatus(REC_UPDATE);
   }

   /*
    * Returns all the tail Nodes and Edges for a given head Node indicator
    * and answer indicator.  
   public Hashtable getEdgesByNodeAndAnswer(int headNodeInd, int answerInd) {
     // not implemented yet - need to be able to handle active/draft answers
     return null;
   }
   **/

   /**
    * Return a String representation of this object.
   **/
   public String toString() {
      return("Symptom " + getInd());
   }


   /**
    * Write the Symptom and all its contained objects to the database.
    * This method assumes that a DatabaseSystem object for the
    * DatabaseSystem.QUEST database has already been connected.
    * 
    * @return 0 for success, nonzero for failure.
   **/
   public int writeToDatabase() {
      LogSystem.log(2, ">>> Starting Symptom.writeToDatabase()");

      /*--------------------------------------------------------------*/
      /*   Assign database keys for this Question and its subobjects. */
      /*--------------------------------------------------------------*/
      if ( !assignDatabaseKeys() ) {
         return(-1);
      }

      SQLMethod sql = new SQLMethod(DatabaseSystem.QUEST,
                                    "Symptom.writeToDatabase()",10);
      int returnCode = 0 ;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sql.createStoredProc ("QUEST.ExtSqlRunner", 3);
         String sqlToRun = SymptomWriter.getSQL(this);
         if ( sqlToRun.length() == 0 ) {
            LogSystem.log(1, "No changes to be written");
            return(0);
         }
         String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                       SqlRunner.END_DELIM, "\n" );

         LogSystem.log ( 1, "Starting to send statements to EXTSQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      
         stmt.setInt   ( 2, 0 );  // commit interval

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            returnCode = retRec.sqlCode ;
            if ( returnCode == 0 ) returnCode = -1;
         }

         stmt.close() ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
         returnCode = -1 ;
      } finally {
         sql.close() ;
      }

      if ( returnCode == 0 ) {
         setRecNoChange();      // set all to REC_NOCHANGE
      }

      LogSystem.log(2, ">>> Symptom.writeToDatabase() complete");

      return(returnCode);
   }

   /**
    * Set the recStatus of this object and 
    * all its contained objects to REC_NOCHANGE.
   **/
   void setRecNoChange() {

      // this object
      updateRecStatus(REC_NOCHANGE);
      super.setOriginalCountryListFromCurrent();

      // contained objects:
      if ( nodes_ != null ) {
         for ( int i=0; i<nodes_.size(); i++ ) {
            Node n = (Node) nodes_.elementAt(i);
            n.updateRecStatus(REC_NOCHANGE);
         }
      }
      if ( edgeGroups_ != null ) {
         for ( int i=0; i<edgeGroups_.size(); i++ ) {
            EdgeGroup g = (EdgeGroup) edgeGroups_.elementAt(i);
            g.setRecNoChange();
         }
      }
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            SymptomTitle st = (SymptomTitle) titles_.elementAt(i);
            st.setRecNoChange();
         }
      }
   }

   /**
    * Read an Symptom object from the database.
    * 
    * @param symptomInd  the database indicator of the Symptom to be read.
    * @param languageInd the language to read the text items for 
    *                    (0 = read all languages).
    * @return the Symptom object or null if an error occured or
    *         the object was not found.
   **/
   public static Symptom readFromDatabase(int symptomInd, int languageInd) {
      LogSystem.log(2, ">>> Starting Symptom.readFromDatabase()");
      long start = System.currentTimeMillis();
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "Symptom.readFromDatabase()", 5 );
      StoredProcRec retRec     = null;
      Symptom       retSymptom = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, symptomInd );             // object ind
         stmt.setInt( 2, OAReader.SYMPTOM );       // object type
         stmt.setInt( 3, OAReader.READ_ACTIVE );   // read active object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Symptom and the StoredProcReturnRec objects     */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retSymptom = (Symptom)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      long end = System.currentTimeMillis();
      LogSystem.log(1, "Elapsed time: " + (end-start) + " msec");
      LogSystem.log(2, ">>> Symptom.readFromDatabase() completed");

      if ( retSymptom != null ) {
         // performance: dont update TypeRec data if in web mode
         if ( OAMode.currentMode != OAMode.WEB ) {
            retSymptom.updateTypeRecData();  // update the Type*Rec objects
            retSymptom.setRecNoChange();     // sets the original country list
         }
      } else {
         LogSystem.log(2, "No object found for indicator " + symptomInd);
      }

      return(retSymptom);
   }                                       

   /**
    * Update the TypeRec objects in the Action's subobjects with 
    * the proper descriptions.
    * Called by readFromDatabase();
   **/
   void updateTypeRecData() {
      super.updateTypeRecData();

      TypeList t = TypeList.getInstance();

      // symptom type
      TypeSymptomRec newSymptomType;
      newSymptomType = (TypeSymptomRec) t.objectFromInd( symptomType_.getInd(),
                                                         TypeList.TYPE_SYMPTOM );
      if ( newSymptomType != null ) {
         symptomType_ = newSymptomType;
      }

      // symptom status 
      TypeStatusRec newSymptomStatus;
      newSymptomStatus = (TypeStatusRec) t.objectFromInd( symptomStatus_.getInd(),
                                                          TypeList.TYPE_STATUS );
      if ( newSymptomStatus != null ) {
         symptomStatus_ = newSymptomStatus;
      }

      // update sub-objects TypeRec data:
      if ( titles_ != null ) {
         for ( int i=0; i<titles_.size(); i++ ) {
            SymptomTitle title = (SymptomTitle) titles_.elementAt(i);
            title.updateTypeRecData();
         }
      }

      if ( edgeGroups_ != null ) {
         for ( int i=0; i<edgeGroups_.size(); i++ ) {
            EdgeGroup group = (EdgeGroup) edgeGroups_.elementAt(i);
            Vector edgeVec = group.getEdges();
            if ( edgeVec != null ) {
               for ( int j=0; j<edgeVec.size(); j++ ) {
                  Edge edge = (Edge) edgeVec.elementAt(j);
                  edge.updateTypeRecData();
               }
            }
         }
      }

      return;
   }

   /**
    * Find an open draft for a Symptom if one exists.
    * This method requires that a connection to DatabaseSystem.QUEST 
    * is already established.
    * 
    * @param symptomInd the database indicator of the active (published)
    *                   Symptom to find a draft for.
    * @return the indicator of the open draft for the Symptom or 0 if there
    *         is no open draft or an error occurs.
   **/
   public static int findOpenDraft(int symptomInd) {
      LogSystem.log(2, ">>> Starting Symptom.findOpenDraft()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "Symptom.findOpenDraft()", 1 );

      int draftSymptomInd = 0;

      try {
         Statement stmt = sql.createStatement();
         ResultSet rs = stmt.executeQuery(
                                         "SELECT symptomInd " +
                                         "FROM oadraft.symptom " +
                                         "WHERE activeSymptomInd=" + symptomInd + " " +
                                         "AND stopDate IS NULL " +
                                         "FOR FETCH ONLY"
                                         );

         if ( rs.next() ) {
            draftSymptomInd = rs.getInt(1);
         } else {
            LogSystem.log(1, "No open draft found for " + symptomInd);
         }
         rs.close();
      } catch ( Exception e ) {
         LogSystem.log ( 3, e );
      } finally {
         sql.close();
      }

      LogSystem.log(2, ">>> Symptom.findOpenDraft() completed");

      return(draftSymptomInd);
   }

   /**
    * Return the Edge with the specified indicator.
    * <b>Note:</b> This method uses a Hashtable that indexes
    * each Edge by its database indicator to look up the
    * specified Edge object.  Therefore this method is only
    * useful for Edge that have been assigned database indicators.
   **/
   public Edge getEdgeByInd(int edgeInd) {
      if ( edgesByInd_ == null ) {
         buildEdgeTable();
      }
      return(Edge) edgesByInd_.get( new Integer(edgeInd) );
   }

   /**
    * Creates a Hashtable that indexes the Edges in this Symptom
    * by database indicator.  Use the getEdgeByInd() method to
    * retrieve the Edge object for a given indicator.
   **/
   private void buildEdgeTable() {
      edgesByInd_ = new Hashtable();

      for ( int i=0; i<edgeGroups_.size(); i++ ) {
         EdgeGroup group = (EdgeGroup) edgeGroups_.elementAt(i);
         Vector edgeVec = group.getEdges();
         for ( int j=0; j<edgeVec.size(); j++ ) {
            Edge edge = (Edge) edgeVec.elementAt(j);
            edgesByInd_.put( new Integer( edge.getInd() ), edge );
         }
      }
   }




   /*------------------------------------------------------------*/
   /* setCountryList
    */
   /**
    *    This method will set the country list that this
    *  object contains
    *  
    *
    *  @param countries A Vector of TypeCountryCodeRec objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCountryList ( Vector countries ) {
      super.setCountryList ( countries ) ;

      /*------------------------------------------------------------------*/
      /*                                                                  */
      /*   Update the default title to have our counties and doc class    */
      /*                                                                  */
      /*------------------------------------------------------------------*/
      if ( ( titles_ != null ) && ( titles_.size() > 0 ) )
         ((SymptomTitle)titles_.elementAt(titles_.size()-1)).setCountryList ( countries ) ;

   } /* setCountryList */






   /*------------------------------------------------------------*/
   /* setDocClass
    */
   /**
    *    This method will set the document classification
    *  of this object. The classification goes as follows<br><br>
    *  
    *  IBM Confidential - For content used in Info Tips<br>
    *  IBM Dealer Confidential - For content used in Business Partner<br>
    *  Unclassified - For content used on all sites<br><br>
    *
    *  @param classRec The TypeDocClassRec that represents the classification (as defined in TIGRIS.TYPEDOCCLASS)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocClass ( TypeDocClassRec classRec ) {
      super.setDocClass ( classRec ) ;
      docClass_ = classRec ;

      if ( ( titles_ != null ) && ( titles_.size() > 0 ) )
         ((SymptomTitle)titles_.elementAt(titles_.size()-1)).setDocClass ( classRec ) ;

   }




   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the default title object
    *  for this symptom. If there are no titles, then this
    *  method returns NULL
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return SymptomTitle
    *//*---------------------------------------------------------*/
   public SymptomTitle getDefaultTitle () {
      if ( ( titles_ != null ) && ( titles_.size() > 0 ) )
         return( (SymptomTitle)titles_.elementAt ( titles_.size()-1 ) ) ;
      return( null ) ;
   }
}
