
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;


/**
 * Reader for Online Assistant ActionDrafts.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.8
 * @author  Vishnukumar. E
**/
public class SymptomDraftReader {

  private static final short TYPE_STATUS_SYMPTOM = 4;

  /**
   * Read and populate the specified Symptom object.
   * The SymptomDraft object's indicator should contain the symptomInd 
   * of the draft to be read.
   * @param conn  the database Connection object
   * @param s     the Symptom to populate
   * @param lang  the language to read the sub-objects for (0 = all languages)
   * @return true if the Symptom was successfully read,
             false if the Symptom was not found in the database
  **/
  public static boolean readFromDB(Connection conn, SymptomDraft sd,
                                   int languageInd) 
                        throws SQLException
  {
    Statement   stmt = conn.createStatement();
    ResultSet   rs;
    boolean     foundSymptom;

    // turn on DB2 explain:
    //stmt.executeUpdate("SET CURRENT EXPLAIN MODE YES");
    //stmt.executeUpdate("SET CURRENT EXPLAIN SNAPSHOT YES");

    rs = stmt.executeQuery( 
             "SELECT " +
               "activeSymptomInd, symptomTypeInd, rootNodeInd, " +
               "brandInd,         typeWorkInd,    docClassInd, statusInd, comments, " +
               "maintLock,        startDate,      stopDate,    recycled,  archived, " +
               "created,          dbUser,         changedTime " +
             "FROM oadraft.symptom " +
             "WHERE symptomInd=" + sd.getInd() + " " +
             "FOR FETCH ONLY"
    );

    if ( !rs.next() ) {
      foundSymptom = false;
      stmt.close();
    } else {
      foundSymptom = true;

      int rootNodeInd;
                      
      //---------------------------------------------------------------------
      // Read the base SymptomDraft data
      //---------------------------------------------------------------------
      sd.setActiveSymptomInd( rs.getInt(1) );
      sd.setSymptomType( new TypeSymptomRec( rs.getShort(2), "" ) );
      rootNodeInd = rs.getInt(3);
      sd.setBrand( new TypeBrandRec( rs.getInt(4), "", 0 ) );
      sd.setTypeWorkRequired( new TypeWorkRec( rs.getInt(5), "") );
      sd.setDocClass( new TypeDocClassRec( rs.getShort(6), "" ) );
      sd.setSymptomStatus( new TypeStatusRec( rs.getShort(7), 
                                              TYPE_STATUS_SYMPTOM, "" ) );
      sd.setComments( rs.getString(8) );
      if ( rs.wasNull() ) {
        sd.setComments("");
      }
      sd.setMaintLock( DatabaseUtil.booleanFromDB( rs.getString(9) ) );
      sd.setStartDate( rs.getString(10) );
      sd.setStopDate( rs.getString(11) );
      sd.setRecycled( DatabaseUtil.booleanFromDB( rs.getString(12) ) );
      sd.setArchived( DatabaseUtil.booleanFromDB( rs.getString(13) ) );
      sd.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(14) ) );
      sd.setDbUser( rs.getString(15).trim() );
      sd.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(16) ) );
 
      stmt.close();

      //---------------------------------------------------------------------
      // Read the SymptomDraft Countries
      //---------------------------------------------------------------------
      sd.setCountryList(
         CountryReader.readCountries( conn, "oadraft.symptomCountry",
                                      "symptomInd", sd.getInd() ) );

      //---------------------------------------------------------------------
      // Read the SymptomDraft Titles based on the language passed in
      //---------------------------------------------------------------------
      TitleReader.readSymptomDraftTitles( conn, sd, languageInd );

      //---------------------------------------------------------------------
      // Read the SymptomDraft Nodes and EdgeGroups/Edges
      //---------------------------------------------------------------------
      NodeDraftReader.readNodesForSymptom( conn, sd );
      EdgeGroupDraftReader.readEdgeGroupsForSymptom( conn, sd );

      // set the root node:
      Node n = sd.getNodeByInd( rootNodeInd );
      sd.setRootNode(n);


      sd.updateRecStatus( sd.REC_NOCHANGE );
      // dont sent the following data back from the stored procedure --
      // it can be rebuilt on the client (reason: less network traffic?).
      sd.edgesByInd_ = null;
      sd.nodesByInd_ = null;
    }

    return foundSymptom;
  }

  /**
   * Get a draft for an Symptom.
   * Check to see if there is an open draft -- if there is, then return it.
   * If there is no open draft, then create a new one.
   *
   * @exception PendingDraftException if openDraft='Y' on the active symptom
   *            and there is no open draft with stopDate=NULL
  **/
  public static SymptomDraft getDraft(Connection conn, int symptomInd, 
                                      int languageInd) 
                             throws SQLException, PendingDraftException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs;

    SymptomDraft returnedDraft = null;

    // check to see if there is already an open draft....
    rs = stmt.executeQuery(
      "SELECT symptomInd " +
      "FROM oadraft.symptom " +
      "WHERE stopDate IS NULL AND activeSymptomInd=" + symptomInd + " " +
      "FOR FETCH ONLY"
    );

    if ( rs.next() ) {
      // there is an open draft -- get it.
      int draftSymptomInd = rs.getInt(1);
      rs.close();
      SymptomDraft sd = new SymptomDraft( draftSymptomInd );
      if ( SymptomDraftReader.readFromDB( conn, sd, languageInd ) ) {
        returnedDraft = sd;
      }
    } else {
      // there is no open draft -- read the symptom & create a new draft.
      rs.close();
      Symptom s = new Symptom( symptomInd );
      if ( SymptomReader.readFromDB( conn, s, languageInd, OAMode.DEFAULT ) ) {
        if ( s.hasOpenDraft() ) {
          throw new PendingDraftException();
        }
        SymptomDraft sd = new SymptomDraft( s );
        returnedDraft = sd;
        // if we opened a new draft, then update openDraft in oa.symptom
        stmt.executeUpdate(
          "UPDATE oa.symptom " +
          "SET openDraft='Y' " + 
          "WHERE symptomInd=" + symptomInd
        );
      }
    }

    stmt.close();

    return returnedDraft;
  }
}
