
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;

/**
 * @version 1.18
 * @author  Mark DeBiase
**/
public class SymptomWriter {

  /**
   * Return the SQL for this Symptom.
  **/
  static String getSQL(Symptom s) {

    StringBuffer sb = new StringBuffer();

    sb.append( getSymptomSQL(s) );
    if ( s.valid() ) {
      // insert/update the sub-objects as necessary
      sb.append( getObjectSQL(s) );
    }

    // if this is a new symptom, update the root node indicator
    // (it was inserted as NULL due to circular dependency with node table)
    if ( s.getRecStatus() == s.REC_INSERT ) {
      sb.append( getUpdateRootNodeSQL(s) );
    }

    if ( !s.modified() && sb.length() > 0 ) {
      // Symptom wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oa.symptom " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE symptomInd=")
        .append( s.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the Symptom's 
   * SymptomTitle, Node and EdgeGroup/Edge objects.
   * 
   * @param symptom        the symptom to be saved
  **/
  static String getObjectSQL(Symptom symptom) {

    StringBuffer sb = new StringBuffer();
    Vector v;

    //---------------------------------------------------------------------
    // Loop through each subobject contained in the Symptom object.
    // Update the sortOrder, parentInd and dbUser for the
    // subobject.  setSortOrder() may affect the modification state of
    // the subobject if its order has changed, but setParentInd() and
    // setDbUser() will not affect the modification state.
    //---------------------------------------------------------------------
 
    v = symptom.getSymptomTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        SymptomTitle t = (SymptomTitle) v.elementAt(i);
        if ( t.valid() ) { 
          t.setSortOrder( sortOrder++ ); 
          t.setSymptomInd( symptom.getInd() );
          t.setDbUser( symptom.getDbUser() );
        }
        sb.append( SymptomTitleWriter.getSQL(t) );
      }
    }

    //-----------------------------------------------------------------
    // Process nodes in two passes:
    // For insert, Nodes must be inserted before the EdgeGroups.
    // For delete, EdgeGroups must be deleted before the Nodes.
    //-----------------------------------------------------------------

    // Node pass 1: insert/update only
    v = symptom.getNodes();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        Node n = (Node) v.elementAt(i);
        if ( n.valid() ) { 
          n.setSymptomInd( symptom.getInd() );
          n.setDbUser( symptom.getDbUser() );
          sb.append( NodeWriter.getSQL(n) );
        }
      }
    }

    v = symptom.getEdgeGroups();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        EdgeGroup e = (EdgeGroup) v.elementAt(i);
        if ( e.valid() ) { 
          e.setSymptomInd( symptom.getInd() );
          e.setDbUser( symptom.getDbUser() );
        }
        sb.append( EdgeGroupWriter.getSQL(e) );
      }
    }

    // Node pass 2: delete only
    v = symptom.getNodes();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        Node n = (Node) v.elementAt(i);
        if ( !n.valid() ) { 
          sb.append( NodeWriter.getSQL(n) );
        }
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Symptom to the database.
  **/
  static String getSymptomSQL(Symptom s) {
    String sql;

    switch ( s.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(s);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(s);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(s);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  /**
   * Returns the SQL String to insert the Symptom in the database.
  **/
  static String getInsertSQL(Symptom s) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.symptom " +
         "(symptomInd, symptomTypeInd, rootNodeInd, brandInd, docClassInd, " +
         "statusInd, archived, dbUser, changedTime, openDraft) " +
       "VALUES ("
    );
    // symptomInd:
    sb.append( s.getInd()                                  ).append(',');

    // symptomTypeInd:
    sb.append( s.getSymptomType().getInd()                 ).append(',');

    // rootNodeInd:
    // due to circular dependency between rootNodeInd in oa.symptom and
    // symptomInd in oa.node, we must insert the root node as null and
    // then go back and update it after the nodes are inserted.
    sb.append( "NULL," );

    // oaBrandInd:
    sb.append( s.getBrand().getInd()                       ).append(',');

    // docClassInd:
    sb.append( s.getDocClass().getInd()                    ).append(',');

    // statusInd:
    sb.append( s.getSymptomStatus().getInd()               ).append(',');

    // archived:
    sb.append( DatabaseUtil.booleanToDB( s.isArchived() )  ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( s.getDbUser() )    ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP, 'N')")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // insert the countries into oa.symptomCountry
    sb.append( CountryWriter.getInsertSQL( "oa.symptomCountry",
                                           s.getInd(),
                                           s.getCountryList(),
                                           s.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns SQL to update the Symptom root node after all the 
   * Nodes/Egdges have been inserted when creating a new Symptom.
  **/
  static String getUpdateRootNodeSQL(Symptom s) {

    // first check for missing/improperly added root node

    if ( s.getRootNode() == null ) {
      throw new IllegalStateException("Root Node has not been set");
    }

    if ( s.getRootNodeInd() == 0 ) {
      throw new IllegalStateException("Root Node indicator is 0 -- " +
                                      "the Node was probably not put in " +
                                      "the Symptom's Vector of Nodes");
    }

    return "UPDATE oa.symptom set rootNodeInd=" + s.getRootNodeInd() +
           " WHERE symptomInd=" + s.getInd() +
           SqlRunner.END_DELIM;
  }

  /**
   * Returns the SQL String to update the Symptom in the database.
  **/
  static String getUpdateSQL(Symptom s) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oa.symptom SET");

    sb.append( " symptomTypeInd=" )
      .append( s.getSymptomType().getInd() );

    sb.append( ",brandInd=" )
      .append( s.getBrand().getInd()  );

    sb.append( ",docClassInd=" )
      .append( s.getDocClass().getInd() );

    sb.append( ",statusInd=" )
      .append( s.getSymptomStatus().getInd() );

    sb.append( ",archived=" )
      .append( DatabaseUtil.booleanToDB( s.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( s.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE symptomInd=").append( s.getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oa.symptomCountry",
                                           "symptomInd",
                                            s.getInd(),
                                            s.getCountryList(),
                                            s.getOriginalCountryList(),
                                            s.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to delete the Symptom from the database.
  **/
  static String getDeleteSQL(Symptom s) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);
    int symptomInd = s.getInd();

    // Update the rootNode indicator to NULL so we can delete the nodes
    sb.append( "UPDATE oa.symptom SET rootNodeInd=NULL WHERE symptomInd=" )
      .append( symptomInd ).append(SqlRunner.END_DELIM);

    // Delete the Nodes & Edges
    sb.append( EdgeGroupWriter.getDeleteAllForSymptomSQL(symptomInd) );
    sb.append( NodeWriter.getDeleteAllForSymptomSQL(symptomInd) );

    // Delete the Symptom Titles
    sb.append( SymptomTitleWriter.getDeleteAllForSymptomSQL(symptomInd) );

    // Delete the Symptom itself
    sb.append(
      "DELETE from oa.symptomcountry where symptomind="
    ).append( symptomInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oa.symptom where symptomind="
    ).append( symptomInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return the SQL to publish a Symptom from a SymptomDraft object.
   * 
   * @return a String containing SQL statements separated by 
   *         SqlRunner.END_DELIM.
  **/
  public static String getPublishSQL(Connection conn, SymptomDraft draft)
                       throws Exception
  {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);


    // construct a new Symptom object from the draft object
    // the recstatus and indicators will be set appropriately
    Symptom active = new Symptom( draft );
    active.assignDatabaseKeys(conn);  // throws Exception on failures

    // if there is already an active object, then the Titles, Nodes,
    // EdgeGroups and Edges associated with it must be wiped out
    if (active.getRecStatus() == PersistentRec.REC_UPDATE) {
      int symptomInd = active.getInd();
      sb.append( SymptomTitleWriter.getDeleteAllForSymptomSQL(symptomInd) );
      sb.append( EdgeGroupWriter.getDeleteAllForSymptomSQL(symptomInd) );

      // set the rootNodeInd to NULL so we can delete all the nodes
      sb.append( "UPDATE oa.symptom SET rootNodeInd=NULL WHERE symptomInd=" )
        .append( symptomInd ).append(SqlRunner.END_DELIM);

      sb.append( NodeWriter.getDeleteAllForSymptomSQL(symptomInd) );
    }

    // generate the sql to to publish the symptom
    sb.append( getSQL(active) );
    if (active.getRecStatus() == PersistentRec.REC_UPDATE) {
      // update the symptom with the new root node
      sb.append( getUpdateRootNodeSQL(active) );
    }

    // update the symptomResponse table:
    // 1. delete all existing entries for this symptomInd
    // 2. insert all geos contained in oa.symptomCountry
    //    for this symptomInd, setting the response to 'N'
    // MAD 6/18: put countries instead of geos in actionResponse table
    sb.append( "DELETE FROM oa.symptomResponse WHERE symptomInd=")
      .append( active.getInd() ).append(SqlRunner.END_DELIM);
    sb.append(
       "INSERT INTO oa.symptomResponse " +
          "SELECT DISTINCT " +
             "s.symptomInd, gc.geoInd, 'N', s.dbUser, s.changedTime " +
          "FROM oa.symptom s, oa.symptomCountry sc, tigris.geocountry gc " +
          "WHERE s.symptomInd=" + active.getInd() +
          "  AND sc.symptomInd=s.symptomInd "+
          "  AND sc.countryInd = gc.countryCodeInd "
    ).append(SqlRunner.END_DELIM);

    // update the oadraft.node table with the active symptom indicator.
    // this only has to be done for drafts which were never published before.
    if (active.getRecStatus() == PersistentRec.REC_INSERT) {
      sb.append(
         "UPDATE oadraft.node " +
         "SET activeSymptomInd=" + active.getInd() + " " +
         "WHERE draftSymptomInd=" + draft.getInd()
      ).append(SqlRunner.END_DELIM);
    }

    // update the openDraft flag.  this is done seperately at the end
    // in case there are any failures publishing the draft.
    sb.append(
      "UPDATE oa.symptom " +
      "SET openDraft='N' " +
      "WHERE symptomInd="
    ).append( active.getInd() ).append(SqlRunner.END_DELIM);

    // finally, update the stopDate & activeSymptomInd on the draft object
    sb.append(
      "UPDATE oadraft.symptom " +
      "SET stopDate=CURRENT DATE, " +
          "changedTime=CURRENT TIMESTAMP, " +
          "activeSymptomInd=" + active.getInd() + " " +
      "WHERE symptomInd=" + draft.getInd() 
    ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
