
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.util.*;
import java.sql.*;

/**
 * Represents a Conditional Title in an Action or Question.
 *
 * @version 1.20
 * @author  Mark DeBiase
**/
public class Title extends ConditionalObject {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;     

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   toString Types                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * Setting this toStringType will return only plain text
     **/
   public static final int TEXT = 1 ;

   /**
     * This toStringType will return A Title in HTML
     **/
   public static final int HTML_TITLE = 2 ;

   /**
     * This toStringType will return a small title in HTML
     **/
   public static final int HTML = 3 ;

   /**
     * This toStringType will return a bold title in HTML
     **/
   public static final int HTML_BOLD = 4 ;

   /**
     * This toStringType will return diagnostic information
     **/
   public static final int DIAG = 5 ;


   private int             titleInd_       = 0;    // database indicator
   private int             parentInd_      = 0;    // foreign key to parent table
   private String          title_          = null; // title text
   private TypeLanguageRec language_       = null; // language
   private String          lastTranslated_ = null; // last translation date
   private boolean         textUpdated_    = false;  // set to true if title 
                                                     // text is updated
   private int             toStringType    = 0 ;

   /**
    * New object.
   **/
   public Title() {
      this("");
   }

   /**
    * New object, with specified title text.
   **/
   public Title(String title) {
      super();
      title_  = title;
      updateRecStatus(REC_INSERT);
   }

   /**
    * Title object read from the database.
   **/
   Title(int titleInd) {
      super();
      titleInd_ = titleInd;
      updateRecStatus(REC_NOCHANGE);
   }

   /**
    * Assign a database key for this Title.
    * This method can be called for a title attached to an Action, a
    * Question or an Answer so the counter name in tigris.counter 
    * to be used is passed as a parameter.
    * @param  counterName the counter name to use to get the next key
    * @return true if a database key was assigned, otherwise false
   **/
   private boolean assignDatabaseKey(String counterName) {
      boolean success = true;

      if ( titleInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for Title: " + counterName);
         titleInd_ = Counter.getCounter( LogSystem.getInstance(), counterName );
         if ( titleInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for Title");
            success = false;
         }
      }

      return (success);
   }
   // version to be called by publish stored procedure:
   private void assignDatabaseKey(Connection conn, String counterName) 
   throws Exception
   {

      if ( titleInd_ == 0 ) {
         titleInd_ = Counter.getCounter( conn, counterName );
         if ( titleInd_ == 0 ) {
            throw new Exception("Failed to get database key for Title");
         }
      }

   }

   /**
    * Assign database key for an Action Title.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForAction() {
      return (assignDatabaseKey("ACTIONTITL"));
   }
   void assignDatabaseKeyForAction(Connection conn) throws Exception {
      assignDatabaseKey(conn, "ACTIONTITL");
   }
   /**
    * Assign database key for a Question Title.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForQuestion() {
      return (assignDatabaseKey("QSTNTITLE"));
   }
   void assignDatabaseKeyForQuestion(Connection conn) throws Exception {
      assignDatabaseKey(conn, "QSTNTITLE");
   }
   /**
    * Assign database key for an Answer Title.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForAnswer() {
      return (assignDatabaseKey("ANSWERTITL"));
   }
   void assignDatabaseKeyForAnswer(Connection conn) throws Exception {
      assignDatabaseKey(conn, "ANSWERTITL");
   }
   /**
    * Assign database key for a draft Action Title.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForActionDraft() {
      return (assignDatabaseKey("DACTNTITL"));
   }
   void assignDatabaseKeyForActionDraft(Connection conn) throws Exception {
      assignDatabaseKey(conn, "DACTNTITL");
   }
   /**
    * Assign database key for a draft Question Title.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForQuestionDraft() {
      return (assignDatabaseKey("DQSTNTITLE"));
   }
   void assignDatabaseKeyForQuestionDraft(Connection conn) throws Exception {
      assignDatabaseKey(conn, "DQSTNTITLE");
   }
   /**
    * Assign database key for an draft Answer Title.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignDatabaseKeyForAnswerDraft() {
      return (assignDatabaseKey("DANSRTITL"));
   }
   void assignDatabaseKeyForAnswerDraft(Connection conn) throws Exception {
      assignDatabaseKey(conn, "DANSRTITL");
   }

   /**
    * Get the indicator for this Title.
   **/
   public int getInd() {
      return (titleInd_);
   }

   /**
    * Set the indicator for this Title.
   **/
   void setInd(int titleInd) {
      titleInd_ = titleInd;
   }

   /**
    * Get the parent objects's indicator.
   **/
   int getParentInd() {
      return (parentInd_);
   }

   /**
    * Set the parent objects's indicator.
   **/
   void setParentInd(int parentInd) {
      parentInd_ = parentInd;
   }

   /**
    * Get the title string.
   **/
   public String getTitle() {
      return (title_);
   }

   /**
    * Set the title string.
   **/
   public void setTitle(String title) {
      title_       = title;
      textUpdated_ = true;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Called by Readers to reset the textUpdated flag after reading a Title
    * from the database.
   **/
   void setTextUpdated(boolean textUpdated) {
      textUpdated_ = textUpdated;
   }

   /**
    * Returs true if the title text has been updated.
   **/
   boolean wasTextUpdated() {
      return (textUpdated_);
   }

   /**
    * Gets the language.
    * @return TypeLanguageRec the language
   **/
   public TypeLanguageRec getLanguage() {
      return (language_);
   }

   /**
    * Sets the language.
    * @parm language the language
   **/
   public void setLanguage(TypeLanguageRec language) {
      language_ = language;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Get the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @return a String representing a DB2 timestamp
   **/
   public String getLastTranslationDate() {
      return (lastTranslated_);
   }

   /**
    * Set the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @param translationDate a String representing a DB2 timestamp
   **/
   public void setLastTranslationDate(String translationDate) {
      lastTranslated_ = translationDate;
   }

   /**
    * Perform a shallow copy of this object.
   **/
   public Object clone() {
      // the conditions are cloned by ConditionalObject.clone()
      return (super.clone());
   }

   /**
    * Return a String representation of this object.
   **/
   public String toString() {
      return ( toString ( toStringType ) ) ;
   }

   public String toString ( int toStringType ) {
      String retStr = title_ ;


      switch ( toStringType ) {
         case DIAG :
            retStr = ""+titleInd_+": "+title_ ;
            break;

         case HTML :
            retStr = "<html><body><table><tr><td><font = \"Helvetica, Arial\" size=2>"+title_+"</font></td></tr></table></body></html>" ;
            break ;

         case HTML_BOLD :
            retStr = "<html><body><font = \"Helvetica, Arial\" size=2><b>"+title_+"</b></font></body></html>" ;
            break ;

         case HTML_TITLE :
            retStr = "<html><body><font = \"Helvetica, Arial\" size=4>"+title_+"</font></body></html>" ;
            break ;


      }

      return (retStr);
   }

   public String debugString() {
      return ("Title " + titleInd_ + ": \"" + title_ + 
             "\" (lang=" + language_.getInd() + ")");
   }

   /**
    * Reset values in this object.
    * Sets the indicator to 0, sets the recStatus to REC_INSERT.
   **/
   public void resetNew() {
      setInd(0);
      setParentInd(0);
      // Force the insert
      updateRecStatus(REC_NOCHANGE);
      updateRecStatus(REC_INSERT);
   }

   /**
    * Update the TypeRec objects in the Title object with 
    * the proper descriptions.
    * Called after a Title is read from the database.
   **/
   void updateTypeRecData() {
      super.updateTypeRecData();

      TypeList t = TypeList.getInstance();

      // language
      TypeLanguageRec newLangRec;
      newLangRec = (TypeLanguageRec) t.objectFromInd( language_.getInd(),
                                                      TypeList.TYPE_LANGUAGE );
      if ( newLangRec != null ) {
         language_ = newLangRec;
      }

      return;
   }





   /*------------------------------------------------------------*/
   /* setToStringType
    */
   /**
    *    This method will set what type of String is returned
    *  from the toString method.
    *  
    *
    *  @param toStringType ( HTML, HTML_BOLD, DIAG etc..)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setToStringType ( int toStringType ) {
      this.toStringType = toStringType ;
   }
}
