
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant Titles (Action, Question, Symptom, Answer).
 * This class is used by the OA stored procedures.
 * 
 * @version 1.12
 * @author  Mark DeBiase
**/


class TitleReader {

   public static final String ACTIVE_SCHEMA = "oa";
   public static final String DRAFT_SCHEMA  = "oadraft";

//=============================================================================

   /**
    * Read Action Titles for all languages (oa schema).
    * @param conn  the database connection
    * @param a     the Action whose titles are to be read
   **/
   static void readActionTitles(Connection conn, Action a) 
   throws SQLException
   {
      readActionTitles(conn, a, ACTIVE_SCHEMA);
   }

   /**
    * Read Action Titles for all languages (oadraft schema).
    * @param conn  the database connection
    * @param a     the Action whose titles are to be read
   **/
   static void readActionDraftTitles(Connection conn, Action a) 
   throws SQLException
   {
      readActionTitles(conn, a, DRAFT_SCHEMA);
   }

   /**
    * Read Action Titles for all languages (common function).
    * @param conn   the database connection
    * @param a      the Action whose titles are to be read
    * @param schema the database schema to read (oa or oadraft)
   **/
   private static void readActionTitles(Connection conn, Action a, String schema)
   throws SQLException
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // select all titles
      rs = stmt.executeQuery( "SELECT " +
                              "actionTitleInd, sortOrder, " +
                              "title, docClassInd, created, " +
                              "dbUser, changedTime, " +
                              "languageInd, lastTranslated" +
                              " FROM " + schema + ".actionTitle" +
                              " WHERE actionInd=" + a.getInd() + 
                              " ORDER BY sortOrder, languageInd" +
                              " FOR FETCH ONLY" );

      Vector v = new Vector();
      while ( rs.next() ) {
         Title t = new Title( rs.getInt(1) );
         t.setParentInd( a.getInd() );

         t.setSortOrder( rs.getShort(2) );
         t.setTitle( rs.getString(3) );
         t.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
         t.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
         t.setDbUser( rs.getString(6).trim() );
         t.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
         t.setLanguage( new TypeLanguageRec( rs.getShort(8), "" ) );
         t.setLastTranslationDate(
                                 DatabaseUtil.timestampFromDB( rs.getString(9) ) 
                                 );

         // read the conditions/countries
         // MAD performance: title conditions are now read all at once below
         //t.setConditions(
         //   ConditionReader.readConditions( conn, schema + ".actionTitleCond",
         //                                   "actionTitleInd", t )
         //);
         // MAD performance: title countries are now read all at once below
         //t.setCountryList(
         //   CountryReader.readCountries( conn, schema + ".actionTitleCountry",
         //                                "actionTitleInd", t.getInd() )
         //);

         t.setTextUpdated( false );
         t.updateRecStatus(PersistentRec.REC_NOCHANGE);
         v.addElement(t);
      }

      rs.close();
      stmt.close();

      a.setTitles(v);

      // read conditions/countries for all action titles
      // we should have always read at least one title
      ConditionReader.readAllActionTitleConditions(conn, a, schema, true);
      CountryReader.readAllActionTitleCountries(conn, a, schema, true);
   }

//=============================================================================

   /**
    * Read Action Titles for a specific language (oa schema).
    * If languageInd=0 (read all languages), 
    * then readActionTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param a            the Action whose titles are to be read
    * @param languageInd  the language to read
   **/
   static void readActionTitles(Connection conn, Action a, int languageInd) 
   throws SQLException
   {
      if ( languageInd == 0 ) {
         readActionTitles(conn, a, ACTIVE_SCHEMA);
      } else {
         readActionTitles(conn, a, languageInd, ACTIVE_SCHEMA);
      }
   }

   /**
    * Read Action Titles for a specific language (oadraft schema).
    * If languageInd=0 (read all languages), 
    * then readActionTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param a            the Action whose titles are to be read
    * @param languageInd  the language to read
   **/
   static void readActionDraftTitles(Connection conn, Action a, int languageInd) 
   throws SQLException
   {
      if ( languageInd == 0 ) {
         readActionTitles(conn, a, DRAFT_SCHEMA);
      } else {
         readActionTitles(conn, a, languageInd, DRAFT_SCHEMA);
      }
   }

   /**
    * Read Action Titles for a specific language (common function).
    * This method will attempt to read the titles in the specified language.
    * If a title is not found in the requested language, the English title
    * will be read.
    * @param conn         the database connection
    * @param a            the Action whose titles are to be read
    * @param languageInd  the language to read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   private static void readActionTitles(Connection conn, Action a, 
                                        int languageInd, String schema) 
   throws SQLException
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // first select statement is to get titles in the requested language...
      String selectStmt = "SELECT " +
                          "actionTitleInd, sortOrder, " +
                          "title, docClassInd, created, " +
                          "dbUser, changedTime, " +
                          "languageInd, lastTranslated " +
                          "FROM " + schema + ".actionTitle " +
                          "WHERE actionInd="   + a.getInd()  + " " +
                          "AND   languageInd=" + languageInd + " ";

      // ... if the requested language was not English, then we have to UNION
      // with a second select statement that will get the English title for
      // those that were not available in the requested language.
      // the second select statements uses a subselect to exclude those titles
      // which exist in the requested language (and thus were selected by the
      // previous select statement).
      if ( languageInd != 1 ) {
         selectStmt += "UNION " +
                       "SELECT " +
                       "actionTitleInd, sortOrder, " +
                       "title, docClassInd, created, " +
                       "dbUser, changedTime, " +
                       "languageInd, lastTranslated " +
                       "FROM " + schema + ".actionTitle " +
                       "WHERE actionInd=" + a.getInd()  + " " +
                       "AND   languageInd=1 " +
                       "AND   actionTitleInd NOT IN (" +
                       "SELECT actionTitleInd " +
                       "FROM " + schema + ".actionTitle " +
                       "WHERE actionInd="   + a.getInd() + " " +
                       "AND   languageInd=" + languageInd +
                       ") ";
      }

      // finish off the select statement:
      selectStmt += "ORDER BY sortOrder " +
                    "FOR FETCH ONLY";

      rs = stmt.executeQuery( selectStmt );

      Vector v = new Vector();
      while ( rs.next() ) {
         Title t = new Title( rs.getInt(1) );
         t.setParentInd( a.getInd() );

         t.setSortOrder( rs.getShort(2) );
         t.setTitle( rs.getString(3) );
         t.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
         t.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
         t.setDbUser( rs.getString(6).trim() );
         t.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
         t.setLanguage( new TypeLanguageRec( rs.getShort(8), "" ) );
         t.setLastTranslationDate(
                                 DatabaseUtil.timestampFromDB( rs.getString(9) ) 
                                 );

         // read the conditions/countries
         // MAD performance: title conditions are now read all at once below
         //t.setConditions(
         //   ConditionReader.readConditions( conn, schema + ".actionTitleCond",
         //                                   "actionTitleInd", t )
         //);
         // MAD performance: title countries are now read all at once below
         //t.setCountryList(
         //   CountryReader.readCountries( conn, schema + ".actionTitleCountry",
         //                                "actionTitleInd", t.getInd() )
         //);

         t.setTextUpdated( false );
         t.updateRecStatus(PersistentRec.REC_NOCHANGE);
         v.addElement(t);
      }

      rs.close();
      stmt.close();

      a.setTitles(v);

      // read conditions/countries for all action titles
      // we should have always read at least one title
      ConditionReader.readAllActionTitleConditions(conn, a, schema, false);
      CountryReader.readAllActionTitleCountries(conn, a, schema, false);
   }

//=============================================================================

   /**
    * Read Question Titles for all languages (oa schema).
    * @param conn         the database connection
    * @param q            the Question whose titles are to be read
   **/
   static void readQuestionTitles(Connection conn, Question q) 
   throws SQLException 
   {
      readQuestionTitles(conn, q, ACTIVE_SCHEMA);
   }

   /**
    * Read Question Titles for all languages (oadraft schema).
    * @param conn         the database connection
    * @param q            the Question whose titles are to be read
   **/
   static void readQuestionDraftTitles(Connection conn, Question q) 
   throws SQLException 
   {
      readQuestionTitles(conn, q, DRAFT_SCHEMA);
   }

   /**
    * Read Question Titles for all languages.
    * @param conn         the database connection
    * @param q            the Question whose titles are to be read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   private static void readQuestionTitles(Connection conn, Question q,
                                          String schema) 
   throws SQLException 
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // select all titles
      rs = stmt.executeQuery( "SELECT " +
                              "questionTitleInd, sortOrder, " +
                              "title, docClassInd, created, " +
                              "dbUser, changedTime, " +
                              "languageInd, lastTranslated" +
                              " FROM " + schema + ".questionTitle" +
                              " WHERE questionInd=" + q.getInd() +
                              " ORDER BY sortOrder, languageInd" +
                              " FOR FETCH ONLY" );


      Vector v = new Vector();        
      while ( rs.next() ) {
         Title t = new Title( rs.getInt(1) );
         t.setParentInd( q.getInd() );

         t.setSortOrder( rs.getShort(2) );
         t.setTitle( rs.getString(3) );
         t.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
         t.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
         t.setDbUser( rs.getString(6).trim() );
         t.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
         t.setLanguage( new TypeLanguageRec( rs.getShort(8), "" ) );
         t.setLastTranslationDate(DatabaseUtil.timestampFromDB( rs.getString(9) ) );

         // read the conditions/countries
         // MAD performance: title conditions are now read all at once below
         //t.setConditions(
         //   ConditionReader.readConditions( conn, schema + ".questionTitleCond",
         //                                   "questionTitleInd", t )
         //);
         // MAD performance: title countries are now read all at once below
         //t.setCountryList(
         //   CountryReader.readCountries( conn, schema + ".questionTitleCntry",
         //                                "questionTitleInd", t.getInd() )
         //);

         t.setTextUpdated( false );
         t.updateRecStatus(PersistentRec.REC_NOCHANGE);
         v.addElement(t);
      }

      rs.close();          
      stmt.close();

      q.setTitles(v);

      // read conditions/countries for all question titles
      // we should have always read at least one title
      ConditionReader.readAllQuestionTitleConditions(conn, q, schema, true);
      CountryReader.readAllQuestionTitleCountries(conn, q, schema, true);
   }

//=============================================================================

   /**
    * Read Question Titles for a specific language (oa schema).
    * If languageInd=0 (read all languages), 
    * then readQuestionTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param q            the Question whose titles are to be read
    * @param languageInd  the language to read
   **/
   static void readQuestionTitles(Connection conn, Question q, int languageInd) 
   throws SQLException 
   {

      if ( languageInd == 0 ) {
         readQuestionTitles(conn, q, ACTIVE_SCHEMA);
      } else {
         readQuestionTitles(conn, q, languageInd, ACTIVE_SCHEMA);
      }
   }

   /**
    * Read Question Titles for a specific language (oadraft schema).
    * If languageInd=0 (read all languages), 
    * then readQuestionTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param q            the Question whose titles are to be read
    * @param languageInd  the language to read
   **/
   static void readQuestionDraftTitles(Connection conn, Question q,
                                       int languageInd) 
   throws SQLException 
   {
      if ( languageInd == 0 ) {
         readQuestionTitles(conn, q, DRAFT_SCHEMA);
      } else {
         readQuestionTitles(conn, q, languageInd, DRAFT_SCHEMA);
      }
   }

   /**
    * Read Question Titles for a specific language (common function).
    * This method will attempt to read the titles in the specified language.
    * If a title is not found in the requested language, the English title
    * will be read.
    * @param conn         the database connection
    * @param q            the Question whose titles are to be read
    * @param languageInd  the language to read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   private static void readQuestionTitles(Connection conn, Question q,
                                          int languageInd, String schema) 
   throws SQLException 
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // first select statement is to get titles in the requested language...
      String sql = "SELECT " +
                   "questionTitleInd, sortOrder, " +
                   "title, docClassInd, created, " +
                   "dbUser, changedTime, " +
                   "languageInd, lastTranslated " +
                   "FROM " + schema + ".questionTitle " +
                   "WHERE questionInd=" + q.getInd()  + " " +
                   "AND   languageInd=" + languageInd + " ";

      // ... if the requested language was not English, then we have to UNION
      // with a second select statement that will get the English title for
      // those that were not available in the requested language.
      // the second select statements uses a subselect to exclude those titles
      // which exist in the requested language (and thus were selected by the
      // previous select statement).
      if ( languageInd != 1 ) {
         sql += "UNION " +
                "SELECT " +
                "questionTitleInd, sortOrder, " +
                "title, docClassInd, created, " +
                "dbUser, changedTime, " +
                "languageInd, lastTranslated " +
                "FROM " + schema + ".questionTitle " +
                "WHERE questionInd=" + q.getInd() + " " +
                "AND   languageInd=1 " +
                "AND   questionTitleInd NOT IN (" +
                "SELECT questionTitleInd " +
                "FROM " + schema + ".questionTitle " +
                "WHERE questionInd=" + q.getInd() + " " +
                "AND languageInd=" + languageInd +
                ") ";
      }

      // finish off the select statement:
      sql += "ORDER BY sortOrder " +
             "FOR FETCH ONLY";

      rs = stmt.executeQuery( sql );

      Vector v = new Vector();        
      while ( rs.next() ) {
         Title t = new Title( rs.getInt(1) );
         t.setParentInd( q.getInd() );

         t.setSortOrder( rs.getShort(2) );
         t.setTitle( rs.getString(3) );
         t.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
         t.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
         t.setDbUser( rs.getString(6).trim() );
         t.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
         t.setLanguage( new TypeLanguageRec( rs.getShort(8), "" ) );
         t.setLastTranslationDate(
                                 DatabaseUtil.timestampFromDB( rs.getString(9) ) 
                                 );

         // read the conditions/countries
         // MAD performance: title conditions are now read all at once below
         //t.setConditions(
         //   ConditionReader.readConditions( conn, schema + ".questionTitleCond",
         //                                   "questionTitleInd", t )
         //);
         // MAD performance: title countries are now read all at once below
         //t.setCountryList(
         //   CountryReader.readCountries( conn, schema + ".questionTitleCntry",
         //                                "questionTitleInd", t.getInd() )
         //);

         t.setTextUpdated( false );
         t.updateRecStatus(PersistentRec.REC_NOCHANGE);
         v.addElement(t);
      }

      rs.close();          
      stmt.close();

      q.setTitles(v);

      // read conditions/countries for all question titles
      // we should have always read at least one title
      ConditionReader.readAllQuestionTitleConditions(conn, q, schema, false);
      CountryReader.readAllQuestionTitleCountries(conn, q, schema, false);
   }

//=============================================================================

   /**
    * Read Answer Titles for all languages (oa schema).
    * Note: this method is obsolete -- readAllAnswerTitles() is used instead
   **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readAnswerTitles(Connection conn, Answer a)
         throws SQLException
  {
    readAnswerTitles(conn, a, ACTIVE_SCHEMA);
  }
------------------------- END COMMENTED OUT ----------------------------- */

   /**
    * Read Answer Titles for all languages (oadraft schema).
    * Note: this method is obsolete -- readAllAnswerTitles() is used instead
   **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readAnswerDraftTitles(Connection conn, Answer a)
         throws SQLException
  {
    readAnswerTitles(conn, a, DRAFT_SCHEMA);
  }
------------------------- END COMMENTED OUT ----------------------------- */

   /**
    * Read Answer Titles for all languages (common function).
    * Note: this method is obsolete -- readAllAnswerTitles() is used instead
   **/
/* ---------------------- START COMMENTED OUT ------------------------------
  private static void readAnswerTitles(Connection conn, Answer a, String schema)
                 throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    // select all titles for the answer
    rs = stmt.executeQuery( "SELECT " +
                              "answerTitleInd, sortOrder, " +
                              "title, docClassInd, created, " +
                              "dbUser, changedTime, " +
                              "languageInd, lastTranslated" +
                            " FROM " + schema + ".answerTitle" +
                            " WHERE answerInd=" + a.getInd() +
                            " ORDER BY sortOrder, languageInd" +
                            " FOR FETCH ONLY" );


    Vector v = new Vector();        
    while( rs.next() )
    {
      Title t = new Title( rs.getInt(1) );
      t.setParentInd( a.getInd() );

      t.setSortOrder( rs.getShort(2) );
      t.setTitle( rs.getString(3) );
      t.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
      t.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
      t.setDbUser( rs.getString(6).trim() );
      t.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
      t.setLanguage( new TypeLanguageRec( rs.getShort(8), "" ) );
      t.setLastTranslationDate( 
         DatabaseUtil.timestampFromDB( rs.getString(9) ) 
      );

      // read the conditions/countries
      t.setConditions(
         ConditionReader.readConditions( conn, schema + ".answerTitleCond",
                                         "answerTitleInd", t )
      );
      t.setCountryList(
         CountryReader.readCountries( conn, schema + ".answerTitleCountry",
                                      "answerTitleInd", t.getInd() )
      );

      t.setTextUpdated( false );
      t.updateRecStatus(PersistentRec.REC_NOCHANGE);
      v.addElement(t);
    }
         
    rs.close();          
    stmt.close();

    a.setTitles(v);

  }
------------------------- END COMMENTED OUT ----------------------------- */

//=============================================================================

   /**
    * Read all Answer Titles for all languages for a given Question
    * (oa schema).
    * @param conn         the database connection
    * @param q            the Question whose Answer titles are to be read
   **/
   static void readAllAnswerTitles(Connection conn, Question q)
   throws SQLException
   {
      readAllAnswerTitles(conn, q, ACTIVE_SCHEMA);
   }

   /**
    * Read all Answer Titles for all languages for a given Question
    * (oadraft schema).
    * @param conn         the database connection
    * @param q            the Question whose Answer titles are to be read
   **/
   static void readAllAnswerDraftTitles(Connection conn, Question q)
   throws SQLException
   {
      readAllAnswerTitles(conn, q, DRAFT_SCHEMA);
   }

   /**
    * Read all Answer Titles for all languages for a given Question
    * (common function).
    * @param conn         the database connection
    * @param q            the Question whose Answer titles are to be read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   private static void readAllAnswerTitles(Connection conn, Question q,
                                           String schema)
   throws SQLException
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // select titles for all answers to the question
      rs = stmt.executeQuery( "SELECT " +
                              "at.answerTitleInd, at.sortOrder, " +
                              "at.title, at.docClassInd, at.created, " +
                              "at.dbUser, at.changedTime, " +
                              "at.languageInd, at.lastTranslated, " +
                              "at.answerInd" +
                              " FROM " + schema + ".answerTitle at, " +
                              schema + ".answer a" +
                              " WHERE at.answerInd=a.answerInd" + 
                              " AND a.questionInd=" + q.getInd() + 
                              " ORDER BY at.answerInd, " +
                              "at.sortOrder, at.languageInd" +
                              " FOR FETCH ONLY" );

      int answerInd = 0;             // current title's answerInd
      int previousAnswerInd = -99;   // previous title's answerInd

      Vector titleVec = null;

      while ( rs.next() ) {

         answerInd = rs.getInt(10);  // get current answerInd

         // when we encounter a new answerInd, set the Vector of Titles
         // for the previous answerInd.
         if ( answerInd != previousAnswerInd ) {
            if ( titleVec != null ) {  // if not first time through
               Answer ans = q.getAnswerByInd( previousAnswerInd );
               ans.setTitles( titleVec );
            }
            titleVec = new Vector(1, 5);
            previousAnswerInd = answerInd;
         }

         Title t = new Title( rs.getInt(1) );
         t.setParentInd( answerInd );

         t.setSortOrder( rs.getShort(2) );
         t.setTitle( rs.getString(3) );
         t.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
         t.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
         t.setDbUser( rs.getString(6).trim() );
         t.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
         t.setLanguage( new TypeLanguageRec( rs.getShort(8), "" ) );
         t.setLastTranslationDate(
                                 DatabaseUtil.timestampFromDB( rs.getString(9) ) 
                                 );

         t.setTextUpdated( false );
         t.updateRecStatus(PersistentRec.REC_NOCHANGE);
         titleVec.addElement(t);
      }

      rs.close();          
      stmt.close();

      // put the last Vector of Titles into the appropriate Answer object
      // titleVec can be null if no data was selected -- shouldnt happen!
      // (we should always have at least 1 answer title)
      if ( titleVec != null ) {
         Answer ans = q.getAnswerByInd( answerInd );
         ans.setTitles( titleVec );
      }

      // read the conditions/countries for all answer titles
      // we should have always read at least one title
      ConditionReader.readAllAnswerTitleConditions(conn, q, schema, true);
      CountryReader.readAllAnswerTitleCountries(conn, q, schema, true);
   }

//=============================================================================

   /**
    * Read Answer Titles for a specific language (oa schema).
    * Note: this method is obsolete -- readAllAnswerTitles() is used instead.
   **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readAnswerTitles(Connection conn, Answer a, int languageInd)
         throws SQLException
  {
    readAnswerTitles(conn, a, languageInd, ACTIVE_SCHEMA);
  }
------------------------- END COMMENTED OUT ----------------------------- */

   /**
    * Read Answer Titles for a specific language (oadraft schema).
    * Note: this method is obsolete -- readAllAnswerTitles() is used instead.
   **/
/* ---------------------- START COMMENTED OUT ------------------------------
  static void readAnswerDraftTitles(Connection conn, Answer a, int languageInd)
         throws SQLException
  {
    readAnswerTitles(conn, a, languageInd, DRAFT_SCHEMA);
  }
------------------------- END COMMENTED OUT ----------------------------- */

   /**
    * Read Answer Titles for a specific language (common function).
    * This method will attempt to read the titles in the specified language.
    * If a title is not found in the requested language, the English title
    * will be read.
    * Note: this method is obsolete -- readAllAnswerTitles() is used instead.
   **/
/* ---------------------- START COMMENTED OUT ------------------------------
  private static void readAnswerTitles(Connection conn, Answer a, 
                                       int languageInd, String schema)
                 throws SQLException
  {
    PreparedStatement pstmt = null;
    Statement  stmt = conn.createStatement();
    ResultSet  rs1,    // result set for all unique title indicators
               rs2;    // result set for specific title/language

    // prepare a statement to read the titles by language indicator
    pstmt = conn.prepareStatement( "SELECT " +
                                     "answerTitleInd, sortOrder, " +
                                     "title, docClassInd, created, " +
                                     "dbUser, changedTime, " +
                                     "languageInd, lastTranslated" +
                                   " FROM " + schema + ".answerTitle" +
                                   " WHERE answerTitleInd=?" +
                                   " AND languageInd=?" + 
                                   " FOR FETCH ONLY");

    // select all the title indicators for the Answer
    rs1 = stmt.executeQuery("SELECT DISTINCT answerTitleInd, sortOrder" +
                           " FROM " + schema + ".answerTitle" +
                           " WHERE answerInd=" + a.getInd() +
                           " ORDER BY sortOrder" +
                           " FOR FETCH ONLY" );

    Vector v = new Vector();        
    while( rs1.next() )
    {
      // try to read the title in the requested language
      int answerTitleInd = rs1.getInt(1);
      pstmt.setInt(1, answerTitleInd);
      pstmt.setInt(2, languageInd);
      rs2 = pstmt.executeQuery();

      if(rs2.next() == false)
      {
        // could not find the requested language -- read English title
        pstmt.setInt(1, answerTitleInd);
        pstmt.setInt(2, 1);
        rs2 = pstmt.executeQuery();
        rs2.next();
      } 

      Title t = new Title( rs2.getInt(1) );
      t.setParentInd( a.getInd() );

      t.setSortOrder( rs2.getShort(2) );
      t.setTitle( rs2.getString(3) );
      t.setDocClass( new TypeDocClassRec( rs2.getShort(4), "") );
      t.setCreationTime( DatabaseUtil.timestampFromDB( rs2.getString(5) ) );
      t.setDbUser( rs2.getString(6).trim() );
      t.setChangedTime( DatabaseUtil.timestampFromDB( rs2.getString(7) ) );
      t.setLanguage( new TypeLanguageRec( rs2.getShort(8), "" ) );
      t.setLastTranslationDate( 
         DatabaseUtil.timestampFromDB( rs2.getString(9) ) 
      );
      rs2.close();

      // read the conditions/countries
      t.setConditions(
         ConditionReader.readConditions( conn, schema + ".answerTitleCond",
                                         "answerTitleInd", t )
      );
      t.setCountryList(
         CountryReader.readCountries( conn, schema + ".answerTitleCountry",
                                      "answerTitleInd", t.getInd() )
      );

      t.setTextUpdated( false );
      t.updateRecStatus(PersistentRec.REC_NOCHANGE);
      v.addElement(t);
    }
         
    rs1.close();          
    stmt.close();
    pstmt.close();

    a.setTitles(v);
  }
------------------------- END COMMENTED OUT ----------------------------- */

//=============================================================================

   /**
    * Read all Answer titles for a specific langauge for a given Question
    * (oa schema).
    * If languageInd=0 (read all languages), 
    * then readAllAnswerTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param q            the Question whose Answer titles are to be read
    * @param languageInd  the language to read
   **/
   static void readAllAnswerTitles(Connection conn, Question q, int languageInd)
   throws SQLException
   {
      if ( languageInd == 0 ) {
         readAllAnswerTitles(conn, q, ACTIVE_SCHEMA);
      } else {
         readAllAnswerTitles(conn, q, languageInd, ACTIVE_SCHEMA);
      }
   }

   /**
    * Read all Answer titles for a specific langauge for a given Question
    * (oadraft schema).
    * If languageInd=0 (read all languages), 
    * then readAllAnswerTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param q            the Question whose Answer titles are to be read
    * @param languageInd  the language to read
   **/
   static void readAllAnswerDraftTitles(Connection conn, Question q, 
                                        int languageInd)
   throws SQLException
   {
      if ( languageInd == 0 ) {
         readAllAnswerTitles(conn, q, DRAFT_SCHEMA);
      } else {
         readAllAnswerTitles(conn, q, languageInd, DRAFT_SCHEMA);
      }
   }

   /**
    * Read all Answer titles for a specific langauge for a given Question
    * (common function).
    * This method will attempt to read the titles in the specified language.
    * If a title is not found in the requested language, the English title
    * will be read (English = languageInd 1).
    * @param conn         the database connection
    * @param q            the Question whose Answer titles are to be read
    * @param languageInd  the language to read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   private static void readAllAnswerTitles(Connection conn, Question q, 
                                           int languageInd, String schema)
   throws SQLException
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // read all titles for all answers to the question:
      // select all titles for the requested language....
      String selectStmt = "SELECT " +
                          "at.answerTitleInd, at.sortOrder, " +
                          "at.title, at.docClassInd, at.created, " +
                          "at.dbUser, at.changedTime, " +
                          "at.languageInd, at.lastTranslated, " +
                          "at.answerInd" +
                          " FROM " + schema + ".answerTitle at, " +
                          schema + ".answer a" +
                          " WHERE at.languageInd=" + languageInd +
                          " AND at.answerInd=a.answerInd" + 
                          " AND a.questionInd=" + q.getInd() + " ";

      // ... if the requested language was not English, then we have to UNION
      // with a second select statment that will get the English titles for 
      // those that were not available in the requested language.
      // use a subselect to exclude those titles which exist in the requested
      // language (and thus were selected by the previous select stmt).
      if ( languageInd != 1 ) {
         selectStmt += "UNION " +
                       "SELECT " +
                       "at.answerTitleInd, at.sortOrder, " +
                       "at.title, at.docClassInd, at.created, " +
                       "at.dbUser, at.changedTime, " +
                       "at.languageInd, at.lastTranslated, " +
                       "at.answerInd" +
                       " FROM " + schema + ".answerTitle at, " +
                       schema + ".answer a" +
                       " WHERE at.languageInd=1" + 
                       " AND at.answerInd=a.answerInd" + 
                       " AND a.questionInd=" + q.getInd() +
                       " AND at.answerTitleInd NOT IN (" +
                       "SELECT at.answerTitleInd " +
                       "FROM " + schema + ".answerTitle at, " +
                       schema + ".answer a " +
                       "WHERE at.languageInd=" + languageInd + " " +
                       "AND at.answerInd=a.answerInd " +
                       "AND a.questionInd=" + q.getInd() +
                       ") ";
      }

      // finish off the select statement:
      selectStmt += "ORDER BY answerInd, sortOrder " +
                    "FOR FETCH ONLY";


      rs = stmt.executeQuery( selectStmt );

      int answerInd = 0;             // current title's answerInd
      int previousAnswerInd = -99;   // previous title's answerInd

      Vector titleVec = null;

      while ( rs.next() ) {

         answerInd = rs.getInt(10);  // get current answerInd

         // when we encounter a new answerInd, set the Vector of Titles
         // for the previous answerInd.
         if ( answerInd != previousAnswerInd ) {
            if ( titleVec != null ) {  // if not first time through
               Answer ans = q.getAnswerByInd( previousAnswerInd );
               ans.setTitles( titleVec );
            }
            titleVec = new Vector(1, 5);
            previousAnswerInd = answerInd;
         }

         Title t = new Title( rs.getInt(1) );
         t.setParentInd( answerInd );

         t.setSortOrder( rs.getShort(2) );
         t.setTitle( rs.getString(3) );
         t.setDocClass( new TypeDocClassRec( rs.getShort(4), "") );
         t.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
         t.setDbUser( rs.getString(6).trim() );
         t.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
         t.setLanguage( new TypeLanguageRec( rs.getShort(8), "" ) );
         t.setLastTranslationDate(
                                 DatabaseUtil.timestampFromDB( rs.getString(9) ) 
                                 );

         t.setTextUpdated( false );
         t.updateRecStatus(PersistentRec.REC_NOCHANGE);
         titleVec.addElement(t);
      }

      rs.close();          
      stmt.close();

      // put the last Vector of Titles into the appropriate Answer object
      // titleVec can be null if no data was selected -- shouldnt happen!
      // (we should always have at least 1 answer title)
      if ( titleVec != null ) {
         Answer ans = q.getAnswerByInd( answerInd );
         ans.setTitles( titleVec );
      }

      // read the conditions/countries for all answer titles
      // we should have always read at least one title
      ConditionReader.readAllAnswerTitleConditions(conn, q, schema, false);
      CountryReader.readAllAnswerTitleCountries(conn, q, schema, false);
   }

//=============================================================================

   /**
    * Read Symptom Titles for all languages (oa schema).
    * @param conn         the database connection
    * @param s            the Symptom whose titles are to be read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   static void readSymptomTitles(Connection conn, Symptom s)
   throws SQLException
   {
      readSymptomTitles(conn, s, ACTIVE_SCHEMA);
   }

   /**
    * Read Symptom Titles for all languages (oadraft schema).
    * @param conn         the database connection
    * @param s            the Symptom whose titles are to be read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   static void readSymptomDraftTitles(Connection conn, Symptom s)
   throws SQLException
   {
      readSymptomTitles(conn, s, DRAFT_SCHEMA);
   }

   /**
    * Read Symptom Titles for all languages (common function).
    * @param conn         the database connection
    * @param s            the Symptom whose titles are to be read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   private static void readSymptomTitles(Connection conn, Symptom s,
                                         String schema)
   throws SQLException
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // select all titles
      rs = stmt.executeQuery( "SELECT " +
                              "symptomTitleInd, sortOrder, title, " +
                              "keywords, languageInd, lastTranslated, " +
                              "docClassInd, indexable, " +
                              "dbUser, changedTime" +
                              " FROM " + schema + ".symptomTitle " +
                              " WHERE symptomInd= " + s.getInd() +
                              " ORDER BY sortOrder, languageInd" +
                              " FOR FETCH ONLY" );


      Vector v = new Vector();
      while ( rs.next() ) {
         SymptomTitle st = new SymptomTitle( rs.getInt(1) );
         st.setSymptomInd( s.getInd() );

         st.setSortOrder( rs.getShort(2) );
         st.setTitle( rs.getString(3) );
         st.setKeywords( rs.getString(4) );
         st.setLanguage( new TypeLanguageRec( rs.getShort(5), "" ) );
         st.setLastTranslationDate(
                                  DatabaseUtil.timestampFromDB( rs.getString(6) )
                                  );
         st.setDocClass( new TypeDocClassRec( rs.getShort(7), "") );
         st.setIndexable( DatabaseUtil.booleanFromDB( rs.getString(8) ) );
         st.setDbUser( rs.getString(9).trim() );
         st.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(10) ) );

         // read the countries
         // MAD performance: title countries are now read all at once below
         //st.setCountryList(
         //   CountryReader.readCountries( conn, schema + ".symptomTitleCntry",
         //                                "symptomTitleInd", st.getInd() )
         //);

         st.setTextUpdated( false );
         st.updateRecStatus(PersistentRec.REC_NOCHANGE);
         v.addElement(st);
      }

      rs.close();
      stmt.close();

      s.setSymptomTitles(v);

      // read countries for all symptom titles
      // we should have always read at least one title
      CountryReader.readAllSymptomTitleCountries(conn, s, schema, true);
   }

//=============================================================================

   /**
    * Read Symptom Titles for a specific language (oa schema).
    * If languageInd=0 (read all languages), 
    * then readSymptomTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param s            the Symptom whose titles are to be read
    * @param languageInd  the language to read
   **/
   static void readSymptomTitles(Connection conn, Symptom s, int languageInd)
   throws SQLException
   {
      if ( languageInd == 0 ) {
         readSymptomTitles(conn, s, ACTIVE_SCHEMA);
      } else {
         readSymptomTitles(conn, s, languageInd, ACTIVE_SCHEMA);
      }
   }

   /**
    * Read Symptom Titles for a specific language (oadraft schema).
    * If languageInd=0 (read all languages), 
    * then readSymptomTitles(conn, q) will be called.
    * @param conn         the database connection
    * @param s            the Symptom whose titles are to be read
    * @param languageInd  the language to read
   **/
   static void readSymptomDraftTitles(Connection conn, Symptom s, 
                                      int languageInd)
   throws SQLException
   {
      if ( languageInd == 0 ) {
         readSymptomTitles(conn, s, DRAFT_SCHEMA);
      } else {
         readSymptomTitles(conn, s, languageInd, DRAFT_SCHEMA);
      }
   }

   /**
    * Read Symptom Titles for a specific language (common function).
    * This method will attempt to read the titles in the specified language.
    * If a title is not found in the requested language, the English title
    * will be read.
    * @param conn         the database connection
    * @param s            the Symptom whose titles are to be read
    * @param languageInd  the language to read
    * @param schema       the database schema to read (oa or oadraft)
   **/
   private static void readSymptomTitles(Connection conn, Symptom s,
                                         int languageInd, String schema)
   throws SQLException
   {
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // first select statement is to get titles in the requested language...
      String selectStmt = "SELECT " +
                          "symptomTitleInd, sortOrder, title, " +
                          "keywords, languageInd, lastTranslated, " +
                          "docClassInd, indexable, " +
                          "dbUser, changedTime " +
                          "FROM " + schema + ".symptomTitle " +
                          "WHERE symptomInd="  + s.getInd()  + " " +
                          "AND   languageInd=" + languageInd + " ";

      // ... if the requested language was not English, then we have to UNION
      // with a second select statement that will get the English title for
      // those that were not available in the requested language.
      // the second select statements uses a subselect to exclude those titles
      // which exist in the requested language (and thus were selected by the
      // previous select statement).
      if ( languageInd != 1 ) {
         selectStmt += "UNION " +
                       "SELECT " +
                       "symptomTitleInd, sortOrder, title, " +
                       "keywords, languageInd, lastTranslated, " +
                       "docClassInd, indexable, " +
                       "dbUser, changedTime " +
                       "FROM " + schema + ".symptomTitle " +
                       "WHERE symptomInd="  + s.getInd()  + " " +
                       "AND   languageInd=1 " + 
                       "AND   symptomTitleInd NOT IN (" +
                       "SELECT symptomTitleInd " +
                       "FROM " + schema + ".symptomTitle " +
                       "WHERE symptomInd="  + s.getInd() + " " +
                       "AND   languageInd=" + languageInd +
                       ") ";
      }

      // finish off the select statement:
      selectStmt += "ORDER BY sortOrder " +
                    "FOR FETCH ONLY";

      rs = stmt.executeQuery( selectStmt );


      Vector v = new Vector();
      while ( rs.next() ) {

         SymptomTitle st = new SymptomTitle( rs.getInt(1) );
         st.setSymptomInd( s.getInd() );

         st.setSortOrder( rs.getShort(2) );
         st.setTitle( rs.getString(3) );
         st.setKeywords( rs.getString(4) );
         st.setLanguage( new TypeLanguageRec( rs.getShort(5), "" ) );
         st.setLastTranslationDate(
                                  DatabaseUtil.timestampFromDB( rs.getString(6) )
                                  );
         st.setDocClass( new TypeDocClassRec( rs.getShort(7), "") );
         st.setIndexable( DatabaseUtil.booleanFromDB( rs.getString(8) ) );
         st.setDbUser( rs.getString(9).trim() );
         st.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(10) ) );

         // read the countries
         // MAD performance: title countries are now read all at once below
         //st.setCountryList(
         //   CountryReader.readCountries( conn, schema + ".symptomTitleCntry",
         //                                "symptomTitleInd", st.getInd() )
         //);

         st.setTextUpdated( false );
         st.updateRecStatus(PersistentRec.REC_NOCHANGE);
         v.addElement(st);
      }

      rs.close();
      stmt.close();

      s.setSymptomTitles(v);

      // read countries for all symptom titles
      // we should have always read at least one title
      CountryReader.readAllSymptomTitleCountries(conn, s, schema, false);
   }
}
