package com.ibm.nzna.projects.common.quest.product ;
/*-----------------------------------------------------------*/
/*   OPICMMapper                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OPICMMapper                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class holds APIs for mapping Quest Brands Families with OPIC/M Brands, Families and Series.
 * It also has APIs for mapping the Geos/Countries from OPIC/M to Quest Geos/Countries
 *
 *
 * @version 1.0, 9/2/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OPICMMapper                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class OPICMMapper extends Object {


   /**
     *  This is the hash table which stores all information from OPICM.BRANDFAMILYMAP
     **/
   private Hashtable seriesHash = new Hashtable() ;

   /**
     *  This is the hash table which stores all information from OPICM.GEOCOUNTRYMAP
     **/
   private Hashtable countryHash = new Hashtable() ;

   /**
     *  This is the default constructor which reads from the BRANDFAMILYMAP and
     * GEOCOUNTRYMAP tables. It will not return until both tables are read in
     **/
   public OPICMMapper () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Preread the OPIC/M Information                          */                     
      /*                                                           */
      /*-----------------------------------------------------------*/
      TypeCategory.getCategories ( TypeCategory.LEVEL_OPICM_BRAND ) ;
      TypeCategory.getCategories ( TypeCategory.LEVEL_OPICM_FAMILY ) ;
      TypeCategory.getCategories ( TypeCategory.LEVEL_OPICM_SERIES ) ;
      TypeCategory.getCategories ( TypeCategory.LEVEL_OPICM_GEOGRAPHY ) ;
      TypeCategory.getCategories ( TypeCategory.LEVEL_OPICM_COUNTRY ) ;

      readBrandFamilyMap() ;
      readGeoCountryMap() ;
   }



   /**
     *  This method will return the TypeCategoryRec object representing the
     * OPIC/M Brand, Family and Series passed in
     *
     * @param brand The OPIC/M Brand 
     * @param family The OPIC/M Family
     * @param series The OPIC/M Series
     *
     * @return TypeCategoryRec of the Quest Family, or NULL if no mapping was made
     **/
   public TypeCategoryRec getQuestFamilyFromOPICM ( String brand, String family, String series ) {
      TypeCategoryRec questFamily = null ;
      try {
         TypeCategoryRec opicmBrand  = TypeCategory.getCategoryFromDescription ( brand,  TypeCategory.LEVEL_OPICM_BRAND ) ;
         TypeCategoryRec opicmFamily = TypeCategory.getCategoryFromDescription ( family, TypeCategory.LEVEL_OPICM_FAMILY, opicmBrand.getInd() ) ;
         TypeCategoryRec opicmSeries = TypeCategory.getCategoryFromDescription ( series, TypeCategory.LEVEL_OPICM_SERIES, opicmFamily.getInd() ) ;                                          

         questFamily = (TypeCategoryRec)seriesHash.get ( opicmSeries ) ;

      } catch ( NullPointerException e ) {
         // Do nothing. %99.9 of the time this is because the Brand.Family or Series is not found 
      }

      return( questFamily ) ;
   }



   /**
     *  This method will return the TypeCountryCodeRec for the OPIC/M Country
     *
     * @param geo The OPIC/M Geography
     * @param country The OPIC/M Country
     *
     * @return TypeCountryCodeRec representing the Quest country
     **/
   public TypeCountryCodeRec getQuestCountryFromOPICM ( String geo, String country ) {
      TypeCountryCodeRec questCountry = null ;
      try {
         TypeCategoryRec opicmGeo     = TypeCategory.getCategoryFromDescription ( geo,     TypeCategory.LEVEL_OPICM_GEOGRAPHY ) ;
         TypeCategoryRec opicmCountry = TypeCategory.getCategoryFromDescription ( country, TypeCategory.LEVEL_OPICM_COUNTRY, opicmGeo.getInd() ) ;

         questCountry = (TypeCountryCodeRec)countryHash.get ( opicmCountry ) ;

      } catch ( NullPointerException e ) {
         // Do nothing. %99.9 of the time this is because the Brand.Family or Series is not found 
      }

      return( questCountry ) ;
   }


   /**
     *  This method will populate our countryHash table with all OPIC/M Countries and Quest Countries
     * information from the OPICM.GEOCOUNTRMAP table
     **/
   private void readBrandFamilyMap () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readBrandFamilyMap", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT OPICMTYPECATIND, QUESTTYPECATIND "+
                                              "FROM OPICM.BRANDFAMILYMAP FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            TypeCategoryRec seriesRec = TypeCategory.getCategory ( rs.getInt(1) ) ;
            TypeCategoryRec familyRec = TypeCategory.getCategory ( rs.getInt(2) ) ;

            if ( seriesRec != null ) {
               if ( seriesHash.get ( seriesRec ) == null ) {
                  seriesHash.put ( seriesRec, familyRec ) ;
               }
            }
         }

         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;
   }



   /**
     *  This method will populate our seriesHash table with all OPIC/M Series and Quest Family
     * information from the OPICM.BRANDFAMILYMAP table
     **/
   private void readGeoCountryMap () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readGeoCountryMap", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT TYPECATIND, COUNTRYCODEIND "+
                                              "FROM OPICM.GEOCOUNTRYMAP FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            TypeCategoryRec    opicmCountry = TypeCategory.getCategory ( rs.getInt(1) ) ;
            TypeCountryCodeRec countryRec   = (TypeCountryCodeRec)TypeList.getInstance().objectFromInd ( rs.getInt(2), TypeList.TYPE_COUNTRYCODE ) ;

            if ( countryHash.get ( opicmCountry ) == null ) {
               countryHash.put ( opicmCountry, countryRec ) ;
            }
         }

         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;
   }


























   /**
     *  This is a test main function only for testing purposes
     **/
   public static void main ( String args[] ) {
      try {

         new DatabaseSystem ( false, null, false ) ;
         if ( DatabaseSystem.createConnection ( DatabaseSystem.QUEST,
                                                "quest",
                                                "defiant",
                                                "5703",
                                                "dlentz",
                                                "m0seley" ) ) {
            TypeList typeList = new TypeList () ;
            typeList.initialize ( false ) ;
            new TypeCategory() ;
            OPICMMapper mapper = new OPICMMapper() ;


            System.out.println ( "Quest Family for OPICM Family 1:" ) ;
            System.out.println ( mapper.getQuestFamilyFromOPICM ( "OPICMBrand1", "OPICMFamily1", "OPICMSeries1" )+"\n\n" ) ;
            System.out.println ( "Quest Family for OPICM Family 2:" ) ;
            System.out.println ( mapper.getQuestFamilyFromOPICM ( "OPICMBrand2", "OPICMFamily2", "OPICMSeries2" )+"\n\n" ) ;

            System.out.println ( "Country for OPICMCountry:" ) ;
            System.out.println ( mapper.getQuestCountryFromOPICM ( "OPICMGeography", "OPICMCountry" ) ) ;
         }

      } catch ( Exception e ) {
      }
   }


} /* OPICMMapper */
