package com.ibm.nzna.projects.common.quest.product;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      ProductDraft
/*   Creation Date: APRIL 13, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
/**
 * The ProductDraft class subclasses the Product class
 * to produce an Object that the user may make changes to
 * and save to the database without affecting the current
 * existing product. Quest Input only makes changes to drafts.
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* ProductDraft
/*                                  
/*----------------------------------------------------*/
public class ProductDraft extends Product {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
    * This is the user editable comment of the product draft. The user normally puts 
    * information into this field that tells what they changed and why they changed it
    */
   private ProductComment comment = null ;

   /**
     *  This is the original comments that are currently saved in the database. These
     * comments are read only
     **/
   private String originalComments = null ;

   /**
    * The Existing Product Indicator is the Product Primary Key for this Draft's
    * Published Product. This indicator does not need to be in the
    * PRODRAFT.PRODUCT table. For example, if this draft represents a Draft for
    * a new Product, then the Existing Product Indicator might be a non-zero
    * value, but it also is not in the PRODRAFT.PRODUCT table until the draft
    * is published.
    */
   private int existingProductInd = 0 ;

   /**
    * This read only date is the date that the draft was started on. This date defaults
    * to the date that this record was created but can be modified through setStartDate
    */
   private String startDate = new CDate ( CDate.DATE_STAMP ).today() ;

   /**
    * This read only date is the date that this draft was stopped on 
    */
   private String stopDate = null ;

   /**
    * This identifies this product draft as a 'Misfit' product. 
    */
   private boolean misfit = false ;

   /**
    * This property identifies if this product is currently in the recycling
    * bin. If TRUE then the lastTouchedBy and lastTouched properties tell who
    * placed the product draft in the recycling bin and when.
    */
   private boolean recycled = false ;

   /**
    * If this Draft is cancelled for some reason, than this indicator
    * ties to the PRODUCTDRAFT.TYPECANCEL table telling a reason
    * why this product was cancelled. This property should not be
    * set unless this draft is going to be cancelled. 
    */
   private int typeCancelInd = 0 ;

   /**
     * This is the original product draft indicator for the draft that
     * published the active product that this draft represents
     **/
   private int originalProductInd = 0 ;

   /**
     * This is the OPIC/M Primary Key
     **/
   private String opicmKey = null ;

   /**
     * This boolean property tells the ProductDraft if the OPIC/M Key was updated
     **/
   protected boolean opicmKeyUpdated = false ;


   /**
     * This is the person who created this draft
     **/
   private String draftCreatedBy = null ;

   /**
     * This is the time the draft was created
     **/
   private String draftCreatedOn = null ;

   /**
    * This method will return the work required for this draft. The 
    * Work Required Indicator is defined in the TIGRIS.TYPEWORK
    * class and can be either EDIT, DELETE or CREATE. 
    */
   private int workRequired = TypeList.WORKREQ_CREATE ;


   /**
     *  This flag determines if this product is a template
     **/
   private boolean template = false ;


   /**
     *  This is the OPIC/M Description of this product
     **/
   private String opicmDescript = null ;

   /**
     *  This is the OPIC/M Brand of this product
     **/
   private TypeCategoryRec opicmBrand = null ;

   /**
     *  This is the OPIC/M Family of this product
     **/
   private TypeCategoryRec opicmFamily = null ;

   /**
     *  This is the OPIC/M Series of this product
     **/
   private TypeCategoryRec opicmSeries = null ;

   /**
     *  This is the OPIC/M Country list
     **/
   private Vector opicmCountries = null ;

   /**
     *  This is the OPIC/M Geo list
     **/
   private Vector opicmGeographies = null ;


   /**
     * This default constructor creates a blank ProductDraft suitable
     * for creating a brand new product in the Quest Database.
     **/
   public ProductDraft () {
      workRequired = TypeList.WORKREQ_CREATE ;
   }

   /**
     * This Constructor creates a blank ProductDraft that can be
     * read from the database.
     *
     * @param The Product Draft Indicator that already exists in the
     *        Quest 6.0 Database.
     **/
   public ProductDraft ( int productInd ) {
      super ( productInd ) ;
   }

   /**
     *  This constructor will create a brand new Product Draft
     * for a new product. The information for this new product
     * will be copied from the ProductDraft object passed in
     **/
   public ProductDraft ( ProductDraft draft ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, copy all of the information over                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      initializeProperties ( draft ) ;                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, adjust our indicators and status                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      existingProductInd = 0 ;
      template=false;
      setProductInd(0) ;
      updateRecStatus ( REC_NOCHANGE ) ;
      updateRecStatus ( REC_INSERT ) ;
      setCreatedOn ( new CDate ( CDate.DATE_DATE ).today() ) ;

   }

   /**
     * This method will return the current comments of this draft. Please
     * keep in mind that this may be different than the 'originalComments'
     * property.
     **/
   public ProductComment getComment() {
      return(comment);
   }

   /**
     * This method will set the current comments of this draft. Please keep
     * in mind that every time a draft is read in, the current comments of
     * the draft are placed in the 'originalComment' property. This property
     * starts blank for every read.
     **/
   public void setComment(String comment) {
      this.comment = new ProductComment ( getProductInd(), comment ) ;
      updateRecStatus ( REC_UPDATE );
   }

   /**
     * This method will set the current comments of this draft. Please keep
     * in mind that every time a draft is read in, the current comments of
     * the draft are placed in the 'originalComment' property. This property
     * starts blank for every read.
     **/
   public void setComment(ProductComment comment) {
      this.comment = comment ;
      updateRecStatus ( REC_UPDATE );
   }



   /**
     * This method will return the Existing Product Indicator that this
     * Product Draft was created for
     **/
   public int getExistingProductInd() {
      return(existingProductInd);
   }

   /**
     * This method will set the Existing Product Indicator that this Product Draft
     * was created for. If this indicator is zero, than it is assumed that this
     * draft was created for a new product that doesn't exist yet and the
     * workRequired property will be updated accordingly. If the Existing Product Indicator
     * passed in non zero, it is up to you to set the appropriate Work Required.
     *
     * @param existingProductInd The Existing Product Indicator
     **/
   protected void setExistingProductInd ( int existingProductInd ) {
      this.existingProductInd = existingProductInd; 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the existing indicator is 0, make our work required  */
      /* a Create                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( existingProductInd == 0 ) {
         workRequired = TypeList.WORKREQ_CREATE ;
      }

      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     * This method will return the start date for this product Draft
     */
   public String getStartDate() {
      return(startDate);
   }

   /**
     * This method will set the start date for this product draft. This
     * start date cannot be null.
     */
   protected void setStartDate ( String startDate ) throws IllegalArgumentException  {
      if ( startDate != null ) {
         this.startDate = startDate ;
         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Start Date cannot be NULL" ) ;
   }

   /**
     * This method will return the stop date for this product draft.
     */
   public String getStopDate() {
      return(stopDate);
   }

   /**
     * This method will set the stop date for this product draft. Once
     * this product draft has a stop date in the Quest database, the draft
     * is considered closed.
     */
   public void setStopDate ( String stopDate ) {
      this.stopDate = stopDate ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     * This method will return TRUE if this product is current a misfit
     * product. A Misfit product draft is created when ProdGen cannot find
     * a Country/Geo or Brand/Family mapping in Quest.
     */
   public boolean getMisfit() {
      return(misfit);
   }

   /**
     * This method will set the misfit flag
     */
   public void setMisfit ( boolean misfit ) {
      this.misfit = misfit ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     * This method will return the Recycled property. If TRUE then
     * this product draft should be considered in the recycling bin. No changes
     * should be made to it.
     */
   public boolean getRecycled() {
      return(recycled);
   }

   /**
     * This method will set the recycled field for this product draft
     **/
   public void setRecycled(boolean recycled) {
      this.recycled = recycled;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     * This method will return the typeCancelInd for this product draft
     **/
   public int getTypeCancelInd() {
      return(typeCancelInd);
   }

   /**
     * This method will set the typeCancelInd for this productDraft.
     **/
   public void setTypeCancelInd(int typeCancelInd) {
      this.typeCancelInd = typeCancelInd;
      updateRecStatus ( REC_UPDATE ) ;
   }



   /**
     * This method will save all information that is unsaved to the database.
     * This method will return 0 if no error occurred and the SQLCode of
     * an error if an error did occur
     **/
   public int writeToDatabase () {
      String sql = toSQL () ;
      int    rc  = 0 ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have at least 1 SQL, run it                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( sql != null ) && ( sql.length() > 0 ) ) {
         rc = SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, sql ) ;

         if ( rc == 0 ) {
            setLastTouchedOn ( ProductSQL.getDBTimestampFromDatabase() ) ;
            updateRecStatus ( REC_NOCHANGE ) ;
            super.resetUpdateVariables() ;
         }
      }

      return( rc ) ;
   }

   /**
     * This method will publish this draft. This means that this draft will
     * be closed and all of this drafts information will be written over what
     * is currently in production.
     * This method will return 0 if no error occurred and the SQLCode of
     * an error if an error did occur
     **/
   public int publish () {
      int rc = 0 ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the stop date since this is publishing              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setStopDate ( new CDate ( CDate.DATE_STAMP ).today() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save this draft to the database                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rc = writeToDatabase() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the draft was saved Ok then continue                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         Product product = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is a work required for new Product, create      */
         /* a new Product object now                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( workRequired == TypeList.WORKREQ_CREATE )
            product = new Product() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, read in the product                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            product = new Product ( existingProductInd ) ;

            rc = product.readFromDatabase() ;

            switch ( rc ) {
               case 0 :
                  break ;

               case 100 :
                  LogSystem.log ( 1, "While publishing a ProductDraft with work required for EDIT, could not find existing product. Creating new product with existing Indicator of "+existingProductInd ) ;
                  product=null ;
                  break;

               default:
                  LogSystem.log ( 1, "While publishing a ProductDraft with work required for EDIT, could not read Existing Product "+existingProductInd ) ;
                  LogSystem.log ( 1, "Product.readFromDatabase returned a code of "+rc ) ;
                  product=null ;
                  break ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we still have a  product, populate and save it       */                                 
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( product != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, populate the Product object with the new           */
            /* data                                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            product.initializeProperties ( this ) ;                        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the Machine/Model accordingly                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            product.createCategoryFromDraft ( this ) ;                                       


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the work required is for Delete, make sure we set    */
            /* the publish flag to FALSE                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getWorkRequired() == TypeList.WORKREQ_DELETE ) {
               product.updateRecStatus ( REC_NOCHANGE ) ;
               product.updateRecStatus ( REC_DELETE ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save the new product                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rc = product.writeToDatabase() ;
         }

      } /* if the product draft was saved  */

      return( 0 ) ;
   }

   /**
     * This method will read in all information for this product draft.
     * This method will return 0 if no error occurred and the SQLCode of
     * an error if an error did occur
     */
   public int readFromDatabase () {
      String    rtnName   = "ProductDraft.readFromDatabase" ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      int       rc        = 0 ;

      if ( sqlMethod != null ) {
         try {
            CallableStatement stmt  = sqlMethod.createStoredProc ( "QUEST.ProDraftReader", 3 ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the call                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.setInt               ( 1, getProductInd() ) ;
            stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
            stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            LogSystem.log ( 1, "Reading Product Draft:"+getProductInd() ) ;
            stmt.execute() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the StoredProcRec from the Lob                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Throw an error if need be                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( retRec != null ) && ( retRec.error ) ) {
               LogSystem.beginLogSection ( "ProductDraftReader Returned Error" ) ;
               LogSystem.log ( 1, "sqlCode:   "+retRec.sqlCode ) ;
               LogSystem.log ( 1, "userError: "+retRec.userError ) ;
               LogSystem.log ( 1, "errorStr:  "+retRec.errorStr ) ;
               LogSystem.log ( 1, "error:     "+retRec.error ) ;
               try {
                  rc = new Integer ( retRec.sqlCode ).intValue() ;
               } catch ( Exception e ) {
                  LogSystem.log ( 1, "Could not translate retRec.sqlCode to integer" ) ;
                  rc=-1;
               }
               LogSystem.endLogSection () ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, continue                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get our return object                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               initializeProperties ( (Product) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ) ;
               forceRecStatus ( REC_NOCHANGE ) ;
               rc = 0 ;

            } /* Else if we have no error  */

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            rc = -1 ;
         }
      } /* if sqlMethod is not null  */

      sqlMethod.close() ;

      return( rc ) ;
   }

   /**
     * This method will cancel the current draft. No information will be
     * saved to this product draft's existing product.
     * This method will return 0 if no error occurred and the SQLCode of
     * an error if an error did occur
     **/
   public int cancel () {
      return( 0 ) ;
   }

   /**
     * This method will attempt to create a ProductDraft object from the
     * Product object passed in. The Product object passed in MUST already
     * have been read in
     **/
   public static CreateProductDraft createProductDraft ( Product product ) {
      CreateProductDraft returnRec   = null ;
      ProductDraft       draft       = null ;
      int                newDraftInd = draftExists ( product.getProductInd() ) ;                                           


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we receive an error, report it. If we receive    */
      /* no prod indicator for the draft, create a new draft and   */
      /* if we receive a prod ind for the draft, read the draft in */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( newDraftInd ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   0 Means that no draft was found, create a new one       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            draft = new ProductDraft() ;
            draft.setWorkRequired ( TypeList.WORKREQ_EDIT ) ;
            draft.initializeProperties ( product ) ;
            draft.createCategoryFromExisting() ;
            returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_CREATED,
                                                 0,
                                                 draft ) ;
            break;                          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   -1 means an error, report the error                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case -1 :
            LogSystem.log ( 1, "ProductDraft.createDraft ( "+product.getProductInd()+" ): Could not find Existing Product." ) ;
            returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_NONE,
                                                 0,
                                                 null ) ;
            draft = null ;
            break ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, anything else that is returned is a draft, read it  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         default :
            int draftRC = 0 ;
            draft = new ProductDraft ( newDraftInd ) ;

            draftRC = draft.readFromDatabase() ;
            switch ( draftRC ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   0 Means everything read fine, adjust the REC_STATUS     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               case 0 :
                  draft.forceRecStatus ( REC_NOCHANGE ) ;
                  if ( draft.getRecycled() )
                     returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_RECYCLED,
                                                          0,
                                                          draft ) ;
                  else
                     returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_ALREADY_EXISTS,
                                                          0,
                                                          draft ) ;
                  break ;                                 

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   This should never happen, but just in case              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               case 100 :
                  LogSystem.log ( 1, "ProductDraft.createDraft ( "+product.getProductInd()+" ): Wierd, found draft "+newDraftInd+" for existing product, but can't read it! rc 100" ) ;
                  draft = null ;
                  returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_NONE,
                                                       0,
                                                       null ) ;
                  break ;                        


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Anything else, report as an error                       */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               default:
                  LogSystem.log ( 1, "ProductDraft.createDraft ( "+product.getProductInd()+" ): Read Draft Product "+newDraftInd+". rc:"+draftRC ) ;
                  returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_SQL_ERROR,
                                                       draftRC,
                                                       null ) ;
                  draft = null ;
                  break ;

            }
            break ;

      } /* switch newDraftInd  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   To catch all errors                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         LogSystem.log ( 1, "ProductDraft.createDraft ( "+product.getProductInd()+" ) could not figure out what to do" ) ;
         returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_NONE,
                                              0,
                                              null ) ;
      }

      return( returnRec ) ;
   }



   /**
     * This method will attempt to create a ProductDraft object from the
     * Existing Product Indicator passed in
     *
     * @param productInd The EXISTING Product Indicator located in the PRODRAFT.PRODUCTS table
     **/
   public static CreateProductDraft createProductDraft ( int productInd ) {
      CreateProductDraft returnRec  = null ;
      Product            product    = new Product ( productInd ) ;
      int                rc         = product.readFromDatabase() ;

      switch ( rc ) {
         case 0 :
            returnRec = createProductDraft ( product ) ;
            break ;

         case 100 :
            //ProductDraft draft = new ProductDraft () ;

            //draft.setExistingProductInd ( productInd ) ;
            //draft.setWorkRequired ( TypeList.WORKREQ_EDIT ) ;
            returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_NO_ACTIVE_PRODUCT,
                                                 100,
                                                 null ) ;
            break ;

         default:
            returnRec = new CreateProductDraft ( CreateProductDraft.DRAFT_SQL_ERROR,
                                                 rc,
                                                 null ) ;
            break ;
      }


      return( returnRec ) ;
   }

   /**
     * This method will attempt to create a product draft from the
     * Machine/Model combination passed in
     *
     * @param family The family of the product
     * @param machine The machine of the product
     * @param model The model of the product
     *
     * @return CreateProductDraft object
     **/
   public static CreateProductDraft createProductDraft ( TypeCategoryRec family, String machine, String model ) throws Exception {
      TypeCategoryRec machineRec = TypeCategory.getCategoryFromDescriptionUnderParent ( machine, TypeCategory.LEVEL_MACHINE, family.getInd() ) ;

      /*-------------------------------------------------------------------------*/
      /*                                                                         */
      /*  If we could not find the active machine, look for the machine in draft */
      /*                                                                         */
      /*-------------------------------------------------------------------------*/
      if ( machineRec == null )
         machineRec = TypeCategory.getDraftCategoryFromDescriptionUnderParent ( machine, TypeCategory.LEVEL_MACHINE, family.getInd() ) ;

      /*-------------------------------*/
      /*                               */
      /*  If machine is good, continue */
      /*                               */
      /*-------------------------------*/
      if ( machineRec != null ) {
         TypeCategoryRec modelRec   = machineRec.getIsDraft() ? 
                                      TypeCategory.getDraftCategoryFromDescription ( model, machineRec.getInd() ) :
                                      TypeCategory.getCategoryFromDescription ( model, machineRec.getInd() ) ;

         /*-----------------------------*/
         /*                             */
         /*  If model is good, continue */
         /*                             */
         /*-----------------------------*/
         if ( modelRec != null ) {
            int productInd = modelRec.getIsDraft() ? 
                             ProductSQL.getDraftProductIndFromModel ( modelRec ) : 
                             ProductSQL.getProductIndFromModel ( modelRec ) ;

            if ( productInd != 0 )
               return( createProductDraft ( productInd ) ) ;
            else
               throw new Exception ( "Cannot create product draft because model doesn't exist in PRODUCT.CATEGORY.\nModel:"+model+"\nMachine:"+machine+"\nFamily:"+family.toString() ) ;


         } else
            throw new Exception ( "Cannot create product draft for model that does not exist.\nModel:"+model+"\nMachine:"+machine+"\nFamily:"+family.toString() ) ;

      } else
         throw new Exception ( "Cannot create product draft for machine that does not exist.\nModel:"+model+"\nMachine:"+machine+"\nFamily:"+family.toString() ) ;
   }

   /**
     * This method will attempt to create a ProductDraft object from the
     * part number passed in
     *
     * @param partNumber The part Number to create a Draft for
     *
     * @return CreateProductDraft object
     **/
   public static CreateProductDraft createProductDraft ( String partNumber ) throws Exception {
      TypeCategoryRec partNumberRec = TypeCategory.getCategoryFromDescription ( partNumber, TypeCategory.LEVEL_PARTNUMBER ) ;

      if ( partNumberRec != null ) {
         int productInd = ProductSQL.getProductIndFromModel ( partNumberRec ) ;

         if ( productInd != 0 )
            return( createProductDraft ( productInd ) ) ;
         else
            throw new Exception ( "Cannot create product draft because part number doesn't exist in PRODUCT.CATEGORY.\nPart Number:"+partNumber ) ;

      } else
         throw new Exception ( "Cannot create draft from part number because part number doesn't exist.\nPart Number:"+partNumber ) ;

   }


   /**
     * This method will immeditaly check out this product record. Please
     * keep in mind that this method automatically overwrites the current
     * checked out state in the database.
     **/
   public void checkOut ( String userId ) throws Exception {
      if ( getProductInd() != 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "checkOut ( "+userId+" )", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            setCheckedOutBy ( userId ) ;
            if ( userId != null )
               stmt.executeUpdate ( "UPDATE PRODRAFT.PRODUCT SET CHECKEDOUTBY = '"+userId+"' where productInd = "+getProductInd() ) ;
            else
               stmt.executeUpdate ( "UPDATE PRODRAFT.PRODUCT SET CHECKEDOUTBY = NULL where productInd = "+getProductInd() ) ;

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            throw ( e ) ;
         }

         sqlMethod.close() ;
      }
   }
   /**
     * This method will return the work required for this draft
     **/
   public int getWorkRequired() {
      return(workRequired);
   }


   /**
     *  This method will set the workRequired for this draft. The
     * work required indicator passed in MUST be in the TIGRIS.TYPEWORKREQUIRED table
     *
     * @param workInd The indicator for the work required.
     **/
   public void setWorkRequired ( int workInd ) {
      this.workRequired = workInd ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /*------------------------------------------------------------*/
   /* draftExists
    */
   /**
    *    This static method will do a simple look up
    *  to find our if a draft exists for the existing
    *  product indicator passed in
    *
    *  @param existingDocInd The existing product indicator to look for
    *  @param None
    *  @param None
    *
    *  @return The indicator of the draft if it exists, -1 if an error, 0 if no draft exists
    *//*---------------------------------------------------------*/
   public static int draftExists ( int existingProductInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "draftExists ("+existingProductInd+")", 5 ) ;
      int       returnInd = -1 ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND "+
                                              "FROM PRODRAFT.PRODUCT "+
                                              "WHERE STOPDATE IS NULL AND "+
                                              "      EXISTINGPRODUCTIND = "+existingProductInd+" "+
                                              "FOR FETCH ONLY" ) ;

         if ( rs.next() )
            returnInd = rs.getInt(1) ;
         else
            returnInd = 0 ;

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      return( returnInd ) ;
   }



   /**
     *  This method will initialize this draft's properties to the Product
     * or ProductDraft record passed in
     *
     * @param product The Product or ProductDraft object to initialize with
     **/
   public void initializeProperties ( Product product ) {
      super.initializeProperties ( product ) ;

      if ( product instanceof ProductDraft ) {
         ProductDraft draft = (ProductDraft) product ;

         super.setProductInd ( draft.getProductInd() ) ;
         existingProductInd   = draft.existingProductInd ;
         comment              = draft.comment ;
         startDate            = draft.startDate ;
         stopDate             = draft.stopDate ;
         misfit               = draft.misfit ;
         recycled             = draft.recycled ;
         typeCancelInd        = draft.typeCancelInd ;
         workRequired         = draft.workRequired ;
         originalComments     = draft.originalComments ;
         originalProductInd   = draft.originalProductInd ;
         opicmDescript        = draft.opicmDescript ;
         opicmBrand           = draft.opicmBrand ;
         opicmFamily          = draft.opicmFamily ;
         opicmSeries          = draft.opicmSeries ;
         opicmKey             = draft.opicmKey ;
         opicmKeyUpdated      = draft.opicmKeyUpdated ;
         draftCreatedOn       = draft.draftCreatedOn ;
         draftCreatedBy       = draft.draftCreatedBy ;

         if ( draft.opicmCountries != null )
            opicmCountries         = (Vector)draft.opicmCountries.clone() ;

         if ( draft.opicmGeographies != null )
            opicmGeographies       = (Vector)draft.opicmGeographies.clone() ;

      } else {
         TypeCategoryRec currentProject = product.getProject() ;

         existingProductInd = product.getProductInd() ;


         /*---------------------------------------------------------*/
         /*                                                         */
         /*  If our project is not in draft form, get the draft one */
         /* now                                                     */
         /*                                                         */
         /*---------------------------------------------------------*/
         if ( product.getProject() != null ) {
            if ( !product.getProject().getIsDraft() )
               setProject ( TypeCategory.getDraftCategoryFromCategory ( product.getProject() ) ) ;
         }

         /*--------------------------------------------------*/
         /*                                                  */
         /*  If our machine is not in draft form, get it now */
         /*                                                  */
         /*--------------------------------------------------*/
         if ( product.getMachine() != null ) {
            if ( !product.getMachine().getIsDraft() ) {
               TypeCategoryRec machine = TypeCategory.getDraftCategoryFromCategory ( product.getMachine() ) ;

               machine.setParentCatInd ( product.getProject().getInd() ) ;
               setMachine ( machine ) ;
            }
         }

         /*--------------------------------------------------*/
         /*                                                  */
         /*  If our SubCat  is not in draft form, get it now */
         /*                                                  */
         /*--------------------------------------------------*/
         if ( product.getSubCategory() != null ) {
            if ( !product.getSubCategory().getIsDraft() ) {
               TypeCategoryRec subCategory = TypeCategory.getDraftCategoryFromCategory ( product.getSubCategory() ) ;

               subCategory.setParentCatInd ( product.getProject().getInd() ) ;
               setSubCategory ( subCategory ) ;
            }
         }


         /*--------------------------------------------------*/
         /*                                                  */
         /*  If our model is not in draft form, get it now   */
         /*                                                  */
         /*--------------------------------------------------*/
         if ( product.getModel() != null ) {
            if ( !product.getModel().getIsDraft() ) {
               TypeCategoryRec model   = TypeCategory.getDraftCategoryFromCategory ( product.getModel() ) ;
               TypeCategoryRec machine = (getProductType()==TypeList.PRODUCTTYPE_OPTION) ? product.getSubCategory() : product.getMachine() ;

               model.setParentCatInd ( machine.getInd() ) ;
               setModel ( model ) ;
            }
         }
      } /* else  */
   }



   /**
     * This method will return the SQL needed to save this entire product to the database,
     * and it includes the SQL to save information for Locale, Category and XRef objects
     *
     * @return String. The SQL returned is delimited by the SqlRunner.END_DELIM.
     */
   public String toSQL () {
      String sql = "" ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Generate the keys if needed                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( generateKeys() ) {
         String commentStr = originalComments ;

         if ( commentStr == null )
            commentStr="" ;

         if ( comment != null )
            commentStr+="\n"+comment.toString() ;
         else
            commentStr+="\nN/A" ;


         commentStr = Text.cleanDBString ( commentStr ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, generate the SQL needed to save this product to  */
         /* the PRODRAFT.PRODUCT table                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getRecStatus() ) {
            case REC_DELETE :
               sql = "UPDATE PRODRAFT.PRODUCT "+
                     "SET RECYCLED = 'Y',"+
                     "    LASTCHANGEDBY = '"+getLastTouchedBy()+"', "+
                     "    LASTCHANGEDON = CURRENT TIMESTAMP "+
                     "WHERE PRODUCTIND = "+getProductInd() ;
               break ;

            case REC_INSERT :

               sql = "INSERT INTO PRODRAFT.PRODUCT "+
                     "( PRODUCTIND,     PRODNUM,         DESCRIPTION,        BASEMODEL, TYPECUSTOMERIND, "+
                     "  TYPEPRODUCTIND, TYPEOFFERINGIND, TYPEWARRANTYIND,    INPUT,     SOURCE, "+
                     "  PUBLISH,        CHECKEDOUTBY,    CREATEDON,          CREATEDBY, "+
                     "  LASTTOUCHEDON,  LASTTOUCHEDBY,   EXISTINGPRODUCTIND, COMMENTS,  MISFIT, "+
                     "  TYPECANCELIND,  STARTDATE,       STOPDATE,           RECYCLED,  WORKIND, TEMPLATE, IMPORTOPICM ) values ( "+
                     " "+getProductInd()+", "+
                     " '"+Text.cleanDBString ( getProdNum() )+"', "+
                     " '"+Text.cleanDBString ( getDescription() )+"', "+
                     " "+Text.cleanNullableDBString ( getBaseModel() )+", "+
                     " "+getCustomer()+", "+
                     " "+getProductType()+", "+
                     " "+getOffering()+", "+
                     " "+getWarranty()+", "+
                     " '"+Text.cleanDBString ( getInput() ) +"', "+
                     " '"+Text.cleanDBString ( getSource() ) +"', "+
                     " 'Y', "+
                     " NULL, "+
                     " CURRENT TIMESTAMP, "+  //Created On
                     " '"+getLastTouchedBy()+"', "+ //Created By
                     " CURRENT TIMESTAMP, "+        //Last Touched On
                     " '"+getLastTouchedBy()+"', "+
                     " "+getExistingProductInd()+", "+
                     " '"+commentStr+"', "+
                     " '"+Text.getYN ( misfit )+"', "+
                     "  "+getTypeCancelInd()+", "+
                     "  "+Text.cleanNullableDBString ( getStartDate() )+", "+
                     "  "+Text.cleanNullableDBString ( getStopDate() )+", "+
                     "  'N', "+
                     "  "+workRequired+", "+
                     "  '"+Text.getYN ( template)+"', "+
                     "  '"+(getOPICMImport() ? "Y" : "N")+"')" ;
               break ;

            case REC_UPDATE :
               sql = "UPDATE PRODRAFT.PRODUCT "+
                     "SET PRODNUM            = '"+Text.cleanDBString ( getProdNum() )+"', "+
                     "    DESCRIPTION        = '"+Text.cleanDBString ( getDescription() )+"', "+
                     "    BASEMODEL          = '"+Text.cleanDBString ( getBaseModel() )+"', "+
                     "    TYPECUSTOMERIND    = "+getCustomer()+", "+
                     "    TYPEPRODUCTIND     = "+getProductType()+", "+
                     "    TYPEOFFERINGIND    = "+getOffering()+", "+
                     "    TYPEWARRANTYIND    = "+getWarranty()+", "+
                     "    INPUT              = '"+Text.cleanDBString ( getInput() )+"', "+
                     "    SOURCE             = '"+Text.cleanDBString ( getSource() )+"', "+
                     "    PUBLISH            = '"+(getPublish().equals ( "Y" ) ? "Y" : "N")+"', "+
                     "    CHECKEDOUTBY       = '"+Text.cleanDBString ( getCheckedOutBy() )+"', "+
                     "    LASTTOUCHEDON      = CURRENT TIMESTAMP, "+
                     "    LASTTOUCHEDBY      = '"+Text.cleanDBString ( getLastTouchedBy() )+"', "+
                     "    EXISTINGPRODUCTIND = "+existingProductInd+", "+
                     "    COMMENTS           = '"+commentStr+"', "+
                     "    MISFIT             = '"+Text.getYN ( misfit )+"', "+
                     "    TYPECANCELIND      = "+getTypeCancelInd()+", "+
                     "    STARTDATE          = "+Text.cleanNullableDBString ( startDate )+", "+
                     "    STOPDATE           = "+Text.cleanNullableDBString ( stopDate ) +", "+
                     "    WORKIND            = "+workRequired+", "+
                     "    TEMPLATE           = '"+Text.getYN ( template )+"', "+
                     "    IMPORTOPICM        = '"+(getOPICMImport() ? "Y" : "N")+"' "+
                     "WHERE PRODUCTIND = "+getProductInd() ;
               break ;
         }

         if ( sql.length() > 0 )
            sql=sql+SqlRunner.END_DELIM ;


         /*------------------------------*/
         /*                              */
         /*  Generate the OPIC/M Key SQL */
         /*                              */
         /*------------------------------*/
         if ( opicmKeyUpdated )
            sql=sql+opicmKeyToSQL() ;
         else
            sql=sql+opicmDescriptToSQL() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, generate the SQL needed to save the dates          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+datesToSQL () ;                                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Generate the SQL needed to save the Cross Products      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+xRefToSQL () ;                                 

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Generate the SQL needed to save the Locale              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+localeToSQL () ;                                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Generate the SQL needed to save the categorization      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+categoryToSQL() ;

      } else {
         String error = "Could not generate a Product Indicator. No INSERT can occur. Please check logs for more info" ;

         LogSystem.log ( 1, error ) ;
      }


      return( sql ) ;
   }


   /**
     *  This method will generate the primary keys for this class if the primary keys
     * have not yet been assigned. It will generate the Existing Product Indicator and the
     * product Indicator
     *
     * @return TRUE if generated
     **/
   protected boolean generateKeys () {

      if ( getProductIndNow() == 0 ) {
         setProductInd ( Counter.getCounter ( LogSystem.getInstance(), "PRODRAFT" ) ) ;
      }

      if ( existingProductInd == 0 ) {
         existingProductInd = Counter.getCounter ( LogSystem.getInstance(), "PRODUCT" ) ;
      }

      return( getProductIndNow() != 0 )  ;
   }


   /**
     *  This method will return the SQL needed to save all dates for this product.
     *  Each SQL String returned will have the SqlRunner.END_DELIM
     *  appended to it so that the SQL may be passed to the SqlRunner stored procedure
     *
     **/
   protected String datesToSQL () {
      String sql   = "" ;                              
      Vector dates = getDates() ;
      int         i    = 0 ;
      int         size = dates.size() ;
      ProductDate date = null ;

      sql="DELETE FROM PRODRAFT.DATES WHERE PRODUCTIND = "+getProductInd()+SqlRunner.END_DELIM ;
      while ( i < size ) {

         date = ((ProductDate)dates.elementAt(i)) ;
         date.setProductInd ( getProductInd() ) ;
         sql+=date.toDraftSQL () ;

         i++ ;
      }

      return( sql ) ;
   }


   /**
     *  This method will return the SQL needed to save all XRef objects for
     * this product. Each SQL String returned will have the SqlRunner.END_DELIM
     *  appended to it so that the SQL may be passed to the SqlRunner stored procedure
     *
     * @returns String
     **/
   protected String xRefToSQL () {
      String sql  = "" ;                             
      Vector xRef = getXRef() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only generate SQL if there are changes                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( POMUtil.vectorContainsChanges ( xRef ) ) {
         int  i       = 0 ;
         int  size    = xRef.size() ;
         XRef xRefRec = null ;

         while ( i < size ) {

            xRefRec = (XRef)xRef.elementAt(i) ;

            if ( xRefRec.modified() )
               sql+=xRefRec.toDraftSQL () ;

            i++ ;
         }
      }

      return( sql ) ;
   }


   /**
     *  This method will return the SQL needed to save all locale information (Country
     * and Geography) for this product. Each SQL String returned will have the SqlRunner.END_DELIM
     *  appended to it so that the SQL may be passed to the SqlRunner stored procedure
     *
     * @returns String
     **/
   protected String localeToSQL () {
      String    sql    = "" ;                                       
      LocaleRec locale = getLocale() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only generate the SQL if the locale was modified        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( locale != null ) && ( locale.modified() ) ) {
         Vector countries = locale.getCountryList() ;
         Vector geos      = locale.getGeoListContainingAll() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, we need to delete all dates out of the table,    */
         /* we then will reinsert them                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql = "DELETE FROM PRODRAFT.GEO WHERE PRODUCTIND = "+getProductInd()+SqlRunner.END_DELIM ;
         sql+= "DELETE FROM PRODRAFT.COUNTRY WHERE PRODUCTIND = "+getProductInd()+SqlRunner.END_DELIM ;                                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this locale is representing WORLD-WIDE then we only  */
         /* need to do 1 insert statement, otherwise we must process  */
         /* each country and geography                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( locale.isWorldWide() ) {
            sql+="INSERT INTO PRODRAFT.GEO ( PRODUCTIND, GEOIND ) "+
                 "values ( "+getProductInd()+", "+TypeList.GEO_WORLD_WIDE+" )"+SqlRunner.END_DELIM ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, it wasn't world wide so we must save everything     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, save the countries                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countries != null ) && ( countries.size() > 0 ) ) {
               int                i       = 0 ;
               int                size    = countries.size() ;
               TypeCountryCodeRec country = null ;

               while ( i < size ) {
                  country = (TypeCountryCodeRec) countries.elementAt(i) ;
                  sql+="INSERT INTO PRODRAFT.COUNTRY ( PRODUCTIND, COUNTRYCODEIND ) "+
                       "values ( "+getProductInd()+", "+country.getInd()+" )"+SqlRunner.END_DELIM ;

                  i++ ;
               } /* while  */


            } /* if  */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, save the geographies                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( geos != null ) && ( geos.size() > 0 ) ) {
               int          i    = 0 ;
               int          size = geos.size() ;
               TypeGeoRec   geo  = null ;

               while ( i < size ) {
                  geo = (TypeGeoRec) geos.elementAt(i) ;
                  sql+="INSERT INTO PRODRAFT.GEO ( PRODUCTIND, GEOIND ) "+
                       "values ( "+getProductInd()+", "+geo.getInd()+" )"+SqlRunner.END_DELIM ;

                  i++ ;
               } /* while  */


            } /* if  */
         } /* else, not world wide  */
      } /* if modified  */

      return( sql ) ;
   }


   /**
     *  This method will save all of our category information. After the information is saved,
     * this method will issue a delete of all Models, Machines and Part Numbers that are no longer referenced
     * in the PRODRAFT.CATEGORY and TIGRIS.CATEGORY tables. This will clean up all models, machines and 
     **/
   protected String categoryToSQL () {
      String sql = "" ;                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the brand was updated, change it                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandUpdated ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, delete the current brand if the product status   */
         /* is not INSERT. We will not have an entry in the database  */
         /* if the rec status is INSERT.                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getRecStatus() != REC_INSERT )
            sql+= "DELETE FROM PRODRAFT.CATEGORY "+
                  "WHERE PRODUCTIND = "+getProductInd()+" AND "+
                  "      TYPECATIND = (select b.typecatind "+
                  "                    from product.category a, "+
                  "                         product.typecategory b "+
                  "                    where a.productInd      = "+getProductInd()+" and "+
                  "                          a.typecatind    = b.typecatind and "+
                  "                          b.categorylevel = 'brand')"+SqlRunner.END_DELIM ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create our new entry                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql+="INSERT INTO PRODRAFT.CATEGORY "+
              "(PRODUCTIND, TYPECATIND) values ("+
              ""+getProductInd()+", "+
              ""+getBrand().getInd()+" )"+SqlRunner.END_DELIM ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the family was updated, change it                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( familyUpdated ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, delete the current family if the product status  */
         /* is not INSERT. We will not have an entry in the database  */
         /* if the rec status is INSERT.                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getRecStatus() != REC_INSERT )
            sql+= "DELETE FROM PRODRAFT.CATEGORY "+
                  "WHERE PRODUCTIND = "+getProductInd()+" AND "+
                  "      TYPECATIND = (select b.typecatind "+
                  "                    from product.category a, "+
                  "                         product.typecategory b "+
                  "                    where a.productInd      = "+getProductInd()+" and "+
                  "                          a.typecatind    = b.typecatind and "+
                  "                          b.categorylevel = 'family')"+SqlRunner.END_DELIM ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create our new entry                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql+="INSERT INTO PRODRAFT.CATEGORY "+
              "(PRODUCTIND, TYPECATIND) values ("+
              ""+getProductInd()+", "+
              ""+getFamily().getInd()+" )"+SqlRunner.END_DELIM ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the project was updated, change it                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( projectUpdated ) {
         TypeCategoryRec project = getProject() ; // This should create a new Project if it was null

         if ( project != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the SQL to save the machine type category           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sql+=project.toDraftSQL() ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, delete the current brand if the product status   */
            /* is not INSERT. We will not have an entry in the database  */
            /* if the rec status is INSERT.                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getRecStatus() != REC_INSERT )
               sql+= "DELETE FROM PRODRAFT.CATEGORY "+
                     "WHERE PRODUCTIND = "+getProductInd()+" AND "+
                     "      TYPECATIND = (select b.typecatind "+
                     "                    from prodraft.category a, "+
                     "                         prodraft.typecategory b "+
                     "                    where A.productInd      = "+getProductInd()+" and "+
                     "                          a.typecatind    = b.typecatind and "+
                     "                          b.categorylevel = '"+TypeCategory.LEVEL_PROJECT+"')"+SqlRunner.END_DELIM ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, create our new entry                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sql+="INSERT INTO PRODRAFT.CATEGORY "+
                 "(PRODUCTIND, TYPECATIND) values ("+
                 ""+getProductInd()+", "+
                 ""+project.getInd()+" )"+SqlRunner.END_DELIM ;                                           


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, delete all machines that are not referenced in     */
            /* PRODRAFT.CATEGORY, PRODRAFT.CATEGORY, TIGRIS.CATEGORY or   */
            /* QUEST.CATEGORY                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            //sql+="DELETE FROM PRODRAFT.TYPECATCHILD "+
            //     "WHERE TYPECATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY "+
            //     "                     WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_PROJECT+"' AND "+
            //     "                     TYPECATIND NOT IN ( SELECT TYPECATIND FROM PRODRAFT.CATEGORY) AND "+
            //     "                     TYPECATIND NOT IN ( SELECT TYPECATIND FROM TIGRIS.CATEGORY)) OR "+
            //     "      PARENTCATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY "+
            //     "                     WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_PROJECT+"' AND "+
            //     "                     TYPECATIND NOT IN ( SELECT TYPECATIND FROM PRODRAFT.CATEGORY) AND "+
            //     "                     TYPECATIND NOT IN ( SELECT TYPECATIND FROM TIGRIS.CATEGORY))

            //sql+="DELETE FROM PRODRAFT.TYPECATEGORY "+
            //     "WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_PROJECT+"' AND "+
            //     "TYPECATIND NOT IN ( SELECT TYPECATIND FROM PRODRAFT.CATEGORY) AND "+
            //     "TYPECATIND NOT IN ( SELECT TYPECATIND FROM TIGRIS.CATEGORY) "+SqlRunner.END_DELIM ;

         } /* if projectUpdated  */
      } /* if projectUpdateds  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the machine was updated, change it                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( machineUpdated ) {
         boolean isOption = ( getProductType() == TypeList.PRODUCTTYPE_OPTION ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the SQL to save the machine type category           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( isOption )
            sql+=getSubCategory().toDraftSQL() ;
         else
            sql+=getMachine().toDraftSQL() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, delete the current brand if the product status   */
         /* is not INSERT. We will not have an entry in the database  */
         /* if the rec status is INSERT.                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getRecStatus() != REC_INSERT )
            sql+= "DELETE FROM PRODRAFT.CATEGORY "+
                  "WHERE PRODUCTIND = "+getProductInd()+" AND "+
                  "      TYPECATIND = (select b.typecatind "+
                  "                    from prodraft.category a, "+
                  "                         prodraft.typecategory b "+
                  "                    where a.productInd      = "+getProductInd()+" and "+
                  "                          a.typecatind    = b.typecatind and "+
                  "                          b.categorylevel IN ('"+TypeCategory.LEVEL_MACHINE+"', '"+TypeCategory.LEVEL_SUBCATEGORY+"'))"+SqlRunner.END_DELIM ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create our new entry                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( isOption )
            sql+="INSERT INTO PRODRAFT.CATEGORY "+
                 "(PRODUCTIND, TYPECATIND) values ("+
                 ""+getProductInd()+", "+
                 ""+getSubCategory().getInd()+" )"+SqlRunner.END_DELIM ;
         else
            sql+="INSERT INTO PRODRAFT.CATEGORY "+
                 "(PRODUCTIND, TYPECATIND) values ("+
                 ""+getProductInd()+", "+
                 ""+getMachine().getInd()+" )"+SqlRunner.END_DELIM ;                                           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, delete all machines that are not referenced in     */
         /* PRODRAFT.CATEGORY, PRODRAFT.CATEGORY, TIGRIS.CATEGORY or  */
         /* QUEST.CATEGORY                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         // sql+="DELETE FROM PRODRAFT.TYPECATCHILD "+
         //      "WHERE TYPECATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY "+
         //      "WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_MACHINE+"' AND "+
         //      "TYPECATIND NOT IN ( SELECT TYPECATIND FROM PRODRAFT.CATEGORY) AND "+
         //      "TYPECATIND NOT IN ( SELECT TYPECATIND FROM TIGRIS.CATEGORY) "+SqlRunner.END_DELIM ;
         // sql+="DELETE FROM PRODRAFT.TYPECATEGORY "+
         //      "WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_MACHINE+"' AND "+
         //      "TYPECATIND NOT IN ( SELECT TYPECATIND FROM PRODRAFT.CATEGORY) AND "+
         //      "TYPECATIND NOT IN ( SELECT TYPECATIND FROM TIGRIS.CATEGORY) "+SqlRunner.END_DELIM ;


      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the part number was updated, change it               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( partNumUpdated ) {

         if ( getPartNumber() != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the SQL to update the Model information             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sql+=getPartNumber().toDraftSQL () ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, create our new entry only if we are inserting this */
            /* as a new product. If we are updating, we have already     */
            /* updated the Model description text in save for the        */
            /* TypeCategoryRec                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getRecStatus() == REC_INSERT )
               sql+="INSERT INTO PRODRAFT.CATEGORY "+
                    "(PRODUCTIND, TYPECATIND) values ("+
                    ""+getProductInd()+", "+
                    ""+getPartNumber().getInd()+" )"+SqlRunner.END_DELIM ;
         } else
            sql+="DELETE FROM PRODRAFT.CATEGORY WHERE PRODUCTIND = "+getProductInd()+" AND "+
                 "TYPECATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_PARTNUMBER+"')"+SqlRunner.END_DELIM ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the model was updated, change it                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( modelUpdated ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the SQL to update the Model information             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getModel() != null ) {
            sql+=getModel().toDraftSQL () ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, create our new entry only if we are inserting this */
            /* as a new product. If we are updating, we have already     */
            /* updated the Model description text in save for the        */
            /* TypeCategoryRec                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getRecStatus() == REC_INSERT )
               sql+="INSERT INTO PRODRAFT.CATEGORY "+
                    "(PRODUCTIND, TYPECATIND) values ("+
                    ""+getProductInd()+", "+
                    ""+getModel().getInd()+" )"+SqlRunner.END_DELIM ;
         } else
            sql+="DELETE FROM PRODRAFT.CATEGORY WHERE PRODUCTIND = "+getProductInd()+" AND "+
                 "TYPECATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_MODEL+"')"+SqlRunner.END_DELIM ;

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save the OPIC/M Brand and Family and Countries          */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, wipe out everything                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql+="DELETE FROM PRODRAFT.CATEGORY "+
           "WHERE TYPECATIND IN (SELECT TYPECATIND FROM PRODRAFT.TYPECATEGORY WHERE CATEGORYLEVEL IN ( "+
           "'"+TypeCategory.LEVEL_OPICM_BRAND+"', "+
           "'"+TypeCategory.LEVEL_OPICM_SERIES+"', "+
           "'"+TypeCategory.LEVEL_OPICM_FAMILY+"', "+
           "'"+TypeCategory.LEVEL_OPICM_GEOGRAPHY+"', "+
           "'"+TypeCategory.LEVEL_OPICM_COUNTRY+"' ) ) AND "+
           "PRODUCTIND = "+getProductInd()+SqlRunner.END_DELIM ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Brand                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( opicmBrand != null )
         sql+="INSERT INTO PRODRAFT.CATEGORY VALUES ( "+getProductInd()+", "+opicmBrand.getInd()+" )"+SqlRunner.END_DELIM ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Family                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( opicmFamily != null )
         sql+="INSERT INTO PRODRAFT.CATEGORY VALUES ( "+getProductInd()+", "+opicmFamily.getInd()+" )"+SqlRunner.END_DELIM ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Series                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( opicmSeries != null )
         sql+="INSERT INTO PRODRAFT.CATEGORY VALUES ( "+getProductInd()+", "+opicmSeries.getInd()+" )"+SqlRunner.END_DELIM ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   OPIC/M Geographies                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( opicmGeographies != null ) && ( opicmGeographies.size() > 0 ) ) {
         int i     = 0 ;
         int size  = opicmGeographies.size() ;

         while ( i < size ) {
            sql+="INSERT INTO PRODRAFT.CATEGORY "+
                 "VALUES ( "+getProductInd()+", "+
                 ""+((TypeCategoryRec)opicmGeographies.elementAt(i)).getInd()+" )"+SqlRunner.END_DELIM ;
            i++ ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   OPIC/M Countries                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( opicmCountries != null ) && ( opicmCountries.size() > 0 ) ) {
         int i     = 0 ;
         int size  = opicmCountries.size() ;

         while ( i < size ) {
            sql+="INSERT INTO PRODRAFT.CATEGORY "+
                 "VALUES ( "+getProductInd()+", "+
                 ""+((TypeCategoryRec)opicmCountries.elementAt(i)).getInd()+" )"+SqlRunner.END_DELIM ;
            i++ ;
         }
      }


      return( sql ) ;
   }


   /**
     *  This method will simply return a String containing human readable (not customer readable)
     * properties for this draft
     **/
   public String toOut () {
      LocaleRec localeRec      = getLocale() ;
      Vector    xRef           = getXRef() ;
      String    retStr         = "" ;
      Vector    commentHistory = getCommentHistory() ;

      retStr+="Product Ind:            "+getProductIndNow()+"\n" ;
      retStr+="Existing Product Ind:   "+existingProductInd+"\n" ;
      retStr+="Description:            "+getDescription()+"\n" ;
      if ( getBrand() != null )
         retStr+="Brand:                  "+getBrand().toString()+"\n" ;
      else
         retStr+="Brand:                  NONE\n" ;

      if ( getFamily() != null )
         retStr+="Family:                 "+getFamily().toString()+"\n" ;
      else
         retStr+="Family:                  NONE\n" ;                                    

      /*----------*/
      /*          */
      /*  Project */
      /*          */
      /*----------*/
      if ( getProject() != null ) {
         if ( getProject().toString().equals ( "" ) )
            retStr+="Project:                BLANK PROJECT\n" ;
         else
            retStr+="Project:                "+getProject().toString()+"\n" ;
      } else
         retStr+="Project:                NULL!\n" ;


      try {
         retStr+="Machine:                "+getMachine().toString()+"\n" ;
         retStr+="Model:                  "+getModel().toString()+"\n" ;
      } catch ( Exception e ) {
         retStr+="Machine:                NONE\n" ;
         retStr+="Model:                  NONE\n" ;
      }

      try {
         retStr+="Part Number:            "+getPartNumber().toString()+"\n" ;
      } catch ( Exception d1 ) {
         retStr+="Part Number:            NONE\n" ;
      }

      retStr+="publish:                "+getPublish()+"\n" ;
      retStr+="Checked Out By:         "+getCheckedOutBy()+"\n" ;
      retStr+="Source:                 "+getSource()+"\n" ;
      retStr+="Offering:               "+getOffering()+"\n" ;
      retStr+="Customer:               "+getCustomer()+"\n" ;
      retStr+="Warranty:               "+getWarranty()+"\n" ;

      if ( comment != null )
         retStr+="Comments:\n   "+comment.toString()+"\n\n" ;
      else
         retStr+="Comments:\n   NONE\n" ;

      retStr+="Start Date:             "+startDate+"\n" ;
      retStr+="Stop Date:              "+stopDate+"\n" ;
      retStr+="Recycled:               "+recycled+"\n" ;
      retStr+="Cancel Ind:             "+typeCancelInd+"\n" ;
      retStr+="Work Ind:               "+workRequired+"\n" ;



      if ( ( xRef != null ) && ( xRef.size() > 0 ) ) {
         int i    = 0 ;
         int size = xRef.size() ;

         while ( i < size ) {
            retStr+=((XRef)xRef.elementAt(i)).toOut()+"\n\n" ;
            i++ ;
         }

      } else
         retStr+="\nNo XRef Objects\n" ;

      if ( localeRec != null )
         retStr+=localeRec.toOut()+"\n\n" ;
      else
         retStr+="No locale information\n\n" ;


      if ( ( commentHistory != null ) && ( commentHistory.size() > 0 ) ) {
         int i    = 0 ;
         int size = commentHistory.size() ;


         while ( i < size ) {
            retStr+="   "+((ProductComment)commentHistory.elementAt(i)).toString()+"\n-----------------------------------------\n" ;
            i++ ;
         }
      }

      return( retStr ) ;
   }



   /**
     *  This method will return a blank ProductDraft object (even though the method is
     * coded to return a Product object) that matches the Product Indicator passed and 
     * is initialized with the Product Indicator passed. To see the data in the record
     * you must call 'ProductDraft.readFromDatabase'
     *
     * @param productInd The Product Indicator to look for.
     *
     * @return A blank ProductDraft object.
     **/
   public static Product find ( int productInd ) {
      ProductDraft productDraft = null ;
      SQLMethod    sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "ProductDraft.find ( "+productInd+" )", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND FROM PRODRAFT.PRODUCT WHERE PRODUCTIND = "+productInd+" FOR FETCH ONLY" ) ;

         if ( rs.next() )
            productDraft = new ProductDraft ( productInd ) ;

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      return( (Product)productDraft ) ;
   }


   /**
     *  This method will return a blank ProductDraft object (even though the method is
     * coded to return a Product object) that matches the categorization passed and 
     * is initialized with the Product Indicator of the product found. To see the data in the record
     * you must call 'ProductDraft.readFromDatabase'
     *
     * @param brand THe brand indicator of the Brand
     * @param familyInd The Family indicator of the family
     * @param machineInd The Machine indicator of the machine
     * @param modelInd The model indiator of the model
     *
     * @return A blank ProductDraft object.
     **/
   public static Product findFromCategory ( int brandInd, int familyInd, int machineInd, int modelInd ) {
      ProductDraft productDraft = null ;
      SQLMethod    sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "ProductDraft.find ( "+familyInd+", "+machineInd+", "+modelInd+" )", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND "+
                                              "FROM   PRODRAFT.PRODUCT "+
                                              "WHERE "+
                                              "STOPDATE IS NULL AND "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODRAFT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+familyInd+") AND "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODRAFT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+brandInd+") AND "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODRAFT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+machineInd+") AND "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODRAFT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+modelInd+") FOR FETCH ONLY " ) ;
         if ( rs.next() )
            productDraft = new ProductDraft ( rs.getInt(1) ) ;

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      return( (Product)productDraft ) ;
   }


   /**
     *  This method will return a blank ProductDraft object (even though the method is
     * coded to return a Product object) that matches the categorization passed and 
     * is initialized with the Product Indicator of the product found. To see the data in the record
     * you must call 'ProductDraft.readFromDatabase'
     *
     * @param brand The exact matching description of the brand
     * @param family The exact matching description of the family
     * @param machine The exact matching description of the machine
     * @param model The exact matching description of the model
     *
     * @return A blank ProductDraft object.
     **/
   public static Product findFromCategory ( String brand, String family, String machine, String model ) {
      TypeCategoryRec brandRec   = TypeCategory.getCategory ( brand, TypeCategory.LEVEL_BRAND ) ;
      TypeCategoryRec familyRec  = TypeCategory.getCategory ( family, TypeCategory.LEVEL_FAMILY ) ;
      TypeCategoryRec machineRec = TypeCategory.getDraftCategory ( machine, TypeCategory.LEVEL_MACHINE ) ;
      TypeCategoryRec modelRec   = TypeCategory.getDraftCategory ( model, TypeCategory.LEVEL_MODEL ) ;

      if ( ( familyRec  != null ) &&
           ( machineRec != null ) &&
           ( modelRec   != null ) )
         return( findFromCategory ( brandRec.getInd(), familyRec.getInd(), machineRec.getInd(), modelRec.getInd() ) ) ;
      return( null ) ;
   }


   /**
     *  This method will attempt to find the machine and model category in the PRODRAFT.TYPECATEGORY
     * table for the current Machine/Model. This is because when a draft is created from an Existing
     * Active Product, the Machine/Model might not be in the PRODRAFT.TYPECATEGORY table. If the machine
     * or model is not found then the machine/model will be created.
     **/
   public void createCategoryFromExisting () {
      TypeCategoryRec currentProject = getProject() ;
      TypeCategoryRec currentMachine = getMachine() ;
      TypeCategoryRec currentModel   = getModel() ;
      TypeCategoryRec currentSubCat  = getSubCategory() ;
      TypeCategoryRec currentPartNum = getPartNumber() ;
      TypeCategoryRec draftProject   = null ;
      TypeCategoryRec draftMachine   = null ;
      TypeCategoryRec draftModel     = null ;                      
      TypeCategoryRec draftSubCat    = null ;
      TypeCategoryRec draftPartNum   = null ;                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, create the draft project                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( currentProject != null ) {
         draftProject = TypeCategory.getDraftCategoryFromDescription ( currentProject.toString(), 
                                                                       TypeCategory.LEVEL_PROJECT, 
                                                                       getFamily().getInd() ) ;

         /* If we did not find one, create one now  */
         if ( draftProject == null ) {
            draftProject = TypeCategory.createDraftTypeCategory ( getFamily().getInd(),
                                                                  currentProject.toString(),
                                                                  TypeCategory.LEVEL_PROJECT,
                                                                  currentProject.getOrderNum(),
                                                                  currentProject.getChildSort(),
                                                                  currentProject.getOwner() ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We can only continue if we made a Draft Project         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draftProject != null ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, if we have a machine, process it                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( currentMachine != null ) {
            draftMachine = TypeCategory.getDraftCategoryFromDescription ( currentMachine.toString(), 
                                                                          TypeCategory.LEVEL_MACHINE, 
                                                                          draftProject.getInd() ) ;

            /* If we did not find one, create one now  */
            if ( draftMachine == null ) {
               draftMachine = TypeCategory.createDraftTypeCategory ( draftProject.getInd(),
                                                                     currentMachine.toString(),
                                                                     TypeCategory.LEVEL_MACHINE,
                                                                     currentMachine.getOrderNum(),
                                                                     currentMachine.getChildSort(),
                                                                     currentMachine.getOwner() ) ;
            }
         }


         if ( draftMachine != null ) {
            /* Ok, now do the model  */
            if ( currentModel != null ) {
               draftModel = TypeCategory.getDraftCategoryFromDescription ( currentModel.toString(), TypeCategory.LEVEL_MODEL, draftMachine.getInd() ) ;

               /* If we did not find one, create one now  */
               if ( draftModel == null ) {
                  draftModel = TypeCategory.createDraftTypeCategory ( draftMachine.getInd(),
                                                                      currentModel.toString(),
                                                                      TypeCategory.LEVEL_MODEL,
                                                                      currentModel.getOrderNum(),
                                                                      currentModel.getChildSort(),
                                                                      currentModel.getOwner() ) ;
               }
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, do the SubCategory and Part Number                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( currentSubCat != null ) {
            draftSubCat = TypeCategory.getDraftCategoryFromDescription ( currentSubCat.toString(), 
                                                                         TypeCategory.LEVEL_SUBCATEGORY, 
                                                                         draftProject.getInd() ) ;

            /* If we did not find one, create one now  */
            if ( draftSubCat == null ) {
               draftSubCat = TypeCategory.createDraftTypeCategory ( draftProject.getInd(),
                                                                    currentSubCat.toString(),
                                                                    TypeCategory.LEVEL_SUBCATEGORY,
                                                                    currentSubCat.getOrderNum(),
                                                                    currentSubCat.getChildSort(),
                                                                    currentSubCat.getOwner() ) ;
            }
         }

         if ( draftSubCat != null ) {
            /* Ok, now do the model  */
            if ( currentPartNum != null ) {
               draftPartNum = TypeCategory.getDraftCategoryFromDescription ( currentPartNum.toString(), 
                                                                             TypeCategory.LEVEL_PARTNUMBER, 
                                                                             draftSubCat.getInd() ) ;

               /* If we did not find one, create one now  */
               if ( draftPartNum == null ) {
                  draftPartNum = TypeCategory.createDraftTypeCategory ( draftSubCat.getInd(),
                                                                        currentPartNum.toString(),
                                                                        TypeCategory.LEVEL_SUBCATEGORY,
                                                                        currentPartNum.getOrderNum(),
                                                                        currentPartNum.getChildSort(),
                                                                        currentPartNum.getOwner() ) ;
               }
            }
         }

         /* Save these new machines/models to the database  */

         if ( draftMachine != null )
            setMachine   ( draftMachine ) ;
         if ( draftModel != null )
            hardSetModel ( draftModel ) ;

         if ( draftSubCat != null )
            setSubCategory ( draftSubCat ) ;
         if ( draftPartNum != null )
            setPartNumber ( draftPartNum ) ;
      }
   }


   /**
     * This method will return the TypeProjectRec representing the
     * project that this product belongs to. If this product does not belong
     * to a project, then the blank project for this product's family will be
     * returned. In any event, if you wish to change the project you should never
     * set the description on the project, you should either create a new project
     * or get another project from the TypeCategory class. If this product
     * does not have a family yet and this product does not have a project
     * then this method will return NULL.<br><br>
     *
     * <b>Please Note</b>: If the blank project is not created then this method
     * will create it and save it to the database using the TypeCategory.writeToDatabase method<br><br>
     *
     * <b>Please Note</b>: Any TypeCategory object created by this method will be saved to the DRAFT tables and ACTIVE tables
     **/
   public TypeCategoryRec getProject () {
      if ( project == null ) {
         if ( getFamily() != null ) {
            boolean         writeToDatabase   = false ;
            TypeCategoryRec blankProject      = TypeCategory.getCategoryFromDescription ( "Default",
                                                                                          TypeCategory.LEVEL_PROJECT,
                                                                                          getFamily().getInd() ) ;
            TypeCategoryRec blankDraftProject = TypeCategory.getDraftCategoryFromDescription ( "Default",
                                                                                               TypeCategory.LEVEL_PROJECT,
                                                                                               getFamily().getInd() ) ;

            if ( blankProject == null ) {
               blankProject = TypeCategory.createTypeCategory ( getFamily().getInd(),
                                                                "Default",
                                                                TypeCategory.LEVEL_PROJECT,
                                                                0,
                                                                "N",
                                                                "QUEST" ) ;

               writeToDatabase = true ;
            }

            if ( blankDraftProject == null ) {
               blankDraftProject = TypeCategory.createDraftTypeCategory ( blankProject.getInd(),
                                                                          getFamily().getInd(),
                                                                          "Default",
                                                                          TypeCategory.LEVEL_PROJECT,
                                                                          0,
                                                                          "N",
                                                                          "QUEST" ) ;

               writeToDatabase = true ;
            }

            if ( writeToDatabase )
               TypeCategory.writeToDatabase() ;

            projectUpdated = true ;
            this.project = blankProject ;
         } /* if family  */
      }

      return( project ) ;
   } /* getProject  */


   /**
     *  This method will set the OPIC/M Brand
     **/
   public void setOPICMBrand ( TypeCategoryRec opicmBrand ) {
      this.opicmBrand = opicmBrand ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will set the OPIC/M Family
     **/
   public void setOPICMFamily ( TypeCategoryRec opicmFamily ) {
      this.opicmFamily = opicmFamily ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will set the OPIC/M Series
     **/
   public void setOPICMSeries ( TypeCategoryRec opicmSeries ) {
      this.opicmSeries = opicmSeries ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /**
     *  This method will return the OPIC/M Brand
     **/
   public TypeCategoryRec getOPICMBrand () {
      return( opicmBrand ) ;
   }

   /**
     *  This method will return the OPIC/M Family
     **/
   public TypeCategoryRec getOPICMFamily () {
      return( opicmFamily ) ;
   }


   /**
     *  This method will return the OPIC/M Series
     **/
   public TypeCategoryRec getOPICMSeries() {
      return( opicmSeries ) ;
   }


   /**
     *  this method will return all OPIC/M Countries
     **/
   public Vector getOPICMCountries () {
      return( opicmCountries ) ;
   }

   /**
     *  This method will return all OPIC/M Geos
     **/
   public Vector getOPICMGeographies () {
      return( opicmGeographies ) ;
   }

   /**
     *  This method will set the list of OPIC/M Countries
     **/
   public void setOPICMCountries ( Vector countries ) {
      this.opicmCountries = countries ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will set the list of OPIC/M Geos
     **/
   public void setOPICMGeographies ( Vector geos ) {
      this.opicmGeographies = geos ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /**
     *  This method will return the description of this product in OPIC/M
     **/
   public String getOPICMDescript () {
      return( opicmDescript ) ;
   }


   /**
     *  This method will set the description of this product for OPIC/M
     **/
   public void setOPICMDescript ( String opicmDescript ) {
      this.opicmDescript = opicmDescript.trim() ;
   }


   /**
     * This method will return the original product draft indicator that
     * was first used to publish the product that this draft represents
     **/
   public int getOriginalProductInd () {
      return( originalProductInd ) ;
   }

   /**
     * This method will set the original product draft indicator that
     * was first used to publish the product that this draft represents
     *
     * @param productInd The original product indicator ;
     **/
   public void setOriginalProductInd ( int productInd ) {
      this.originalProductInd = productInd ;
   }

   /**
     * This method will return the OPIC/M Primary Key
     *
     * @return String The OPIC/M Key
     **/
   public String getOPICMKey  () {
      return( opicmKey ) ;
   }

   /**
     * This method will set the OPIC/M Primary key
     *
     * @param opicmKey The OPIC/M Primary Key
     **/
   public void setOPICMKey ( String opicmKey ) {

      /*------------------------------------------------*/
      /*                                                */
      /*  We only allow the set of an OPICMKey if there */
      /* is currently no key. DO NOT ALLOW THE UPDATE   */
      /* OF A CURRENT KEY!                              */
      /*                                                */
      /*------------------------------------------------*/
      if ( this.opicmKey == null ) {
         opicmKeyUpdated = true ;
         this.opicmKey = opicmKey ;
      }
   }




   /**
     *  This method will return TRUE if this product is a template
     **/
   public boolean getTemplate () {
      return( template ) ;
   }

   /**
     *  This method will set the template property. It will also NULL out
     * the Model value since templates shouldn't have the same model
     **/
   public void setTemplate (boolean template ) {
      this.template = template ;
      hardSetModel ( null ) ;
      hardSetPartNumber ( null ) ;
   }


   /**
     *  This method will set the product's brand
     **/
   public void setBrand ( TypeCategoryRec brand ) {
      if ( brand != null ) {
         if ( !brand.getIsDraft() ) {
            brand = (TypeCategoryRec)brand.clone() ;
            brand.setIsDraft ( true ) ;
         }
      }
      super.setBrand ( brand ) ;
   }

   /**
     *  This method will set the product's family
     **/
   public void setFamily ( TypeCategoryRec family ) {
      if ( family != null ) {
         if ( !family.getIsDraft() ) {
            family = (TypeCategoryRec)family.clone() ;
            family.setIsDraft ( true ) ;
         }
      }
      super.setFamily ( family ) ;
   }

   /**
     *  This method will set the product's model
     **/
   public void setModel ( TypeCategoryRec model ) {
      if ( model != null ) {
         if ( !model.getIsDraft() ) {
            if ( TypeCategory.getDraftCategory ( model.getInd() ) != null ) {
               model = (TypeCategoryRec)model.clone() ;
            } else {
               model = (TypeCategoryRec)model.clone() ;
               model.updateRecStatus ( REC_NOCHANGE )  ;
               model.updateRecStatus ( REC_INSERT ) ;
            }

            model.setIsDraft ( true ) ;
         }
         super.setModel ( model ) ;
      } else
         super.hardSetModel ( null ) ;
   }

   /**
     *  This method will set the product's project
     **/
   public void setProject ( TypeCategoryRec project ) {
      if ( project != null ) {
         if ( !project.getIsDraft() ) {
            if ( TypeCategory.getDraftCategory ( project.getInd() ) != null ) {
               project = (TypeCategoryRec)project.clone() ;
            } else {
               project = (TypeCategoryRec)project.clone() ;
               project.updateRecStatus ( REC_NOCHANGE )  ;
               project.updateRecStatus ( REC_INSERT ) ;
            }

            project.setIsDraft ( true ) ;
         }
      }
      super.setProject ( project ) ;
   }


   /**
     *  This method will set the product's machine
     **/
   public void setMachine ( TypeCategoryRec machine ) {
      if ( machine != null ) {
         if ( !machine.getIsDraft() ) {
            if ( TypeCategory.getDraftCategory ( machine.getInd() ) != null ) {
               machine = (TypeCategoryRec)machine.clone() ;
            } else {
               machine = (TypeCategoryRec)machine.clone() ;
               machine.updateRecStatus ( REC_NOCHANGE )  ;
               machine.updateRecStatus ( REC_INSERT ) ;
            }

            machine.setIsDraft ( true ) ;
         }
      }
      super.setMachine ( machine ) ;
   }

   /**
     *  This method will set the product's subCategory
     **/
   public void setSubCategory ( TypeCategoryRec subCategory ) {
      if ( subCategory != null ) {
         if ( !subCategory.getIsDraft() ) {
            if ( TypeCategory.getDraftCategory ( subCategory.getInd() ) != null ) {
               subCategory = (TypeCategoryRec)subCategory.clone() ;
            } else {
               subCategory = (TypeCategoryRec)subCategory.clone() ;
               subCategory.updateRecStatus ( REC_NOCHANGE )  ;
               subCategory.updateRecStatus ( REC_INSERT ) ;
            }

            subCategory.setIsDraft ( true ) ;
         }
      }
      super.setSubCategory ( subCategory ) ;
   }

   /**
     *  This method will set the product's partNum
     **/
   public void setPartNumber ( TypeCategoryRec partNumber ) {
      if ( partNumber != null ) {
         if ( !partNumber.getIsDraft() ) {
            if ( TypeCategory.getDraftCategory ( partNumber.getInd() ) != null ) {
               partNumber = (TypeCategoryRec)partNumber.clone() ;
            } else {
               partNumber = (TypeCategoryRec)partNumber.clone() ;
               partNumber.updateRecStatus ( REC_NOCHANGE )  ;
               partNumber.updateRecStatus ( REC_INSERT ) ;
            }

            partNumber.setIsDraft ( true ) ;
         }
      }
      super.setPartNumber ( partNumber ) ;
   }



   /**
     *  This method will create a new ProductDraft object that will save as
     * a new draft but has the same machine/model information as this current one
     *
     * @param lowCategory This is the lowest category for the product. Either the Model or Part Number
     **/
   public ProductDraft createNew ( TypeCategoryRec lowCategory ) {
      ProductDraft newDraft = new ProductDraft() ;


      newDraft.setWorkRequired     ( getWorkRequired() ) ;
      newDraft.setDescription      ( getDescription () ) ;
      newDraft.setBrand            ( getBrand() ) ;
      newDraft.setFamily           ( getFamily() ) ;
      newDraft.setProject          ( getProject() ) ;
      newDraft.setMachine          ( getMachine() ) ;
      newDraft.setSubCategory      ( getSubCategory() ) ;
      newDraft.setDates            ( getDates() ) ;
      newDraft.setCustomer         ( getCustomer() ) ;
      newDraft.setBaseModel        ( getBaseModel() ) ;
      newDraft.setLocale           ( getLocale() ) ;
      newDraft.setInput            ( getInput() ) ;
      newDraft.setComment          ( getComment() ) ;
      newDraft.setOffering         ( getOffering() ) ;
      newDraft.setWarranty         ( getWarranty() ) ;
      newDraft.setProductType      ( getProductType() ) ;
      newDraft.setPublish          ( getPublish() ) ;
      newDraft.setRecycled         ( getRecycled() ) ;
      newDraft.setOriginalComments ( getOriginalComments()) ;
      newDraft.setStartDate        ( new CDate ( CDate.DATE_STAMP ).today() ) ;
      newDraft.setXRef             ( getXRef() ) ;
      newDraft.setSource           ( getSource() ) ;

      if ( getProductType() == TypeList.PRODUCTTYPE_OPTION ) {
         newDraft.setPartNumber ( lowCategory ) ;
      } else {
         newDraft.setModel ( lowCategory ) ;
      }


      return( newDraft ) ;
   }



   /**
     *  This method will simply return TRUE if this product came in from OPIC/M
     **/
   public boolean isOPICM () {
      if ( getInput() != null )
         return( getInput().equals ( "OPIC/M" ) ) ;
      return( false ) ;
   }


   /**
     *  This method will return the original comments
     **/
   public String getOriginalComments () {
      return( originalComments ) ;
   }

   /**
     *  This method will set the original comments, this method
     * is only called by the Product Reader
     **/
   protected void setOriginalComments ( String comments ) {
      this.originalComments = comments ;
   }



   /**
     * This method will return the SQL needed to insert the OPIC/M Key into the
     * OPICM.PRIMARYLINK table
     **/
   private String opicmKeyToSQL () {
      String retStr = "" ;

      if ( opicmKeyUpdated ) {
         retStr = "INSERT INTO OPICM.PRIMARYLINK ( KEY, PRODUCTIND, DESCRIPT ) "+
                  "VALUES ( "+
                  " '"+opicmKey+"', "+
                  "  "+getProductInd()+", "+
                  " '"+Text.cleanDBString ( opicmDescript )+"')"+SqlRunner.END_DELIM ;
      }

      return( retStr ) ;
   }


   /**
     * This methodw will return the SQL needed to update the OPIC/M Description in
     * the OPICM.PRIMARYLINK table
     **/
   private String opicmDescriptToSQL () {
      String retStr = "" ;

      if ( ( opicmKey != null ) && ( opicmKey.length() > 0 ) ) {
         retStr = "UPDATE OPICM.PRIMARYLINK "+
                  "SET DESCRIPT = '"+Text.cleanDBString ( opicmDescript )+"', "+
                  "    KEY      = '"+opicmKey+"' "+
                  "WHERE PRODUCTIND = "+getProductInd()+SqlRunner.END_DELIM ;
      }

      return( retStr ) ;
   }


   /**
     * This method will set the user id of the person who created this draft
     *
     * @param userId The ID of the user who created the draft
     **/
   public void setDraftCreatedBy ( String userId ) {
      this.draftCreatedBy = userId ;
   }


   /**
     * This method will return the User Id of the person who created this draft
     **/
   public String getDraftCreatedBy () {
      return ( draftCreatedBy ) ;
   }


   /**
     * This method will set the time that this draft was created
     *
     * @param timestamp The ID of the user who created the draft
     **/
   public void setDraftCreatedOn ( String timestamp ) {
      this.draftCreatedOn = timestamp ;
   }


   /**
     * This method will return the timestamp of when this draft was created
     **/
   public String getDraftCreatedOn () {
      return ( draftCreatedOn ) ;
   }


} /* ProductDraft */

