package com.ibm.nzna.projects.common.quest.type;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      TypeCategoryRec
/*   Creation Date: APRIL 11, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.quest.Counter ;
import com.ibm.nzna.shared.util.Text ;
import com.ibm.nzna.shared.util.LogSystem ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This class holds information about a single row in the PRODUCT.TYPECATEGORY table.
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* Class1
/*
/*----------------------------------------------------*/
public class TypeCategoryRec extends TypeRec implements IQuickSort {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     * The Category Level describes what kind of category this is,
     * currently valid values are:<br><br>
     * 
     * <pre>
     * Value           Description<br>
     * --------------- --------------------------------------------<br>
     * brand           This category is an IBM Brand (Aptiva etc..)<br>
     * family          This category is an IBM Family (Thinkpad 770 etc..)<br>
     * project         This category is the project name (Yoshi etc..)<br>
     * machine         This category is a Machine (2168, 2673 etc..)<br>
     * model           This category is a Model (57C, 56C etc..)<br>
     * familytype      This category is a Family Type (32 Gig etc..)<br>
     * opicmbrand      This category is an OPIC/M Brand<br>
     * opicmfamily     This category is an OPIC/M Family<br>
     * opicmseries     This category is an OPIC/M Series<br>
     * opicmgeo        This category is an OPIC/M Geography<br>
     * opicmcountry    This category is an OPIC/M Country<br>
     * partnumber      This category is an Option's Part Number
     * </pre>
     *
     * <b>Note</b> You should NEVER hard code the names of these values, you
     * should <b>ALWAYS</b> use the TypeCategory.LEVEL_XXXX Constants.
     **/
   private String categoryLevel    = null ;



   /**
     * The Parent Category Indicator of this association. This relates this
     * category to a parent category.
     **/
   private int parentCatInd        = 0 ;


   /**
     * The order number is the order in which this category should appear
     * in a list. If this number is 0 then this category should appear in
     * alphabetical order.
     **/
   private int orderNum = 0 ;

   /**
     * The ChildSort property specifies wether the children of this
     * category should be sorted by the orderNum or alphabetically. Valid
     * values include 'Y', 'N'. Yes meaning that the orderNum should be used
     * in positioning this category in a list. A value of 'N' means that the
     * default alphabetical sorting should occur.
     **/
   private String childSort = "N" ;

   /**
     * This is the User Id of the person who owns this cateogry. If this
     * cateogry's level is equal to 'brand' then this is the Brand Program
     * Manager's User Id.
     **/
   private String owner = null ;


   /**
     * The 'valid' value should be used to determine if the data in the
     * database is valid. If this value is FALSE, then the row that this
     * record represents should be considered deleted.
     */
   private boolean valid = true ;



   /**
     * This flag tells wether this TypeCategory object is in Draft mode or
     * production mode. If this flag specifies this category is in Draft mode,
     * then all data saved will be to the PRODRAFT.TYPECATEGORY table
     */
   private boolean isDraft = false ;

   /**
     *  This property tells if the parent indicator property has been updated,
     * because if it has we will need to do more work in the save method
     **/
   private boolean parentUpdated = false ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/


   /**
     *
     * This consturctor creates and initializes a TypeCategoryRec object for
     * an existing TypeCategogry entry in PRODUCT.TYPECATEGORY.
     */
   public TypeCategoryRec (int typeCatInd) {
      super ( typeCatInd ) ;
   }


   /**
     *  This constructor creates and initializes a TypeCategoryRec object for
     * a new type category object. Before calling this method to create a new
     * object please check the TypeCategory class for any duplicate objects.
     *
     * @param description The description of the new TypeCategory
     * @param categoryLevel The Category Level of the new TypeCategory
     **/
   public TypeCategoryRec ( String description, String categoryLevel ) {
      updateRecStatus ( REC_INSERT ) ;
      setDescript ( description ) ;
      setCategoryLevel ( categoryLevel ) ;
   }

   /**
    * This Constructor creates a blank TypeCategoryRec object. This
    * is useful when you want to create a new TypeCategory row in
    * the PRODUCT.TYPECATEGORY table. 
    */
   public TypeCategoryRec ( ) {
      updateRecStatus ( REC_INSERT ) ;
   }


   /*------------------------------------------------------------*/
   /* getCategoryLevel
    */
   /**
    *    This method will return this association's Category Level.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String Currently either brand,family,familytype,machine,model,
    *                 project,opicmfamily,opicmbrand,opicmseries,opicmgeo,
    *                 opicmcountry or partnumber.
    *//*---------------------------------------------------------*/
   public String getCategoryLevel () {
      return( categoryLevel ) ;
   }


   /**
    *    This method will return the category's parent
    *  typeCatInd
    */
   public int getParentCatInd () {
      return( parentCatInd ) ;
   }

   /**
    * This method will return the owner property assigned to this
    * type of category
    */
   public String getOwner () {
      return( owner ) ;
   }

   /**
     * This method will set the owner of this category. The owner
     * is normally the User Id of a person in the database however
     * it does not have to be. Max limit is 12 characters
     **/
   public void setOwner ( String owner ) throws IllegalArgumentException {
      if ( owner != null ) {
         owner = owner.trim() ;
         if ( owner.length() <= 12 ) {
            updateRecStatus ( REC_UPDATE ) ;
            this.owner = owner.trim() ;
         } else
            throw new IllegalArgumentException ( "Owner cannot be "+
                                                 owner.length()+" characters "+
                                                 "long. Max is 12" ) ;
      } else {
         updateRecStatus ( REC_UPDATE ) ;
         this.owner = null ;
      }
   }

   /**
     * This method will set this cateogry's parent. This
     * method also updates the record status to UPDATE. To
     * NULL out the Parent (making this category a super parent)
     * simply pass a zero for the parentCatInd
     */
   public void setParentCatInd ( int parentCatInd ) {
      if ( this.parentCatInd != parentCatInd ) {
         this.parentCatInd = parentCatInd ;
         updateRecStatus ( REC_UPDATE ) ;
         parentUpdated = true ;
      }
   }



   /**
    * This method will set the categoryLevel property. This property is a
    * simply 15 character plain description of what this categogry is. Please
    * check the property description above for more information
    */
   public void setCategoryLevel ( String categoryLevel ) throws IllegalArgumentException {
      if ( categoryLevel != null ) 
         categoryLevel = categoryLevel.trim() ;

      if ( ( categoryLevel != null ) && ( categoryLevel.length() < 15 ) ) {
         this.categoryLevel = categoryLevel ;
         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Category Level cannot "+
                                              "be NULL and Max length is 12. The CategoryLevel passed was :"+categoryLevel+":" ) ;
   }


   /**
     * This method will return the order number that this category should appear
     * in a list
     */
   public int getOrderNum () {
      return( orderNum ) ;
   }

   /**
     * This method will set the order number that this category will appear in
     * a list
     **/
   public void setOrderNum ( int orderNum ) {
      if ( this.orderNum != orderNum ) {
         this.orderNum = orderNum ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   }


   /**
    * This method will set the child sort property. This property defines
    * if the children of this category should be sorted alphabetically (default)
    * or by using the children's order number. The ChildSort property will
    * be 'Y' if the list of children should default to the alphabetical listing,
    * and will be 'N' if the children have their order numbers set.
    **/
   public String getChildSort() {
      return( childSort ) ;
   }

   /**
     * This method will set the Child Sort value for our Category. This Child
     * sort value must be either 'Y' indicating that this category's children
     * should be order by their order number, 'N' indicates that this category's
     * children should be sorted alphabetically
     **/
   public void setChildSort ( String childSort ) throws IllegalArgumentException {
      if ( childSort != null )
         childSort=childSort.trim() ;

      if ( ( childSort == null ) ||
           ( ( !childSort.equals ( "Y" ) ) &&
             ( !childSort.equals ( "N" ) ) ) )
         throw new IllegalArgumentException ( "The childSort property MUST be"+
                                              "either 'Y' or 'N'" ) ;
      else {
         this.childSort = childSort ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   } /* setChildSort */


   /**
     *  This method will return the SQL String that is needed to save this record
     * to the database. The SQL that is returned will always have the SqlRunner.END_DELIM
     * appended to the end of the SQL String. This SQL is intended to be sent to the
     * SqlRunner Stored Procedure because in some cases multiple SQL Statements will
     * be needed to save this record.<br><br>
     *
     * If this record's status is to delete, then this TypeCategory and all children
     * of this category will be deleted. If a foreign key is using this category or
     * any of this category's children in the Quest database then the SQL given from
     * this method will fail. It's up to YOU to delete all referencing objects to this
     * category and all of this category's children. For example, if you delete a Brand then
     * all Families, Machines, Models, Parts and Document Categories under that brand
     * would be deleted as well.<br><br>
     *
     * Please note that since this method does not actually SAVE the record to the database,
     * so this method will never know if the SQL formed actually ran. You will have to manually
     * set the RecStatus to a valid value once you have confirmed the SQL ran.
     **/
   public String toSQL () {
      String sql     = "" ;

      if ( owner == null )
         owner = "QUEST" ;

      if ( getIsDraft() )
         return( toDraftSQL() ) ;

      else {
         Vector parents = TypeCategory.getCategoryParents ( this ) ;

         switch ( getRecStatus() ) {
            case REC_INSERT :

               generateKeys() ;

               sql = "insert into product.TypeCategory "+
                     "( typeCatInd,    description, parentCatInd, "+
                     "  categoryLevel, orderNum,    childSort, "+
                     "  owner, valid ) "+
                     "values ( "+
                     " "+getInd()+", "+
                     " '"+Text.cleanDBString ( toString() )+"', "+
                     " "+parentCatInd+", "+
                     " '"+categoryLevel+"', "+
                     "  "+orderNum+", "+
                     " '"+childSort+"', "+
                     "  "+Text.cleanNullableDBString ( owner )+", "+
                     " '"+Text.getYN ( valid )+"' )"+SqlRunner.END_DELIM ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, we must insert our parents into the                */
               /* PRODUCT.TYPECATCHILD table.                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( parents != null ) && ( parents.size() > 0 ) ) {
                  int              i           = 0 ;
                  int              size        = parents.size() ;
                  TypeCategoryRec  categoryRec = null ;

                  while ( i < size ) {
                     categoryRec = (TypeCategoryRec)parents.elementAt(i) ;
                     if ( categoryRec != null )
                        sql+="insert into product.typeCatChild values ( "+categoryRec.getInd()+", "+getInd()+" )"+SqlRunner.END_DELIM ;
                     i++ ;
                  }
               }

               break ;

            case REC_UPDATE :

               /* We have to delete the entries in TypeCatChild and Reinsert them because
                  if the Parent changed, we have no way of knowing what was previously there  */
               sql = "update product.TypeCategory "+
                     "set categoryLevel = '"+categoryLevel+"', "+
                     "    description   = '"+Text.cleanDBString ( toString() )+"', "+
                     "    parentCatInd  = "+parentCatInd+", "+
                     "    orderNum      = "+orderNum+", "+
                     "    childSort     = '"+childSort+"', "+
                     "    owner         = "+Text.cleanNullableDBString ( owner )+", "+
                     "    valid         = '"+Text.getYN ( valid )+"' "+
                     "where typeCatInd = "+getInd()+SqlRunner.END_DELIM ;


               if ( parentUpdated ) {

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Now, we must insert our parents into the                */
                  /* PRODUCT.TYPECATCHILD table.                               */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  sql+="delete from product.typeCatChild where typeCatInd = "+getInd()+SqlRunner.END_DELIM ;
                  if ( ( parents != null ) && ( parents.size() > 0 ) ) {
                     int              i           = 0 ;
                     int              size        = parents.size() ;
                     TypeCategoryRec  categoryRec = null ;

                     while ( i < size ) {
                        categoryRec = (TypeCategoryRec)parents.elementAt(i) ;
                        if ( categoryRec != null )
                           sql+="insert into product.typeCatChild values ( "+categoryRec.getInd()+", "+getInd()+" )"+SqlRunner.END_DELIM ;
                        i++ ;
                     }
                  }
               }
               break ;

            case REC_DELETE :

               /* Delete all entries for Documents, Document Drafts, Products and
                  Product Drafts including parents etc..  */
               sql = "delete from product.category where typeCatInd = "+getInd()+SqlRunner.END_DELIM+
                     "delete from prodraft.category where typeCatInd = "+getInd()+SqlRunner.END_DELIM+
                     "delete from tigris.category where typeCatInd = "+getInd()+SqlRunner.END_DELIM+
                     "delete from quest.category where typeCatInd = "+getInd()+SqlRunner.END_DELIM+
                     "delete from product.TypeCatChild where parentCatInd = "+getInd()+SqlRunner.END_DELIM+
                     "delete from product.TypeCatChild where typeCatInd = "+getInd()+SqlRunner.END_DELIM+
                     "delete from product.TypeCategory where TypeCatInd = "+getInd()+SqlRunner.END_DELIM ;
               break ;

         }
      }

      parentUpdated = false ;

      return( sql ) ;
   }



   /**
     *  This method will return the SQL String that is needed to save this record
     * to the database in the PRODRAFT schema. The SQL that is returned will always have the SqlRunner.END_DELIM
     * appended to the end of the SQL String. This SQL is intended to be sent to the
     * SqlRunner Stored Procedure because in some cases multiple SQL Statements will
     * be needed to save this record.<br><br>
     *
     * If this record's status is to delete, then this TypeCategory and all children
     * of this category will be deleted. If a foreign key is using this category or
     * any of this category's children in the Quest database then the SQL given from
     * this method will fail. It's up to YOU to delete all referencing objects to this
     * category and all of this category's children. For example, if you delete a Brand then
     * all Families, Machines, Models, Parts and Document Categories under that brand
     * would be deleted as well
     **/
   public String toDraftSQL () {
      String sql     = "" ;
      Vector parents = TypeCategory.getDraftCategoryParents ( this ) ;

      switch ( getRecStatus() ) {
         case REC_INSERT :
            generateKeys() ;

            sql = "insert into proDraft.TypeCategory "+
                  "( typeCatInd,    description, parentCatInd, "+
                  "  categoryLevel, orderNum,    childSort, "+
                  "  owner, valid ) "+
                  "values ( "+
                  " "+getInd()+", "+
                  " '"+Text.cleanDBString ( toString() )+"', "+
                  " "+parentCatInd+", "+
                  " '"+categoryLevel+"', "+
                  "  "+orderNum+", "+
                  " '"+childSort+"', "+
                  "  "+Text.cleanNullableDBString ( owner )+", "+
                  " '"+Text.getYN ( valid )+"' )"+SqlRunner.END_DELIM ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, we must insert our parents into the                */
            /* PRODRAFT.TYPECATCHILD table.                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( parents != null ) && ( parents.size() > 0 ) ) {
               int              i           = 0 ;
               int              size        = parents.size() ;
               TypeCategoryRec  categoryRec = null ;

               while ( i < size ) {
                  categoryRec = (TypeCategoryRec)parents.elementAt(i) ;
                  if ( categoryRec != null )
                     sql+="insert into proDraft.typeCatChild values ( "+categoryRec.getInd()+", "+getInd()+" )"+SqlRunner.END_DELIM ;
                  i++ ;
               }
            }

            break ;

         case REC_UPDATE :
            sql = "update proDraft.TypeCategory "+
                  "set categoryLevel = '"+categoryLevel+"', "+
                  "    description   = '"+Text.cleanDBString ( toString() )+"', "+
                  "    parentCatInd  = "+parentCatInd+", "+
                  "    orderNum      = "+orderNum+", "+
                  "    childSort     = '"+childSort+"', "+
                  "    owner         = "+Text.cleanNullableDBString ( owner )+", "+
                  "    valid         = '"+Text.getYN ( valid )+"' "+
                  "where typeCatInd = "+getInd()+SqlRunner.END_DELIM ;


            if ( parentUpdated ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, we must insert our parents into the                */
               /* PRODUCT.TYPECATCHILD table.                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               sql+="delete from proDraft.typeCatChild where typeCatInd = "+getInd()+SqlRunner.END_DELIM ;
               if ( ( parents != null ) && ( parents.size() > 0 ) ) {
                  int              i           = 0 ;
                  int              size        = parents.size() ;
                  TypeCategoryRec  categoryRec = null ;

                  while ( i < size ) {
                     categoryRec = (TypeCategoryRec)parents.elementAt(i) ;
                     if ( categoryRec != null )
                        sql+="insert into prodraft.typeCatChild values ( "+categoryRec.getInd()+", "+getInd()+" )"+SqlRunner.END_DELIM ;
                     i++ ;
                  }
               }
            }
            break ;

         case REC_DELETE :
            sql = "delete from proDraft.TypeCatChild where parentInd = "+getInd()+SqlRunner.END_DELIM+
                  "delete from proDraft.TypeCategory where TypeCatInd = "+getInd()+SqlRunner.END_DELIM ;
            break ;

      }

      updateRecStatus ( REC_NOCHANGE ) ;

      return( sql ) ;
   }



   /**
     *  This method will generate the typeCatInd key for this class if
     * one has not yet been assigned
     **/
   protected boolean generateKeys() {
      if ( super.getInd() == 0 ) {
         //if ( !isDraft )
            setInd ( Counter.getCounter ( LogSystem.getInstance(), "TYPECATIND" ) ) ;
         //else
         //   setInd ( Counter.getCounter ( LogSystem.getInstance(), "DCATIND" ) ) ;
      }
      return( super.getInd() != 0 ) ;
   } /* generateKeys  */

   /**
     * This method will set the 'valid' property
     *
     * @param valid TRUE = This record is valid
     */
   public void setValid ( boolean valid ) {
      this.valid = valid ;
   }


   /**
     * This method will return the valid property
     *
     * @return boolean
     */
   public boolean getValid ( ) {
      return( valid ) ;
   }

   /**
     *  This method will return the indicator for the primary key
     * of the Type Category. If an indicator has not been set yet
     * an indicator will be assigned now.
     **/
   public int getInd () {
      if ( super.getInd() == 0 ) {
         generateKeys() ;
      }

      return( super.getInd() ) ;
   }


   /**
     * This method will return the isDraft property
     *
     * @boolean
     */
   public boolean getIsDraft() {
      return( isDraft ) ;
   }

   /**
     * This method will set the isDraft property
     */
   public void setIsDraft ( boolean isDraft ) {
      this.isDraft = isDraft ;
   }


   /**
     *  This method will compare this object to another object and
     * return -1 if this object is less than the object passed, 0 if
     * the objects are equal and 1 if this object is greater.<br><br>
     *
     * If this TypeCategoryRec has a orderNum of zero, then it is assumed
     * that this category should be sorted via description.
     *
     * @param object The object to compare to
     *
     * @return -1, 0 or 1
     **/
   public int compareTo ( IQuickSort object ) {
      int rc = 0 ;                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the object we were passed is a TypeCategoryRec       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( object instanceof TypeCategoryRec ) {
         TypeCategoryRec compareRec = (TypeCategoryRec) object ;                               


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If ordernum is zero, sort by description                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( orderNum == 0 ) {
            rc = toString().compareTo ( compareRec.toString() ) ;

            if ( rc < 0 )
               rc= QuickSort.COMP_LESS;
            else
               if ( rc > 0 )
               rc= QuickSort.COMP_GREATER;
            else
               rc= QuickSort.COMP_EQUAL;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, sort by ordernum                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            if ( orderNum > compareRec.orderNum )
               rc=QuickSort.COMP_GREATER ;
            else if ( orderNum < compareRec.orderNum )
               rc=QuickSort.COMP_LESS ;
            else
               rc=QuickSort.COMP_EQUAL ;
         }
      }


      return( rc ) ;
   }


   public boolean equals ( Object object ) {
      boolean equals = (object==this);

      if ( object instanceof TypeCategoryRec ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   IF both are not drafts, compare by the indicator        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         //if ( ( !getIsDraft() ) && ( !((TypeCategoryRec)object).getIsDraft() ) )
            equals = ( ((TypeCategoryRec)object).getInd() == getInd() ) ;

         ///*-----------------------------------------------------------*/
         ///*                                                           */
         ///*   Otherwise, compare by the description and level         */
         ///*                                                           */
         ///*-----------------------------------------------------------*/
         //else {
         //   TypeCategoryRec compareRec = (TypeCategoryRec)object ;
         
         //   equals = ( ( toString().equals ( compareRec.toString() ) ) &&
         //              ( getCategoryLevel().equals ( compareRec.getCategoryLevel() ) ) ) ;
         //}

      }
      return( equals ) ;
   }


   /**
     *  This method will return an exact replica of this object
     **/
   public Object clone () {
      TypeCategoryRec categoryRec = new TypeCategoryRec ( getInd() ) ;

      categoryRec.setDescript      ( getDescript() ) ;
      categoryRec.setCategoryLevel ( getCategoryLevel() ) ;
      categoryRec.setParentCatInd  ( getParentCatInd() ) ;
      categoryRec.setOrderNum      ( getOrderNum() ) ;
      categoryRec.setChildSort     ( getChildSort() ) ;
      categoryRec.setOwner         ( getOwner() ) ;
      categoryRec.setValid         ( getValid() ) ;

      return( categoryRec ) ;
   }


   public int writeToDatabase () {
      int rc = 0 ;

      if ( getIsDraft() )
         rc = SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, toDraftSQL() ) ;
      else
         rc = SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, toSQL() ) ;

      if ( rc == 0 )
         updateRecStatus ( REC_NOCHANGE ) ;

      return ( rc ) ;
   }
} /* TypeCategoryRec  */