package com.ibm.nzna.projects.common.storedProc.extSqlRunner ; 
/*-----------------------------------------------------------*/
/*   SqlRunner                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SqlRunner                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This Stored Procedure will pick apart the CLOB sent to it and
 * will run whatever SQL is inside of the CLOB. The CLOB must adhere
 * to this format:
 *
 * SQL STATEMENT<END_DELIM>SQL STATEMENT<END_DELIM>
 *
 * For example:
 * DELETE FROM TIGRIS.DOCBODY WHERE DOCIND = 123<END>INSERT INTO TIGRIS.DOCBODY<END>
 *
 * Please note that this Stored Procedure is different than SqlRunner because that it will
 * parse the SQL directly from the incoming CLOB stream, and it will perform interval commits
 * as stated in params. Please note that a value of 0 passed to the param 'commitInterval' will
 * not commit until all statements have been executed.
 *
 * It will return the SQL Code and any messages in the StoredProcRetRec
 *
 * The END_DELIM must be sqlRunner.END_DELIM ;
 *
 *
 * <pre>
 * Param Number      Description<br>
 * ----------------- ----------------------
 * 1                 The CLOB containing the SQL Buffer to run
 * 2                 The number of statements to execute before committing. A value of 0 means no commit until all statements are done
 * 3                 The return record for errors.
 *
 * @version 1.0, 9/15/1999 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import java.sql.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Clob ;
import COM.ibm.db2.app.Blob ;
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SqlRunner                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class ExtSqlRunner extends StoredProc {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String END_DELIM           = new Character ( (char)3).toString() ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Program Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int           sqlCode          = 0 ;          // The SQLCODE we will return
   private Connection    con              = null ;       // Our connection to DB/2
   private String        strClob          = null ;       // The string holding the clob
   private String        errorStr         = null ;       // The string used to hold errors.
   private StoredProcRec returnRec        = new StoredProcRec() ;
   private String        lastSQL          = "" ;
   private int           statementsRun    = 0 ;
   private int           commitInterval   = 0 ;


   /*------------------------------------------------------------*/
   /* runSQL    
    */
   /**
    *    This method will be called by DB/2 when
    *  we are called by a client. It will process reading
    *  all of the document information
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*--------------------------------------------------------*/
   public void runSQL ( Clob inSql, int commitInterval, Blob storedProcRetRec ) throws Exception  {                                                       
      String     sqlToRun[] = null ;            

      con = getConnection() ;                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Turn auto commit off so we can control it               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.setAutoCommit(false);      // MAD 4/27
      this.commitInterval = commitInterval ; 

      try {

         parseAndRunSQL ( inSql ) ;

      } catch ( SQLException sqlEx ) {
         StoredProcUtil.writeException ( sqlEx, returnRec ) ;
         returnRec.errorStr+= "\nLast SQL Run:"+lastSQL ;
         returnRec.sqlCode  = sqlEx.getErrorCode()  ;
         con.rollback() ;
      } catch ( Throwable t ) {
         StoredProcUtil.writeException ( t, returnRec ) ;
         returnRec.errorStr+= "\nLast SQL Run:"+lastSQL ;
         returnRec.sqlCode  = sqlCode ;
         con.rollback() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   CLose up shop                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 3, storedProcRetRec ) ;

   }





   /*------------------------------------------------------------*/
   /* parseAndRunSQL
    */
   /**
    *    This method will attempt to pull a character
    *  String from the Clob passed in
    *  
    *
    *  @param lob The Clob to pull from
    *  @param None
    *  @param None
    *
    *  @return TRUE = Pull was good, FALSE = Pull was bad
    *//*---------------------------------------------------------*/
   private void parseAndRunSQL ( Clob lob ) throws Exception {
      final int WORD_CH_MIN = '\u0000';
      final int WORD_CH_MAX = '\u00ff';
      final int DELIM_CH    = (int) END_DELIM.charAt(0);

      BufferedReader  reader   = new BufferedReader( lob.getReader() );
      StreamTokenizer st       = new StreamTokenizer( reader );
      Statement       stmt     = con.createStatement() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the StreamTokenizer to read the SQL statements:  */
      /*   All characters are word characters except DELIM which   */
      /*   is the word delimiter character.                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st.resetSyntax();
      st.whitespaceChars(DELIM_CH, DELIM_CH);
      st.wordChars(WORD_CH_MIN, DELIM_CH-1);
      st.wordChars(DELIM_CH+1, WORD_CH_MAX);


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Run each SQL statement                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      int type;
      int count = 0;
      while ( (type = st.nextToken()) != StreamTokenizer.TT_EOF ) {
        count++;
        runSQL( st.sval, stmt );
      }
      // return statement count in userError string for debugging
      returnRec.userError = "ExtSqlRunner: processed " + count + " statements";
      reader.close();

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close our statement                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      stmt.close() ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Run the final commit                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;

   } /* parseAndRunSQL  */






   /*------------------------------------------------------------*/
   /* runSQL
    */
   /**
    *    This method will run the SQL Statement
    *  passed in
    *  
    *
    *  @param currentSQL The current SQL to run
    *  @param stmt The Statement
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void runSQL ( String currentSQL, Statement stmt ) throws Exception {

      lastSQL = currentSQL ;
      stmt.executeUpdate ( currentSQL ) ;

      statementsRun++ ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit if need be                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( statementsRun > commitInterval ) &&
           ( commitInterval > 0 ) ) {
         con.commit() ;
         statementsRun=0 ;
      }


   }

   public static int sqlRunnerCode ( int databaseId, String sqlRunnerString ) {
      SQLMethod sql     = new SQLMethod ( databaseId, "sqlRunner", 5 ) ;
      int       sqlCode = 0 ;

      try {
         CallableStatement stmt        = sql.createStoredProc ( "QUEST.ExtSqlRunner", 3 ) ;
         String            debugString = Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n" ) ;

         LogSystem.log ( 1, "Starting to send statements to EXT SQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the Callable Stmt                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlRunnerString.getBytes() ) ;      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute the SQL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log( 2, "Running SQLRunner:" ) ;
         LogSystem.log( 2, Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n    " ) ) ;
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            sqlCode = retRec.sqlCode ;
         }

         stmt.close() ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         sqlCode=-1 ;
      }

      sql.close() ;

      return( sqlCode ) ;
   }


} /* SqlRunner */
