
package com.ibm.nzna.projects.common.storedProc.oaPublish;

import com.ibm.nzna.projects.common.quest.oa.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.util.*;
import COM.ibm.db2.app.*;
import COM.ibm.db2.app.Blob;
import java.io.*;
import java.sql.*;
import java.util.*;

/**
 * DB2 Stored Procedure to publish draft Online Assistant objects.
 * The stored procedure known to DB2 as QUEST.OAPublish.
 * 
 * @version 1.10
 * @author  Mark DeBiase
**/
public class OAPublish extends StoredProc {

  //*************************************************************************
  // if DEBUG=true, then the SQL generated to publish the object will be
  // returned in the userError member of the returned StoredProcRec.
  //*************************************************************************
  private static final boolean DEBUG=true;
  //*************************************************************************

  // objects types that can be published
  public static final int ACTION     = 0;
  public static final int QUESTION   = 1;
  public static final int SYMPTOM    = 2;
  public static final int LINK       = 3;
  public static final int LINKGROUP  = 4;

  // our return record
  StoredProcRec returnRec = new StoredProcRec();

  /**
   * Method invoked by DB2 when the stored procedure is called.
   * Will call the appropriate reader method based on the objectType parm.
   *
   * @param draftObjectInd    the draft indicator of the object to be published
   * @param objectType        the type of object to be published
   * @param storedProcRetRec  returns processing status
  **/
  public void publishOAObject(int draftObjectInd, int objectType,
                              Blob storedProcRetRec)
              throws Exception
  {
    Connection conn = null;
    int sqlCode = 0;

    try {
      conn = getConnection();
      conn.setAutoCommit(false);

      switch (objectType) {
        case ACTION:
          {
            ActionDraft ad = new ActionDraft(draftObjectInd);
            if ( !ActionDraftReader.readFromDB(conn, ad, 0) ) {
              returnRec.error = true;
              returnRec.errorStr = "Failed to read draft Action " + 
                                   draftObjectInd + " from the database";
            } else {
              // these methods will throw PublishException on error
              checkPublishable(ad);
              String publishSQL = ActionWriter.getPublishSQL(conn, ad);
              runSQL(conn, publishSQL);
              updateActionAddtlInfo( conn, ad.getAdditionalInfo() );
            }
          }
          break;

        case QUESTION:
          {
            QuestionDraft qd = new QuestionDraft(draftObjectInd);
            if ( !QuestionDraftReader.readFromDB(conn, qd, 0) ) {
              returnRec.error = true;
              returnRec.errorStr = "Failed to read draft Question " + 
                                   draftObjectInd + " from the database";
            } else {
              // these methods will throw PublishException on error
              checkPublishable(qd);
              String publishSQL = QuestionWriter.getPublishSQL(conn, qd);
              runSQL(conn, publishSQL);
              updateQuestionAddtlInfo( conn, qd.getAdditionalInfo() );
            }
          }
          break;

        case SYMPTOM:
          {
            SymptomDraft sd = new SymptomDraft(draftObjectInd);
            if ( !SymptomDraftReader.readFromDB(conn, sd, 0) ) {
              returnRec.error = true;
              returnRec.errorStr = "Failed to read draft Symptom " + 
                                   draftObjectInd + " from the database";
            } else {
              // these methods will throw PublishException on error
              checkPublishable(sd);
              String publishSQL = SymptomWriter.getPublishSQL(conn, sd);
              runSQL(conn, publishSQL);
            }
          }
          break;

        case LINK:
          {
            LinkDraft ld = LinkDraftReader.readFromDB(conn, draftObjectInd, 0);
            if ( ld == null ) {
              returnRec.error = true;
              returnRec.errorStr = "Failed to read draft Link " + 
                                   draftObjectInd + " from the database";
            } else {
              // these methods will throw PublishException on error
              checkPublishable(ld);
              String publishSQL = LinkWriter.getPublishSQL(conn, ld);
              runSQL(conn, publishSQL);
            }
          }
          break;

        case LINKGROUP:
          {
            LinkGroupDraft lgd = new LinkGroupDraft(draftObjectInd);
            if ( !LinkGroupDraftReader.readFromDB(conn, lgd, 0) ) {
              returnRec.error = true;
              returnRec.errorStr = "Failed to read draft LinkGroup " + 
                                   draftObjectInd + " from the database";
            } else {
              // these methods will throw PublishException on error
              checkPublishable(lgd);
              String publishSQL = LinkGroupWriter.getPublishSQL(conn, lgd);
              runSQL(conn, publishSQL);
            }
          }
          break;

        default:
          {
            returnRec.error = true;
            returnRec.errorStr = "Invalid objectType " + objectType;
          }
          break;
      }
    }
    catch(PublishException publishEx) {
      // the method that threw the PublishException should have 
      // already done the following:
      // 1. Performed a rollback if necessary
      // 2. Populated returnRec.errorStr
      // 3. Set returnRec.error = true
      // nothing left to do here....
    }
    catch(SQLException se) {
      StoredProcUtil.writeException( se, returnRec );
      returnRec.sqlCode = se.getErrorCode();
      returnRec.error = true;
      conn.rollback();
    }
    catch(Exception e) {
      StoredProcUtil.writeException( e, returnRec );
      returnRec.error = true;
      conn.rollback();
    }
    catch(Error err) {
      StoredProcUtil.writeException( err, returnRec );
      returnRec.error = true;
      conn.rollback();
    }

    // if all processing was successfull, commit the transaction
    conn.commit();
    conn.close();

    // write out the return information
    storedProcRetRec = Lob.newBlob();
    StoredProcUtil.setObjectInBlob( storedProcRetRec, returnRec );

    set( 3, storedProcRetRec );
  }

  /**
   * Execute the SQL statements (INSERT/UPDATE/DELETE only) that are 
   * in the passed String.  Each statement should be separated by
   * SqlRunner.END_DELIM.
   * The DB2 transaction will be rolled back if an error occurs.
   *
   * @exception PublishException  if an error is encountered
   * @exception SQLException      only if the rollback() on error fails.
  **/
  //
  // error messges will be written to returnRec
  //
  private void runSQL(Connection conn, String sqlString) 
               throws SQLException, PublishException
  {
    //***********************************************************************
    if (DEBUG) {
      // write the SQL statements to the userError string for debugging:
      String tmp;
      tmp = Text.replaceAllStrInStr(sqlString, SqlRunner.END_DELIM, "\n");
      if (tmp.length() > (Short.MAX_VALUE * 2)) {
        // can't serialize a String > 65535 bytes, so truncate it
        returnRec.userError = tmp.substring(1, Short.MAX_VALUE * 2);
      } else {
        returnRec.userError = tmp;
      }
    }
    //***********************************************************************

    StringTokenizer st = new StringTokenizer(sqlString, SqlRunner.END_DELIM);
    Statement stmt = null;
    String currentStatement = "";

    // execute the SQL statements:
    try {
      stmt = conn.createStatement();

      while ( st.hasMoreTokens() ) {
        currentStatement = st.nextToken();
        stmt.executeUpdate( currentStatement );
      }
    }
    catch(SQLException se) {
      StoredProcUtil.writeException( se, returnRec );
      returnRec.errorStr += "===> Current SQL Statement: " + currentStatement;
      returnRec.sqlCode = se.getErrorCode();
      returnRec.error = true;
      conn.rollback();
      throw new PublishException();
    }
    finally {
      try { if (stmt != null) stmt.close(); } catch(Exception ignore) { }
    }

  }

  private void updateActionAddtlInfo(Connection conn, Vector v)
                  throws SQLException, PublishException
  {
    updateAddtlInfo(conn, v, "oa.actionAddInfo", "actionAddInfoInd");
  }
  private void updateQuestionAddtlInfo(Connection conn, Vector v)
                  throws SQLException, PublishException
  {
    updateAddtlInfo(conn, v, "oa.questionAddInfo", "questionAddInfoInd");
  }

  /**
   * Update the additional information text on publish.
   * The DB2 transaction will be rolled back if an error occurs.
   *
   * @exception PublishException  if an error is encountered
   * @exception SQLException      only if the rollback() on error fails.
  **/
  //
  // error messges will be written to returnRec
  //
  private void updateAddtlInfo(Connection conn, Vector addtlInfoVec,
                                  String tableName, String indName)
               throws SQLException, PublishException
  {
    boolean success = true;
    PreparedStatement pstmt = null;

    try {
      // all the addtl infos must be updated because they are 
      // deleted and re-inserted on publish.
      if ( addtlInfoVec != null && addtlInfoVec.size() > 0 ) {
        pstmt = conn.prepareStatement(
          "UPDATE " + tableName + 
          " SET addInfo=?" +
          " WHERE " + indName + "=?" +
          " AND languageInd=?"
        );

        for (int i=0; i < addtlInfoVec.size(); i++ ) {
          AddtlInfo ai = (AddtlInfo) addtlInfoVec.elementAt(i);

          pstmt.setString( 1, ai.getAddtlInfo() );
          pstmt.setInt( 2, ai.getInd() );
          pstmt.setInt( 3, ai.getLanguage().getInd() );
          pstmt.executeUpdate();
        }
      }
    }
    catch(SQLException se) {
      success = false;
      StoredProcUtil.writeException( se, returnRec );
      returnRec.errorStr += "Failure while updating AddtlInfo objects";
      returnRec.sqlCode = se.getErrorCode();
      returnRec.error = true;
      conn.rollback();
      throw new PublishException();
    }
    finally {
      try { if (pstmt != null) pstmt.close(); } catch(Exception ignore) { }
    }

  }

  /**
   * Check to make sure this draft object is eligible to be published.
   * The following checks are made:
   * <ul>
   * <li>stopDate is set to null
   * <li>maintLock is set to false
   * <li>recycled is set to false
   * </ul>
   *
   * @exception PublishException  if the draft cannot be published.
  **/
  //
  // error messges will be written to returnRec
  //
  private void checkPublishable(DraftObject draft) throws PublishException {

    if ( draft.getStopDate() != null ) {
      returnRec.errorStr = "Draft was closed on " + draft.getStopDate();
      returnRec.error = true;
    } else if ( draft.getMaintLock() ) {
      returnRec.errorStr = "Maint lock is set";
      returnRec.error = true;
    } else if ( draft.isRecycled() ) {
      returnRec.errorStr = "Draft is recycled";
      returnRec.error = true;
    }

    if (returnRec.error) {
      throw new PublishException();
    }
  }
}
