package com.ibm.nzna.projects.common.storedProc.oaReader;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.oa.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.shared.util.* ;
import COM.ibm.db2.app.*;
import COM.ibm.db2.app.Blob;
import java.sql.*;

/**
 * DB2 Stored Procedure to read Online Assistant objects from the database.
 * The stored procedure known to DB2 as QUEST.OAReader.
 * 
 * @version 1.14
 * @author  Mark DeBiase
**/
public class OAReader extends StoredProc {

   // objects types that can be read
   public static final int ACTION        = 0;
   public static final int QUESTION      = 1;
   public static final int SYMPTOM       = 2;
   public static final int LINK          = 3;
   public static final int LINKGROUP     = 4;

   // type of read to perform
   /** Read specific active object by active object indicator */
   public static final int READ_ACTIVE = 0;
   /** Read specific draft object by draft object indicator */
   public static final int READ_DRAFT  = 1;
   /** Read draft object for active object indicator. If there is an open
       draft, it will be read.  If not, one will be created. */
   public static final int GET_DRAFT   = 2;

   // Return code to be set in StoredProcRec.sqlCode when a draft is pending
   // (this value should never be returned as a SQLCODE in a SQLException)
   public static final int PENDING_DRAFT_RC = 100;

   /**
    * Method invoked by DB2 when the stored procedure is called.
    * Will call the appropriate reader method based on the objectType parm.
    * <br>
    * Note: the mode paramater is only used when readType = OAReader.READ_ACTIVE.
    * In that case, setting mode = OAMode.WEB provides optimizations for 
    * the WebRenderer (i.e., not reading data req'd only by QIT).
    *
    * @param objectInd         the indicator of the object to be read
    * @param objectType        the type of object to be read
    * @param readType          type of read to perform: READ_ACTIVE, READ_DRAFT
    *                          or GET_DRAFT.
    * @param languageInd       the language to select text items for
    * @param mode              one of the OAMode values (i.e., OAMode.DEFAULT,
    *                          OAMode.WEB).
    * @param returnedObject    returns the object that was read
    *                          or null if object not found or error
    * @param storedProcRetRec  returns processing status
   **/
   public void readOAObject(int objectInd, int objectType,
                            int readType, int languageInd, int mode,
                            Blob returnedObject, Blob storedProcRetRec)
   throws Exception
   {
      Object objectToReturn = null;
      StoredProcRec returnRec = new StoredProcRec();
      Connection conn = null;
      int sqlCode = 0;

      try {
         conn = getConnection();
         conn.setAutoCommit(true);

         switch ( objectType ) {
            case ACTION:
               {
                  if ( readType == READ_ACTIVE ) {
                     Action a = new Action( objectInd );
                     if ( ActionReader.readFromDB( conn, a, languageInd, mode ) ) {
                        objectToReturn = a;
                     }
                  } else if ( readType == READ_DRAFT ) {
                     ActionDraft ad = new ActionDraft( objectInd );
                     if ( ActionDraftReader.readFromDB( conn, ad, languageInd ) ) {
                        objectToReturn = ad;
                     }
                  } else if ( readType == GET_DRAFT ) {
                     ActionDraft ad = ActionDraftReader.getDraft( conn, objectInd,
                                                                  languageInd );
                     objectToReturn = ad;
                  } else {
                     returnRec.error = true;
                     returnRec.errorStr = "Invalid readType " + readType + 
                                          " for ACTION";
                  }
               }
               break;

            case QUESTION:

               {
                  if ( readType == READ_ACTIVE ) {
                     Question q = new Question( objectInd );
                     if ( QuestionReader.readFromDB( conn, q, languageInd, mode ) ) {
                        objectToReturn = q;
                     }
                  } else if ( readType == READ_DRAFT ) {
                     QuestionDraft qd = new QuestionDraft( objectInd );
                     if ( QuestionDraftReader.readFromDB( conn, qd, languageInd ) ) {
                        objectToReturn = qd;
                     }
                  } else if ( readType == GET_DRAFT ) {
                     QuestionDraft qd = QuestionDraftReader.getDraft( conn, objectInd,
                                                                      languageInd );
                     objectToReturn = qd;
                  } else {
                     returnRec.error = true;
                     returnRec.errorStr = "Invalid readType " + readType + 
                                          " for QUESTION";
                  }
               }
               break;

            case SYMPTOM:
               {
                  if ( readType == READ_ACTIVE ) {
                     Symptom s = new Symptom( objectInd );
                     if ( SymptomReader.readFromDB( conn, s, languageInd, mode ) ) {
                        objectToReturn = s;
                     }
                  } else if ( readType == READ_DRAFT ) {
                     SymptomDraft sd = new SymptomDraft( objectInd );
                     if ( SymptomDraftReader.readFromDB( conn, sd, languageInd ) ) {
                        objectToReturn = sd;
                     }
                  } else if ( readType == GET_DRAFT ) {
                     SymptomDraft sd = SymptomDraftReader.getDraft( conn, objectInd,
                                                                    languageInd );
                     objectToReturn = sd;
                  } else {
                     returnRec.error = true;
                     returnRec.errorStr = "Invalid readType " + readType + 
                                          " for SYMPTOM";
                  }
               }
               break;

            case LINK:
               {
                  if ( readType == READ_ACTIVE ) {
                     // LinkReader.readFromDB() returns null if Link is not found
                     objectToReturn = LinkReader.readFromDB( conn, objectInd,
                                                             languageInd );
                  } else if ( readType == READ_DRAFT ) {
                     // LinkDraftReader.readFromDB() returns null if draft not found
                     objectToReturn = LinkDraftReader.readFromDB( conn, objectInd,
                                                                  languageInd );
                  } else if ( readType == GET_DRAFT ) {
                     LinkDraft ld = LinkDraftReader.getDraft( conn, objectInd,
                                                              languageInd );
                     objectToReturn = ld;
                  } else {
                     returnRec.error = true;
                     returnRec.errorStr = "Invalid readType " + readType + 
                                          " for LINK";
                  }
               }
               break;

            case LINKGROUP:
               {
                  if ( readType == READ_ACTIVE ) {
                     LinkGroup lg = new LinkGroup( objectInd );
                     if ( LinkGroupReader.readFromDB( conn, lg, languageInd ) ) {
                        objectToReturn = lg;
                     }
                  } else if ( readType == READ_DRAFT ) {
                     LinkGroupDraft lgd = new LinkGroupDraft( objectInd );
                     if ( LinkGroupDraftReader.readFromDB( conn, lgd, languageInd ) ) {
                        objectToReturn = lgd;
                     }
                  } else if ( readType == GET_DRAFT ) {
                     LinkGroupDraft lgd = LinkGroupDraftReader.getDraft( conn,
                                                                         objectInd,
                                                                         languageInd );
                     objectToReturn = lgd;
                  } else {
                     returnRec.error = true;
                     returnRec.errorStr = "Invalid readType " + readType + 
                                          " for LINKGROUP";
                  }
               }
               break;

            default:
               {
                  returnRec.error = true;
                  returnRec.errorStr = "Invalid objectType " + objectType;
               }
               break;
         }


         if ( readType == GET_DRAFT ) {
            setDateAndTimeForNewDraft( conn, objectToReturn );
         }

      } catch ( SQLException se ) {
         StoredProcUtil.writeException( se, returnRec );
         returnRec.sqlCode = se.getErrorCode();
         returnRec.error = true;
         conn.rollback();
      } catch ( PendingDraftException pend ) {
         returnRec.error = true;
         returnRec.errorStr = "Draft is pending for indicator " + objectInd;
         returnRec.sqlCode = PENDING_DRAFT_RC;
      } catch ( Exception e ) {
         StoredProcUtil.writeException( e, returnRec );
         returnRec.error = true;
         conn.rollback();
      } catch ( Error err ) {
         StoredProcUtil.writeException( err, returnRec );
         returnRec.error = true;
         conn.rollback();
      }

      conn.commit();
      conn.close();

      // write the object that was read to the blob
      returnedObject = Lob.newBlob();
      StoredProcUtil.setObjectInBlob( returnedObject, objectToReturn );

      // write out the return information
      storedProcRetRec = Lob.newBlob();
      StoredProcUtil.setObjectInBlob( storedProcRetRec, returnRec );

      set( 6, returnedObject   );
      set( 7, storedProcRetRec );
   }

   /**
    * If we are returning a newly opened draft, then set the creation time,
    * changed time and start date from DB2 values.
   **/
   private void setDateAndTimeForNewDraft(Connection conn, Object obj) {

      // make sure this is a new draft before proceeding
      if ( obj != null && 
           ((PersistentRec) obj).getRecStatus() == PersistentRec.REC_INSERT ) {
         // get the current timestamp/date values from DB2
         // select against typeAction because it has only a few rows....
         try {
            Statement stmt = conn.createStatement();
            ResultSet rs = stmt.executeQuery(
                                            "SELECT DISTINCT CURRENT TIMESTAMP, CURRENT DATE FROM oa.typeAction"
                                            );
            rs.next();
            String timestamp = rs.getString(1);
            String date      = rs.getString(2);

            // set the start date
            DraftObject draft = (DraftObject) obj;
            draft.setStartDate( date );

            // set the creation/changed times
            timestamp = timestamp.substring(0, timestamp.lastIndexOf('.'));
            if ( obj instanceof OAObject ) {
               OAObject oaobj = (OAObject) obj;
               oaobj.setCreationTime( timestamp );
               oaobj.setChangedTime( timestamp );
            } else if ( obj instanceof Link ) {
               Link lnk = (Link) obj;
               lnk.setCreationTime( timestamp );
               lnk.setChangedTime( timestamp );
            } else if ( obj instanceof LinkGroup ) {
               LinkGroup lnkGrp = (LinkGroup) obj;
               lnkGrp.setCreationTime( timestamp );
               lnkGrp.setChangedTime( timestamp );
            }

            rs.close();
            stmt.close();
         } catch ( SQLException ignore ) {
         }
      }
   }
}
