
package com.ibm.nzna.projects.common.storedProc.oaValidate;

import com.ibm.nzna.projects.common.storedProc.*;
import java.util.*;

/**
 * The FailureRec object contains information about validation failures
 * for draft Symptom and Question objects.
 * This class holds the following information about the failure:
 * <ul>
 * <li> objectInd     - The indicator of the object which caused the failure.
 * <li> objectType    - The type of object (Action, Question or Symptom).
 * <li> title         - The default title of the object.
 * <li> failureReason - Indicates the reason for validation failure.
 * <li> activeObject  - If true, this is an active object.
 *                      If false, it is a draft object.
 * </ul>
 * If the failure was due to a specific Answer to a Question, the 
 * following fields will also be populated.
 * <ul>
 * <li> answerInd   - The Answer's indicator.
 * <li> answerTitle - The Answer's default title.
 * </ul>
 * <p>
 *
 * <i>Note</i>: The Symptom validation failures have been changed to
 *              corequisites and are now reported in the xxxCoreqVec
 *              fields of the ValidationRec object.
 * @see ValidationRec
 * 
 * @version 1.4
 * @author  Mark DeBiase
**/
public class FailureRec implements java.io.Serializable {

  //------------------------------------------------------------------------
  // objectType values
  //------------------------------------------------------------------------
  /** objectType: objectInd is for an Action object */
  public static final short TYPE_ACTION    = 800;
  /** objectType: objectInd is for a Question object */
  public static final short TYPE_QUESTION  = 801;
  /** objectType: objectInd is for a Symptom object */
  public static final short TYPE_SYMPTOM   = 802;
  /** objectType: objectInd is for a LinkGroup object */
  public static final short TYPE_LINKGROUP = 803;
  /** objectType: objectInd is for a Link object */
  public static final short TYPE_LINK      = 804;

  //------------------------------------------------------------------------
  // Reasons for failure
  //------------------------------------------------------------------------

  // Symptom Validation Failures:
  /** Symptom: open draft exists for a Question used in a Node 
      @deprecated Now repored in questionCoreqVec 
      @see ValidationRec.questionCoreqVec */
  public static final short SFAILURE_OPEN_DRAFT_QUESTION   = 901;
  /** Symptom: unpublished object used in a Node 
      @deprecated Now repored in xxxCoreqVec 
      @see ValidationRec.actionCoreqVec
      @see ValidationRec.questionCoreqVec
      @see ValidationRec.symptomCoreqVec   */
  public static final short SFAILURE_UNPUBLISHED_NODE      = 902;
  /** Symptom: unpublished Action used in a EdgeAlist 
      @deprecated Now reported in actionCoreqVec 
      @see ValidationRec.actionCoreqVec */
  public static final short SFAILURE_UNPUBLISHED_ALIST     = 903;
  /** Symptom: unpublished Question used in a EdgeUnanswer 
      @deprecated Now reported in questionCoreqVec 
      @see ValidationRec.questionCoreqVec */
  public static final short SFAILURE_UNPUBLISHED_UNANSWER  = 904;
  /** Symptom: unpublished Answer used in a EdgePreanswer 
      @deprecated Now reported in questionCoreqVec 
      @see ValidationRec.questionCoreqVec */
  public static final short SFAILURE_UNPUBLISHED_PREANSWER = 905;

  // Question Validation Failures:
  /** Question: no open draft for Symptom which uses the Question */
  public static final short QFAILURE_NO_OPEN_DRAFT_SYMPTOM = 906;
  /** Question: Answer used in Action AddtlInfo Condition deleted */
  public static final short QFAILURE_ACTION_INFO_COND      = 907;
  /** Question: Answer used in Action DocLink Condition deleted */
  public static final short QFAILURE_ACTION_LINK_COND      = 908;
  /** Question: Answer used in Action Title Condition deleted */
  public static final short QFAILURE_ACTION_TITLE_COND     = 909;
  /** Question: Answer used in Question AddtlInfo Condition deleted */
  public static final short QFAILURE_QUESTION_INFO_COND    = 910;
  /** Question: Answer used in Question DocLink Condition deleted */
  public static final short QFAILURE_QUESTION_LINK_COND    = 911;
  /** Question: Answer used in Question Title Condition deleted */
  public static final short QFAILURE_QUESTION_TITLE_COND   = 912;
  /** Question: Answer used in Answer Condition deleted */
  public static final short QFAILURE_ANSWER_COND           = 913;
  /** Question: Answer used in Answer Title Condition deleted */
  public static final short QFAILURE_ANSWER_TITLE_COND     = 914;
  /** Question: Answer used in Answer Concl Condition deleted */
  public static final short QFAILURE_ANSWER_CONCL_COND     = 915;
  /** Question: Answer used in Edge Condition deleted */
  public static final short QFAILURE_EDGE_COND             = 916;
  /** Question: Answer used in Edge deleted */
  public static final short QFAILURE_EDGE                  = 917;
  /** Question: Answer used in Edge Preanswer deleted */
  public static final short QFAILURE_EDGE_PREANSWER        = 918;

  // Failures applicable to any type of object:
  /** Maintlock is set for a draft object */
  public static final short FAILURE_MAINTLOCK_SET          = 919;

  //------------------------------------------------------------------------
  // Object member fields
  //------------------------------------------------------------------------

  /**
   * Indicator of the object which caused the failure.
   * The objectType field indicates the type of object the indicator 
   * represents.
   * The activeObject field indicates whether it is an active or draft 
   * object indicator.
  **/
  public int objectInd;

  /**
   * The default title of the object represented by objectInd.
  **/
  public String title;

  /**
   * The type of object represented by objectInd.
   * Will be one of the following values:
   * <ul>
   * <li> TYPE_ACTION
   * <li> TYPE_QUESTION
   * <li> TYPE_SYMPTOM
   * </ul>
  **/
  public short objectType;

  /**
   * The reason for failure.
   * Will be one of the following values:
   * <ul>
   * <li> SFAILURE_OPEN_DRAFT_QUESTION     (Symptom validation only) <b>deprecated</b>
   * <li> SFAILURE_UNPUBLISHED_NODE        (Symptom validation only) <b>deprecated</b>
   * <li> SFAILURE_UNPUBLISHED_ALIST       (Symptom validation only) <b>deprecated</b>
   * <li> SFAILURE_UNPUBLISHED_UNANSWER    (Symptom validation only) <b>deprecated</b>
   * <li> SFAILURE_UNPUBLISHED_PREANSWER   (Symptom validation only) <b>deprecated</b>
   *
   * <li> QFAILURE_NO_OPEN_DRAFT_SYMPTOM   (Question validation only)
   * <li> QFAILURE_ACTION_INFO_COND        (Question validation only)
   * <li> QFAILURE_ACTION_LINK_COND        (Question validation only)
   * <li> QFAILURE_ACTION_TITLE_COND       (Question validation only)
   * <li> QFAILURE_QUESTION_INFO_COND      (Question validation only)
   * <li> QFAILURE_QUESTION_LINK_COND      (Question validation only)
   * <li> QFAILURE_QUESTION_TITLE_COND     (Question validation only)
   * <li> QFAILURE_ANSWER_COND             (Question validation only)
   * <li> QFAILURE_ANSWER_TITLE_COND       (Question validation only)
   * <li> QFAILURE_ANSWER_CONCL_COND       (Question validation only)
   * <li> QFAILURE_EDGE_COND               (Question validation only)
   * <li> QFAILURE_EDGE                    (Question validation only)
   * <li> QFAILURE_EDGE_PREANSWER          (Question validation only)
   *
   * <li> FAILURE_MAINTLOCK_SET            (For any object type)
   * </ul>
  **/
  public short failureReason;

  /**
   * Indicates whether objectInd is an active or draft indicator.
   * If true, objectInd is an active object indicator.
   * If false, objectInd is a draft object indicator (for an OPEN draft).
  **/
  public boolean activeObject;

  /**
   * The indicator of the specific Answer involved when failureReason 
   * indicates failure for Conditions on an Answer.
   * This field is only populated for the following usageType values:
   * <ul>
   * <li> QFAILURE_ANSWER_COND
   * <li> QFAILURE_ANSWER_TITLE_COND
   * <li> QFAILURE_ANSWER_CONCL_COND
   * </ul>
   * If usageType has any other value, answerInd will have the value -1.
  **/
  public int answerInd;

  /**
   * The default title of the Answer represented by answerInd.
   * This field is only populated for the following usageType values:
   * <ul>
   * <li> QFAILURE_ANSWER_COND
   * <li> QFAILURE_ANSWER_TITLE_COND
   * <li> QFAILURE_ANSWER_CONCL_COND
   * </ul>
   * If usageType has any other value, answerTitle will be null.
  **/
  public String answerTitle = null;

  /**
   * The DBUSER who is currently editing a draft object.  
   * This field is only valid when failureReason is FAILURE_MAINTLOCK_SET.
  **/
  public String dbUser = null;


  //------------------------------------------------------------------------
  // Constructors
  //------------------------------------------------------------------------

  /** Used by OAValidate stored procedure. */
  FailureRec(int objectInd, short objectType, 
             short failureReason, boolean activeObject)
  {
    this(objectInd, objectType, failureReason, activeObject, -1);
  }

  /** Used by OAValidate stored procedure. */
  FailureRec(int objectInd, short objectType, short failureReason,
             boolean activeObject, int answerInd)
  {
    this.objectInd     = objectInd;
    this.objectType    = objectType;
    this.failureReason = failureReason;
    this.activeObject  = activeObject;
    this.answerInd     = answerInd;
  }

  //------------------------------------------------------------------------
  // equals/hashCode methods
  // equals() is needed to determine if a FailureRec already exists in 
  // a Vector
  //------------------------------------------------------------------------

  /**
   * Compare two FailureRec objects for eqality.
   * The FailureRec objects are equal if their objectType, objectInd,
   * activeObject and failureReason fields are all equal.
  **/
  public boolean equals(Object o) {
    boolean isEqual = false;

    if ( o instanceof FailureRec ) {
      FailureRec other = (FailureRec) o;
      if ( this.objectType    == other.objectType   &&
           this.objectInd     == other.objectInd    &&
           this.activeObject  == other.activeObject &&
           this.failureReason == other.failureReason )
      {
        isEqual = true;
      }
    }

    return isEqual;
  }

  /**
   * Returns the objectInd field as the hashCode for the FailureRec object.
  **/
  public int hashCode() {
    return objectInd;
  }

}
