
package com.ibm.nzna.projects.common.storedProc.oaValidate;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.oa.*;
import com.ibm.nzna.projects.common.storedProc.*;
import COM.ibm.db2.app.*;
import COM.ibm.db2.app.Blob;
import java.sql.*;
import java.util.*;

/**
 * DB2 Stored Procedure to validate Online Assistant objects before publish.
 * The stored procedure known to DB2 as QUEST.OAValidate.
 * 
 * @version 1.12
 * @author  Mark DeBiase
**/
public class OAValidate extends StoredProc {

  // objects types that can be validated
  public static final int ACTION    = 100;
  public static final int QUESTION  = 101;
  public static final int SYMPTOM   = 102;
  public static final int LINK      = 103;
  public static final int LINKGROUP = 104;

  // the DB2 connection
  private Connection conn = null;

  /**
   * Method invoked by DB2 when the stored procedure is called.
   *
   * @param objectInd       the indicator of the DRAFT object to be validated
   * @param objectType      the type of object to be validated
   * @param validationRec   returns processing status
  **/
  public void validate(int objectInd, int objectType, Blob validationRec)
              throws Exception
  {
    //-----------------------------------------------------------------------
    // Processing:
    // Validate the objectInd that was passed to the stored procedure.
    // Make the ValidationRec that is returned the "master" ValidationRec.
    // If the validation returned any coreqs, then push them onto a stack
    // (there are seperate stacks for action, question and symptom coreqs).
    // While there are items in the coreq stacks, pop the first one and
    // validate it, merging the returned ValidationRec into the master
    // ValidationRec and adding any new coreqs that are returned to the
    // appropriate stack.  When done, return the master ValidationRec.
    //-----------------------------------------------------------------------

    printDebug("Starting validation:");

    // master validation rec -- holds the results off all validations
    ValidationRec masterRec = null;

    // coreq stacks
    Stack actionCoreqStack   = new Stack();
    Stack questionCoreqStack = new Stack();
    Stack symptomCoreqStack  = new Stack();

    int i;

    try {
      if (!DEBUG) {
        conn = getConnection();
      }

      //---------------------------------------------------------------------
      // Validate the object that was passed in.
      //---------------------------------------------------------------------
      masterRec = validateObject(objectType, objectInd);

      //---------------------------------------------------------------------
      // Push any coreqs onto the appropriate stack.
      //---------------------------------------------------------------------
      for (i=0; i<masterRec.actionCoreqVec.size(); i++) {
        actionCoreqStack.push( masterRec.actionCoreqVec.elementAt(i) );
      }

      for (i=0; i<masterRec.questionCoreqVec.size(); i++) {
        questionCoreqStack.push( masterRec.questionCoreqVec.elementAt(i) );
      }

      for (i=0; i<masterRec.symptomCoreqVec.size(); i++) {
        symptomCoreqStack.push( masterRec.symptomCoreqVec.elementAt(i) );
      }

      //---------------------------------------------------------------------
      // Validate each of the objects on the coreq Stacks, merging the
      // resulting ValidationRec into the master ValidationRec and adding
      // any new coreqs to the appropriate stack.
      //---------------------------------------------------------------------
      while ( !actionCoreqStack.empty()   && 
              !questionCoreqStack.empty() &&
              !symptomCoreqStack.empty()  )
      {
        ValidationRec tempRec;
        if ( !actionCoreqStack.empty() ) {
          int draftActionInd = ((Integer) actionCoreqStack.pop()).intValue();
          tempRec = validateAction( draftActionInd );
        } else if ( !questionCoreqStack.empty() ) {
          int draftQuestionInd = ((Integer)questionCoreqStack.pop()).intValue();
          tempRec = validateQuestion( draftQuestionInd );
        } else {
          int draftSymptomInd = ((Integer) symptomCoreqStack.pop()).intValue();
          tempRec = validateSymptom( draftSymptomInd );
        }

        mergeValidationRec(masterRec, tempRec, 
                           actionCoreqStack,
                           questionCoreqStack,
                           symptomCoreqStack);
      }

      //---------------------------------------------------------------------
      // Finally, get the default title for each object listed in the
      // three coreq Vectors in the master ValidationRec object.
      //---------------------------------------------------------------------
      int vecSize;

      vecSize = masterRec.actionCoreqVec.size();
      if (vecSize > 0) {
        masterRec.actionCoreqTitles = new Vector( vecSize );
      }
      for (i=0; i<vecSize; i++) {
        int actionInd;
        actionInd = 
                 ((Integer) masterRec.actionCoreqVec.elementAt(i)).intValue();
        String title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        masterRec.actionCoreqTitles.addElement(title);
      }

      vecSize = masterRec.questionCoreqVec.size();
      if (vecSize > 0) {
        masterRec.questionCoreqTitles = new Vector( vecSize );
      }
      for (i=0; i<vecSize; i++) {
        int questionInd;
        questionInd = 
                ((Integer)masterRec.questionCoreqVec.elementAt(i)).intValue();
        String title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        masterRec.questionCoreqTitles.addElement(title);
      }

      vecSize = masterRec.symptomCoreqVec.size();
      if (vecSize > 0) {
        masterRec.symptomCoreqTitles = new Vector( vecSize );
      }
      for (i=0; i<vecSize; i++) {
        int symptomInd;
        symptomInd = 
                 ((Integer)masterRec.symptomCoreqVec.elementAt(i)).intValue();
        String title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
        masterRec.symptomCoreqTitles.addElement(title);
      }

    }
    catch(SQLException se) {
      if (masterRec == null) { masterRec = new ValidationRec(); }
      StoredProcUtil.writeException( se, masterRec );
      masterRec.sqlCode = se.getErrorCode();
      masterRec.error = true;
      conn.rollback();
      printDebug( masterRec.errorStr );
    }
    catch(Exception e) {
      if (masterRec == null) { masterRec = new ValidationRec(); }
      StoredProcUtil.writeException( e, masterRec );
      masterRec.error = true;
      conn.rollback();
      printDebug( masterRec.errorStr );
    }
    catch(Error err) {
      if (masterRec == null) { masterRec = new ValidationRec(); }
      StoredProcUtil.writeException( err, masterRec );
      masterRec.error = true;
      conn.rollback();
      printDebug( masterRec.errorStr );
    }
    finally {
      // close any statements that were prepared for reading titles
      closeTitleSelectStmts();
    }

    conn.commit();
    conn.close();

    // write out the return information (master ValidationRec)
    if (!DEBUG) {
      validationRec = Lob.newBlob();
      StoredProcUtil.setObjectInBlob( validationRec, masterRec );

      set( 3, validationRec );
    }

    printDebug("Validation complete!");
  }

  /**
   * Merge a ValidationRec into the master ValidationRec and populate
   * the coreq Stacks with new coreqs.
   * Any coreqs that appear in the ValidationRec that is being merged
   * into the master and which do not already exist in the master are
   * added to the appropriate coreq Stack.
   *
   * @param masterRec          the master ValidationRec
   * @param anotherRec         the ValidationRec to be merged into the master
   * @param actionCoreqStack   the action coreq stack
   * @param questionCoreqStack the question coreq stack
   * @param symptomCoreqStack  the symptom coreq stack
  **/
  private void mergeValidationRec(ValidationRec masterRec, 
                                  ValidationRec anotherRec,
                                  Stack actionCoreqStack,
                                  Stack questionCoreqStack,
                                  Stack symptomCoreqStack)
  {
    int i;

    //-----------------------------------------------------------------------
    // set the publishable flag on the master ValidationRec:
    //-----------------------------------------------------------------------
    masterRec.publishable = masterRec.publishable && anotherRec.publishable;

    //-----------------------------------------------------------------------
    // merge the UsageRecs into the master ValidationRec:
    //-----------------------------------------------------------------------
    for (i=0; i<anotherRec.usageVec.size(); i++) {
      UsageRec anotherUR = (UsageRec) anotherRec.usageVec.elementAt(i);
      int masterIndex = masterRec.usageVec.indexOf( anotherUR );
      if ( masterIndex == -1 ) {
        // was not found in master... add it now
        masterRec.usageVec.addElement( anotherUR );
      } else {
        // was found in master... make sure isError is set appropriately
        UsageRec masterUR = (UsageRec)masterRec.usageVec.elementAt(masterIndex);
        masterUR.isError = masterUR.isError && anotherUR.isError;
      }
    }

    //-----------------------------------------------------------------------
    // merge the FailureRecs into the master FailureRec
    //-----------------------------------------------------------------------
    for (i=0; i<anotherRec.failureVec.size(); i++) {
      FailureRec anotherFR = (FailureRec) anotherRec.failureVec.elementAt(i);
      int masterIndex = masterRec.failureVec.indexOf( anotherFR );
      if ( masterIndex == -1 ) {
        // was not found in master... add it now
        masterRec.failureVec.addElement( anotherFR );
      }
    }

    //-----------------------------------------------------------------------
    // merge the coreqVecs into the master, adding new entries to
    // the appropriate Stack
    //-----------------------------------------------------------------------
    for (i=0; i<anotherRec.actionCoreqVec.size(); i++) {
      Integer I = (Integer) anotherRec.actionCoreqVec.elementAt(i);
      int masterIndex = masterRec.actionCoreqVec.indexOf( I );
      if (masterIndex == -1 ) {
        // not found in master... add it to coreqVec and Stack now
        masterRec.actionCoreqVec.addElement( I );
        actionCoreqStack.push( I );
      }
    }
    for (i=0; i<anotherRec.questionCoreqVec.size(); i++) {
      Integer I = (Integer) anotherRec.questionCoreqVec.elementAt(i);
      int masterIndex = masterRec.questionCoreqVec.indexOf( I );
      if (masterIndex == -1) {
        // not found in master... add it to coreqVec and Stack now
        masterRec.questionCoreqVec.addElement( I );
        questionCoreqStack.push( I );
      }
    }
    for (i=0; i<anotherRec.symptomCoreqVec.size(); i++) {
      Integer I = (Integer) anotherRec.symptomCoreqVec.elementAt(i);
      int masterIndex = masterRec.symptomCoreqVec.indexOf( I );
      if (masterIndex == -1) {
        // not found in master... add it to coreqVec and Stack now
        masterRec.symptomCoreqVec.addElement( I );
        symptomCoreqStack.push( I );
      }
    }
  }

  /**
   * Call the appropriate validateXxx() method based on objectType.
  **/
  private ValidationRec validateObject(int objectType, int objectInd)
                        throws Exception
  {
    ValidationRec retRec;

    switch (objectType) {
      case ACTION:
        retRec = validateAction( objectInd );
        break;

      case QUESTION:
        retRec = validateQuestion( objectInd );
        break;

      case SYMPTOM:
        retRec = validateSymptom( objectInd );
        break;

      case LINK:
        retRec = validateLink( objectInd );
        break;

      case LINKGROUP:
        retRec = validateLinkGroup( objectInd );
        break;

      default:
        throw new Exception("Invalid objectType " + objectType);
    }

    return retRec;
  }


  /**
   * Validate a draft Action object for publishing.
  **/
  private ValidationRec validateAction(int draftActionInd) throws SQLException
  {
    ValidationRec vRec = new ValidationRec();
    Statement stmt = conn.createStatement();
    ResultSet rs;

    try {

      boolean actionPublishable = true;

      //---------------------------------------------------------------------
      // Get the active action indicator & archived flag from the 
      // draft object.
      //---------------------------------------------------------------------
      printDebug("Getting activeActionInd for " + draftActionInd + "...");
      rs = stmt.executeQuery(
           "SELECT activeActionInd, archived, maintLock, dbUser " +
           "FROM oadraft.action " +
           "WHERE actionInd=" + draftActionInd + " " +
           "FOR FETCH ONLY"
      );

      //---------------------------------------------------------------------
      // If draft object was not found, return with error.
      //---------------------------------------------------------------------
      if ( !rs.next() ) {
        rs.close();
        printDebug("Indicator not found!!");
        vRec.error = true;
        vRec.errorStr = "Indicator " + draftActionInd + " was not found";
        return vRec;
      }

      int     activeActionInd = rs.getInt(1);
      boolean archived        = DatabaseUtil.booleanFromDB( rs.getString(2) );
      boolean maintLock       = DatabaseUtil.booleanFromDB( rs.getString(3) );

      //---------------------------------------------------------------------
      // Make sure maintLock is set to false -- validation fails if the
      // object is open for editing
      //---------------------------------------------------------------------
      if ( maintLock ) {
        printDebug("MAINTLOCK is set");
        actionPublishable = false;
        FailureRec fr;
        fr = new FailureRec(draftActionInd, FailureRec.TYPE_ACTION,
                            FailureRec.FAILURE_MAINTLOCK_SET, false);
        fr.dbUser = rs.getString(4).trim();
        vRec.failureVec.addElement(fr);
      }
      rs.close();

      //*********************************************************************
      // Check for usage of the Action in Symptom Nodes and Preanswers (for
      // EdgeAlist entries) for both active and open draft Symptoms.
      // If the action is being archived, then these occurences are flagged
      // as errors (ValidationRec.publishable = false).
      // If the action is not being archived, then these occurrences are not
      // flagged as errors (ValidationRec.publishable = true).  They are 
      // still returned in the ValidationRec so the user can see where the
      // Action is being used.
      //*********************************************************************

      //---------------------------------------------------------------------
      // Check active Symptom Nodes (only if the Action was previously 
      // published):
      //---------------------------------------------------------------------
      if ( activeActionInd != 0 ) {
        printDebug("Checking active symptom nodes...");
        rs = stmt.executeQuery(
             "SELECT DISTINCT symptomInd " +
             "FROM oa.node " +
             "WHERE actionInd=" + activeActionInd + " " +
             "AND nodeType=" + Node.NODE_TYPE_ACTION + " " +
             "ORDER BY symptomInd " +
             "FOR FETCH ONLY"
        );
        while ( rs.next() ) {
          if (archived) {
            actionPublishable = false;
          }
          int symptomInd = rs.getInt(1);

          UsageRec ur = new UsageRec( symptomInd, UsageRec.TYPE_SYMPTOM,
                                      UsageRec.USAGE_SYMPTOM_NODE,
                                      true, archived );
          ur.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
          vRec.usageVec.addElement( ur );
        }
        rs.close();
      }

      //---------------------------------------------------------------------
      // Check open draft Symptom Nodes:
      //---------------------------------------------------------------------
      printDebug("Checking open draft symptom nodes...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM oadraft.node n, oadraft.symptom s " +
           "WHERE (" +
                      "n.activeActionInd=" + activeActionInd + 
                      " OR " +
                      "n.draftActionInd=" + draftActionInd + 
                 ")" +
           "AND n.nodeType=" + Node.NODE_TYPE_ACTION + " " +
           "AND n.symptomInd=s.symptomInd " +
           "AND s.stopDate IS NULL " +
           "ORDER BY symptomInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          actionPublishable = false;
        }
        int draftSymptomInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftSymptomInd, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_SYMPTOM_NODE,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.usageVec.addElement( ur );

        // MAD 6/1 -- also add object to co-requisites
        vRec.symptomCoreqVec.addElement( new Integer( draftSymptomInd ) );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check active Symptom Preanswers for EdgeAlists (only if the Action
      // was previously published):
      //---------------------------------------------------------------------
      if ( activeActionInd != 0 ) {
        printDebug("Checking active symptom preanswers...");
        rs = stmt.executeQuery(
             "SELECT DISTINCT g.symptomInd " +
             "FROM oa.edgePreanswer p, oa.edge e, oa.edgeGroup g " +
             "WHERE p.actionInd=" + activeActionInd + " " +
             "AND p.preanswerType=" + EdgeObject.EDGE_ALIST + " " +
             "AND e.edgeGroupInd=g.edgeGroupInd " + 
             "ORDER BY symptomInd "+
             "FOR FETCH ONLY"
        );
        while ( rs.next() ) {
          if (archived) {
            actionPublishable = false;
          }
          int symptomInd = rs.getInt(1);

          UsageRec ur = new UsageRec( symptomInd, UsageRec.TYPE_SYMPTOM,
                                      UsageRec.USAGE_EDGE_ALIST,
                                      true, archived );
          ur.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
          vRec.usageVec.addElement( ur );
        }
        rs.close();
      }

      //---------------------------------------------------------------------
      // Check open draft Symptom Preanswers for EdgeAlists:
      //---------------------------------------------------------------------
      printDebug("Checking open draft symptom preanswers...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM oadraft.edgePreanswer p, oadraft.edge e, " +
                "oadraft.edgeGroup g, oadraft.symptom s " +
           "WHERE (" +
                      "p.activeActionInd=" + activeActionInd +
                      " OR " +
                      "p.draftActionInd=" + draftActionInd +
                 ")" +
           "AND p.preanswerType=" + EdgeObject.EDGE_ALIST + " " +
           "AND e.edgeGroupInd=g.edgeGroupInd " + 
           "AND s.symptomInd=g.symptomInd " +
           "AND s.stopDate IS NULL " +
           "ORDER BY symptomInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          actionPublishable = false;
        }
        int draftSymptomInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftSymptomInd, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_EDGE_ALIST,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.usageVec.addElement( ur );

        // MAD 6/1 -- also add object to co-requisites
        vRec.symptomCoreqVec.addElement( new Integer( draftSymptomInd ) );
      }
      rs.close();

      vRec.publishable = actionPublishable;

    }
    finally { 
      stmt.close();
    }
   
    return vRec;
  }

  /**
   * Validate a draft Question object for publishing.
  **/
  private ValidationRec validateQuestion(int draftQuestionInd)
                        throws SQLException
  {
    ValidationRec vRec = new ValidationRec();
    Statement stmt = conn.createStatement();
    ResultSet rs;

    try {

      boolean questionPublishable = true;

      //---------------------------------------------------------------------
      // Get the active question indicator & archived flag from the 
      // draft object.
      //---------------------------------------------------------------------
      printDebug("Getting activeQuestionInd for " + draftQuestionInd + "...");
      rs = stmt.executeQuery(
           "SELECT activeQuestionInd, archived, maintLock, dbUser " +
           "FROM oadraft.question " +
           "WHERE questionInd=" + draftQuestionInd + " " +
           "FOR FETCH ONLY"
      );

      //---------------------------------------------------------------------
      // If draft object was not found, return with error.
      //---------------------------------------------------------------------
      if ( !rs.next() ) {
        rs.close();
        printDebug("Indicator not found!!");
        vRec.error = true;
        vRec.errorStr = "Indicator " + draftQuestionInd + " was not found";
        return vRec;
      }

      int     activeQuestionInd = rs.getInt(1);
      boolean archived          = DatabaseUtil.booleanFromDB( rs.getString(2) );
      boolean maintLock         = DatabaseUtil.booleanFromDB( rs.getString(3) );

      //---------------------------------------------------------------------
      // Make sure maintLock is set to false -- validation fails if the
      // object is open for editing
      //---------------------------------------------------------------------
      if ( maintLock ) {
        printDebug("MAINTLOCK is set");
        questionPublishable = false;
        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.FAILURE_MAINTLOCK_SET, false);
        fr.dbUser = rs.getString(4).trim();
        vRec.failureVec.addElement(fr);
      }
      rs.close();

      //*********************************************************************
      // Check for usage of the Question in Symptom Nodes and Preanswers (for
      // EdgeUnanswer and EdgePreanswer entries) for both active and open 
      // draft Symptoms.  Also check for usage of the Question in Conditions.
      // If the question is being archived, then these occurences are flagged
      // as errors (ValidationRec.publishable = false).
      // If the question is not being archived, then these occurrences are not
      // flagged as errors (ValidationRec.publishable = true).  They are 
      // still returned in the ValidationRec so the user can see where the
      // Question is being used.
      //*********************************************************************

      //---------------------------------------------------------------------
      // Check active Symptom Nodes (only if the Question was previously 
      // published):
      //---------------------------------------------------------------------
      if ( activeQuestionInd != 0 ) {
        printDebug("Checking active symptom nodes...");
        rs = stmt.executeQuery(
             "SELECT DISTINCT symptomInd " +
             "FROM oa.node " +
             "WHERE questionInd=" + activeQuestionInd + " " +
             "AND nodeType=" + Node.NODE_TYPE_QUESTION + " " +
             "ORDER BY symptomInd " +
             "FOR FETCH ONLY"
        );
        while ( rs.next() ) {
          if (archived) {
            questionPublishable = false;
          }
          int symptomInd = rs.getInt(1);

          UsageRec ur = new UsageRec( symptomInd, UsageRec.TYPE_SYMPTOM,
                                      UsageRec.USAGE_SYMPTOM_NODE,
                                      true, archived );
          ur.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
          vRec.usageVec.addElement( ur );
        }
        rs.close();
      }

      //---------------------------------------------------------------------
      // Check open draft Symptom Nodes:
      //---------------------------------------------------------------------
      printDebug("Checking open draft symptom nodes...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM oadraft.node n, oadraft.symptom s " +
           "WHERE (" +
                      "n.activeQuestionInd=" + activeQuestionInd +
                      " OR " +
                      "n.draftQuestionInd=" + draftQuestionInd +
                 ")" +
           "AND n.nodeType=" + Node.NODE_TYPE_QUESTION + " " +
           "AND n.symptomInd=s.symptomInd " +
           "AND s.stopDate IS NULL " +
           "ORDER BY symptomInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftSymptomInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftSymptomInd, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_SYMPTOM_NODE,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.usageVec.addElement( ur );

        // MAD 6/1 -- also add object to co-requisites
        vRec.symptomCoreqVec.addElement( new Integer( draftSymptomInd ) );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check active Symptom Preanswers for EdgeUnanswers (only if the 
      // Question was previously published):
      //---------------------------------------------------------------------
      if ( activeQuestionInd != 0 ) {
        printDebug("Checking active symptom edgeunanswers...");
        rs = stmt.executeQuery(
             "SELECT DISTINCT g.symptomInd " +
             "FROM oa.edgePreanswer p, oa.edge e, oa.edgeGroup g " +
             "WHERE p.questionInd=" + activeQuestionInd + " " +
             "AND p.preanswerType=" + EdgeObject.EDGE_UNANSWER + " " +
             "AND e.edgeGroupInd=g.edgeGroupInd " + 
             "ORDER BY symptomInd "+
             "FOR FETCH ONLY"
        );
        while ( rs.next() ) {
          if (archived) {
            questionPublishable = false;
          }
          int symptomInd = rs.getInt(1);

          UsageRec ur = new UsageRec( symptomInd, UsageRec.TYPE_SYMPTOM,
                                      UsageRec.USAGE_EDGE_UNANSWER,
                                      true, archived );
          ur.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
          vRec.usageVec.addElement( ur );
        }
        rs.close();
      }

      //---------------------------------------------------------------------
      // Check open draft Symptom Preanswers for EdgeUnanswers:
      //---------------------------------------------------------------------
      printDebug("Checking open draft symptom edgeunanswers...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM oadraft.edgePreanswer p, oadraft.edge e, " +
                "oadraft.edgeGroup g, oadraft.symptom s " +
           "WHERE (" +
                      "p.activeQuestionInd=" + activeQuestionInd +
                      " OR " +
                      "p.draftQuestionInd=" + draftQuestionInd +
                 ")" +
           "AND p.preanswerType=" + EdgeObject.EDGE_UNANSWER + " " +
           "AND e.edgeGroupInd=g.edgeGroupInd " + 
           "AND s.symptomInd=g.symptomInd " +
           "AND s.stopDate IS NULL " +
           "ORDER BY symptomInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftSymptomInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftSymptomInd, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_EDGE_UNANSWER,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.usageVec.addElement( ur );

        // MAD 6/1 -- also add object to co-requisites
        vRec.symptomCoreqVec.addElement( new Integer( draftSymptomInd ) );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check active Symptom Preanswers for EdgePreanswers that use one
      // of the Question's Answers (only if the Question was previously 
      // published):
      //---------------------------------------------------------------------
      if ( activeQuestionInd != 0 ) {
        printDebug("Checking active symptom edgepreanswers...");
        rs = stmt.executeQuery(
             "SELECT DISTINCT g.symptomInd " +
             "FROM oa.edgePreanswer p, oa.edge e, oa.edgeGroup g, " +
                  "oa.question q, oa.answer a " +
             "WHERE q.questionInd=" + activeQuestionInd + " " +
             "AND a.questionInd=q.questionInd " +
             "AND p.answerInd=a.answerInd " +
             "AND p.preanswerType=" + EdgeObject.EDGE_PREANSWER + " " +
             "AND e.edgeGroupInd=g.edgeGroupInd " + 
             "ORDER BY symptomInd "+
             "FOR FETCH ONLY"
        );
        while ( rs.next() ) {
          if (archived) {
            questionPublishable = false;
          }
          int symptomInd = rs.getInt(1);

          UsageRec ur = new UsageRec( symptomInd, UsageRec.TYPE_SYMPTOM,
                                      UsageRec.USAGE_EDGE_PREANSWER,
                                      true, archived );
          ur.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
          vRec.usageVec.addElement( ur );
        }
        rs.close();
      }

      //---------------------------------------------------------------------
      // Check open draft Symptom Preanswers for EdgePreanswers that use
      // one of the Question's Answers:
      //---------------------------------------------------------------------
      printDebug("Checking open draft symptom edgepreanswers...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT sd.symptomInd " +
           "FROM oadraft.edgePreanswer pd, oadraft.edge ed, " +
                "oadraft.edgeGroup gd, oadraft.symptom sd, " +
                "oadraft.question qd, oadraft.answer ad, " +
                "oa.question q, oa.answer a " +
           "WHERE (" +
                       "(" +
                           "q.questionInd=" + activeQuestionInd + " " +
                           "AND a.questionInd=q.questionInd " +
                           "AND pd.activeAnswerInd=a.answerInd" +
                       ")" +
                  " OR " +
                       "(" +
                           "qd.questionInd=" + draftQuestionInd + " " +
                           "AND ad.questionInd=qd.questionInd " +
                           "AND pd.draftAnswerInd=ad.answerInd" +
                       ")" +
                 ")" +
           "AND pd.preanswerType=" + EdgeObject.EDGE_UNANSWER + " " +
           "AND ed.edgeGroupInd=gd.edgeGroupInd " + 
           "AND sd.symptomInd=gd.symptomInd " +
           "AND sd.stopDate IS NULL " +
           "ORDER BY symptomInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftSymptomInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftSymptomInd, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_EDGE_PREANSWER,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.usageVec.addElement( ur );

        // MAD 6/1 -- also add object to co-requisites
        vRec.symptomCoreqVec.addElement( new Integer( draftSymptomInd ) );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check for usage of the Question in the condition tables (both
      // active and draft).  Update publishable status accordingly.
      //---------------------------------------------------------------------
      if (activeQuestionInd != 0) {
        questionPublishable = questionPublishable &&
                              checkConditionTablesForQuestion(activeQuestionInd,
                                                              archived, vRec);
      }

      //*********************************************************************
      // Check for failures publishing this Question.
      // Failures are:
      // 1. Any Symptom which uses this Question in a node and does
      //    not have an open draft.
      // 2. Answers which were deleted from the draft Question but are
      //    still used in active objects or open drafts.
      //*********************************************************************


      //---------------------------------------------------------------------
      // 1. Check for Symptoms that use this Question and do not have 
      //    an open draft.
      //---------------------------------------------------------------------
      if (activeQuestionInd != 0) {
        printDebug("Checking for symptoms without an open draft...");
        rs = stmt.executeQuery(
             "SELECT DISTINCT symptomInd " +
             "FROM oa.node " +
             "WHERE questionInd=" + activeQuestionInd + " " +
             "AND nodeType=" + Node.NODE_TYPE_QUESTION + " " +
             "AND symptomInd NOT IN (" +
                "SELECT DISTINCT s.activeSymptomInd " + 
                "FROM oadraft.node n, oadraft.symptom s " +
                "WHERE n.activeQuestionInd=" + activeQuestionInd + " " +
                "AND n.nodeType=" + Node.NODE_TYPE_QUESTION + " " +
                "AND n.symptomInd=s.symptomInd " +
                "AND s.stopDate IS NULL " +
                "AND s.activeSymptomInd IS NOT NULL" +
             ")" +
             "ORDER BY symptomInd "+
             "FOR FETCH ONLY"
        );
        while ( rs.next() ) {
          //if (archived) {
            questionPublishable = false;
          //}
          int symptomInd = rs.getInt(1);

          FailureRec fr;
          fr = new FailureRec(symptomInd, FailureRec.TYPE_SYMPTOM,
                              FailureRec.QFAILURE_NO_OPEN_DRAFT_SYMPTOM, false);
          fr.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
          vRec.failureVec.addElement( fr );
        }
        rs.close();
      }

      //---------------------------------------------------------------------
      // 2. Check for Answers which were deleted from the Question draft
      //    but are still used in active objects or open drafts.
      //---------------------------------------------------------------------
      if (activeQuestionInd != 0) {
        questionPublishable = questionPublishable && 
                              checkDeletedAnswers( activeQuestionInd, 
                                                   draftQuestionInd, vRec );
      }

      vRec.publishable = questionPublishable;

    }
    finally { 
      stmt.close();
    }
   
    return vRec;
  }

  /**
   * Validate a draft Symptom object for publishing.
  **/
  private ValidationRec validateSymptom(int draftSymptomInd) 
                        throws SQLException
  {
    ValidationRec vRec = new ValidationRec();
    Statement stmt = conn.createStatement();
    ResultSet rs;

    try {

      boolean symptomPublishable = true;

      //---------------------------------------------------------------------
      // Get the active symptom indicator & archived flag from the 
      // draft object.
      //---------------------------------------------------------------------
      printDebug("Getting activeSymptomInd for " + draftSymptomInd + "...");
      rs = stmt.executeQuery(
           "SELECT activeSymptomInd, archived, maintLock, dbUser " +
           "FROM oadraft.symptom " +
           "WHERE symptomInd=" + draftSymptomInd + " " +
           "FOR FETCH ONLY"
      );

      //---------------------------------------------------------------------
      // If draft object was not found, return with error.
      //---------------------------------------------------------------------
      if ( !rs.next() ) {
        rs.close();
        printDebug("Indicator not found!!");
        vRec.error = true;
        vRec.errorStr = "Indicator " + draftSymptomInd + " was not found";
        return vRec;
      }

      int     activeSymptomInd = rs.getInt(1);
      boolean archived         = DatabaseUtil.booleanFromDB( rs.getString(2) );
      boolean maintLock        = DatabaseUtil.booleanFromDB( rs.getString(3) );

      //---------------------------------------------------------------------
      // Make sure maintLock is set to false -- validation fails if the
      // object is open for editing
      //---------------------------------------------------------------------
      if ( maintLock ) {
        printDebug("MAINTLOCK is set");
        symptomPublishable = false;
        FailureRec fr;
        fr = new FailureRec(draftSymptomInd, FailureRec.TYPE_SYMPTOM,
                            FailureRec.FAILURE_MAINTLOCK_SET, false);
        fr.dbUser = rs.getString(4).trim();
        vRec.failureVec.addElement(fr);
      }
      rs.close();

      //*********************************************************************
      // Check for usage of the Symptom in Symptom Nodes for both active 
      // and open draft Symptoms.
      // If the symptom is being archived, then these occurences are flagged
      // as errors (ValidationRec.publishable = false).
      // If the symptom is not being archived, then these occurrences are not
      // flagged as errors (ValidationRec.publishable = true).  They are 
      // still returned in the ValidationRec so the user can see where the
      // Symptom is being used.
      //*********************************************************************

      //---------------------------------------------------------------------
      // Check active Symptom Nodes (only if the Symptom was previously 
      // published):
      //---------------------------------------------------------------------
      if ( activeSymptomInd != 0 ) {
        printDebug("Checking active symptom nodes...");
        rs = stmt.executeQuery(
             "SELECT DISTINCT symptomInd " +
             "FROM oa.node " +
             "WHERE symptomInd=" + activeSymptomInd + " " +
             "AND nodeType=" + Node.NODE_TYPE_SYMPTOM + " " +
             "ORDER BY symptomInd " +
             "FOR FETCH ONLY"
        );
        while ( rs.next() ) {
          if (archived) {
            symptomPublishable = false;
          }
          int symptomInd = rs.getInt(1);

          UsageRec ur = new UsageRec( symptomInd, UsageRec.TYPE_SYMPTOM,
                                      UsageRec.USAGE_SYMPTOM_NODE,
                                      true, archived );
          ur.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
          vRec.usageVec.addElement( ur );
        }
        rs.close();
      }

      //---------------------------------------------------------------------
      // Check open draft Symptom Nodes:
      //---------------------------------------------------------------------
      printDebug("Checking open draft symptom nodes...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM oadraft.node n, oadraft.symptom s " +
           "WHERE (" +
                      "n.activeSymptomInd=" + activeSymptomInd +
                      " OR " +
                      "n.draftSymptomInd=" + draftSymptomInd +
                 ")" +
           "AND n.nodeType=" + Node.NODE_TYPE_SYMPTOM + " " +
           "AND n.symptomInd=s.symptomInd " +
           "AND s.stopDate IS NULL " +
           "ORDER BY s.symptomInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          symptomPublishable = false;
        }
        int draftSymptomInd2 = rs.getInt(1);

        UsageRec ur = new UsageRec( draftSymptomInd2, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_SYMPTOM_NODE,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd2);
        vRec.usageVec.addElement( ur );

        // MAD 6/1 -- also add object to co-requisites
        vRec.symptomCoreqVec.addElement( new Integer( draftSymptomInd2 ) );
      }
      rs.close();

      //*********************************************************************
      // MAD 5/29 - these 6 failures have been changed to co-requisites
      //            that are reported in the xxxCoreqVec fields of the 
      //            ValidationRec instead of in FailureRecs
      // MAD 6/1  - changed the criteria on co-requisites 2-6 from
      //            "was never published" to "has an open draft".
      //
      // Check for failures publishing this Symptom.
      // Failures are:
      // 1. Any node for a Question which has an open draft.
      // 2. Any EdgeUnanswer for a Question which has an open draft.
      // 3. Any node for an Action which has an open draft.            
      // 4. Any EdgeAlist for an Action which has an open draft.
      // 5. Any Symptom used in this Symptom has an open draft.
      // 6. Any EdgePreanswer for an Answer has an open draft.
      //
      // NOTES:
      // - We don't specifically check for Edges with unpublished Answers
      //   because they would fall under the category of #1.
      // - For #1, all open drafts of Questions used in the Symptom
      //   must be closed before the Symptom can be publisehd.  This is 
      //   required because a change to a Question can require a change to
      //   a Symptom (specifically, if an answer is added/deleted).
      // - For #2-#6, it doesn't matter if there is an open draft of
      //   the object -- it is only an error if the object was never
      //   published. MAD 6/1: if there is an open draft, it is a coreq.
      //*********************************************************************

      //---------------------------------------------------------------------
      // 1. Check for Question nodes which have an open draft:
      //---------------------------------------------------------------------
      printDebug("Checking for question nodes with an open draft...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT qd.questionInd " +
           "FROM oadraft.node n, oadraft.question qd " +
           "WHERE n.symptomInd=" + draftSymptomInd + " " +
           "AND n.nodeType=" + Node.NODE_TYPE_QUESTION + " " +
           "AND (" +
                   "(n.activeQuestionInd IS NOT NULL AND" +
                   " qd.activeQuestionInd=n.activeQuestionInd)" +
                   " OR " +
                   "(n.draftQuestionInd IS NOT NULL AND" +
                   " qd.questionInd=n.draftQuestionInd)" +
               ")" +
           "AND qd.stopDate IS NULL " +
           "ORDER BY qd.questionInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        vRec.questionCoreqVec.addElement( new Integer(rs.getInt(1)) );

/* MAD 5/29 - deprecated:
        symptomPublishable = false;
        int draftQuestionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.SFAILURE_OPEN_DRAFT_QUESTION, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.failureVec.addElement( fr );
*/
      }
      rs.close();

      //---------------------------------------------------------------------
      // 2. Check for EdgeUnanswers for Questions which have an open draft:
      //---------------------------------------------------------------------
      printDebug("Checking for edgeunanswer for unpublised questions...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT q.questionInd " +
           "FROM oadraft.edgePreanswer p, oadraft.edge e, " +
                "oadraft.edgeGroup g, oadraft.question q " +
           "WHERE p.preanswerType=" + EdgeObject.EDGE_UNANSWER + " " +
           "AND (" +
                   "(p.activeQuestionInd IS NOT NULL AND" +
                   " q.activeQuestionInd=p.activeQuestionInd)" +
                   " OR " +
                   "(p.draftQuestionInd IS NOT NULL AND" +
                   " q.questionInd=p.draftQuestionInd)" +
               ")" +
           "AND q.stopDate IS NULL " + 
           "AND p.edgeInd=e.edgeInd " +
           "AND e.edgeGroupInd=g.edgeGroupInd " +
           "AND g.symptomInd=" + draftSymptomInd + " " +
           "ORDER BY q.questionInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        vRec.questionCoreqVec.addElement( new Integer(rs.getInt(1)) );

/* MAD 5/29 - deprecated:
        symptomPublishable = false;
        int draftQuestionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.SFAILURE_UNPUBLISHED_UNANSWER, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.failureVec.addElement( fr );
*/
      }
      rs.close();

      //---------------------------------------------------------------------
      // 3. Check for Action nodes which have an open draft:
      //---------------------------------------------------------------------
      printDebug("Checking for action nodes which were never published...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT a.actionInd " +
           "FROM oadraft.node n, oadraft.action a " +
           "WHERE n.symptomInd=" + draftSymptomInd + " " +
           "AND n.nodeType=" + Node.NODE_TYPE_ACTION + " " +
           "AND (" +
                   "(n.activeActionInd IS NOT NULL AND" +
                   " a.activeActionInd=n.activeActionInd)" +
                   " OR " +
                   "(n.draftActionInd IS NOT NULL AND" +
                   " a.actionInd=n.draftActionInd)" +
               ")" +
           "AND a.stopDate IS NULL " +
           "ORDER BY a.actionInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        vRec.actionCoreqVec.addElement( new Integer(rs.getInt(1)) );

/* MAD 5/29 - deprecated:
        symptomPublishable = false;
        int draftActionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftActionInd, FailureRec.TYPE_ACTION,
                            FailureRec.SFAILURE_UNPUBLISHED_NODE, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.failureVec.addElement( fr );
*/
      }
      rs.close();

      //---------------------------------------------------------------------
      // 4. Check for EdgeAlists for Actions which have an open draft:
      //---------------------------------------------------------------------
      printDebug("Checking for edgealist for unpublished actions...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT a.actionInd " +
           "FROM oadraft.edgePreanswer p, oadraft.edge e, " +
                "oadraft.edgeGroup g, oadraft.action a " +
           "WHERE p.preanswerType=" + EdgeObject.EDGE_ALIST + " " +
           "AND (" +
                   "(p.activeActionInd IS NOT NULL AND" +
                   " a.activeActionInd=p.activeActionInd)" +
                   " OR " +
                   "(p.draftActionInd IS NOT NULL AND" +
                   " a.actionInd=p.draftActionInd)" +
               ")" +
           "AND a.stopDate IS NULL " +
           "AND p.edgeInd=e.edgeInd " +
           "AND e.edgeGroupInd=g.edgeGroupInd " +
           "AND g.symptomInd=" + draftSymptomInd + " " +
           "ORDER BY a.actionInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        vRec.actionCoreqVec.addElement( new Integer(rs.getInt(1)) );

/* MAD 5/29 - deprecated:
        symptomPublishable = false;
        int draftActionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftActionInd, FailureRec.TYPE_ACTION,
                            FailureRec.SFAILURE_UNPUBLISHED_ALIST, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.failureVec.addElement( fr );
*/
      }
      rs.close();

      //---------------------------------------------------------------------
      // 5. Check for Symptom nodes which have an open draft:
      //---------------------------------------------------------------------
      printDebug("Checking for symptom nodes which were never published...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM oadraft.node n, oadraft.symptom s " +
           "WHERE n.symptomInd=" + draftSymptomInd + " " +
           "AND n.nodeType=" + Node.NODE_TYPE_SYMPTOM + " " +
           "AND (" +
                   "(n.activeSymptomInd IS NOT NULL AND" +
                   " s.activeSymptomInd=n.activeSymptomInd)" +
                   " OR " +
                   "(n.draftSymptomInd IS NOT NULL AND" +
                   " s.symptomInd=n.draftSymptomInd)" +
               ")" +
           "AND s.stopDate IS NULL " +
           "ORDER BY s.symptomInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        vRec.symptomCoreqVec.addElement( new Integer(rs.getInt(1)) );

/* MAD 5/29 - deprecated:
        symptomPublishable = false;
        int draftSymptomInd2 = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftSymptomInd2, FailureRec.TYPE_SYMPTOM,
                            FailureRec.SFAILURE_UNPUBLISHED_NODE, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd2);
        vRec.failureVec.addElement( fr );
*/
      }
      rs.close();

      //---------------------------------------------------------------------
      // 6. Check for EdgePreanswers for Answers in open draft Questions:
      //---------------------------------------------------------------------
      printDebug("Checking for edgepreanswer for unpublished answers...");
      rs = stmt.executeQuery(
           "SELECT DISTINCT a.questionInd, a.answerInd " +
           "FROM oadraft.edgePreanswer p, oadraft.edge e, " +
                "oadraft.edgeGroup g, " +
                "oadraft.answer a, oadraft.question q " +
           "WHERE p.preanswerType=" + EdgeObject.EDGE_PREANSWER + " " +
           "AND (" +
                   "(p.activeAnswerInd IS NOT NULL AND" +
                   " a.activeAnswerInd=p.activeAnswerInd)" +
                   " OR " +
                   "(p.draftAnswerInd IS NOT NULL AND" +
                   " a.answerInd=p.draftAnswerInd)" +
               ")" +
           "AND a.questionInd=q.questionInd " +
           "AND q.stopDate IS NULL " + 
           "AND p.edgeInd=e.edgeInd " +
           "AND e.edgeGroupInd=g.edgeGroupInd " +
           "AND g.symptomInd=" + draftSymptomInd + " " +
           "ORDER BY a.questionInd " +
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        vRec.questionCoreqVec.addElement( new Integer(rs.getInt(1)) );

/* MAD 5/29 - deprecated:
        symptomPublishable = false;
        int draftQuestionInd = rs.getInt(1);
        int draftAnswerInd   = rs.getInt(2);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.SFAILURE_UNPUBLISHED_PREANSWER,
                            false, draftAnswerInd);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        fr.answerTitle = readDefaultTitle(SELECT_DRAFT_ANSWER_TITLE,
                                    draftAnswerInd);
        vRec.failureVec.addElement( fr );
*/
      }
      rs.close();

      vRec.publishable = symptomPublishable;
    }
    finally { 
      stmt.close();
    }
   
    return vRec;
  }

  /**
   * Validate a draft Link object for publishing.
   * If the Link was never published or it is not being archived, it
   * is publishable. Check the DocLinks in Actions and Questions and
   * the Links in LinkGroups for a reference to the Link (both active
   * object and open drafts will be checked).
  **/
  private ValidationRec validateLink(int draftLinkInd) throws SQLException {

    ValidationRec vRec = new ValidationRec();
    Statement stmt = conn.createStatement();
    ResultSet rs;

    try {

      boolean linkPublishable = true;

      //---------------------------------------------------------------------
      // Get the active link indicator & archived flag from the draft object.
      //---------------------------------------------------------------------
      printDebug("Getting activeLinkInd for " + draftLinkInd + "...");
      rs = stmt.executeQuery(
           "SELECT activeLinkInd, archived, maintLock, dbUser " +
           "FROM oadraft.link " +
           "WHERE linkInd=" + draftLinkInd + " " +
           "FOR FETCH ONLY"
      );

      //---------------------------------------------------------------------
      // If draft object was not found, return with error.
      //---------------------------------------------------------------------
      if ( !rs.next() ) {
        rs.close();
        printDebug("Indicator not found!!");
        vRec.error = true;
        vRec.errorStr = "Indicator " + draftLinkInd + " was not found";
        return vRec;
      }

      int     activeLinkInd = rs.getInt(1);
      boolean archived      = DatabaseUtil.booleanFromDB( rs.getString(2) );
      boolean maintLock     = DatabaseUtil.booleanFromDB( rs.getString(3) );

      //---------------------------------------------------------------------
      // Make sure maintLock is set to false -- validation fails if the
      // object is open for editing
      //---------------------------------------------------------------------
      if ( maintLock ) {
        printDebug("MAINTLOCK is set");
        linkPublishable = false;
        FailureRec fr;
        fr = new FailureRec(draftLinkInd, FailureRec.TYPE_LINK,
                            FailureRec.FAILURE_MAINTLOCK_SET, false);
        fr.dbUser = rs.getString(4).trim();
        vRec.failureVec.addElement(fr);
      }
      rs.close();

      //---------------------------------------------------------------------
      // If the activeIndicator is 0 (no active object) or the
      // link is not being archived, the Link is publishable.
      // Remember only active Links can be used in draft objects.
      // We still continue processing to populate the usageRec Vector.
      //---------------------------------------------------------------------
      if ( activeLinkInd == 0 || !archived ) {
        printDebug("Link is publishable!!");
        linkPublishable = true;
      }

      //*********************************************************************
      // Check for usage in Action DocLinks, Question DocLinks and
      // LinkGroups (for both active objects and open drafts).
      // If we find anything and archived=true, then it is an error.
      //*********************************************************************

      //---------------------------------------------------------------------
      // Check active Action DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking active action links...");
      rs = stmt.executeQuery(
           "SELECT actionInd " +
           "FROM oa.actionLink " +
           "WHERE linkInd=" + activeLinkInd + " " +
           "ORDER BY actionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkPublishable = false;
        }
        int actionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( actionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_DOCLINK,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check active Question DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking active question links...");
      rs = stmt.executeQuery(
           "SELECT questionInd " +
           "FROM oa.questionLink " +
           "WHERE linkInd=" + activeLinkInd + " " +
           "ORDER BY questionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkPublishable = false;
        }
        int questionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_DOCLINK,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check Links in active LinkGroups:
      //---------------------------------------------------------------------
      printDebug("Checking active linkgroup links...");
      rs = stmt.executeQuery(
           "SELECT linkGroupInd " +
           "FROM oa.linkGroupLinks " +
           "WHERE linkInd=" + activeLinkInd + " " +
           "ORDER BY linkGroupInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkPublishable = false;
        }
        int linkGroupInd = rs.getInt(1);

        UsageRec ur = new UsageRec( linkGroupInd, UsageRec.TYPE_LINKGROUP,
                                    UsageRec.USAGE_LINKGROUP_LINK,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_LINKGROUP_TITLE, linkGroupInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check open draft Action DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking draft action links...");
      rs = stmt.executeQuery(
           "SELECT al.actionInd " + 
           "FROM oadraft.actionLink al, oadraft.action a " + 
           "WHERE al.linkInd=" + activeLinkInd + " " +
           "AND al.actionInd=a.actionInd " + 
           "AND a.stopDate IS NULL " +
           "ORDER BY actionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkPublishable = false;
        }
        int draftActionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftActionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_DOCLINK,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check open draft Question DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking draft question links...");
      rs = stmt.executeQuery(
           "SELECT ql.questionInd " + 
           "FROM oadraft.questionLink ql, oadraft.question q " + 
           "WHERE ql.linkInd=" + activeLinkInd + " " +
           "AND ql.questionInd=q.questionInd " + 
           "AND q.stopDate IS NULL " +
           "ORDER BY questionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_DOCLINK,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check open draft LinkGroups:
      //---------------------------------------------------------------------
      printDebug("Checking draft linkgroup links...");
      rs = stmt.executeQuery(
           "SELECT lg.linkGroupInd " + 
           "FROM oadraft.linkGroupLinks lgl, oadraft.linkGroup lg " + 
           "WHERE lgl.linkInd=" + activeLinkInd + " " +
           "AND lgl.linkGroupInd=lg.linkGroupInd " + 
           "AND lg.stopDate IS NULL " +
           "ORDER BY linkGroupInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkPublishable = false;
        }
        int draftLinkGroupInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftLinkGroupInd, UsageRec.TYPE_LINKGROUP,
                                    UsageRec.USAGE_LINKGROUP_LINK,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_LINKGROUP_TITLE,
                                    draftLinkGroupInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      vRec.publishable = linkPublishable;

    }
    finally { 
      stmt.close();
    }
   
    return vRec;
  }

  /**
   * Validate a draft LinkGroup object for publishing.
   * If the LinkGroup was never published or it is not being archived, it
   * is publishable. Check the DocLinks in Actoins and Questions for a 
   * reference to the LinkGroup (both active objects and open drafts will
   * be checked).
  **/
  private ValidationRec validateLinkGroup(int draftLinkGroupInd) 
                        throws SQLException
  {

    ValidationRec vRec = new ValidationRec();
    Statement stmt = conn.createStatement();
    ResultSet rs;

    try {

      boolean linkGroupPublishable = true;

      //---------------------------------------------------------------------
      // Get the active linkgroup indicator and the archived flag from 
      // the draft object.
      //---------------------------------------------------------------------
      printDebug("Getting activeLinkInd for " + draftLinkGroupInd + "...");
      rs = stmt.executeQuery(
           "SELECT activeLinkGroupInd, archived, maintLock, dbUser " +
           "FROM oadraft.linkGroup " +
           "WHERE linkGroupInd=" + draftLinkGroupInd + " " +
           "FOR FETCH ONLY"
      );

      //---------------------------------------------------------------------
      // If draft object was not found, return with error.
      //---------------------------------------------------------------------
      if ( !rs.next() ) {
        rs.close();
        printDebug("Indicator not found!!");
        vRec.error = true;
        vRec.errorStr = "Indicator "+ draftLinkGroupInd +" was not found";
        return vRec;
      }

      int     activeLinkGroupInd = rs.getInt(1);
      boolean archived           = DatabaseUtil.booleanFromDB(rs.getString(2));
      boolean maintLock          = DatabaseUtil.booleanFromDB(rs.getString(3));

      //---------------------------------------------------------------------
      // Make sure maintLock is set to false -- validation fails if the
      // object is open for editing
      //---------------------------------------------------------------------
      if ( maintLock ) {
        printDebug("MAINTLOCK is set");
        linkGroupPublishable = false;
        FailureRec fr;
        fr = new FailureRec(draftLinkGroupInd,FailureRec.TYPE_LINKGROUP,
                            FailureRec.FAILURE_MAINTLOCK_SET, false);
        fr.dbUser = rs.getString(4).trim();
        vRec.failureVec.addElement(fr);
      }
      rs.close();

      //---------------------------------------------------------------------
      // If the activeIndicator is 0 (no active object) or the
      // linkgroup is not being archived, the LinkGroup is publishable.
      // Remember only active LinkGroups can be used in draft objects.
      // We still continue processing to populate the usageRec Vector.
      //---------------------------------------------------------------------
      if ( activeLinkGroupInd == 0 || !archived ) {
        printDebug("LinkGroup is publishable!!");
        linkGroupPublishable = true;
      }

      //*********************************************************************
      // Check for usage in Action DocLinks and Question DocLinks (for
      // both active objects and open drafts).
      // If we find anything and archived=true, then it is an error.
      //*********************************************************************

      //---------------------------------------------------------------------
      // Check active Action DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking active action links...");
      rs = stmt.executeQuery(
           "SELECT actionInd " +
           "FROM oa.actionLink " +
           "WHERE linkGroupInd=" + activeLinkGroupInd + " " +
           "ORDER BY actionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkGroupPublishable = false;
        }
        int actionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( actionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_DOCLINK,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check active Question DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking active question links...");
      rs = stmt.executeQuery(
           "SELECT questionInd " +
           "FROM oa.questionLink " +
           "WHERE linkGroupInd=" + activeLinkGroupInd + " " +
           "ORDER BY questionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkGroupPublishable = false;
        }
        int questionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_DOCLINK,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check open draft Action DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking draft action links...");
      rs = stmt.executeQuery(
           "SELECT al.actionInd " + 
           "FROM oadraft.actionLink al, oadraft.action a " + 
           "WHERE al.linkGroupInd=" + activeLinkGroupInd + " " +
           "AND al.actionInd=a.actionInd " + 
           "AND a.stopDate IS NULL " +
           "ORDER BY actionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkGroupPublishable = false;
        }
        int draftActionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftActionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_DOCLINK,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //---------------------------------------------------------------------
      // Check open draft Question DocLinks:
      //---------------------------------------------------------------------
      printDebug("Checking draft question links...");
      rs = stmt.executeQuery(
           "SELECT ql.questionInd " + 
           "FROM oadraft.questionLink ql, oadraft.question q " + 
           "WHERE ql.linkGroupInd=" + activeLinkGroupInd + " " +
           "AND ql.questionInd=q.questionInd " + 
           "AND q.stopDate IS NULL " +
           "ORDER BY questionInd "+
           "FOR FETCH ONLY"
      );
      while ( rs.next() ) {
        if (archived) {
          linkGroupPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_DOCLINK,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      vRec.publishable = linkGroupPublishable;

    }
    finally { 
      stmt.close();
    }
   
    return vRec;
  }

  /**
   * Check the condition tables (actionTitleCond, questionTitleCond, edgeCond,
   * etc.) for usage of a Question to be published.
   * @parm activeQuestionInd  the active indicator of the Question to 
   *                          be published.
   * @parm archived           true if the Question is being archived.
   * @parm vRec               a ValidationRec to which the validation results
   *                          the will be added.
   * 
   * @return true if no errors were found; false if errors were
   *         found (question is not publishable).
  **/
  private boolean checkConditionTablesForQuestion(int activeQuestionInd, 
                                                  boolean archived,
                                                  ValidationRec vRec) 
               throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs   = null;

    boolean questionPublishable = true;

    printDebug("Checking condition tables for question...");

    try {
      //*********************************************************************
      // Check active condition tables for questionInd:
      // ActionInfoCond, ActionLinkCond, ActionTitleCond,
      // QuestionInfoCond, QuestionLinkCond, QuestionTitleCond
      // AnswerCond, AnswerTitleCond, AnswerConclCond, EdgeCond
      //*********************************************************************

      // action info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ai.actionInd " +
           "FROM   oa.actionAddInfo ai, oa.actionInfoCond aic " +
           "WHERE  aic.questionInd=" + activeQuestionInd + " " +
           "AND    aic.actionAddInfoInd=ai.actionAddInfoInd " +
           "ORDER BY ai.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int actionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( actionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_INFO_COND,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // action link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT al.actionInd " +
           "FROM   oa.actionLink al, oa.actionLinkCond alc " +
           "WHERE  alc.questionInd=" + activeQuestionInd + " " +
           "AND    alc.actionLinkInd=al.actionLinkInd " +
           "ORDER BY al.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int actionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( actionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_LINK_COND,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // action title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT at.actionInd " +
           "FROM   oa.actionTitle at, oa.actionTitleCond atc " +
           "WHERE  atc.questionInd=" + activeQuestionInd + " " +
           "AND    atc.actionTitleInd=at.actionTitleInd " +
           "ORDER BY at.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int actionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( actionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_TITLE_COND,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // question info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qi.questionInd " +
           "FROM   oa.questionAddInfo qi, oa.questionInfoCond qic " +
           "WHERE  qic.questionInd=" + activeQuestionInd + " " +
           "AND    qic.questionAddInfoInd=qi.questionAddInfoInd " +
           "ORDER BY qi.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int questionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_INFO_COND,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // question link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ql.questionInd " +
           "FROM   oa.questionLink ql, oa.questionLinkCond qlc " +
           "WHERE  qlc.questionInd=" + activeQuestionInd + " " +
           "AND    qlc.questionLinkInd=ql.questionLinkInd " +
           "ORDER BY ql.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int questionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_LINK_COND,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // question title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qt.questionInd " +
           "FROM   oa.questionTitle qt, oa.questionTitleCond qtc " +
           "WHERE  qtc.questionInd=" + activeQuestionInd + " " +
           "AND    qtc.questionTitleInd=qt.questionTitleInd " +
           "ORDER BY qt.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int questionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_TITLE_COND,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // answer conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT n.questionInd, n.answerInd " +
           "FROM   oa.answer n, oa.answerCond nc " +
           "WHERE  nc.questionInd=" + activeQuestionInd + " " +
           "AND    nc.parentAnswerInd=n.answerInd " +
           "ORDER BY n.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int questionInd = rs.getInt(1);
        int answerInd   = rs.getInt(2);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_ANSWER_COND,
                                    true, archived, answerInd );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        ur.answerTitle = readDefaultTitle(SELECT_ANSWER_TITLE, answerInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // answer title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT n.questionInd, n.answerInd " +
           "FROM   oa.answer n, oa.answerTitle nt, oa.answerTitleCond ntc " +
           "WHERE  ntc.questionInd=" + activeQuestionInd + " " +
           "AND    ntc.answerTitleInd=nt.answerTitleInd " +
           "AND    nt.answerInd=n.answerInd " +
           "ORDER BY n.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int questionInd = rs.getInt(1);
        int answerInd   = rs.getInt(2);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_ANSWER_TITLE_COND,
                                    true, archived, answerInd );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        ur.answerTitle = readDefaultTitle(SELECT_ANSWER_TITLE, answerInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // answer conclusion conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT n.questionInd, n.answerInd " +
           "FROM   oa.answer n, oa.answerConclCond ncc " +
           "WHERE  ncc.questionInd=" + activeQuestionInd + " " +
           "AND    ncc.fromAnswerInd=n.answerInd " +
           "ORDER BY n.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int questionInd = rs.getInt(1);
        int answerInd   = rs.getInt(2);

        UsageRec ur = new UsageRec( questionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_ANSWER_CONCL_COND,
                                    true, archived, answerInd );
        ur.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        ur.answerTitle = readDefaultTitle(SELECT_ANSWER_TITLE, answerInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // edge conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT g.symptomInd " +
           "FROM   oa.edgeGroup g, oa.edge e, oa.edgeCond ec " +
           "WHERE  ec.questionInd=" + activeQuestionInd + " " +
           "AND    ec.edgeInd=e.edgeInd " +
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "ORDER BY g.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int symptomInd = rs.getInt(1);

        UsageRec ur = new UsageRec( symptomInd, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_EDGE_COND,
                                    true, archived );
        ur.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      //*********************************************************************
      // Check draft condition tables for open drafts using questionInd:
      // ActionInfoCond, ActionLinkCond, ActionTitleCond,
      // QuestionInfoCond, QuestionLinkCond, QuestionTitleCond
      // AnswerCond, AnswerTitleCond, AnswerConclCond, EdgeCond
      //*********************************************************************

      // action info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ai.actionInd " +
           "FROM   oadraft.actionAddInfo ai, oadraft.actionInfoCond aic, " +
                  "oadraft.action a " +
           "WHERE  aic.questionInd=" + activeQuestionInd + " " +
           "AND    aic.actionAddInfoInd=ai.actionAddInfoInd " +
           "AND    ai.actionInd=a.actionInd " +
           "AND    a.stopDate IS NULL " +
           "ORDER BY ai.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftActionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftActionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_INFO_COND,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // action link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT al.actionInd " +
           "FROM   oadraft.actionLink al, oadraft.actionLinkCond alc, " +
                  "oadraft.action a " +
           "WHERE  alc.questionInd=" + activeQuestionInd + " " +
           "AND    alc.actionLinkInd=al.actionLinkInd " +
           "AND    al.actionInd=a.actionInd " +
           "AND    a.stopDate IS NULL " +
           "ORDER BY al.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftActionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftActionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_LINK_COND,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // action title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT at.actionInd " +
           "FROM   oadraft.actionTitle at, oadraft.actionTitleCond atc, " +
                  "oadraft.action a " +
           "WHERE  atc.questionInd=" + activeQuestionInd + " " +
           "AND    atc.actionTitleInd=at.actionTitleInd " +
           "AND    at.actionInd=a.actionInd " +
           "AND    a.stopDate IS NULL " +
           "ORDER BY at.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftActionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftActionInd, UsageRec.TYPE_ACTION,
                                    UsageRec.USAGE_ACTION_TITLE_COND,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // question info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qi.questionInd " +
           "FROM   oadraft.questionAddInfo qi, oadraft.questionInfoCond qic, " +
                  "oadraft.question q " +
           "WHERE  qic.questionInd=" + activeQuestionInd + " " +
           "AND    qic.questionAddInfoInd=qi.questionAddInfoInd " +
           "AND    qi.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY qi.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_INFO_COND,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // question link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ql.questionInd " +
           "FROM   oadraft.questionLink ql, oadraft.questionLinkCond qlc, " +
                  "oadraft.question q " +
           "WHERE  qlc.questionInd=" + activeQuestionInd + " " +
           "AND    qlc.questionLinkInd=ql.questionLinkInd " +
           "AND    ql.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY ql.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_LINK_COND,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // question title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qt.questionInd " +
           "FROM   oadraft.questionTitle qt, oadraft.questionTitleCond qtc, " +
                  "oadraft.question q " +
           "WHERE  qtc.questionInd=" + activeQuestionInd + " " +
           "AND    qtc.questionTitleInd=qt.questionTitleInd " +
           "AND    qt.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY qt.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_QUESTION_TITLE_COND,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // answer conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT q.questionInd, n.answerInd " +
           "FROM   oadraft.answer n, oadraft.answerCond nc, " +
                  "oadraft.question q " +
           "WHERE  nc.questionInd=" + activeQuestionInd + " " +
           "AND    nc.parentAnswerInd=n.answerInd " +
           "AND    n.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY q.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);
        int draftAnswerInd   = rs.getInt(2);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_ANSWER_COND,
                                    false, archived, draftAnswerInd );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        ur.answerTitle = readDefaultTitle(SELECT_DRAFT_ANSWER_TITLE,
                                          draftAnswerInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // answer title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT q.questionInd, n.answerInd " +
           "FROM   oadraft.answer n, oadraft.answerTitle nt,  " +
                  "oadraft.answerTitleCond ntc, oadraft.question q " +
           "WHERE  ntc.questionInd=" + activeQuestionInd + " " +
           "AND    ntc.answerTitleInd=nt.answerTitleInd " +
           "AND    nt.answerInd=n.answerInd " +
           "AND    n.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY q.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);
        int draftAnswerInd   = rs.getInt(2);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_ANSWER_TITLE_COND,
                                    false, archived, draftAnswerInd );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        ur.answerTitle = readDefaultTitle(SELECT_DRAFT_ANSWER_TITLE,
                                          draftAnswerInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // answer conclusion conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT q.questionInd, n.answerInd " +
           "FROM   oadraft.answer n, oadraft.answerConclCond ncc, " +
                  "oadraft.question q " +
           "WHERE  ncc.questionInd=" + activeQuestionInd + " " +
           "AND    ncc.fromAnswerInd=n.answerInd " +
           "AND    n.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY q.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftQuestionInd = rs.getInt(1);
        int draftAnswerInd   = rs.getInt(2);

        UsageRec ur = new UsageRec( draftQuestionInd, UsageRec.TYPE_QUESTION,
                                    UsageRec.USAGE_ANSWER_CONCL_COND,
                                    false, archived, draftAnswerInd );
        ur.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        ur.answerTitle = readDefaultTitle(SELECT_DRAFT_ANSWER_TITLE,
                                          draftAnswerInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

      // edge conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM   oadraft.edgeGroup g, oadraft.edge e, oadraft.edgeCond ec, " +
                  "oadraft.symptom s " +
           "WHERE  ec.questionInd=" + activeQuestionInd + " " +
           "AND    ec.edgeInd=e.edgeInd " +
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "AND    g.symptomInd=s.symptomInd " +
           "AND    s.stopDate IS NULL " +
           "ORDER BY s.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        if (archived) {
          questionPublishable = false;
        }
        int draftSymptomInd = rs.getInt(1);

        UsageRec ur = new UsageRec( draftSymptomInd, UsageRec.TYPE_SYMPTOM,
                                    UsageRec.USAGE_EDGE_COND,
                                    false, archived );
        ur.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.usageVec.addElement( ur );
      }
      rs.close();

    }
    finally {
      stmt.close();
    }

    return questionPublishable;
  }

  /**
   * Check the condition tables (actionTitleCond, questionTitleCond, edgeCond,
   * etc.), the Edge table and the Preanswer table for usage of answers that
   * have been deleted from a question that is being published.
   * 
   * @return true if no errors were found (publishable) or 
   *         false if errors were found (not publishable) 
  **/
  private boolean checkDeletedAnswers( int pActiveQuestionInd, 
                                       int pDraftQuestionInd,
                                       ValidationRec vRec ) 
               throws SQLException
  {
    Statement stmt = conn.createStatement();
    ResultSet rs   = null;

    boolean questionPublishable = true;

    printDebug("Checking deleted answers...");

    try {
      //*********************************************************************
      // Determine if any of the answers will be deleted on publish:
      //*********************************************************************
      rs = stmt.executeQuery(
           "SELECT answerind " +
           "FROM oa.answer " +
           "WHERE questionInd=" + pActiveQuestionInd + " " +
           "AND answerInd NOT IN (" +
                "SELECT activeAnswerInd " +
                "FROM oadraft.answer " +
                "WHERE questionInd=" + pDraftQuestionInd + " " +
                "AND activeAnswerInd IS NOT NULL" +
           ")"
      );

      boolean answersDeleted = false;
      StringBuffer        sb = null;

      while( rs.next() ) {
        // construct the list of deleted answers 
        if (sb == null) {
          answersDeleted = true;
          sb = new StringBuffer();
          sb.append('(');
        }
        answersDeleted = true;
        sb.append( rs.getInt(1) ).append(',');
      }
      rs.close();

      //---------------------------------------------------------------------
      // if there were no deleted answers, then just return -- no work to 
      // be done.  otherwise, prepare to check all the condition tables for
      // the deleted answers.
      //---------------------------------------------------------------------
      if ( !answersDeleted ) {
        printDebug("===> no deleted answers");
        return true;
      } else {
        sb.setCharAt( sb.length()-1, ')' );         // change last ',' to ')'
        sb.append(' ');                             // trailing space
      }

      String deletedAnswers = sb.toString();        // "(ansInd1,ansInd2) "
      printDebug("deletedAnswers=\"" + deletedAnswers + "\"");
      
      //*********************************************************************
      // Check active tables for the deleted answers:
      // ActionInfoCond, ActionLinkCond, ActionTitleCond,
      // QuestionInfoCond, QuestionLinkCond, QuestionTitleCond
      // AnswerCond, AnswerTitleCond, AnswerConclCond, EdgeCond,
      // Edge, EdgePreanswer
      //*********************************************************************

      // action info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ai.actionInd " +
           "FROM   oa.actionAddInfo ai, oa.actionInfoCond aic " +
           "WHERE  aic.answerInd IN " + deletedAnswers + 
           "AND    aic.actionAddInfoInd=ai.actionAddInfoInd " +
           "ORDER BY ai.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int actionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(actionInd, FailureRec.TYPE_ACTION,
                            FailureRec.QFAILURE_ACTION_INFO_COND, true);
        fr.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // action link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT al.actionInd " +
           "FROM   oa.actionLink al, oa.actionLinkCond alc " +
           "WHERE  alc.answerInd IN " + deletedAnswers + 
           "AND    alc.actionLinkInd=al.actionLinkInd " +
           "ORDER BY al.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int actionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(actionInd, FailureRec.TYPE_ACTION,
                            FailureRec.QFAILURE_ACTION_LINK_COND, true);
        fr.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // action title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT at.actionInd " +
           "FROM   oa.actionTitle at, oa.actionTitleCond atc " +
           "WHERE  atc.answerInd IN " + deletedAnswers + 
           "AND    atc.actionTitleInd=at.actionTitleInd " +
           "ORDER BY at.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int actionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(actionInd, FailureRec.TYPE_ACTION,
                            FailureRec.QFAILURE_ACTION_TITLE_COND, true);
        fr.title = readDefaultTitle(SELECT_ACTION_TITLE, actionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // question info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qi.questionInd " +
           "FROM   oa.questionAddInfo qi, oa.questionInfoCond qic " +
           "WHERE  qic.answerInd IN " + deletedAnswers + 
           "AND    qic.questionAddInfoInd=qi.questionAddInfoInd " +
           "ORDER BY qi.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int questionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(questionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_QUESTION_INFO_COND, true);
        fr.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // question link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ql.questionInd " +
           "FROM   oa.questionLink ql, oa.questionLinkCond qlc " +
           "WHERE  qlc.answerInd IN " + deletedAnswers + 
           "AND    qlc.questionLinkInd=ql.questionLinkInd " +
           "ORDER BY ql.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int questionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(questionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_QUESTION_LINK_COND, true);
        fr.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // question title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qt.questionInd " +
           "FROM   oa.questionTitle qt, oa.questionTitleCond qtc " +
           "WHERE  qtc.answerInd IN " + deletedAnswers + 
           "AND    qtc.questionTitleInd=qt.questionTitleInd " +
           "ORDER BY qt.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int questionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(questionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_QUESTION_TITLE_COND, true);
        fr.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // answer conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT n.questionInd, n.answerInd " +
           "FROM   oa.answer n, oa.answerCond nc " +
           "WHERE  nc.answerInd IN " + deletedAnswers + 
           "AND    nc.parentAnswerInd=n.answerInd " +
           "ORDER BY n.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int questionInd = rs.getInt(1);
        int answerInd   = rs.getInt(2);

        FailureRec fr;
        fr = new FailureRec(questionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_ANSWER_COND,
                            true, answerInd);
        fr.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        fr.answerTitle = readDefaultTitle(SELECT_ANSWER_TITLE, answerInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // answer title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT n.questionInd, n.answerInd " +
           "FROM   oa.answer n, oa.answerTitle nt, oa.answerTitleCond ntc " +
           "WHERE  ntc.answerInd IN " + deletedAnswers + 
           "AND    ntc.answerTitleInd=nt.answerTitleInd " +
           "AND    nt.answerInd=n.answerInd " +
           "ORDER BY n.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int questionInd = rs.getInt(1);
        int answerInd   = rs.getInt(2);

        FailureRec fr;
        fr = new FailureRec(questionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_ANSWER_TITLE_COND,
                            true, answerInd);
        fr.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        fr.answerTitle = readDefaultTitle(SELECT_ANSWER_TITLE, answerInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // answer conclusion conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT n.questionInd, n.answerInd " +
           "FROM   oa.answer n, oa.answerConclCond ncc " +
           "WHERE  ncc.answerInd IN " + deletedAnswers + 
           "AND    ncc.fromAnswerInd=n.answerInd " +
           "ORDER BY n.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int questionInd = rs.getInt(1);
        int answerInd   = rs.getInt(2);

        FailureRec fr;
        fr = new FailureRec(questionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_ANSWER_CONCL_COND,
                            true, answerInd);
        fr.title = readDefaultTitle(SELECT_QUESTION_TITLE, questionInd);
        fr.answerTitle = readDefaultTitle(SELECT_ANSWER_TITLE, answerInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // edge conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT g.symptomInd " +
           "FROM   oa.edgeGroup g, oa.edge e, oa.edgeCond ec " +
           "WHERE  ec.answerInd IN " + deletedAnswers + 
           "AND    ec.edgeInd=e.edgeInd " +
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "ORDER BY g.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int symptomInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(symptomInd, FailureRec.TYPE_SYMPTOM,
                            FailureRec.QFAILURE_EDGE_COND, true);
        fr.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // edges
      rs = stmt.executeQuery(
           "SELECT DISTINCT g.symptomInd " +
           "FROM   oa.edgeGroup g, oa.edge e " +
           "WHERE  e.answerInd IN " + deletedAnswers + 
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "ORDER BY g.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int symptomInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(symptomInd, FailureRec.TYPE_SYMPTOM,
                            FailureRec.QFAILURE_EDGE, true);
        fr.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // edge preanswers
      rs = stmt.executeQuery(
           "SELECT DISTINCT g.symptomInd " +
           "FROM   oa.edgeGroup g, oa.edge e, oa.edgePreanswer ep " +
           "WHERE  ep.answerInd IN " + deletedAnswers + 
           "AND    ep.preanswerType=" + EdgeObject.EDGE_PREANSWER + " " +
           "AND    ep.edgeInd=e.edgeInd " +
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "ORDER BY g.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int symptomInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(symptomInd, FailureRec.TYPE_SYMPTOM,
                            FailureRec.QFAILURE_EDGE_PREANSWER, true);
        fr.title = readDefaultTitle(SELECT_SYMPTOM_TITLE, symptomInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      //*********************************************************************
      // Check draft tables for open drafts that are using 
      // the deleted answers:
      // ActionInfoCond, ActionLinkCond, ActionTitleCond,
      // QuestionInfoCond, QuestionLinkCond, QuestionTitleCond
      // AnswerCond, AnswerTitleCond, AnswerConclCond, EdgeCond,
      // Edge, EdgePreanswer
      //*********************************************************************

      // action info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ai.actionInd " +
           "FROM   oadraft.actionAddInfo ai, oadraft.actionInfoCond aic, " +
                  "oadraft.action a " +
           "WHERE  aic.answerInd IN " + deletedAnswers + 
           "AND    aic.actionAddInfoInd=ai.actionAddInfoInd " +
           "AND    ai.actionInd=a.actionInd " +
           "AND    a.stopDate IS NULL " +
           "ORDER BY ai.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftActionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftActionInd, FailureRec.TYPE_ACTION,
                            FailureRec.QFAILURE_ACTION_INFO_COND, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // action link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT al.actionInd " +
           "FROM   oadraft.actionLink al, oadraft.actionLinkCond alc, " +
                  "oadraft.action a " +
           "WHERE  alc.answerInd IN " + deletedAnswers + 
           "AND    alc.actionLinkInd=al.actionLinkInd " +
           "AND    al.actionInd=a.actionInd " +
           "AND    a.stopDate IS NULL " +
           "ORDER BY al.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftActionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftActionInd, FailureRec.TYPE_ACTION,
                            FailureRec.QFAILURE_ACTION_LINK_COND, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // action title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT at.actionInd " +
           "FROM   oadraft.actionTitle at, oadraft.actionTitleCond atc, " +
                  "oadraft.action a " +
           "WHERE  atc.answerInd IN " + deletedAnswers + 
           "AND    atc.actionTitleInd=at.actionTitleInd " +
           "AND    at.actionInd=a.actionInd " +
           "AND    a.stopDate IS NULL " +
           "ORDER BY at.actionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftActionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftActionInd, FailureRec.TYPE_ACTION,
                            FailureRec.QFAILURE_ACTION_TITLE_COND, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_ACTION_TITLE,
                                    draftActionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // question info conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qi.questionInd " +
           "FROM   oadraft.questionAddInfo qi, oadraft.questionInfoCond qic, " +
                  "oadraft.question q " +
           "WHERE  qic.answerInd IN " + deletedAnswers + 
           "AND    qic.questionAddInfoInd=qi.questionAddInfoInd " +
           "AND    qi.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY qi.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftQuestionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_QUESTION_INFO_COND, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // question link conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT ql.questionInd " +
           "FROM   oadraft.questionLink ql, oadraft.questionLinkCond qlc, " +
                  "oadraft.question q " +
           "WHERE  qlc.answerInd IN " + deletedAnswers + 
           "AND    qlc.questionLinkInd=ql.questionLinkInd " +
           "AND    ql.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY ql.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftQuestionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_QUESTION_LINK_COND, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // question title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT qt.questionInd " +
           "FROM   oadraft.questionTitle qt, oadraft.questionTitleCond qtc, " +
                  "oadraft.question q " +
           "WHERE  qtc.answerInd IN " + deletedAnswers + 
           "AND    qtc.questionTitleInd=qt.questionTitleInd " +
           "AND    qt.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY qt.questionInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftQuestionInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_QUESTION_TITLE_COND, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // answer conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT q.questionInd, n.answerInd " +
           "FROM   oadraft.answer n, oadraft.answerCond nc, " +
                  "oadraft.question q " +
           "WHERE  nc.answerInd IN " + deletedAnswers + 
           "AND    nc.parentAnswerInd=n.answerInd " +
           "AND    n.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY q.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftQuestionInd = rs.getInt(1);
        int draftAnswerInd   = rs.getInt(2);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_ANSWER_COND,
                            false, draftAnswerInd);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        fr.answerTitle = readDefaultTitle(SELECT_DRAFT_ANSWER_TITLE,
                                          draftAnswerInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // answer title conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT q.questionInd, n.answerInd " +
           "FROM   oadraft.answer n, oadraft.answerTitle nt,  " +
                  "oadraft.answerTitleCond ntc, oadraft.question q " +
           "WHERE  ntc.answerInd IN " + deletedAnswers + 
           "AND    ntc.answerTitleInd=nt.answerTitleInd " +
           "AND    nt.answerInd=n.answerInd " +
           "AND    n.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY q.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftQuestionInd = rs.getInt(1);
        int draftAnswerInd   = rs.getInt(2);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_ANSWER_TITLE_COND,
                            false, draftAnswerInd);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        fr.answerTitle = readDefaultTitle(SELECT_DRAFT_ANSWER_TITLE,
                                          draftAnswerInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // answer conclusion conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT q.questionInd, n.answerInd " +
           "FROM   oadraft.answer n, oadraft.answerConclCond ncc, " +
                  "oadraft.question q " +
           "WHERE  ncc.answerInd IN " + deletedAnswers + 
           "AND    ncc.fromAnswerInd=n.answerInd " +
           "AND    n.questionInd=q.questionInd " +
           "AND    q.stopDate IS NULL " +
           "ORDER BY q.questionInd, n.answerInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftQuestionInd = rs.getInt(1);
        int draftAnswerInd   = rs.getInt(2);

        FailureRec fr;
        fr = new FailureRec(draftQuestionInd, FailureRec.TYPE_QUESTION,
                            FailureRec.QFAILURE_ANSWER_CONCL_COND,
                            false, draftAnswerInd);
        fr.title = readDefaultTitle(SELECT_DRAFT_QUESTION_TITLE,
                                    draftQuestionInd);
        fr.answerTitle = readDefaultTitle(SELECT_DRAFT_ANSWER_TITLE,
                                          draftAnswerInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // edge conditions
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM   oadraft.edgeGroup g, oadraft.edge e, oadraft.edgeCond ec, " +
                  "oadraft.symptom s " +
           "WHERE  ec.answerInd IN " + deletedAnswers + 
           "AND    ec.edgeInd=e.edgeInd " +
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "AND    g.symptomInd=s.symptomInd " +
           "AND    s.stopDate IS NULL " +
           "ORDER BY s.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftSymptomInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftSymptomInd, FailureRec.TYPE_SYMPTOM,
                            FailureRec.QFAILURE_EDGE_COND, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // edges
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM   oadraft.edgeGroup g, oadraft.edge e, oadraft.symptom s " +
           "WHERE  e.activeAnswerInd IN " + deletedAnswers + 
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "AND    g.symptomInd=s.symptomInd " +
           "AND    s.stopDate IS NULL " +
           "ORDER BY s.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftSymptomInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftSymptomInd, FailureRec.TYPE_SYMPTOM,
                            FailureRec.QFAILURE_EDGE, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

      // edge preanswers
      rs = stmt.executeQuery(
           "SELECT DISTINCT s.symptomInd " +
           "FROM   oadraft.edgeGroup g, oadraft.edge e, " +
                  "oadraft.edgePreanswer ep, oadraft.symptom s " +
           "WHERE  ep.activeAnswerInd IN " + deletedAnswers + 
           "AND    ep.preanswerType=" + EdgeObject.EDGE_PREANSWER + " " +
           "AND    ep.edgeInd=e.edgeInd " +
           "AND    e.edgeGroupInd=g.edgeGroupInd " +
           "AND    g.symptomInd=s.symptomInd " +
           "AND    s.stopDate IS NULL " +
           "ORDER BY s.symptomInd FOR FETCH ONLY"
      );
      while( rs.next() ) {
        questionPublishable = false;
        int draftSymptomInd = rs.getInt(1);

        FailureRec fr;
        fr = new FailureRec(draftSymptomInd, FailureRec.TYPE_SYMPTOM,
                            FailureRec.QFAILURE_EDGE_PREANSWER, false);
        fr.title = readDefaultTitle(SELECT_DRAFT_SYMPTOM_TITLE,
                                    draftSymptomInd);
        vRec.failureVec.addElement( fr );
      }
      rs.close();

    }
    finally {
      stmt.close();
    }

    return questionPublishable;
  }

  //*************************************************************************
  // Methods/members used for reading the default titles:
  //*************************************************************************


  //-------------------------------------------------------------------------
  // Identifiers for the select statements used to read the default title
  // for each type of object.  Each select statement has a corresponding
  // entry in the pstmts[] array of PreparedStatements.  The first time a
  // select statement is to be used, a PreparedStatement will be created
  // in the pstmts[] array.  Any statements that were prepared are closed
  // by closeTitleSelectStmts() when the stored procedure completes.
  // SELECT_MAX should be set to the number of select statements -- it is
  // used to allocate the pstmts[] array.
  //-------------------------------------------------------------------------
  private static final short SELECT_ACTION_TITLE          = 0;
  private static final short SELECT_DRAFT_ACTION_TITLE    = 1;
  private static final short SELECT_QUESTION_TITLE        = 2;
  private static final short SELECT_DRAFT_QUESTION_TITLE  = 3;
  private static final short SELECT_SYMPTOM_TITLE         = 4;
  private static final short SELECT_DRAFT_SYMPTOM_TITLE   = 5;
  private static final short SELECT_LINKGROUP_TITLE       = 6;
  private static final short SELECT_DRAFT_LINKGROUP_TITLE = 7;
  private static final short SELECT_ANSWER_TITLE          = 8;
  private static final short SELECT_DRAFT_ANSWER_TITLE    = 9;
  private static final short SELECT_MAX                   = 10;

  // array of PreparedStatements used for reading the default titles
  private PreparedStatement[] pstmts = new PreparedStatement[SELECT_MAX];

  /**
   * Create a PreparedStatement to read the default title for the specified
   * type of object.
   *
   * @param id  Must be one of the SELECT_* values which identifies the
   *            select statement to be prepared.
  **/
  private void prepareTitleSelectStmt(short id) throws SQLException {
    String sql = null;

    //----------------------------------------------------------------------
    // NOTE: These select statments are executed by the readDefaultTitle()
    //       method.  Every select statement must have excactly two
    //       parameter placeholders ("?"), both of wich will be substituted
    //       with the object indicator when the statement is executed.
    //       Any exceptions to this must be handled as special cases in 
    //       readDefaultTitle() -- i.e., LinkGroup titles.
    //----------------------------------------------------------------------

    // get the sql for the select statement
    switch (id) {
      case SELECT_ACTION_TITLE:
           sql = "SELECT title FROM oa.actionTitle " +
                 "WHERE actionInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oa.actionTitle " +
                     "WHERE actionInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_DRAFT_ACTION_TITLE:
           sql = "SELECT title FROM oadraft.actionTitle " +
                 "WHERE actionInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oadraft.actionTitle " +
                     "WHERE actionInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_QUESTION_TITLE:
           sql = "SELECT title FROM oa.questionTitle " +
                 "WHERE questionInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oa.questionTitle " +
                     "WHERE questionInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_DRAFT_QUESTION_TITLE:
           sql = "SELECT title FROM oadraft.questionTitle " +
                 "WHERE questionInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oadraft.questionTitle " +
                     "WHERE questionInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_SYMPTOM_TITLE:
           sql = "SELECT title FROM oa.symptomTitle " +
                 "WHERE symptomInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oa.symptomTitle " +
                     "WHERE symptomInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_DRAFT_SYMPTOM_TITLE:
           sql = "SELECT title FROM oadraft.symptomTitle " +
                 "WHERE symptomInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oadraft.symptomTitle " +
                     "WHERE symptomInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_LINKGROUP_TITLE:
           sql = "SELECT title FROM oa.linkGroupTitle " +
                 "WHERE linkGroupInd=? " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_DRAFT_LINKGROUP_TITLE:
           sql = "SELECT title FROM oadraft.linkGroupTitle " +
                 "WHERE linkGroupInd=? " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_ANSWER_TITLE:
           sql = "SELECT title FROM oa.answerTitle " +
                 "WHERE answerInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oa.answerTitle " +
                     "WHERE answerInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      case SELECT_DRAFT_ANSWER_TITLE:
           sql = "SELECT title FROM oadraft.answerTitle " +
                 "WHERE answerInd=? AND sortOrder=" +
                    "(SELECT MAX(sortOrder) FROM oadraft.answerTitle " +
                     "WHERE answerInd=?) " +
                 "ORDER BY languageInd " +
                 "FOR FETCH ONLY";
           break;

      default:
           sql = null;
           break;
    }

    // prepare the statement
    if ( sql != null && pstmts[id] == null ) {
      pstmts[id] = conn.prepareStatement( sql );
    }
  }

  /**
   * Close any PreparedStatments that were opened for reading titles.
  **/
  private void closeTitleSelectStmts() {
    // loop through the array of PreparedStatements and close
    // any entries that have non-null values.
    for (int i=0; i<SELECT_MAX; i++) {
      if ( pstmts[i] != null ) {
        try {
          pstmts[i].close();
          pstmts[i] = null;    // allow gc
        }
        catch (SQLException ignore) { }
      }
    }
  }

  /**
   * Read the default title for the specified object.
   *
   * @param id         the type type of title to read.  must be one of
   *                   the SELECT_* identifiers.
   * @param objectInd  the object's indicator
   * 
   * @return the default title
  **/
  private String readDefaultTitle(short id, int objectInd)
                 throws SQLException
  {
    if (pstmts[id] == null) {
      // statement has not been used before -- prepare it now
      prepareTitleSelectStmt(id);
    }

    // set the parameters
    pstmts[id].setInt(1, objectInd);
    // linkGroup title select statements only have one parameter (no subselect)
    if (id != SELECT_LINKGROUP_TITLE && id != SELECT_DRAFT_LINKGROUP_TITLE) {
      pstmts[id].setInt(2, objectInd);
    }

    // execute the statement
    ResultSet rs = pstmts[id].executeQuery();

    // get the title
    String title;
    if ( rs.next() ) {
      title = rs.getString(1);   // get the first title ordered by languageInd
    } else {
      title = "N/A";             // default title not available
    }
    rs.close();

    return title;
  }


  //*************************************************************************
  // For testing the validateXxx() methods from the command line:
  //*************************************************************************
  private static boolean DEBUG=false;

  public static void main(String[] args) throws Exception {
    DEBUG=true;  // only when running from command line

    // database connection information
    String database = "OATEST";
    String userid   = "quest";
    String password = "j0hnny";
    String url      = "jdbc:db2:" + database;

    // load the DB2 app driver & open the connection
    OAValidate validateSP = new OAValidate();

    // connect via db2
    Class.forName("COM.ibm.db2.jdbc.app.DB2Driver");
    //validateSP.conn = DriverManager.getConnection( "jdbc:db2:" + database,
    //                                               userid, password );
    // connect via odbc
    //Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    //validateSP.conn = DriverManager.getConnection( "jdbc:odbc:" + database,
                                                   //userid, password );

/*
    // run all the default title select statements to check for errors
    for (short i=0; i<SELECT_MAX; i++) {
      System.out.println("Running Statement " + i);
      System.out.println("   \"" + validateSP.readDefaultTitle(i,1) + "\"");
    }
    System.out.println("Closing Prepared Statements");
    validateSP.closeTitleSelectStmts();
    System.out.println();
    if (true) return;
*/

    //-----------------------------------------------------------------------
    System.out.println();
    long start, stop;
    start = System.currentTimeMillis();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(50203568,LINK,null);
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(28476510,LINKGROUP,null);
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(331480002,ACTION,null);   // has active action
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(332565492,ACTION,null);   // does not have active action
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(115219904,SYMPTOM,null);  // has active symptom
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(117065049,SYMPTOM,null);  // doesnt have active symptom
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(298280737,QUESTION,null); // has active quesiton
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(14153396 ,QUESTION,null); // doesnt have active quesiton
    System.out.println();

    validateSP.conn = DriverManager.getConnection( url, userid, password );
    validateSP.validate(298445263,QUESTION,null); // has deleted answers
    System.out.println();

    stop = System.currentTimeMillis();
    System.out.println("Execution Time: " + (stop-start) + " ms");
    //-----------------------------------------------------------------------

    validateSP.conn.close();
  }
  private static void printDebug(String str) {
    if (DEBUG) System.out.println(str);
  }

}
