
package com.ibm.nzna.projects.common.storedProc.oaValidate;

import com.ibm.nzna.projects.common.storedProc.*;
import java.util.*;

/**
 * The UsageRec object contains information about an object which is
 * using (is dependent on) the object which is being validated.
 * This class holds the following information about the object
 * which has the dependency on the object being validated:
 * <ul>
 * <li> objectInd    - The indicator of the object that has the dependency.
 * <li> objectType   - The type of object (Action, Question, Symptom
 *                     or LinkGroup).
 * <li> title        - The default title of the object.
 * <li> usageType    - Indicates how this object is using the object
 *                     being validated.
 * <li> activeObject - If true, this is an active object.
 *                     If false, it is a draft object.
 * <li> isError      - If true, the dependency represents an error.
 *                     This is usally only true if the object being
 *                     validated is being archived.
 *                     <b>Note</b>: the publishable flag in the ValidationRec
 *                     will tell whether or not the object is publishable
 *                     without having to check all the UsageRec objects.
 * </ul>
 * If the dependency was on a specific Answer to a Question, the 
 * following fields will also be populated:
 * <ul>
 * <li> answerInd   - the Answer's indicator.
 * <li> answerTitle - the Answer's default title.
 * </ul>
 * 
 * @version 1.4
 * @author  Mark DeBiase
**/
public class UsageRec implements java.io.Serializable {

  //------------------------------------------------------------------------
  // objectType values
  //------------------------------------------------------------------------
  /** objectType: objectInd is for an Action object */
  public static final short TYPE_ACTION    = 800;
  /** objectType: objectInd is for a Question object */
  public static final short TYPE_QUESTION  = 801;
  /** objectType: objectInd is for a Symptom object */
  public static final short TYPE_SYMPTOM   = 802;
  /** objectType: objectInd is for a LinkGroup object */
  public static final short TYPE_LINKGROUP = 803;
  /** objectType: objectInd is for a Link object */
  public static final short TYPE_LINK      = 804;

  //------------------------------------------------------------------------
  // usageType values
  //------------------------------------------------------------------------
  /** usageType: Link or LinkGroup that is being validated
      is being used in an Action DocLink */
  public static final short USAGE_ACTION_DOCLINK      = 500;
  /** usageType: Link or LinkGroup that is being validated
      is being used in a Question DocLink */
  public static final short USAGE_QUESTION_DOCLINK    = 501;
  /** usageType: Link that is being validated
      is being used in a LinkGroup */
  public static final short USAGE_LINKGROUP_LINK      = 502;
  /** usageType: Action, Question or Symptom that is being validated
      is being used in a Symptom Node */
  public static final short USAGE_SYMPTOM_NODE        = 503;
  /** usageType: An Answer to a Question that is being validated
      is being used in an Edge Preanswer */
  public static final short USAGE_EDGE_PREANSWER      = 504;
  /** usageType: Question that is being validated
      is being used in an Edge Unanswer */
  public static final short USAGE_EDGE_UNANSWER       = 505;
  /** usageType: Action that is being validated
      is being used in an Edge Alist */
  public static final short USAGE_EDGE_ALIST          = 506;
  /** usageType: Question that is being validated
      is being used in a Condition on an Action AddtlInfo */
  public static final short USAGE_ACTION_INFO_COND    = 507;
  /** usageType: Question that is being validated
      is being used in a Condition on an Action DocLink */
  public static final short USAGE_ACTION_LINK_COND    = 508;
  /** usageType: Question that is being validated
      is being used in a Condition on an Action Title */
  public static final short USAGE_ACTION_TITLE_COND   = 509;
  /** usageType: Question that is being validated
      is being used in a Condition on a Question AddtlInfo */
  public static final short USAGE_QUESTION_INFO_COND  = 510;
  /** usageType: Question that is being validated
      is being used in a Condition on a Question DocLink */
  public static final short USAGE_QUESTION_LINK_COND  = 511;
  /** usageType: Question that is being validated
      is being used in a Condition on a Question Title */
  public static final short USAGE_QUESTION_TITLE_COND = 512;
  /** usageType: Question that is being validated
      is being used in a Condition on an Answer */
  public static final short USAGE_ANSWER_COND         = 513;
  /** usageType: Question that is being validated
      is being used in a Condition on an Answer Title */
  public static final short USAGE_ANSWER_TITLE_COND   = 514;
  /** usageType: Question that is being validated
      is being used in a Condition on an Answer Conclusion */
  public static final short USAGE_ANSWER_CONCL_COND   = 515;
  /** usageType: Question that is being validated
      is being used in a Condition on an Symptom Edge */
  public static final short USAGE_EDGE_COND           = 516;


  //------------------------------------------------------------------------
  // Object member fields
  //------------------------------------------------------------------------

  /**
   * The indicator of an object which is using (has a dependency on)
   * the object being validated.
   * The objectType field indicates the type of object the indicator 
   * represents.
   * The activeObject field indicates whether it is an active or draft 
   * object indicator.
  **/
  public int objectInd;

  /**
   * The default title of the object represented by objectInd.
  **/
  public String title;

  /**
   * The type of object represented by objectInd.
   * Will be one of the following values:
   * <ul>
   * <li> TYPE_ACTION
   * <li> TYPE_QUESTION
   * <li> TYPE_SYMPTOM
   * <li> TYPE_LINKGROUP
   * </ul>
  **/
  public short objectType;

  /**
   * Indicates how the object being validated is being used by the
   * object represented by objectInd.
   * Will be one of the following values:
   * <ul>
   * <li> USAGE_ACTION_DOCLINK
   * <li> USAGE_QUESTION_DOCLINK
   * <li> USAGE_LINKGROUP_LINK
   * <li> USAGE_SYMPTOM_NODE
   * <li> USAGE_EDGE_PREANSWER
   * <li> USAGE_EDGE_UNANSWER
   * <li> USAGE_EDGE_ALIST
   * <li> USAGE_ACTION_INFO_COND
   * <li> USAGE_ACTION_LINK_COND
   * <li> USAGE_ACTION_TITLE_COND
   * <li> USAGE_QUESTION_INFO_COND
   * <li> USAGE_QUESTION_LINK_COND
   * <li> USAGE_QUESTION_TITLE_COND
   * <li> USAGE_ANWSER_COND
   * <li> USAGE_ANSWER_TITLE_COND
   * <li> USAGE_ANSWER_CONCL_COND
   * <li> USAGE_EDGE_COND
   * </ul>
  **/
  public short usageType;

  /**
   * Indicates whether objectInd is an active or draft indicator.
   * If true, objectInd is an active object indicator.
   * If false, objectInd is a draft object indicator (for an OPEN draft).
  **/
  public boolean activeObject;

  /**
   * Indicates whether this UsageRec represents an error which prevents
   * validation.
   * If true, validation was failed due to a dependency on the object
   * being validated by the object represented by objectInd (this most
   * likely means the object being validated is to be archived).
   * If false, validation was <b>not</b> failed -- there is a
   * dependency on the object being validated, but it is not an error
   * condition.
   * <p>
   * If isError is false, objectInd is just being returned for informational
   * purposes -- it may be displayed to the user as a list of objects that
   * will be affected by the publish (or it can just be ignored).
  **/
  public boolean isError;

  /**
   * The indicator of the specific Answer involved when usageType indicates
   * usage in Conditions on an Answer.
   * This field is only populated for the following usageType values:
   * <ul>
   * <li> USAGE_ANWSER_COND
   * <li> USAGE_ANSWER_TITLE_COND
   * <li> USAGE_ANSWER_CONCL_COND
   * </ul>
   * If usageType has any other value, answerInd will have the value -1.
  **/
  public int answerInd;

  /**
   * The default title of the Answer represented by answerInd.
   * This field is only populated for the following usageType values:
   * <ul>
   * <li> USAGE_ANWSER_COND
   * <li> USAGE_ANSWER_TITLE_COND
   * <li> USAGE_ANSWER_CONCL_COND
   * </ul>
   * If usageType has any other value, answerTitle will be null.
  **/
  public String answerTitle = null;


  //------------------------------------------------------------------------
  // Constructors
  //------------------------------------------------------------------------

  /** Used by OAValidate stored procedure. */
  UsageRec(int objectInd, short objectType, short usageType,
           boolean activeObject, boolean isError)
  {
    this(objectInd, objectType, usageType, activeObject, isError, -1);
  }

  /** Used by OAValidate stored procedure. */
  UsageRec(int objectInd, short objectType, short usageType,
           boolean activeObject, boolean isError, int answerInd)
  {
    this.objectInd    = objectInd;
    this.objectType   = objectType;
    this.usageType    = usageType;
    this.activeObject = activeObject;
    this.isError      = isError;
    this.answerInd    = answerInd;
  }

  //------------------------------------------------------------------------
  // equals/hashCode methods
  // equals() is needed to determine if a UsageRec already exists in 
  // a Vector
  //------------------------------------------------------------------------

  /**
   * Compare two UsageRec objects for eqality.
   * The UsageRec objects are equal if their objectType, objectInd,
   * activeObject and usageType fields are all equal.
  **/
  public boolean equals(Object o) {
    boolean isEqual = false;

    if ( o instanceof UsageRec ) {
      UsageRec other = (UsageRec) o;
      if ( this.objectType   == other.objectType   &&
           this.objectInd    == other.objectInd    &&
           this.activeObject == other.activeObject &&
           this.usageType    == other.usageType )
      {
        isEqual = true;
      }
    }

    return isEqual;
  }

  /**
   * Returns the objectInd field as the hashCode for the UsageRec object.
  **/
  public int hashCode() {
    return objectInd;
  }

}
