package com.ibm.nzna.projects.htmlProductImport ;              
/*-----------------------------------------------------------*/
/*   HTMLProductImport                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HTMLProductImport                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.qit.app.LocaleRec ;
import com.ibm.nzna.projects.qit.app.StatusRec ;
/**
 * This batch imports an HTML file that contains a table. The table
 * inside of the HTML document should contain information on a list
 * of products to insert or insert/update. The following parameters
 * are taken by this Batch:<br><br><pre>
 *
 * Parameter         Description                           Example
 * ----------------- ------------------------------------- --------
 * /D:               This is which database to import into /D:QUEST
 * /S:               This is which server the db is on     /S:CHAOS
 * /P:               This is the port number of JDBC       /P:5703
 * /INSERT           *Only tries to insert each product    /INSERT
 * /UPDATE           Only tries to update each product     /UPDATE
 * /INSERT_UPDATE    Tried to update, then insert          /INSERT_UPDATE
 * /USER:            The User Id to log into the db with   /USER:DLENTZ
 * /PASSWORD:        The password to log into the db with  /PASSWORD:MYPASS
 * /FILE:            The HTML file to load and process     /FILE:MYHTML.HTML
 * /DATABASEWIN      Show the database window              /DATABASEWIN      
 *
 * * = Default Value
 * </pre><br><br><br>
 *
 * This batch expects the HTML file to contain a table. Each row in the table
 * represents a product. The columns in the table are specified below:<br><pre>
 *
 * <b>Column      Description</b>
 * -------------- -------------------------
 *  1             Brand Name
 *  2             Family Name
 *  3             Product Number
 *  4             Machine Type
 *  5             Model
 *  6             Description
 *  7             Announce Date
 *  8             Country.
 *
 * All columns are <b>required</b>
 * </pre><br><br>
 *
 * @version 1.0, 4/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HTMLProductImport                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class HTMLProductImport extends Batch implements ProductConstants {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int UPDATEONLY          = 0 ;
   private static final int INSERTONLY          = 1 ;
   private static final int INSERTUPDATE        = 2 ;
   private static final int INDEX_BRAND         = 0 ;      
   private static final int INDEX_FAMILY        = 1 ;     
   private static final int INDEX_PRODUCTNUM    = 2 ; 
   private static final int INDEX_MACHINE       = 3 ;    
   private static final int INDEX_MODEL         = 4 ;      
   private static final int INDEX_DESCRIPT      = 5 ;   
   private static final int INDEX_ANNOUNCEDATE  = 6 ;
   private static final int INDEX_COUNTRY       = 7 ;    





   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String          databaseName         = null ;
   private String          portNum              = "5703" ;
   private String          serverName           = null ;
   private String          userId               = null ;
   private String          password             = null ;
   private String          importFilename       = null ;
   private BatchLogSystem  logSystem            = null ;
   private boolean         insertUpdate         = false ;
   private DatabaseSystem  dbSys                = null ;
   private int             step                 = 0 ;
   private boolean         continueImport       = true ;
   private int             updateType           = INSERTONLY ;
   private Vector          rowVec               = null ;
   private int             rowCount             = 0 ;
   private boolean         showDatabaseWin      = false ;
   private int             MAX_NUMBER_OF_STEPS  = 12 ; 





   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This is the main entry point for the batch.
    *  Like all batches, this batch will simply create
    *  a new instance of itself passing as it's only
    *
    *  @param parameter the Params passed in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String args[] ) {
      HTMLProductImport batch = new HTMLProductImport ( args ) ;

      if ( batch.getContinue () )
         batch.startBatch() ;

      System.exit( batch.getContinue() ? 1 : 0 ) ;

   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will initialize the BatchLogSystem.
    *  and the batch. It will then kick off and run
    *  the batch
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HTMLProductImport ( String args[] ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, always create our log system                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      logSystem = new BatchLogSystem ( "HTMLProductImport", this ) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read the params                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      continueImport = readParams ( args ) ;          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Validate the params                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueImport )
         continueImport = validateParams() ;

   }





   /*------------------------------------------------------------*/
   /* getContinue
    */
   /**
    *    This method will return TRUE if it is ok
    *  to start the import
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getContinue () {
      return( continueImport ) ;
   }


   /*------------------------------------------------------------*/
   /* startBatch
    */
   /**
    *    This method will start the batch
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void startBatch () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our database connection                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueImport ) {

         logProgress ( "Creating DatabaseSystem", MAX_NUMBER_OF_STEPS, step++ ) ;
         try {
            dbSys = new DatabaseSystem ( true, (LogSystem) logSystem, false ) ;

            if ( dbSys.createConnection ( DatabaseSystem.QUEST, 
                                          databaseName,
                                          serverName,
                                          portNum,
                                          userId,
                                          password ) )
               logProgress ( "Connected.", MAX_NUMBER_OF_STEPS, step++ ) ;
            else
               continueImport = false ;
         } catch ( Exception e ) {
            logSevereError ( e ) ;
            logProgress ( "Cannot connect", MAX_NUMBER_OF_STEPS, MAX_NUMBER_OF_STEPS ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Show the database window                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( continueImport ) && ( showDatabaseWin ) )
         new DatabaseDlg() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our type lists                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      new TypeList() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, run through the file                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueImport )
         continueImport = parseHTMLFile() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit the import                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueImport )
         commitImport() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clean up database                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      logProgress ( "Shutting Down", MAX_NUMBER_OF_STEPS, step++ ) ;


      logProgress ( "Batch Completed", MAX_NUMBER_OF_STEPS, step++ ) ;

      System.out.println ( "Steps:"+step ) ;
   }


   /*------------------------------------------------------------*/
   /* readParams
    */
   /**
    *    This method will traverse through the
    *  array of strings. Each string represents
    *  a parameter
    *
    *  @param args The parameters 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean readParams ( String[] args ) {                          
      boolean rc = false ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have any parameters, read all                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( args != null ) && ( args.length > 0 ) ) {
            int length = args.length ;
            int i      = 0 ;


            logSystem.beginLogSection ( "Reading Command Line Parameters. Number of params passed:"+args.length ) ;
            while ( i < length ) {
               processParam ( args[i].toUpperCase(), args[i] ) ;
               i++ ;
            }
            logSystem.endLogSection () ;
            rc=true ;
         }
      } catch ( Exception e ) {
         logSevereError ( e ) ;
      }

      return( rc ) ;

   }



   /*-----------------------------------------------------------*/
   /*   processParam                                            */           
   /*                                                           */
   /*   Description:                                            */
   /*      This method proceses one param                       */                                               
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void processParam ( String param, String mixedCaseParam ) throws Exception {                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out certain params. Hide others                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !param.startsWith ( "/PASSWORD:" ) )
         logProgress ( "Parameter:"+mixedCaseParam, MAX_NUMBER_OF_STEPS, step++ ) ;

      if ( param.startsWith ( "/D:" ) )
         databaseName = mixedCaseParam.substring ( 3, param.length() ) ;


      else if ( param.startsWith ( "/S:" ) )
         serverName = mixedCaseParam.substring ( 3, param.length() ) ;

      else if ( param.startsWith ( "/DATABASEWIN" ) )
         showDatabaseWin = true ;

      else if ( param.startsWith ( "/USER:" ) )
         userId = mixedCaseParam.substring ( 6, param.length() ) ;

      else if ( param.startsWith ( "/PASSWORD:" ) )
         password = mixedCaseParam.substring ( 10, param.length() ) ;

      else if ( param.startsWith ( "/P:" ) )
         portNum = param.substring ( 3, param.length() ) ;

      else if ( param.startsWith ( "/FILE:" ) )
         importFilename = mixedCaseParam.substring ( 6, param.length() ) ;

      else if ( param.startsWith ( "/INSERT_UPDATE" ) )
         updateType = INSERTUPDATE ;

      else if ( param.startsWith ( "/INSERT" ) )
         updateType = INSERTONLY ;

      else if ( param.startsWith ( "/UPDATE" ) )
         updateType = UPDATEONLY ;




      else if ( param.startsWith ( "/?" ) ) {
         printParams() ;
         System.exit(0) ;
      }

      else if ( param.startsWith ( "?" ) ) {
         printParams() ;
         System.exit(0) ;
      }


      /* If we still don't have the parameter, notify the user */
      else {
         System.out.println ( "Unknown parameter:"+param ) ;
      }
   }

   /*-----------------------------------------------------------*/
   /* printParams
    */
   /**
    *    This methos will print out all of our paramters
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void printParams () {
      System.out.println ( "" ) ;
      System.out.println ( "Usage: java com.ibm.nzna.projects.htmlProductImport.HTMLProductImport <params>" ) ;
      System.out.println ( "Parameter         Description                           Example" ) ;
      System.out.println ( "----------------- ------------------------------------- --------" ) ;
      System.out.println ( "/D:               This is which database to import into /D:QUEST" ) ;
      System.out.println ( "/S:               This is which server the db is on     /S:CHAOS" ) ;
      System.out.println ( "/P:               This is the port number of JDBC       /P:5703" ) ;
      System.out.println ( "/INSERT           *Only tries to insert each product    /INSERT" ) ;
      System.out.println ( "/INSERT           Only tries to update each product     /UPDATE" ) ;
      System.out.println ( "/INSERT_UPDATE    Tried to update, then insert          /INSERT_UPDATE" ) ;
      System.out.println ( "/USER:            The User Id to log into the db with   /USER:DLENTZ" ) ;
      System.out.println ( "/PASSWORD:        The password to log into the db with  /PASSWORD:MYPASS" ) ;
      System.out.println ( "/FILE:            The HTML file to load and process     /FILE:MYHTML.HTML" ) ;
      System.out.println ( "/?                Get this help                         /? " ) ;
      System.out.println ( "\n* = Default\n" ) ;

   }




   /*------------------------------------------------------------*/
   /* validateParams
    */
   /**
    *    This method will validate that we have enough
    *  information from the parameters to start
    *  the batch
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean validateParams () {
      boolean rc = false ;

      if ( databaseName == null )
         logSevereError ( "No database specified from command line parameter /D:" ) ;
      else if ( serverName == null )
         logSevereError ( "No server specified from command line parameter /S:" ) ;
      else if ( userId == null )
         logSevereError ( "No User Id specified from command line parameter /USER:" ) ;
      else if ( password == null )
         logSevereError ( "No password specified from command line parameter /PASSWORD:" ) ;
      else if ( importFilename == null )
         logSevereError ( "No import filename specified from command line parameter /FILE:" ) ;
      else
         rc = true ;


      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* commitImport
    */
   /**
    *    This method will commit the import. This is
    *  where each imported product will be inserted or updated
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void commitImport () {
      if ( ( rowVec != null ) && ( rowVec.size() > 0 ) ) {
         int              i           = 0 ;
         int              length      = rowVec.size() ;
         ProductRec       prodRec     = null ;


         while ( ( i < length ) && ( continueImport ) ) {
            prodRec = (ProductRec) rowVec.elementAt(i) ;

            importProductRec ( prodRec ) ;

            i++ ;

            logProgress ( "Committed Product:"+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ), MAX_NUMBER_OF_STEPS, step++ ) ;
         }


      }
   }



   /*------------------------------------------------------------*/
   /* parseHTMLFile
    */
   /**
    *    This method will attempt to read the HTML File,
    *  and pull out all of the rows. Each row will be added
    *  to the rowVec. A row is defined as being between
    *  the <TR> and </TR> html tag. The row added to the vector
    *  will be a string. This string contains the <TD> and </TD>
    *  tags.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if the parsing was good.
    *//*---------------------------------------------------------*/
   private boolean parseHTMLFile () {
      boolean rc = false ;

      logProgress ( "Parsing HTML Table", MAX_NUMBER_OF_STEPS, step++ ) ;

      try {
         String          inFile   = FileUtil.readFile ( importFilename ) ;
         boolean         cont     = true ;
         String          tempStr  = null ;
         boolean         error    = false ;
         int             rowIndex = 1 ;

         /* If we have read something from this file,
            process it now */
         if ( ( inFile != null ) && ( inFile.length() > 0 ) ) {

            /* First, clean the file  */
            inFile = cleanHTMLString ( inFile ) ;
            MAX_NUMBER_OF_STEPS+=rowCount ;

            /* If we have rows, continue  */
            if ( rowCount > 0 ) {

               /* First, check for a table tag. We cannot continue without it */
               if ( ( inFile.indexOf ( "<table" ) == -1 ) &&
                    ( inFile.indexOf ( "<TABLE" ) == -1 ) &&
                    ( inFile.indexOf ( "</table" ) == -1 ) &&
                    ( inFile.indexOf ( "</TABLE" ) == -1 ) )
                  logError ( "HTML File contains no table" ) ;

               else {

                  /* Upper case all <tr> and <td> tags */
                  inFile = Text.replaceAllStrInStr ( inFile, "<tr",  "<TR" ) ;
                  inFile = Text.replaceAllStrInStr ( inFile, "<td",  "<TD" ) ;
                  inFile = Text.replaceAllStrInStr ( inFile, "</tr", "</TR" ) ;
                  inFile = Text.replaceAllStrInStr ( inFile, "</td", "</TD" ) ;

                  /* Check for at least 1 row tag */
                  if ( inFile.indexOf ( "<TR" ) == -1 )
                     logError ( "The HTML Table contains no rows" ) ;

                  /* Check for at least 1 column tag */
                  else if ( inFile.indexOf ( "<TD" ) == -1 )
                     logError ( "The HTML Table contains no columns" ) ;

                  else {
                     int startIndex = 0 ;
                     int stopIndex  = 0 ; 

                     /* Trim our string down */
                     inFile = inFile.substring ( inFile.indexOf ( "<TR" ), inFile.length() ) ;

                     /* Initialize our row vector */
                     if ( rowVec != null )
                        rowVec.removeAllElements() ;
                     else
                        rowVec = new Vector ( 10, 10 ) ;

                     /* now walk the string */
                     while ( ( cont ) && ( !error ) && ( continueImport ) ) {

                        /* Advance the startIndex to the character
                           after the beginning <TR> Tag */
                        startIndex = inFile.indexOf ( "<TR", startIndex ) ;
                        if ( startIndex == -1 )
                           cont = false ;
                        if ( cont ) {
                           startIndex = inFile.indexOf ( ">", startIndex ) + 1 ;
                           if ( startIndex == -1 )
                              error = true ;
                        }


                        /* Advance the stopIndex to the character
                           before the ending </TR> tag */
                        if ( ( !error ) && ( cont ) ) {
                           stopIndex = inFile.indexOf ( "</TR", startIndex ) ;
                           if ( stopIndex == -1 )
                              error = true ;
                        }


                        /* Ok, if we haven't encountered any errors,
                           just add a product record for this string in */
                        if ( ( cont ) && ( !error ) ) {
                           ProductRec prodRec = null ;

                           /* Get just the text between the <TR> and </TR> tags */
                           tempStr = inFile.substring ( startIndex, stopIndex ) ;

                           /* Create a product record using this table row */
                           if ( ( tempStr.indexOf ( "<TD>" ) >= 0 ) && ( tempStr.indexOf(  "</TD>" ) >= 0 ) )
                              prodRec = createProdRecFromColumns ( tempStr, rowIndex ) ;
                           else
                              logError ( "There are no table cells in row "+rowIndex ) ;

                           if ( prodRec != null ) {
                              rowVec.addElement ( prodRec ) ;
                              logProgress ( "Parsed Product Number: "+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ), MAX_NUMBER_OF_STEPS, step++ ) ;
                              logProgress ( "       Description:    "+prodRec.getFieldData ( PRODFIELD_DESCRIPTION ),   MAX_NUMBER_OF_STEPS, step++ ) ;
                           }

                        }

                        rowIndex++ ;
                     } /* while */

                     /* If error is false, we finished processing all of the rows. */
                     if ( !error ) {

                        /* Trim the vector */
                        rowVec.trimToSize() ;

                        /* If we do not have elements, notify the user, otherwise everything is ok */
                        if ( rowVec.size() == 0 )
                           logError ( "There was no data in the table" ) ;
                        else {
                           rc = true ;
                        }
                     }

                  } /* else */
               } /* else */
            } /* if rowCount  */

         } /* if we have good data */
      } catch ( Exception e ) {
         logSevereError ( e ) ;
      }

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* createProdRecFromColumns
    */
   /**
    *    This method will take the html code that is
    *  between the <TR></TR> tags and attempt to
    *  create a product record from it.
    *
    *  @param colString The string containing the column information
    *  @param None
    *  @param None
    *
    *  @return ProductRec The new product record, NULL if an error occured
    *//*---------------------------------------------------------*/
   private ProductRec createProdRecFromColumns ( String colString, int rowIndex ) {
      ProductRec      retRec       = null ;
      int             startIndex   = 0 ;
      int             stopIndex    = 0 ;
      String          dataStr      = null ;
      boolean         cont         = true ;
      boolean         error        = false ;
      int             index        = 0 ;


      try {
         retRec = new ProductRec() ;

         while ( ( cont ) && ( !error ) && ( continueImport ) ) {

            /* Advance the index to the character after the first <TD> tag */
            startIndex = colString.indexOf ( "<TD", startIndex ) ;
            if ( startIndex == -1 )
               cont = false ;
            else {
               startIndex = colString.indexOf ( ">", startIndex )+1 ;

               if ( startIndex == -1 )
                  error = true ;
            }

            /* Now, advance the stop index to the character before the ending </TD> tag */
            if ( ( !error ) && ( cont ) ) {
               stopIndex = colString.indexOf ( "</TD>", startIndex ) ;

               if ( stopIndex == -1 )
                  error = true ;


               /* Ok, everything was good so far, get our data string */
               else {

                  dataStr = colString.substring ( startIndex, stopIndex ).trim() ;

                  error = !processColumnString ( retRec, dataStr, colString, index, rowIndex ) ;

                  index++ ;

               } /* else */

            } /* if cont */

         } /* while cont */

         if ( error )
            retRec = null ;

      } catch ( Exception e ) {
         logSevereError ( e ) ;
         retRec = null ;
      }


      return( retRec ) ;
   }






   /*------------------------------------------------------------*/
   /* processColumnString
    */
   /**
    *    This method will process the 1 string inside
    *  of the column tags. Please note that the 
    *  string must not contain any of the column tags
    *
    *  @param retRec The ProductRec that this string is applied to
    *  @param dataStr The data string
    *  @param fullStr This string should be the full row code. It is used for error reporting.
    *  @param index The index ( column number ) in the table
    *
    *  @return TRUE if no error was encountered
    *//*---------------------------------------------------------*/
   private boolean processColumnString ( ProductRec retRec, String dataStr, String fullStr, int index, int rowIndex ) {
      boolean rc = false ;

      retRec.autoSetProductNumber ( false ) ;

      switch ( index ) {
         case INDEX_BRAND :
            retRec.setFieldData  ( PRODFIELD_BRAND, TypeList.getInstance().objectFromString ( dataStr, TypeList.TYPE_BRAND ) ) ;
            if ( retRec.getFieldData ( PRODFIELD_BRAND ) == null )
               logError ( "The Brand was not found. Brand:"+dataStr+". This was in row "+rowIndex ) ;
            else
               rc = true ;
            break ;

         case INDEX_FAMILY :
            retRec.setFieldData ( PRODFIELD_FAMILY, TypeList.getInstance().objectFromString ( dataStr, TypeList.TYPE_FAMILY ) ) ;
            if ( retRec.getFieldData ( PRODFIELD_FAMILY ) == null )
               logError ( "The Family was not found. Family:"+dataStr+". This was in row "+rowIndex ) ;
            else
               rc = true ;
            break ;

         case INDEX_PRODUCTNUM :
            if ( dataStr != null ) {

               if ( dataStr.length() > ProductRec.LENGTH_PRODNUM )
                  retRec.setFieldData ( PRODFIELD_PRODUCTNUMBER, dataStr.substring ( 0, ProductRec.LENGTH_PRODNUM ) ) ;
               else if ( dataStr.length() != 0 )
                  retRec.setFieldData ( PRODFIELD_PRODUCTNUMBER, dataStr ) ;

               rc = true ;
            } else
               logError ( "The was no Product Number found This was in row "+rowIndex ) ;
            break ;

         case INDEX_MACHINE :
            if ( dataStr != null ) {

               if ( ( retRec.getFieldData( PRODFIELD_PRODUCTNUMBER ) == null ) ||
                    ( ((String)retRec.getFieldData( PRODFIELD_PRODUCTNUMBER )).length() < ProductRec.LENGTH_PRODNUM ) )
                  retRec.autoSetProductNumber ( true ) ;

               if ( dataStr.length() > ProductRec.LENGTH_MACHINE )
                  retRec.setFieldData ( PRODFIELD_MACHINE, dataStr.substring ( 0, ProductRec.LENGTH_MACHINE ) ) ;
               else
                  retRec.setFieldData ( PRODFIELD_MACHINE, dataStr )  ;

               rc = true ;
            } else
               logError ( "The was no Machine Type found This was in row "+rowIndex ) ;
            break ;

         case INDEX_MODEL :
            if ( dataStr != null ) {

               if ( ( retRec.getFieldData( PRODFIELD_PRODUCTNUMBER ) == null ) ||
                    ( ((String)retRec.getFieldData( PRODFIELD_PRODUCTNUMBER )).length() < ProductRec.LENGTH_PRODNUM ) )
                  retRec.autoSetProductNumber ( true ) ;

               if ( dataStr.length() > ProductRec.LENGTH_MODEL )
                  retRec.setFieldData ( PRODFIELD_MODEL, dataStr.substring ( 0, ProductRec.LENGTH_MODEL ) ) ;
               else
                  retRec.setFieldData ( PRODFIELD_MODEL, dataStr ) ; ;

               rc = true ;
            } else
               logError ( "The was no Model found This was in row "+rowIndex ) ;
            break ;

         case INDEX_DESCRIPT :
            if ( dataStr != null ) {

               if ( dataStr.length() > ProductRec.LENGTH_PRODDESC )
                  retRec.setFieldData ( PRODFIELD_DESCRIPTION, dataStr.substring ( 0, ProductRec.LENGTH_PRODDESC ) );
               else
                  retRec.setFieldData ( PRODFIELD_DESCRIPTION, dataStr ) ;

               rc = true ;
            } else
               logError ( "The was no Description found This was in row "+rowIndex ) ;
            break ;

         case INDEX_COUNTRY :
            rc = processGeoString ( retRec, dataStr, fullStr, rowIndex ) ;
            break ;

         case INDEX_ANNOUNCEDATE :
            rc = processDateString ( retRec, dataStr, fullStr, rowIndex ) ;
            break ;

         default:
            rc = true ;
            break ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* processGeoString       
    */
   /**
    *    This method will take a geo     string
    *  and add each geo     to the product. This method
    *  will parse through a comma delimeted string of countries.
    *
    *  @param retRec The product record to add countries to
    *  @param dataStr The string containing the countries
    *  @param fullStr The full string of the row html code. Used for error reporting
    *
    *  @return TRUE if data was added succesfully.
    *//*---------------------------------------------------------*/
   private boolean processGeoString ( ProductRec retRec, String dataStr, String fullStr, int rowIndex ) {
      boolean         rc     = false ;
      StringTokenizer strtok = new StringTokenizer ( dataStr, "," ) ;
      TypeGeoRec      geoRec = null ;
      String          geoStr = null ;
      LocaleRec       locRec = (LocaleRec) retRec.getFieldData ( PRODFIELD_LOCALE ) ;

      if ( locRec == null )
         locRec = new LocaleRec() ;

      while ( strtok.hasMoreTokens() ) {
         geoStr = strtok.nextToken().trim() ;
         geoRec = (TypeGeoRec) TypeList.getInstance().objectFromString ( geoStr, TypeList.TYPE_GEOGRAPHY ) ;
         if ( geoRec != null ) {
            locRec.addGeography ( geoRec ) ;
         } else
            logError ( "Cannot import product. The country was unknown. The Country ways: "+geoStr+" Current Row:"+rowIndex ) ;
      }

      retRec.setFieldData ( PRODFIELD_LOCALE, locRec ) ;
      rc = true ;
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* processDateFromString
    */
   /**
    *    This method will take a string representation
    *  of a date, make sure the date is valid, format
    *  the date to a 4 digit year, and then add the date as an 'Announced' status to the product
    *
    *  @param retRec The product record to add an announce status to
    *  @param dataStr The string containing the date
    *  @param fullStr The string containing the full row html code. Used for error reporting
    *  @param rowIndex The current rowIndex
    *
    *  @return TRUE if no errors, FALSE If an error was occured
    *//*---------------------------------------------------------*/
   private boolean processDateString ( ProductRec retRec, String dataStr, String fullStr, int rowIndex ) {
      boolean rc        = false ;
      int     dateCheck = 0 ;

      dataStr   = CDate.format4DigitYear ( dataStr, CDate.DATE_DATE ) ;
      dateCheck = CDate.checkDate ( dataStr, CDate.DATE_DATE ) ;

      if ( dateCheck != 0 )
         logError ( "The date was wrong. The date that was specified is:"+dataStr+".  Current row: "+rowIndex ) ;
      else {
         retRec.setFieldData ( PRODFIELD_STATUS, new StatusRec ( Status.TYPESTATUS_ANNOUNCED, dataStr ) ) ;
         rc = true ;
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* cleanHTMLString
    */
   /**
    *    This method will wipe out all non table
    *  related tags. It will also keep a count
    *  of the number of rows in the file. It will
    *  set the rowCount property of this import
    *  
    *
    *  @param htmlString the html string to clean
    *  @param None
    *  @param None
    *
    *  @return the cleaned html 
    *//*---------------------------------------------------------*/
   private String cleanHTMLString ( String HTMLString ) {
      HTMLParse htmlParse          = null ;
      HTMLTag   tagRec             = null ;
      String    tagStr             = null ;
      int       i                  = 0 ;

      /* First, add a new line character to certain tags */
      htmlParse = new HTMLParse ( HTMLString ) ;

      rowCount = 0 ;

      while ( ( i < htmlParse.tagCount() ) && ( continueImport ) ) {

         tagStr = htmlParse.getTag(i) ;
         if ( tagStr != null ) {

            tagStr = tagStr.toUpperCase() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Kill these tags                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( tagStr.indexOf ( "</TR" ) >= 0 ) {
               htmlParse.replaceTag ( i, "</TR>") ;
            } else if ( tagStr.indexOf ( "</TD" ) >= 0 ) {
               htmlParse.replaceTag ( i, "</TD>") ;
            } else if ( tagStr.indexOf ( "<TR" ) >= 0 ) {
               htmlParse.replaceTag ( i, "<TR>" ) ;
               rowCount++ ;
            } else if ( tagStr.indexOf ( "<TD" ) >= 0 ) {
               htmlParse.replaceTag ( i, "<TD>") ;
            } /* if */
            else if ( ( tagStr.indexOf ( "STYLE" ) >= 0 ) ||
                      ( tagStr.indexOf ( "FONT" ) >= 0 ) ||
                      ( tagStr.indexOf ( "APPLET" ) >= 0 ) ||
                      ( tagStr.indexOf ( "PARAM" ) >= 0 ) ||
                      ( tagStr.indexOf ( "HREF" ) >= 0 ) ||
                      ( tagStr.indexOf ( "FRAME" ) >= 0 ) ||
                      ( tagStr.indexOf ( "H1" ) >= 0 ) || 
                      ( tagStr.indexOf ( "H2" ) >= 0 ) ||
                      ( tagStr.indexOf ( "H3" ) >= 0 ) ||
                      ( tagStr.indexOf ( "H4" ) >= 0 ) ||
                      ( tagStr.indexOf ( "H5" ) >= 0 ) ||
                      ( tagStr.indexOf ( "H6" ) >= 0 ) ||
                      ( tagStr.indexOf ( "IMG" ) >= 0 ) ||
                      ( tagStr.indexOf ( "MAILTO" ) >= 0 ) ||
                      ( tagStr.indexOf ( "!" ) >= 0 ) ||
                      ( tagStr.indexOf ( "META" ) >= 0 ) ||
                      ( tagStr.indexOf ( "FORM" ) >= 0 ) ||
                      ( tagStr.indexOf ( "BGCOLOR" ) >= 0 ) ||
                      ( tagStr.indexOf ( "COLOR" ) >= 0 ) ||
                      ( tagStr.indexOf ( "SCRIPT" ) >= 0 ) ||
                      ( tagStr.indexOf ( "COMMENT" ) >= 0 ) ||
                      ( tagStr.indexOf ( "HTML" ) >= 0 ) ||
                      ( tagStr.indexOf ( "HEAD" ) >= 0 ) ||
                      ( tagStr.indexOf ( "P" ) >= 0 ) ||
                      ( tagStr.equals ( "<B>" ) ) ||
                      ( tagStr.equals ( "</B>" ) ) ||
                      ( tagStr.equals ( "<I>" ) ) ||
                      ( tagStr.equals ( "</I>" ) ) ||
                      ( tagStr.equals ( "<U>" ) ) ||
                      ( tagStr.equals ( "</U>" ) ) ||
                      ( tagStr.equals ( "<S>" ) ) ||
                      ( tagStr.equals ( "</S>" ) ) ) {
               htmlParse.removeTag ( i ) ;
            }
         } /* if tagStr */
         i++ ;
      }/* while */
      HTMLString = htmlParse.getHTMLString() ;

      return( HTMLString ) ;
   }





   /*------------------------------------------------------------*/
   /* importProductRec
    */
   /**
    *    This method will save the product record
    *  to the database. It will either update or insert
    *  the product 
    *
    *  @param sql The SQL Object to use
    *  @param prodRec The Product Record
    *  @param None
    *
    *  @return TRUE if the product was saved, FALSE if otherwise
    *//*---------------------------------------------------------*/
   private boolean importProductRec ( ProductRec prodRec ) {
      boolean rc = false ;


      /* First, we need to get the prodint for this product no matter
         what we are going to do in the database  */
      prodRec.setFieldData ( PRODFIELD_PRODINT, new Integer ( checkProdExist ( prodRec ) ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   INSERT_ONLY - We will only attempt to insert the product*/
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( updateType == INSERTONLY ) {
         if ( prodRec.getProdInt() != -1 )
            logError ( "Cannot insert Product Number: "+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER )+" because it is already in the database" ) ;
         else {
            prodRec.updateRecStatus ( prodRec.REC_NOCHANGE ) ;
            prodRec.updateRecStatus ( prodRec.REC_INSERT ) ;
            prodRec.setFieldData ( PRODFIELD_PRODINT, new Integer ( 0 ) ) ;
            rc = prodRec.saveToDatabase ( true ) ;
            if ( rc )
               logProgress ( "Inserted Product Number:"+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ), MAX_NUMBER_OF_STEPS, step++ ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   UPDATEONLY - We will only attempt to update the product */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( updateType == UPDATEONLY ) {
         if ( prodRec.getProdInt() != -1 ) {

            if ( !prodRec.checkProductLock ( ) ) {
               prodRec.updateRecStatus ( prodRec.REC_NOCHANGE ) ;
               prodRec.updateRecStatus ( prodRec.REC_UPDATE ) ;
               rc = prodRec.saveToDatabase ( true ) ;
               if ( rc )
                  logProgress ( "Updated Product Number:"+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ), MAX_NUMBER_OF_STEPS, step++ ) ;
            } else
               logError ( "Cannot update Product because it is locked. Product Number:"+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ) ) ;
         } else
            logError ( "Cannot update Product because it does not exist. Product Number:"+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ) ) ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   INSERTUPDATE - Will try to update, and then insert      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( updateType == INSERTUPDATE ) {
         if ( prodRec.getProdInt() != -1 ) {

            if ( !prodRec.checkProductLock ( ) ) {
               prodRec.updateRecStatus ( prodRec.REC_NOCHANGE ) ;
               prodRec.updateRecStatus ( prodRec.REC_UPDATE ) ;
               rc = prodRec.saveToDatabase ( true ) ;
               if ( rc )
                  logProgress ( "Updated Product Number: "+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ), MAX_NUMBER_OF_STEPS, step++ ) ;
            } else
               logError ( "Cannot update product because it is locked. Product Number:"+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ) ) ;
         } else {
            prodRec.updateRecStatus ( prodRec.REC_NOCHANGE ) ;
            prodRec.updateRecStatus ( prodRec.REC_INSERT ) ;
            prodRec.setFieldData ( PRODFIELD_PRODINT, new Integer ( 0 ) ) ;
            rc = prodRec.saveToDatabase ( true ) ;
            if ( rc )
               logProgress ( "Updated Product Number: "+prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ), MAX_NUMBER_OF_STEPS, step++ ) ;
         }
      }
      return( rc ) ;
   }

   /*-----------------------------------------------------------*/
   /* checkProdExist
    */
   /**
    *    This method will check to see if a product
    *  exists. If it does, this method will return the product's
    *  prodint.
    *             
    *  @param docRec The DocRec to check
    *  @param 
    *  @param 
    *
    *  @return The product's prodint, otherwise it will return -1
    */
   private int checkProdExist ( ProductRec prodRec ) {
      String    rtnName = "checkProdExist" ;
      SQLMethod sql    = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      String    strStmt = "SELECT PRODINT FROM TIGRIS.PRODUCTS WHERE PRODNUM = '" ;
      int       prodInt = -1 ;   

      try {

         if ( prodRec != null ) {
            String productNumber = (String) prodRec.getFieldData ( PRODFIELD_PRODUCTNUMBER ) ;

            if ( ( productNumber != null ) &&
                 ( productNumber.length() != 0 ) &&
                 ( sql != null ) ) {
               Statement stmt = sql.createStatement() ;
               ResultSet rs   = null ;

               strStmt = strStmt+productNumber+"' FOR FETCH ONLY" ;

               rs = stmt.executeQuery ( strStmt ) ;

               if ( rs.next() )
                  prodInt = rs.getInt(1) ;

               rs.close() ;
            } /* if prodNum */
         } /* if prodRec */
      } catch ( Exception e ) {
         logSevereError ( e ) ;
         sql.rollBack() ;
      }

      sql.close() ;

      return( prodInt ) ;
   }

} /* HTMLProductImport */
