/* DBAPI.java is the core API Class and directly interacts with the database classes */

package com.ibm.nzna.projects.oa.engine;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.sql.*;
import com.ibm.nzna.projects.common.quest.oa.*;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.util.*;    // LogSystem, Text
import com.ibm.nzna.shared.db.*;      // DatabaseSystem

/* HISTORY
 *
 *  04/28-06/30  Venu developed and debugged most of it
 *  08/03  mdr  add calls to OAMemoryInterface methods around all getSourceFlag() tests
 *  08/07  mdr  change the String names for the tags "DA", "AA" etc. from DA to DraftAction, etc.
 *              make spacing between methods uniform and line up a few of the javadoc comments
 *  08/09  mdr  revise debug prints to be uniform in line spacing, etc.
 *  08/14  mdr  change logic for memory/draft/active to also include lookup in active last
 *  08/16  mdr  change loop limits in concatenateTwoVectors + debug preceding
 *  08/17  mdr  add null tests to concatenateTwoVectors + callers send empty Vector, not null A
 *              allow getAnswerTitle to return null, getTitleStringForAnswer return empty string
 *              allow getAnswerByInd to return null, also getTitleStringForAction, -Question
 *  08/23  mdr  shorten debug messages for conditions valid true and DocClassInd = whatever
 *              remove getMemoryInterface and use new public var qitMem, to be set by OAEngine
 *                (or whoever calls this one's constructor)
 *  08/29  mdr  add close() (two) to getProductConclusionsForNewSym; move close() to finally in
 *                a couple of places
 *              fix trailing comments for several methods
 *  09/06  mdr  add node indicator argument to getDraftQuestionFromDatabase, check that question
 *                indicator is really draft based on column in node (use getDraftObjectInd)
 *  09/08  mdr  add node indicator arg to getValidAnswersForQuestion, getTitlesForValidAnswers,
 *                getQuestionByInd, getAnswerByInd, getAnswersForQuestion
 *  09/16  mdr  add debug print to getDraftQuestionFromDataBase
 *  09/17  mdr  revise getDraftActionFromDataBase and getDraftSymptomFromDataBase similar to 9/06
 *              add node arg to getSymptomObject, getActionByInd
 *              remove version of getSymptomObject with symptom indicator object - always current
 *              add true arg to getEdgesForNodeAndAnswer when drafts wanted
 *  09/18  mdr  new method getTitleForCurrentSymptom
 *              cosmetic / comment changes
 *  09/20  mdr  add log path as argument to constructor, class variable holds value; also class
 *                variable for the log file name; replace usages of oadbconnection.log with vars
 *  10/03  mdr  change init. in getLinkURLsForAction from null to new vector (var linkURLs)
 *                + same for getLinkURLsForQuestion
 *  10/16  mdr  add useAppDrv switch to constructor arg list
 *  11/01  mdr  replace println(...) with LogSystem.log(1, ...) (all are inside Debug)
 *              new getDraftSymptomObject() -- used by getDraftSymptomFromDataBase() to avoid
 *                infinite loop -- also new getDraftSymptomFromDataBaseOnly()
 *         TO DO:
 *              a lot of these methods don't really need the oas parameter -- could simplify
 *
 * END HISTORY
 */

/*********************************************************
 * This Class directly interfaces with the database to get 
 * the nodes,symptoms,questions,answers and actions... 
 * The class also handles the connection creation to the 
 * quest and clio databases.
 *
 * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
 * Created: 4/28/2000  10:17am
 * @author  Venu Koneru   
 * @version 1.0
 *********************************************************/

public class DBAPI implements Serializable {
    String DraftAction    = "DA";
    String ActiveAction   = "AA";
    String ActiveQuestion = "AQ";
    String DraftQuestion  = "DQ";
    String DraftSymptom   = "DS";
    String ActiveSymptom  = "AS";

    private OAMemoryInterface qitMem = null;
    private static String database   = "";       // Database name to connect
    private static String servName   = "";       //The server name on which database sits
    private static String port       = "";       // port number to connect to database
    private static String userId     = "";       //userid to connect
    private static String passwd     = "";       //passwd to connect 
    private static String logpath    = "";       // path where log file will be written
    private static String logname    = "oadbconnection.log";
    private static boolean useAppDrv = false;
    private boolean Debug            = false;     // switch for debug outputs set at compile time

    /* Constructor */
    public DBAPI(String dbname,String server,String portNum,String user,String password,
                 OAMemoryInterface memi,String lpath,boolean appDrv) throws Exception {

        database    = dbname;
        servName    = server;
        port        = portNum;
        userId      = user;
        passwd      = password;
        qitMem      = memi;
        logpath     = lpath;
        useAppDrv   = appDrv;

        init();

        //System.out.println("INIT function has been called");
        /*constructor for the class does nothing*/
    }


    /*********************************************************
    * Get a Symptom Object given SessionState
    *
    * @param nodInd Node indicator of node containing symptom
    * @param oas SessionState object
    *
    * @return Symptom Object - Returns the current Symptom Object
    *********************************************************/
    public Symptom getSymptomObject(int nodInd, OASessionState oas)throws Exception {

        Symptom sym     = null;
        int     symInd  = oas.getCurrentSymptom();  // assume current symptom always
        int     langId  = oas.getLanguage();

        if (oas.getSourceFlag()) {
            sym = qitMem.getDraftSymptomFromMemory(symInd, langId);
            if (sym == null) sym = getDraftSymptomFromDataBase(symInd, nodInd, oas);
            oas.setNodeFlag(DraftSymptom);
        }
        if (sym == null) {
            sym = getSymptomFromDataBase(symInd,langId);
            oas.setNodeFlag(ActiveSymptom);
        }
        if (Debug) {
            LogSystem.log(1, "DBAPI: Symptom Ind ="+symInd+" (current, "+oas.getNodeFlag()+") "
                               + sym);
        }
        return sym;
    }//getSymptomObject()


    /*********************************************************
    * Get a Draft Symptom Object given SessionState (if it exists)
    *
    * @param nodInd Node indicator of node containing symptom
    * @param oas SessionState object
    *
    * @return Symptom Object - Returns the current Symptom Object or null
    *********************************************************/
    public Symptom getDraftSymptomObject(OASessionState oas)throws Exception {

        int     symInd  = oas.getCurrentSymptom();  // assume current symptom always

        Symptom sym = getDraftSymptomFromDataBaseOnly(symInd, oas);
        if (sym != null) oas.setNodeFlag(DraftSymptom);
        if (Debug) {
            LogSystem.log(1, "DBAPI: Draft Symptom Ind ="+symInd+" (current, "
                          + oas.getNodeFlag()+") " + sym);
        }
        return sym;
    }//getDraftSymptomObject()


    /*********************************************************
    * Get a Node Object given a Node indicator and Symptom indicator
    *
    * @param nodeInd Node Indicator of the Node Object
    * @param symInd Symptom Indicator of the Symptom
    *
    * @return Node Object - Returns the corresponding Node Object
    *********************************************************/
    public Node getNodeByInd(int nodeInd, Symptom sym) throws Exception {
        Node node = null;
        if (nodeInd != 0) node = sym.getNodeByInd(nodeInd);
        else if (Debug) 
            LogSystem.log(1, "DBAPI/getNodeByInd: ERROR: node indicator is "+nodeInd);
        return node;
    }//getNodeByInd()


    /*********************************************************
    * Get a Root Node Object given a Symptom 
    *
    * @param sym the Symptom for which the root node is needed
    * @param oas SessionState object
    *
    * @return Node Object - Returns the corresponding Node Object
    *********************************************************/
    public Node getRootNode(Symptom sym, OASessionState oas) throws Exception {

        Node node   = sym.getRootNode();
        return node;
    }//getRootNode()


    /*********************************************************
    * Get the OA Object given a Node
    *
    * @param node the Node for which the OA Object is needed
    *
    * @return ind - Returns the corresponding indicator of the OAObject
    *********************************************************/
    public int getOAObjectIndForNode(Node node,OASessionState oas) throws Exception {

        int ind = 0;

        if (oas.getSourceFlag()) {
            ind = qitMem.getMemoryObjectInd(node);
            if (ind == 0) ind = node.getDraftObjectInd();
        }
        if (ind == 0) {
            ind = node.getActiveObjectInd();
        }

        return ind;
    }//getOAObjectIndForNode()


    /*********************************************************
    * Get a Root Node Indicator for a given node 
    *
    * @param sym the Symptom for which the root node is needed
    * @param oas SessionState object
    *
    * @return Node Object - Returns the corresponding Node Object
    *********************************************************/
    public int getRootNodeInd(Symptom sym, OASessionState oas)throws Exception {
        int nodeId   = sym.getRootNodeInd();
        return nodeId;
    }//getRootNodeInd()


    /*********************************************************
    * Get a Question Object given a Question indicator and Node indicator
    *
    * @param quesInd Question Indicator of the Question Object
    * @param nodeInd Node Indicator of the Question Node Object
    * @param oas OA SessionState object
    *
    * @return Question - the corresponding Question Object
    *********************************************************/
    public Question getQuestionByInd(int qInd, int nInd, OASessionState oas)throws Exception {

        Question ques   = null;
        int      langId = oas.getLanguage();

        if (oas.getSourceFlag()) {
            ques = qitMem.getDraftQuestionFromMemory(qInd, langId);
            if (ques == null)
                ques = getDraftQuestionFromDataBase(qInd, nInd, oas);
            oas.setNodeFlag(DraftQuestion);
        }
        if (ques == null) {
            ques = getQuestionFromDataBase(qInd, langId);
            oas.setNodeFlag(ActiveQuestion);
        }
        if (Debug && ques == null)
            LogSystem.log(1, "DBAPI/getQuestionByInd: Warning: returning null for qInd = "
                               + qInd + " and node " + nInd + " within "
                               + oas.getCurrentSymptom() + " (Language " + langId + ", flag " 
                               + oas.getNodeFlag() + ")");

        return ques;
    }//getQuestionByInd()


    /*********************************************************
    * Get action object given action indicator
    *
    * @param aInd Action indicator of the Action Object
    * @param nInd Node indicator of node containing Action Object
    * @param oas SessionState object
    *
    * @return Action Object - Returns the corresponding Action Object
    *********************************************************/
    public Action getActionByInd(int aInd, int nInd, OASessionState oas) throws Exception {

        Action act = null;
        int langId = oas.getLanguage();

        if (oas.getSourceFlag()) {
            act = qitMem.getDraftActionFromMemory(aInd, langId);
            if (act == null) act = getDraftActionFromDataBase(aInd, nInd, oas);
            oas.setNodeFlag(DraftAction);
        }
        if (act == null) {
            act = getActionFromDataBase(aInd, langId);
            oas.setNodeFlag(ActiveAction);
        }
        if (Debug && act == null)
            LogSystem.log(1, "DBAPI/getQuestionByInd: Warning: returning null for aInd = "
                               + aInd + " and node " + nInd + " within "
                               + oas.getCurrentSymptom());

        return act;
    }//getActionByInd()


    /*********************************************************
     * Get action indicator given Action object
     *
     * @param act: the Action Object
     * @param oas: SessionState object
     *
     * @return int - Returns the corresponding Action indicator
    *********************************************************/
    public int getActionInd(Action act,OASessionState oas)throws Exception {

        int actInd = 0;
        actInd=act.getInd();
        return actInd;
    }//getActionInd()
    

    /*********************************************************
    * Get a Question indicator given a Question Object
    *
    * @param ques: the Question Object
    * @param oas: SessionState object
    *
    * @return int - Returns the corresponding Question indicator
    *********************************************************/
    public int getQuestionInd(Question ques,OASessionState oas)throws Exception {

        int quesInd = 0;
        quesInd=ques.getInd();
        return quesInd;
    }//getQuestionInd()
    

    /*********************************************************
    * Get a Answer Objects given a Question indicator
    *
    * @param quesInd Question Indicator of the Question Object
    * @param nodeInd node indicator
    * @param oas SessionState object
    *
    * @return Vector - Returns a Vector of Answers
    *********************************************************/
    public Vector getAnswersForQuestion(int quesInd, int nodeInd, OASessionState oas)
    throws Exception {

        Question ques =  getQuestionByInd(quesInd, nodeInd, oas);
        Vector answers = ques.getAnswers();
        return answers;
    }//getAnswersForQuestion()

    
    /*********************************************************
    * Get an Answer Object given an Answer indicator
    *
    * @param ansInd Answer Indicator 
    * @param nodeInd node indicator
    * @param oas SessionState object
    *
    * @return Answer - Returns an Answer Object
    *********************************************************/
    public Answer getAnswerByInd(int quesInd, int nodeInd, int ansInd, OASessionState oas)
    throws Exception {

        Question ques = getQuestionByInd(quesInd, nodeInd, oas);
        if (ques == null) return null;
        Answer ans = ques.getAnswerByInd(ansInd);
        return ans;
    }//getAnswerByInd()

    
    /*********************************************************
    * Get an Answer indicator given a AnswerConcl object
    *
    * @param cncl Conclusion for an answer
    *
    * @return ansId - Returns the Answer Indicator of the answer the 
    *                conclusion points to.
    *********************************************************/
    public int getAnswerForConclusion(AnswerConcl c) throws Exception {

        int ans = c.getToAnswerInd();
        return ans;
    }//getAnswerForConclusion()


    /*********************************************************
    * Get an Question indicator given a AnswerConcl object
    *
    * @param cncl Conclusion for an answer
    *
    * @return quesId - Returns the Question Indicator of the Question the 
    *                conclusion points to.
    *********************************************************/
    public int getQuestionForConclusion(AnswerConcl c)throws Exception {

        int ques = c.getToQuestionInd();
        return ques;
    }//getQuestionForConclusion()

    
    /*
    * Get a Answer Object given a Answer indicator
    *
    * @param aInd Answer Indicator of the Answer Object
    * @param oas SessionState object
    *
    * @return Answer Object - Returns the corresponding Answer Object
    */
    // public Answer getAnswerByInd(int aInd){
    //Answer ans = new Answer(aInd);
    //return ans;
    //}//getAnswerByInd()


    /*********************************************************
    * Get a preanswers for a given Edge
    *
    * @param Edge e
    *
    * @return Vector of EdgePreanswer
    *********************************************************/
    public Vector getPreanswers (Edge e)throws Exception {

        Vector preanswers = e.getPreanswers();
        return preanswers;
    }//getPreanswers()

    
    /*********************************************************
    * Get a unanswers for a given Edge
    *
    * @param Edge e
    *
    * @return Vector of EdgeUnanswer
    *********************************************************/
    public Vector getUnanswers (Edge e)throws Exception {

        Vector unanswers = e.getUnanswers();
        return unanswers;
    }//getUnanswers()

    
    /*********************************************************
    * Get Actionlist for a given Edge
    *
    * @param Edge e
    *
    * @return Vector of EdgeAlist
    *********************************************************/
    public Vector getActionListsForEdge (Edge e)throws Exception {

        Vector alists = e.getActionList();
        return alists;
    }//getActionListsForEdge()

    
    /*********************************************************
    * Get the Action indicator associated with each Alist
    *
    * @param ealist - EdgeAlist Object
    * @param oas - SessionState object
    *
    * @return int - the Action indicator
    *********************************************************/
    public int getActionForEdgeAlist(EdgeAlist ealist,OASessionState oas)throws Exception {

        int aid = 0;

        if (oas.getSourceFlag()) {
            aid = qitMem.getMemoryActionInd(ealist);
            if (aid == 0) aid = ealist.getDraftActionInd();
        }
        if (aid == 0) aid = ealist.getActiveActionInd();
        return aid;
    }//getActionForEdgeAlist()


    /*********************************************************
    * Get the question indicator associated with each unanswer
    *
    * @param unanswer-EdgeUnanswer Object
    * @param oas SessionState object
    *
    * @return int-the question indicator
    *********************************************************/
    public int getQuestionForEdgeUnanswer(EdgeUnanswer unanswer,OASessionState oas)throws Exception {

        int qid = 0;

        if (oas.getSourceFlag()) {
            qid = qitMem.getMemoryUnQuestionInd(unanswer);
            if (qid == 0) qid = unanswer.getDraftQuestionInd();
        }
        if (qid == 0) qid = unanswer.getActiveQuestionInd();
        return qid;
    }//getQuestionForEdgeUnanswer()

    
    /*********************************************************
    * Get the question indicator associated with each preanswer
    *
    * @param preanswer-EdgePreanswer Object
    * @param oas SessionState object
    *
    * @return int-the question indicator
    *********************************************************/
    public int getQuestionForEdgePreanswer(EdgePreanswer preanswer,OASessionState oas)throws Exception {

        int qid = 0;

        if (oas.getSourceFlag()) {
            qid = qitMem.getMemoryPreQuestionInd(preanswer);
            if (qid == 0) qid = preanswer.getDraftQuestionInd();
        }
        if (qid == 0) qid = preanswer.getActiveQuestionInd();
        return qid;
    }//getQuestionForEdgePreanswer()

    
    /*********************************************************
    * Get the Answerindicator associated with each preanswer
    *
    * @param preanswer-EdgePreanswer Object
    * @param oas SessionState object
    *
    * @return int-the Answerindicator
    *********************************************************/
    public int getAnswerForEdgePreanswer(EdgePreanswer preanswer,OASessionState oas)throws Exception {

        int aid = 0;

        if (oas.getSourceFlag()) {
            aid = qitMem.getMemoryAnswerInd(preanswer);
            if (aid == 0) aid = preanswer.getDraftAnswerInd();
        }
        if (aid == 0) aid = preanswer.getActiveQuestionInd();

        return aid;
    }//getAnswerForEdgePreanswer()

    
    /*********************************************************
    * Get all the edges for a node
    *
    * @param oas SessionState object
    *
    * @return Hashtable-the Hashtable containing all the edges
    *********************************************************/
    public Hashtable getEdgesForNodeAndAnswer(OASessionState oas)throws Exception {
        
        int cr          = oas.getCurrentResponse();
        int cn          = oas.getCurrentNode();
        Symptom s       = getSymptomObject(cn, oas);
        Hashtable Edges = new Hashtable();

        if (oas.getSourceFlag()) {
            Edges = qitMem.getEdgesForMemoryNodeAndAnswer(s, cn, cr);  // get from memory objects
            if (Edges.isEmpty()) Edges = s.getEdgesForNodeAndAnswer(cn, cr, true); // get drafts
            // note Venu had no arg there, which defaults to false, was same as next - mdr 17 Sep
        }
        if (Edges.isEmpty()) Edges = s.getEdgesForNodeAndAnswer(cn, cr, false);  // get actives
        return Edges;

    }//getEdgesForNodeAndAnswer()


    /*********************************************************
    * Get all the Other edges for a node
    *
    * @param oas SessionState object
    *
    * @return Hashtable-the Hashtable containing all the other edges
    *********************************************************/
    public Hashtable getOtherEdgesForNode(OASessionState oas)throws Exception {

        int cn          = oas.getCurrentNode();
        Symptom s       = getSymptomObject(cn, oas);
        Hashtable Edges = s.getTypeOtherEdgesForNode(cn);
        return Edges;

    }//getOtherEdgesForNode()

    
    /*********************************************************
    * Get docClassInd for an edge
    *
    * @param edge
    *
    * @return int-the docClassInd for the edge
    *********************************************************/
    public int getEdgeDocClassInd(Edge e)throws Exception {
        int docClassInd = e.getDocClass().ind;
        return docClassInd;
    }//getEdgeDocClassInd()


    /*********************************************************
    * Get docClassInd for a Conclusion
    *
    * @param conclusion
    *
    * @return int-the docClassInd for the Conclusion
    *********************************************************/
    public int getAnswerConclDocClassInd(AnswerConcl c)throws Exception {
        int docClassInd = c.getDocClass().ind;
        return docClassInd;
    }//getAnswerConclDocClassInd()

    
    /*********************************************************
    * Get CountryList for an Edge
    *
    * @param Edge
    *
    * @return Vector-the CountryList for the Edge
    *********************************************************/
    public Vector getEdgeCountryList(Edge e)throws Exception {
        Vector CountryList = e.getCountryList();
        return CountryList;
    }//getEdgeCountryList()   


    /*********************************************************
    * Get CountryList for an AnswerConcl
    *
    * @param AnswerConcl
    *
    * @return Vector-the CountryList for the AnswerConcl
    *********************************************************/
    public Vector getAnswerConclCountryList(AnswerConcl e)throws Exception {
        Vector CountryList = e.getCountryList();
        return CountryList;
    }//getAnswerConclCountryList()   


    /*********************************************************
     * Get docClassInd for an Answer
     *
     * @param Answer
     *
     * @return int-the docClassInd for the Answer
    *********************************************************/
    public int getAnswerDocClassInd(Answer ans)throws Exception {
        int docClassInd = ans.getDocClass().ind;
        return docClassInd;
    }//getAnswerDocClassInd()

    
    /*********************************************************
    * Get CountryList for an Answer
    *
    * @param Answer
    *
    * @return Vector-the CountryList for the Answer
    *********************************************************/
    public Vector getAnswerCountryList(Answer ans)throws Exception {
        Vector CountryList = ans.getCountryList();
        return CountryList;
    }//getAnswerCountryList()


    /*********************************************************
    * Get ConditionTerms for any Conditional Objects like edges,
    * titles,questions,answers,
    *
    * @param conditions
    *
    * @return Vector-the ConditionTerms Vector for the Conditional
    *                Object
    *********************************************************/
    public boolean areConditionsValid(Conditions cons, OASessionState oas)throws Exception {
        // getconditions is a method in the conditionalobject class 
        // which the edge obect extends
        boolean flag            = false;
        Vector conditionTerms   = cons.getConditionTerms();
        if (!conditionTerms.isEmpty()) {
            if (cons.size()>0) {
                if ( oas.testCondition(conditionTerms)) {
                    flag=true;
                }
            }
            else {
                flag=true;
            }
        }
        else
            flag = true;

        if (Debug) {
            if (!flag) LogSystem.log(1, "DBAPI: Conditions valid: "+flag);
            else System.out.print(".");
        }
        return flag;
    }//areConditiosValid()


    /*********************************************************
    * This methods returns a vector of valid answers for a question.
    *
    * @param quesInd: indicator for Question whose answers are needed
    * @param nodeInd: the node indicator for the question
    * @param oas:     the OASessionState object of the current session
    * @return Vector: Answer objects valid for the Question
    *********************************************************/
    public Vector getValidAnswersForQuestion(int quesInd, int nodeInd, OASessionState oas)
    throws Exception {

        Vector validAns = new Vector();
        Vector answers  = getAnswersForQuestion(quesInd, nodeInd, oas);
        for (int i=0;i<answers.size();i++) {
            Answer answer   = (Answer)answers.elementAt(i);
            Vector cList    = getAnswerCountryList(answer);
            if (isCountryMatch(cList,oas)) {
                int docInd  = getAnswerDocClassInd(answer);
                if (isDocClassIndMatch(docInd,oas)) {
                    Conditions conditions   = answer.getConditions();
                    if (areConditionsValid(conditions,oas)) {
                        validAns.addElement(answer);
                    }//if
                }//if
            }//if
        }//for

        return validAns;
    }//getValidAnswersForQuestion()


    /*********************************************************
    * Method to get the titles of the valid answers for a question
    *
    * @param quesInd: Question indicator
    * @param nodeInd: Node indicator
    * @param oas:     OASessionState object
    * @return Hashtable: with answer indicator and the title
    *********************************************************/
    public Hashtable getTitlesForValidAnswers(int quesInd, int nodeInd, OASessionState oas)
    throws Exception {

        Vector answers  = getValidAnswersForQuestion(quesInd, nodeInd, oas);
        Hashtable titles = new Hashtable();
        for (int i=0;i<answers.size();i++) {
            Answer answer   = (Answer)answers.elementAt(i);
            int ansInd      = answer.getInd();
            String ansTitle = getTitleStringForAnswer(answer,oas);
            titles.put(new Integer(ansInd),ansTitle);
        }
        return titles;
    }//getTitlesForValidAnswers()

    
    /*********************************************************
    * Method to get titles for Actions 
    *
    * @param act:The action object
    * @param oas: OASessionState Object
    * @return Title: the title of the Action object
    *********************************************************/
    public Title getActionTitle(Action act,OASessionState oas)throws Exception {

        Title title=null;
        //find flag needed to break out of loop and return the first match

        boolean find= false;
        Vector titles   = act.getTitles();
        int i=0;
        while ((i<titles.size()) && !(find)) {
            title = (Title)titles.elementAt(i);
            int ind     = title.getDocClass().ind;

            if ((isDocClassIndMatch(ind,oas)) && !(find)) {
                Vector clist    = title.getCountryList();
                if ( isCountryMatch(clist,oas)) {
                    Conditions con  = title.getConditions();
                    if (areConditionsValid(con,oas)) {
                        find = true;

                    }
                }
            }//if
            i++;
        }//while
        //stop here not completed
        //}//if
        return title;
    }//getActionTitle()


    /*********************************************************
    * Method to get title for Answer 
    *
    * @param ans:The answer object
    * @param oas: OASessionState Object
    * @return title: the title for the answer 
    *********************************************************/
    public Title getAnswerTitle(Answer ans, OASessionState oas)throws Exception {

        Title title=null;
        //find flag needed to break out of loop and return the first match
        boolean find= false;
        int tind = 0;

        if (ans == null) return null;
        Vector titles = ans.getTitles();
        int i=0;
        while (i<titles.size() && !find) {
            title = (Title)titles.elementAt(i);
            tind = title.getDocClass().ind;
            if (isDocClassIndMatch(tind, oas) && !find) {
                if (isCountryMatch(title.getCountryList(), oas))
                    find = areConditionsValid(title.getConditions(), oas);
            }//if
            i++;
        }//while
        return title;
    }//getAnswerTitle()

    
    /*********************************************************
    * Method to get title for an Question 
    *
    * @param ques: the Question object
    * @param oas: OASessionState Object
    * @return Vector: the vector of title strings objects 
    *********************************************************/
    public Title getQuestionTitle(Question ques,OASessionState oas)throws Exception {

        Title title=null;
        //find flag needed to break out of loop and return the first match
        boolean find= false;

        Vector titles   = ques.getTitles();
        int i=0;
        while ( (i<titles.size()) && !(find) ) {

            title = (Title)titles.elementAt(i);
            int ind     = title.getDocClass().ind;


            if (isDocClassIndMatch(ind,oas) ) {
                Vector clist    = title.getCountryList();
                if ( isCountryMatch(clist,oas)) {
                    Conditions con  = title.getConditions();
                    if (areConditionsValid(con,oas)) {
                        find = true;
                    }
                }
            }//if
            i++;
        }//while
        //stop here not completed
        //}//if
        return title;
    }//getQuestionTitle()


    /*********************************************************
    * Method to get AddtlInfo for a actions in a list
    * list
    *
    * @param : OASessionState Object
    * @return Vector: a vector of additional Info strings
    *********************************************************/
    public AddtlInfo getAddtlInfoForAction(Action act,OASessionState oas)throws Exception {

        //Vector strAddInfo = new Vector();
        Vector Actions;  
        AddtlInfo addInfo=null;
        //find flag needed to break out of loop and return the first match

        //Enumeration actList = oas.getActionList();
        //if (actList.hasMoreElements()) {
        boolean find= false;
        //int actInd      = ((Integer)actList.nextElement()).intValue();
        //Action act      = getActionFromDataBase(actInd,oas.getLanguage());
        Vector addtlInfo= act.getAdditionalInfo();
        int i = 0;
        while ((i <addtlInfo.size()) && !(find)) {
            addInfo= (AddtlInfo)addtlInfo.elementAt(i);
            int ind     = addInfo.getDocClass().ind;

            if ((isDocClassIndMatch(ind,oas)) && !(find)) {
                Vector clist    = addInfo.getCountryList();
                if ( (isCountryMatch(clist,oas)) && !(find) ) {
                    Conditions con  = addInfo.getConditions();
                    if ((areConditionsValid(con,oas)) && !(find)) {
                        find = true;
                    }
                }
            }//if
            i++;
        }//while
        //stop here not completed
        //}//if
        if (find) {
            return addInfo;
        }
        else
            return null;

    }//getAddtlInfoForAction()


    /*********************************************************
    * Method to get AddtlInfo for a Question
    *
    * @param : OASessionState Object
    * @return Vector: a vector of additional Info strings
    *********************************************************/
    public AddtlInfo getAddtlInfoForQuestion(Question ques,OASessionState oas)throws Exception {


        Vector Questions;  
        AddtlInfo addInfo=null;
        //find flag needed to break out of loop and return the first match

        boolean find= false;
        Vector addtlInfo= ques.getAdditionalInfo();
        int i = 0;
        while ((i <addtlInfo.size()) && !(find)) {
            addInfo= (AddtlInfo)addtlInfo.elementAt(i);
            int ind     = addInfo.getDocClass().ind;

            if ((isDocClassIndMatch(ind,oas)) && !(find)) {
                Vector clist    = addInfo.getCountryList();
                if ( (isCountryMatch(clist,oas)) && !(find) ) {
                    Conditions con  = addInfo.getConditions();
                    if ((areConditionsValid(con,oas)) && !(find)) {
                        find = true;

                    }
                }
            }//if
            i++;
        }//while
        //stop here not completed
        //}//if
        if (find) {
            return addInfo;
        }
        else
            return null;

    }//getAddtlInfoForQuestion()

    
    /*********************************************************
    * Method to get docLinks for a actions in a list
    *
    * @param act :The Action object
    * @param oas : OASessionState Object
    * @return Vector:A vector of doclink object
    *********************************************************/
    public Vector getDocLinksForAction(Action act,OASessionState oas)throws Exception {

        Vector retDocLinks  = new Vector();  
        DocLink docLink=null;
        Vector docLinks= act.getDocLinks();
        for (int i=0;i<docLinks.size();i++) {
            docLink = (DocLink)docLinks.elementAt(i);
            int ind = docLink.getDocClass().ind;

            if (isDocClassIndMatch(ind,oas)) {
                Vector clist    = docLink.getCountryList();
                if ( isCountryMatch(clist,oas) ) {
                    Conditions con  = docLink.getConditions();
                    if ( areConditionsValid(con,oas) ) {

                        retDocLinks.addElement(docLink);
                    }
                }
            }//if

        }//for
        //stop here not completed
        //}//if
        return retDocLinks;
    }//getDocLinksForAction()


    /*********************************************************
    * Method to get docLinks for a Questions in a list
    *
    * @param ques :The question object
    * @param oas : OASessionState Object
    * @return Vector:A vector of doclink object
    *********************************************************/
    public Vector getDocLinksForQuestion(Question ques,OASessionState oas)throws Exception {

        Vector retDocLinks  = new Vector();  
        DocLink docLink= null;

        Vector docLinks= ques.getDocLinks();
        for (int i=0;i<docLinks.size();i++) {
            docLink = (DocLink)docLinks.elementAt(i);
            int ind = docLink.getDocClass().ind;

            if (isDocClassIndMatch(ind,oas)) {
                Vector clist    = docLink.getCountryList();
                if ( isCountryMatch(clist,oas) ) {
                    Conditions con  = docLink.getConditions();
                    if ( areConditionsValid(con,oas) ) {

                        retDocLinks.addElement(docLink);
                    }
                }
            }//if

        }//for
        //stop here not completed
        //}//if
        return retDocLinks;
    }//getDocLinksForQuestion()

    
    /*********************************************************
    * Method to Process the doclinks to get the links.This
    * goes through the linkgroups to return a vector of links
    *
    * @param docLinks:Vector of DocLinks objects
    * @return Vector:Vector of link objects
    *********************************************************/
    public Vector getLinksFromDocLinks(Vector docLinks,OASessionState oas)throws Exception {

        Vector links= new Vector();
        for (int i=0;i<docLinks.size();i++) {
            DocLink docLink = (DocLink)(docLinks.elementAt(i));
            Vector temp = new Vector();
            if (docLink.hasLinkGroup()) {
                int ind = docLink.getLinkGroupInd();
                LinkGroup lnGroup = getLinkGroupFromDataBase(ind,oas.getLanguage());
                temp = lnGroup.getLinks();
                for (int j=0;j<temp.size();j++) {
                    Link link = getLinkFromDataBase(((Link) temp.elementAt(j)).getInd(),oas.getLanguage());
                    links.addElement(link);
                }

            }
            else {
                Link link   = getLinkFromDataBase(docLink.getLinkInd(),oas.getLanguage());
                links.addElement(link);
            }


        }
        return links;
    }//getLinksFromDocLinks()

    
    /*********************************************************
     * This method compares the country for a given edge.
     *
     * @param edge: The current edge
     * @param oas: The current sessionstate object
     *
     * @param boolean - If the country for the edge matches the country then
     *                  the it returns true else false
    *********************************************************/
    public boolean checkCountryForEdge(Edge e,OASessionState oas)throws Exception {

        Vector countryList  = getEdgeCountryList(e);
        boolean flag = isCountryMatch(countryList,oas);
        return flag;
    }//checkCountryForEdge()
    

    /*********************************************************
     * This method compares the country for a given AnswerConcl.
     *
     * @param AnswerConcl: The current AnswerConcl
     * @param oas: The current sessionstate object
     *
     * @param boolean - If the country for the AnswerConcl matches the country then
     *                  the it returns true else false
    *********************************************************/
    public boolean checkCountryForAnswerConcl(AnswerConcl c,OASessionState oas)throws Exception {

        Vector countryList  = getAnswerConclCountryList(c);
        boolean flag = isCountryMatch(countryList,oas);
        return flag;
    }//checkCountryForAnswerConcl()

    
    /*********************************************************
    * Verify if the docclassind is valid 
    *
    * @param oas : OASessionState object
    * @param vector: docclassind vector
    * @return boolean: true if the docclassind is a match and
    *                  False if the docclassind is not found 
    *********************************************************/
    public boolean isDocClassIndMatch(int ind,OASessionState oas)throws Exception {

        int i           = 0;
        boolean flag    = false;

        // Return all for internal site
        if (oas.getDocClassInd()==1) {
            flag    = true;
        }
        // Return dealer confidential and unclassified for BP site
        if ((oas.getDocClassInd()==2) &&((ind==2)|(ind==3))) {
            flag = true;
        }
        // Return only unclassified for the external site
        if ( (oas.getDocClassInd()==3) && (ind==3)) {
            flag=true;
        }
        if (Debug) {
            // System.out.println("DBAPI: DocClassInd ="+oas.getDocClassInd());
            if (flag) System.out.print(",");
            else System.out.print("-");
        }
        return flag;
    }//isDocClassIndMatch()

    
    /*********************************************************
    * Verify if the country is a match  
    * 
    * @param oas : OASessionState object
    * @param vector: countrylist vector
    * @return boolean: true if the country is a match and
    *                  False if the country is not found 
    *********************************************************/
    public boolean isCountryMatch(Vector cList, OASessionState oas)throws Exception {

        if (cList == null || cList.isEmpty()) return false;
        for (int i = 0; i < cList.size(); i++) {
            int cId = ( (TypeCountryCodeRec)cList.elementAt(i) ).getInd();
            if (cId == oas.getCountry()) return true;
        }
        return false;
    }//isCountryMatch()


    /*********************************************************
    * This method gets the title string for an action.
    *
    * @param Action:the action object 
    * @return String: the title in string format
    *********************************************************/
    public String getTitleStringForAction(Action act,OASessionState oas)throws Exception {

        if (act == null) return "";
        Title title = getActionTitle(act,oas);
        if (title == null) return "";
        return title.getTitle();

    }//getTitleStringForAction()

    
    /*********************************************************
    * This method gets the title string for a question.
    * 
    * @param Question:the question object 
    * @return String: the title in string format
    *********************************************************/
    public String getTitleStringForQuestion(Question ques,OASessionState oas)throws Exception {

        if (ques == null) return "";
        Title title = getQuestionTitle(ques,oas);
        if (title == null) return "";
        return title.getTitle();

    }//getTitleStringForQuestion()

    
    /*********************************************************
    * This method gets the title string for an answer.
    *
    * @param Answer:the answer object 
    * @return String: the title in string format
    *********************************************************/
    public String getTitleStringForAnswer(Answer ans, OASessionState oas) throws Exception {

        if (ans == null) return "";
        Title title = getAnswerTitle(ans,oas);
        if (title == null) return "";
        return title.getTitle();

    }//getTitleStringForAnswer()

    
    /*********************************************************
    * Get title string for a symptom, given indicators for it and node.
    *
    * @param iNod    : node indicator
    * @param oas     : OA Session State object
    * @return String : the appropriate title string for country & conditions
    *********************************************************/
    public String getTitleForCurrentSymptom(int iNod, OASessionState oas)
    throws Exception {
        Symptom      sym    = getSymptomObject(iNod, oas);
        if (sym == null) return "";
        Vector       titles = sym.getSymptomTitles();
        SymptomTitle title  = null;
        String       retStr = "";

        for (int ii = 0; ii < titles.size(); ii++) {
            title = (SymptomTitle)titles.elementAt(ii);
            if (isDocClassIndMatch(title.getDocClass().ind, oas)) {
                if (isCountryMatch(title.getCountryList(), oas)) {
                    if (title.getLanguage() != null &&
                        ((TypeLanguageRec)title.getLanguage()).getInd() == oas.getLanguage()) {
                        // note that symptom title is not a conditional object, no conditions
                        retStr = title.getTitle();
                        break;
                    }//if language
                }//if country
            }//if docClass
        }//for titles

        if (retStr.equals("") && !titles.isEmpty() && titles.elementAt(0) != null) {
            title = (SymptomTitle)titles.elementAt(0); // pick the first title string if no match
            retStr = title.getTitle();
        }
        return retStr;

    }//end getTitleStringForSymptom()

    
    /*********************************************************
    * This method returns the AddtlInfo as a string
    * 
    * @param Action : the action Object
    * @return String : The additional Info String
    *********************************************************/
    public String getAddInfoStringForAction(Action act,OASessionState oas)throws Exception {

        String retString    = "";
        AddtlInfo addinfo   = getAddtlInfoForAction(act,oas);
        if (addinfo!=null) {
            //retString           += "<ActInfo "+act.getInd()+">";
            retString           += addinfo.getAddtlInfo();
        }
        return retString;

    }//getAddInfoStringForAction()


    /*********************************************************
    * This method returns the AddtlInfo as a string
    *
    * @param ques : the Question Object
    * @return String : The additional Info String
    *********************************************************/
    public String getAddInfoStringForQuestion(Question ques,OASessionState oas)throws Exception {

        String retString    = "";
        AddtlInfo addinfo   = getAddtlInfoForQuestion(ques,oas);
        //retString           += "<QuesInfo "+ques.getInd()+">";
        if (addinfo!=null) {
            retString           += addinfo.getAddtlInfo();
        }
        //retString           += "No additional Info for this Question";

        return retString;

    }//getAddInfoStringForQuestion()

    
    /*********************************************************
    * This method returns the Link Titles for and Question 
    *
    * @param Question : the Question Object
    * @return Vector : The Link Titles returned in a Vector 
    *********************************************************/
    public Vector getLinkTitlesForQuestion(Question ques,OASessionState oas)throws Exception {

        String retString    = "";

        Vector doclinks     = getDocLinksForQuestion(ques, oas);
        Vector links        = getLinksFromDocLinks(doclinks,oas);
        Vector linkTitles   = new Vector();
        //made changes to return vector of links-vk-6/6

        for (int i=0;i<links.size();i++) {
            Link link       = ((Link)links.elementAt(i));
            concatenateTwoVectors(linkTitles,link.getLinkTitles());
        }
        if (Debug) {
            LogSystem.log(1, "DBAPI: Links are done.");
        }

        return linkTitles;
    }//getLinkTitlesForQuestion()
    

    /*********************************************************
    * This method returns the Link titles for and Action 
    * 
    * @param Action : the action Object
    * @return Vector : A vector of Link titles
    *********************************************************/
    public Vector getLinkTitlesForAction(Action act,OASessionState oas)throws Exception {

        Vector doclinks     = getDocLinksForAction(act, oas);
        Vector links        = getLinksFromDocLinks(doclinks,oas);
        Vector linkTitles   = new Vector();
        //changed the function to return a vector of links instead
        //of a string-vk-6/6

        for (int i=0;i<links.size();i++) {
            Link link       = ((Link)links.elementAt(i));
            concatenateTwoVectors(linkTitles,link.getLinkTitles());
        }

        return linkTitles;
    }//getLinkTitlesForAction()

    
    /*********************************************************
    * This method returns the Link URLs for and Action 
    *
    * @param Action : the action Object
    * @return Vector : A vector of Link URLs
    *********************************************************/
    public Vector getLinkURLsForAction(Action act,OASessionState oas)throws Exception {

        Vector doclinks     = getDocLinksForAction(act, oas);
        Vector links        = getLinksFromDocLinks(doclinks,oas);
        Vector linkURLs     = new Vector();
        //changed the function to return a vector of links instead
        //of a string-vk-6/6

        for (int i=0;i<links.size();i++) {
            Link link = ((Link)links.elementAt(i));
            linkURLs.addElement(link.getUrlString());
        }

        return linkURLs;
    }//getLinkURLsForAction()
    

    /*********************************************************
    * This method returns the Link URLs for and Question 
    *
    * @param Question : the Question Object
    * @return Vector : A vector of Link URLs
    *********************************************************/
    public Vector getLinkURLsForQuestion(Question ques,OASessionState oas)throws Exception {

        Vector doclinks     = getDocLinksForQuestion(ques, oas);
        Vector links        = getLinksFromDocLinks(doclinks,oas);
        Vector linkURLs     = new Vector();
        //changed the function to return a vector of links instead
        //of a string-vk-6/6

        for (int i=0;i<links.size();i++) {
            Link link = ((Link)links.elementAt(i));
            linkURLs.addElement(link.getUrlString());
        }

        return linkURLs;
    }//getLinkURLsForQuestion()

    
    /*********************************************************
    * This method concatenates two vectors
    *
    * @param A: a vector A to be concatenated (empty vector OK, null NOT OK)
    * @param B :a vector B (null OK)
    *********************************************************/
    public void concatenateTwoVectors(Vector A,Vector B)throws Exception {

        if (B == null) return;
        // A cannot be null coming in, concatenate won't work if so + this will throw exception;
        // A can be an empty vector, though, which is what it should be initialized to (caller)
        for (int i=0; i<B.size(); i++) {
            A.addElement(B.elementAt(i));
        }
    }//concatenateTwoVectors()


    /*********************************************************
    * This method reads the Question Object from the database
    * 
    * @param qid: question indicator of question to get from database
    * @param langId: Language indicator for the question
    * @return Question:this returns a fully populated Question 
    *                   Object
    *********************************************************/
    public Question getQuestionFromDataBase(int qid,int langId)throws Exception {

        Question ques   = Question.readFromDatabase(qid,langId);
        return ques;
    }//getQuestionFromDataBase()


    /*********************************************************
    * This method reads draft Question Object from the database.
    * 
    * @param dqid: question indicator of draft question to get from database
    * @param nid:  node indicator for the question
    * @param oas:  OASessionState object
    * @return Question : fully populated draft Question Object, or null.
    *********************************************************/
    public Question getDraftQuestionFromDataBase(int dqid, int nid, OASessionState oas)
      throws Exception {

        Symptom sym    = getSymptomObject(nid, oas);
        Node    nd     = getNodeByInd(nid, sym);
        Question quesd = null;

        if (nd.getDraftObjectInd() != 0) {
            quesd = QuestionDraft.readFromDatabase(dqid, oas.getLanguage());
            if (Debug && quesd == null)
                LogSystem.log(1, "DBAPI/getDraftQ: Warning: returning null for draft ques = "
                                   + dqid + " and node " + nid + " within " + sym.toString());
        } // end if node has a draft object
        return quesd;
    }//getDraftQuestionFromDataBase()


    /*********************************************************
    * This method reads the Action Object from the database
    * 
    * @param aid: Action indicator of the Action to get from database
    * @param langId: Language indicator for the Action
    * @return Action : Action object with all the data
    *********************************************************/
    public Action getActionFromDataBase(int aid,int langId)throws Exception {

        Action act = new Action().readFromDatabase(aid,langId);
        return act;
    }//getActionFromDataBase()


    /*********************************************************
    * This method reads the Draft Action Object from the database.
    * 
    * @param daid: Action indicator of draft Action to get from database
    * @param nid:  node indicator for the question
    * @param oas:  OASessionState object
    * @return ActionDraft : Draft Action object with all the data, or null.
    *********************************************************/
    public Action getDraftActionFromDataBase(int daid, int nid, OASessionState oas)
      throws Exception {

        Symptom sym    = getSymptomObject(nid, oas);
        Node    nd     = getNodeByInd(nid, sym);
        Action  actd   = null;

        if (nd.getDraftObjectInd() != 0) {
            actd = ActionDraft.readFromDatabase(daid, oas.getLanguage());
            if (Debug && actd == null)
                LogSystem.log(1, "DBAPI/getDraftAct: Warning: returning null for draft act = "
                                   + daid + " and node " + nid + " within " + sym.toString());
        } // end if node has a draft object
        return actd;
    }//getDraftActionFromDataBase()


    /*********************************************************
    * This method reads the Symptom Object from the database
    * 
    * @param sid:    indicator of Symptom to get from database
    * @param langId: Language indicator for the Symptom
    * @return Symptom : Symptom object with all the data, or null.
    *********************************************************/
    public Symptom getSymptomFromDataBase(int sid,int langId)throws Exception {

        Symptom sym = Symptom.readFromDatabase(sid,langId);
        if (Debug && sym == null) {
            LogSystem.log(1, "DBAPI/getSym: Warning: Symptom.readFromDataBase returned "
                               + "null for "+sid+" (lang: "+langId+")");
        }
        return sym;
    }//getSymptomFromDataBase()


    /**************************************************************
    * This method reads the Draft Symptom Object from the database
    * 
    * @param dsid: Symptom indicator of Draft Symptom to get from database
    * @param nid:  node indicator for the question
    * @param oas:  OASessionState object
    * @return Symptom : Symptom object with all the data
    **************************************************************/
    public Symptom getDraftSymptomFromDataBase(int dsid, int nid, OASessionState oas)
      throws Exception {
        
        Symptom sym    = getDraftSymptomObject(oas);
        if (sym == null) return null;
        Node    nd     = getNodeByInd(nid, sym);
        Symptom symd   = null;

        if (nd.getDraftObjectInd() != 0) {
            symd = SymptomDraft.readFromDatabase(dsid, oas.getLanguage());
            if (Debug && symd == null)
                LogSystem.log(1, "DBAPI/getDraftSym: Warning: returning null for draft sym = "
                                   + dsid + " and node " + nid + " within " + sym.toString());
        } // end if node has a draft object
        return symd;
    }//getDraftSymptomFromDataBase()


    /**************************************************************
    * This method reads the Draft Symptom Object from the database
    * 
    * @param dsid: Symptom indicator of Draft Symptom to get from database
    * @param nid:  node indicator for the question
    * @param oas:  OASessionState object
    * @return Symptom : Symptom object with all the data
    **************************************************************/
    public Symptom getDraftSymptomFromDataBaseOnly(int dsid, OASessionState oas)
      throws Exception {
        
        Symptom symd = SymptomDraft.readFromDatabase(dsid, oas.getLanguage());
        if (Debug && symd == null)
            LogSystem.log(1, "DBAPI/getDraftSymOnly: Warning: returning null for draft sym = "
                          + dsid);
        return symd;
    }//getDraftSymptomFromDataBaseOnly()


    /*********************************************************
    * This method reads the Links from the database
    * 
    * @param lnInd : Link indicator of the link to be read
    * @param langInd : the language indicator of the link
    * @return Link : Link object that has the data
    *********************************************************/
    public Link getLinkFromDataBase(int lnInd, int langInd)throws Exception {

        Link link   = Link.readFromDatabase(lnInd,langInd);
        return link;
    }//getLinkFromDataBase()


    /*********************************************************
    * This method reads the LinkGroup object from the database
    * 
    * @param lnInd : indicator of the linkgroup to be read
    * @param langInd : the language indicator of the linkgroup
    * @return LinkGroup : LinkGroup object that has the data
    *********************************************************/
    public LinkGroup getLinkGroupFromDataBase(int lnInd, int langInd)throws Exception {

        LinkGroup linkgrp   = new LinkGroup(lnInd).readFromDatabase(lnInd,langInd);
        return linkgrp;
    }//getLinkGroupFromDataBase()


    /*********************************************************
    *  Database connection is created in this method...
    * 
    *
    **********************************************************/
    private static void init() throws Exception {

        Class.forName( "COM.ibm.db2.jdbc.net.DB2Driver");
        // initialize the log system
//        LogSystem logSys = new LogSystem(10, logpath + logname);
//        logSys.setProvideGUI(false);

        // initialize the database system
        DatabaseSystem dbSys = new DatabaseSystem(true, LogSystem.getInstance(), useAppDrv);

        DatabaseSystem.createConnection(DatabaseSystem.QUEST, database, servName, port, userId, passwd);

        LogSystem.log(1, "");
        LogSystem.log(1, "---------------------------------------------");
        LogSystem.log(1, "===> Using database " + database);

        LogSystem.log(1, "---------------------------------------------");
        LogSystem.log(1, "");
    }//init()


    /*********************************************************
    * This method gets the product conclusions for New Symptom   
    *
    * @param oas :OASessionState Object
    * @return Hashtable : a hashtable of question and answers 
    *                    that have to be preanswered for this
    *                    model and machinetype
    *********************************************************/
    public Hashtable getProductConclusionsForNewSym(OASessionState oas) throws Exception {

        Hashtable prodConcl   =   new Hashtable();
        int prodInt          = 0;

        SQLMethod sqlMethod = new SQLMethod( DatabaseSystem.QUEST, 
                                             "DBAPI.getProductionConclusionsForNewSym()", 1);

        Statement stmt = sqlMethod.createStatement();
        ResultSet   rs = stmt.executeQuery("select c.prodint from tigris.typebrand"+
                                           " a,tigris.brands b,tigris.products c where  a.brandind= "+
                                           " b.brandind  and a.descript='"+oas.getBrand()+"' and b.brandgroupind="+
                                           "c.brandgroupind and c.prodnum='"+oas.getModel()+"'");
        while (rs.next()) {
            prodInt         = rs.getInt(1);
        }
        if (Debug) {
                LogSystem.log(1, "DBAPI: Brand "+oas.getBrand()+ " / Model "+oas.getModel()
                                   +" / ProductInd "+prodInt);
        }

        ProductConcl prcl   = new ProductConcl(prodInt);
        prodConcl           = prcl.getProductConclusions(prodInt);

        rs.close();
        sqlMethod.close();
        return prodConcl;
    } //getProductConclusionsForNewSym()

    
    /*********************************************************
    * Returns the three character country code from clio.custaddress for
    * the given CLIO customer number.
    *
    * <b>Note:</b> this method requires that a connection for
    * DatabaseSystem.CLIO has already been created.
    * @see com.ibm.nzna.shared.db.DatabaseSystem.createConnection
    * 
    * @param custNum  the CLIO customer number
    * 
    * @return the three character country code or null if an error occurs
    *********************************************************/
    public String getCLIOCountryCodeForCustNum(int custNum,String dbname,String servName,
                                               String port,String user,String password)
    throws Exception {
        
        Class.forName( "COM.ibm.db2.jdbc.net.DB2Driver");
        // initialize the log system
//         LogSystem logSys = new LogSystem(10, logpath + logname);
//        logSys.setProvideGUI(false);

        // initialize the database system
        DatabaseSystem dbSys = new DatabaseSystem(true, LogSystem.getInstance(), useAppDrv);

        DatabaseSystem.createConnection(DatabaseSystem.CLIO, dbname, servName, port, userId,passwd);

        String countryCode = null;

        SQLMethod sqlm = new SQLMethod( DatabaseSystem.CLIO, 
                                        "getCLIOCountryCodeForCustNum()", 1 );
        ResultSet rs = null;

        try {
            Statement stmt = sqlm.createStatement();

            rs = stmt.executeQuery(
                                            "SELECT countryCode " +
                                            "FROM clio.custaddress " + 
                                            "WHERE custNum=" + custNum + " " +
                                            "FOR READ ONLY"
                                            );

            if ( rs.next() ) {
                countryCode = rs.getString(1);
            }
        }
        catch (Exception e) {
            LogSystem.log(1, e);
        }
        finally {
            rs.close();
            sqlm.close();
        }
        return countryCode;
    }//getCLIOCountryCodeForCustNum


    /*********************************************************
      * This method extracts the language Ind from the Language 
      * part of the geo
      * 
      * @param
      * @return
      *********************************************************/
    public int getLangIndFromDatabase(String descript) throws Exception {

        // initialize the database system -- now done in init()

        SQLMethod sqlMethod = new SQLMethod( DatabaseSystem.QUEST, 
                                             "DBAPI.getLangIndFromDatabase()", 1);

        int langInd     = 0;
        ResultSet   rs  = null;
        try {
            Statement stmt  = sqlMethod.createStatement();
            rs  = stmt.executeQuery(           "SELECT languageInd " +
                                               "FROM tigris.typelanguage " + 
                                               "WHERE descript='" + descript + "' " +
                                               "FOR READ ONLY"
                                               );
            while (rs.next()) {
                langInd     = rs.getInt(1);
            }
        }
        catch (Exception e) {
            LogSystem.log(1, e);
        }
        finally {
            rs.close();
            sqlMethod.close();
        }
        if (Debug) {
            LogSystem.log(1, "DBAPI: Language Ind ="+langInd);
        }
        return langInd;
    }//getLangIndFromGeo()


}//DBAPI Class

