/*********************************************************
 * OA ENGINE this class processes the next 
 * question/answer/symptom/action to be presented to the 
 * user based on the user history. 
 *
 * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
 * Created: 5/05/2000  10:19am
 * @author  Venu Koneru  
 * @version 1.0
 *********************************************************/

package com.ibm.nzna.projects.oa.engine;
import java.util.*;
import java.io.*;
import com.ibm.nzna.shared.util.*;    // LogSystem, Text
import com.ibm.nzna.projects.common.quest.oa.*;

/* HISTORY
 *
 *  05/05-06/30  Venu developed and debugged most of it
 *  08/09  mdr  revise debug prints to be uniform in line spacing, etc.; add displayError(3...)
 *              take out displayError(1,oas) for find == true
 *              add a return after symptomJump / processNode
 *              set current node to 0 for a new symptom, before jumping
 *              numerous minor formatting changes for uniformity & readability
 *  08/10  mdr  add cr to debug statement, try to see where addHistoryItem should be getting done
 *  08/14  mdr  change objectInd returned to communicate the new symptom (child symptom), in two
 *                methods -- returns for question and action
 *  08/16  mdr  change loop in getHistoryAsString to for, was weird while not empty vector forever
 *  08/20  mdr  change arg to addHistoryItem to be getOAObjectIndForNode rather than cn
 *  08/21  mdr  add title to return from displayHistory
 *  08/23  mdr  add OAMemoryInterface parameter to constructor, and send it in turn to DBAPI
 *  08/24  mdr  set current node again after jumping based on nextNode returned
 *              also same after getting new question node!
 *  08/29  mdr  add debug printout after processOtherEdges, updateConclusions, processPreanswers
 *              add segment for question node using getPreanswerItem -- use preanswers to process
 *              add sequence # and symptom to history return
 *  08/30  mdr  take out sequence # and symptom from history, plus exclude preanswers (c prevoid)
 *  08/30  mdr  add condition of cr == 0 to point where rootnode of symptom is sought, would not
 *                work for next branch of if stmt anyway -- NOT! retract this change
 *              rewrite a few !(xx) conditions to be easier to read
 *              set SeqNo in various returnVars objects + update comments on those methods
 *  08/31  mdr  tighten logic for error in getHistoryAsString
 *              check for history answer similar to preanswer logic (8/29)
 *  09/06  mdr  add error (1) if null node after other edges not found; move some debug messages
 *              addHistoryItem takes a new argument, the node indicator
 *              getQuestionByInd needs a node argument, which is added throughout here
 *  09/08  mdr  add nodeInd parameter to getAnswerTitlesForQuestion, getAnswerIndsForQuestion
 *                & to their callers & to calls to getTitlesForValidAnswers within those methods
 *              same for getAnswerByInd
 *  09/17  mdr  getSymptomObject now has a node indicator argument, no longer a symptom indicator
 *              getActionByInd and its callers need node indicator arg, similar to 9/08
 *  09/18  mdr  new method getSymptomTitleAsString
 *  09/20  mdr  add arguments to constructor, which are passed to DBAPI: pQitMem and lpath
 *  10/16  mdr  add argument to constructor: whether to use app driver or net driver (boolean)
 *  11/01  mdr  replace println(...) with LogSystem.log(1, ...) (all are inside Debug)
 *                + include Tracey's higher level logging for the first 5 such
 *  11/04  mdr  turn on debug
 *  11/05  mdr  add more debug info to error msg strings, eg, nodes/edges/etc. missing #s
 *       >>> TO DO:
 *              is there a stack of nodes for jumps, or was that abandoned by Venu?
 *
 * END HISTORY
 */

   /***************************************
   * The Engine is the heart of the Online Assistant.  This class
   * contains all the logic to process the next question, answer
   * or actions etc to be presented to the user.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 5/05/2000  10:29am
   * @author Venu Koneru
   * @version 1.0
   ****************************************/
public class OAEngine {
    
    DBAPI dba                       = null;
    int sym_loop                    = 0;
    boolean Debug                   = true;
    int debugLevel                  = 10;
    String warnings                 = "";

    /**
     *  OAEngine constructor takes in database name, server name, port (string), user, password,
     *    OAMemoryInterface object if the caller is Quest Input Tool (or null), and a path for
     *    the database connection log file produced by DBAPI
     **/
    public OAEngine(String dbname,String server,String port,String user,String passwd,
                                  OAMemoryInterface pQitMem,String lpath, boolean appDrv)
     throws Exception{
        if (Debug) {
            LogSystem.log(debugLevel, "Engine: OAEngine constructor.");
        }
        
        dba = new DBAPI(dbname,server,port,user,passwd,pQitMem,lpath,appDrv);

    } //OAEngine() constructor
    //String nodeString = node.toString();
    //System.out.print(nodeString);

    /**************************************
     * This method returns the next Node to be presented to the user
     *
     * @param oas The current Session properties.
     *
     * @return Node - The next item(node) to be displayed to the user
     *               May also return
     *              #Resolved
     *              #Unresolved
     *              #Alist
     **************************************/
    public OAReturnVars processNode(OASessionState oas) throws Exception {
        int cn = oas.getCurrentNode();//current node Indicator
        int cr = oas.getCurrentResponse();//current response indicator
        OAReturnVars nextNode= null; //next node 
        Edge edge = null;
        Node currentnode = null;

        if (Debug) {
            LogSystem.log(debugLevel, "Engine: source flag = "+oas.getSourceFlag());
        }
        
        Symptom sympt = dba.getSymptomObject(cn, oas);

        try {
            currentnode= dba.getNodeByInd(cn,sympt);
        }
        catch (Exception e) {
            if (e!=null) {
                //The symptom Ind is valid but the node Ind is not valid
                nextNode = displayError(2,oas);
                return nextNode;
            }
        }
        if (Debug) {
            LogSystem.log(debugLevel, "Engine: process current node = " + cn + " and response = " + cr);
        }
        Node node = null;  // this will be the new node after the response

        //if both the current node and response are valid then add to history
        if (cn != 0 && cr != 0 ) {
            oas.addHistoryItem(cn, dba.getOAObjectIndForNode(currentnode, oas), cr);
        }
        if (cn != 0) {
            try {
                updateConclusions(currentnode,oas);
            }
            catch (Exception e) {
                if (e!=null) {
                    nextNode = displayError(2,oas);
                    return nextNode;
                }
            }
        }
        oas.setPreviousNode(cn);
        oas.setPreviousResponse(cr);
        //get the symptom
        Symptom s = dba.getSymptomObject(cn, oas);

        if (cn == 0) {
            try {
                node = dba.getRootNode(s, oas);
            }
            catch (Exception e) {
                if (e!=null) {
                    //Call the errordisplay-reason invalid symInd or there
                    //there is no rootnode for this symptom
                    warnings = warnings + " -- No root node for " + s;
                    nextNode = displayError(1,oas);
                    return nextNode;
                }
            }
            if (Debug) {
                LogSystem.log(debugLevel, "Engine: RootNode received "
                                   + ((node != null) ? " (non-null)" : " NULL!"));
            }
            if (node == null) warnings = warnings + " -- No root node for " + s;
            Hashtable preanswers    = dba.getProductConclusionsForNewSym(oas);
            if (Debug) {
                LogSystem.log(debugLevel, "Engine: Preanswers for this product are "+preanswers);
            }
            processPreanswers(preanswers,oas);
        }
        else {  // get all the edges for a node in the current symptom
            if (Debug) {
                LogSystem.log(1, "Engine: Current Response = "+cr);
            }
            Hashtable edges = dba.getEdgesForNodeAndAnswer(oas);
            Enumeration noOfEdges;
            noOfEdges   = edges.keys();
            int docInd  = oas.getDocClassInd();
            boolean find= false;
            
            while ( (noOfEdges.hasMoreElements()) && !find ) {
                edge = (Edge) (noOfEdges.nextElement());
                //check if the docclassind matches
                if ( dba.isDocClassIndMatch(dba.getEdgeDocClassInd(edge),oas) ) {
                    if (Debug) {
                        LogSystem.log(1, "Engine: Passed docClass "+dba.getEdgeDocClassInd(edge));
                    }
                    if (dba.checkCountryForEdge(edge,oas)) {
                        Conditions conditions   = edge.getConditions();
                        if (Debug) {
                            LogSystem.log(1, "Engine: Passed CountryCode");
                        }
                        //Evaluate the validity of all the conditions on that edge
                        if (dba.areConditionsValid(conditions,oas)) {
                            processOtherEdges(oas,edge);
                            node = (Node)(edges.get(edge));
                            find = true;  // mdr moved from after messages 6 Sep 00
                            if (Debug) {
                                LogSystem.log(1, "Engine: Next node to traverse is "
                                                   + node.getInd() + " / Edge " + edge
                                                   + " / node " + node
                                                   + " / value of find " + find);
                            }
                            if (node==null) {
                               warnings = warnings + " -- No node for edge " + edge;
                               if (Debug)
                                   LogSystem.log(1, 
                                       "Engine: WARNING: There is no Node for this edge");
                            }
                        }//if conditions
                    }//if country
                }//if docClass
            }//while edges
            // if (Debug) {  -- OUT: this is redundant with debug messages either above or below
            //     System.out.println("Engine: node found is "+ find);
            // }

            if (!find) {  // check Other nodes
                if (Debug) {
                    LogSystem.log(1, "Engine: checking Other nodes");
                }
                Hashtable otheredges = dba.getOtherEdgesForNode(oas);
                if (Debug) {
                    // System.out.println("\nHashTable contents "+edges);
                    LogSystem.log(1, "Engine: Other edges hashtable "+otheredges);  // debug
                }
                noOfEdges   = otheredges.keys();
                
                while ( (noOfEdges.hasMoreElements()) && !find ) {

                    //Edge
                    edge   = (Edge) (noOfEdges.nextElement());
                    //check if the docclassind matches
                    if ( dba.isDocClassIndMatch(dba.getEdgeDocClassInd(edge),oas) ) {
                        if (Debug) {
                            LogSystem.log(1, "Engine: Passed docClass"+dba.getEdgeDocClassInd(edge));
                        }
                        if (dba.checkCountryForEdge(edge,oas)) {
                            Conditions conditions   = edge.getConditions();
                            if (Debug) {
                                LogSystem.log(1, "Engine: Passed CountryCode");
                            }
                            //Evaluate the validity of all the conditions on that edge
                            if (dba.areConditionsValid(conditions,oas)) {
                                processOtherEdges(oas,edge);
                                node = (Node)(otheredges.get(edge));
                                find = true;
                                if (Debug) {
                                    LogSystem.log(1, "Engine: The Node is ---- " + node);
                                    if (node==null)
                                        LogSystem.log(1, 
                                            "Engine: WARNING: There is no Node for this edge");
                                }
                            }//if
                        }//if
                    }//if
                }//while
                if (node == null) warnings = warnings + " -- No nodes for edges " + otheredges;
                if (Debug) {  // mdr moved this inside else branch, was always printing 6 Sep 00
                    LogSystem.log(1, "Engine: Other Node is **** " + node);
                    if (node==null) {
                        LogSystem.log(1, "Engine: ERROR: There are no Other nodes for this edge ");
                    }
                }
            } // end check Other nodes
        }//end else get all the edges
        if (node == null) return displayError(1,oas);  // mdr added 6 Sep 00
        if (Debug) {
            LogSystem.log(1, "Engine: Check for symptom node");
        }
        // Check if there is a symptom jump and get the new symptom and get the next node
        if (node.getNodeType()==node.NODE_TYPE_SYMPTOM) {
            if (Debug) {
                LogSystem.log(1, "Engine: This is a symptom Node -- Jumping.");
            }
            s       = symptomJump(oas,node);
            oas.setCurrentNode(0);  // new 9 Aug (mdr)
            nextNode = processNode(oas);
            oas.setCurrentNode(nextNode.nodeInd);  // new 24 Aug (mdr)
            if (Debug) {
                LogSystem.log(1, "Engine: After Jumping, node is " + nextNode.nodeInd);
            }
            return nextNode;  // new 9 Aug (mdr)
        }
        if (Debug) {
            LogSystem.log(1, "Engine: Check for question node");
        }
        // if the node is a question node then display question (if not preanswer or duplicate)
        if (node.getNodeType()==node.NODE_TYPE_QUESTION) {
            int quesInd = dba.getOAObjectIndForNode(node,oas);
            if (oas.isAnswered(quesInd)) {  // question preanswered or answereed in history ...
                oas.setCurrentResponse(oas.getAnswer(quesInd));  // answer it internally now
                if (Debug) {
                    LogSystem.log(1, "Engine: Question Node is preanswered, processing...");
                }
                oas.setCurrentNode(node.getInd());
                nextNode = processNode(oas);
                oas.setCurrentNode(nextNode.nodeInd);
                if (Debug) {
                    LogSystem.log(1, "Engine: After pre-answer, node is " + nextNode.nodeInd);
                }
                return nextNode;  // new 29 Aug (mdr)
            }  // end preanswer
            Question ques   = dba.getQuestionByInd(quesInd, node.getInd(), oas);
            if (Debug) {
                LogSystem.log(1, "Engine: This is a Question Node.");
            }
            nextNode        = processOAReturnVarsForQuestion(ques,node,oas);
            oas.setCurrentNode(nextNode.nodeInd);  // new 24 Aug (mdr)
        } // call displayQuestion()
        if (Debug) {
            LogSystem.log(1, "Engine: Check for Action");
        }
        // If the node is an Action node then display the actions from 
        // from the action list to the user
        if (node.getNodeType()==node.NODE_TYPE_ACTION) {
            int actInd      = dba.getOAObjectIndForNode(node,oas);
            Action action   = dba.getActionByInd(actInd, node.getInd(), oas);
            nextNode        = processOAReturnVarsForAction(action,node,oas);
            if (Debug) {
                LogSystem.log(1, "Engine: This is an Action Node.");
            }
        } // call displayAction()
        // Check if there is a symptom return and where to go back and get the next node ??
        if (node.getNodeType()==node.NODE_TYPE_RETURN) {
            //loop to check if a return node exists and set that as the next 
            //node sometimes no return node is set so to break out of infinite loop: limit 100
            // mdr says who-what?? sym_loop never increments, there's no while here . . .
            if (Debug) {
                LogSystem.log(1, "Engine: Inside node_type_return loop");
            }
            if (sym_loop<100) {
                symptomReturn(oas);
                EdgeReturn edgeRet = edge.getReturnNode();
                if (edgeRet!=null) {
                    int retNodeInd     = edgeRet.getNodeInd();
                    oas.setCurrentNode(retNodeInd);
                }
                nextNode = processNode(oas);
            }
            else {
                // to check for wrong data imports and existing faulty data 
                if (Debug) {
                    LogSystem.log(1, "Engine: error in return node loop");
                }
                nextNode = displayError(3, oas);
            }
        }//if
        if (Debug) {
            LogSystem.log(1, "Engine: Check Resolved/Unresolved/Alist.");
        }
        // If the node is an Resolved node then display #RESOLVED
        if (node.getNodeType()==node.NODE_TYPE_RESOLVED) {

            String output   = "<OARESOLVED>";
            nextNode        = processOAReturnVarsForSpNode(node,output,oas);
        }
        // If the node is an Unresolved node then display #UNRESOLVED
        if (node.getNodeType()==node.NODE_TYPE_UNRESOLVED) {

            String output   = "<OAUNRESOLVED>";
            nextNode        = processOAReturnVarsForSpNode(node,output,oas);
        }
        // If the node is an ALIST node then display Actions
        if (node.getNodeType()==node.NODE_TYPE_ALIST) {

            //String output   = getActionListString(oas);
            nextNode        = processOAReturnVarsForAlistNode(node,oas);
        }
        if (Debug) {
            LogSystem.log(1, "Engine: Nextnode returned back to LogicAPI = "+nextNode);
        }
        return nextNode;
    } // processNode()


    /**************************************
     * This method processes the preanswers for a given edge.
     *
     * @param edge: The current edge
     * @param oas - The current session state information.
     **************************************/
    public void processOtherEdges(OASessionState oas, Edge e)throws Exception  {

        Vector preanswers,unanswers,alists ;

        //Process the preanswers and add them to preanswer List
        preanswers = dba.getPreanswers(e);
        for (int i=0;i<preanswers.size();i++) {
            EdgePreanswer ep = (EdgePreanswer)preanswers.elementAt(i);
            int qid = dba.getQuestionForEdgePreanswer(ep,oas);
            int aid = dba.getAnswerForEdgePreanswer(ep,oas);
            oas.addPreanswerItem(qid,aid);
        }

        //Process the unanswers and remove them from preanswer List
        unanswers = dba.getUnanswers(e);
        for (int i=0;i<unanswers.size();i++) {
            EdgeUnanswer eu = (EdgeUnanswer)unanswers.elementAt(i);
            int qid = dba.getQuestionForEdgeUnanswer(eu,oas);
            oas.removePreanswerItem(qid);
        }

        //Process actionlist items on an EdgeAlist
        alists = dba.getActionListsForEdge(e);
        for (int i=0;i<alists.size();i++) {
            EdgeAlist ea = (EdgeAlist)alists.elementAt(i);
            int aid = dba.getActionForEdgeAlist(ea,oas);
            oas.addActionListItem(aid);
        }
        if (Debug) {
            LogSystem.log(1, "Engine/processOtherEdges: After processing, preanswers: "
                               + oas.getPreanswerString() + ", actionlist: "
                               + oas.getActionListString());
        }
    }//processOtherEdges()

    
    /**************************************
     * This method processes the SymptomJump for a given Symptom.
     *
     * @param cnode: the current node
     * @param oas: the current SessionState
     * @return sym - The symptom to jump to
     **************************************/
    public Symptom symptomJump( OASessionState oas, Node cnode)throws Exception  {

        int cn  = oas.getCurrentNode();
        int cr  = oas.getCurrentResponse();
        int ind = dba.getOAObjectIndForNode(cnode,oas);
        oas.setCurrentSymptom(ind);
        //donno how to add return node to the session state
        Symptom symptom = dba.getSymptomObject(cn, oas);
        return symptom;

    }//symptomJump()


    /**************************************
     * Return from the current symptom to the previous Symptom.
     *
     * @param oas: the current SessionState
     * @param e : Edge to get the return node
     **************************************/
    public void symptomReturn(OASessionState oas)throws Exception  {

        int sym = 0;
        sym = oas.popSymptom();
        oas.setCurrentSymptom(sym);
        // not complete donno how to add return/current node 
        // to the sessionstate
        oas.setCurrentNode(0);
        oas.setCurrentResponse(0);
    }//symptomReturn()


    /**************************************
    * Update the preanswer list based on the conclusions of the Current Node and
    * current response.
    *
    * @param oas The current session state.  The current Node and Response are obtained
    *           from it and the preanswer list in it will be updated.
    ***************************************/
    public void updateConclusions(Node currNode, OASessionState oas)throws Exception  {

        Vector     conclusion;
        AnswerConcl c = null;
        int      qind = dba.getOAObjectIndForNode(currNode, oas);
        int      cr   = oas.getCurrentResponse();
        Question q    = dba.getQuestionByInd(qind, currNode.getInd(), oas);
        Answer   a    = dba.getAnswerByInd(qind, currNode.getInd(), cr, oas);

        if (Debug) {
            LogSystem.log(1, "Engine/Update: Updating Conclusions -- questionInd = "+qind);
        }
        if (Debug) {
            LogSystem.log(1, "Engine/Update: answerInd = "+cr);
        }
        
        if (a != null) {
            conclusion = a.getConclusions();
            //check for conditions on the conclusions object
            for (int i=0;i<conclusion.size();i++) {
                c = (AnswerConcl)conclusion.elementAt(i);
                if ( dba.isDocClassIndMatch(dba.getAnswerConclDocClassInd(c),oas) ) {
                    //System.out.println("\nPassed docClass");
                    if (dba.checkCountryForAnswerConcl(c,oas)) {
                        Conditions conditions   = c.getConditions();
                        //System.out.println("\nPassed CountryCode");
                        //Evaluate the validity of all the conditions on that edge
                        if (dba.areConditionsValid(conditions,oas)) {
                            int qid = dba.getQuestionForConclusion(c);
                            int aid = dba.getAnswerForConclusion(c);
                            oas.addPreanswerItem(qid,aid);
                            //System.out.println("\n"+node.getInd());
                            //nextNode= node;
                            //find = true;
                        }//if Conditions
                    }//if Country
                }//if ClassInd
            }//for
        }//if a != null
        if (Debug) {
            LogSystem.log(1, "Engine/Update: After updateConclusions, preanswers: "
                               + oas.getPreanswerString() + ", actionlist: "
                               + oas.getActionListString());
            }
    }//updateConclusions()


    /*********************************************************
    * This method populates the OAReturnVars object for Action.
    *
    * @param oas:OASessionState Object
    * @param Action:Action object which is to be returned 
    * @return OAReturnVars object populated with the data
    *********************************************************/
    public OAReturnVars processOAReturnVarsForAction(Action act,Node node, OASessionState oas)throws Exception  {
        //System.out.println("\nthe return vector is started");
        //System.out.println("\nthe sessionId="+oas.getSessionId());
        OAReturnVars oars   = new OAReturnVars(oas.getSessionId());
        // OUT (23 Aug) oars.qit_flag       = oas.getSourceFlag();
        oars.addInfo        = dba.getAddInfoStringForAction(act,oas);
        oars.linkTitles     = dba.getLinkTitlesForAction(act,oas);
        oars.linkURLs       = dba.getLinkURLsForAction(act,oas);
        oars.title          = dba.getTitleStringForAction(act,oas);
        // oars.objectInd      = dba.getActionInd(act,oas);
        oars.objectInd      = oas.getCurrentSymptom();  // mdr changed for child symptoms 15 Aug
        oars.nodeInd        = node.getInd();  
        oars.node_flag      = oas.getNodeFlag();
        oars.sessionSeqNo   = oas.getCurrentSeqNo();  
        //System.out.println("the return vector is done");
        return oars;
    }//processOAReturnVars()


    /*********************************************************
    * This method populates the OAReturnVars object for Question.
    *
    * @param oas: OASessionState Object
    * @param ques: Question object which is to be returned 
    * @return OAReturnVars object populated with the data
    *********************************************************/
    public OAReturnVars processOAReturnVarsForQuestion(Question ques,Node node, OASessionState oas) throws Exception {
        //System.out.println("\nthe return vector is started");
        //System.out.println("\nthe sessionId="+oas.getSessionId());
        OAReturnVars oars   = new OAReturnVars(oas.getSessionId());
        // OUT (23 Aug) oars.qit_flag       = oas.getSourceFlag();
        oars.addInfo        = dba.getAddInfoStringForQuestion(ques,oas);
        oars.linkTitles     = dba.getLinkTitlesForQuestion(ques,oas);
        oars.title          = dba.getTitleStringForQuestion(ques,oas);
        oars.linkURLs       = dba.getLinkURLsForQuestion(ques,oas);
        //System.out.println("\nthe qtitles are done");
        oars.answerTitles   = getAnswerTitlesForQuestion(ques,node.getInd(),oas);
        oars.answerInds     = getAnswerIndsForQuestion(ques,node.getInd(),oas);
        //System.out.println("\nthe atitles are done");
        // oars.objectInd      = dba.getQuestionInd(ques,oas);  // mdr changed for child symptoms 15 Aug
        oars.objectInd      = oas.getCurrentSymptom();
        oars.nodeInd        = node.getInd(); 
        oars.node_flag      = oas.getNodeFlag();
        oars.sessionSeqNo   = oas.getCurrentSeqNo();  

        //System.out.println("\nthe return vector is done");
        return oars;
    }//processOAReturnVarsForQuestion()

    
    /*********************************************************
    * Get Answer Titles to present to User.
    * 
    * @param ques:    Question for which answers are being retrieved
    * @param nodeInd: indicator for node of the question
    * @param oas:     OASessionState Object
    * @return Vector: All the answer titles to be presented to the user.
    *********************************************************/
    public Vector getAnswerTitlesForQuestion(Question ques, int nodeInd, OASessionState oas)
    throws Exception {

        int quesInd         = dba.getQuestionInd(ques,oas);
        Hashtable titles    = dba.getTitlesForValidAnswers(quesInd,nodeInd,oas);
        Vector ansTitles    = new Vector();
        //Vector ansTitles    = titles.elementsVector();-doesn't work donno why
        Enumeration enHash  = titles.elements();
        while (enHash.hasMoreElements()) {

            ansTitles.addElement(enHash.nextElement());
        }
        return ansTitles;

    }//getAnswerTitlesForQuestion()


    /*********************************************************
    * Get Answer indicators for the question.
    *
    * @param ques:    Question for which answers are being retrieved
    * @param nodeInd: indicator for node of the question
    * @param oas:     OASessionState Object
    * @return Vector: All the answer indicators to be presented.
    *********************************************************/
    public Vector getAnswerIndsForQuestion(Question ques, int nodeInd, OASessionState oas)
    throws Exception {

        int quesInd         = dba.getQuestionInd(ques,oas);
        Hashtable titles    = dba.getTitlesForValidAnswers(quesInd,nodeInd,oas);
        //Vector ansInds      = titles.keysVector();
        Vector ansInds      = new Vector();
        Enumeration enHash  = titles.keys();
        while (enHash.hasMoreElements()) {

            ansInds.addElement(enHash.nextElement());
        }
        return ansInds;

    }//getAnswerIndsForQuestion()

    
    /*********************************************************
    * This method populates the OAReturnVars object for Special node.
    *
    * @param oas: OASessionState Object
    * @param Node: node that is identified as a special node
    * @return OAReturnVars: object populated with the data
    *********************************************************/
    public OAReturnVars processOAReturnVarsForSpNode(Node node,String output, OASessionState oas) throws Exception {
        OAReturnVars oars = new OAReturnVars(oas.getSessionId());
        // OUT (23 Aug) oars.qit_flag     = oas.getSourceFlag();
        oars.addInfo      = output;
        oars.nodeInd      = node.getInd(); 
        oars.node_flag    = oas.getNodeFlag();
        oars.sessionSeqNo = oas.getCurrentSeqNo();

        //System.out.println("\nthe return vector is done");
        return oars;
    }//processOAReturnVarsForSpNode()


    /*********************************************************
    * This method populates the OAReturnVars object for Alist node.
    *
    * @param oas: OASessionState Object
    * @param Node: node that is identified as a special node
    * @return OAReturnVars: object populated with the data
    *********************************************************/
    public OAReturnVars processOAReturnVarsForAlistNode(Node node, OASessionState oas) throws Exception {
        OAReturnVars oars   = new OAReturnVars(oas.getSessionId());
        int nInd = node.getInd();
        // OUT (23 Aug) oars.qit_flag       = oas.getSourceFlag();
        //oars.addInfo      = ;
        oars.actionInds     = getActionIndsForAlistNode(oas);
        oars.actionTitles   = getActionTitlesForAlistNode(nInd, oas);
        oars.linkTitles     = getLinkTitlesForAlistNode(nInd, oas);
        oars.linkURLs       = getLinkURLsForAlistNode(nInd, oas);
        oars.nodeInd        = node.getInd(); 
        oars.node_flag      = oas.getNodeFlag();
        oars.sessionSeqNo   = oas.getCurrentSeqNo();  

        //System.out.println("\nthe return vector is done");
        return oars;
    }//processOAReturnVarsForQuestion()


    /*********************************************************
    * This method populates the OAReturnVars object for History.
    *
    * @param oas: OASessionState Object
    * @param Node: node that is identified as a special node
    * @return OAReturnVars: object populated with the data
    *********************************************************/
    public OAReturnVars displayHistory(OASessionState oas) throws Exception {
        OAReturnVars oars = new OAReturnVars(oas.getSessionId());
        // OUT (23 Aug) oars.qit_flag     = oas.getSourceFlag();
        //oars.queshistory  = oas.getQuestionHistory();
        //oars.anshistory   = oas.getAnswerHistory();  
        oars.title        = "Please select the question to which you would like to return:";
        oars.addInfo      = getHistoryAsString(oas);
        oars.nodeInd      = oas.getCurrentNode(); 
        oars.node_flag    = oas.getNodeFlag();
        oars.sessionSeqNo = oas.getCurrentSeqNo();  

        //System.out.println("\nthe return vector is done");
        return oars;
    }//processOAReturnVarsForQuestion()


    /*********************************************************
    * This method populates the OAReturnVars object in Case of 
    * Errors encountered in the processing.
    *
    * @param oas: OASessionState Object
    * @param Node: node that is identified as a special node
    * @return OAReturnVars: object populated with the data
    *********************************************************/
    public OAReturnVars displayError(int source, OASessionState oas) throws Exception {
        OAReturnVars oars   = new OAReturnVars(oas.getSessionId());
        // OUT (23 Aug) oars.qit_flag       = oas.getSourceFlag();
        //oars.queshistory  = oas.getQuestionHistory();
        //oars.anshistory   = oas.getAnswerHistory();
        String error        = "THE ONLINE ASSISTANT ENCOUNTERED AN ERROR" ;
        oars.title          = error;
        switch (source) {
        case 1:
            oars.addInfo    = "<!-- Node could not be found for this Symptom Indicator" +
                              " -- possibly wrong SymInd or data error or no" +
                              " rootnode for this symptom " + warnings + oas.toString() + " -->";
            break;
        case 2:
            oars.addInfo    = "<!-- Node indicator is not valid or it doesn't have" +
                              " a corresponding object in database " + warnings +
                              oas.toString() + " -->";
            break;
        case 3:
            oars.addInfo    = "<!-- Infinite loop looking for return node "+oas.toString()+" -->";
            break;
        default:
            oars.addInfo    = "<!-- OAEngine failed -- unknown error type" + warnings +
                              oas.toString() + " -->";

        }
        //oars.addInfo      = getErrorString(source,oas);
        oars.node_flag      = oas.getNodeFlag();
        oars.sessionSeqNo   = oas.getCurrentSeqNo();  

        //System.out.println("\nthe return vector is done");
        return oars;
    }//processOAReturnVarsForQuestion()

    
    /*********************************************************
    * Get the question and answer pairs for the history 
    * 
    * @param History :Hashtable with the history stored in it.
    * @return String :The question and answer string pairs
    *********************************************************/
    public String getHistoryAsString(OASessionState oas) throws Exception {

        Vector quesHist     = oas.getQuestionHistory();
        Vector ansHist      = oas.getAnswerHistory();
        Vector nodeHist     = oas.getNodeHistory();
        // OUT (30 Aug.): Vector symHist = oas.getSymptomHistory();
        Vector seqHist      = oas.getSequenceHistory();
        String returnString = "";
        int qid             = 0;
        int aid             = 0;
        int nid             = 0;
        // OUT (30 Aug.): int sid = 0;
        int oid             = 0;
        int prevoid         = -1;
        Question ques       = null;
        Answer ans          = null;

        if (Debug && ( quesHist.size() != ansHist.size() || quesHist.size() != seqHist.size() ))
            LogSystem.log(1, "Engine/getHist: vector size mismatch");
        for (int i=0; i < quesHist.size(); i++) {
            if (i >= ansHist.size() || i >= seqHist.size()) break;
            qid = ((Integer)quesHist.elementAt(i)).intValue();
            aid = ((Integer)ansHist.elementAt(i)).intValue();
            nid = ((Integer)nodeHist.elementAt(i)).intValue();
            // OUT (30 Aug.): sid = ((Integer)symHist.elementAt(i)).intValue();
            oid = ((Integer)seqHist.elementAt(i)).intValue();
            if (oid == prevoid) continue;  // skip if seq is same, must be a preanswered question
            prevoid = oid;
            ques = dba.getQuestionByInd(qid, nid, oas);
            ans  = dba.getAnswerByInd(qid, nid, aid, oas);
            returnString +="<Question "+qid+">";
            returnString += dba.getTitleStringForQuestion(ques, oas);
            // OUT (30 Aug.): returnString +="<Symptom "+sid+">";
            // OUT (30 Aug.): returnString +="<Sequence "+oid+">";
            returnString +="<Answer "+aid+">";
            returnString += dba.getTitleStringForAnswer(ans, oas);
        }
        if (Debug) {
             LogSystem.log(1, "Engine/getHist: The history for this session is "+returnString);
        }
        return returnString;

    }//getHistoryAsString()


    /*********************************************************
    * Process the ALIST Node and return all the action titles
    * from the action list in history.
    * 
    * @param nodInd : node indicator
    * @param oas : OASessionState Object
    * @return Vector : all actions that need to be presented to user
    *********************************************************/
    public Vector getActionTitlesForAlistNode(int nodInd, OASessionState oas) throws Exception{

        String returnString     = "";
        Enumeration actList     = oas.getActionList();
        Vector actTitles        = null;
        while (actList.hasMoreElements()) {
            int actInd          = ( (Integer) actList.nextElement()).intValue();
            Action actObject    = dba.getActionByInd(actInd, nodInd, oas);
            actTitles.addElement(dba.getTitleStringForAction(actObject,oas));
        }
        return actTitles;
    }//getActionTitlesForAlistNode()

    
    /*********************************************************
    * Process the ALIST Node and return all the action titles
    * from the action list in history
    *
    * @param oas : OASessionState Object
    * @return Vector : all actions that need to be presented to user
    *********************************************************/
    public Vector getActionIndsForAlistNode(OASessionState oas) throws Exception{

        String returnString     = "";
        Enumeration actList     = oas.getActionList();
        Vector actInds          = null;
        while (actList.hasMoreElements()) {
            actInds.addElement(actList.nextElement());
        }
        return actInds;
    }//getActionIndsForAlistNode()


    /*********************************************************
    * Process the ALIST Node and return all the link titles
    * from the action list in history
    *
    * @param nodInd : node indicator
    * @param oas : OASessionState Object
    * @return String : all the actions that need to be presented to the user
    *********************************************************/
    public Vector getLinkTitlesForAlistNode(int nodInd, OASessionState oas) throws Exception{

        String returnString     = "";
        Enumeration actList     = oas.getActionList();
        Vector actLinkTitles    = new Vector();
        while (actList.hasMoreElements()) {
            int actInd          = ( (Integer) actList.nextElement()).intValue();
            Action actObject    = dba.getActionByInd(actInd, nodInd, oas);
            dba.concatenateTwoVectors(actLinkTitles,dba.getLinkTitlesForAction(actObject,oas));
        }
        return actLinkTitles;
    }//getLinkTitlesForAlistNode()


    /*********************************************************
    * Process the ALIST Node and return all the link URLs
    * from the action list in history
    * 
    * @param nodInd : node indicator
    * @param oas : OASessionState Object
    * @return Vector : all action that need to be presented to user
    *********************************************************/
    public Vector getLinkURLsForAlistNode(int nodInd, OASessionState oas) throws Exception{

        String returnString     = "";
        Enumeration actList     = oas.getActionList();
        Vector actLinkURLs      = new Vector();
        while (actList.hasMoreElements()) {
            int actInd          = ( (Integer) actList.nextElement()).intValue();
            Action actObject    = dba.getActionByInd(actInd, nodInd, oas);
            dba.concatenateTwoVectors(actLinkURLs,dba.getLinkURLsForAction(actObject,oas));
        }
        return actLinkURLs;
    }//getLinkURLsForAlistNode()


    /*********************************************************
    * Get title for symptom, using node from history.
    *
    * @param  oas: OASessionState
    * @return String
    *********************************************************/
    public String getSymptomTitleAsString(OASessionState oas) throws Exception {
        Vector nodeHist  = oas.getNodeHistory();
        Vector symHist   = oas.getSymptomHistory();
        int iSeek        = oas.getCurrentSymptom();
        int iNod         = 0;

        for (int jj = 0; jj < symHist.size(); jj++) {
            if (jj >= nodeHist.size()) break;
            if (((Integer)symHist.elementAt(jj)).intValue() == iSeek) {
                iNod = ((Integer)nodeHist.elementAt(jj)).intValue();
                break;
            }
        }// for symHist

        if (iNod == 0) return "";
        else return dba.getTitleForCurrentSymptom(iNod, oas);
    }//end getSymptomTitleAsString()


    /*********************************************************
    * Process the product conclusions
    * 
    * @param preanswers:hashtable containing the qid and qid pairs
    *                   that have to be added to the preanswers
    *********************************************************/
    public void processPreanswers(Hashtable prodConcl,OASessionState oas) throws Exception {

        Enumeration ques    = prodConcl.keys();
        Enumeration ans     = prodConcl.elements();
        int qid             = 0;
        int aid             = 0;

        for (int i=0;i <prodConcl.size();i++) {
            qid = ((Integer)ques.nextElement()).intValue();
            aid = ((Integer)ans.nextElement()).intValue();
            oas.addPreanswerItem(qid,aid);
        }
        if (Debug) {
            LogSystem.log(1, "Engine/processPreanswers: After processing, preanswers: "
                               + oas.getPreanswerString() + ", actionlist: "
                               + oas.getActionListString());
        }
    }//processPreanswers()

    
    /*********************************************************
    * Gets the Three character countrycode for a custnum
    * 
    * @param oas: The OASessionState Object
    * @return String :The three character countrycode
    *********************************************************/
    public String getThreeCharCountryCode(OASessionState oas,String dbname,String server,String port,String user,String passwd)throws Exception{
        String country      = dba.getCLIOCountryCodeForCustNum(oas.getCustNum(),dbname,server,port,user,passwd);
        //added the port number and the servername parameters to the input
        //parameters list...the logicAPI gets this from Webrenderer...6/22 Venu
        return country;
    }//getThreeCharCountryCode()


    /*********************************************************
    * get the language ind from the database and set it in the 
    * OASessionState Object
    * 
    * @param String :The language in string format
    * @param oas    :OASessionState Object
    *********************************************************/
    public void getLanguageIndandSet(String language,OASessionState oas)throws Exception {

        int langInd     = dba.getLangIndFromDatabase(language);
        oas.setLanguage(langInd);
    }//getLanguageInd()
}

