/*********************************************************
* This Class stores the session information of the active
* Session.
*
* Legal: Copyright (c) 2000 IBM. All Rights Reserved.
* Created: 4/27/2000  10:15am
* @author  Venu Koneru  
* @version 1.0
*********************************************************/


package com.ibm.nzna.projects.oa.engine;
import java.io.*;
import java.util.*;
import com.ibm.nzna.shared.util.*;    // LogSystem, Text
import com.ibm.nzna.projects.common.quest.oa.*;

   /*********************************************************
   * This class stores all the session variables for each 
   * independant user session based on the sessionId. 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/27/2000  10:21am
   * @author Venu Koneru
   * @version 1.0
   *********************************************************/

public class OASessionState implements Serializable,Cloneable {

    private int         currentSymptom  = 0;    // The symptom currently being processed
    private Stack       symptomStack    = null;  // List of all previously processed symptoms
    private int         currentNode     = 0;    // The currently displayed question/action
    private int         currentResponse = 0;    // The users response to the current question
    private int         previousNode    = 0;
    private int         previousResponse= 0;
    private int         custNum         = 0;
    private String      brand           = "";
    private String      model           = "";
    private String      geo             = "";
    private int         country         = 110;//country needs to be changed to 0 not to default to US
    private int         language        = 1;
    private int         docClassInd     = 3;
    private Vector      historyOrder    = new Vector();
    private Vector      questionHistory = new Vector();
    private Vector      answerHistory   = new Vector();
    private Vector      nodeHistory     = new Vector();  // (mdr) record nodes too 6 Sep 00.
    private Vector      symptomHistory  = new Vector();  // (mdr) record symptoms too 24 Aug.
    private Vector      sequenceHistory = new Vector();  // (mdr) record sequence too 29 Aug.
    private Hashtable   preanswers      = new Hashtable();
    private Vector      actionList      = new Vector();
    private String      sessionTime     = "";    //new Timestamp();
    private boolean     qit_flag        = false;
    // OUT (23 Aug) private OAMemoryInterface qitMemory = null;  // added 3 Aug. (mdr), for QIT Validator
    // also (23 Aug) removed the methods getMemoryInterface and setMemoryInterface
    private int         sessionId       = 0;
    private String      node_flag       = "";
    //private Hashtable   prevSessions    =new Hashtable();
    private int         currentSeqNo    = 0;  
    private boolean     Debug           = false;


    /*********************************************************
    * Constructor for the OASessionState Class which takes in
    * a sessId Parameter.
    *
    * @param sessId: the sessionId for the given SessionState
    *                Object
    *********************************************************/
    public OASessionState(int sessId)throws Exception{

        sessionId   = sessId;
    }//OASessionState()


   /**
    * Determine if a Question has already been answered.
    *
    * @param qid ID number of the question to check.
    *
    * @return boolean - Returns true if the the question has already been answered.
    *                   Returns false otherwise.
    */
    public boolean isAnswered(int iqid)throws Exception {
        Integer qid = new Integer(iqid);

        return questionHistory.contains(qid) || preanswers.containsKey(qid);

    } // isAnswered()


    /**
     * Determine if a Question has been preanswered.
     *
     * @param qid ID number of the question to check.
     *
     * @return boolean - Returns true if the the question has been preanswered.
     *                   Returns false otherwise.
     */
    public boolean isPreanswered(int iqid)throws Exception {
        Integer qid     = new Integer(iqid);

        return preanswers.containsKey(qid);

    } // isPreanswered()


    /**
     * Determines if a Question has already been answered with a specified answer.
     * This method check the preanswer list in addition to history.
     *
     * @param qid The id of the question to check.
     * @param aid The id of the answer to check for.
     *
     * @return boolean - Returns true if qid has already been answered with
     *                   answer aid.  Returns false otherwise.
     */
    public boolean isAnswerTrue(int iqid, int iaid)throws Exception {
        Integer qid     = new Integer(iqid);
        Integer aid     = new Integer(iaid);

        int answer   = 0;

        if (questionHistory.contains(qid)) {
            if (answerHistory.contains(aid))
                answer = ((Integer)answerHistory.elementAt(questionHistory.indexOf(qid))).intValue();
        }
        else if (preanswers.containsKey(qid))
            answer = ((Integer)preanswers.get(qid)).intValue();

        if (answer == iaid) return true;
        else return false;

    } // isAnswerTrue()


    /**
     * Returns the answer of a Question from history/preanswers.
     *
     * @param qid Question Id of the question you wish to retrieve the answer of
     *
     * @return int- The answer id associated with the question in history/preanswers.
     *               Returns 0 if the question is not found in history/preanswers.
     */
    public int getAnswer(int iqid)throws Exception {

        // We need to check the history for both the specified qid
        // and we need to check the baseID in case qid represents
        // a cloned question.
        Integer qid  = new Integer(iqid);

        if (questionHistory.contains(qid)) 
            return ((Integer)answerHistory.elementAt(questionHistory.indexOf(qid))).intValue();
        else if (preanswers.containsKey(qid)) 
            return ((Integer)preanswers.get(qid)).intValue();
        else return 0;

    } // getAnswer()


    /**
     * Sets the current symptom for this OASessionState.
     * @param sid The id of the symptom to make current.
     */
    public void setCurrentSymptom(int sid)throws Exception {
        currentSymptom = sid;
    }

    /**
     * Gets the current symptom for this OASessionState.
     * @return int- The current symptom for this OASessionState.
     */
    public int getCurrentSymptom()throws Exception {
        return currentSymptom;
    }

    /**
     * Adds a symptom to the symptom stack.
     *
     * @param sid The id of the symptom to add to the stack.  This id may
     *            be encoded to include information related to symptom
     *            jumps and returns.
     */
    public void pushSymptom(int isid)throws Exception {
        Integer sid     = new Integer(isid);
        if (symptomStack == null) {
            symptomStack = new Stack();
        }
        symptomStack.push(sid);
    }

    /**
     * Retrieves the last symptom that was added to the symptom stack.
     *
     * @return int - The id of the last symptom that was added to
     *                  the symptom stack.
     */
    public int popSymptom()throws Exception {
        return((Integer)symptomStack.pop()).intValue();
    }

    /**
     * Returns a string representation of the symptom stack.
     *
     * @return String - The String representation of the symptom stack.
     */
    public String getSymptomStackString()throws Exception {
        return symptomStack.toString();
    }



    /**
     * Sets the current Node for this OASessionState.
     * @param nodeID  The id of the Node to make the current node for this
     *                session state.
     */
    public void setCurrentNode(int nodeID)throws Exception {
        currentNode=nodeID;
    }


    /**
     * Gets the current Node from this OASessionState.
     *
     * @return int- The current node id for this session state.
     */
    public int getCurrentNode()throws Exception {
        return currentNode;
    }


    /**
     * Sets the current response for this session state.
     *
     * @param responseID The id of the response (answer) supplied
     *                   for this session state.
     */
    public void setCurrentResponse(int responseID)throws Exception {
        currentResponse = responseID;
    }


    /**
     * Gets the current response from this session state.
     *
     * @return int- The current response supplied for this session state.
     */
    public int getCurrentResponse()throws Exception {
        return currentResponse;
    }


    /**
     * Sets the previous Node for this Session State.
     *
     * @param nodeID The id of the previous Node in this session state.
     */
    public void setPreviousNode(int nodeID)throws Exception {
        previousNode=nodeID;
    }


    /**
     * Gets the previous Node for this session state.
     *
     * @return int- The id of the previous node for this session state.
     */
    public int getPreviousNode()throws Exception {
        return previousNode;
    }

    /**
     * Sets the previous response for this session state.
     *
     * @param responseID The id of the previous response supplied to
     *                   this NodeID
     */
    public void setPreviousResponse(int responseID)throws Exception {
        previousResponse = responseID;
    }


    /**
     * Gets the id of the previous response supplied to this session state.
     *
     * @return int- The id of previous response supplied to this
     *                  session state.
     */
    public int getPreviousResponse()throws Exception {
        return previousResponse;
    }

    public void setBrand(String brandName)throws Exception {
        brand = brandName;
    }

    public String getBrand()throws Exception {
        return brand;
    }

    public void setGeo(String geoName)throws Exception {
        geo = geoName;
    }

    public String getGeo()throws Exception {
        return geo;
    }

    public void setCountry(int countryName)throws Exception {
        country = countryName;
    }

    public int getCountry()throws Exception {
        return country;
    }

    public void setLanguage(int lang)throws Exception {
        language = lang;
    }

    public int getLanguage()throws Exception {
        return language;
    }

    public void setDocClassInd (int docInd)throws Exception {
        docClassInd = docInd;
    }

    public int getDocClassInd()throws Exception {
        return docClassInd;
    }

    public void setModel(String newmodel) throws Exception {
        model = newmodel;
    }

    public String getModel() throws Exception { 
        return model;
    }

    public int getCustNum() throws Exception { 
        return custNum;
    }

    public void setCustNum(int custnum) throws Exception { 
        
        custNum = custnum;
    }


    /**
     * Adds an entry to the history list.
     *
     * @param nid Node ID of item to add to history -- will be 0 for preanswers.
     * @param qid Question ID of item to add to history.
     * @param aid Answer ID of item to add to history.
     */
    public void addHistoryItem(int inid, int iqid, int iaid) throws Exception {
        Integer nid     = new Integer(inid);
        Integer qid     = new Integer(iqid);
        Integer aid     = new Integer(iaid);
        Integer sid     = new Integer(getCurrentSymptom());
        Integer oid     = new Integer(getCurrentSeqNo());
        historyOrder.addElement(qid);
        nodeHistory.addElement(nid);  // add nodes to history (mdr) 6 Sep 00 -- new param
        questionHistory.addElement(qid);
        answerHistory.addElement(aid);
        symptomHistory.addElement(sid);
        sequenceHistory.addElement(oid);
        if (Debug) {
            LogSystem.log(1, "SessionState/addHistory: node " + inid + ", question " + iqid
                               + ", answer " + iaid + ", symptom " + sid.toString()
                               + ", sequence# " + oid.toString());
        }

    } // addHistoryItem()


    /*********************************************************
     * Method to store the history and symptoms visited to be 
     * presented to the user.
     * 
     *
     * @param oas:OASessionState object 
     * @return
     *********************************************************/
    //public void saveCurrentState(OASessionState oas) throws Exception {

    //Integer seqId  = new Integer(oas.getCurrentHistoryOrder());
    //prevSessions.put(seqId,oas);

    //}//saveHistory()




    /**
     * Returns the order in which items were added to history.
     *
     * @return Enumeration - the keys to the history table in the order
     *                       they were added.
     */
    public Enumeration getHistoryOrder()throws Exception {
        return historyOrder.elements();
    }

    /*********************************************************
    * This returns the value of the current History Order
    * 
    *
    * @return int : the current value of order
    *********************************************************/
    //public int getCurrentHistoryOrder() throws Exception {

    //return (historyOrder.size());
    // }//getCurrentHistoryOrder()


    /**
     * Returns the answerID associated with a given QuestionID in history.
     *
     * @param qid String - The ID of the question to lookup.
     *
     * @return int - The ID of the answer associated with qid in history.
     */
    public int getHistoryItem(String qid)throws Exception {
        Integer Iqid = new Integer(qid);  // mdr added 17 Aug. convert to proper type for item

        return(((Integer)answerHistory.elementAt(questionHistory.indexOf(Iqid))).intValue());

    } // getHistoryItem()


    /**
     * Returns the QuestionInd history vector.
     *
     * @return Vector - The Question Indicators in history.
     */
    public Vector getQuestionHistory()throws Exception {

        return questionHistory;

    } // getQuestionHistory()


    /**
     * Returns the AnswerInd history vector.
     *
     * @return Vector - The Answer Indicators in history.
     */
    public Vector getAnswerHistory()throws Exception {

        return answerHistory;

    } // getAnswerHistory()


    /**
     * Returns the NodeInd history vector.
     *
     * @return Vector - The Node Indicators in history.
     */
    public Vector getNodeHistory()throws Exception {

        return nodeHistory;

    } // getNodeHistory()


    /**
     * Returns the SymptomInd history vector.  (mdr) added 24 Aug.
     *
     * @return Vector - The Symptom Indicators in history.
     */
    public Vector getSymptomHistory() throws Exception {

        return symptomHistory;

    } // getSymptomHistory()


    /**
     * Returns the SeqNo history vector.  (mdr) added 29 Aug.
     *
     * @return Vector - The Sequence Numbers in history.
     */
    public Vector getSequenceHistory() throws Exception {

        return sequenceHistory;

    } // getSequenceHistory()


    /**
     * Adds an entry to the preamswer list.
     *
     * @param qid Question Id of item to add to preanswers.
     * @param aid Answer Id of item to add to preanswers
     */
    public void addPreanswerItem(int iqid, int iaid)throws Exception {
        Integer qid     = new Integer(iqid);
        Integer aid     = new Integer(iaid);

        preanswers.put(qid,aid);
        // Update the history also, but only if the question is already there
        if (questionHistory.contains(qid)) addHistoryItem(0, iqid, iaid);
        // mdr 31 Aug: replaced primitive actions with addHistoryItem method call here

    } // addPreanswerItem()


    /**
     * Returns the answerID associated with a given QuestionID in preanswers.
     * @param qid The ID of the question to lookup.
     * @return int - The ID of the answer associated with qid in preanswers.
     */
    public int getPreanswerItem(int iqid)throws Exception {
        Integer qid     = new Integer(iqid);
        // mdr added null test here 29 Aug. 00
        if (preanswers.get(qid) != null) return ((Integer)preanswers.get(qid)).intValue();
        else return 0;
    } // getPreanswerItem()


    /**
     * Returns a string representation of the preanswer list for this
     * OASessionState.
     *
     * @return String - The string representation of the preanswers for this
     *                  OASessionState.
     */
    public String getPreanswerString()throws Exception {
        return preanswers.toString();
    }


    /**
     * Removes the specifed question from the preanswer list.
     *
     * @param qid  The id of the question to remove from the preanswer list.
     */
    public void removePreanswerItem(int iqid)throws Exception {
        Integer qid     = new Integer(iqid);
        preanswers.remove(qid);
    }


    /**
     * Adds an entry to the actionlist list.
     *
     * @param aid Id of item to add to actionlist.
     */
    public void addActionListItem(int aid)throws Exception {
        Integer iaid=new Integer(aid);
        actionList.addElement(iaid);

    } // addActionListItem()


    /**
     * Returns the ActionListInd associated with a given actionList.
     * @param aid The ID of the ActionList to lookup.
     * @return int - The ID of the actionList.
     */
    public int getActionListItem(int aid)throws Exception {
        Integer iaid = new Integer(aid);
        int Ind = ((Integer)actionList.elementAt(actionList.indexOf(iaid))).intValue();
        return Ind;

    } // getActionListItem()


    /**
     * Returns a list of the id of the objects stored in the action list.
     *
     * @return Enumeration - The list of id's of all items stored in the
     *                       action list.
     */
    public Enumeration getActionList()throws Exception {
        return actionList.elements();
    }


    /**
     * Returns a string representation of the action list.
     *
     * @return String - The string representation of the action list for
     *                  this OASessionState.
     */
    public String getActionListString()throws Exception {
        return actionList.toString();
    }


    /**
     * Removes the specified item from the action list.
     *
     * @param aid The id of the item to remove from the action list.
     */
    public void removeActionListItem(int iaid)throws Exception {
        Integer aid     = new Integer(iaid);
        boolean check   = actionList.removeElement(aid);
    }


    /**
    * Resets this OASessionState object back to its default values.
    */
    public void clear()throws Exception {

        currentSymptom  = 0;
        if (symptomStack != null) {
            symptomStack.removeAllElements();
        }
        currentNode     = 0;
        currentResponse = 0;
        previousNode    = 0;
        previousResponse= 0;
        brand           = "";
        geo             = "";
        country         = 0;
        language        = 0;
        docClassInd     = 0;
        historyOrder.removeAllElements();
        questionHistory.removeAllElements();
        answerHistory.removeAllElements();
        preanswers.clear();
        actionList.removeAllElements();
        //model           = "";
        sessionTime     = "";

    }


    /**
     * Creates a duplicate of this OASessionState object.
     *
     * @return Object - The copy of this object.
     */
    public Object clone() {

        try {
            OASessionState s = (OASessionState)super.clone();
            if (symptomStack != null) {
                s.symptomStack = (Stack)symptomStack.clone();
            }
            else {
                s.symptomStack = null;
            }
            s.historyOrder = (Vector)historyOrder.clone();
            s.questionHistory      = (Vector)questionHistory.clone();
            s.answerHistory      = (Vector)answerHistory.clone();
            s.preanswers   = (Hashtable)preanswers.clone();
            s.actionList   = (Vector)actionList.clone();
            return s;
        }
        catch (CloneNotSupportedException e) {
            return null;
        }
    }


    /**
     * Given a vector containing a set of conditionterms, determine whether those
     * conditions are met by this SessionState. 
     *
     * @param conditions The vector containing the conditionterms to evaluate.
     *
     * @return boolean - Returns true if the conditions are met or false otherwise.
     */
    public boolean testCondition(Vector conditionTerms)throws Exception {

        boolean result        = booleanEvaluate(conditionTerms);
        return result;
    } // testCondition


    /**
     * Given a question answer pair this evaluates if they exist in History
     *
     * @param qid and aid to check if the qid was answered with 
     *        the particular aid
     *
     * @return boolean - Returns true if the question has that answerid as
     *                   answer in History.
     */
    public boolean checkQAPair(int qid,int aid)throws Exception{

        boolean result = false;
        if (aid == 0) {
            result = isAnswered(qid);
        }
        else {
            result = isAnswerTrue(qid,aid);
        }
        // result = isAnswered(qid);  mdr replace true in ==0 branch above with this one (15 Aug)
        return result;
    } // checkQAPair


    /**
     * This method receives a Vector of boolean terms and operators and 
     * evaluates it.
     *
     * @param conditions The Vector of conditions that you wish to parse into
     *                   discrete values.
     *
     * @return boolean - A boolean
     *                 the supplied conditions.
     */
    private boolean booleanEvaluate(Vector conditionterms)throws Exception {

        Stack   inarray     = new Stack();
        int     counter     = 0;
        String  operators   = "";
        boolean result      = false;
        boolean bwarn       = false;  // mdr adds warn and empty switches as format checks 15 Aug. 00
        boolean bempty      = false;
        boolean val1        = false;
        boolean val2        = false;
        ConditionTerm cterm = null;   // mdr move declarations outside for-loop 5 Sep 00
        int     qid         = 0;
        int     aid         = 0;

        for (int i=0;i<conditionterms.size();i++) {
            cterm = (ConditionTerm)(conditionterms.elementAt(i));
            // OUT: if (Debug) System.out.println("SessionState/boolEval: cterm[" + i + "]: "
            // this print adds no info if warning below is printed, so take it out
            // OUT:                       + cterm.toString());  // mdr added printout 5 Sep 00
            qid = cterm.getQuestionInd();
            aid = cterm.getAnswerInd();
            operators = cterm.getOperators();
            if (operators.equals("") | operators.equals(" ") | operators.equals("  ")) {
                // mdr change if-condition to allow 1 or 2 spaces there 5 Sep 00
                inarray.push(new Boolean(checkQAPair(qid,aid)));
            }
            else {
                bempty = false;
                if (inarray.empty()) { val1 = false; bempty = true; bwarn = true; }
                else val1 = ((Boolean)inarray.pop()).booleanValue();
                val2 = checkQAPair(qid,aid);
                for (int j=0;j<operators.length();j++) {
                    if ((operators.valueOf(operators.charAt(j))).equals("&")) {
                        if (bempty) result = val2;
                        else result = val1 && val2;
                    }
                    else if ((operators.valueOf(operators.charAt(j))).equals("|")) {
                        // note val1 even if bempty is set to right value
                        result = val1 || val2;
                    }
                    else if ( (operators.valueOf(operators.charAt(j))).equals("!")) {
                        result = !val1;
                    }
                    inarray.push(new Boolean(result));
                }//end for operators
            }//end else
        }//end for conditions
        if (inarray.empty()) bwarn = true;  // will also get an empty stack exception below
        if (bwarn) LogSystem.log(1, "OASessionState/boolEval: Warning bad data format - "
                                     + conditionterms.toString());
        return((Boolean)inarray.pop()).booleanValue();
    }  // booleanParse()


    /**
     * This method returns the value of the qit flag.
     *
     * @return boolean - A boolean
     */
    public boolean getSourceFlag() throws Exception {

        return qit_flag;

    }//getSourceFlag()


   /**
    * This method sets the value of the qit flag; caller is responsible for setting this to
    *   indicate that there is an OAMemoryInterface object available, ie, we are in
    *   Quest Input Tool.
    *
    * @param status boolean : the flag that indicates if the session is a qit (true)
    *                   or a web session (false).
    *
    * 
    */
    public void setSourceFlag(boolean status) throws Exception{

        qit_flag = status; // this will be removed soon -- 3 Aug. (mdr)

    }//setSourceFlag()


    /*
     * Returns the value of the qit Memory Interface object.
     *
     * @return OAMemoryInterface - an object or null
     */
    
    // OUT (23 Aug) public OAMemoryInterface getMemoryInterface() throws Exception {

        // OUT (23 Aug) return qitMemory;  // added 3 Aug. (mdr), for QIT Validator

    // OUT (23 Aug)  }  // getMemoryInterface()

   /*
    * Sets the value of the qit Memory Interface object.
    *
    * @param mem OAMemoryInterface : the object usable to call various QIT memory access methods.
    * 
    */
    // OUT (23 Aug) public void setMemoryInterface(OAMemoryInterface mem) throws Exception{

        // OUT (23 Aug) if (mem != null) qit_flag = true;
        // OUT (23 Aug) qitMemory = mem;  // added 3 Aug. (mdr), for QIT Validator

    // OUT (23 Aug) }  // setMemoryInterface()


    /*********************************************************
    * This method returns the session Id of the current Session.
    * 
    * @return int: The id of the current Session
    *********************************************************/
    public int getSessionId()throws Exception {

        return sessionId;
    }//getSessionId()


    /*********************************************************
    * This method sets the session Id of the current Session.
    * 
    * @return void
    *********************************************************/
    public void setSessionId(int id) throws Exception{

        sessionId = id;
    }//setSessionId()



    /*********************************************************
    * This method gets the sequence of the current Session.
    * 
    * @return int : current sequence number for this session
    *********************************************************/
    public int getCurrentSeqNo() throws Exception{


        return currentSeqNo;
    }//getCurrentSeqNo()



    /*********************************************************
    * This method sets the sequence of the current Session.
    * 
    * @param int : current sequence number for this session
    *********************************************************/
    public void setCurrentSeqNo(int seqNo) throws Exception{

        currentSeqNo=seqNo;
    }//setcurrentSeqNo()





    /*********************************************************
    * This method sets the node_flag.
    *
    * @param nfl:String value to set the node_flag
    *********************************************************/
    public void setNodeFlag(String nfl)throws Exception {

        node_flag = nfl;
    }//setNodeFlag()


    /*********************************************************
    * This method returns the node_flag.
    *
    * @return String:value of the node_flag
    *********************************************************/
    public String getNodeFlag() throws Exception{

        return node_flag;
    }//getNodeFlag()


    /*********************************************************
    * This method returns a string representation of all the 
    * values of the session state parameters.
    *
    * @param
    * @return String: A string of the Sessionstate parameters
    *********************************************************/

    public String toString() {

        String response = "<p><p><b>Session State</b>" + "<br>\n";
        response += "Curent Symptom: " + String.valueOf(currentSymptom) + "<br>\n";
        response += "Symptom Stack: " + symptomStack + "<br>\n";
        response += "Current Node:" + String.valueOf(currentNode) + "<br>\n";
        response += "Current Response:" + String.valueOf(currentResponse) + "<br>\n";
        response += "Previous Node: " + String.valueOf(previousNode) + "<br>\n";
        response += "Previous Response:" + String.valueOf(previousResponse) + "<br>\n";
        response += "Brand:" + brand + "<br>\n";
        response += "Geography:" + geo + "<br>\n";
        response += "Country:" + String.valueOf(country) + "<br>\n";
        response += "Language: " + String.valueOf(language) + "<br>\n";
        response += "Type/Model:" + model + "<br>\n";
        response += "Session Time:" + sessionTime + "<br>\n";
        response += "QuestionHistory: " + questionHistory.toString() + "<br>\n";
        response += "AnswerHistory: " + answerHistory.toString() + "<br>\n";
        response += "HistoryOrder: " + historyOrder.toString() + "<br>\n";
        response += "Preanswers: " + preanswers.toString() + "<br>\n";
        response += "Action List: " + actionList.toString() + "<br>\n";
        response += "Node_Flag:" + node_flag + "<br>\n";
        response += "Node_Flag:" + node_flag + "<br>\n";
        response += "Qit_Flag:" + String.valueOf(qit_flag) + "<br>\n";
        response += "DocClassInd:" + String.valueOf(docClassInd) + "<br>\n";
        response += "SessionId:" + String.valueOf(sessionId) + "<br>\n";
        response += "CurrentSeqNo:" + String.valueOf(currentSeqNo) + "<br>\n";

        return response;
    } // end toString()


} // class OASessionState

