package com.ibm.nzna.projects.qit.app ;   
/*-----------------------------------------------------------*/
/*   CacheSystem                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CacheSystem                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.util.* ;   
import com.ibm.nzna.shared.util.* ;   
import com.ibm.nzna.shared.db.* ;   
import java.io.* ;
import java.sql.* ;
/**
 * The CacheSystem is primarily responsible for saving and loading serialized objects from disk. 
 * The CacheSystem saves and loads cached objects from the directory that quest was started in a sub directory
 * called databasename.databaseserver. So, if the database name is 'quest', and the database server is
 * 'chaos', and the Quest application was run from 'c:\qit', then the CacheSystem will read and write
 * objects to the following directory:<br><br>
 *
 * c:\qit\quest.chaos<br><br>
 *
 * The cache system does not guarentee a cached object is in the directory, and will return NULL 
 * from getCachedObject if the filename passed was not found.<br><br>
 *
 * The CacheSystem uses the PropertySystem to read in the database name and servers, so this System
 * must be created <b>AFTER</b> the PropertySystem.<br><br>
 *
 * The cache system also has it's own thread to keep track of time. This thread will first initialize by reading
 * the database timestamp, then it will continue to update that time every second.
 *
 * @version 1.0, 3/14/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CacheSystem                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class CacheSystem extends Object implements Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Variables                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String  cacheDir           = null ;
   private static boolean enabled            = false ;
   private static String  currentDBTimestamp = null ;
   private static int     second             = 0 ;
   private static int     minute             = 0 ;
   private static int     hour               = 0 ;
   private static int     month              = 0 ;
   private static int     day                = 0 ;
   private static int     year               = 0 ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will simply form our cache
    *  directory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CacheSystem () {
      try {
         String databaseServer = PropertySystem.getString ( PropertySystem.PROPERTY_APPDB_SERVER ) ;
         String database       = PropertySystem.getString ( PropertySystem.PROPERTY_APPDB ) ;
         String homeDir        = PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR ) ;
         File   fileCacheDir   = null ;                                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, kick off our thread to keep track of time        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         new Thread ( this ).start() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Form the directory                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         cacheDir = homeDir+File.separator+database+"."+databaseServer ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the directory does not exist, create it now          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         fileCacheDir = new File ( cacheDir ) ;

         if ( !fileCacheDir.exists() ) {
            fileCacheDir.mkdirs() ;
            LogSystem.log ( 1, "Caching enabled, Creating Cache Directory:"+cacheDir ) ;
         } else
            LogSystem.log ( 1, "Caching enabled, Cache Directory:"+cacheDir ) ;


         enabled = true ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Could not find cache directory of '"+cacheDir+"', caching is disabled" ) ;
         enabled = false ;
      }
   }






   /*------------------------------------------------------------*/
   /* cacheObject
    */
   /**
    *    This method will cache an object to the
    *  cached directory.
    *  
    *
    *  @param filename The name of the file to create for the object
    *  @param object The object to cache
    *  @param None
    *
    *  @return TRUE if cached, FALSE if not cached
    *//*---------------------------------------------------------*/
   public static boolean cacheObject ( String filename, Object object ) {
      boolean rc = false ;

      if ( enabled ) {
         String timestamp = getDBTimestamp() ;

         QuestUtil.writeObjectToDisk ( cacheDir+File.separator+filename,                  object ) ;
         QuestUtil.writeObjectToDisk ( cacheDir+File.separator+filename+".cached_time",   timestamp ) ;
         rc = true ;
      }

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* getCachedObject
    */
   /**
    *    This method will attempt to de-serialize a
    *  cached object from disk and returns it.
    *  
    *
    *  @param filename The filename of the cached object
    *  @param None
    *  @param None
    *
    *  @return The object, NULL if an error occurred
    *//*---------------------------------------------------------*/
   public static Object getCachedObject ( String filename ) {
      Object retObject = null ;

      if ( enabled ) {
         LogSystem.log ( 1, "Reading Cache Object:"+cacheDir+File.separator+filename ) ;
         retObject = QuestUtil.readObjectFromDisk ( cacheDir+File.separator+filename ) ;
      }

      return( retObject ) ;
   }





   /*------------------------------------------------------------*/
   /* getCacheTime
    */
   /**
    *    This method will return the time that the
    *  cache object was last cached. If the object was never cached,
    *  a date of 1980-01-01.07.00.00.000000 will be returned
    *  
    *
    *  @param filename The filename of the cache object to check
    *  @param None
    *  @param None
    *
    *  @return DB/2 Timestamp version of the time
    *//*---------------------------------------------------------*/
   public static String getCacheTime ( String filename ) {
      String cacheTime = null ;

      try {
         cacheTime = (String) QuestUtil.readObjectFromDisk ( cacheDir+File.separator+filename+".cached_time" ) ;

         if ( cacheTime == null )
            cacheTime = "1980-01-01-07.00.00.000000" ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   This forces all elements of the date to the correct     */
         /* padding                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         cacheTime = CDate.createDate ( CDate.parse ( cacheTime, CDate.DATE_YEAR, CDate.DATE_STAMP ),
                                        CDate.parse ( cacheTime, CDate.DATE_MONTH, CDate.DATE_STAMP ),
                                        CDate.parse ( cacheTime, CDate.DATE_DAY, CDate.DATE_STAMP ),
                                        CDate.parse ( cacheTime, CDate.DATE_HOUR, CDate.DATE_STAMP ),
                                        CDate.parse ( cacheTime, CDate.DATE_MINUTE, CDate.DATE_STAMP ),
                                        0,
                                        CDate.DATE_STAMP ) ;

      } catch ( Exception e ) {
         cacheTime = "1980-01-01-07.00.00.000000" ;
      }

      return( cacheTime ) ;
   }




   /*------------------------------------------------------------*/
   /* getDBTimestamp
    */
   /**
    *    This method will get the current timestamp
    *  from DB/2
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static String getDBTimestamp() {
      if ( currentDBTimestamp != null )
         return( currentDBTimestamp ) ;

      return( getDBTimestampFromDatabase() ) ;
   }





   /*------------------------------------------------------------*/
   /* getDBTimestampFromDatabase
    */
   /**
    *    This method should never be called except
    *  for the getDBTimestamp method and the
    *  time keeping thread. Please do not ever call it!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static String getDBTimestampFromDatabase() {
      String    retTimestamp = null; 
      SQLMethod sql          = null ;

      try {
         Statement stmt = null ;
         ResultSet rs   = null ;

         sql  = new SQLMethod ( DatabaseSystem.QUEST, "getDBTimestamp", 5 ) ;

         if ( sql != null ) {
            stmt = sql.createStatement() ;
            rs   = stmt.executeQuery ( "SELECT CURRENT TIMESTAMP FROM TIGRIS.USERID FETCH FIRST 1 ROWS ONLY" ) ;

            if ( rs.next() )
               retTimestamp = rs.getString(1).trim() ;

            rs.close() ;
         }
      } catch ( Exception e ) {
         sql.rollBack() ;
      }

      if ( sql != null )
         sql.close() ;

      return( retTimestamp ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread keeps track of time. When it initializes,
    *  it will read the time from the database, and it will
    *  then keep updating it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read the time from the database. If the database */
      /* is not yet up, wait 3 seconds, and then try again         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( currentDBTimestamp == null ) {
         try {
            Thread.currentThread().sleep ( 60000 ) ;
         } catch ( Exception e ) {
         }
         currentDBTimestamp = getDBTimestampFromDatabase() ;                              
      }

      parseTimestamp ( currentDBTimestamp ) ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add 1 minute for lag time                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      minute++ ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We want this thread to continue forever                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( true ) {
         try {

            Thread.currentThread().sleep ( 1000 ) ;

            second++ ;

            if ( second > 59 ) {
               second=0 ;
               minute++ ;
            }

            if ( minute > 59 ) {
               minute=0 ;
               hour++ ;
            }

            if ( hour > 24 ) {
               String updateTimestamp = year+"-"+month+"-"+day+"-"+hour+"."+minute+"."+second+".999999" ;

               CDate.addDaysToDate ( updateTimestamp, 1, CDate.DATE_STAMP ) ;
               parseTimestamp ( updateTimestamp ) ;
            }

            currentDBTimestamp = ""+year+"-"+month+"-"+day+"-"+hour+"."+minute+"."+second+".999999" ;
         } catch ( Exception e ) {
         }
      }

   }





   /*------------------------------------------------------------*/
   /* parseTimestamp
    */
   /**
    *    This method will parse the timestamp given
    *  into hours, minutes, seconds, years, months
    *  and days. These static values are stored in this class
    *
    *  @param timestamp The timestamp to parse
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void parseTimestamp ( String timestamp ) {
      year   = CDate.parse ( timestamp, CDate.DATE_YEAR,   CDate.DATE_STAMP ) ;
      month  = CDate.parse ( timestamp, CDate.DATE_MONTH,  CDate.DATE_STAMP ) ;
      day    = CDate.parse ( timestamp, CDate.DATE_DAY,    CDate.DATE_STAMP ) ;
      hour   = CDate.parse ( timestamp, CDate.DATE_HOUR,   CDate.DATE_STAMP ) ;
      minute = CDate.parse ( timestamp, CDate.DATE_MINUTE, CDate.DATE_STAMP ) ;
      second = CDate.parse ( timestamp, CDate.DATE_SECOND, CDate.DATE_STAMP ) ;
   }





   /*------------------------------------------------------------*/
   /* getCacheDir
    */
   /**
    *    This method will return the current directory
    *  that QIT is using to cache to. This does not include the
    *  trailing Path Separator
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getCacheDir () {
      return (cacheDir );
   }
} /* CacheSystem */
