package com.ibm.nzna.projects.qit.app ;                       
/*-----------------------------------------------------------*/
/*   ProcessManagerSystem                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProcessManagerSystem                */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The intent of this sub system is to provide a way for individual
 * processes of an application (editing an item, refreshing a list...)
 * to have a user stop. This means that the stop button displayed on
 * the Main Window will have the ability to send a message to a process
 * that the user has requested for that process to stop. <br><br>
 *
 * Since Quest can handle multiple processes, each press of the 'Stop' 
 * button will result in the last process added to the ProcessManagerSystem
 * to be sent a notification to stop. The ProcessManager works in a FILO
 * flow.<br><br>
 * 
 *
 * For a process to work with the 'Stop' button, it first must implement the
 * QuestProcess interface. Then, when the process starts, it should add itself
 * to the ProcessManagerSystem via the 'addProcess' method. When the process
 * stops regardless if the process was interupted or completed it's life, the
 * 'removeProcess' should be called.
 *
 *
 * @version 1.0, 11/13/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProcessManagerSystem                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProcessManagerSystem extends Object implements ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  The processes vector holds all of the processes being monitored
     **/
   private static Vector processes    = null ;

   /**
     *  All of the stop buttons that could be used to stop a process
     **/
   private static Vector stopButtons  = null ;

   /**
     *  The last time a stop button was pressed, this is the instance of it
     **/
   private static ToolBarButton lastButtonEventSource = null ;


   /**
     *  This is the sole instance of the ProcessManagerSystem
     **/
   private static ProcessManagerSystem instance = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Like all good Sub Systems, this constructor
    *  only initializes the bare properties needed to run
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProcessManagerSystem () {
      processes = new Vector ( 10 ) ;
      stopButtons = new Vector ( 1 ) ;
      instance = this ;
   }





   /*------------------------------------------------------------*/
   /* addProcess
    */
   /**
    *    This method will add a process to the stack. This
    *  process will be the first process to be issued a 'stopProcess'
    *  method if the user hits the 'Stop' Button
    *
    *  @param questProcess The QuestProcess to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addProcess ( QuestProcess questProcess ) {
      processes.insertElementAt ( questProcess, 0 ) ;

      setStopButtonsEnabled ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* removeProcess
    */
   /**
    *    This method will remove a process from the stack.
    *  This method should be called by any process that
    *  has registered itself by calling the 'addProcess' method
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeProcess ( QuestProcess questProcess ) {
      processes.removeElement ( questProcess ) ;
      setStopButtonsEnabled ( processes.size() > 0 ) ;
      try {
         GUISystem.getParentDefWin ( lastButtonEventSource ).setStatus ( null ) ;
      } catch ( Exception e ) {
         // Do nothing here, this is normally a null pointer exception if lastButtonEventSource is null
         // or old
      }
   }





   /*------------------------------------------------------------*/
   /* createStopButton
    */
   /**
    *    This method will return a TaskBarButton
    *  that can be used as a stop button. Please remember
    *  to call 'destroyStopButton' when you are done with it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TaskbarButton
    *//*---------------------------------------------------------*/
   public static ToolBarButton createStopButton() {                 
      ToolBarButton button = new ToolBarButton ( Str.getStr ( STR_STOP ), 
                                                 ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.STOP ), 
                                                 Str.getStr ( STR_STOP ) ) ;

      button.setEnabled ( false ) ;
      button.addActionListener ( getInstance() ) ;

      stopButtons.addElement ( button ) ;
      return( button ) ;
   }





   /*------------------------------------------------------------*/
   /* destroyStopButton
    */
   /**
    *    This method will destroy a stop button that has
    *  been returned VIA the 'createStopButton'
    *  method.
    *
    *  @param stopButton The StopButton to stop
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void destroyStopButton ( ToolBarButton stopButton ) {
      stopButtons.removeElement ( stopButton ) ;
   }





   /*------------------------------------------------------------*/
   /* setStopButtonsEnabled
    */
   /**
    *    This method will either enable or disable
    *  all of the stop buttons.
    *  
    *
    *  @param enable TRUE = Enable them, FALSE = disable them
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void setStopButtonsEnabled ( boolean enabled ) {
      if ( ( stopButtons != null ) && ( stopButtons.size() > 0 ) ) {
         int i    = 0 ;
         int size = stopButtons.size() ;

         while ( i < size ) {
            ((ToolBarButton)stopButtons.elementAt(i)).setEnabled ( enabled ) ;
            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when the user
    *  hits one of our stop buttons. We will take
    *  this event and stop the first running process
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      try {
         QuestProcess questProcess = (QuestProcess)processes.elementAt(0) ;

         if ( questProcess != null ) {
            lastButtonEventSource = (ToolBarButton)e.getSource() ;

            GUISystem.getParentDefWin ( lastButtonEventSource ).setStatusLock ( false ) ;
            GUISystem.getParentDefWin ( lastButtonEventSource ).setStatus ( Str.getStr ( STR_STOPPING ) ) ;
            questProcess.stopProcess() ;
         }
      } catch ( Exception ex ) {
         LogSystem.log ( 1, ex, false ) ;
      }

   }




   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the sole instance of the
    *  ProcessManagerSystem
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static ProcessManagerSystem getInstance() {
      return( instance ) ;
   }
} /* ProcessManagerSystem */
