package com.ibm.nzna.projects.qit.bookmark ;
/*-----------------------------------------------------------*/
/*   Bookmark                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Bookmark                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
/**
 * This record represents a single bookmark. This bookmark can be of 
 * any type of bookmark. Valid types are as follows:<br><pre>
 *
 * 1. Folder          - This means that this bookmark doesn't hold an Object,<br>
 *                      it simply represents a folder.<br>
 * 2. Product         - This bookmark contains an Integer object representing the Product Indicator<br>
 * 3. Document        - This bookmark contains an Integer Object representing the Document Indicator<br>
 * 4. Question        - This bookmark contains a Question as an Object<br>
 * 5. Action          - This bookmark contains a Action as an Object<br>
 * 6. Symptom         - This bookmark contains a Symptom as an Object<br>
 * 7. Link            - This bookmark contains a Link as an Object<br>
 * 8. LinkGroup       - This bookmark contains a Link Group as an Object<br>
 * 10. URL            - This bookmark contains a String as an Object. The String is the URL of the bookmark<br>
 * 11. Document Draft - This bookmark contains an Integer object representing the Document Draft indicator<br><br></pre>
 *
 * <b>Important</b><br>
 * Anything can be a bookmark, however keep in mind that bookmarks are serialized
 * to the database, which means if your object is not serializable, the user will not
 * be able to save their bookmarks. Also, keep in mind that it is a good idea <b>NOT</b>
 * to use an actual Object like 'UserRec' or 'DocRec' or something because those will change,
 * and if they do, the user will have to delete his/her bookmarks. Instead of using the actual
 * object, use some class that won't change. For example, all DocRec bookmarks are represented
 * in the bookmark as an java.lang.Integer object set to their docInd.
 *
 * @version 1.0, 4/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Bookmark                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class Bookmark extends com.ibm.nzna.shared.pom.PersistentRec implements AppConst {

   /*------------------------------------------------------------*/
   /*                                                            */
   /*   This is the serialization ID. If you add fields to this  */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is     */
   /* fine as long as you know what you are doing!               */
   /*                                                            */
   /* If you change this number, you break everyone's Bookmarks  */
   /* to the point where they will not load, so you better write */
   /* a converter or something to run on everyone's bookmarks    */
   /*                                                            */
   /*------------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String    descript       = null ;       // The user defined description of this object
   private Object    object         = null ;       // The Object that this bookmark represents
   private int       objectInd      = 0 ;          // The object indicator used for database access
   private String    parentFolder   = null ;       // The folder that this Bookmark belongs in
   private int       type           = 0 ;          // The type of bookmark that the object represents
   private Hashtable extraProps     = null ;       // Usefull for storing future properties



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Bookmark () {
   }




   /*------------------------------------------------------------*/
   /* isFolder
    */
   /**
    *    This method will return TRUE if this
    *  bookmark represents a folder
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean TRUE if a folder, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean isFolder () {
      return( object == null ) ;
   }





   /*------------------------------------------------------------*/
   /* setObject
    */
   /**
    *    This method will set the object that is 
    *  represented by this Bookmark
    *  
    *
    *  @param object The Object to set.
    *  @param type The type of object, you can use any number except one of the predefined BOOKMARK_XX values defined in com.ibm.nzna.projects.qit.app.AppConst
    *  @param None
    *  @see com.ibm.nzna.projects.qit.app.AppConst
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setObject ( Object object, int type ) {

      this.object = object ;
      this.type   = type ;

   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon that should
    *  represent this bookmark
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      ImageIcon retImage = null ;

      switch ( type ) {
         case BOOKMARK_QUESTION :
         case BOOKMARK_SYMPTOM :
         case BOOKMARK_ACTION :
         case BOOKMARK_LINK :
         case BOOKMARK_LINKGROUP :
            retImage = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.OA_SMALL ) ;
            break ;

         case BOOKMARK_PRODUCT :
            retImage = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.PRODUCT_SMALL ) ;
            break ;

         case BOOKMARK_URL :
            retImage = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.NEW ) ;
            break ;

         default:
            retImage = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.DOCUMENT_SMALL ) ;
            break ;

      }

      return( retImage ) ;
   }





   /*------------------------------------------------------------*/
   /* getDescript
    */
   /**
    *    This method will return our description
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getDescript () {
      return( descript ) ;
   }




   /*------------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    This method will set the descript of
    *  this bookmark
    *  
    *
    *  @param descript The descript to display to the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      this.descript = descript ;
   }





   /*------------------------------------------------------------*/
   /* getBookmarkType
    */
   /**
    *    This method will return one of the bookmark
    *  types as defined in AppConst.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int . For example ( BOOKMARK_DOCUMENT, BOOKMARK_ACTION etc.. )
    *//*---------------------------------------------------------*/
   public int getBookmarkType () {
      return( type ) ;
   }




   /*------------------------------------------------------------*/
   /* getObject
    */
   /**
    *    This method will return the object that
    *  is represented by this bookmark
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getObject () {
      return( object ) ;
   }





   /*------------------------------------------------------------*/
   /* getParentFolder
    */
   /**
    *    This method will return the name of the
    *  parent folder of this bookmark
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getParentFolder () {
      return( parentFolder ) ;
   }





   /*------------------------------------------------------------*/
   /* setParentFolder
    */
   /**
    *    This method will set the parent folder
    *  name of this bookmark
    *  
    *
    *  @param parentFolder The name of the folder
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setParentFolder ( String parentFolder ) {
      this.parentFolder = parentFolder ;
   }




   /*------------------------------------------------------------*/
   /* putExtraProp
    */
   /**
    *    This method will place an extra property
    *  on this bookmark
    *  
    *
    *  @param property The String name of the property
    *  @param value The value
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void putExtraProp ( String property, Object value ) {
      if ( extraProps == null )
         extraProps = new Hashtable() ;

      extraProps.put ( property, value ) ;
   }





   /*------------------------------------------------------------*/
   /* getExtraProp
    */
   /**
    *    This method will return an extra property
    *  
    *  
    *
    *  @param property The property to get
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getExtraProp ( String property ) {
      if ( extraProps != null )
         return( extraProps.get ( property ) ) ;
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* clone
    */
   /**
    *    This method will return a new instance of this
    *  object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object clone () {
      Bookmark bookmark = new Bookmark() ;
      
      bookmark.descript     = descript ;
      bookmark.object       = object ;
      bookmark.objectInd    = objectInd ;
      bookmark.parentFolder = parentFolder ;
      bookmark.type         = type ;
      bookmark.extraProps   = (Hashtable)extraProps.clone() ;

      return ( bookmark ) ;
   }

} /* Bookmark */
