package com.ibm.nzna.projects.qit.chat ;  
/*-----------------------------------------------------------*/
/*   ChatServer                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ChatServer                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.net.* ;
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This server runs through the life of the input tool. It communicates with 
 * other input tools. The server monitors the port number that is defined in the
 * contant 'LISTEN_PORT'. The chat server is also responsible for dispatching
 * data received from the port to the registered listeners. Each listener registers
 * with an 'Initialize String Command'. This Initialize String Command is always sent
 * as the first data string from a client Input Tool. <br>
 *
 * Some Initialize String Command functions are already defined and should not be used
 * by anyone but the ChatServer. These commands are defined as static final Strings in
 * this class.<br><br>
 *
 * The typical chat sessions are as follows:<br><br>
 *
 * Initiating a new chat:<br>
 * - Client calls ChatServer.beginChat ( ipAddr, InitializeStringCommand, Object )<br>
 * - ChatServer returns ChatSession depending on if other client responds. NULL if no response<br>
 * - Client can then use 'sendData' method to send a command and receive a response<br>
 * - Client calls ChatServer.endChat ( ChatSession )<br><br><br>
 *
 * Another client calls you:<br>
 * - First, your class must implement 'ChatListener' and must already be added to the<br>
 *   ChatServer. This is done by ChatServer.addChatListener method<br>
 * - Your client's 'dataReceived' method will be called with a String and Object.<br>
 * - You can then respond by returning data from the dataReceived method, or NULL<br>
 * - Other client issues endChat, which breaks the connection<br>
 
 *
 * @version 1.0, 5/8/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ChatServer                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ChatServer extends Object implements Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public  static final int    LISTEN_PORT         = 7469 ;                     
   private static final byte   END_COMMAND_STRING  = 3 ;
   private static final byte   END_DATA            = 4 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public, static Socket commands                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String VERIFY_CHAT_USER  = "VERIFY_CHAT_USER" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static ChatServer     instance     = null ;
   private static String         ipAddress    = null ;
   private static ServerSocket   serverSocket = null ;
   private static int            tempFileNum  = 0 ;

   private boolean      runServer      = true ;
   private Thread       serverThread   = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create and start the
    *  server. If the server is already running, nothing
    *  will happen
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ChatServer ( ) {                                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the instance static var is null, we are not running  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( instance == null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Assign local variables                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         instance = this ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start the server                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         startServer() ;

      }
   }






   /*------------------------------------------------------------*/
   /* startServer
    */
   /**
    *    This method will start the server that will
    *  monitor our socket for communication from
    *  other tools
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void startServer () {
      try {
         serverSocket = new ServerSocket ( LISTEN_PORT ) ;
         ipAddress = InetAddress.getLocalHost().getHostAddress() ;
         LogSystem.log ( 1, "ChatServer serving from TCP/IP Address:"+ipAddress ) ;

         serverThread = new Thread ( this ) ;
         serverThread.start() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getLocalAddress
    */
   /**
    *    This method will return the local client's
    *  IP Address in the form of xxx.xxx.xxx.xxx
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static final String getLocalAddress () {
      return( ipAddress ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our server 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      while ( runServer ) {
         try {
            Socket socket  = serverSocket.accept() ;
            String command = getStringFromSocket ( socket ) ;

            LogSystem.log ( 1, "ChatServer received socket. Command:"+command ) ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* stopServer
    */
   /**
    *    This method will stop the server
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopServer () {
      runServer = false ;

      if ( serverThread != null )
         serverThread.interrupt() ;
   } /* stopServer */






   /*------------------------------------------------------------*/
   /* getStringFromSocket
    */
   /**
    *    This method will read a string from the
    *  socket and will return the string that was
    *  sent in the socket
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private static final String getStringFromSocket ( Socket socket ) throws Exception {
      BufferedInputStream in     = new BufferedInputStream ( socket.getInputStream() ) ;
      String              retStr = "" ;
      byte[]              bytes  = new byte[1024] ;
      boolean             c      = true ;

      while ( c ) {
         if ( in.read ( bytes, 0, bytes.length ) > 0 ) {
            retStr+=new String ( bytes ) ;
         } else
            c = false ;
      }

      in.close() ;
      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* beginChat
    */
   /**
    *    This method will begin a new Chat session
    *  with the I/P Address passed in
    *  
    *
    *  @param ipAddr The IP Address of the client to communicate with
    *  @param initializeCommandString The Initialize Command String to send
    *  @param object Any serializable data object. If this object is not serializable, NULL is sent
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static ChatSession beginChat ( ChatListener listener, String ipAddr, String initializeStringCommand, Object data ) throws Exception {
      ChatSession chatObject = new ChatSession() ;
      try {
         Socket       socket    = new Socket ( ipAddr, LISTEN_PORT ) ;
         OutputStream out       = socket.getOutputStream() ;
         byte[]       dataBytes = null ;            


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the ChatSession up                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         chatObject.setIPAddr       ( ipAddr ) ;
         chatObject.setSocket       ( socket ) ;               
         chatObject.setChatListener ( listener ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Send the initial command                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sendAndReceiveData ( chatObject, initializeStringCommand, data ) == null )
            chatObject = null ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         return(null) ;
      }

      return( chatObject ) ;
   }






   /*------------------------------------------------------------*/
   /* getObjectBytes
    */
   /**
    *    This method will return the serialized
    *  bytes from an object
    *  
    *
    *  @param chatObject The ChatSession for this session
    *  @param data The data object
    *  @param None
    *
    *  @return byte[]
    *//*---------------------------------------------------------*/
   private static byte[] getObjectBytes ( ChatSession chatObject, Object object ) {
      try {
         String   tempFilename = chatObject.getUniqueId()+(tempFileNum++) ;

         FileUtil.writeObjectToDisk ( tempFilename , object ) ;

         return( FileUtil.readBytesFromFile ( tempFilename ) ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* sendData
    */
   /**
    *    This method will send the data that is
    *  passed to the client who matches the
    *  ChatSession passed. It will return immediatly
    *
    *  @param chatObject The ChatSession representing this client's chat session
    *  @param command The String command to send
    *  @param data The data to send
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void sendData ( ChatSession chatObject, String command, Object data ) throws Exception {
      Socket       socket    = chatObject.getSocket() ;
      OutputStream out       = socket.getOutputStream() ;
      byte[]       dataBytes = getObjectBytes ( chatObject, data ) ;

      out.write ( command.getBytes() ) ;
      out.write ( END_COMMAND_STRING ) ;

      if ( dataBytes != null )
         out.write ( dataBytes ) ;
      out.write ( END_DATA ) ;

      out.flush() ;

   }





   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will wait for the client that
    *  the ChatSession passed is communicating with
    *  to respond, and will return the data
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The object sent from the other client
    *//*---------------------------------------------------------*/
   public static Object getData ( ChatSession chatObject ) throws Exception {
      Socket               socket    = chatObject.getSocket() ;
      InputStream          in        = socket.getInputStream() ;
      int                  maxChunk  = 1024 ;
      byte[]               fByte     = new byte[ maxChunk ] ;
      byte[]               retBytes  = null ;
      int                  i         = 0 ;
      int                  readCount = 0 ;
      Object               retObject = null ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in all of the bytes                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( readCount >= 0 ) {
         readCount = in.read ( fByte ) ;
         if ( readCount>= 0 ) {
            if ( retBytes == null ) {
               retBytes = new byte[readCount] ;
               System.arraycopy ( fByte, 0, retBytes, 0, readCount ) ;
            } else {
               byte[] tempbyte = new byte[retBytes.length] ;

               System.arraycopy ( retBytes, 0, tempbyte, 0, retBytes.length ) ;
               retBytes = new byte [ tempbyte.length+readCount ] ;
               System.arraycopy ( tempbyte, 0, retBytes, 0, tempbyte.length ) ;
               System.arraycopy ( fByte, 0, retBytes, tempbyte.length, readCount ) ;
            }
         }
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, form the object                                    */           
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retBytes != null ) {
         ByteArrayInputStream byteIn   = new ByteArrayInputStream ( retBytes ) ;
         ObjectInputStream    objectIn = new ObjectInputStream ( byteIn ) ;

         retObject = objectIn.readObject() ;
         objectIn.close() ;
         byteIn.close() ;
      }

      return( retObject ) ;
   }




   /*------------------------------------------------------------*/
   /* sendAndReceiveData
    */
   /**
    *    This method will send the data passed in and
    *  will not return until data is sent back from the 
    *  other computer.
    *
    *  @param chatObject The chat object that represents the communication session
    *  @param command The String command to send
    *  @param data The data object to send
    *
    *  @return Object returned from other client
    *//*---------------------------------------------------------*/
   public static Object sendAndReceiveData ( ChatSession chatObject, String command, Object data ) throws Exception {
      sendData ( chatObject, command, data ) ;
      return( getData ( chatObject ) ) ;
   }






   /*------------------------------------------------------------*/
   /* getValidUsers
    */
   /**
    *    This method will return a list of logged on users
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getValidUsers () {
      SQLMethod sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "readUsers (Chat)", 5 ) ;
      ChatUser  newRec       = null ;
      Vector    users        = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read all valid users                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT NAME, IPADDR "+
                                              "FROM TIGRIS.USERID "+
                                              "WHERE IPADDR IS NOT NULL AND "+
                                              "IPADDR <> '000.000.000.000' AND "+
                                              "USERID <> '"+UserSystem.getUserId()+"' FOR FETCH ONLY" ) ;

         users.removeAllElements() ;
         while ( rs.next() ) {
            newRec = new ChatUser ( rs.getString(1).trim(), rs.getString(2).trim() ) ;
            users.addElement ( newRec ) ;
         }

         rs.close() ;


      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_COULD_NOT_READ_USERS ) ;
      }

      return( users ) ;
   }
} /* ChatServer */
