package com.ibm.nzna.projects.qit.doc.storedProc.docListReader ;   
/*-----------------------------------------------------------*/
/*   DocListReader                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocListReader                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.doc.DocumentRow ;
import com.ibm.nzna.shared.util.CDate ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.Clob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read a Vector of DocumentRow objects that are read from the database.
 * The first parameter passed 'query' is a query clause to be run. The Query MUST select
 * the following values in the following order:
 * 
 * <pre>
 * 1. DOCIND - The Document Indicator from QUEST.DOCUMENTS. If reading a list of<br>
 *             Documents from TIGRIS.DOCUMENTS, use a '0' in place of this column in<br>
 *             the select clause<br>
 * 2. EXISTINGDOCIND - The Existing Document Indicator from TIGRIS.DOCUMENTS.<br>
 * 3. TITLE - The Title of the document<br>
 * 4. OWNER - The Owner of the document<br>
 * 5. DOCCLASSIND - The Classification of the document<br>
 * 6. LNDOCID - The Lotus Notes ID of the Document<br>
 * 7. WORKIND - The Work Required Indicator<br>
 * 8. DBUSER - The DBUSER column from the XXX.DOCUMENTS table<br>
 * 9. CHANGEDTIME - The CHANGEDTIME column from the XXX.DOCUMENTS table<br>
 * <pre><br><br>
 *
 * Paramter #4 (startReadIndex) is used so that you can specify how many documents to skip over
 * before starting the record results. For example, If you already have the first 100 results of
 * your query from a previous call to this procedure, you can execute another call to this stored procedure
 * with the 'startReadIndex' set to 100 so that the records you already have read in are not returned again.
 * This is <b>particularly</b> useful when 'stepping' through 15,000 documents. Obviously you wouldn't want
 * this procedure to return that much data, so you only read in 200 or 300 at a time.<br><br>
 *
 * <b>
 * Main Execution Point</b>: readDocList
 *
 * <br><br><pre><b>
 * Param Number   Descript<br>
 * -------------- -------------------------------------------</b><br>
 * 1              query          - The Query to run<br>
 * 2              maxReturn      - The maximum amount of records to return. <b>Note</b>: Pass a -1 to return all<br>
 * 3              startReadIndex - How many records to skip before reading  <br>
 * 4              vector         - A Vector of DocRecs<br>
 * 5              StoredProcRec ( Allocated and returned by DocListReader )<br>
 * <br>
 * </pre>
 * <br><br>
 *
 * <b>Note</b> This procedure automatically adds 'FETCH FIRST x ROWS ONLY' to all queries if the startReadIndex,
 * is none 0. The 'x' in the clause is the sum of maxReturn and startReadIndex. If maxReturn is -1 
 * then the FETCH FIRST clause is not added<br><br>
 *
 * <b>Note</b> The 'actualRecordCount' is only populated if startReadIndex is 0<br><br><br>
 *
 * <b>Note</b> Do not add 'FOR FETCH ONLY' to your query, this is done automatically
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocListReader                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocListReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec       = new StoredProcRec () ;
   private Vector        returnVec       = null ;
   private Connection    con             = null ;
   private String        query           = null ;
   private int           startReadIndex  = 0 ;
   private int           maxRecordReturn = 0 ;
   private int           actualReadCount = 0 ;





   /*------------------------------------------------------------*/
   /* readDocList
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readDocList ( Clob query, int maxRecordReturn, int startReadIndex, int actualRecordCount, Blob returnBlob, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our return vector                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVec            = new Vector ( 1, 10 ) ;
         this.query           = StoredProcUtil.getStringFromClob ( query ) ;
         this.maxRecordReturn = (maxRecordReturn==-1) ? 100000 : maxRecordReturn ;
         this.startReadIndex  = startReadIndex ;                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Append the 'Fetch First' clause                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( startReadIndex != 0 )
            this.query+=" FETCH FIRST "+(maxRecordReturn+startReadIndex)+" ROWS ONLY" ;

         this.query+=" FOR FETCH ONLY" ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in our data                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         readData() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our Vector and return rec                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnBlob = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( returnBlob, returnVec );
         set ( 4, actualReadCount ) ;
         set ( 5, returnBlob ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 6, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readData
    */
   /**
    *    This method will read all of our data in
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readData () throws SQLException {
      Statement stmt          = con.createStatement() ;
      ResultSet rs            = stmt.executeQuery ( query ) ;
      int       count         = 0 ;    
      String    changedTime   = null ;
      boolean   isRequest     = ( query.toUpperCase().indexOf ( "QUEST.DOCUMENT" ) >= 0 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Run the Query                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( query ) ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read in the results                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {

         actualReadCount++ ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start Reading after the startReadIndex                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( count < (maxRecordReturn+startReadIndex) ) {
            if ( count >= startReadIndex ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in the record                                      */
               /*                                                           */
               /*-----------------------------------------------------------*/
               changedTime = rs.getString(9) ;
               returnVec.addElement ( new DocumentRow ( ""+rs.getInt(1),             // Document Indicator
                                                        ""+rs.getInt(2),             // Existing Document Indicator
                                                        rs.getString(3).trim(),      // Title
                                                        rs.getString(4),             // Owner
                                                        rs.getInt(5),                // Document Classification
                                                        rs.getString(6),             // Filename
                                                        rs.getInt(7),                // Work Required
                                                        rs.getString(8).trim(),      // Last Touched By
                                                        CDate.convertDate ( CDate.DATE_STAMP,
                                                                            CDate.DATE_WW_DATETIME,
                                                                            changedTime ),      // Last Touched pretty
                                                        changedTime,      // Last Touched Stamp
                                                        isRequest ) ) ;

            } /* if count >= startReadIndex */

            count++ ;

         } /* if maxRecordReturn  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have read to our limit and are not reading the    */
         /* rest of the items to provide the 'actualRecordCount'      */
         /* value, break out of the loop here                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            if ( startReadIndex != 0 )
               break ;
         }

      } /* while  */




      rs.close() ;
      stmt.close() ;

   }

} /* DocListReader */
