package com.ibm.nzna.projects.qit.ftp ;
/*-----------------------------------------------------------*/
/*   FTPSite                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPSite                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The FTP Site is a bean that can be used to transfer and
 * manipulate files on an FTP Server
 *
 * @version 1.0, 6/1/1999  
 * @author  David Lentz
 */
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.ftp.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   FTP Site                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPSite extends Object implements FtpListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   FTP SITES                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int FTPSITE_FILES_PROD               = 1 ;
   public static final int FTPSITE_FILES_TEMP               = 2 ;
   public static final int FTPSITE_IMAGES_PROD              = 3 ;
   public static final int FTPSITE_IMAGES_TEMP              = 4 ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  String       host      = null ;
   private  String       user      = null ;
   private  String       password  = null ;
   private  String       localDir  = "" ;
   private  String       remoteDir = "" ;
   private  Ftp          ftp       = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create an FTPSite instance
    *  with the supplied values as default
    *  
    *
    *  @param host The host FTP machine to connect to
    *  @param user The user id for the FTP account
    *  @param localDir The local directory on the client to be in
    *  @param remoteDir The remote directory on the host machine to be in
    *  @param password The password to use for the user id
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FTPSite ( String host, String user, String localDir, String remoteDir, String password ) {
      this.host      = host ;
      this.user      = user ;
      this.localDir  = localDir ;
      this.remoteDir = remoteDir ;
      this.password  = password ;
      if ( this.localDir == null )
         this.localDir = "" ;
      if ( this.remoteDir == null )
         this.remoteDir = "" ;
      ftp            = new Ftp() ;
      ftp.addFtpListener ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* getHost
    */
   /**
    *    This method will return what host
    *  this FTP Bean is connected to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getHost() {
      return( host ) ;
   }





   /*------------------------------------------------------------*/
   /* connect
    */
   /**
    *    This method will make the initial connection
    *  to the FTP host. This method will not log
    *  you in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true if connected
    *//*---------------------------------------------------------*/
   public boolean connect() {
      boolean rc = false ;
      try {
         ftp.connect( host );
         LogSystem.log ( 1, "Connected To Server:"+host ) ;
         rc = true ;
      } catch ( Exception e ) {
         showError ( Str.getStr ( STR_COULD_NOT_CONNECT_TO_FTP )+host ) ;
         LogSystem.log  ( 1, "FTPSite - connect ( "+host+" )" ) ;
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* login
    */
   /**
    *    This method will login the currect user into
    *  the current ftp host.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if user was logged in
    *//*---------------------------------------------------------*/
   public boolean login() {
      boolean rc = false ;

      try {
         LogSystem.log ( 1, "logging into server:"+host ) ;
         ftp.login( user, password );

         /* Set the transfer type to binary */
         setBinary() ;

         /* Set the current directories */
         ftp.cd ( remoteDir ) ;
         LogSystem.log ( 1, "Changing Directory to "+remoteDir ) ;

         rc =  setLocalDir ( localDir ) ;
      } catch ( Exception e ) {
         showError ( Str.getStr ( STR_COULD_NOT_LOG_INTO_FTP ) ) ;
         LogSystem.log ( 1, "FTPSite, Loggin in with user "+user ) ;
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* move
    */
   /**
    *    This method will move a file from the
    *  client to the host. This method will delete
    *  the file from the client once it has been moved
    *
    *  @param filename The filename to move
    *
    *  @return TRUE if moved
    *//*---------------------------------------------------------*/
   public boolean move( String filename ) {
      boolean rc = false ;

      try {
         /* place the file on the server */
         if ( put ( filename ) )
            /* If there was no error remove the file */
            FileUtil.deleteFile ( filename ) ;
         rc = true ;

      } catch ( Exception e ) {
         showError ( Str.getStr ( STR_CANNOT_MOVE_FILE )+filename ) ;
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* delete
    */
   /**
    *    This method will delete a file from 
    *  the host FTP Server and the Client
    *  
    *
    *  @param filename The file to delete
    *  @param None
    *  @param None
    *
    *  @return TRUE if the file was deleted
    *//*---------------------------------------------------------*/
   public boolean delete ( String filename ) {
      boolean rc = false ;

      try {
         /* delete the file on the server */
         ftp.delete( filename ) ;
         FileUtil.deleteFile ( filename ) ;
         rc = true ;
      } catch ( Exception e ) {
         showError ( Str.getStr ( STR_CANNOT_FTP_DELETE )+filename ) ;
         LogSystem.log ( 1, "FTPSite - delete ( "+filename+" )" ) ;
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will return TRUE if the
    *  object passed is an equal FTPSite instance
    *  
    *
    *//*---------------------------------------------------------*/
   public boolean equals ( FTPSite ftpSite ) {
      boolean rc = false ;

      try {
         if ( ( ftpSite.host.equals( this.host ) ) &&
              ( ftpSite.user.equals( this.user ) ) &&
              ( ftpSite.remoteDir.equals ( this.remoteDir ) ) )
            rc = true ;
      } catch ( NullPointerException e ) {
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* put
    */
   /**
    *    This method will put a file from the client
    *  to the FTP Server
    *  
    *
    *  @param filename The file to put
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean put( String filename ) {
      boolean rc = false ;

      try {
         new StreamPut ( ftp, new File ( localDir+filename ), filename, false ).start() ;
         LogSystem.log ( 1, "Put File:"+filename+"   From Local:"+localDir+filename ) ;
         rc = true ;

      } catch ( FtpError ftpError ) {
         int errorCode = ftpError.getReplyCode() ;

         /* Check for file doesn't exist on FTP Server  */
         if ( errorCode == 550 ) {
            //new ModalPrintBox ( SystemData.getMainWin(), Str.getStr ( STR_ERROR ), Str.getStr ( STR_FILENOTONFTP )+filename, ICON_ERROR, null ).getResult() ;
            rc = true ;
         }

         else if ( errorCode == 452 ) {
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_FTP_OUT_OF_DISK_SPACE ) ) ;
         } else {

            LogSystem.log ( 1, "FTP PUT Returned an error code of:"+errorCode ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* get
    */
   /**
    *    This method will get the file from the host
    *  and save it to the client.
    *  
    *
    *  @param filename The filename to get
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean get( String filename ) {
      boolean rc = false ;

      try {
         //ftp.get( filename );

         LogSystem.log ( 1, "Getting File:"+filename+"   To Local:"+localDir+filename ) ;
         new StreamGet ( ftp, new File ( localDir+"ftmp_"+filename ), filename ).start() ;

         /* Ok, we downloaded the file, rename it to it's correct name  */
         FileUtil.copyFile ( localDir+"ftmp_"+filename, localDir+filename ) ;
         FileUtil.deleteFile ( localDir+"ftmp_"+filename ) ;

         LogSystem.log ( 1, "Get File returning TRUE" ) ;
         rc = FileUtil.exists ( localDir+filename ) ;
      } catch ( FtpError ftpError ) {
         int errorCode = ftpError.getReplyCode() ;

         /* Check for file doesn't exist on FTP Server  */
         if ( errorCode == 550 ) {

            /* Delete the file because the FTP Bean will create a file with 0 bytes  */
            try {
               LogSystem.log ( 1, "Deleting file: "+localDir+"ftmp_"+filename+" Because 550 Error Code was received" ) ;
               FileUtil.deleteFile ( localDir+"ftmp_"+filename ) ;
            } catch ( Exception e ) {
               GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_COULD_NOT_DELETE_FILE ) ) ;
               LogSystem.log ( 1, e, false ) ;
            }
            rc = true ;
         }


         ftpError.printStackTrace() ;
         LogSystem.log ( 1, "FTP GET Returned an error code of:"+errorCode ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }


   /*-----------------------------------------------------------*/
   /* getRemoteDir
    */
   /**
    *    This method will return the remote directory
    *  that the FTPSite class will either send or
    *  put files to
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return String the String representation of a remoteDir
    */
   public String getRemoteDir () {
      try {
         return( ftp.pwd() ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( "" ) ;
   }





   /*------------------------------------------------------------*/
   /* setBinary
    */
   /**
    *    This method will set the current file transaction
    *  processes to binary mode
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean setBinary() {
      boolean rc = false ;

      try {
         ftp.setType ( ftp.BINARY ) ;
         LogSystem.log ( 1, "FTP Set to Binary" ) ;
         rc = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setAscii
    */
   /**
    *    Basically calls the 'ascii' command on the
    *  FTP.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if passed
    *//*---------------------------------------------------------*/
   public boolean setAscii() {
      try {
         ftp.setType ( ftp.ASCII ) ;
         LogSystem.log ( 1, "FTP Set to ASCII" ) ;
         return(true) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( false ) ;
   }






   /*------------------------------------------------------------*/
   /* disconnect
    */
   /**
    *    This method will disconnect from the FTP Server
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if disconnected
    *//*---------------------------------------------------------*/
   public boolean disconnect() {
      boolean rc = false ;

      try {
         ftp.disconnect() ;
         rc = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setRemoteDir
    */
   /**
    *    This method will set the remote directory
    *  that file transactions will happen at
    *  
    *
    *  @param dir The directory to change to
    *  @param None
    *  @param None
    *
    *  @return TRUE if it was changed
    *//*---------------------------------------------------------*/
   public boolean setRemoteDir ( String dir ) {
      boolean rc = false ;

      try {
         if ( dir != null ) {
            remoteDir = dir ;
            ftp.cd ( dir ) ;
            rc = true ;
         }
      } catch ( Exception e ) {
         showError ( Str.getStr ( STR_CANNOT_FTP_CHANGE_DIR ) ) ;
         LogSystem.log ( 1, "FTPSite, setRemoteDir to "+dir+"  on host:"+host ) ;
         LogSystem.log ( 1, e ) ;
      }
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* setLocalDir
    */
   /**
    *    This method will set the local directory
    *  that the get and put commands use to get
    *  file and put file
    *
    *  @param localDir The full path of the local directory
    *  @param None
    *  @param None
    *
    *  @return true if it was set
    *//*---------------------------------------------------------*/
   public boolean setLocalDir ( String localDir ) {
      boolean rc = false ;

      if ( localDir != null ) {
         this.localDir = localDir ;
         if ( !this.localDir.endsWith ( File.separator ) )
            this.localDir=this.localDir+File.separator ;

         rc = true ;
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getFTPSite
    */
   /**
    *    This method will return an instance of 
    *  a specific FTP Site. The siteInd is defined
    *  in Const.java and can be FTPSITE_IMAGES_PROD etc..
    *
    *  @param siteInd The site to get
    *  @param None
    *  @param None
    *
    *  @return FTPSite The instance of the FTP Bean
    *//*---------------------------------------------------------*/
   public static FTPSite getFTPSite ( int siteInd ) {
      FTPSite  ftpSite   = null ;
      String   host      = null ;
      String   user      = null ;
      String   localDir  = null ;
      String   remoteDir = null ;
      String   password  = null ;
      localDir           = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

      user=null ;

      switch ( siteInd ) {
         case FTPSITE_FILES_TEMP:
            host        = Constants.getStrConst ( "IT_FILEHOSTTEMP" ) ;
            remoteDir   = Constants.getStrConst ( "IT_FILELOCTEMP" ) ;
            /* If we do not have a temp server we will use the production server */
            if ( host == null ) {
               host        = Constants.getStrConst ( "IT_FILEHOST" ) ;
               remoteDir   = Constants.getStrConst ( "IT_FILELOC" ) ;
            }
            user="quest" ;
            password="hgra2l" ;
            ftpSite     = new FTPSite ( host, user, localDir, remoteDir, password ) ;
            break ;
         case FTPSITE_FILES_PROD:
            host        = Constants.getStrConst ( "IT_FILEHOST" ) ;
            remoteDir   = Constants.getStrConst ( "IT_FILELOC" ) ;
            user="quest" ;
            password="hgra2l" ;
            ftpSite     = new FTPSite ( host, user, localDir, remoteDir, password ) ;
            break ;
         case FTPSITE_IMAGES_TEMP:
            host        = Constants.getStrConst ( "IT_IMAGEHOSTTEMP" ) ;
            remoteDir   = Constants.getStrConst ( "IT_IMAGELOCTEMP" ) ;
            /* If we do not have a temp server we will use the production server */
            if ( host == null ) {
               host        = Constants.getStrConst ( "IT_IMAGEHOST" ) ;
               remoteDir   = Constants.getStrConst ( "IT_IMAGELOC" ) ;
            }
            user="quest" ;
            password="hgra2l" ;
            ftpSite     = new FTPSite ( host, user, localDir, remoteDir, password ) ;
            break ;
         case FTPSITE_IMAGES_PROD:
            host        = Constants.getStrConst ( "IT_IMAGEHOST" ) ;
            remoteDir   = Constants.getStrConst ( "IT_IMAGELOC" ) ;
            user=FTP_USERID_PRODUCTION ;
            password="k1ngarthur" ;
            ftpSite     = new FTPSite ( host, user, localDir, remoteDir, password ) ;
            break ;
      }

      return( ftpSite ) ;
   }


   /*------------------------------------------------------------*/
   /* removeFile
    */
   /**
    *    This method will remove a file from this 
    *  site
    *  
    *
    *  @param filename The Filename to delete
    *  @param local TRUE = Delete the local, FALSE = delete the remote file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean delete ( String filename, boolean local ) {
      boolean rc = false ;

      try {
         ftp.delete ( filename ) ;
         if ( local )
            FileUtil.deleteFile ( filename ) ;
         rc = true ;
      } catch ( FtpError ftpError ) {
         int errorCode = ftpError.getReplyCode() ;

         /* Check for file doesn't exist on FTP Server, if it doesn't don't worry about it  */
         if ( errorCode == 550 )
            rc = true ;

         LogSystem.log ( 1, "FTP PUT Returned an error code of:"+errorCode ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* getLocalDir
    */
   /**
    *    This method will return what local directory
    *  the FTP Protocol is using
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String representation of the local dir
    *//*---------------------------------------------------------*/
   public String getLocalDir () {
      return( localDir ) ;
   }


   public void ftpReply(ReplyEvent re) {
      MainWindow.getInstance().setStatus ( re.getReply().getMessage() ) ;

   }
   public void transferEnded(TransferEvent te) {
   }

   public void transferProgress(TransferEvent te) {
   }

   public void transferStarted(TransferEvent te) {
   }




   /*------------------------------------------------------------*/
   /* list
    */
   /**
    *    This method will return a string of filenames
    *  that are in the current remote directory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String[]
    *//*---------------------------------------------------------*/
   public String[] list () {
      try {
         return( ftp.list() ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return(null) ;
   }




   /*------------------------------------------------------------*/
   /* list
    */
   /**
    *    This method will return a list of files
    *  that match the filename passed in. The filename
    *  may have wildcards
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String[]
    *//*---------------------------------------------------------*/
   public String[] list ( String filename ) {
      try {
         return( ftp.list ( filename ) ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return(null) ;
   }




   /*------------------------------------------------------------*/
   /* showError
    */
   /**
    *    A simple routine that will pop up a modal
    *  print box for the FTP errors
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showError ( String errorMsg ) {
      GUISystem.printBox ( Str.getStr ( STR_ERROR ), errorMsg ) ;
   }
}
