package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   NavList                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NavList                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This control basically subclasses JPanel and allows the programmer to
 * add Components to it. Each component that is added will appear in a list like
 * format from top to bottom. If the control is a DButton, the button will be added
 * and it will act like a URL Link in a web browser. If the DButton is selected, it's
 * font will change to a bold font, and all other DButton fonts will return to normal.
 * To have scroll bars, add this component to a JScrollPane.
 *
 * To use this control correctly depends on how many items you have in the list. For example,
 * if you have less than 3 or 4 DButtons in the list, handle the actionPerformed from the buttons
 * yourself. However, if you have 4 or greater, add yourself as an action listener to this
 * panel and you will be sent the correct action event for the button. This simply eliminates you
 * from having to add yourself as an action listener to each button
 *
 * A set of controls maybe grouped into a NavListGroup. The NavListGroup will be an expandable box
 * inside of the NavList.
 *
 * @version 1.0, 7/9/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   NavList                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class NavList extends DPanel implements ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final Font     defaultFont = new Font ( "SansSerif", Font.PLAIN, 12 ) ;
   private static final Insets   insets      = new Insets ( 0, 0, 0, 0 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Dimension prefSize          = new Dimension ( 0, 0 ) ;
   private DButton   curSelectedButton = null ;
   private Vector    actionListenerVec = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a blank NavList
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavList () {
      setLayout ( null ) ;
      setBackground ( Color.white ) ;
   }



   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will add a component to the
    *  list of components that this panel is displaying
    *  
    *
    *  @param component The Component to add
    *
    *  @return The component added
    *//*---------------------------------------------------------*/
   public Component add ( Component comp ) {

      if ( comp instanceof DButton ) {
         DButton button = (DButton) comp ;

         button.setMargin ( insets ) ;
         button.actAsLink ( true ) ;
         button.setFont ( defaultFont ) ;
         button.setBorder ( null ) ;
         button.setFocusPainted ( false ) ;
         button.setOpaque ( false ) ;
         button.setContentAreaFilled ( false ) ;
         button.addActionListener ( this ) ;
         button.setHorizontalAlignment ( button.LEFT ) ;
         button.setHighlightForeground ( Color.blue ) ;
      }

      else if ( comp instanceof HotLinkLabel )
         ((HotLinkLabel)comp).addActionListener ( this ) ;

      invalidate() ;
      return( super.add ( comp ) ) ;

   }


   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out our controls
    *  in top to bottom order with only 1 column
    *  of controls
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension   size      = getSize() ;
      Component[] component = getComponents() ;
      int         i         = 0 ;
      Insets      insets    = getInsets() ;
      int         y         = 0 ;
      int         x         = 0 ;

      if ( insets == null ) {
         x=0 ;
      } else {
         insets.left+=2 ;
         x = insets.left ;
         y = insets.top ;
         size.width-=insets.right-insets.left ;
         size.height-=insets.top-insets.bottom ;
      }


      if ( ( component != null ) && ( component.length >= 0 ) ) {
         Dimension compPrefSize = null ;

         while ( i < component.length ) {
            compPrefSize = component[i].getPreferredSize() ;
            component[i].setBounds ( x, y, size.width-(x*2), compPrefSize.height ) ;
            y+=compPrefSize.height ;
            i++;
         }
      }

      super.doLayout() ;
   }







   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred
    *  size of the panel. This size will be large enough
    *  to accomodate all controls in a list manner
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      Component[] component = getComponents() ;
      int         i         = 0 ;

      if ( ( component != null ) && ( component.length > 0 ) ) {
         Dimension compPrefSize = null ;

         prefSize.height=0 ;
         prefSize.width=0 ;

         while ( i < component.length ) {
            compPrefSize = component[i].getPreferredSize() ;
            prefSize.height+= compPrefSize.height ;
            if ( compPrefSize.width > prefSize.width )
               prefSize.width = compPrefSize.width ;
            i++ ;
         }
      }

      return( prefSize ) ;
   }





   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    Simply returns the same value as getPreferredSize
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }







   /*------------------------------------------------------------*/
   /* getSelectedButton
    */
   /**
    *    If this list contains at least 1 DButton component
    *  and that component has been selected (Pushed)
    *  this method will return that component
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DButton
    *//*---------------------------------------------------------*/
   public DButton getSelectedButton () {
      return( curSelectedButton ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are waiting for has been fired
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() instanceof DButton ) {
         DButton button = (DButton) e.getSource() ;

         setFonts ( defaultFont ) ;
         //button.setFont ( FontSystem.smallTitleFont ) ;
         curSelectedButton = button ;
         repaint() ;
         fireActionEvent ( e ) ;
      } else if ( e.getSource() instanceof HotLinkLabel ) {
         HotLinkLabel button = (HotLinkLabel) e.getSource() ;
         //repaint() ;
         fireActionEvent ( e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setButtonFonts
    */
   /**
    *    This method will set all fonts in our
    *  list to the font passed in
    *  
    *
    *  @param font The new font to set all DButtons to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setFonts ( Font font ) {
      Component[] components = getComponents() ;
      int         i          = 0 ;

      if ( ( components != null ) && ( components.length > 0 ) ) {
         while ( i < components.length ) {
            components[i].setFont ( font ) ;
            i++ ;
         }
      }

      invalidate() ;
   }





   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will add a Vector of components
    *  to the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void add ( Vector vec ) {
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i         = 0 ;
         int length    = vec.size() ;

         while ( i < length ) {
            add ( (Component)vec.elementAt(i) ) ;
            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener
    *  to the list of listeners
    *  
    *
    *  @param listener The new action listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      if ( actionListenerVec == null )
         actionListenerVec = new Vector ( 1, 1 ) ;
      actionListenerVec.addElement ( listener ) ;
   }



   /**
     *  This method will remove an action listener
     **/
   public void removeActionListener ( ActionListener listener ) {
      if ( actionListenerVec != null ) {
         actionListenerVec.removeElement (listener) ;
         actionListenerVec.trimToSize() ;
         if ( actionListenerVec.size() == 0 )
            actionListenerVec = null ;
      }

   }


   /*------------------------------------------------------------*/
   /* fireActionEvent
    */
   /**
    *    This method will fire an action event
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireActionEvent ( ActionEvent e ) {
      if ( ( actionListenerVec != null ) && ( actionListenerVec.size() > 0 ) ) {
         int i      = 0 ;
         int length = actionListenerVec.size() ;

         while ( i < length ) {
            ((ActionListener)actionListenerVec.elementAt(i)).actionPerformed ( e ) ;
            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* contains
    */
   /**
    *    This method will determine if the component
    *  passed is listed in this NavList
    *  
    *
    *  @param comp The component to look for
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public boolean contains ( Component comp ) {
      Component[] components = getComponents() ;
      int         i          = 0 ;
      boolean     found      = false ;

      if ( ( components != null ) && ( components.length > 0 ) ) {
         while ( ( !found ) && ( i < components.length ) ) {
            if ( components[i] == comp )
               found = true ;
            i++ ;
         }
      }

      return( found ) ;
   }





   /*------------------------------------------------------------*/
   /* scrollSmooth
    */
   /**
    *    This method wil scroll smoothly either
    *  a section up or down. The 'y' position passed
    *  should be the point where the top of scrolling should start,
    *  the 'length' variable should be the length of the scroll. If
    *  the value of length is positive, a scroll down will happen from 'y',
    *  if the value of length is negative, the bottom of the scroll
    *  area is scrolled up.
    *
    *  @param y The top of the scroll 
    *  @param length The length in pixels of the scroll
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void scrollSmooth ( int y, int length ) {
      Graphics  g        = getGraphics() ;  
      Dimension size     = getSize() ;
      int       i        = 0 ;
      int       waitTime = 0 ;
      boolean   wait     = true ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Scroll Down                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( length > 0 ) && ( g != null ) ) {
         for ( i = 0; i < length; i++ ) {}
            g.copyArea ( 0, y+i,                            
                         size.width, size.height-(y+i)-1,
                         0, 1 ) ;
         //   try {
         //      if ( wait ) {
         //         if ( (5-waitTime) > 3 )
         //            Thread.currentThread().sleep(5-waitTime) ;
         //         else
         //            Thread.currentThread().sleep(3) ;
         //         wait = false ;
         //      } else {
         //         Thread.currentThread().sleep(5-waitTime) ;
         //         waitTime++ ;
         //         wait = true ;
         //      }

         //   } catch ( Exception e ) {
         //   }
         //}
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Scroll Up                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( length < 0 ) && ( g != null ) ) {
         length=java.lang.Math.abs ( length ) ;
         for ( i = 0; i < length; i++ ) {}
            g.copyArea ( 0, y-i,                            
                         size.width, size.height-(y-i)-1,
                         0, -1 ) ;
         //   try {
         //      if ( wait ) {
         //         if ( (5-waitTime) > 3 )
         //            Thread.currentThread().sleep(5-waitTime) ;
         //         else
         //            Thread.currentThread().sleep(3) ;
         //         wait = false ;
         //      } else {
         //         Thread.currentThread().sleep(5-waitTime) ;
         //         waitTime++ ;
         //         wait = true ;
         //      }

         //   } catch ( Exception e ) {
         //   }
         //}
      }

      revalidate() ;

   }

} /* NavList */
