package com.ibm.nzna.projects.qit.product.productImport ;              
/*-----------------------------------------------------------*/
/*   ProductImport                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductImport                       */
/*      This is based on the HTMLProductImport class         */
/*      created by David Lentz 4/4/2000.                     */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import java.io.*;
import java.awt.event.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.projects.common.quest.LocaleRec ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.qit.app.*;
import com.ibm.nzna.projects.qit.product.*;
import com.ibm.nzna.projects.qit.gui.*;


/**
 * This batch imports a delimited file that contains product information.
 * The file should contain information on a list of products to insert,
 * update or insert/update.  The following parameters are taken by this
 * Batch:<br><br><pre>
 *
 * Parameter         Description                           Example
 * ----------------- ------------------------------------- --------
 * /INSERT           Only tries to insert each product     /INSERT
 * /UPDATE           Only tries to update each product     /UPDATE
 * /INSERT_UPDATE  * Tries to update, then insert          /INSERT_UPDATE
 * /OPTIONSONLY      Only tries to process Options         /OPTIONS
 * /SYSTEMSONLY    * Only tries to process Systems         /SYSTEMS
 * /MONITORSONLY     Only tries to process Monitors        /MONITORS
 * /FILE:            The Delimted file to load and process /FILE:MYCSVFILE.TXT 
 *
 * * = Default Value
 * </pre><br><br><br>
 *
 *
 * This batch expects the delimited file to contain rows where each row in the
 * file represents a product. The rows should be setup are specified below for
 * Systems:<br><pre>
 *
 * <b>Field	      Description</b>
 * -------------- -------------------------
 *  1             Brand Name
 *  2             Family Name
 *  3             Product Number
 *  4             Machine Type
 *  5             Model
 *  6             Quest Description
 *  7             Locale
 *  8             Project Name
 *  9             Comments
 * 10             Warranty
 *	11            Offering
 *	12            Customer Name
 *
 * Columns 1 - 10 are <b>required</b>
 * </pre><br><br>
 *
 * This batch expects the delimited file to contain a table. Each row in the
 * file represents a product. The rows should be setup are specified below for
 * Options:<br><pre>
 *
 * <b>Field      Description</b>
 * -------------- -------------------------
 *  1             Brand Name
 *  2             Family Name
 *  3             Part Number
 *  4             OBI Family Type
 *  5             Quest Description
 *  6             Locale
 *  7             Project Name
 *  8             Comments
 *  9             Warranty
 *	10             Offering
 *	11            Customer Name
 *
 * Columns 1 - 9 are <b>required</b>
 * </pre><br><br>
 * 
 *  @version 1.0, 7/27/2001                                  
 *  @author  Asha Rucker                                     
 */




/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductImport                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductImport extends Batch implements ProductImportConst, ProductConst, ActionListener, QuestProcess {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Static Members                                          */
    /*                                                           */
    /*-----------------------------------------------------------*/



    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
     * This is the userid of the person currently logged in. 
     */
    private String          userId               = UserSystem.getUserId().trim() ;


    /**
     * This is the complete name of the file to be imported.  It should also contain the location of the file. 
     */
    private String         importFilename       = null ;

    /**
     * This is used to write errors/warnings to the log file. 
     */
    private BatchLogSystem  logSystem            = null ;

    /**
     * This is used to denote the type of action of this import.  The possible action types are INSERTONLY, UPDATEONLY or INSERTUPDATE. 
     */
    private int         productActionType         = INSERTUPDATE ;

    /**
     * This denotes what type of products are to be imported.  There are three types:  Systems, Monitors and Options. Only one type will be imported per file.   The three possible values for this field are :  SYSTEMONLY, MONITORSONLY or OPTIONONLY.
     */
    private int         productImportType    = SYSTEMSONLY ;

    /**
     * This is used to track which phase of the batch is currently processing. 
     */
    private int             step                 = 0 ;

    /**
     * This is used to determine whether or not the import batch should continue. 
     */
    private boolean         continueImport       = true ;

    /**
     * This is used to determine whether or not the file has been completely parsed. 
     */
    private boolean         parsingComplete      = false; 

    /**
     * This is used to hold each product String.
     */
    private Vector          productStrVec        = null ;


    /**
     * This is used to hold each product.
     */
    private Vector          productVec           = null;


    /**
     * This is used to set the total number of steps the entire batch takes to complete it's processing. 
     */
    private int             MAX_NUMBER_OF_STEPS  = 12 ;

    /**
     * This is the Commit button
     */
    private ActionButton    pb_COMMIT = null;

    /**
     * This is the Rerun button
     */
    private ActionButton    pb_RERUN  = null;

    /**
     * This is used to hold the machine
     */
    private String machineStr = null;

    /**
     * This is used to hold the model
     */
    private String modelStr = null;

    /**
     * This is used to hold the subcategory
     */
    private String subCategoryStr = null;

    /**
     * This is used to hold the part number
     */
    private String partNumStr           = null;

    /**
     * This is used to hold the warranty
     */
    private String warrantyStr           = null;









    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This constructor will initialize the BatchLogSystem and the batch.  It
     *  will then kick off and run the batch.
     *
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ProductImport ( String[] args ) {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First, always create our log system                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        logSystem = new BatchLogSystem ( "ProductImport", this ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Read the params                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/

        continueImport = readParams ( args ) ; 

    }


    /*------------------------------------------------------------*/
    /* startBatch
     */
    /**
     *    This method will start the batch
     *
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void startBatch () {

        continueImport = true ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create a QuestProcess for this                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        ProcessManagerSystem.addProcess(this);

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Run through the file and enable the Rerun button        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( continueImport ) {

            pb_RERUN.setEnabled(true);
            continueImport = parseCSVFile() ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Run through the Vector                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( continueImport )
            continueImport = processProductStrVec(productStrVec) ;

    }

    /*------------------------------------------------------------*/
    /* stopBatch
     */
    /**
     *    This method will remove the Quest Process
     *
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/


    public void stopBatch() {      
        /* Quest Process is now done */
        ProcessManagerSystem.removeProcess(this);
        /* Batch is finished */
        logProgress ( "Shutting Down", MAX_NUMBER_OF_STEPS, step++ ) ;
        logProgress ( "Batch Completed", MAX_NUMBER_OF_STEPS, step++ ) ;

    }

    /*------------------------------------------------------------*/
    /* actionPerformed
     */
    /**
     *  This method will be called when an 
     *  action we are listening for has been fired
     *  
     *
     *  @param e The event
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void actionPerformed ( ActionEvent e ) {
        // add code later

        if ( e.getSource() == pb_COMMIT ) {
            if ( continueImport && parsingComplete) {
                pb_COMMIT.setEnabled(false);
                pb_RERUN.setEnabled(false);
                commitImport();
            }
        }

        if (e.getSource() == pb_RERUN ) {
            if ( !parsingComplete )
                rerun();
        }


    }

    /*------------------------------------------------------------*/
    /* readParams
     */
    /**
     *    This method will traverse through the  array of strings. 
     *    Each string represents  a parameter.  Each parameter is converted 
     *    to uppercase.  This method will return TRUE if it successfully read 
     *    all parameters; otherwise, it will return FALSE.
     *
     *  @param args The parameters 
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean readParams ( String[] args ) {                          
        boolean rc = false ;

        try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have any parameters, read all                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( args != null ) && ( args.length > 0 ) ) {
                int length = args.length ;
                int i      = 0 ;


                logSystem.beginLogSection ( "Reading Parameters. Number of params passed:"+args.length ) ;
                while ( i < length ) {
                    processParam ( args[i].toUpperCase(), args[i] ) ;
                    i++ ;
                }

                rc = true ;
                logSystem.endLogSection () ;

            }
        } catch ( Exception e ) {
            logSevereError ( e ) ;
        }

        return( rc ) ;

    }



    /*------------------------------------------------------------*/
    /* processParam
     */
    /**
     *   Description:                                            
     *      This method will process a String value into an expected parameter.                     *                                               
     *                                                           
     *   Parameters:                                             
     *     param - the param in uppercase
     *     mixedCaseParam- the actual param                  
     *                                                           
     *   Returns:                                                
     *      Nothing                                              
     *-----------------------------------------------------------*/

    public void processParam ( String param, String mixedCaseParam ) throws Exception {                            

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Write out certain params. Hide others                   */
        /*                                                           */
        /*-----------------------------------------------------------*/

        logProgress( "Parameter:"+mixedCaseParam, MAX_NUMBER_OF_STEPS, step++ );        

        if ( param.startsWith ( "/FILE:" ) )
            importFilename = mixedCaseParam.substring ( 6, param.length() ) ;

        else if ( param.startsWith ( "/INSERTUPDATE" ) )
            productActionType = INSERTUPDATE ;

        else if ( param.startsWith ( "/INSERT" ) )
            productActionType = INSERTONLY ;

        else if ( param.startsWith ( "/UPDATE" ) )
            productActionType = UPDATEONLY ;

        else if ( param.startsWith ( "/SYSTEMS" ) )
            productImportType = SYSTEMSONLY ;

        else if ( param.startsWith ( "/MONITORS" ) )
            productImportType = MONITORSONLY ;

        else if ( param.startsWith ( "/OPTIONS" ) )
            productImportType = OPTIONSONLY ;

    }


    /*------------------------------------------------------------*/
    /* parseCSVFile
     */
    /**
     *  This method will attempt to read the CSV File,
     *  and pull out all of the rows. Each row will be added to the productStrVec. 
     *  A row is defined as ending with a Carriage Return. The row added to the vector will 
     *  be a string. This string contains the End of Line character.
     *
     *  @param None
     *
     *  @return TRUE if the parsing was good.
     */
    private boolean parseCSVFile () {
        boolean rc = false ;

        logProgress ("Parsing import file", MAX_NUMBER_OF_STEPS, step++);
        try {
            BufferedReader csvFile = new BufferedReader (new FileReader(importFilename));
            String rowRead         = null;                   /* last row read from csv file     */
            int rowCount           = 0;                      /* number of rows read             */

            /* Initialize string vector */
            if (productStrVec != null)
                productStrVec.removeAllElements();
            else
                productStrVec = new Vector(10,10);



            /* Read in rows from file */
            while ((rowRead = csvFile.readLine()) != null) {
                productStrVec.addElement(rowRead);
                rowCount++;
            }

            if (productStrVec.size() == 0) {
                logError("The import file contains no data." );
                stopBatch();
            } else {
                rc = true;
                MAX_NUMBER_OF_STEPS+=rowCount;

            }

            csvFile.close();

        } catch ( Exception e ) {
            logSevereError ( e ) ;
        }


        return( rc ) ;
    }

    /*------------------------------------------------------------*/
    /* processProductStrVec
     */
    /**
     *   This method converts the Vector of product strings into a vector 
     *   of ProductDraftObjects.
     *
     *  @param productStrVec  The vector containing all the product string 
     *                        rows read from file
     *
     *  @return TRUE if the parsing was good.
     *//*---------------------------------------------------------*/
    private boolean processProductStrVec (Vector productStrVec) {
        boolean rc = false ;
        int length = productStrVec.size();
        int i      = 0;
        ProductDraft prodDraft = null;

        try {
            /* initialize product draft vector */
            if ( (productVec != null) )
                productVec.removeAllElements();
            else
                productVec = new Vector(10, 10);

            while ( (i < length) && continueImport) {
                prodDraft = createProductDraft( productStrVec.elementAt(i).toString(), i  );

                /* if product Draft was created, add it to vector and update the progress log */
                if (prodDraft != null) {
                    productVec.addElement(prodDraft);
                    if (productImportType == OPTIONSONLY)
                        logProgress("Parsed PartNumber:    "+prodDraft.getPartNumber().toString(), MAX_NUMBER_OF_STEPS, step++);
                    else {

                        logProgress("Parsed ProductNumber: "+prodDraft.getProdNum(), MAX_NUMBER_OF_STEPS, step++);
                    }

                    logProgress(    "       Description:   "+prodDraft.getDescription(), MAX_NUMBER_OF_STEPS, step++);   
                }
                i++;

            }  /* while */

            if ((productVec.size() == 0)|| !continueImport) {

                if (productVec.size() ==0)
                    logError("There are no products to import.");

                stopBatch();
            }

            else {
                rc = true;
                logProgress("Parsing Complete. Please press the Commit button to commit these Products.", MAX_NUMBER_OF_STEPS, step++);
                parsingComplete = true;
                /* Quest Process is now done */
                ProcessManagerSystem.removeProcess(this);
                pb_COMMIT.setEnabled(true);
            }

        } catch (Exception e) {
            logSevereError(e);
        }

        return( rc ) ;
    }


    /*------------------------------------------------------------*/
    /* createProductDraft
     */
    /**
     * This method converts a delimited String into a ProductDraft object.
     * @param productStr The delimited string.
     *
     * @return ProductDraft The newly created ProductDraft object.
     */

    private ProductDraft createProductDraft ( String productStr, int rowIndex) {
        int column          = 0;
        ProductDraft retRec = null;
        boolean cont        = true;     //  keep parsing string if no error was encountered
        rowIndex ++;                    //increment rowIndex by one so the line numbers will match up
        try {
            retRec = new ProductDraft();
            StringTokenizer st = new StringTokenizer( productStr, ",");


            // probably need to add more conditions later
            while ((st.hasMoreTokens()) && cont && continueImport) {
                /* for options, cols 4 -9 are the same as cols 5 -10 for systems.
                   need to increment by one when we reach col 4 for options so they
                   will map correctly to systems cols */
                if ( (column == 4) && (productImportType == OPTIONSONLY) ) {
                    column++;
                }
                cont = processProductString(retRec, st.nextToken().trim(), rowIndex, column);
                column++;

            }

            if (!continueImport) {
                retRec = null;
            } else {

                /* check to see if required fields were read if an error was not encountered */
                if (cont && (((column < 8) && (productImportType != OPTIONSONLY)) ||
                             ((column < 7) && (productImportType == OPTIONSONLY)) )) {
                    logError("The product is missing one or more required fields on line: "+rowIndex);
                    cont = false;
                }

                if (!cont) {
                    retRec = null;
                }
            }

        } catch (Exception e) {
            logSevereError(e);
            retRec = null;
        }

        return( retRec ) ;
    }


    /*------------------------------------------------------------*/
    /* processProductString
     */
    /**
     *  This method will process one string.
     *  @param retRec   The productDraft record 
     *  @param dataStr  The data string
     *  @param rowIndex The current row from file
     *  @param column   The current column in the row
     *
     *  @return TRUE if no error was encountered
     */
    private boolean processProductString ( ProductDraft retRec, String dataStr, int rowIndex, int column) {
        boolean rc = false ; 


        switch (column) {
        
        case INDEX_BRAND :
            if (dataStr.length() != 0) {
                TypeCategoryRec brand = TypeCategory.getCategory(dataStr, TypeCategory.LEVEL_BRAND ) ;

                if ( brand == null )
                    logError ( "The brand on row "+rowIndex+" was not found. Brand:"+dataStr ) ;
                else {
                    retRec.setBrand(brand);
                    rc = true ;  
                }
            } else {
                logError("The Brand on row "+rowIndex+" is missing and is required.");
            }

            break ;

        case INDEX_FAMILY :
            if (dataStr.length() != 0) {
                TypeCategoryRec family = TypeCategory.getCategory(dataStr, TypeCategory.LEVEL_FAMILY ) ;

                if ( family == null )
                    logError ( "The Family on row "+rowIndex+" was not found. Family:"+dataStr ) ;
                else {
                    retRec.setFamily(family);
                    rc = true ;
                }
            } else
                logError("The Family on row "+rowIndex+" is missing and is required.");

            break ;

        case INDEX_PRODUCTNUMBER :
            /* Since product number is set automatically for systems/monitors we only need
            to read in the product number for options  */

            if (productImportType==OPTIONSONLY) {
                // this is really a part number for options
                if (dataStr.length() != 0) {
                    partNumStr = new String(dataStr);
                    rc = true;
                }

                else
                    logError("The Part Number on row "+rowIndex+" is missing and is required.");
            } else {
                // we can ignore for systems/monitor so return true
                rc = true;
            }

            break;

        case INDEX_MACHINE :
            if (productImportType==OPTIONSONLY) {
                // this is really the option's subcategory

                if (dataStr.length() != 0 ) {
                    subCategoryStr = new String(dataStr);
                    rc = true;
                } else
                    logError("The OBI Family Type on row "+rowIndex+" is missing and is required.");


            } else {
                /* this is a system or monitor  */

                if (dataStr.length() !=0 ) {
                    machineStr = new String(dataStr);
                    rc = true;  

                } else
                    logError ( "The Machine on row "+rowIndex+" is missing and is required." ) ;

            }

            break ;

        case INDEX_MODEL :

            if ( dataStr.length() != 0 ) {
                modelStr = new String(dataStr);
                rc = true;
            }

            else
                logError ( "The Model on row "+rowIndex+" is missing and is required." ) ;

            break ;

        case INDEX_DESCRIPT :
            if ( dataStr.length() != 0) {
                if (dataStr.length() > 254 )
                    logError("The Description on row "+rowIndex+" exceeds the 254 character limit.");

                else {
                    retRec.setDescription (dataStr) ;
                    rc = true ;
                }
            }

            else
                logError ( "The Description on row "+rowIndex+" is missing and is required." ) ;
            break ;

        case INDEX_LOCALE:   
            if (dataStr.length() != 0) {
                rc = processLocaleString(retRec, dataStr, rowIndex);

            } else
                logError("The Locale on row "+rowIndex+" is missing and is required.");

            break;

        case INDEX_PROJECT:
            TypeCategoryRec project = null;

            if (dataStr.length() != 0) {
                project = TypeCategory.getCategoryFromDescription(dataStr, TypeCategory.LEVEL_PROJECT, retRec.getFamily().getInd() );
                if (project != null)
                    retRec.setProject( project ) ;
                else {

                    project = TypeCategory.createTypeCategory(retRec.getFamily().getInd(),
                                                              "Default", 
                                                              TypeCategory.LEVEL_PROJECT,
                                                              0,"N","QUEST");


                    retRec.setProject(project);
                }

            } else {

                project = TypeCategory.getCategoryFromDescription("Default", TypeCategory.LEVEL_PROJECT, retRec.getFamily().getInd() );


                if (project != null) {

                    retRec.setProject( project ) ;

                } else {
                    project = TypeCategory.createTypeCategory(retRec.getFamily().getInd(),
                                                              "Default", 
                                                              TypeCategory.LEVEL_PROJECT,
                                                              0,"N","QUEST");

                    retRec.setProject(project);

                }


            } 
            if (productImportType != OPTIONSONLY) {
                TypeCategoryRec machine = null;
                TypeCategoryRec model   = null;


                machine = TypeCategory.getCategoryFromDescription(machineStr, TypeCategory.LEVEL_MACHINE, retRec.getProject().getInd());
                if (machine == null) {
                    retRec.setMachine(TypeCategory.createTypeCategory(project.getInd(),
                                                                      machineStr, 
                                                                      TypeCategory.LEVEL_MACHINE,
                                                                      0,"N","QUEST"));


                } else
                    retRec.setMachine(machine);

                model =  TypeCategory.getCategoryFromDescription(modelStr, TypeCategory.LEVEL_MODEL, retRec.getMachine().getInd());
                if (model == null) {
                    retRec.setModel(TypeCategory.lazyCreateTypeCategory(retRec.getMachine().getInd(),
                                                                        modelStr, 
                                                                        TypeCategory.LEVEL_MODEL,
                                                                        0,"N","QUEST"));


                } else
                    retRec.setModel(model);
                rc = true;
                // project
            } else {
                // this is an option 
                TypeCategoryRec subCategory =  TypeCategory.getCategoryFromDescription(subCategoryStr, TypeCategory.LEVEL_SUBCATEGORY,retRec.getProject().getInd() );
                TypeCategoryRec partNum     = null;
                if ( subCategory == null ) {


                    retRec.setSubCategory((TypeCategory.createTypeCategory(project.getInd(),
                                                                           subCategoryStr,
                                                                           TypeCategory.LEVEL_SUBCATEGORY,                             
                                                                           0, "N", "QUEST")));

                } else
                    retRec.setSubCategory(subCategory);

                partNum = TypeCategory.getCategoryFromDescription(partNumStr, TypeCategory.LEVEL_PARTNUMBER, retRec.getSubCategory().getInd());
                if ( partNum == null ) {

                    retRec.setPartNumber((TypeCategory.lazyCreateTypeCategory(retRec.getSubCategory().getInd(),
                                                                              partNumStr,
                                                                              TypeCategory.LEVEL_PARTNUMBER,
                                                                              0, "N", "QUEST")));

                } else {
                    retRec.setPartNumber(partNum);
                }

                rc = true;
            }
            break;

        case INDEX_COMMENT:
            if (dataStr.length() != 0) {
                if (dataStr.length() > 1024 )
                    logError("The Comment on row "+rowIndex+" exceeds the 1024 character limit.");
                else {
                    retRec.setComment(dataStr);
                    rc = true;
                }
            }

            else
                logError("The Comment on row "+rowIndex+" is missing and is required.");

            break;


        case INDEX_WARRANTY:
            if (dataStr.length() != 0 ) {
                int warranty =  TypeList.getInstance().indFromString ( dataStr, TypeList.TYPE_WARRANTY );
                //retRec.setWarranty ( TypeList.getInstance().indFromString ( dataStr, TypeList.TYPE_WARRANTY ) ) ;
                if ( warranty != 0) {
                    retRec.setWarranty(warranty);
                    rc = true;
                } else
                    logError("The Warranty on row "+rowIndex+" was not found. Warranty:"+dataStr);
            }
            //else
            //    logError("The Warranty on row "+rowIndex+" is missing and is required.");
            // if no warranty was given let it get the default value
            else
                rc = true;
            break;

        case INDEX_OFFERING:
            if (dataStr.length() != 0) {
                retRec.setOffering(TypeList.getInstance().indFromString(dataStr, TypeList.TYPE_OFFERING));
            }
            // offering isn't required so no error if blank
            rc = true;
            break;

        case INDEX_CUSTOMERNAME:
            if (dataStr.length() != 0) {
                retRec.setCustomer(TypeList.getInstance().indFromString(dataStr, TypeList.TYPE_CUSTOMER));           
            }
            // customer name isn't required so no error if blank
            rc = true;
            break;

        default: 
            rc = true;
            break;

        }

        return( rc ) ;
    }


    /*------------------------------------------------------------*/
    /* processLocaleString
     */
    /**
     *    This method will take a string representation of the countries/dates,
     *  make sure that the countries are valid, and make sure the date is valid
     *  and properly formatted.
     *
     *  @param retRec    The dataStr that is parsed
     *  @param datsStr   The string containing the countdate
     *  @param rowIndex  The current row from import file
     *
     *  @return TRUE if no errors, FALSE If an error was occured
     */
    private boolean processLocaleString ( ProductDraft retRec, String dataStr, int rowIndex ) {

        boolean rc        = false;
        String newDataStr = dataStr+";";
        int countryCode   = 0;         
        LocaleRec locRec  = new LocaleRec();
        ProductDate pDate = null;
        int numDates      = 0;       // number of dates processed
        boolean error     = false;   // true if an error is encountered
        boolean done      = false;   // true when locale/date processing is complete
        int startIndex    = 0;
        int stopIndex     = 0;
        String dateStr    = null;
        int dateCheck     = 0;


        while (!done && !error && continueImport) {
            // read in country Code and set locale
            stopIndex = newDataStr.indexOf(":", startIndex);
            if (stopIndex == -1 ) { // need to think about if there is more stuff in string but no date treat as junk?
                if (startIndex >= newDataStr.length()) {
                    retRec.setLocale(locRec);
                    done = true;
                    rc = true;
                } else {
                    logError("The Country on row "+rowIndex+" is missing and is required.");  
                    error = true;
                }    
            } else {

                countryCode = TypeList.getInstance().indFromString(newDataStr.substring(startIndex, stopIndex).trim() , TypeList.TYPE_COUNTRYCODE);

                if (countryCode != 0 ) {

                    locRec.addCountry(countryCode);

                } else {
                    logError("The Country on row "+rowIndex+" was not found. Country: "+newDataStr.substring(startIndex, stopIndex).trim() );
                    error = true;
                }
            }

            while ( !done && !error && (numDates <10) && continueImport) {
                startIndex = stopIndex + 1;
                stopIndex = newDataStr.indexOf(";", startIndex);
                if (startIndex == -1) {
                    logError("One or more dates in row "+rowIndex+" is missing and is required.");
                    error = true;
                } else {
                    switch (numDates) {
                    case 0:
                        pDate = new ProductDate(ANNOUNCED_DATE);
                        break;

                    case 1:
                        pDate = new ProductDate(PRE_ANNOUNCED_DATE);
                        break;
                    case 2:
                        pDate = new ProductDate(GA_DATE);
                        break;

                    case 3:
                        pDate = new ProductDate(WITHDRAWN_DATE);
                        break;

                    case 4:
                        pDate = new ProductDate(EO_SERVICE_DATE);
                        break;

                    case 5:
                        pDate = new ProductDate(EO_PROD_DATE);
                        break;

                    case 6:
                        pDate = new ProductDate(EO_MAN_DATE);
                        break;

                    case 7:
                        pDate = new ProductDate(EO_MARKET_DATE);
                        break;

                    case 8:
                        pDate = new ProductDate(FIRST_ORDER_DATE);
                        break;

                    case 9:
                        pDate = new ProductDate(SHIP_SUPPORT_DATE);
                        break;

                    default:
                        break;
                    }

                    dateStr = CDate.format4DigitYear(newDataStr.substring(startIndex, stopIndex), CDate.DATE_DATE);
                    dateCheck = CDate.checkDate(dateStr, CDate.DATE_DATE);
                    if (dateCheck != 0) {
                        logError("A date in row "+rowIndex+" is formatted incorrectly.  The date was: "+dateStr);
                        error = true;
                    }

                    else {
                        pDate.setActualDate(dateStr);
                        pDate.setCountryCodeInd(countryCode);
                        retRec.addDate(pDate);

                    }

                    numDates++;
                }
            }     /*end while(!done && !error && numDates...) */
            startIndex = stopIndex + 1;
            numDates = 0;


        } /* end while(!done && !error) */
        if (!continueImport)
            rc = false;

        return( rc ) ;
    }


    /*------------------------------------------------------------*/
    /* commitImport
     */
    /**
     *    This method will commit the import. This is
     *  where each imported product will be inserted or updated in the database.
     *
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void commitImport () {
        boolean success    = true;
        boolean error = false;


        /* Quest Process is now done */
        //ProcessManagerSystem.removeProcess(this);
        if ( ( productVec != null ) && ( productVec.size() > 0 ) ) {
            int              i           = 0 ;
            int              length      = productVec.size() ;
            ProductDraft     prodDraft     = null ;

            while ( ( i < length ) && ( continueImport ) ) {
                prodDraft = (ProductDraft) productVec.elementAt(i) ;
                switch (checkForNewProduct(prodDraft)) {
                case NEW_PRODUCT:
                    success = importProductDraft(prodDraft);
                    break;

                case EXISTING_PRODUCT:
                    success = importCreateProductDraft( prodDraft );
                    break;

                case DRAFT_EXISTS:
                    if (productImportType == OPTIONSONLY )
                        logError("The Product Draft you attempted to import already exists for Part Number:"+prodDraft.getPartNumber().toString());
                    else
                        logError("The Product Draft you attempted to import already exists for Product Number:"+prodDraft.getProdNum());
                    error = true;
                    break;

                default:
                    error = true;
                    break;
                }      

                i++ ;
                if (success && !error) {

                    if (productImportType == OPTIONSONLY)
                        logProgress ( "Committed Product:"+prodDraft.getPartNumber().toString(), MAX_NUMBER_OF_STEPS, step++ ) ;
                    else
                        logProgress ( "Committed Product:"+prodDraft.getProdNum(), MAX_NUMBER_OF_STEPS, step++ ) ;
                } else {
                    if (productImportType == OPTIONSONLY)
                        logProgress ( "Unable to commit Product:"+prodDraft.getPartNumber().toString(), MAX_NUMBER_OF_STEPS, step++ ) ;
                    else
                        logProgress ( "Unable to commit Product:"+prodDraft.getProdNum(), MAX_NUMBER_OF_STEPS, step++ ) ;


                }

            }
            /* Batch has completed */      
            logProgress ( "Shutting Down", MAX_NUMBER_OF_STEPS, step++ ) ;
            logProgress ( "Batch Completed", MAX_NUMBER_OF_STEPS, step++ ) ;

        }
    }



    /*------------------------------------------------------------*/
    /* checkForNewProduct
     */
    /**
     *    This method checks to see if the product we want to 
     *    import is new.   
     *   
     *
     *  @param product - The productDraft to check
     *
     *  @return True - if product is new, False otherwise
     *//*---------------------------------------------------------*/

    public int checkForNewProduct(ProductDraft product) {
        int rc         = EXISTING_PRODUCT;  
        int productInd = 0;
        try {

            if (productImportType == OPTIONSONLY) {

                TypeCategoryRec partNumRec         = TypeCategory.getCategoryFromDescription ( product.getPartNumber().toString(), product.getSubCategory().getInd() ) ;
                TypeCategoryRec draftProjectRec    = TypeCategory.getDraftCategoryFromDescription(product.getProject().toString(), TypeCategory.LEVEL_PROJECT, product.getFamily().getInd());
                TypeCategoryRec draftSubCatRec     = null;
                TypeCategoryRec draftPartNumRec    = null;

                if (draftProjectRec != null)
                    draftSubCatRec = TypeCategory.getDraftCategoryFromDescription(product.getSubCategory().toString(), TypeCategory.LEVEL_SUBCATEGORY, draftProjectRec.getInd());

                if (draftSubCatRec != null)
                    draftPartNumRec = TypeCategory.getDraftCategoryFromDescription ( product.getPartNumber().toString(), TypeCategory.LEVEL_PARTNUMBER, draftSubCatRec.getInd() ) ;


                // check to see if this is a new product with an existing draft 
                if (draftPartNumRec != null) {
                    productInd = ProductSQL.getDraftProductIndFromModel ( draftPartNumRec );
                    if (productInd != 0) {
                        if ( draftExists(productInd)  )
                            rc = DRAFT_EXISTS;
                    }
                }


                if ( rc != DRAFT_EXISTS ) {

                    if (partNumRec != null) {
                        productInd = ProductSQL.getProductIndFromModel ( partNumRec ) ;

                        if ( productInd == 0 )
                            rc = NEW_PRODUCT;
                    } else
                        rc = NEW_PRODUCT;
                }


            }

            else {// this is a system/monitor

                TypeCategoryRec machineRec      = TypeCategory.getCategoryFromDescription ( product.getMachine().toString(), product.getProject().getInd() ) ;
                TypeCategoryRec draftProjectRec = TypeCategory.getDraftCategoryFromDescription (product.getProject().toString(), TypeCategory.LEVEL_PROJECT, product.getFamily().getInd());


                TypeCategoryRec draftMachineRec = null;
                if (draftProjectRec != null)
                    draftMachineRec = TypeCategory.getDraftCategoryFromDescription (product.getMachine().toString(), TypeCategory.LEVEL_MACHINE, draftProjectRec.getInd());


                if ( draftMachineRec != null ) {
                    TypeCategoryRec draftModelRec   = TypeCategory.getDraftCategoryFromDescription ( product.getModel().toString(), TypeCategory.LEVEL_MODEL, draftMachineRec.getInd() ) ;

                    if ( draftModelRec != null ) {

                        productInd = ProductSQL.getDraftProductIndFromModel ( draftModelRec ) ;
                        if ( productInd != 0 ) {
                            if (draftExists(productInd))
                                rc = DRAFT_EXISTS;
                        }
                    }
                } // draftMachineRec is null

                if ( rc != DRAFT_EXISTS ) {
                    if ( machineRec != null ) {

                        TypeCategoryRec modelRec   = TypeCategory.getCategoryFromDescription ( product.getModel().toString(),  machineRec.getInd() ) ;

                        if ( modelRec != null ) {
                            productInd = ProductSQL.getProductIndFromModel ( modelRec ) ;

                            if ( productInd == 0 )
                                rc = NEW_PRODUCT;

                        } else //modelRec is null 
                            rc = NEW_PRODUCT;
                    } else // machineRec is null 
                        rc = NEW_PRODUCT;

                }
            }
        } catch (Exception e) {
            logError(e.toString());
            rc = -1;
        }
        return(rc);
    }


    /*------------------------------------------------------------*/
    /* importProductDraft
     */
    /**
     *  This method will save the product draft to the database.
     *  It will insert a new product.
     * 
     *
     *  @param prodDraft The product Draft
     *
     *  @return TRUE if no errors, FALSE If an error was occured
     *//*---------------------------------------------------------*/

    private boolean importProductDraft( ProductDraft prodDraft )  {
        // this means we are importing a brand spanking new product

        boolean  rc   = false;
        int errorCode = 0;


        if (productActionType == UPDATEONLY) {
            if (productImportType == OPTIONSONLY) {
                logError("The Product Draft could not be updated. A Product with Part Number:"+prodDraft.getPartNumber().toString()+" does not exist.");
            } else {
                logError("The Product Draft could not be updated. A Product with Product Number:"+prodDraft.getProdNum()+" does not exist.");
            }
        }

        else {
            //set additional fields
            prodDraft.setProductType(productImportType);
            prodDraft.setLastTouchedBy(userId);
            prodDraft.setWorkRequired(TypeList.WORKREQ_CREATE);
            prodDraft.setCreatedBy(userId);
            prodDraft.setInput("Author");
            prodDraft.setSource(userId);

            errorCode = prodDraft.writeToDatabase();

            if (errorCode== 0 ) {
                rc = true;
            } else
                logError("The Product Draft could not be imported due to SQL Error: "+errorCode);

        }

        return(rc);
    }

    /*------------------------------------------------------------*/
    /* importCreateProductDraft
     */
    /**
     *  This method will save the product draft to the database.
     *  It will update an existing product
     * 
     *
     *  @param prodDraft The product Draft
     *
     *  @return TRUE if no errors, FALSE If an error was occured
     *//*---------------------------------------------------------*/

    private boolean importCreateProductDraft( ProductDraft prodDraft)  {
        boolean rc               = false;
        CreateProductDraft cpd   = null;
        ProductDraft pd          = null;
        String sourceStr         = null;
        String pubFlag           = null;
        int errorCode            = 0;
        String partNum           = null;
        String machine           = null;
        String model             = null;



        try {
            //if this method was called you be updating an existing product
            if (productActionType == INSERTONLY) {
                if (productImportType == OPTIONSONLY) {
                    logError("The Product Draft could not be inserted.  A Product already exists for Part Number:"+prodDraft.getPartNumber().toString());
                } else {
                    logError("The Product Draft could not be inserted.  A Product already exists for Product Number:"+prodDraft.getProdNum());
                }
            } else {


                if ( productImportType == OPTIONSONLY) {

                    cpd = ProductDraft.createProductDraft(prodDraft.getPartNumber().toString());

                }

                else {

                    cpd = ProductDraft.createProductDraft(prodDraft.getFamily(),prodDraft.getMachine().toString(), prodDraft.getModel().toString());
                }


                switch (cpd.getReturnCode()) {
                
                case CreateProductDraft.DRAFT_CREATED: 

                    pd = cpd.getProductDraft();
                    pubFlag =  pd.getPublish();

                    // check to see if this is an existing active product  
                    // this case shouldn't happen but check just in case
                    if ( productActionType == UPDATEONLY && pubFlag.equals("N") ) {
                        if (productImportType == OPTIONSONLY) {
                            logError("The Product Draft could not be updated. A Product with Part Number:"+pd.getPartNumber().toString()+" does not exist.");
                        } else {
                            logError("The Product Draft could not be updated. A Product with Product Number:"+pd.getProdNum()+" does not exist.");
                        }
                    }

                    else {
                        // set imported fields
                        pd.setCustomer(prodDraft.getCustomer());
                        pd.setDates(prodDraft.getDates());
                        pd.setDescription(prodDraft.getDescription());
                        pd.setLocale(prodDraft.getLocale());    
                        pd.setOffering(prodDraft.getOffering());
                        pd.setWarranty(prodDraft.getWarranty());

                        //set additional fields
                        pd.setProductType(productImportType);
                        pd.setLastTouchedBy(userId);
                        pd.setInput("Author");

                        if ( (productActionType == UPDATEONLY) || ((productActionType == INSERTUPDATE) && (pubFlag.equals("Y")))) {

                            pd.setWorkRequired(TypeList.WORKREQ_EDIT);
                            sourceStr = pd.getSource();

                            if (sourceStr.startsWith("OPIC/M") )
                                pd.setSource( new String(sourceStr+"-"+userId));
                            else
                                pd.setSource(userId);   

                            errorCode = pd.writeToDatabase();

                            if (errorCode== 0 ) {
                                rc = true;
                            } else
                                logError("The Product Draft could not be imported due to SQL Error: "+errorCode);
                        }
                    }

                    break;


                case CreateProductDraft.DRAFT_ALREADY_EXISTS:
                    // reject
                    if (productImportType == OPTIONSONLY )
                        logError("The Product Draft you attempted to import already exists for Part Number:"+prodDraft.getPartNumber().toString());
                    else
                        logError("The Product Draft you attempted to import already exists for Product Number:"+prodDraft.getProdNum());


                    break;

                case CreateProductDraft.DRAFT_MISFIT:

                    // reject
                    if (productImportType == OPTIONSONLY)
                        logError("The Product Draft you attempted to import for Part Number:"+pd.getPartNumber().toString()+" already exists and is a misfit");
                    else
                        logError("The Product Draft you attempted to import for Product Number:"+pd.getProdNum()+" already exists and is a misfit");
                    break;

                case CreateProductDraft.DRAFT_RECYCLED:
                    // reject 
                    if (productImportType == OPTIONSONLY)
                        logError("The Product Draft you attempted to import already exists and is in the recycling bin for Part Number:"+pd.getPartNumber().toString());
                    else
                        logError("The Product Draft you attempted to import already exists and is in the recycling bin for Product Number:"+pd.getProdNum());
                    break;

                case CreateProductDraft.DRAFT_SQL_ERROR:
                    // reject
                    if (productImportType == OPTIONSONLY)
                        logError("The Product Draft could not be created due to SQL Error: "+ cpd.getSqlCode()+" for Part Number:"+prodDraft.getPartNumber().toString()) ;
                    else
                        logError("The Product Draft could not be created due to SQL Error: "+ cpd.getSqlCode()+" for Product Number:"+prodDraft.getProdNum());
                    break;

                default:
                    if (productImportType == OPTIONSONLY)
                        logError("The Product Draft could not be created due to an error for Part Number:"+prodDraft.getPartNumber().toString()) ;
                    else
                        logError("The Product Draft could not be created due to an error for Product Number:"+prodDraft.getProdNum());
                    break;

                }
            }
        } catch (Exception e) {
            logSevereError(e);
            logError(e.toString());

        }


        return(rc);
    }

    /*------------------------------------------------------------*/
    /* getProcessName
     */
    /**
     *    This method will return the process name 
     *   
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/


    public String getProcessName () {
        return( Str.getStr(STR_PRODUCT_IMPORT) ) ;
    }

    /*------------------------------------------------------------*/
    /* stopProcess
     */
    /**
     *    This method will stop a process that is running
     *   
     *
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/

    public void stopProcess () {
        continueImport = false ;
        logProgress("**Product Import was stopped by user request.**", MAX_NUMBER_OF_STEPS, step++);
    }

    /*------------------------------------------------------------*/
    /* setButtons
     */
    /**
     *    This method sets the ActionButtons that are used with
     *    ProductImport and add the batch as listeners for the buttons.
     *   
     *
     *  @param ActionButton Commit
     *  @param ActionButton Rerun
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/


    public void setButtons(ActionButton commit, ActionButton rerun) {

        pb_COMMIT = commit;
        pb_RERUN  = rerun;

        pb_COMMIT.addActionListener(this);
        pb_RERUN.addActionListener(this);

    }

    /*------------------------------------------------------------*/
    /* rerun
     */
    /**
     *    This method restarts the batch.
     *   
     *
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/

    public void rerun() {
        stopBatch();
        MAX_NUMBER_OF_STEPS = 12;
        logProgress ( "Rerunning the Batch", MAX_NUMBER_OF_STEPS, step++ ) ;
        startBatch();

    }

    /*------------------------------------------------------------*/
    /* draftExists
     */
    /**
     *    This method checks for an existing open draft
     *   
     *
     *  @param productind - the productind to look up
     *
     *  @return boolean - true if open draft exists, false otherwise
     *//*---------------------------------------------------------*/


    public boolean draftExists ( int productInd ) {
        ProductDraft productDraft = null ;
        SQLMethod    sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "ProductImport.findDraft ( "+productInd+" )", 5 ) ;
        boolean      rc           = false;
        try {
            Statement stmt = sqlMethod.createStatement() ;
            ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND FROM PRODRAFT.PRODUCT WHERE PRODUCTIND = "+productInd+" AND STOPDATE IS NULL FOR FETCH ONLY" ) ;

            if ( rs.next() )
                rc = true;

            rs.close() ;

        } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
        }

        sqlMethod.close() ;

        return( rc ) ;
    }






} /* ProductImport */

