package com.ibm.nzna.projects.qit.storedProc.readUsers ;   
/*-----------------------------------------------------------*/
/*   ReadUsers                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ReadUsers                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.Clob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read available users in the database. Since the user
 * list may get too long for a stored procedure to read in, the first parameter is a comma
 * separated list of user ids. Each user id in the list will be read in and returned. If
 * the string 'UNIQUE_USERS' is passed as the first paramter, a Vector of Strings will be returned. Each
 * string is a unique user id.<br><br>
 *
 * A good way to never have a failure when reading in this large list of user ids is to first
 * call the ReadUsers stored procedure with 'UNIQUE_USERS' to get a list of User Ids. Then, form a list
 * of 200 user ids each separated by commas, and call the stored procedure again and again until
 * all of the users are read in. Passing more than 200 may be dangerous.<br><br>
 * 
 * Main Execution Point: readUsers<br><br>
 *
 * <pre>
 * Param Number   Descript<br>
 * -------------- -------------------------------------------<br>
 * 1              A Comma separated list of User ids, or 'UNIQUE_USER' to return a Vector of Strings (each string is a user id)<br>
 * 2              If a comma separated list was passed, a Vector of UserRecs, if a blank string was passed, a Vector of strings<br>
 * 3              StoredProcRec ( Allocated and returned by ReadUsers )<br>
 *</pre>
 * 
 *
 * @version 1.0, 2/23/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ReadUsers                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class ReadUsers extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec         = new StoredProcRec () ;
   private Vector        returnVec         = null ;
   private Connection    con               = null ;
   private String        userList          = "" ;





   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readUsers ( Clob userListClob, Blob retReadUsersRec, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         this.userList = userList ;
         con = getConnection() ;                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the string from the clob                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getStringFromClob ( userListClob ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read everything in                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( userList.equals ( "UNIQUE_USERS" ) )
            readUniqueUserIds() ;
         else
            returnVec = readUsers ( con, userList ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our ReadUsersRec                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retReadUsersRec = Lob.newBlob() ;

         StoredProcUtil.setObjectInBlob ( retReadUsersRec, returnVec );
         set    ( 2, retReadUsersRec ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
         StoredProcUtil.setObjectInBlob ( retReadUsersRec, new Vector ( 1 ) );
         set ( 2, retReadUsersRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;
      set ( 3, storedProcRetRec ) ;

   }






   /*------------------------------------------------------------*/
   /* readUsers
    */
   /**
    *    This method will read in all of the users
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readUsers ( Connection con, String userList ) throws Exception {
      Statement stmt     = con.createStatement() ;
      String    sql      = null ;
      ResultSet rs       = null ;
      UserRec   userRec  = null ;
      Vector    retVec   = new Vector ( 1, 10 ) ;

      sql = "SELECT A.USERID,      A.NAME,          A.MANAGER,   A.MANAGERUSERID, "+
            "       A.PHONENUM,    A.BACKUPUSERID,  A.EMAILADDR, A.PASSWORD,      "+
            "       B.LANGUAGEIND, C.EMAILADDR "+
            "FROM TIGRIS.USERID A, "+
            "     TIGRIS.USERLANGUAGE B, "+
            "     TIGRIS.USERID C "+
            "WHERE A.USERID in ( "+userList+" ) AND "+
            "      A.USERID = B.USERID AND "+
            "      C.USERID = A.MANAGERUSERID "+
            "ORDER BY A.USERID FOR FETCH ONLY" ;

      rs = stmt.executeQuery ( sql ) ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop through and read all values                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {

         userRec = new UserRec ( rs.getString(1).trim(),
                                 rs.getString(2).trim() ) ;

         userRec.setIsManager     ( rs.getString(3).trim().equals ( "Y" ) ) ;
         userRec.setManagerUserId ( rs.getString(4).trim() ) ;
         userRec.setPhone         ( rs.getString(5).trim() ) ;
         userRec.setBackUp        ( rs.getString(6).trim() ) ;
         userRec.setEMail         ( rs.getString(7).trim() ) ;
         userRec.setPassword      ( userRec.decryptPassword ( rs.getString(8).trim() ) ) ;
         userRec.setLanguage      ( rs.getInt(9) ) ;
         userRec.setManagerEMail  ( rs.getString(9).trim() ) ;                                             

         retVec.addElement ( userRec ) ;
      } 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now read the rest of the information for each user  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( retVec != null ) && ( retVec.size() > 0 ) ) {
         int     i       = 0 ;
         int     size    = retVec.size() ;

         while ( i < size ) {
            userRec = (UserRec) retVec.elementAt(i) ;
            readUserCountry ( con, userRec ) ;
            readUserTeams   ( con, userRec ) ;
            readUserBrands  ( con, userRec ) ;
            i++ ;
         }
      }

      rs.close() ;
      stmt.close() ;

      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* readUserCountry
    */
   /**
    *    This method will read in all of the user's
    *  countries.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void readUserCountry ( Connection con, UserRec userRec ) throws Exception {
      Statement stmt = con.createStatement() ;
      String    sql  = null ;
      ResultSet rs   = null ;

      sql = "SELECT COUNTRYCODEIND FROM TIGRIS.USERIDCOUNTRY WHERE USERID = '"+userRec.getUserId()+"'" ;
      rs  = stmt.executeQuery ( sql ) ;

      while ( rs.next() ) {
         userRec.addCountry ( rs.getInt(1) ) ;
      }

      rs.close() ;
      stmt.close() ;
   }

   /*------------------------------------------------------------*/
   /* readUserTeams
    */
   /**
    *    This method will read in all of the user's
    *  teams.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void readUserTeams ( Connection con, UserRec userRec ) throws Exception {
      Statement stmt = con.createStatement() ;
      String    sql  = null ;
      ResultSet rs   = null ;

      sql = "SELECT USERGROUP FROM TIGRIS.USERIDUSERGROUP WHERE USERID = '"+userRec.getUserId()+"'" ;
      rs  = stmt.executeQuery ( sql ) ;

      while ( rs.next() )
         userRec.addTeam ( rs.getInt(1) ) ;

      rs.close() ;
      stmt.close() ;
   }


   /*------------------------------------------------------------*/
   /* readUserBrands
    */
   /**
    *    This method will read in all of the user's
    *  brands.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void readUserBrands ( Connection con, UserRec userRec ) throws Exception {
      Statement stmt = con.createStatement() ;
      String    sql  = null ;
      ResultSet rs   = null ;

      sql = "SELECT BRANDIND, TYPEBRANDMGR FROM TIGRIS.USERIDBRAND WHERE USERID = '"+userRec.getUserId()+"'" ;
      rs  = stmt.executeQuery ( sql ) ;

      while ( rs.next() )
         userRec.addBrandMgrRec ( new BrandMgrRec ( rs.getString(2).trim(), rs.getInt(1) ) ) ;

      rs.close() ;
      stmt.close() ;
   }




   /*------------------------------------------------------------*/
   /* readUniqueUserIds
    */
   /**
    *    THis method will read only the unique user ids
    *  and stores each User Id as a string in the
    *  return vector
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readUniqueUserIds ( ) throws Exception {
      Statement stmt = con.createStatement() ;
      ResultSet rs   = stmt.executeQuery ( "SELECT USERID FROM TIGRIS.USERID FOR FETCH ONLY" ) ;

      returnVec = new Vector ( 500, 1 ) ;

      while ( rs.next() )
         returnVec.addElement ( rs.getString(1).trim() ) ;

      rs.close() ;
      stmt.close() ;
   }




   /*------------------------------------------------------------*/
   /* getStringFromClob
    */
   /**
    *    This method will attempt to pull a character
    *  String from the Clob passed in
    *  
    *
    *  @param lob The Clob to pull from
    *  @param None
    *  @param None
    *
    *  @return TRUE = Pull was good, FALSE = Pull was bad
    *//*---------------------------------------------------------*/
   private boolean getStringFromClob ( Clob lob ) throws Exception {
      Reader        reader    = lob.getReader() ;
      int           max       = 256 ;
      char[]        clobData  = new char[max];           
      int           read      = 0 ;
      StringBuffer  retStr    = new StringBuffer() ; 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Copy over the bytes                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( read > -1  ) && ( reader != null ) ) {
         read = reader.read ( clobData ) ;
         if ( read > -1 )
            retStr.append ( new String ( clobData ).substring(0,read) ) ;
      }

      if ( retStr.length() > 0 ) {
         userList = retStr.toString() ;
         return( true ) ;
      }

      return( false ) ;
   }





} /* ReadUsers */
