package com.ibm.nzna.projects.scrape ;
/*-----------------------------------------------------------*/
/*   Scrape                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Scrape                              */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This batch program will export all document information into
 * a text file. The text file will have one document per line and
 * the values on each line will be separated by a PIPE symbol. The
 * values on each line MUST follow this layout:<br><br>
 *
 * DocID | Title | Classification | Geography | Country | Brand | Webable<br><br>
 *
 * In the case of Multiple Geographies, Countries or Brands, the values
 * for the respective data will be separated by commas<br><br>
 *
 * <b>Please Note!</b> Per Customer request, this batch modifies the title
 * of the document so that the pipe delimiter is removed!<br><br>
 *
 * The basic flow of this batch is as follows:<br>
 * - Initialize LogSystem<br>
 * - Connect to Quest<br>
 * - Read all documents where PUBLISH='Y'<br>
 * - Create a SmallDoc object to hold the document's Title, geo, docid and brand.<br>
 * - Add each SmallDoc object to the docs Hashtable<br>
 * - Add the countries to each object in the docs Hashtable.<br>
 * - Write out each SmallDoc object in the docs Hashtable.<br>
 * - Disconnect<br>
 * - Exit<br>
 * <br><br><Br>
 *
 * This batch program has a property file called 'scrape.property'. Currently the only
 * options that are configurable in this property file are:<br><br>
 *
 * <pre>
 * Property        Description
 * ----------      -------------------------------------------------
 * outFile         The full path and filename of the file that is created and written to
 * publishFTPSite  The I/P Address or Host Name of the FTP Site to Publish to
 * publishFTPDir   The directory that the published file will be written to
 * outputLog       The full path and filename of the log file.
 *
 * </pre><br><br><br>
 * <b>Please Note</b> The reason that the countries are read separatly from all
 * other information is because not all documents in the Quest database have an
 * entry in the TIGRIS.DOCCOUNTRY table. Therefore, they must be read separatly.
 * 
 *
 * @version 1.0, 9/26/1998 
 * @author  David Lentz
 */
import java.io.* ;
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.ftp.* ;



/*-----------------------------------------------------------*/
/*                                                           */
/*   Scrape                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class Scrape extends Object {

    /**
      *  This is the 'scrape.property' property file 
      **/
    static Properties properties = new Properties() ;

    /**
      *  This is the file that is generated by this batch
      **/
    static String outFile = "scrappy.txt" ;

    /**
      * This is the file that is logged
      */
    static String logFile = "scrape.out" ;

    /**
      * This is the FTP Site that the outFile is published to
      */
    static String ftpSite = "" ;

    /**
      * This is the directory on the FTP Site that the outFile is placed in
      */
    static String ftpDir = "" ;

    /**
      *  This is the Hashtable which holds all of the SmallDoc objects
      * that are read from the database. The Keys in this hash will be
      * a String representation of the DOCIND.
      **/
    static Hashtable docs = new Hashtable() ;

    /**
      * This is the main method. The arguments that must
      * be specified are the database name, database server and
      * database port number
      */
    public static void main ( String args[] ) {
        boolean useDefaultProperties = false ;
        long    startTime            = System.currentTimeMillis() ;
        long    stopTime             = 0 ;
        long    runTime              = 0 ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   If we don't have all of the args, exit                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( ( args == null ) || ( args.length < 3 ) ) {
            System.out.println ( "Usage: java com.ibm.nzna.projects.scrape.Scrape <databaseName> <databaseServer> <portNumber>" ) ;
            System.exit(-1) ;
        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Read our properties                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        try {
            Properties prop = StandardPropertyLoader.load ( "scrape.properties" )  ;

            if ( prop != null ) {
                outFile = prop.getProperty ( "outFile",        "scrappy.txt" ) ;
                ftpSite = prop.getProperty ( "publishFTPSite", "f3.pc.ibm.com" ) ;
                ftpDir  = prop.getProperty ( "publishFTPDir",  "/home/ftp/pub/pccbbs/incoming/scrappy" ) ;
                logFile = prop.getProperty ( "outputLog",      "/home/quest/logs/scrape.out" ) ;

            } else
                useDefaultProperties = true ;

        } catch ( Exception e ) {
            LogSystem.log ( 1, "Cannot load scrape.property" ) ;
            LogSystem.log ( 1, e , false ) ;
            useDefaultProperties = true ;
        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Initialize a Log System                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        new LogSystem ( 1, logFile ) ;          

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   If we are to use the default properties, do so now      */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( useDefaultProperties ) {
            LogSystem.log ( 1, "Loading Default Properties" ) ;
            outFile = "scrappy.txt" ;
            ftpSite = "ftp.pc.ibm.com" ;
            ftpDir  = "/pub/pccbbs/etsosurvey/scrappy" ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Connect to the database                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( connectToDatabase ( args[0], args[1], args[2] ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read our information                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( readInfo() ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Read the country information                            */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( readCountryInfo() ) {

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Write the file                                          */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( writeFile() ) {

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   FTP the file to the FTP Site                            */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        publishFile() ;

                    }
                }


            }




            DatabaseSystem.shutdown() ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Write out the run information                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        LogSystem.beginLogSection ( "Batch Run Stats" ) ;
        stopTime = System.currentTimeMillis() ;
        runTime = ((stopTime-startTime)) ;
        LogSystem.log ( 1, "Total Run Time:"+runTime+"(ms)" ) ;

        System.exit(0) ;
    }


    /**
      * This method will connect to the database. It will return TRUE if a connection
      * was established
      */
    private static boolean connectToDatabase ( String databaseName, String server, String port ) {
        boolean rc = false ;

        try {
            new DatabaseSystem ( true, LogSystem.getInstance(), false ) ;

            rc = DatabaseSystem.createConnection ( DatabaseSystem.QUEST, 
                                                   databaseName,
                                                   server,
                                                   port,
                                                   "quest",
                                                   "k1ngarthur" ) ;
            LogSystem.log ( 1, "Connected to database "+databaseName+" on Server "+server ) ;
        } catch ( Exception e ) {
            rc = false ;
            LogSystem.log ( 1, e, false ) ;
        }

        return( rc ) ;
    }



    /**
      * This method will read in all valid documents and creates a SmallDoc object for each.
      * It then adds that SmallDoc object to the docs hashtable.
      *
      * @returns TRUE if all information was read in
      */
    private static boolean readInfo () {
        SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readInfo", 5 ) ;
        SmallDoc  readRec   = null ;
        boolean   rc        = false ;

        try {
            Statement        stmt      = sqlMethod.createStatement() ;
            ResultSet        rs        = null ;

            rs = stmt.executeQuery ( "SELECT DOC.DOCIND, DOC.LNDOCID,      DOC.TITLE,            TYPECLASS.DESCRIPT, "+
                                     "       TYPEGEO.DESCRIPT, TYPEBRAND.DESCRIPT, DOC.WEBABLE "+
                                     "FROM TIGRIS.DOCUMENTS       DOC,"+
                                     "     TIGRIS.DOCBRAND        DOCBRAND, "+
                                     "     TIGRIS.DOCGEO          DOCGEO, "+
                                     "     TIGRIS.TYPEGEOGRAPHY   TYPEGEO,"+
                                     "     TIGRIS.TYPEBRAND       TYPEBRAND, "+
                                     "     TIGRIS.TYPEDOCCLASS    TYPECLASS, "+
                                     "     TIGRIS.BRANDS          BRANDS "+
                                     "WHERE DOCBRAND.DOCIND            = DOC.DOCIND AND "+
                                     "      BRANDS.BRANDGROUPIND       = DOCBRAND.BRANDGROUPIND AND "+
                                     "      TYPEBRAND.BRANDIND         = BRANDS.BRANDIND AND "+
                                     "      DOCGEO.DOCIND              = DOC.DOCIND AND "+
                                     "      TYPEGEO.GEOIND             = DOCGEO.GEOIND AND "+
                                     "      TYPECLASS.DOCCLASSIND      = DOC.DOCCLASSIND AND "+
                                     "      DOC.LNDOCID IS NOT NULL AND "+
                                     "      DOC.PUBLISH                = 'Y' FOR FETCH ONLY" ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Walk the list                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( rs.next() ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   First, find the object in our hash                      */
                /*                                                           */
                /*-----------------------------------------------------------*/
                readRec = (SmallDoc) docs.get ( ""+rs.getInt(1) ) ;                             

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   If we found it, use the current record                  */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( readRec != null ) {
                    if ( readRec.geography.indexOf ( rs.getString(5).trim() ) == -1 )
                        readRec.geography+=";"+rs.getString(5).trim() ;
                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Otherwise, create a new one                             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else {

                    readRec = new SmallDoc() ;
                    readRec.lnDocId        = rs.getString(2).trim() ;
                    readRec.title          = rs.getString(3).trim() ;
                    readRec.classification = rs.getString(4).trim() ;
                    readRec.geography      = rs.getString(5).trim() ;
                    readRec.brand          = rs.getString(6).trim() ;
                    readRec.webable        = rs.getString(7).trim() ;
                    docs.put ( ""+rs.getInt(1), readRec ) ;
                }

            }

            rs.close() ;
            rc = true ;

        } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
        }

        sqlMethod.close() ;

        return( rc ) ;
    }


    /**
      *  Read in all of the countries for each document in the docs hashtable
      **/
    private static boolean readCountryInfo () {
        SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readCountryInfo", 5 ) ;
        SmallDoc  readRec   = null ;
        boolean   rc        = false ;

        try {
            Statement        stmt      = sqlMethod.createStatement() ;
            ResultSet        rs        = null ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute our query                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            LogSystem.log ( 1,"Entering Phase 2" );
            rs = stmt.executeQuery ( "SELECT DOCCOUNTRY.DOCIND,  TYPECOUNTRYCODE.DESCRIPT, TYPEGEO.DESCRIPT,  "+
                                     "       DOCUMENTS.TITLE,    DOCUMENTS.LNDOCID,        TYPEDOCCLASS.DESCRIPT, "+
                                     "       TYPEBRAND.DESCRIPT, DOCUMENTS.WEBABLE "+
                                     "FROM TIGRIS.DOCCOUNTRY      DOCCOUNTRY, "+
                                     "     TIGRIS.TYPECOUNTRYCODE TYPECOUNTRYCODE, "+
                                     "     TIGRIS.TYPEGEOGRAPHY   TYPEGEO, "+
                                     "     TIGRIS.GEOCOUNTRY      GEOCOUNTRY, "+
                                     "     TIGRIS.DOCUMENTS       DOCUMENTS, "+
                                     "     TIGRIS.TYPEDOCCLASS    TYPEDOCCLASS, "+
                                     "     TIGRIS.TYPEBRAND       TYPEBRAND, "+
                                     "     TIGRIS.DOCBRAND        DOCBRAND, "+
                                     "     TIGRIS.BRANDS          BRANDS "+
                                     "WHERE DOCCOUNTRY.COUNTRYCODEIND      = GEOCOUNTRY.COUNTRYCODEIND AND "+
                                     "      TYPEGEO.GEOIND                 = GEOCOUNTRY.GEOIND AND "+
                                     "      TYPECOUNTRYCODE.COUNTRYCODEIND = DOCCOUNTRY.COUNTRYCODEIND AND "+
                                     "      DOCUMENTS.DOCIND               = DOCCOUNTRY.DOCIND AND "+
                                     "      DOCUMENTS.PUBLISH              = 'Y' AND "+
                                     "      TYPEDOCCLASS.DOCCLASSIND       = DOCUMENTS.DOCCLASSIND AND "+
                                     "      DOCBRAND.DOCIND                = DOCCOUNTRY.DOCIND AND "+
                                     "      BRANDS.BRANDGROUPIND           = DOCBRAND.BRANDGROUPIND AND "+
                                     "      DOCUMENTS.LNDOCID              IS NOT NULL AND "+
                                     "      TYPEBRAND.BRANDIND             = BRANDS.BRANDIND FOR FETCH ONLY" ) ;               

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through the results                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( rs.next() ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   First, find the object in our hash                      */
                /*                                                           */
                /*-----------------------------------------------------------*/
                readRec = (SmallDoc) docs.get ( ""+rs.getInt(1) ) ;  

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   If we found it, use the current record                  */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( readRec != null ) {
                    if ( readRec.geography.indexOf ( rs.getString(3).trim() ) == -1 )
                        readRec.geography+=";"+rs.getString(3).trim() ;
                    if ( readRec.country.indexOf ( rs.getString(2).trim() ) == -1 )
                        readRec.country+=";"+rs.getString(2).trim() ;
                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Otherwise, create a new one                             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else {
                    readRec = new SmallDoc() ;
                    readRec.country        = rs.getString(2).trim() ;
                    readRec.geography      = rs.getString(3).trim() ;
                    readRec.title          = rs.getString(4).trim() ;
                    readRec.lnDocId        = rs.getString(5).trim() ;
                    readRec.classification = rs.getString(6).trim() ;
                    readRec.brand          = rs.getString(7).trim() ;
                    readRec.webable        = rs.getString(8).trim() ;
                    docs.put ( ""+rs.getInt(1), readRec ) ;
                }


            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Cleanup                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs.close() ;
            rc = true ;

        } catch ( Exception e ) {
            LogSystem.log ( 1, "Exception while processing Countries." ) ;
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
        }

        sqlMethod.close() ;

        return( rc ) ;

    } /* readCountryInfo  */


    /**
      *  This method will write out each document to the file. The file that
      * is written to is specified in the FileOutputStream that is passed
      *
      * @param out The FileOuputStream to write to
      **/
    private static boolean writeFile ( ) {
        File             file      = new File ( outFile ) ;
        FileOutputStream out       = null ;
        SmallDoc         readRec   = null ;                   
        Enumeration      enum      = null ;
        boolean          rc        = true ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create the file                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        try {
            out = new FileOutputStream ( file ) ;
        } catch ( Exception outEx ) {
            LogSystem.log ( 1, "Cannot open out file "+outFile+" for writing" ) ;
            LogSystem.log ( 1, outEx, false ) ;
            rc = false ;
        }



        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ok, now write the file                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( out != null ) {
            for ( enum = docs.elements() ; enum.hasMoreElements() ; ) {
                try {
                    readRec = (SmallDoc)enum.nextElement() ;

                    /* Write out the previous one now!  */
                    if ( readRec != null ) {

                        /* Take out the delimiter  */
                        if ( readRec.title.indexOf ( "|" )>=0 )
                            readRec.title = Text.replaceAllStrInStr ( readRec.title, "|", " " ) ;
                        if ( readRec.title.indexOf ( "\n" )>=0 )
                            readRec.title = Text.replaceAllStrInStr ( readRec.title, "\n", " " ) ;

                        out.write ( new String ( readRec.lnDocId+"|"+
                                                 readRec.title+"|"+
                                                 readRec.classification+"|"+
                                                 readRec.geography+"|"+
                                                 readRec.country+"|"+
                                                 readRec.brand+"|"+
                                                 readRec.webable+"\n" ).getBytes() ) ;
                    }
                } catch ( Exception e ) {
                    LogSystem.log ( 1, "Exception caught writing file" ) ;
                    LogSystem.log ( 1, e, false ) ;
                    rc = false ;
                }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Cleanup                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            try {
                out.close() ;
            } catch ( Exception closeX ) {
                LogSystem.log ( 1, "Cannot close "+outFile ) ;
                LogSystem.log ( 1, closeX, false ) ;
                rc = false ;
            }
        } /* if  */

        return( rc ) ;

    } /* writeFile  */



    /**
      * This method will attempt to publish the outFile. It will connect
      * VIA FTP to the ftpSite, change directories to the ftpDir and then
      * put the outFile there.
      *
      * @return TRUE = Everything Published Fine
      */
    private static boolean publishFile () {
        boolean     rc  = false ;

        LogSystem.beginLogSection ( "Publishing "+outFile ) ;
        try {
            FTPManager ftp     = new FTPManager ( ftpSite, "ftpbot", "java0ne" ) ;
            FTPFile    ftpFile = new FTPFile ( new File ( outFile ) ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Tell the FTP Manager to use this logging                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setLogging ( LogSystem.getInstance() ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the FTP File                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftpFile.setRemoteFilename  ( Text.stripPath ( outFile ) ) ;
            ftpFile.setRemoteDirectory ( ftpDir ) ;

            if ( ftp.open() ) {
                ftp.copyToFtp ( ftpFile ) ;
            } else
                LogSystem.log ( 1, "Could not FTP outFile:"+outFile ) ;

            LogSystem.log ( 1, "Disconnecting from FTP Site" );
            ftp.close() ;

        } catch ( Exception e ) {
            LogSystem.log ( 1, "Exception while publishing File" ) ;
            LogSystem.log ( 1, e, false ) ;
        } /* catch  */

        LogSystem.endLogSection() ;

        return( rc ) ;
    }
}



class SmallDoc extends Object {

    public String lnDocId        = null ;
    public String title          = null ;
    public String classification = null ;
    public String geography      = "" ;
    public String country        = "" ;
    public String brand          = "" ;
    public String webable        = "" ;
}
