package com.ibm.nzna.shared.batch ;
/*-----------------------------------------------------------*/
/*   Batch                                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Batch                               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.io.* ;
/**
 * This is the superclass of all batches written under Dept. NZNA. 
 * The following methods MUST be used to log information:<br><br>
 *
 * logProgress - Simply indicates that progress has been made.<br>
 * logError - Indicates a non-severe error has happened.<br>
 * logSevereError - Inidicates a severe, batch stopping error happened.<br>
 *
 * @version 1.0, 3/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Batch                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class Batch extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector batchListenerVec = null ;




   /*------------------------------------------------------------*/
   /* removeBatchListener
    */
   /**
    *    This method will remove a batch listener
    *  from this batch
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeBatchListener ( BatchListener listener ) {
      if ( batchListenerVec != null )
         batchListenerVec.removeElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* addBatchListener
    */
   /**
    *    This method should be subclassed to add
    *  a BatchListener to this batch.
    *  
    *
    *  @param batchListener The batch listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addBatchListener ( BatchListener listener ) {
      if ( batchListenerVec == null )
         batchListenerVec = new Vector ( 1, 1 ) ;

      batchListenerVec.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* logProgress
    */
   /**
    *    This method should be called to notify
    *  progress.
    *  
    *
    *  @param outputStr The output for the progress
    *  @param totalOperations The total number of operations needed to complete the batch
    *  @param operation The current operation that is being done
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void logProgress ( String outputStr, int totalOperations, int operation ) {
      if ( batchListenerVec != null ) {
         int i    = 0 ;
         int size = batchListenerVec.size() ;

         while ( i < size ) {
            ((BatchListener)batchListenerVec.elementAt(i)).logProgress ( outputStr, totalOperations, operation ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* logError
    */
   /**
    *    This method should be called to notify
    *  of non severe errors.
    *  
    *
    *  @param errorStr The error for the progress
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void logError ( String errorStr ) {
      if ( batchListenerVec != null ) {
         int i    = 0 ;
         int size = batchListenerVec.size() ;

         while ( i < size ) {
            ((BatchListener)batchListenerVec.elementAt(i)).logError ( errorStr ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* logSevereError
    */
   /**
    *    This method should be called to notify
    *  of severe errors.
    *  
    *
    *  @param errorStr The error for the progress
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void logSevereError ( String errorStr ) {
      if ( batchListenerVec != null ) {
         int i    = 0 ;
         int size = batchListenerVec.size() ;

         while ( i < size ) {
            ((BatchListener)batchListenerVec.elementAt(i)).logSevereError ( errorStr) ;
            i++ ;
         }
      }
   }



   /*------------------------------------------------------------*/
   /* logSevereError
    */
   /**
    *    This method should be called to notify
    *  of severe errors.
    *  
    *
    *  @param e The exception to log
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void logSevereError ( Exception e ) {
      StringWriter    sw      = new StringWriter () ;
      PrintWriter     pw      = new PrintWriter ( sw ) ;

      logSevereError ( sw.toString() ) ;
   }





   /*------------------------------------------------------------*/
   /* startBatch
    */
   /**
    *    This method should start the batch
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void startBatch () {
   }




   /*------------------------------------------------------------*/
   /* stopBatch
    */
   /**
    *    This method should stop the batch at the
    *  next point that it can be reliably stopped
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopBatch () {
   }


   /**
     * This method will return the list of listeners that this batch
     * notifies
     *
     * @return Vector
     **/
   public Vector getBatchListeners () {
      return ( batchListenerVec ) ;
   }
} /* Batch */
