package com.ibm.nzna.shared.batch ;       
/*-----------------------------------------------------------*/
/*   BatchLogSystem                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BatchLogSystem                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The BatchLogSystem is a form of LogSystem that is tuned specifically for 
 * batches of any kind. Each batch written under Dept. NZNA should utilize this
 * class for log file outputs.<br><br>
 *
 * The BatchLogSystem creates two log files. These include an error log and an output
 * log. The error log will be named batchName_error.current_timestamp. The output log
 * will be named batchName_output.current_timestamp.<br><br>
 *
 * There is a very usefull method in this class called 'deleteOldLogs' which will
 * delete all logs over a certain amount of days old.<br>
 *
 * @version 1.0, 3/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BatchLogSystem                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class BatchLogSystem extends LogSystem implements  BatchListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String  batchName            = null ;
   private static String  currentDirectory     = null ;
   private static String  errorFile            = null ;
   private static String  progressFile         = null ;
   private static CDate   date                 = null ;
   private static String  runTimestamp         = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the Log
    *  System. It will also delete
    *  the old log files.
    *
    *  @param batchName The name of the batch that is running.
    *  @param batch The Batch to listen to and log for
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BatchLogSystem ( String batchName, Batch batch ) {
      Properties p       = System.getProperties () ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our local vars                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      date = new CDate () ;
      currentDirectory = p.getProperty ( "user.dir" ) ;
      runTimestamp     = date.getDate ( CDate.DATE_STAMP ) ;
      errorFile        = batchName+"_error."+runTimestamp ;
      progressFile     = batchName+"_out."+runTimestamp ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the new log files                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      Log.clearLogFile ( errorFile ) ;
      Log.clearLogFile ( progressFile ) ;                                    
      setLogFile ( 1, progressFile ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, write out the standard Header information           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      log ( "*---------------------------------------------------------------------------------------*" ) ;
      log ( "*" ) ;
      log ( "* Batch Name:         "+batchName ) ;
      log ( "* Start Time:         "+runTimestamp ) ;
      log ( "* Home Dir:           "+currentDirectory ) ;
      log ( "* JDK Level:          "+p.getProperty ( "java.fullversion" ) ) ;
      log ( "* Platform:           "+p.getProperty ( "os.name" ) ) ;
      log ( "* Platform Version:   "+p.getProperty ( "os.version" ) ) ;
      log ( "* User Home:          "+p.getProperty ( "user.home" ) ) ;
      log ( "*" ) ;
      log ( "*---------------------------------------------------------------------------------------*" ) ;
      log ( "" ) ;
      log ( "" ) ;

      Log.log ( errorFile,  "*---------------------------------------------------------------------------------------*\n" ) ;
      Log.log ( errorFile,  "*\n" ) ;
      Log.log ( errorFile,  "* Batch Name:         "+batchName+"\n" ) ;
      Log.log ( errorFile,  "* Start Time:         "+runTimestamp+"\n" ) ;
      Log.log ( errorFile,  "* Home Dir:           "+currentDirectory+"\n" ) ;
      Log.log ( errorFile,  "* JDK Level:          "+p.getProperty ( "java.fullversion" )+"\n" ) ;
      Log.log ( errorFile,  "* Platform:           "+p.getProperty ( "os.name" )+"\n" ) ;
      Log.log ( errorFile,  "* Platform Version:   "+p.getProperty ( "os.version" )+"\n" ) ;
      Log.log ( errorFile,  "* User Home:          "+p.getProperty ( "user.home" )+"\n" ) ;
      Log.log ( errorFile,  "*\n" ) ;
      Log.log ( errorFile,  "*---------------------------------------------------------------------------------------*\n" ) ;
      Log.log ( errorFile,  "\n" ) ;
      Log.log ( errorFile,  "\n" ) ;

      batch.addBatchListener ( this ) ;
      this.batchName = batchName ;
   }




   /*------------------------------------------------------------*/
   /* deleteOldLogs
    */
   /**
    *    This method will delete all logs files older
    *  than 'param days' days from today.
    *  
    *
    *  @param days The number of days to keep the logs, everything older will be deleted
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void deleteOldLogs ( int days ) {
      File       dir          = new File ( currentDirectory ) ;
      String[]   dirList      = dir.list() ;
      Vector     logFiles     = new Vector ( dirList.length ) ;
      int        i            = 0 ;
      int        size         = 0 ;     
      String     filename     = null ;
      String     logTimestamp = null ;

      beginLogSection ( "Scanning all log files" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, filter out only the logs files for this batch       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      for ( int k = 0; k < dirList.length; k++ ) {
         if ( ( dirList[k].startsWith ( batchName+"_error" ) ) ||
              ( dirList[k].startsWith ( batchName+"_out" ) ) )
            logFiles.addElement ( dirList[k] ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now go through each file and compare the dates      */                                         
      /*                                                           */
      /*-----------------------------------------------------------*/
      size = logFiles.size() ;

      while ( i < size ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Extract the timestamp from the filename                 */
         /* We just take everything from the first period on.         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         filename = (String) logFiles.elementAt(i) ;
         try {
            logTimestamp = filename.substring ( filename.length()-26 ) ;
            log ( "Found Old Header with Date:"+logTimestamp ) ;
         } catch ( Exception e ) {
            log ( "Could not parse date from file: "+filename ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, Now compare the date to today's date                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( CDate.differenceDate ( runTimestamp, logTimestamp, CDate.DATE_STAMP ) > 2 ) {
            File file = new File ( filename ) ;

            log ( "Deleting old log file:"+filename ) ;
            file.delete() ;
         } else
            log ( "Preserving old log file:"+filename ) ;

         i++ ;
      }

      endLogSection() ;

   }









   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will log a String to the
    *  log file.
    *  
    *
    *  @param debugLevel The Debug Level that this message should appear in
    *  @param message The message to log
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void log ( String message ) {                                  

      Log.log ( progressFile, message+"\n" ) ;

   }



   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will log an Exception
    *  to the log file. It will also display the
    *  Exception to the user
    *
    *  @param debugLevel The level of debug that the debug information should write with
    *  @param e The Exception to write
    *  @param showErrorDlg TRUE = Show the 'Error in last action took' window.
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void logSevereError ( Exception e) {
      try {

         StringWriter    sw      = new StringWriter () ;
         PrintWriter     pw      = new PrintWriter ( sw ) ;
         StringTokenizer strtok  = null ;
         String          tempStr = "" ;

         /* Get the stackTrace */
         e.printStackTrace ( pw ) ;
         tempStr = pw.toString() ;    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log the error                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         Log.log ( errorFile, e ) ;                 
         Log.log ( errorFile, "-----------------------------------------\n" ) ;                 


      } catch ( Exception e1 ) {
         System.out.println ( "Could not log Message:\n----------------------------------" ) ;
         e.printStackTrace() ;
         System.out.println ( "\n\nReceived Exception during log:\n" ) ;
         e1.printStackTrace() ;
      }
   }






   /*------------------------------------------------------------*/
   /* beginLogSection
    */
   /**
    *    This method will begin a new section
    *  in the log file. Each new section will have a full
    *  block heading with the description.
    *
    *  @param name The name of the section
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void beginLogSection ( String name ) {
      Log.beginLogSection ( progressFile, name ) ;
   }





   /*------------------------------------------------------------*/
   /* endLogSection
    */
   /**
    *    This method will end the current 
    *  log section.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void endLogSection () {
      Log.endLogSection ( progressFile ) ;
      Log.log ( progressFile, "\n\n\n" ) ;
   }




   public void logProgress ( String msg, int operations, int operation ) {
      log ( msg ) ;
   }

   public void logSevereError ( String string ) {
      Log.log ( errorFile, string+"\n" ) ;
   }


   public void batchStart () {
   }

   public void batchStop () {
   }

   public void logError ( String message ) {
      Log.log ( errorFile, message+"\n" ) ;
   }


   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will log an Exception
    *  to the log file. It will also display the
    *  Exception to the user
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void log ( int debugLevel, Exception e ) {
      log ( debugLevel, e, false ) ;
   }


   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method will log an Exception
    *  to the log file. It will also display the
    *  Exception to the user
    *
    *  @param debugLevel The level of debug that the debug information should write with
    *  @param e The Exception to write
    *  @param showErrorDlg TRUE = Show the 'Error in last action took' window.
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void log ( int debugLevel, Exception e, boolean showErrorDlg ) {
      try {

         System.out.println ( "Logging exception!" ) ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are to show the exception, get the string         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getDebugLevel() <= debugLevel ) {
            StringWriter    sw      = new StringWriter () ;
            PrintWriter     pw      = new PrintWriter ( sw ) ;
            StringTokenizer strtok  = null ;
            String          tempStr = "" ;

            /* Get the stackTrace */
            e.printStackTrace ( pw ) ;
            tempStr = pw.toString() ;    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Log the error                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            Log.log ( errorFile, e ) ;                 
            Log.log ( errorFile, "-----------------------------------------\n" ) ;                 


         }

      } catch ( Exception e1 ) {
         System.out.println ( "Could not log Message:\n----------------------------------" ) ;
         e.printStackTrace() ;
         System.out.println ( "\n\nReceived Exception during log:\n" ) ;
         e1.printStackTrace() ;
      }
   }


   /**
     *  This method will return the name of the progress file being written
     **/
   public static String getProgressFile () {
      return ( progressFile ) ;
   }

   /**
     *  This method will return the name of the error file being written
     **/
   public static String getErrorFile () {
      return ( errorFile ) ;
   }

} /* BatchLogSystem */
