package com.ibm.nzna.shared.db ;  
/*-----------------------------------------------------------*/
/*   Database                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Database                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * The Database object holds information about a specific database 
 * This class doesn't need to be created for a connection. A Connection
 * to a database should be obtained through the DatabaseSystem.
 *
 *
 * The Secondary Connection property means that a connection to this database is already
 * established, however that connection is currently running an SQL so another
 * connection was made to run a single SQL. A secondary connection will be closed
 * immediatly after the SQL is complete though.
 *
 * @version 1.0, 2/3/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Database                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class Database extends Object implements Runnable, MultiListRow {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String     name                = null ;
   private String     server              = null ;
   private String     port                = null ;
   private String     url                 = null ;
   private Connection connection          = null ;
   private boolean    connected           = false ;
   private String     userId              = null ;
   private String     password            = null ;
   private boolean    secondaryConnection = false ;
   private boolean    busy                = false ;
   private int        databaseId          = 0 ;
   private boolean    useAppDriver        = false ;
   private Thread     idleThread          = null ;
   private boolean    continueIdleThread  = true ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the object
    *  to the values passed
    *  
    *
    *  @param name The name of the database
    *  @param server The name of the server holding the database
    *  @param prot The port number that the server is listening on
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Database ( int databaseId, String name, String server, String port, boolean useAppDriver ) {
      this.useAppDriver = useAppDriver ;
      this.databaseId   = databaseId ;
      setName   ( name ) ;
      setServer ( server ) ;
      setPort   ( port ) ;
   }





   /*------------------------------------------------------------*/
   /* Database
    */
   /**
    *    This constructor initializes the Database object
    *  to the connection passed in. Please note that
    *  this connection is not automatically disconnected after no use!
    *
    *  @param databaseId The database ID of this database
    *  @param connection The connection
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Database ( int databaseId, Connection connection ) {
      this.databaseId = databaseId ;
      this.connection = connection ;
   }




   /*------------------------------------------------------------*/
   /* finalize
    */
   /**
    *    This method will kill our running thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finalize () {
      try {
         continueIdleThread = false ;
         interruptIdleThread() ;
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }



   /*------------------------------------------------------------*/
   /* setName
    */
   /**
    *    This method will set the name of the
    *  database that this object represents
    *  
    *
    *  @param name The new name of the database
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setName ( String name ) {
      this.name = name ;
      setURL () ;
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name of the
    *  database that this object represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName ( ) {
      return( name ) ;
   }





   /*------------------------------------------------------------*/
   /* setServer
    */
   /**
    *    This method will set the name of the
    *  server that this database object's database
    *  resides on
    *
    *  @param server The name of the database server
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setServer ( String server ) {
      this.server = server ;
      setURL () ;
   }




   /*------------------------------------------------------------*/
   /* getServer
    */
   /**
    *    This method will return the name of the
    *  server that the database of this object
    *  resides on
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected String getServer () {
      return( server ) ;
   }





   /*------------------------------------------------------------*/
   /* setPort
    */
   /**
    *    This method will set the port number
    *  that the server this database resides on
    *  is listening on 
    *
    *  @param port The port number
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setPort ( String port ) {
      this.port = port ;
      setURL () ;
   }





   /*------------------------------------------------------------*/
   /* getPort
    */
   /**
    *    This method will return the port number
    *  that the server this database resides on
    *  is listening on
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPort () {
      return( port ) ;
   }






   /*------------------------------------------------------------*/
   /* connect
    */
   /**
    *    This method will connect to the
    *  database that this object represents
    *  
    *
    *  @param secondaryConnection TRUE if this should be considered a secondary connection
    *  @param None
    *  @param None
    *
    *  @return TRUE = A Connection was Established, FALSE = Not 
    *//*---------------------------------------------------------*/
   protected boolean connect ( boolean secondaryConnection ) throws Exception {

      if ( url != null ) {

         this.secondaryConnection = secondaryConnection;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 2, "Connecting to "+url+" Using user "+userId+" Password is "+password.length()+" characters long." ) ;

         try {
            connection = DriverManager.getConnection( url, userId, password );
            connected = ( connection != null ) ;
            if ( ( connected ) && 
                 ( DatabaseSystem.getAutoDisconnect() ) &&
                 ( !secondaryConnection ) ) {
               if ( idleThread == null ) {
                  idleThread = new Thread ( this ) ;
                  idleThread.setPriority ( 1 ) ;
                  idleThread.start() ;
               }
            }

         } catch ( Exception e ) {
            String eString = e.toString() ;

            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, put up a meaningful error msg                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( eString.indexOf ( "suitable" ) >= 0 )
               new Exception ( "The DB/2 Driver could not be located" ) ;
         }

      }


      return( connected ) ;
   }






   /*------------------------------------------------------------*/
   /* disconnect
    */
   /**
    *    This method will disconnect from the
    *  database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void disconnect () {
      if ( getConnected() ) {
         try {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, "Disconnecting From Database "+name+"  on server "+server ) ;
            connection.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
         }
         connected = false ;
      }
   }



   /*------------------------------------------------------------*/
   /* setURL
    */
   /**
    *    This method will form the url from
    *  the current name, server and port number.
    *  Currently this method only creates DB/2 Specific URLS
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setURL () {

      if ( useAppDriver )
         url = "jdbc:db2:"+name ;
      else
         url = "jdbc:db2://"+server+":"+port+"/"+name ;
   }





   /*------------------------------------------------------------*/
   /* getConnected
    */
   /**
    *    This method will return the connected
    *  state. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Connected, FALSE = Not Connected
    *//*---------------------------------------------------------*/
   public boolean getConnected () {
      return( connected ) ;
   }





   /*------------------------------------------------------------*/
   /* setUserId
    */
   /**
    *    This method will set the user that we
    *  use to connect to this database with
    *  
    *
    *  @param userId The User Id to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setUserId ( String userId ) {
      this.userId = userId ;
   }




   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will return the User Id
    *  that we are connected to the database with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getUserId () {
      return( userId ) ;
   }





   /*------------------------------------------------------------*/
   /* setPassword
    */
   /**
    *    This method will set the password that
    *  will be used in conjunction with the User Id
    *  
    *
    *  @param password The password to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setPassword ( String password ) {
      this.password = password ;
   }





   /*------------------------------------------------------------*/
   /* getSecondaryConnection
    */
   /**
    *    This method will return TRUE if this
    *  connection represents a Secondary Connection. Please
    *  see the class description for a description of a Secondary Connection
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getSecondaryConnection () {
      return( secondaryConnection ) ;
   }


   /*------------------------------------------------------------*/
   /* setSecondaryConnection
    */
   /**
    *    This method will set the secondary
    *  Connection property. Please see the class description
    *  for a description of a Secondary Connection
    *
    *  @param secondaryConnection The new value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setSecondaryConnection ( boolean secondaryConnection ) {
      this.secondaryConnection = secondaryConnection ;
   }





   /*------------------------------------------------------------*/
   /* setBusy
    */
   /**
    *    This method will set the busy property. This
    *  means that if busy is TRUE, this connection
    *  currently is in use by a SQLMethod.
    *
    *  @param busy TRUE or FALSE 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setBusy ( boolean busy ) {
      this.busy = busy ;
   }





   /*------------------------------------------------------------*/
   /* getBusy
    */
   /**
    *    This method will return the busy property. This
    *  property is TRUE if this connection currently
    *  has an SQL running
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = This connection is currently running an SQL, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean getBusy () {
      return( busy ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return what data should
    *  appear in what column in a MultiList. These columns are
    *  specified as:
    *
    *  <pre>
    *  Column     Description
    *  ---------- ---------------------------------------------
    *  0          A small image of a database. A Small green light will be on if connected, a small red light if disconnected
    *  1          The name of the database
    *  2          The server that the database resides on
    *  3          The port number we are connected to
    *  4          TRUE/FALSE if the connection is busy
    *  5          TRUE/FALSE if the connection is a Secondary Connection
    *  
    *
    *  @param column The column to return
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = connected ? "Connected" : "Disconnected" ;
            break ;

         case 1 :
            retObj = name ;
            break ;

         case 2 :
            retObj = server ;
            break ;

         case 3 :
            retObj = port ;
            break ;

         case 4 :
            retObj = ""+busy ;
            break ;

         case 5 :
            retObj = ""+secondaryConnection ;
            break ;
      }

      return( retObj ) ;

   }





   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will be called when we need
    *  to compare this record with another record
    *  inside of a MultiList
    *
    *  @param object The object to compare to
    *  @param column The column to compare with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( MultiListSort object, int column ) {
      int rc = 0 ;
      if ( object instanceof Database ) {
         Database database = (Database) object ;

         switch ( column ) {
            
            case 0 :
               if ( database.connected = connected )
                  rc = 0 ;
               else
                  rc = 1 ;
               break ;

            case 1 :
               rc = database.name.compareTo ( name ) ;
               break ;

            case 2 :
               rc = database.server.compareTo ( server ) ;
               break ;

            case 3 :
               rc = database.port.compareTo ( port ) ;
               break ;

            case 4 :
               if ( database.busy == busy )
                  rc = 0 ;
               else
                  rc = 1 ;
               break ;

            case 5 :
               if ( database.secondaryConnection == secondaryConnection )
                  rc = 0 ;
               else
                  rc = 1 ;
               break ;
         }
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getConnection
    */
   /**
    *    This method will return the connection
    *  that this database has
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Connection getConnection () {
      return( connection ) ;
   }





   /*------------------------------------------------------------*/
   /* getDatabaseId
    */
   /**
    *    This method will return the unique ID
    *  specified for this database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getDatabaseId () {
      return( databaseId ) ;
   }





   /*------------------------------------------------------------*/
   /* getPassword
    */
   /**
    *    This method will return the unencrypted
    *  password
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected String getPassword () {
      return( password ) ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will return TRUE if the object
    *  passed in is equal to this object
    *  
    *
    *  @param compareObject The object to compare
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE 
    *//*---------------------------------------------------------*/
   public boolean equals ( Object compareObject ) {
      boolean rc = false ;

      if ( compareObject instanceof Database ) {
         Database database = (Database) compareObject ;

         if ( ( database.databaseId          == databaseId ) &&
              ( database.secondaryConnection == secondaryConnection ) )
            rc = true ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* commit
    */
   /**
    *    This method will commit all transactions
    *  on the current database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void commit () {
      try {
         connection.commit() ;
      } catch ( Exception e ) {

      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will monitor the connection and
    *  will disconnect if it is idle for more than 2 minutes. This
    *  thread simply waits for 2 minutes and then disconnects. If you
    *  do not wish to disconnect, interrupt this thread.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      while ( ( continueIdleThread ) && 
              ( DatabaseSystem.getAutoDisconnect() ) ) {
         try {
            Thread.sleep ( 300000 );
            if ( ( DatabaseSystem.getAutoDisconnect() ) && ( connected ) && ( !busy ) )
               DatabaseSystem.disconnect(databaseId) ;
         } catch ( Exception e ) {
         }
      }
   }




   /*------------------------------------------------------------*/
   /* interruptIdleThread
    */
   /**
    *    This method will interupt the idle thread so
    *  that another two minutes will be waited
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void interruptIdleThread () {
      if ( idleThread != null )
         idleThread.interrupt() ;
   }
} /* Database */
