package com.ibm.nzna.shared.db ;           
/*-----------------------------------------------------------*/
/*   DatabaseSystem                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DatabaseSystem                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The DatabaseSystem is responsible for making/breaking connections  
 * to external data sources for the Applications. This includes
 * database connection time out, monitoring/logging SQL work being done etc..
 * Any method wishing to use SQL will go through the DatabaseSystem. The
 * DatabaseSystem basically holds and manages the external connections for
 * the applications.
 *
 * @version 1.0, 2/4/100 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   DatabaseSystem                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DatabaseSystem extends Object {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Predefined databases                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static final int QUEST           = 1 ;  // Default Quest database
    public static final int CLIO            = 2 ;  // Default CLIO database
    public static final int OPICM           = 3 ;  // The OPIC/M Database

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Vars                                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static Vector          activeDatabaseVec          = null ;
    private static Vector          activeMethodVec            = null ;
    private static Vector          dormantMethodVec           = null ;
    private static Vector          listenerVec                = null ;
    private static Vector          statusListenerVec          = null ;
    private static LogSystem       log                        = null ;
    private static boolean         debug                      = false ;
    private static boolean         useAppDriver               = false ;
    private static boolean         allowMultiple              = true ;
    private static boolean         autoDisconnect             = true ;
    private static DatabaseSystem  instance                   = null ;




    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create and initialize the
     *  DatabaseSystem. Please note that this
     *  object should only be created once on application startup
     *
     *  @param debug If TRUE, active methods will be tracked 
     *  @param log The LogSystem to use for database logging
     *  @param useAppDriver If TRUE, use the DB/2 AppDriver, otherwise use the Net Driver
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public DatabaseSystem ( boolean debug, LogSystem log, boolean useAppDriver ) throws Exception {
        this.useAppDriver = useAppDriver ; 
        this.debug        = debug ;
        this.log          = log ;

        try {
            String driverName = null ;

            if ( useAppDriver )
                driverName = "COM.ibm.db2.jdbc.app.DB2Driver" ;
            else
                driverName = "COM.ibm.db2.jdbc.net.DB2Driver" ;

            if ( log != null )
                log.log ( 1, "Loading DB/2 Driver: "+driverName ) ;
            Class.forName ( driverName ).newInstance() ;

        } catch ( Exception e ) {
            System.out.println ( "Could not load DB/2 Driver:\nException:"+e ) ;
            if ( log != null ) {
                log.log ( 1, "Could not load DB/2 Drvier: Exception:" ) ;
                log.log ( 1, e ) ;
            }

            throw ( e ) ;
        }

        instance = this ;
    }






    /*------------------------------------------------------------*/
    /* createConnection
     */
    /**
     *    This method will attempt to create a connection
     *  to the database specified in the parameters. If a connection
     *  to the same database already exists, no new connection will
     *  be created and TRUE will be returned
     *  
     *
     *  @param databaseId The unique ID for the database. This is internal to the application and only used when 
     *                    getting a connection.
     *  @param name The name of the database
     *  @param server The name of the server that holds the database
     *  @param port The port number that the server is listening on
     *  @param userId The User Id to connect to
     *  @param password The password for the User Id
     *
     *  @return TRUE = A Connection was established, FALSE if a connection was not established
     *//*---------------------------------------------------------*/
    public static boolean createConnection ( int databaseId, String name, String server, String port, String userId, String password ) {
        Database database = findDatabase ( name, server, port, userId ) ;
        boolean  rc       = false ;

        try {
            if ( database == null ) {
                database = new Database ( databaseId, name, server, port, useAppDriver ) ;
                database.setUserId ( userId ) ;
                database.setPassword ( password ) ;

                new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTING, 
                                           database, 
                                           statusListenerVec ) ;

                rc = database.connect ( false ) ;
                if ( rc )
                    addDatabase ( database ) ;

                new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTED, 
                                           database, 
                                           statusListenerVec ) ;

            } else
                rc = true ;
        } catch ( Exception e ) {
            if ( log != null )
                log.log ( 1, e ) ;
            else
                e.printStackTrace() ;
        }

        return( rc ) ;
    }


    /**
      * This method will auto allocate a certain number of database connections. Each
      * connection allocated will remain connected regardless. Please note that the initial
      * connection to the database passed as databaseId must have already been connected using
      * the createConnection method.
      *
      * @param databaseId The ID of the database that you wish to auto allocate connections for.
      * @param connections The number of connections
      **/
    public static void autoAllocateConnections ( int databaseId, int connections ) {
        Database   curDatabase      = findDatabase ( databaseId ) ;

        try {
            for ( int i = 0; i < connections; i++ ) {
                Database database = new Database ( curDatabase.getDatabaseId(), 
                                                   curDatabase.getName(), 
                                                   curDatabase.getServer(), 
                                                   curDatabase.getPort(),
                                                   useAppDriver ) ;

                database.setUserId ( curDatabase.getUserId() ) ;
                database.setPassword ( curDatabase.getPassword() ) ;
                database.setSecondaryConnection ( false ) ;

                new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTING, 
                                           database, 
                                           statusListenerVec ) ;

                if ( database.connect ( false ) ) {
                    database.getConnection ( ) ;
                    database.setBusy ( false ) ;
                    addDatabase ( database ) ;
                }

                new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTED, 
                                           database, 
                                           statusListenerVec ) ;
            }

            if ( log != null )
                log.log ( 1, "Auto Allocated "+connections+" connections to database "+curDatabase.getName()+" on server "+curDatabase.getServer() ) ;
            else
                System.out.println ( "Auto Allocated "+connections+" connections to database "+curDatabase.getName()+" on server "+curDatabase.getServer() ) ;

        } catch ( Exception e ) {
            if ( log != null )
                log.log ( 1, e ) ;
            else
                e.printStackTrace() ;
        }

    }


    /*------------------------------------------------------------*/
    /* getConnection
     */
    /**
     *    This method will return a connection
     *  object to the database requested. If the database requested
     *  is busy, then a new connection will be established.
     *  YOU MUST CALL 'releaseConnection' ONCE YOU ARE DONE WITH IT!
     *  
     *
     *  @param databaseId The ID of the database to use
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static Connection getConnection ( int databaseId ) {
        Database   database         = null ;
        Database   curDatabase      = findDatabase ( databaseId ) ;
        Connection retConnection = null ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   If we found the database, check to see if its busy      */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( curDatabase != null ) {

            try {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   If the database is busy, create a new one               */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( ( curDatabase.getBusy() ) && ( allowMultiple ) ) {

                    database = new Database ( curDatabase.getDatabaseId(), 
                                              curDatabase.getName(), 
                                              curDatabase.getServer(), 
                                              curDatabase.getPort(),
                                              useAppDriver ) ;
                    database.setUserId ( curDatabase.getUserId() ) ;
                    database.setPassword ( curDatabase.getPassword() ) ;
                    database.setSecondaryConnection ( true ) ;

                    new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTING, 
                                               database, 
                                               statusListenerVec ) ;

                    if ( database.connect ( true ) ) {
                        retConnection = database.getConnection() ;
                        addDatabase ( database ) ;
                        database.setBusy ( true ) ;
                    }

                    new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTED, 
                                               database, 
                                               statusListenerVec ) ;

                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Otherwise just return this connection                   */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else {

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   If we are not connected, connect                        */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( !curDatabase.getConnected() ) {
                        new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTING, curDatabase, statusListenerVec ) ;
                        curDatabase.connect ( false ) ;
                        new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_CONNECTED, curDatabase, statusListenerVec ) ;
                    }

                    if ( curDatabase.getConnected() ) {
                        retConnection = curDatabase.getConnection() ;
                        curDatabase.setBusy ( true ) ;
                    }

                }
            } catch ( Exception e ) {
                if ( log != null )
                    LogSystem.log ( 1, e ) ;
                else
                    e.printStackTrace() ;
            }
        }

        return( retConnection ) ;
    }





    /*------------------------------------------------------------*/
    /* setConnection
     */
    /**
     *    This method will set a database connection
     *  object. This is mostly used in Stored Procedures
     *  who already have their Connection object
     *
     *  @param databaseId The ID that represents this database
     *  @param connection The Connection to set
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setConnection ( int databaseId, Connection connection ) {
        Database newDatabase = new Database ( databaseId, connection ) ;

        activeDatabaseVec.addElement ( newDatabase ) ;
    }



    /*------------------------------------------------------------*/
    /* findDatabase
     */
    /**
     *    This method will look through our internal
     *  vector of active databases for the database
     *  passed in.
     *
     *  @param name The name of the database
     *  @param server The server that the database resides on
     *  @param port The port number for the server
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static Database findDatabase ( String name, String server, String port, String userId ) {
        Database retDatabase = null ;

        if ( activeDatabaseVec != null ) {
            int         i        = 0 ;
            int         size     = activeDatabaseVec.size() ;
            Database    database = null ;

            while ( ( i < size ) && ( retDatabase == null ) ) {
                database = (Database) activeDatabaseVec.elementAt(i) ;

                if ( ( database.getName().equals   ( name ) ) &&
                     ( database.getServer().equals ( server ) ) &&
                     ( database.getUserId().equals ( userId ) ) &&
                     ( database.getPort().equals   ( port   ) ) )
                    retDatabase = database ;
                i++ ;
            }
        }

        return( retDatabase ) ;
    }


    /*------------------------------------------------------------*/
    /* findDatabase
     */
    /**
     *    This method will look through our internal
     *  vector of active databases for the database
     *  passed in.
     *
     *  @param id The database id to look for
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static Database findDatabase ( int databaseId ) {
        Database retDatabase = null ;

        if ( activeDatabaseVec != null ) {
            int         i        = 0 ;
            int         size     = activeDatabaseVec.size() ;
            Database    database = null ;

            while ( ( i < size ) && ( retDatabase == null ) ) {
                database = (Database) activeDatabaseVec.elementAt(i) ;

                if ( database.getDatabaseId() == databaseId )
                    retDatabase = database ;
                i++ ;
            }
        }

        return( retDatabase ) ;
    }



    /*------------------------------------------------------------*/
    /* findNotBusyDatabase
     */
    /**
     *    This method will look through our internal
     *  vector of active databases for the database
     *  passed in.
     *
     *  @param id The database id to look for
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static Database findNotBusyDatabase ( int databaseId ) {
        Database retDatabase = null ;

        if ( activeDatabaseVec != null ) {
            int         i        = 0 ;
            int         size     = activeDatabaseVec.size() ;
            Database    database = null ;

            while ( ( i < size ) && ( retDatabase == null ) ) {
                database = (Database) activeDatabaseVec.elementAt(i) ;

                if ( ( database.getDatabaseId() == databaseId ) &&
                     ( database.getBusy() == false ) )
                    retDatabase = database ;
                i++ ;
            }
        }

        return( retDatabase ) ;
    }



    /*------------------------------------------------------------*/
    /* getDatabaseFromConnection
     */
    /**
     *    This method will look through our internal
     *  vector of active databases for the database
     *  passed in.
     *
     *  @param id connection The database connection
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static Database getDatabaseFromConnection ( Connection connection ) {
        Database retDatabase = null ;

        if ( activeDatabaseVec != null ) {
            int         i        = 0 ;
            int         size     = activeDatabaseVec.size() ;
            Database    database = null ;

            while ( ( i < size ) && ( retDatabase == null ) ) {
                database = (Database) activeDatabaseVec.elementAt(i) ;

                if ( database.getConnection() == connection )
                    retDatabase = database ;
                i++ ;
            }
        }

        return( retDatabase ) ;
    }





    /*------------------------------------------------------------*/
    /* addDatabase
     */
    /**
     *    This method will add a database to our
     *  list of active databases
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void addDatabase ( Database database ) {

        if ( activeDatabaseVec == null )
            activeDatabaseVec = new Vector ( 1, 1 ) ;

        activeDatabaseVec.addElement ( database ) ;
        fireDatabaseAdded ( database ) ;
    }





    /*------------------------------------------------------------*/
    /* removeDatabase
     */
    /**
     *    This method will remove a database
     *  from the active Database Vector
     *  
     *
     *  @param database The database to remove
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void removeDatabase ( Database database ) {
        database.disconnect() ;
        if ( activeDatabaseVec != null ) {
            activeDatabaseVec.removeElement ( database ) ;
            database.finalize() ;
        }

        fireDatabaseRemoved ( database ) ;
    }





    /*------------------------------------------------------------*/
    /* getActiveDatabases
     */
    /**
     *    This method will return a vector full
     *  of Database objects that are currently
     *  active.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Vector
     *//*---------------------------------------------------------*/
    protected static Vector getActiveDatabases () {
        return( activeDatabaseVec ) ;
    }




    /*------------------------------------------------------------*/
    /* addDatabaseSystemListener
     */
    /**
     *  This method will add a listener to be notified whenever
     *  database activities occur.   
     *  
     *
     *  @param listener The listener to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void addDatabaseSystemListener ( DatabaseSystemListener listener ) {
        if ( listenerVec == null )
            listenerVec = new Vector ( 1, 1 ) ;
        listenerVec.addElement ( listener ) ;
    }




    /*------------------------------------------------------------*/
    /* removeDatabaseSystemListener
     */
    /**
     *    This method will remove a listener
     *  
     *  
     *
     *  @param listener The listener to be removed
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void removeDatabaseSystemListener ( DatabaseSystemListener listener ) {
        if ( listenerVec != null )
            listenerVec.removeElement ( listener ) ;
    }






    /*------------------------------------------------------------*/
    /* fireDatabaseAdded
     */
    /**
     *    This method will notify all listeners
     *  of a database being added
     *  
     *
     *  @param database The database added
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void fireDatabaseAdded ( Database database ) {
        if ( listenerVec != null ) {
            int i    =  0 ;
            int size = listenerVec.size() ;

            while ( i < size ) {
                ((DatabaseSystemListener)listenerVec.elementAt(i)).databaseAdded ( database ) ;
                i++ ;
            }
        }
    }




    /*------------------------------------------------------------*/
    /* fireDatabaseRemoved
     */
    /**
     *    This method will notify all listeners
     *  of a database being removed
     *  
     *
     *  @param database The database removed
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void fireDatabaseRemoved ( Database database ) {
        if ( listenerVec != null ) {
            int i    =  0 ;
            int size = listenerVec.size() ;

            while ( i < size ) {
                ((DatabaseSystemListener)listenerVec.elementAt(i)).databaseRemoved ( database ) ;
                i++ ;
            }
        }
    }




    /*------------------------------------------------------------*/
    /* fireMethodAdded
     */
    /**
     *    This method will notify all listeners
     *  of a method being added
     *  
     *
     *  @param database The method Added
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void fireMethodAdded ( Database database, SQLMethod method ) {
        if ( listenerVec != null ) {
            int i    =  0 ;
            int size = listenerVec.size() ;

            while ( i < size ) {
                ((DatabaseSystemListener)listenerVec.elementAt(i)).methodAdded ( database, method ) ;
                i++ ;
            }
        }
    }


    /*------------------------------------------------------------*/
    /* fireMethodRemoved
     */
    /**
     *    This method will notify all listeners
     *  of a method being added
     *  
     *
     *  @param database The method Removed
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void fireMethodRemoved ( Database database, SQLMethod method ) {
        if ( listenerVec != null ) {
            int i    =  0 ;
            int size = listenerVec.size() ;

            while ( i < size ) {
                ((DatabaseSystemListener)listenerVec.elementAt(i)).methodRemoved ( database, method ) ;
                i++ ;
            }
        }
    }






    /*------------------------------------------------------------*/
    /* shutdown
     */
    /**
     *    This method will disconnect from all
     *  currently connected databases
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static boolean  shutdown () {
        if ( activeDatabaseVec != null ) {
            int i       = 0 ;
            int size    = activeDatabaseVec.size() ;

            while ( i < size ) {
                ((Database)activeDatabaseVec.elementAt(i)).disconnect() ;
                fireDatabaseRemoved ( (Database)activeDatabaseVec.elementAt(i)) ;
                i++ ;
            }
        }

        return( true ) ;
    }





    /*------------------------------------------------------------*/
    /* addSQLMethod
     */
    /**
     *    This method will add a SQL Method to
     *  the list of running methods.
     *  
     *
     *  @param databaseId The ID of the database that this method is running on
     *  @param method The SQLMethod object
     *  @param None
     *
     *//*---------------------------------------------------------*/
    protected static void addSQLMethod ( Database database, SQLMethod method ) {

        try {
            if ( database != null ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Interrupt the idle thread for this database             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                database.interruptIdleThread() ;


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Add this item to our vector                             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( debug ) {
                    if ( activeMethodVec == null )
                        activeMethodVec = new Vector ( 1, 1 ) ;
                    activeMethodVec.addElement ( method ) ;                        
                    if ( log != null )
                        log.log ( 1, "DatabaseSystem Added method:"+method.getColumnData(0) ) ;
                }

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Tell everyone else                                      */
                /*                                                           */
                /*-----------------------------------------------------------*/
                fireMethodAdded ( database, method ) ;
            }
        } catch ( Exception e ) {
            log.log ( 1, e ) ;
        }
    }

    /*------------------------------------------------------------*/
    /* removeSQLMethod
     */
    /**
     *    This method will remove a SQL Method to
     *  the list of running methods. The method will be placed in the
     *  list of dormant methods if and only if the application is running
     *  at debug level 1, 2 or 3.
     *  
     *
     *  @param databaseId The ID of the database that this method is running on
     *  @param method The SQLMethod object
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    protected static void removeSQLMethod ( Database database, SQLMethod method ) {

        try {
            if ( database != null ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Add this item to our vector                             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( debug ) {
                    if ( dormantMethodVec == null )
                        dormantMethodVec = new Vector ( 1, 1 ) ;
                    dormantMethodVec.addElement ( method ) ;         

                    if ( activeMethodVec != null )
                        activeMethodVec.removeElement ( method ) ;

                    if ( log != null )
                        log.log ( 1, "DatabaseSystem removed method:"+method.getColumnData(0) ) ;
                }

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Tell everyone else                                      */
                /*                                                           */
                /*-----------------------------------------------------------*/
                fireMethodRemoved ( database, method ) ;
            }
        } catch ( Exception e ) {
            log.log ( 1, e, false ) ;
        }
    }




    /*------------------------------------------------------------*/
    /* releaseConnection
     */
    /**
     *    This method will release the connection
     *  to a database. The connection will be
     *  reset and removed if this is a secondary database
     *
     *  @param database The database to release
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void releaseConnection ( Database database ) {


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   The database is no longer busy!                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        database.setBusy ( false ) ;                                                             

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Disconnect from this database if it is secondary        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( database.getSecondaryConnection() )
            removeDatabase ( database ) ;

    }


    /*------------------------------------------------------------*/
    /* getActiveMethods
     */
    /**
     *    This method will return the current
     *  list of active methods
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Vector
     *//*---------------------------------------------------------*/
    protected static Vector getActiveMethods () {
        return( activeMethodVec ) ;
    }






    /*------------------------------------------------------------*/
    /* addStatusListener
     */
    /**
     *    This method will add a status listener
     *  
     *  
     *
     *  @param listener The DatabaseStatusListener to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void addStatusListener ( DatabaseStatusListener listener ) {
        if ( statusListenerVec == null )
            statusListenerVec = new Vector ( 1, 1 ) ;
        statusListenerVec.addElement ( listener ) ;
    }




    /*------------------------------------------------------------*/
    /* removeStatusListener
     */
    /**
     *    This method will remove a status listener
     *  from the DatabaseSystem
     *  
     *
     *  @param listener The DatabaseStatusListener to remove
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void removeStatusListener ( DatabaseStatusListener listener ) {
        if ( statusListenerVec != null )
            statusListenerVec.removeElement ( listener ) ;
    }





    /*------------------------------------------------------------*/
    /* disconnect
     */
    /**
     *    This method will disconnect the passed
     *  database
     *  
     *
     *  @param int The integer identifier you used on a database in the createConnection call
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void disconnect ( int databaseId ) {
        disconnect ( findDatabase ( databaseId ) ) ;
    }



    /*------------------------------------------------------------*/
    /* disconnect
     */
    /**
     *    This method will disconnect the passed
     *  database
     *  
     *
     *  @param database The Database object to disconnect
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void disconnect ( Database database ) {

        if ( database != null ) {
            new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_DISCONNECTING, database, statusListenerVec ) ;
            database.disconnect() ;
            new DatabaseStatusNotify ( DatabaseStatusListener.DATABASE_STATUS_DISCONNECTED, database, statusListenerVec ) ;
        }
    }





    /*------------------------------------------------------------*/
    /* getAutoDisconnect
     */
    /**
     *    This method will return the auto-disconnect
     *  property. This property determines if the
     *  DatabaseSystem will disconnect a database after an idle time out
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static boolean getAutoDisconnect() {
        return( autoDisconnect ) ;
    }





    /*------------------------------------------------------------*/
    /* setAutoDisconnect
     */
    /**
     *    This method will set the Auto Disconnect
     *  property. If set to TRUE, the DatabaseSystem
     *  will disconnect from databases that are inactive
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setAutoDisconnect ( boolean autoDisconnect_ ) {
        autoDisconnect = autoDisconnect_ ;
    }



    /**
      * This method will turn multiple connections on or off. If off the
      * database system will not try to create a new connection to a database
      * when all current connections are busy. This should only be done in a 
      * controlled batch job that you know is not going to create too many
      * connections.
      *
      * @param allowMultiple TRUE/FALSE
      **/
    public static void setAllowMultipleConnections ( boolean allowMultiple_ ) {
        allowMultiple = allowMultiple_ ;
    }



    /*------------------------------------------------------------*/
    /* handleSQLException
     */
    /**
     *    This method will handle the following SQLExceptions
     *  accordingly.
     *  
     *
     *  @param sqlMethod The SQLMethod that threw the exception
     *  @param SQLException The SQLException
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void handleSQLException ( SQLMethod sqlMethod, SQLException sqlError ) {
        int sqlCode = sqlError.getErrorCode() ;

        if ( log != null ) {
            log.beginLogSection ( "DB/2 Error" ) ;
            log.log ( 1, "Logging DB/2 Exception" ) ;
            log.log ( 1, "   SQLCODE: "+sqlCode ) ;
            log.log ( 1, "   SQLSTATE:"+sqlError.getSQLState() ) ;
            log.log ( 1, "   Method:  "+sqlMethod.getMethodName() ) ;
            log.log ( 1, "" ) ;
            log.log ( 1, "DB/2 Exception thrown:" ) ;
            log.log ( 1, sqlError, false ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   If the SQLCode is -99999, then that doesn't mean        */
        /* anything and we need to look at the SQL State.            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( sqlCode == -99999 )
            handleSQLState ( sqlMethod, sqlError.getSQLState() ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Otherwise, handle the SQL Code                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Master switch statement to catch DB/2 SQL Codes         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            switch ( sqlCode ) {
            }

        } /* else  */

    } /* handleSQLException  */






    /*------------------------------------------------------------*/
    /* handleSQLState
     */
    /**
     *    This method will handle the SQL State passed
     *  in for the SQL Method passed
     *  
     *
     *  @param SQLMethod The SQL Method that through the SQL State
     *  @param sqlState The SQL State
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private static void handleSQLState ( SQLMethod sqlMethod, String sqlState ) {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   08S01 (The DB/2 JDBC Daemon is no longer available)     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( sqlState.equals ( "08S01" ) ) {
            disconnect ( sqlMethod.getDatabase() ) ;

            if ( log != null )
                log.log ( 1, "Received SQL State 08S01, meaning that the DB/2 JDBC Daemon on the server is no longer responding" ) ;
        }


    }




    /*------------------------------------------------------------*/
    /* isDatabaseConnected
     */
    /**
     *    This method will return TRUE if the
     *  database passed in is connected, FALSE
     *  otherwise
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static boolean isDatabaseConnected ( int databaseId ) {
        boolean  connected = false ;


        if ( activeDatabaseVec != null ) {
            int         i        = 0 ;
            int         size     = activeDatabaseVec.size() ;
            Database    database = null ;

            while ( ( i < size ) && ( !connected ) ) {
                database = (Database) activeDatabaseVec.elementAt(i) ;

                if ( ( database.getDatabaseId() == databaseId ) &&
                     ( !database.getSecondaryConnection() ) )
                    connected = database.getConnected() ;
                i++ ;
            }
        }

        return( connected ) ;
    }




    /*------------------------------------------------------------*/
    /* getInstance
     */
    /**
     *    This method will return the DatabaseSystem
     *  instance
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static DatabaseSystem getInstance() {
        return( instance ) ;
    }

} /* DatabaseSystem */
