package com.ibm.nzna.shared.db ;
import java.sql.* ;
import java.util.* ;
import java.sql.Date ;
import java.math.BigDecimal;
import com.ibm.nzna.shared.util.* ;
/**
 * <P>CallableStatement is used to execute SQL stored procedures.
 *
 * <P>JDBC provides a stored procedure SQL escape that allows stored
 * procedures to be called in a standard way for all RDBMS's. This
 * escape syntax has one form that includes a result parameter and one
 * that does not. If used, the result parameter must be registered as
 * an OUT parameter. The other parameters may be used for input,
 * output or both. Parameters are refered to sequentially, by
 * number. The first parameter is 1.
 *
 * <P><CODE>
 * {?= call <procedure-name>[<arg1>,<arg2>, ...]}<BR>
 * {call <procedure-name>[<arg1>,<arg2>, ...]}
 * </CODE>
 *    
 * <P>IN parameter values are set using the set methods inherited from
 * PreparedStatement. The type of all OUT parameters must be
 * registered prior to executing the stored procedure; their values
 * are retrieved after execution via the get methods provided here.
 *
 * <P>A Callable statement may return a ResultSet or multiple
 * ResultSets. Multiple ResultSets are handled using operations
 * inherited from Statement.
 *
 * <P>For maximum portability, a call's ResultSets and update counts
 * should be processed prior to getting the values of output
 * parameters.
 *
 * @see Connection#prepareCall
 * @see ResultSet 
 */
public class QuestCallableStatement extends QuestPreparedStatement implements CallableStatement {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   int      paramNum = 0 ;
   String   storedProcName = "" ;
   String[] paramArray = null ;
   boolean  inDebug    = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *  This method will create a default QuestCallableStatement
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestCallableStatement ( CallableStatement statement, String storedProcName, int paramNum ) {
      super ( statement ) ;
      try {
         inDebug = false ; //( PropertySystem.getInt ( PropertySystem.PROPERTY_DEBUGLEVEL ) > 0 ) ;
         this.storedProcName = storedProcName ;
         this.paramNum       = paramNum ;

         if ( inDebug )
            paramArray = new String [ paramNum+1 ] ;
      } catch ( Exception e ) {
      }
   }


   /**
    * Before executing a stored procedure call, you must explicitly
    * call registerOutParameter to register the java.sql.Type of each
    * out parameter.
    *
    * <P><B>Note:</B> When reading the value of an out parameter, you
    * must use the getXXX method whose Java type XXX corresponds to the
    * parameter's registered SQL type.
    *
    * @param parameterIndex the first parameter is 1, the second is 2,...
    * @param sqlType SQL type code defined by java.sql.Types;
    * for parameters of type Numeric or Decimal use the version of
    * registerOutParameter that accepts a scale value
    * @exception SQLException if a database-access error occurs.
    * @see Type 
    */
   public void registerOutParameter(int parameterIndex, int sqlType) throws SQLException {
      registerOutParameter ( parameterIndex, sqlType, 0 ) ;
   }

   /**
    * Use this version of registerOutParameter for registering
    * Numeric or Decimal out parameters.
    *
    * <P><B>Note:</B> When reading the value of an out parameter, you
    * must use the getXXX method whose Java type XXX corresponds to the
    * parameter's registered SQL type.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param sqlType use either java.sql.Type.NUMERIC or java.sql.Type.DECIMAL
    * @param scale a value greater than or equal to zero representing the 
    *              desired number of digits to the right of the decimal point
    * @exception SQLException if a database-access error occurs.
    * @see Type 
    */
   public void registerOutParameter(int parameterIndex, int sqlType, int scale) throws SQLException {
      try {
         (( CallableStatement ) statement).registerOutParameter(parameterIndex, sqlType,  scale) ;

         switch ( sqlType ) {
            case Types.LONGVARCHAR :
               setString ( parameterIndex, "\0" ) ;
               break ;

            case Types.INTEGER :
               setInt ( parameterIndex, 0 ) ;
               break ;

            case Types.SMALLINT :
               setShort ( parameterIndex, (short) 0 ) ;
               break ;


         }
      } catch ( SQLException e ) {
         if ( !e.getSQLState().equals ( "S1093" ) )
            throw ( e ) ;
      }
   }

   /**
    * An OUT parameter may have the value of SQL NULL; wasNull reports 
    * whether the last value read has this special value.
    *
    * <P><B>Note:</B> You must first call getXXX on a parameter to
    * read its value and then call wasNull() to see if the value was
    * SQL NULL.
    *
    * @return true if the last parameter read was SQL NULL 
    * @exception SQLException if a database-access error occurs.
    */
   public boolean wasNull() throws SQLException {
      return( (( CallableStatement ) statement).wasNull()) ;
   }

   /**
    * Get the value of a CHAR, VARCHAR, or LONGVARCHAR parameter as a Java String.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is null
    * @exception SQLException if a database-access error occurs.
    */
   public String getString(int parameterIndex) throws SQLException {
      String value = (( CallableStatement ) statement).getString( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a BIT parameter as a Java boolean.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is false
    * @exception SQLException if a database-access error occurs.
    */
   public boolean getBoolean(int parameterIndex) throws SQLException {
      boolean value = (( CallableStatement ) statement).getBoolean( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a TINYINT parameter as a Java byte.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is 0
    * @exception SQLException if a database-access error occurs.
    */
   public byte getByte(int parameterIndex) throws SQLException {
      byte value = (( CallableStatement ) statement).getByte( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a SMALLINT parameter as a Java short.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is 0
    * @exception SQLException if a database-access error occurs.
    */
   public short getShort(int parameterIndex) throws SQLException {
      short value = (( CallableStatement ) statement).getShort( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of an INTEGER parameter as a Java int.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is 0
    * @exception SQLException if a database-access error occurs.
    */
   public int getInt(int parameterIndex) throws SQLException {
      int value = (( CallableStatement ) statement).getInt( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a BIGINT parameter as a Java long.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is 0
    * @exception SQLException if a database-access error occurs.
    */
   public long getLong(int parameterIndex) throws SQLException {
      long value = (( CallableStatement ) statement).getLong( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a FLOAT parameter as a Java float.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is 0
    * @exception SQLException if a database-access error occurs.
    */
   public float getFloat(int parameterIndex) throws SQLException {
      float value = (( CallableStatement ) statement).getFloat( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a DOUBLE parameter as a Java double.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is 0
    * @exception SQLException if a database-access error occurs.
    */
   public double getDouble(int parameterIndex) throws SQLException {
      double value = (( CallableStatement ) statement).getDouble( parameterIndex ) ;
      return( value ) ;
   }

   /** 
    * Get the value of a NUMERIC parameter as a java.math.BigDecimal object.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    *
    * @param scale a value greater than or equal to zero representing the
    * desired number of digits to the right of the decimal point 
    *
    * @return the parameter value; if the value is SQL NULL, the result is
    * null 
    * @exception SQLException if a database-access error occurs.
    */
   public BigDecimal getBigDecimal(int parameterIndex, int scale) throws SQLException {
      BigDecimal value = (( CallableStatement ) statement).getBigDecimal( parameterIndex, scale ) ;
      return( value ) ;
   }

   /**
    * Get the value of a SQL BINARY or VARBINARY parameter as a Java byte[]
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is null
    * @exception SQLException if a database-access error occurs.
    */
   public byte[] getBytes(int parameterIndex) throws SQLException {
      byte[] value = (( CallableStatement ) statement).getBytes( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a SQL DATE parameter as a java.sql.Date object
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is null
    * @exception SQLException if a database-access error occurs.
    */
   public java.sql.Date getDate(int parameterIndex) throws SQLException {
      java.sql.Date value = (( CallableStatement ) statement).getDate( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a SQL TIME parameter as a java.sql.Time object.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is null
    * @exception SQLException if a database-access error occurs.
    */
   public java.sql.Time getTime(int parameterIndex) throws SQLException {
      java.sql.Time value = (( CallableStatement ) statement).getTime( parameterIndex ) ;
      return( value ) ;
   }

   /**
    * Get the value of a SQL TIMESTAMP parameter as a java.sql.Timestamp object.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @return the parameter value; if the value is SQL NULL, the result is null
    * @exception SQLException if a database-access error occurs.
    */
   public java.sql.Timestamp getTimestamp(int parameterIndex) throws SQLException {
      java.sql.Timestamp value = (( CallableStatement ) statement).getTimestamp( parameterIndex ) ;
      return( value ) ;
   }

   //----------------------------------------------------------------------
   // Advanced features:


   /**
    * Get the value of a parameter as a Java object.
    *
    * <p>This method returns a Java object whose type coresponds to the SQL
    * type that was registered for this parameter using registerOutParameter.
    *
    * <p>Note that this method may be used to read
    * datatabase-specific, abstract data types. This is done by
    * specifying a targetSqlType of java.sql.types.OTHER, which
    * allows the driver to return a database-specific Java type.
    *
    * @param parameterIndex The first parameter is 1, the second is 2, ...
    * @return A java.lang.Object holding the OUT parameter value.
    * @exception SQLException if a database-access error occurs.
    * @see Types 
    */
   public Object getObject(int parameterIndex) throws SQLException {
      Object value = (( CallableStatement ) statement).getObject( parameterIndex ) ;
      return( value ) ;
   }

   public boolean execute() throws SQLException {
      boolean rc     = false ;

      rc = ((CallableStatement) statement).execute() ;

      return( rc ) ;
   }


   public void setNull(int parameterIndex, int sqlType) throws SQLException {
      ((CallableStatement) statement).setNull ( parameterIndex, sqlType ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( "null" ) ;
   }

   public void setBoolean(int parameterIndex, boolean x) throws SQLException {
      ((CallableStatement) statement).setBoolean ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }

   public void setByte(int parameterIndex, byte x) throws SQLException {
      ((CallableStatement) statement).setByte ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }

   public void setShort(int parameterIndex, short x) throws SQLException {
      ((CallableStatement) statement).setShort ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }

   public void setInt(int parameterIndex,int x) throws SQLException {
      ((CallableStatement) statement).setInt ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }

   public void setLong(int parameterIndex,long x) throws SQLException {
      ((CallableStatement) statement).setLong ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }


   public void setBigDecimal(int parameterIndex,BigDecimal x) throws SQLException {
      ((CallableStatement) statement).setBigDecimal ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }


   public void setDouble(int parameterIndex,double x) throws SQLException {
      ((CallableStatement) statement).setDouble ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }

   public void setFloat(int parameterIndex,float x) throws SQLException {
      ((CallableStatement) statement).setFloat ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }



   public void setBytes(int parameterIndex,byte x[]) throws SQLException {
      ((CallableStatement) statement).setBytes ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( x ) ;
   }

   public void setDate(int parameterIndex,Date x) throws SQLException {
      ((CallableStatement) statement).setDate ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }


   public void setTime(int parameterIndex, Time x) throws SQLException {
      ((CallableStatement) statement).setTime ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }

   public void setTimestamp(int parameterIndex, Timestamp x) throws SQLException {
      ((CallableStatement) statement).setTimestamp ( parameterIndex, x ) ;
      if ( inDebug )
         paramArray[parameterIndex] = new String ( ""+x ) ;
   }

   public void setString(int parameterIndex, String x) throws SQLException {
      try {
         ((CallableStatement) statement).setString ( parameterIndex, x ) ;
         if ( inDebug )
            paramArray[parameterIndex] = new String ( x ) ;
      } catch ( SQLException e ) {
         if ( !e.getSQLState().equals ( "S1093" ) )
            throw ( e ) ;
      }
   }

   /*------------------------------------------------------------*/
   /* getSQLString
    */
   /**
    *    This method will create an exact SQL String
    *  that is passed to DB/2 including any params
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getSQLString () {
      String retStr = "CALL "+storedProcName+" ( " ;
      int    i      = 0 ;

      while ( i <= paramNum ) {
         if ( i != paramNum )
            retStr = retStr+paramArray[i]+", " ;

         i++ ;
      }


      return( retStr+")" ) ;
   }



   //--------------------------JDBC 2.0-----------------------------

   /**
    * Retrieves the value of the designated JDBC <code>NUMERIC</code> parameter as a 
    * <code>java.math.BigDecimal</code> object with as many digits to the
    * right of the decimal point as the value contains.
    * @param parameterIndex the first parameter is 1, the second is 2,
    * and so on
    * @return the parameter value in full precision.  If the value is 
    * SQL <code>NULL</code>, the result is <code>null</code>. 
    * @exception SQLException if a database access error occurs
    * @see #setBigDecimal
    * @since 1.2
    */
   public BigDecimal getBigDecimal(int parameterIndex) throws SQLException {
      return ( ((CallableStatement)statement).getBigDecimal ( parameterIndex ) ) ;
   }

   /**
    * Returns an object representing the value of OUT parameter 
    * <code>i</code> and uses <code>map</code> for the custom
    * mapping of the parameter value.
    * <p>
    * This method returns a Java object whose type corresponds to the
    * JDBC type that was registered for this parameter using the method
    * <code>registerOutParameter</code>.  By registering the target
    * JDBC type as <code>java.sql.Types.OTHER</code>, this method can
    * be used to read database-specific abstract data types.  
    * @param i the first parameter is 1, the second is 2, and so on
    * @param map the mapping from SQL type names to Java classes
    * @return a <code>java.lang.Object</code> holding the OUT parameter value
    * @exception SQLException if a database access error occurs
    * @see #setObject
    * @since 1.2
    */
   public Object  getObject (int i, java.util.Map map) throws SQLException {
      return ( ((CallableStatement)statement).getObject ( i, map ) ) ;
   }

   /**
    * Retrieves the value of the designated JDBC <code>REF(&lt;structured-type&gt;)</code>
    * parameter as a {@link Ref} object in the Java programming language.
    * @param i the first parameter is 1, the second is 2, 
    * and so on
    * @return the parameter value as a <code>Ref</code> object in the
    * Java programming language.  If the value was SQL <code>NULL</code>, the value
    * <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public Ref getRef (int i) throws SQLException {
      return ( ((CallableStatement)statement).getRef ( i ) ) ;
   }

   /**
    * Retrieves the value of the designated JDBC <code>BLOB</code> parameter as a
    * {@link Blob} object in the Java programming language.
    * @param i the first parameter is 1, the second is 2, and so on
    * @return the parameter value as a <code>Blob</code> object in the
    * Java programming language.  If the value was SQL <code>NULL</code>, the value
    * <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public Blob getBlob (int i) throws SQLException {
      return ( ((CallableStatement)statement).getBlob ( i ) ) ;
   }

   /**
    * Retrieves the value of the designated JDBC <code>CLOB</code> parameter as a
    * <code>Clob</code> object in the Java programming language.
    * @param i the first parameter is 1, the second is 2, and
    * so on
    * @return the parameter value as a <code>Clob</code> object in the
    * Java programming language.  If the value was SQL <code>NULL</code>, the
    * value <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public Clob getClob (int i) throws SQLException {
      return ( ((CallableStatement)statement).getClob ( i ) ) ;
   }

   /**
    *
    * Retrieves the value of the designated JDBC <code>ARRAY</code> parameter as an
    * {@link Array} object in the Java programming language.
    * @param i the first parameter is 1, the second is 2, and 
    * so on
    * @return the parameter value as an <code>Array</code> object in
    * the Java programming language.  If the value was SQL <code>NULL</code>, the
    * value <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public Array getArray (int i) throws SQLException {
      return ( ((CallableStatement)statement).getArray(i) ) ;
   }

   /**
    * Retrieves the value of the designated JDBC <code>DATE</code> parameter as a 
    * <code>java.sql.Date</code> object, using
    * the given <code>Calendar</code> object
    * to construct the date.
    * With a <code>Calendar</code> object, the driver
    * can calculate the date taking into account a custom timezone and locale.
    * If no <code>Calendar</code> object is specified, the driver uses the
    * default timezone and locale.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, 
    * and so on
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the date
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    *         is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setDate
    * @since 1.2
    */
   public java.sql.Date getDate(int parameterIndex, Calendar cal) 
   throws SQLException {
      return ( ((CallableStatement)statement).getDate ( parameterIndex, cal ) );
   }

   /**
    * Retrieves the value of the designated JDBC <code>TIME</code> parameter as a 
    * <code>java.sql.Time</code> object, using
    * the given <code>Calendar</code> object
    * to construct the time.
    * With a <code>Calendar</code> object, the driver
    * can calculate the time taking into account a custom timezone and locale.
    * If no <code>Calendar</code> object is specified, the driver uses the
    * default timezone and locale.
    *
    * @param parameterIndex the first parameter is 1, the second is 2,
    * and so on
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the time
    * @return the parameter value; if the value is SQL <code>NULL</code>, the result 
    *         is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setTime
    * @since 1.2
    */
   public java.sql.Time getTime(int parameterIndex, Calendar cal) 
   throws SQLException {
      return ( ((CallableStatement)statement).getTime ( parameterIndex, cal ) );
   }

   /**
    * Retrieves the value of the designated JDBC <code>TIMESTAMP</code> parameter as a
    * <code>java.sql.Timestamp</code> object, using
    * the given <code>Calendar</code> object to construct
    * the <code>Timestamp</code> object.
    * With a <code>Calendar</code> object, the driver
    * can calculate the timestamp taking into account a custom timezone and locale.
    * If no <code>Calendar</code> object is specified, the driver uses the
    * default timezone and locale.
    *
    *
    * @param parameterIndex the first parameter is 1, the second is 2, 
    * and so on
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the timestamp
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    *         is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setTimestamp
    * @since 1.2
    */
   public java.sql.Timestamp getTimestamp(int parameterIndex, Calendar cal) 
   throws SQLException {
      return ( ((CallableStatement)statement).getTimestamp ( parameterIndex, cal ) ) ;
   }


   /**
    * Registers the designated output parameter.  This version of 
    * the method <code>registerOutParameter</code>
    * should be used for a user-defined or <code>REF</code> output parameter.  Examples
    * of user-defined types include: <code>STRUCT</code>, <code>DISTINCT</code>,
    * <code>JAVA_OBJECT</code>, and named array types.
    *
    * Before executing a stored procedure call, you must explicitly
    * call <code>registerOutParameter</code> to register the type from
    * <code>java.sql.Types</code> for each
    * OUT parameter.  For a user-defined parameter, the fully-qualified SQL
    * type name of the parameter should also be given, while a <code>REF</code>
    * parameter requires that the fully-qualified type name of the
    * referenced type be given.  A JDBC driver that does not need the
    * type code and type name information may ignore it.   To be portable,
    * however, applications should always provide these values for
    * user-defined and <code>REF</code> parameters.
    *
    * Although it is intended for user-defined and <code>REF</code> parameters,
    * this method may be used to register a parameter of any JDBC type.
    * If the parameter does not have a user-defined or <code>REF</code> type, the
    * <i>typeName</i> parameter is ignored.
    *
    * <P><B>Note:</B> When reading the value of an out parameter, you
    * must use the getter method whose Java type corresponds to the
    * parameter's registered SQL type.
    *
    * @param paramIndex the first parameter is 1, the second is 2,...
    * @param sqlType a value from {@link java.sql.Types}
    * @param typeName the fully-qualified name of an SQL structured type
    * @exception SQLException if a database access error occurs
    * @see Types
    * @since 1.2
    */
   public void registerOutParameter (int paramIndex, int sqlType, String typeName)
   throws SQLException {
      ((CallableStatement)statement).registerOutParameter ( paramIndex, sqlType, typeName ) ;
   }

   //--------------------------JDBC 3.0-----------------------------

   /**
    * Registers the OUT parameter named 
    * <code>parameterName</code> to the JDBC type 
    * <code>sqlType</code>.  All OUT parameters must be registered
    * before a stored procedure is executed.
    * <p>
    * The JDBC type specified by <code>sqlType</code> for an OUT
    * parameter determines the Java type that must be used
    * in the <code>get</code> method to read the value of that parameter.
    * <p>
    * If the JDBC type expected to be returned to this output parameter
    * is specific to this particular database, <code>sqlType</code>
    * should be <code>java.sql.Types.OTHER</code>.  The method 
    * {@link #getObject} retrieves the value.
    * @param parameterName the name of the parameter
    * @param sqlType the JDBC type code defined by <code>java.sql.Types</code>.
    * If the parameter is of JDBC type <code>NUMERIC</code>
    * or <code>DECIMAL</code>, the version of
    * <code>registerOutParameter</code> that accepts a scale value 
    * should be used.
    * @exception SQLException if a database access error occurs
    * @since 1.4
    * @see Types 
    */
   public void registerOutParameter(String parameterName, int sqlType)
   throws SQLException {
      ((CallableStatement)statement).registerOutParameter ( parameterName, sqlType ) ;
   }

   /**
    * Registers the parameter named 
    * <code>parameterName</code> to be of JDBC type
    * <code>sqlType</code>.  This method must be called
    * before a stored procedure is executed.
    * <p>
    * The JDBC type specified by <code>sqlType</code> for an OUT
    * parameter determines the Java type that must be used
    * in the <code>get</code> method to read the value of that parameter.
    * <p>
    * This version of <code>registerOutParameter</code> should be
    * used when the parameter is of JDBC type <code>NUMERIC</code>
    * or <code>DECIMAL</code>.
    * @param parameterName the name of the parameter
    * @param sqlType SQL type code defined by <code>java.sql.Types</code>.
    * @param scale the desired number of digits to the right of the
    * decimal point.  It must be greater than or equal to zero.
    * @exception SQLException if a database access error occurs
    * @since 1.4
    * @see Types 
    */
   public void registerOutParameter(String parameterName, int sqlType, int scale)
   throws SQLException {
      ((CallableStatement)statement).registerOutParameter ( parameterName, sqlType, scale ) ;
   }

   /**
    * Registers the designated output parameter.  This version of 
    * the method <code>registerOutParameter</code>
    * should be used for a user-named or REF output parameter.  Examples
    * of user-named types include: STRUCT, DISTINCT, JAVA_OBJECT, and
    * named array types.
    *
    * Before executing a stored procedure call, you must explicitly
    * call <code>registerOutParameter</code> to register the type from
    * <code>java.sql.Types</code> for each
    * OUT parameter.  For a user-named parameter the fully-qualified SQL
    * type name of the parameter should also be given, while a REF
    * parameter requires that the fully-qualified type name of the
    * referenced type be given.  A JDBC driver that does not need the
    * type code and type name information may ignore it.   To be portable,
    * however, applications should always provide these values for
    * user-named and REF parameters.
    *
    * Although it is intended for user-named and REF parameters,
    * this method may be used to register a parameter of any JDBC type.
    * If the parameter does not have a user-named or REF type, the
    * typeName parameter is ignored.
    *
    * <P><B>Note:</B> When reading the value of an out parameter, you
    * must use the <code>getXXX</code> method whose Java type XXX corresponds to the
    * parameter's registered SQL type.
    *
    * @param parameterName the name of the parameter
    * @param sqlType a value from {@link java.sql.Types}
    * @param typeName the fully-qualified name of an SQL structured type
    * @exception SQLException if a database access error occurs
    * @see Types
    * @since 1.4
    */
   public void registerOutParameter (String parameterName, int sqlType, String typeName)
   throws SQLException {
      ((CallableStatement)statement).registerOutParameter ( parameterName, sqlType, typeName ) ;
   }
   /**
    * Retrieves the value of the designated JDBC <code>DATALINK</code> parameter as a
    * <code>java.net.URL</code> object.
    * 
    * @param parameterIndex the first parameter is 1, the second is 2,...
    * @return a <code>java.net.URL</code> object that represents the 
    *         JDBC <code>DATALINK</code> value used as the designated
    *         parameter
    * @exception SQLException if a database access error occurs
    * @exception java.net.MalformedURLException if the URL being returned is
    *            not a valid URL on the Java platform
    * @see #setURL
    * @since 1.4
    */
   public java.net.URL getURL(int parameterIndex) throws SQLException, 
   java.net.MalformedURLException {
      return ( ((CallableStatement)statement).getURL ( parameterIndex ) ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.net.URL</code> object.
    * The driver converts this to an SQL <code>DATALINK</code> value when
    * it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param val the parameter value
    * @exception SQLException if a database access error occurs
    * @exception java.net.MalformedURLException if a URL is malformed
    * @see #getURL
    * @since 1.4
    */
   public void setURL(String parameterName, java.net.URL val) throws SQLException, 
   java.net.MalformedURLException {
      ((CallableStatement)statement).setURL ( parameterName, val ) ;
   }

   /**
    * Sets the designated parameter to SQL <code>NULL</code>.
    *
    * <P><B>Note:</B> You must specify the parameter's SQL type.
    *
    * @param parameterName the name of the parameter
    * @param sqlType the SQL type code defined in <code>java.sql.Types</code>
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public void setNull(String parameterName, int sqlType) throws SQLException {
      ((CallableStatement)statement).setNull ( parameterName, sqlType ) ;
   }

   /**
    * Sets the designated parameter to the given Java <code>boolean</code> value.
    * The driver converts this
    * to an SQL <code>BIT</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getBoolean
    * @since 1.4
    */
   public void setBoolean(String parameterName, boolean x) throws SQLException {
      ((CallableStatement)statement).setBoolean ( parameterName, x  );
   }

   /**
    * Sets the designated parameter to the given Java <code>byte</code> value.  
    * The driver converts this
    * to an SQL <code>TINYINT</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getByte
    * @since 1.4
    */
   public void setByte(String parameterName, byte x) throws SQLException {
      ((CallableStatement)statement).setByte ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given Java <code>short</code> value. 
    * The driver converts this
    * to an SQL <code>SMALLINT</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getShort
    * @since 1.4
    */
   public void setShort(String parameterName, short x) throws SQLException {
      ((CallableStatement)statement).setShort ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given Java <code>int</code> value.  
    * The driver converts this
    * to an SQL <code>INTEGER</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getInt
    * @since 1.4
    */
   public void setInt(String parameterName, int x) throws SQLException {
      ((CallableStatement)statement).setInt ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given Java <code>long</code> value. 
    * The driver converts this
    * to an SQL <code>BIGINT</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getLong
    * @since 1.4
    */
   public void setLong(String parameterName, long x) throws SQLException {
      ((CallableStatement)statement).setLong ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given Java <code>float</code> value. 
    * The driver converts this
    * to an SQL <code>FLOAT</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getFloat
    * @since 1.4
    */
   public void setFloat(String parameterName, float x) throws SQLException {
      ((CallableStatement)statement).setFloat ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given Java <code>double</code> value.  
    * The driver converts this
    * to an SQL <code>DOUBLE</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getDouble
    * @since 1.4
    */
   public void setDouble(String parameterName, double x) throws SQLException {
      ((CallableStatement)statement).setDouble ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given
    * <code>java.math.BigDecimal</code> value.  
    * The driver converts this to an SQL <code>NUMERIC</code> value when
    * it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getBigDecimal
    * @since 1.4
    */
   public void setBigDecimal(String parameterName, BigDecimal x) throws SQLException {
      ((CallableStatement)statement).setBigDecimal ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given Java <code>String</code> value. 
    * The driver converts this
    * to an SQL <code>VARCHAR</code> or <code>LONGVARCHAR</code> value
    * (depending on the argument's
    * size relative to the driver's limits on <code>VARCHAR</code> values)
    * when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getString
    * @since 1.4
    */
   public void setString(String parameterName, String x) throws SQLException {
      ((CallableStatement)statement).setString ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given Java array of bytes.  
    * The driver converts this to an SQL <code>VARBINARY</code> or 
    * <code>LONGVARBINARY</code> (depending on the argument's size relative 
    * to the driver's limits on <code>VARBINARY</code> values) when it sends 
    * it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value 
    * @exception SQLException if a database access error occurs
    * @see #getBytes
    * @since 1.4
    */
   public void setBytes(String parameterName, byte x[]) throws SQLException {
      ((CallableStatement)statement).setBytes ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Date</code> value.  
    * The driver converts this
    * to an SQL <code>DATE</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getDate
    * @since 1.4
    */
   public void setDate(String parameterName, java.sql.Date x)
   throws SQLException {
      ((CallableStatement)statement).setDate ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Time</code> value.  
    * The driver converts this
    * to an SQL <code>TIME</code> value when it sends it to the database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @exception SQLException if a database access error occurs
    * @see #getTime
    * @since 1.4
    */
   public void setTime(String parameterName, java.sql.Time x) 
   throws SQLException {
      ((CallableStatement)statement).setTime ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Timestamp</code> value.  
    * The driver
    * converts this to an SQL <code>TIMESTAMP</code> value when it sends it to the
    * database.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value 
    * @exception SQLException if a database access error occurs
    * @see #getTimestamp
    * @since 1.4
    */
   public void setTimestamp(String parameterName, java.sql.Timestamp x)
   throws SQLException {
      ((CallableStatement)statement).setTimestamp ( parameterName, x ) ;
   }

   /**
    * Sets the designated parameter to the given input stream, which will have 
    * the specified number of bytes.
    * When a very large ASCII value is input to a <code>LONGVARCHAR</code>
    * parameter, it may be more practical to send it via a
    * <code>java.io.InputStream</code>. Data will be read from the stream
    * as needed until end-of-file is reached.  The JDBC driver will
    * do any necessary conversion from ASCII to the database char format.
    * 
    * <P><B>Note:</B> This stream object can either be a standard
    * Java stream object or your own subclass that implements the
    * standard interface.
    *
    * @param parameterName the name of the parameter
    * @param x the Java input stream that contains the ASCII parameter value
    * @param length the number of bytes in the stream 
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public void setAsciiStream(String parameterName, java.io.InputStream x, int length)
   throws SQLException {
      ((CallableStatement)statement).setAsciiStream ( parameterName, x, length ) ;
   }

   /**
    * Sets the designated parameter to the given input stream, which will have 
    * the specified number of bytes.
    * When a very large binary value is input to a <code>LONGVARBINARY</code>
    * parameter, it may be more practical to send it via a
    * <code>java.io.InputStream</code> object. The data will be read from the stream
    * as needed until end-of-file is reached.
    * 
    * <P><B>Note:</B> This stream object can either be a standard
    * Java stream object or your own subclass that implements the
    * standard interface.
    *
    * @param parameterName the name of the parameter
    * @param x the java input stream which contains the binary parameter value
    * @param length the number of bytes in the stream 
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public void setBinaryStream(String parameterName, java.io.InputStream x, 
                        int length) throws SQLException {
      ((CallableStatement)statement).setBinaryStream ( parameterName, x, length ) ;
   }

   /**
    * Sets the value of the designated parameter with the given object. The second
    * argument must be an object type; for integral values, the
    * <code>java.lang</code> equivalent objects should be used.
    *
    * <p>The given Java object will be converted to the given targetSqlType
    * before being sent to the database.
    *
    * If the object has a custom mapping (is of a class implementing the 
    * interface <code>SQLData</code>),
    * the JDBC driver should call the method <code>SQLData.writeSQL</code> to write it 
    * to the SQL data stream.
    * If, on the other hand, the object is of a class implementing
    * <code>Ref</code>, <code>Blob</code>, <code>Clob</code>, <code>Struct</code>, 
    * or <code>Array</code>, the driver should pass it to the database as a 
    * value of the corresponding SQL type.
    * <P>
    * Note that this method may be used to pass datatabase-
    * specific abstract data types. 
    *
    * @param parameterName the name of the parameter
    * @param x the object containing the input parameter value
    * @param targetSqlType the SQL type (as defined in java.sql.Types) to be 
    * sent to the database. The scale argument may further qualify this type.
    * @param scale for java.sql.Types.DECIMAL or java.sql.Types.NUMERIC types,
    *          this is the number of digits after the decimal point.  For all other
    *          types, this value will be ignored.
    * @exception SQLException if a database access error occurs
    * @see Types
    * @see #getObject
    * @since 1.4 
    */
   public void setObject(String parameterName, Object x, int targetSqlType, int scale)
   throws SQLException {
      ((CallableStatement)statement).setObject ( parameterName, x, targetSqlType, scale ) ;
   }

   /**
    * Sets the value of the designated parameter with the given object.
    * This method is like the method <code>setObject</code>
    * above, except that it assumes a scale of zero.
    *
    * @param parameterName the name of the parameter
    * @param x the object containing the input parameter value
    * @param targetSqlType the SQL type (as defined in java.sql.Types) to be 
    *                      sent to the database
    * @exception SQLException if a database access error occurs
    * @see #getObject
    * @since 1.4
    */
   public void setObject(String parameterName, Object x, int targetSqlType) 
   throws SQLException {
      ((CallableStatement)statement).setObject ( parameterName, x, targetSqlType ) ;
   }

   /**
    * Sets the value of the designated parameter with the given object. 
    * The second parameter must be of type <code>Object</code>; therefore, the
    * <code>java.lang</code> equivalent objects should be used for built-in types.
    *
    * <p>The JDBC specification specifies a standard mapping from
    * Java <code>Object</code> types to SQL types.  The given argument 
    * will be converted to the corresponding SQL type before being
    * sent to the database.
    *
    * <p>Note that this method may be used to pass datatabase-
    * specific abstract data types, by using a driver-specific Java
    * type.
    *
    * If the object is of a class implementing the interface <code>SQLData</code>,
    * the JDBC driver should call the method <code>SQLData.writeSQL</code>
    * to write it to the SQL data stream.
    * If, on the other hand, the object is of a class implementing
    * <code>Ref</code>, <code>Blob</code>, <code>Clob</code>, <code>Struct</code>, 
    * or <code>Array</code>, the driver should pass it to the database as a 
    * value of the corresponding SQL type.
    * <P>
    * This method throws an exception if there is an ambiguity, for example, if the
    * object is of a class implementing more than one of the interfaces named above.
    *
    * @param parameterName the name of the parameter
    * @param x the object containing the input parameter value 
    * @exception SQLException if a database access error occurs or if the given
    *            <code>Object</code> parameter is ambiguous
    * @see #getObject
    * @since 1.4
    */
   public void setObject(String parameterName, Object x) throws SQLException {
      ((CallableStatement)statement).setObject ( parameterName, x ) ;
   }


   /**
    * Sets the designated parameter to the given <code>Reader</code>
    * object, which is the given number of characters long.
    * When a very large UNICODE value is input to a <code>LONGVARCHAR</code>
    * parameter, it may be more practical to send it via a
    * <code>java.io.Reader</code> object. The data will be read from the stream
    * as needed until end-of-file is reached.  The JDBC driver will
    * do any necessary conversion from UNICODE to the database char format.
    * 
    * <P><B>Note:</B> This stream object can either be a standard
    * Java stream object or your own subclass that implements the
    * standard interface.
    *
    * @param parameterName the name of the parameter
    * @param reader the <code>java.io.Reader</code> object that
    *        contains the UNICODE data used as the designated parameter
    * @param length the number of characters in the stream 
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public void setCharacterStream(String parameterName,
                           java.io.Reader reader,
                           int length) throws SQLException  {
      setCharacterStream ( parameterName, reader, length ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Date</code> value,
    * using the given <code>Calendar</code> object.  The driver uses
    * the <code>Calendar</code> object to construct an SQL <code>DATE</code> value,
    * which the driver then sends to the database.  With a
    * a <code>Calendar</code> object, the driver can calculate the date
    * taking into account a custom timezone.  If no
    * <code>Calendar</code> object is specified, the driver uses the default
    * timezone, which is that of the virtual machine running the application.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the date
    * @exception SQLException if a database access error occurs
    * @see #getDate
    * @since 1.4
    */
   public void setDate(String parameterName, java.sql.Date x, Calendar cal)
   throws SQLException{
      setDate ( parameterName, x, cal ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Time</code> value,
    * using the given <code>Calendar</code> object.  The driver uses
    * the <code>Calendar</code> object to construct an SQL <code>TIME</code> value,
    * which the driver then sends to the database.  With a
    * a <code>Calendar</code> object, the driver can calculate the time
    * taking into account a custom timezone.  If no
    * <code>Calendar</code> object is specified, the driver uses the default
    * timezone, which is that of the virtual machine running the application.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the time
    * @exception SQLException if a database access error occurs
    * @see #getTime
    * @since 1.4
    */
   public void setTime(String parameterName, java.sql.Time x, Calendar cal) 
   throws SQLException{
      setTime ( parameterName, x, cal ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Timestamp</code> value,
    * using the given <code>Calendar</code> object.  The driver uses
    * the <code>Calendar</code> object to construct an SQL <code>TIMESTAMP</code> value,
    * which the driver then sends to the database.  With a
    * a <code>Calendar</code> object, the driver can calculate the timestamp
    * taking into account a custom timezone.  If no
    * <code>Calendar</code> object is specified, the driver uses the default
    * timezone, which is that of the virtual machine running the application.
    *
    * @param parameterName the name of the parameter
    * @param x the parameter value 
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the timestamp
    * @exception SQLException if a database access error occurs
    * @see #getTimestamp
    * @since 1.4
    */
   public void setTimestamp(String parameterName, java.sql.Timestamp x, Calendar cal)
   throws SQLException {
      ((CallableStatement)statement).setTimestamp ( parameterName, x, cal ) ;
   }

   /**
    * Sets the designated parameter to SQL <code>NULL</code>.
    * This version of the method <code>setNull</code> should
    * be used for user-defined types and REF type parameters.  Examples
    * of user-defined types include: STRUCT, DISTINCT, JAVA_OBJECT, and 
    * named array types.
    *
    * <P><B>Note:</B> To be portable, applications must give the
    * SQL type code and the fully-qualified SQL type name when specifying
    * a NULL user-defined or REF parameter.  In the case of a user-defined type 
    * the name is the type name of the parameter itself.  For a REF 
    * parameter, the name is the type name of the referenced type.  If 
    * a JDBC driver does not need the type code or type name information, 
    * it may ignore it.     
    *
    * Although it is intended for user-defined and Ref parameters,
    * this method may be used to set a null parameter of any JDBC type.
    * If the parameter does not have a user-defined or REF type, the given
    * typeName is ignored.
    *
    *
    * @param paramIndex an <code>int</code> indicating which parameter 
    *        to set to SQL <code>NULL</code>; numbering starts at 1
    * @param sqlType a value from <code>java.sql.Types</code>
    * @param typeName the fully-qualified name of an SQL user-defined type;
    *        ignored if the parameter is not a user-defined type or 
    *        SQL <code>REF</code> value
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public void setNull (int paramIndex, int sqlType, String typeName) 
   throws SQLException{
      ((CallableStatement)statement).setNull ( paramIndex, sqlType, typeName )  ;
   }

   /**
    * Retrieves the value of a JDBC <code>CHAR</code>, <code>VARCHAR</code>, 
    * or <code>LONGVARCHAR</code> parameter as a <code>String</code> in 
    * the Java programming language.
    * <p>
    * For the fixed-length type JDBC <code>CHAR</code>,
    * the <code>String</code> object
    * returned has exactly the same value the JDBC
    * <code>CHAR</code> value had in the
    * database, including any padding added by the database.
    * @param parameterName the name of the parameter
    * @return the parameter value. If the value is SQL <code>NULL</code>, the result 
    * is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setString
    * @since 1.4
    */
   public String getString(String parameterName) throws SQLException{
      return( ((CallableStatement)statement).getString ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>BIT</code> parameter as a
    * <code>boolean</code> in the Java programming language.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    * is <code>false</code>.
    * @exception SQLException if a database access error occurs
    * @see #setBoolean
    * @since 1.4
    */
   public boolean getBoolean(String parameterName) throws SQLException{
      return( ((CallableStatement)statement).getBoolean ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>TINYINT</code> parameter as a <code>byte</code> 
    * in the Java programming language.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    * is <code>0</code>.
    * @exception SQLException if a database access error occurs
    * @see #setByte
    * @since 1.4
    */
   public byte getByte(String parameterName) throws SQLException {
      return(((CallableStatement)statement).getByte ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>SMALLINT</code> parameter as a <code>short</code>
    * in the Java programming language.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    * is <code>0</code>.
    * @exception SQLException if a database access error occurs
    * @see #setShort
    * @since 1.4
    */
   public short getShort(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getShort ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>INTEGER</code> parameter as an <code>int</code>
    * in the Java programming language.
    *
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, 
    *         the result is <code>0</code>.
    * @exception SQLException if a database access error occurs
    * @see #setInt
    * @since 1.4
    */
   public int getInt(String parameterName) throws SQLException{
      return( ((CallableStatement)statement).getInt ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>BIGINT</code> parameter as a <code>long</code>
    * in the Java programming language.
    *
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, 
    *         the result is <code>0</code>.
    * @exception SQLException if a database access error occurs
    * @see #setLong
    * @since 1.4
    */
   public long getLong(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getLong  ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>FLOAT</code> parameter as a <code>float</code>
    * in the Java programming language.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, 
    *         the result is <code>0</code>.
    * @exception SQLException if a database access error occurs
    * @see #setFloat
    * @since 1.4
    */
   public float getFloat(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getFloat ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>DOUBLE</code> parameter as a <code>double</code>
    * in the Java programming language.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, 
    *         the result is <code>0</code>.
    * @exception SQLException if a database access error occurs
    * @see #setDouble
    * @since 1.4
    */
   public double getDouble(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getDouble ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>BINARY</code> or <code>VARBINARY</code> 
    * parameter as an array of <code>byte</code> values in the Java
    * programming language.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result is 
    *  <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setBytes
    * @since 1.4
    */
   public byte[] getBytes(String parameterName) throws SQLException { 
      return( getBytes ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>DATE</code> parameter as a 
    * <code>java.sql.Date</code> object.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    * is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setDate
    * @since 1.4
    */
   public java.sql.Date getDate(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getDate ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>TIME</code> parameter as a 
    * <code>java.sql.Time</code> object.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    * is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setTime
    * @since 1.4
    */
   public java.sql.Time getTime(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getTime ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>TIMESTAMP</code> parameter as a 
    * <code>java.sql.Timestamp</code> object.
    * @param parameterName the name of the parameter
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result 
    * is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setTimestamp
    * @since 1.4
    */
   public java.sql.Timestamp getTimestamp(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getTimestamp ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a parameter as an <code>Object</code> in the Java 
    * programming language. If the value is an SQL <code>NULL</code>, the 
    * driver returns a Java <code>null</code>.
    * <p>
    * This method returns a Java object whose type corresponds to the JDBC
    * type that was registered for this parameter using the method
    * <code>registerOutParameter</code>.  By registering the target JDBC
    * type as <code>java.sql.Types.OTHER</code>, this method can be used
    * to read database-specific abstract data types.
    * @param parameterName the name of the parameter
    * @return A <code>java.lang.Object</code> holding the OUT parameter value.
    * @exception SQLException if a database access error occurs
    * @see Types
    * @see #setObject
    * @since 1.4
    */
   public Object getObject(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getObject ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>NUMERIC</code> parameter as a 
    * <code>java.math.BigDecimal</code> object with as many digits to the
    * right of the decimal point as the value contains.
    * @param parameterName the name of the parameter
    * @return the parameter value in full precision.  If the value is 
    * SQL <code>NULL</code>, the result is <code>null</code>. 
    * @exception SQLException if a database access error occurs
    * @see #setBigDecimal
    * @since 1.4
    */
   public BigDecimal getBigDecimal(String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getBigDecimal ( parameterName ) ) ;
   }

   /**
    * Returns an object representing the value of OUT parameter 
    * <code>i</code> and uses <code>map</code> for the custom
    * mapping of the parameter value.
    * <p>
    * This method returns a Java object whose type corresponds to the
    * JDBC type that was registered for this parameter using the method
    * <code>registerOutParameter</code>.  By registering the target
    * JDBC type as <code>java.sql.Types.OTHER</code>, this method can
    * be used to read database-specific abstract data types.  
    * @param parameterName the name of the parameter
    * @param map the mapping from SQL type names to Java classes
    * @return a <code>java.lang.Object</code> holding the OUT parameter value
    * @exception SQLException if a database access error occurs
    * @see #setObject
    * @since 1.4
    */
   public Object  getObject (String parameterName, java.util.Map map) throws SQLException {
      return( ((CallableStatement)statement).getObject ( parameterName, map ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>REF(&lt;structured-type&gt;)</code>
    * parameter as a {@link Ref} object in the Java programming language.
    *
    * @param parameterName the name of the parameter
    * @return the parameter value as a <code>Ref</code> object in the
    *         Java programming language.  If the value was SQL <code>NULL</code>, 
    *         the value <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public Ref getRef (String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getRef ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>BLOB</code> parameter as a
    * {@link Blob} object in the Java programming language.
    *
    * @param parameterName the name of the parameter
    * @return the parameter value as a <code>Blob</code> object in the
    *         Java programming language.  If the value was SQL <code>NULL</code>, 
    *         the value <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public Blob getBlob (String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getBlob ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>CLOB</code> parameter as a
    * <code>Clob</code> object in the Java programming language.
    * @param parameterName the name of the parameter
    * @return the parameter value as a <code>Clob</code> object in the
    *         Java programming language.  If the value was SQL <code>NULL</code>, 
    *         the value <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public Clob getClob (String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getClob ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>ARRAY</code> parameter as an
    * {@link Array} object in the Java programming language.
    *
    * @param parameterName the name of the parameter
    * @return the parameter value as an <code>Array</code> object in
    *         Java programming language.  If the value was SQL <code>NULL</code>, 
    *         the value <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */
   public Array getArray (String parameterName) throws SQLException {
      return( ((CallableStatement)statement).getArray ( parameterName ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>DATE</code> parameter as a 
    * <code>java.sql.Date</code> object, using
    * the given <code>Calendar</code> object
    * to construct the date.
    * With a <code>Calendar</code> object, the driver
    * can calculate the date taking into account a custom timezone and locale.
    * If no <code>Calendar</code> object is specified, the driver uses the
    * default timezone and locale.
    *
    * @param parameterName the name of the parameter
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the date
    * @return the parameter value.  If the value is SQL <code>NULL</code>, 
    * the result is <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setDate
    * @since 1.4
    */
   public java.sql.Date getDate(String parameterName, Calendar cal) 
   throws SQLException {
      return( ((CallableStatement)statement).getDate ( parameterName, cal ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>TIME</code> parameter as a 
    * <code>java.sql.Time</code> object, using
    * the given <code>Calendar</code> object
    * to construct the time.
    * With a <code>Calendar</code> object, the driver
    * can calculate the time taking into account a custom timezone and locale.
    * If no <code>Calendar</code> object is specified, the driver uses the
    * default timezone and locale.
    *
    * @param parameterName the name of the parameter
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the time
    * @return the parameter value; if the value is SQL <code>NULL</code>, the result is 
    * <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setTime
    * @since 1.4
    */
   public java.sql.Time getTime(String parameterName, Calendar cal) 
   throws SQLException {
      return( ((CallableStatement)statement).getTime ( parameterName, cal ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>TIMESTAMP</code> parameter as a
    * <code>java.sql.Timestamp</code> object, using
    * the given <code>Calendar</code> object to construct
    * the <code>Timestamp</code> object.
    * With a <code>Calendar</code> object, the driver
    * can calculate the timestamp taking into account a custom timezone and locale.
    * If no <code>Calendar</code> object is specified, the driver uses the
    * default timezone and locale.
    *
    *
    * @param parameterName the name of the parameter
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the timestamp
    * @return the parameter value.  If the value is SQL <code>NULL</code>, the result is 
    * <code>null</code>.
    * @exception SQLException if a database access error occurs
    * @see #setTimestamp
    * @since 1.4
    */
   public java.sql.Timestamp getTimestamp(String parameterName, Calendar cal) 
   throws SQLException {
      return( ((CallableStatement)statement).getTimestamp ( parameterName, cal ) ) ;
   }

   /**
    * Retrieves the value of a JDBC <code>DATALINK</code> parameter as a
    * <code>java.net.URL</code> object.
    *
    * @param parameterName the name of the parameter
    * @return the parameter value as a <code>java.net.URL</code> object in the
    * Java programming language.  If the value was SQL <code>NULL</code>, the
    * value <code>null</code> is returned.
    * @exception SQLException if a database access error occurs
    * @exception java.net.MalformedURLException if there is a problem with the URL
    * @see #setURL
    * @since 1.4
    */
   public java.net.URL getURL(String parameterName) throws SQLException, 
   java.net.MalformedURLException {
      return( ((CallableStatement)statement).getURL ( parameterName ) ) ;
   }

}
