package com.ibm.nzna.shared.db ;

import java.sql.* ;   
import java.math.BigDecimal;
import java.util.Calendar;

/**
 * <P>A SQL statement is pre-compiled and stored in a
 * PreparedStatement object. This object can then be used to
 * efficiently execute this statement multiple times. 
 *
 * <P><B>Note:</B> The setXXX methods for setting IN parameter values
 * must specify types that are compatible with the defined SQL type of
 * the input parameter. For instance, if the IN parameter has SQL type
 * Integer then setInt should be used.
 *
 * <p>If arbitrary parameter type conversions are required then the
 * setObject method should be used with a target SQL type.
 *
 * @see Connection#prepareStatement
 * @see ResultSet 
 */

public class QuestPreparedStatement extends QuestStatement implements PreparedStatement {
   PreparedStatement pStatement = null ;

   public QuestPreparedStatement ( PreparedStatement statement ) {
      super( statement ) ;
      this.pStatement = statement ;
   }
   /**
    * A prepared SQL query is executed and its ResultSet is returned.
    *
    * @return a ResultSet that contains the data produced by the
    * query; never null
    * @exception SQLException if a database-access error occurs.
    */
   public ResultSet executeQuery() throws SQLException {
      ResultSet rs = null ;

      rs = (( PreparedStatement )statement).executeQuery() ;

      return( rs ) ;
   }

   /**
    * Execute a SQL INSERT, UPDATE or DELETE (( PreparedStatement ) statement). In addition,
    * SQL (( PreparedStatement ) statement)s that return nothing such as SQL DDL (( PreparedStatement ) statement)s
    * can be executed.
    *
    * @return either the row count for INSERT, UPDATE or DELETE; or 0
    * for SQL (( PreparedStatement ) statement)s that return nothing
    * @exception SQLException if a database-access error occurs.
    */
   public int executeUpdate() throws SQLException {
      return( (( PreparedStatement ) statement).executeUpdate() ) ;
   }

   /**
    * Set a parameter to SQL NULL.
    *
    * <P><B>Note:</B> You must specify the parameter's SQL type.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param sqlType SQL type code defined by java.sql.Types
    * @exception SQLException if a database-access error occurs.
    */
   public void setNull(int parameterIndex, int sqlType) throws SQLException {
      (( PreparedStatement ) statement).setNull( parameterIndex, sqlType) ;
   }

   /**
    * Set a parameter to a Java boolean value.  The driver converts this
    * to a SQL BIT value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setBoolean(int parameterIndex, boolean x) throws SQLException {
      (( PreparedStatement ) statement).setBoolean( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java byte value.  The driver converts this
    * to a SQL TINYINT value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setByte(int parameterIndex, byte x) throws SQLException {
      (( PreparedStatement ) statement).setByte( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java short value.  The driver converts this
    * to a SQL SMALLINT value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setShort(int parameterIndex, short x) throws SQLException {
      (( PreparedStatement ) statement).setShort( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java int value.  The driver converts this
    * to a SQL INTEGER value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setInt(int parameterIndex, int x) throws SQLException {
      (( PreparedStatement ) statement).setInt( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java long value.  The driver converts this
    * to a SQL BIGINT value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setLong(int parameterIndex, long x) throws SQLException {
      (( PreparedStatement ) statement).setLong( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java float value.  The driver converts this
    * to a SQL FLOAT value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setFloat(int parameterIndex, float x) throws SQLException {
      (( PreparedStatement ) statement).setFloat( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java double value.  The driver converts this
    * to a SQL DOUBLE value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setDouble(int parameterIndex, double x) throws SQLException {
      (( PreparedStatement ) statement).setDouble( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a java.lang.BigDecimal value.  
    * The driver converts this to a SQL NUMERIC value when
    * it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setBigDecimal(int parameterIndex, BigDecimal x) throws SQLException {
      (( PreparedStatement ) statement).setBigDecimal( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java String value.  The driver converts this
    * to a SQL VARCHAR or LONGVARCHAR value (depending on the arguments
    * size relative to the driver's limits on VARCHARs) when it sends
    * it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setString(int parameterIndex, String x) throws SQLException {
      (( PreparedStatement ) statement).setString( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a Java array of bytes.  The driver converts
    * this to a SQL VARBINARY or LONGVARBINARY (depending on the
    * argument's size relative to the driver's limits on VARBINARYs)
    * when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value 
    * @exception SQLException if a database-access error occurs.
    */
   public void setBytes(int parameterIndex, byte x[]) throws SQLException {
      (( PreparedStatement ) statement).setBytes( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a java.sql.Date value.  The driver converts this
    * to a SQL DATE value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setDate(int parameterIndex, java.sql.Date x) throws SQLException {
      (( PreparedStatement ) statement).setDate( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a java.sql.Time value.  The driver converts this
    * to a SQL TIME value when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @exception SQLException if a database-access error occurs.
    */
   public void setTime(int parameterIndex, java.sql.Time x) throws SQLException {
      (( PreparedStatement ) statement).setTime( parameterIndex, x) ;
   }

   /**
    * Set a parameter to a java.sql.Timestamp value.  The driver
    * converts this to a SQL TIMESTAMP value when it sends it to the
    * database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value 
    * @exception SQLException if a database-access error occurs.
    */
   public void setTimestamp(int parameterIndex, java.sql.Timestamp x) throws SQLException {
      (( PreparedStatement ) statement).setTimestamp( parameterIndex, x) ;
   }

   /**
    * When a very large ASCII value is input to a LONGVARCHAR
    * parameter, it may be more practical to send it via a
    * java.io.InputStream. JDBC will read the data from the stream
    * as needed, until it reaches end-of-file.  The JDBC driver will
    * do any necessary conversion from ASCII to the database char format.
    * 
    * <P><B>Note:</B> This stream object can either be a standard
    * Java stream object or your own subclass that implements the
    * standard interface.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the java input stream which contains the ASCII parameter value
    * @param length the number of bytes in the stream 
    * @exception SQLException if a database-access error occurs.
    */
   public void setAsciiStream(int parameterIndex, java.io.InputStream x, int length) throws SQLException {
      (( PreparedStatement ) statement).setAsciiStream( parameterIndex, x, length ) ;
   }

   /**
    * When a very large UNICODE value is input to a LONGVARCHAR
    * parameter, it may be more practical to send it via a
    * java.io.InputStream. JDBC will read the data from the stream
    * as needed, until it reaches end-of-file.  The JDBC driver will
    * do any necessary conversion from UNICODE to the database char format.
    * 
    * <P><B>Note:</B> This stream object can either be a standard
    * Java stream object or your own subclass that implements the
    * standard interface.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...  
    * @param x the java input stream which contains the
    * UNICODE parameter value 
    * @param length the number of bytes in the stream 
    * @exception SQLException if a database-access error occurs.
    * @deprecated
    */
   public void setUnicodeStream(int parameterIndex, java.io.InputStream x, int length) throws SQLException {
      (( PreparedStatement ) statement).setUnicodeStream( parameterIndex, x, length) ;
   }

   /**
    * When a very large binary value is input to a LONGVARBINARY
    * parameter, it may be more practical to send it via a
    * java.io.InputStream. JDBC will read the data from the stream
    * as needed, until it reaches end-of-file.
    * 
    * <P><B>Note:</B> This stream object can either be a standard
    * Java stream object or your own subclass that implements the
    * standard interface.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the java input stream which contains the binary parameter value
    * @param length the number of bytes in the stream 
    * @exception SQLException if a database-access error occurs.
    */
   public void setBinaryStream(int parameterIndex, java.io.InputStream x, int length) throws SQLException {
      (( PreparedStatement ) statement).setBinaryStream( parameterIndex, x, length ) ;
   }

   /**
    * <P>In general, parameter values remain in force for repeated use of a
    * Statement. Setting a parameter value automatically clears its
    * previous value.  However, in some cases it is useful to immediately
    * release the resources used by the current parameter values; this can
    * be done by calling clearParameters.
    *
    * @exception SQLException if a database-access error occurs.
    */
   public void clearParameters() throws SQLException {
      (( PreparedStatement ) statement).clearParameters() ;
   }

   //----------------------------------------------------------------------
   // Advanced features:

   /**
    * <p>Set the value of a parameter using an object; use the
    * java.lang equivalent objects for integral values.
    *
    * <p>The given Java object will be converted to the targetSqlType
    * before being sent to the database.
    *
    * <p>Note that this method may be used to pass datatabase-
    * specific abstract data types. This is done by using a Driver-
    * specific Java type and using a targetSqlType of
    * java.sql.types.OTHER.
    *
    * @param parameterIndex The first parameter is 1, the second is 2, ...
    * @param x The object containing the input parameter value
    * @param targetSqlType The SQL type (as defined in java.sql.Types) to be 
    * sent to the database. The scale argument may further qualify this type.
    * @param scale For java.sql.Types.DECIMAL or java.sql.Types.NUMERIC types
    *          this is the number of digits after the decimal.  For all other
    *          types this value will be ignored,
    * @exception SQLException if a database-access error occurs.
    * @see Types 
    */
   public void setObject(int parameterIndex, Object x, int targetSqlType, int scale) throws SQLException {
      (( PreparedStatement ) statement).setObject( parameterIndex, x, targetSqlType, scale ) ;
   }

   /**
     * This method is like setObject above, but assumes a scale of zero.
     *
     * @exception SQLException if a database-access error occurs.
     */
   public void setObject(int parameterIndex, Object x, int targetSqlType) throws SQLException {
      (( PreparedStatement ) statement).setObject( parameterIndex, x, targetSqlType ) ;
   }

   /**
    * <p>Set the value of a parameter using an object; use the
    * java.lang equivalent objects for integral values.
    *
    * <p>The JDBC specification specifies a standard mapping from
    * Java Object types to SQL types.  The given argument java object
    * will be converted to the corresponding SQL type before being
    * sent to the database.
    *
    * <p>Note that this method may be used to pass datatabase
    * specific abstract data types, by using a Driver specific Java
    * type.
    *
    * @param parameterIndex The first parameter is 1, the second is 2, ...
    * @param x The object containing the input parameter value 
    * @exception SQLException if a database-access error occurs.
    */
   public void setObject(int parameterIndex, Object x) throws SQLException {
      (( PreparedStatement ) statement).setObject( parameterIndex, x) ;
   }

   /**
    * Some prepared (( PreparedStatement ) statement)s return multiple results; the execute
    * method handles these complex (( PreparedStatement ) statement)s as well as the simpler
    * form of (( PreparedStatement ) statement)s handled by executeQuery and executeUpdate.
    *
    * @exception SQLException if a database-access error occurs.
    * @see Statement#execute
    */
   public boolean execute() throws SQLException {
      boolean   rc     = false ;

      rc = (( PreparedStatement ) statement).execute() ;

      return( rc ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.net.URL</code> value. 
    * The driver converts this to an SQL <code>DATALINK</code> value
    * when it sends it to the database.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the <code>java.net.URL</code> object to be set
    * @exception SQLException if a database access error occurs
    * @since 1.4
    */ 
   public void setURL(int parameterIndex, java.net.URL x) throws SQLException {
      pStatement.setURL ( parameterIndex, x ) ;
   }

   /**
    * Retrieves the number, types and properties of this 
    * <code>PreparedStatement</code> object's parameters.
    *
    * @return a <code>ParameterMetaData</code> object that contains information
    *         about the number, types and properties of this 
    *         <code>PreparedStatement</code> object's parameters
    * @exception SQLException if a database access error occurs
    * @see ParameterMetaData
    * @since 1.4
    */
   public ParameterMetaData getParameterMetaData() throws SQLException {
      return( pStatement.getParameterMetaData() ) ;
   }


   /**
    * Adds a set of parameters to this <code>PreparedStatement</code>
    * object's batch of commands.
    * 
    * @exception SQLException if a database access error occurs
    * @see Statement#addBatch
    * @since 1.2
    */
   public void addBatch() throws SQLException {
      pStatement.addBatch() ;
   }

   /**
    * Sets the designated parameter to the given <code>Reader</code>
    * object, which is the given number of characters long.
    * When a very large UNICODE value is input to a <code>LONGVARCHAR</code>
    * parameter, it may be more practical to send it via a
    * <code>java.io.Reader</code> object. The data will be read from the stream
    * as needed until end-of-file is reached.  The JDBC driver will
    * do any necessary conversion from UNICODE to the database char format.
    * 
    * <P><B>Note:</B> This stream object can either be a standard
    * Java stream object or your own subclass that implements the
    * standard interface.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param reader the <code>java.io.Reader</code> object that contains the 
    *        Unicode data
    * @param length the number of characters in the stream 
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setCharacterStream(int parameterIndex,
                java.io.Reader reader,
          int length) throws SQLException {
      pStatement.setCharacterStream ( parameterIndex, reader, length ) ;
   }

   /**
    * Sets the designated parameter to the given
    *  <code>REF(&lt;structured-type&gt;)</code> value.
    * The driver converts this to an SQL <code>REF</code> value when it
    * sends it to the database.
    *
    * @param i the first parameter is 1, the second is 2, ...
    * @param x an SQL <code>REF</code> value
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setRef (int i, Ref x) throws SQLException {
      pStatement.setRef ( i, x ) ;
   }

   /**
    * Sets the designated parameter to the given <code>Blob</code> object.
    * The driver converts this to an SQL <code>BLOB</code> value when it
    * sends it to the database.
    *
    * @param i the first parameter is 1, the second is 2, ...
    * @param x a <code>Blob</code> object that maps an SQL <code>BLOB</code> value
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setBlob (int i, Blob x) throws SQLException {
      pStatement.setBlob ( i, x ) ;
   }

   /**
    * Sets the designated parameter to the given <code>Clob</code> object.
    * The driver converts this to an SQL <code>CLOB</code> value when it
    * sends it to the database.
    *
    * @param i the first parameter is 1, the second is 2, ...
    * @param x a <code>Clob</code> object that maps an SQL <code>CLOB</code> value
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setClob (int i, Clob x) throws SQLException {
      pStatement.setClob ( i, x ) ;
   }

   /**
    * Sets the designated parameter to the given <code>Array</code> object.
    * The driver converts this to an SQL <code>ARRAY</code> value when it
    * sends it to the database.
    *
    * @param i the first parameter is 1, the second is 2, ...
    * @param x an <code>Array</code> object that maps an SQL <code>ARRAY</code> value
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setArray (int i, Array x) throws SQLException {
      pStatement.setArray ( i, x ) ;
   }

   /**
    * Retrieves a <code>ResultSetMetaData</code> object that contains
    * information about the columns of the <code>ResultSet</code> object
    * that will be returned when this <code>PreparedStatement</code> object 
    * is executed.
    * <P>
    * Because a <code>PreparedStatement</code> object is precompiled, it is
    * possible to know about the <code>ResultSet</code> object that it will
    * return without having to execute it.  Consequently, it is possible
    * to invoke the method <code>getMetaData</code> on a
    * <code>PreparedStatement</code> object rather than waiting to execute
    * it and then invoking the <code>ResultSet.getMetaData</code> method
    * on the <code>ResultSet</code> object that is returned.
    * <P>
    * <B>NOTE:</B> Using this method may be expensive for some drivers due
    * to the lack of underlying DBMS support.
    *
    * @return the description of a <code>ResultSet</code> object's columns or
    *         <code>null</code> if the driver cannot return a
    *         <code>ResultSetMetaData</code> object
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public ResultSetMetaData getMetaData() throws SQLException {
      return ( pStatement.getMetaData() ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Date</code> value,
    * using the given <code>Calendar</code> object.  The driver uses
    * the <code>Calendar</code> object to construct an SQL <code>DATE</code> value,
    * which the driver then sends to the database.  With a
    * a <code>Calendar</code> object, the driver can calculate the date
    * taking into account a custom timezone.  If no
    * <code>Calendar</code> object is specified, the driver uses the default
    * timezone, which is that of the virtual machine running the application.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the date
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setDate(int parameterIndex, java.sql.Date x, Calendar cal) throws SQLException {
      pStatement.setDate ( parameterIndex, x, cal ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Time</code> value,
    * using the given <code>Calendar</code> object.  The driver uses
    * the <code>Calendar</code> object to construct an SQL <code>TIME</code> value,
    * which the driver then sends to the database.  With a
    * a <code>Calendar</code> object, the driver can calculate the time
    * taking into account a custom timezone.  If no
    * <code>Calendar</code> object is specified, the driver uses the default
    * timezone, which is that of the virtual machine running the application.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the time
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setTime(int parameterIndex, java.sql.Time x, Calendar cal) throws SQLException {
      pStatement.setTime ( parameterIndex, x, cal ) ;
   }

   /**
    * Sets the designated parameter to the given <code>java.sql.Timestamp</code> value,
    * using the given <code>Calendar</code> object.  The driver uses
    * the <code>Calendar</code> object to construct an SQL <code>TIMESTAMP</code> value,
    * which the driver then sends to the database.  With a
    * a <code>Calendar</code> object, the driver can calculate the timestamp
    * taking into account a custom timezone.  If no
    * <code>Calendar</code> object is specified, the driver uses the default
    * timezone, which is that of the virtual machine running the application.
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
    * @param x the parameter value 
    * @param cal the <code>Calendar</code> object the driver will use
    *            to construct the timestamp
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
   public void setTimestamp(int parameterIndex, java.sql.Timestamp x, Calendar cal) throws SQLException {
      pStatement.setTimestamp ( parameterIndex, x, cal ) ;
   }

   /**
    * Sets the designated parameter to SQL <code>NULL</code>.
    * This version of the method <code>setNull</code> should
    * be used for user-defined types and REF type parameters.  Examples
    * of user-defined types include: STRUCT, DISTINCT, JAVA_OBJECT, and 
    * named array types.
    *
    * <P><B>Note:</B> To be portable, applications must give the
    * SQL type code and the fully-qualified SQL type name when specifying
    * a NULL user-defined or REF parameter.  In the case of a user-defined type 
    * the name is the type name of the parameter itself.  For a REF 
    * parameter, the name is the type name of the referenced type.  If 
    * a JDBC driver does not need the type code or type name information, 
    * it may ignore it.     
    *
    * Although it is intended for user-defined and Ref parameters,
    * this method may be used to set a null parameter of any JDBC type.
    * If the parameter does not have a user-defined or REF type, the given
    * typeName is ignored.
    *
    *
    * @param paramIndex the first parameter is 1, the second is 2, ...
    * @param sqlType a value from <code>java.sql.Types</code>
    * @param typeName the fully-qualified name of an SQL user-defined type;
    *  ignored if the parameter is not a user-defined type or REF 
    * @exception SQLException if a database access error occurs
    * @since 1.2
    */
 public void setNull (int paramIndex, int sqlType, String typeName) throws SQLException {
    pStatement.setNull ( paramIndex, sqlType, typeName ) ;
 }

}
