package com.ibm.nzna.shared.db ;   
/*-----------------------------------------------------------*/
/*   SQLMethod                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SQLMethod                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * The SQL Method is a normal method's interface to a database.
 * Anyone wishing to use SQL to an external database must use
 * this class to obtain Statement objects etc.. This information
 * is also used by the DatabaseSystem to keep track of what SQL
 * methods are running, etc..<br><br>
 *
 * <b>It is very important that you call the SQLMethod 'close()' method.</b>
 * <br><br>
 * If you are using a LogSystem to log messages, this class will use the same
 * log instance.
 *
 *
 * @version 1.0, 2/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SQLMethod                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class SQLMethod extends Object implements MultiListRow {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String            methodName           = null ;
   private int               runSeconds           = 0 ;
   private String            currentSQL           = null ;
   private int               databaseId           = -1 ;
   private String            databaseName         = null ;
   private String            userId               = null ;
   private Connection        connection           = null ;
   private Database          database             = null ;
   private CallableStatement callableStatement    = null ;
   private LogSystem         log                  = null ;
   private Vector            statements           = new Vector ( 1 )  ;
   private boolean           open                 = true ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the default
    *  SQLMethod initialized to the params passed. Please note
    *  that the database passed must already be connected. A new connection
    *  will not be made automatically. For databases like QUEST, a connection
    *  is made at application startup, so you don't need to worry.
    *  
    *
    *  @param database One of the preconnected databases in DatabaseSystem. ie DatabaseSystem.QUEST or DatabaseSystem.CLIO.
    *                  Please see DatabaseSystem.createConnection to see how to establish another connection.
    *  @param methodName The name of the SQL Method
    *  @param runSeconds The amount of time in seconds that this method should take to run
    *  @param log The LogSystem to use for error logging
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SQLMethod ( int databaseId, String methodName, int runSeconds ) {
      this.log         = LogSystem.getInstance() ;
      this.methodName  = methodName ;
      this.runSeconds  = runSeconds ;
      this.databaseId  = databaseId ;
      this.connection  = DatabaseSystem.getConnection ( databaseId ) ;
      this.database    = DatabaseSystem.getDatabaseFromConnection ( connection ) ;
      DatabaseSystem.addSQLMethod ( database, this ) ;
   }





   /*------------------------------------------------------------*/
   /* close
    */
   /**
    *    This method will close all cursors, commit
    *  and release all memory associated with
    *  the SQL Method. THIS METHOD MUST BE CALLED!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void close () {

      try {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Free up any cursors                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( statements != null ) && ( statements.size() > 0 ) ) {
            int i    = 0 ;
            int size = statements.size() ;

            while ( i < size ) {
               ((QuestStatement)statements.elementAt(i)).close() ;
               i++ ;
            }
         }

         else if ( callableStatement != null )
            callableStatement.close() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Commit the database                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( database != null )
            database.commit() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Tell the database that we are no longer running         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( database != null )
            DatabaseSystem.removeSQLMethod ( database, this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Release the connection                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( database != null )
            DatabaseSystem.releaseConnection ( database ) ;


         database  = null ;
         if ( statements != null ) {
            statements.removeAllElements() ;
            statements = null ;
         }
         open       = false ;
      } catch ( Exception e ) {
         if ( log != null )
            log.log ( 1, e, false ) ;
      }
   }

   /*------------------------------------------------------------*/
   /* createStatement
    */
   /**
    *    This method will create a Statement
    *  object that can be used to SELECT/INSERT/UPDATE/DELETE
    *  with. There will only be 1 Statement returned from a call.
    *  Any prior calls to this method will return the same statement
    *  object.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Statement createStatement ( ) {
      QuestStatement questStatement = null ;
      Statement      statement      = null ;

      if ( connection != null ) {
         try {

            statement = questStatement = new QuestStatement ( connection.createStatement() ) ;

            if ( ( statement != null ) && ( questStatement != null ) )
               statements.addElement ( questStatement ) ;

         } catch ( SQLException sqlE ) {
            DatabaseSystem.handleSQLException ( this, sqlE ) ;
            statement=null ;
            questStatement=null ;
         } catch ( Exception e ) {
            if ( log != null ) {
               LogSystem.log ( 1, "Cannot create Prepared Statement because connection to Database "+database.getName()+" is NULL" ) ;
               LogSystem.log ( 1, e, false ) ;
            }
            else
               e.printStackTrace() ;
            statement=null ;
            questStatement=null ;
         }
      }

      return( statement ) ;
   }





   /*------------------------------------------------------------*/
   /* createStoredProc
    */
   /**
    *    This method will create a stored
    *  procedure and returns the CallableStatement
    *  object for it
    *
    *  @param name The name of the Stored Procedure
    *  @param params The number of parameters in/out passed
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CallableStatement createStoredProc ( String name, int params ) {
      CallableStatement retStmt = null ;
      String            strStmt = constructDB2StoredProcCall ( name, params ) ;

      try {
         callableStatement = retStmt = new QuestCallableStatement ( connection.prepareCall ( strStmt ), name, params ) ;
      } catch ( Exception e ) {
         if ( log != null )
            LogSystem.log ( 1, e ) ;
         else
            e.printStackTrace() ;
      }


      return( retStmt ) ;
   }





   /*------------------------------------------------------------*/
   /* constructDB2StoredProcCall
    */
   /**
    *    This method will create the actual
    *  string that is passed to DB/2 for a 
    *  stored procedure
    *
    *  @param name The name of the stored proc.
    *  @param params The number of params
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String constructDB2StoredProcCall ( String name, int params ) {
      String retStr = "CALL "+name+" ( ";
      int    i      = 0 ;

      for ( i = 0; i < params; i++ ) {
         retStr = retStr+"?" ;
         if ( i != (params-1) )
            retStr = retStr+", " ;
      }

      if ( log != null )
         LogSystem.log ( 1, "SQLMethod Creating Stored Procedure Call:"+retStr+")" ) ;

      return( retStr+")" ) ;

   }




   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the data that
    *  should show in a specfic column
    *  
    *
    *  @param column The column to return
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = methodName ;
            break ;

         case 1 :
            retObj = ""+runSeconds ;
            break;
      }

      return( retObj ) ;
   }




   /*------------------------------------------------------------*/
   /* getDatabaseId
    */
   /**
    *    This method will return the database id
    *  that this method is running on
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getDatabaseId () {
      return( databaseId ) ;
   }





   /*------------------------------------------------------------*/
   /* getStatementSQL
    */
   /**
    *    This method will return that last run
    *  SQL from a Statement object created. Currently this is
    *  not supported.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getStatementSQL () {
      return( "Statement SQL not supported yet" ) ;
   }





   /*------------------------------------------------------------*/
   /* rollBack
    */
   /**
    *    This method will perform a rollback
    *  for the current SQL that was run
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void rollBack () {
      try {
         connection.rollback() ;
      } catch ( Exception e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* cleanDBString
    */
   /**
    *    This method will clean a string so that it is
    *  suitable to insert or update into a database
    *  
    *
    *  @param String the string to clean
    *  @param None
    *  @param None
    *
    *  @return The cleaned String
    *//*---------------------------------------------------------*/
   public String cleanDBString ( String string ) {
      String cleanStr = null ;

      if ( string != null ) {
         cleanStr = Text.replaceAllStrInStr ( string, "''", "'" ) ;
         cleanStr = Text.replaceAllStrInStr ( cleanStr, "'", "''" ) ;
      }

      return( cleanStr ) ;
   }





   /*------------------------------------------------------------*/
   /* createPreparedStatement
    */
   /**
    *    This method will create and return a prepared
    *  statement
    *  
    *
    *  @param sql The SQL string for the prepared statement
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PreparedStatement createPreparedStatement ( String sql ) {
      QuestPreparedStatement retStmt = null ;

      try {
         PreparedStatement stmt = connection.prepareStatement ( sql ) ;

         retStmt = new QuestPreparedStatement ( stmt ) ;

      } catch ( Exception e ) {
         if ( log != null ) {
            LogSystem.log ( 1, e ) ;
         }
         else
            e.printStackTrace() ;
      }

      /* return a quest prepared stament so we can trace the sql calls */
      return( retStmt ) ;

   }




   /*------------------------------------------------------------*/
   /* getDatabase
    */
   /**
    *    This method will return the database object
    *  we are working with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Database getDatabase () {
      return( database ) ;
   }




   /*------------------------------------------------------------*/
   /* isOpen
    */
   /**
    *    This method will return TRUE if this SQLMethod
    *  is still connected to a database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isOpen () {
      return( open ) ;
   }

   /**
     * This method will return the method name
     */
   protected String getMethodName () {
       return ( methodName ) ;
   }

   /**
     *  This method will return the actual connection that this SQLMethod is using. Use
     * this connection wisely as any methods called on it are not recorded by the DatabaseSystem
     *
     * @return Connection
     **/
   public Connection getConnection () {
      return ( connection ) ;
   }
} /* SQLMethod */
