// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/Ftp.java,v 1.1.1.1.4.3 2001/09/24 21:40:52 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
//

package com.ibm.nzna.shared.ftp.FTP ;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.*;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import com.ibm.nzna.shared.ftp.* ;

/**
 The main class of the FTP Bean.
 
 <p>The FTP Bean is an FTP client designed according to the
 <a href="http://sunsite.auc.dk/RFC/rfc/rfc959.html">
 <i>File Transfer Protocol Specification</i></a>.<p>
 
 The FTP commands are implemented as follows:<p>

 <table border=1 cellpadding=3 cellspacing=0>
 <tr> <td><b>FTP command&nbsp;</b></td> <td><b>Implementation&nbsp;</b></td> </tr>
 <tr> <td>ABOR</td> <td>abort</td> </tr>
 <tr> <td>ACCT</td> <td>part of login</td> </tr>
 <tr> <td>ALLO</td> <td>N/A</td> </tr>
 <tr> <td>APPE</td> <td>put</td> </tr>
 <tr> <td>CDUP</td> <td>cd ..</td> </tr>
 <tr> <td>CWD </td> <td>cd</td> </tr>
 <tr> <td>DELE</td> <td>delete</td> </tr>
 <tr> <td>HELP</td> <td>N/A</td> </tr>
 <tr> <td>LIST</td> <td>dir</td> </tr>
 <tr> <td>MDTM</td> <td>time</td> </tr>
 <tr> <td>MKD </td> <td>mkdir</td> </tr>
 <tr> <td>MODE</td> <td>only stream mode</td> </tr>
 <tr> <td>NLST</td> <td>list</td> </tr>
 <tr> <td>NOOP</td> <td>N/A</td> </tr>
 <tr> <td>PASS</td> <td>part of login</td> </tr>
 <tr> <td>PASV</td> <td>N/A</td> </tr>
 <tr> <td>PORT</td> <td>when opening data connection</td> </tr>
 <tr> <td>PWD </td> <td>pwd</td> </tr>
 <tr> <td>QUIT</td> <td>logout</td> </tr>
 <tr> <td>REIN</td> <td>N/A</td> </tr>
 <tr> <td>REST</td> <td>restart</td> </tr>
 <tr> <td>RETR</td> <td>get</td> </tr>
 <tr> <td>RMD </td> <td>rmdir</td> </tr>
 <tr> <td>RNFR</td> <td>part of rename</td> </tr>
 <tr> <td>RNTO</td> <td>part of rename</td> </tr>
 <tr> <td>SITE</td> <td>N/A</td> </tr>
 <tr> <td>SIZE</td> <td>size</td> </tr>
 <tr> <td>SMNT</td> <td>N/A</td> </tr>
 <tr> <td>STAT</td> <td>N/A</td> </tr>
 <tr> <td>STOR</td> <td>put</td> </tr>
 <tr> <td>STOU</td> <td>N/A</td> </tr>
 <tr> <td>STRU</td> <td>N/A</td> </tr>
 <tr> <td>SYST</td> <td>N/A</td> </tr>
 <tr> <td>TYPE</td> <td>setType</td> </tr>
 <tr> <td>USER</td> <td>part of login</td> </tr>
 </table><p>

 <b>Note</b>: The FTP protocol is inherently state-oriented, which
 implies that only one command may be active at a time. Thus, most of
 the methods in this class are synchronized, with a few notable
 exceptions, such as abort().<p>

 @author Morten Hindsholm (mhi@eos.dk)
 @version $Revision: 1.1.1.1.4.3 $
**/
public class Ftp implements FtpClient, java.io.Serializable {

    // Property names
    /** Property name ("port") **/
    public static final String PORT = "port";
    /** Property name ("bufferSize") **/
    public static final String BUFFER_SIZE = "bufferSize";
    /** Property name ("connected") **/
    public static final String CONNECTED = "connected";
    /** Property name ("host") **/
    public static final String HOST = "host";
    /** Property name ("password") **/
    public static final String PASSWORD = "password";
    /** Property name ("timeout") **/
    public static final String TIMEOUT = "timeout";
    /** Property name ("trace") **/
    public static final String TRACE = "trace";
    /** Property name ("traceDestination") **/
    public static final String TRACE_DESTINATION = "traceDestination";
    /** Property name ("type") **/
    public static final String TYPE = "type";
    /** Property name ("user") **/
    public static final String USER = "user";

    /** Carriage Return + Line Feed **/
    protected static final String CRLF = "\r\n";

    /** Format used for parsing file modification times **/
    protected static DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss z");

    /** The port number to use. The standard FTP port is 21 **/
    protected int port = 21;
    /** Current host **/
    protected String host = "";
    /** Current user **/
    protected String user = "anonymous";
    /** Current password **/
    protected String password = "";
    /** Current account **/
    protected String account = "";
    /** State **/
    protected transient boolean connected = false;
    /** Buffer size used when getting/putting files **/
    protected int bufferSize = 4096;
    /** Timeout used for socket I/O (msecs) **/
    protected int timeout = 0;
    /** Transfer type **/
    protected char type = ASCII;
    /** Trace flag **/
    protected boolean trace = false;
    /** Destination for trace messages **/
    protected transient PrintWriter traceDestination = new PrintWriter(System.out, true);

    /** Last reply **/
    protected transient FtpReply reply;

    /** Control connection **/
    protected transient Socket controlSocket = null;
    /** Input stream from control connection **/
    protected transient BufferedReader controlIn = null;
    /** Output stream to control connection **/
    protected transient OutputStreamWriter controlOut = null;

    /** Current transfer **/
    protected transient Transfer transfer;

    /**
     * Create new FTP client.
     **/
    public Ftp() {
    }

    //------------------------------------------------------------------
    // Properties

    /**
     * Set the port number to use for creating host connections.
     * The default value is 21.
     * @param p port number
     **/
    public void setPort(int p) {
        if (p != port) {
            int old = port;
            port = p;
            changes.firePropertyChange(PORT, new Integer(old), new Integer(port));
        }
    }

    /**
     * Get the currently used port number.
     * @return port number
     **/
    public int getPort() {
        return port;
    }

    /**
     * Set the size of the buffer associated with get and put.
     * @param size new buffer size in bytes
     **/
    public void setBufferSize(int size) {
        if (size != bufferSize) {
            int old = bufferSize;
            bufferSize = size;
            changes.firePropertyChange(BUFFER_SIZE, new Integer(old), new Integer(bufferSize));
        }
    }

    /**
     * Get the size of the buffer associated with get and put.
     * @return buffer size in bytes
     **/
    public int getBufferSize() {
        return bufferSize;
    }

    /**
     * Set the socket timeout.
     * If a method performing an I/O operation on a socket times out,
     * the method will throw a java.io.InterruptedIOException.
     * A value of zero (which is the default) disables timeout.
     * @param timeout timeout value in milliseconds
     * @see java.io.InterruptedIOException
     **/
    public synchronized void setTimeout(int timeout) throws java.net.SocketException {
        if (this.timeout != timeout) {
            int old = this.timeout;
            this.timeout = timeout;
            changes.firePropertyChange(TIMEOUT, new Integer(old), new Integer(timeout));
            if (controlSocket != null) {
                controlSocket.setSoTimeout(timeout);
            }
        }
    }

    /**
     * Get the socket timeout.
     * A value of zero means that timeout is disabled.
     * @return timeout value in milliseconds
     **/
    public int getTimeout() {
        return timeout;
    }    

    /**
     * Set host name. Note that this method does not attempt to connect to
     * the host, it merely stores the host name. Use <code>connect()</code>
     * to connect to the host.
     * @param hostName host name, e.g. <code>ftp.javasoft.com</code>
     **/
    public void setHost(String hostName) {
        if (!hostName.equals(host)) {
            String old = host;
            host = hostName;
            changes.firePropertyChange(HOST, old, host);
        }
    }

    /**
     * Get host name.
     **/
    public String getHost() {
        return host;
    }

    /**
     * Set the user name
     * @param userName the user name
     **/
    public void setUser(String userName) {
        if (!userName.equals(user)) {
            String old = user;
            user = userName;
            changes.firePropertyChange(USER, old, user);
        }
    }

    /**
     * Get the user name.
     **/
    public String getUser() {
        return user;
    }

    /**
     * Set the password
     * @param pwd password
     **/
    public void setPassword(String pwd) {
        if (!pwd.equals(password)) {
            String old = password;
            password = pwd;
            changes.firePropertyChange(PASSWORD, old, password);
        }
    }

    /**
     * Get the password.
     **/
    public String getPassword() {
        return password;
    }

    /**
     * Set trace flag.
     * @param flag if true, trace output will be generated.
     **/
    public void setTrace(boolean flag) {
        if (trace != flag) {
            trace = flag;
            changes.firePropertyChange(TRACE, new Boolean(!trace), new Boolean(trace));            
        }
    }

    /**
     * Get trace flag
     **/
    public boolean getTrace() {
        return trace;
    }

    /**
     * Set destination for trace output.
     * This is only used if the trace property is true.
     * Default destination is System.out.
     * @param writer trace output destination
     **/
    public void setTraceDestination(PrintWriter writer) {
        if (traceDestination != writer) {
            PrintWriter old = traceDestination;
            traceDestination = writer;
            changes.firePropertyChange(TRACE_DESTINATION, old, traceDestination);
        }   
    }

    /**
     * Get trace output destination.
     * Trace output is only generated when the trace property is true.
     * @return writer which receives trace output
     **/
    public PrintWriter getTraceDestination() {
        return traceDestination;
    }

    //------------------------------------------------------------------
    // Methods
    
    /**
     * Connect using the current value of the <b>Host</b> property
     **/
    public synchronized void connect() throws UnknownHostException, IOException {
        connect(host);
    }

    /**
     * Connect to remote host.
     * Normally, a connect must be followed by a login.
     * @param host name of host to connect to
     * @see Ftp#login
     **/
    public synchronized void connect(String host) throws UnknownHostException, IOException {
        if(isConnected()) {
            // Disconnect the current connection first
            disconnect();
        }
        // Connect
        setHost(host);
        if (trace) {
            traceDestination.println(" --> Connecting to " + host);
        }
        controlSocket = new Socket(host, port);
        if (timeout > 0) {
            controlSocket.setSoTimeout(timeout);
        }
        // Create I/O streams
        controlIn = new BufferedReader(new InputStreamReader(controlSocket.getInputStream()));
        controlOut = new OutputStreamWriter(controlSocket.getOutputStream());
        // Initial intro response
        readReply();
        // Done
        connected = true;
        changes.firePropertyChange(CONNECTED, new Boolean(false), new Boolean(true));            
    }

    /**
     * Check if connected.
     * @return <b>true</b> if connected, false otherwise
     **/
    public boolean isConnected() {
        return connected;
    }

    /**
     * Disconnect from host.
     **/
    public synchronized void disconnect() throws IOException {
        connected = false;
        if (controlIn != null) {
            controlIn.close();
            controlIn = null;
        }
        if (controlOut != null) {
            controlOut.close();
            controlOut = null;
        }
        if (controlSocket != null) {
            controlSocket.close();
            controlSocket = null;
        }
    }

    /**
     * Login to host using the values of the <b>User</b> and <b>Password</b> properties
     * The FTP client must be connected before login can be attempted.
     * @see Ftp#connect
     **/
    public synchronized void login() throws FtpError, IOException {
        login(user, password, "");
    }

    /**
     * Login to host using specified username and password.
     * The FTP client must be connected before login can be attempted.
     * @param user user name
     * @param password user's password
     * @see Ftp#connect
     **/
    public synchronized void login(String user, String password) throws FtpError, IOException {
        login(user, password, "");
    }

    /**
     * Login to host using specified username, password and account.
     * The FTP client must be connected before login can be attempted.
     * @param user user name
     * @param password user's password
     * @param account account to use
     * @see Ftp#connect
     **/
    public synchronized void login(String user, String password, String account)
        throws FtpError, IOException {
        this.user = user;
        this.password = password;
        this.account = account;
        command("USER " + user);
        if (getReply().getType() == FtpReply.POSITIVE_INTERMEDIATE) {
            command("PASS " + password);
            if (getReply().getType() == FtpReply.POSITIVE_INTERMEDIATE) {
                command("ACCT " + account);
            }
        }
    }

    /**
     * Log out from host.
     **/
    public synchronized void logout() throws FtpError, IOException {
        // Should maybe use REIN instead of QUIT, but many servers don't support it
        command("QUIT");
    }

    /**
     * Get a verbose directory listing of the current remote directory.
     * The format of the listing is host-specific.
     **/
    public synchronized String[] dir() throws FtpError, IOException {
        return getList(null, true);
    }

    /**
     * Get a verbose listing of the specified remote path.
     * The format of the listing is host-specific.
     * @param path directory and/or name of file(s) to list.
     *        May include wildcards if the host supports it.
     **/
    public synchronized String[] dir(String path) throws FtpError, IOException {
        return getList(path, true);
    }
    
    /**
     * Get a list of all file names in the current remote directory.
     **/
    public synchronized String[] list() throws FtpError, IOException {
        return getList(null, false);
    }

    /**
     * Get a list of file names in the specified remote path.
     * @param path directory and/or name of file(s) to list.
     *        May include wildcards if the host supports it.
     **/
    public synchronized String[] list(String path) throws FtpError, IOException {
        return getList(path, false);
    }

    /**
     * Get a long or short listing of the specified remote path
     * @param path directory and/or name of file(s) to list.
     *        May include wildcards if the host supports it.
     *        <code>null</code> means current directory
     * @param verbose if <code>true</code>, information (size, date, etc.)
     *        about each file is returned if <code>false</code>,
     *        only the file names are returned
     * @return a long or short directory listing
     **/
    protected synchronized String[] getList(String path, boolean verbose) throws FtpError, IOException {
        transfer = new ListTransfer(this, path, verbose);
        return ((ListTransfer)transfer).getList();
    }

    /**
     * Change remote directory.
     * @param dir remote directory to change to
     **/
    public synchronized void cd(String dir) throws FtpError, IOException {
        command((dir.equals("..")) ? "CDUP" : "CWD " + dir);
    }

    /**
     * Get current remote directory.
     * @return path to current remote directory
     **/
    public synchronized String pwd() throws FtpError, IOException {
        command("PWD");
        // The PWD command should return a reply of the following form:
        // 257 "<directory-name>" <commentary>
        // FtpReply.getMessage() will return the "<directory-name>" <commentary> part
        String msg = getReply().getMessage();
        return msg.substring(1, msg.indexOf('"', 1));
    }

    /**
     * Set transfer type.<p>
     * The transfer type should only be set when the
     * client and host are connected.
     * @param transferType desired transfer type, must be <code>ASCII</code>
     * or <code>IMAGE</code>.
     **/
    public synchronized void setType(char transferType) throws FtpError, IOException {
        if (transferType == ASCII || transferType == IMAGE) {
            if (type != transferType) {
                char old = type;
                type = transferType;
                changes.firePropertyChange(TYPE, new Character(old), new Character(type));
            }
            if (isConnected()) {
                command("TYPE " + type);
            }
        }
    }

    /**
     * Get the active transfer type.
     * @return transfer type, <code>ASCII</code> or <code>IMAGE</code>.
     **/
    public char getType() {
        return type;
    }

    /**
     * Get remote file. The local file will have the same name as the remote
     * file and be created in the current directory.
     * @param remoteFile name of remote file to get
     **/
    public synchronized void get(String remoteFile) throws FtpError, IOException {
        get(remoteFile, remoteFile);
    }
    
    /**
     * Get remote file.
     * @param remoteFile name of remote file to get
     * @param localFile name of local file to create.
     *        This may be a relative or absolute path.
     **/
    public synchronized void get(String remoteFile, String localFile)
        throws FtpError, IOException {
        transfer = new StreamGet(this, new File(localFile), remoteFile);
        transfer.start();            
    }

    /**
     * Put remote file. If the file already exists, it will be overwritten.
     * @param localFile name of local file to transfer
     **/
    public synchronized void put(String localFile) throws FtpError, IOException {
        put(localFile, new File(localFile).getName(), false);
    }

    /**
     * Put remote file and possibly append.
     * If the file already exists, it will be overwritten.
     * @param localFile name of local file to transfer
     * @param append if true, localFile will be appended to
     *        the host file (if it exists)
     **/
    public synchronized void put(String localFile, boolean append)
        throws FtpError, IOException {
        put(localFile, new File(localFile).getName(), append);
    }

    /**
     * Put remote file and possibly append.
     * If the file already exists, it will be overwritten.
     * @param localFile name of local file to transfer
     * @param remoteFile name of remote file
     * @param append if true, localFile will be appended to
     *        the host file (if it exists)
     **/
    public synchronized void put(String localFile, String remoteFile, boolean append)
        throws FtpError, IOException {
        transfer = new StreamPut(this, new File(localFile), remoteFile, append);
        transfer.start();
    }

    /**
     * Check if current transfer is restartable.
     * To be restartable, the current transfer must be a file transfer (i.e.
     * a <i>get</i> or <i>put</i>) and it must have been terminated prematurely
     * because of an error or <i>abort.</i>
     * @return true if restart can be attempted
     **/
    public synchronized boolean isRestartable() {
        if (transfer != null && transfer instanceof FileTransfer) {
            return ((FileTransfer)transfer).isRestartable();
        }
        return false;
    }

    /**
     * Restart current (failed) transfer (if possible).
     *
     * <p>Please note that there has to be a <b>current transfer</b>
     * because otherwise there is no information about where to
     * restart from.
     **/
    public synchronized void restart() throws FtpError, IOException {
        if (transfer != null && transfer instanceof FileTransfer) {
            ((FileTransfer)transfer).restart();
        }
    }

    /**
     * Rename remote file.
     * @param from old name
     * @param to new name
     **/
    public synchronized void rename(String from, String to) throws FtpError, IOException {
        command("RNFR " + from);
        command("RNTO " + to);
    }

    /**
     * Delete remote file.
     * @param filename name of file to delete
     **/
    public synchronized void delete(String filename) throws FtpError, IOException {
        command("DELE " + filename);
    }

    /**
     * Create remote directory.
     * @param directory name of directory to create
     **/
    public synchronized void mkdir(String directory) throws FtpError, IOException {
        command("MKD " + directory);
    }
    
    /**
     * Remove remote directory.
     * @param directory name of directory to remove
     **/
    public synchronized void rmdir(String directory) throws FtpError, IOException {
        command("RMD " + directory);
    }

    /**
     * Get size of remote file.
     * This command (SIZE) is not defined by the FTP standard.
     * It is however part of the current FTPEXT draft
     * <i>Extended Directory Listing and Restart Mechanism for FTP.</i>
     * @param file name of remote file
     * @return the number of bytes needed to transfer in order to
     *         get the remote file, or -1 if the size could not be obtained.
     **/
    public long size(String file) throws FtpError, IOException {
        command("SIZE " + file);
        // Reply: "213 nnnnnn"
        try {
            return Long.parseLong(getReply().getMessage());
        }
        catch (NumberFormatException e) {
            // ignore -- just return -1
        }
        return -1;
    }

    /**
     * Get modification time of remote file.
     * This command (MDTM) is not defined by the FTP standard.
     * It is however part of the current FTPEXT draft
     * <i>Extended Directory Listing and Restart Mechanism for FTP.</i>
     * @param file name of remote file
     * @return modification time of <i>file</i> expressed as UTC (GMT)
     *         or null if the time could not be obtained.
     **/
    public Date time(String file) throws FtpError, IOException {
        command("MDTM " + file);
        // Reply: "213 YYYYMMDDHHMMSS.sss"
        String s = getReply().getMessage();
        int stop = s.indexOf('.');
        if (stop > 0) {
            // ignore msecs
            s = s.substring(0, stop);
        }
        try {
            return dateFormat.parse(s + " GMT.");
        }
        catch (java.text.ParseException e) {
            // ignored -- just return null
        }
        return null;
    }
    
    /**
     * Abort current file transfer.
     **/
    public void abort() {
        Transfer t = transfer;
        if (t != null) {
            t.abort();
        }
    }

    /**
     * Get the reply to the last command
     **/
    public FtpReply getReply() {
        return reply;
    }

    /**
     * Execute an FTP command and read reply from host
     * @param cmd command to send to host
     * @exception FtpError if the command fails or the client is not connected
     * @exception IOException if an I/O error occurs
     **/
    public synchronized void command(String cmd) throws FtpError, IOException {
        if (trace) {
            traceDestination.println(" --> " + ((cmd.startsWith("PASS") ? "PASS ******" : cmd)));
        }
        if (isConnected()) {
            controlOut.write(cmd + CRLF);
            controlOut.flush();
            readReply();
        }
        else {
            reply = new FtpReplyImpl(530, "Not logged in.");
        }
        int replyType = getReply().getType();
        if (replyType == FtpReply.TRANSIENT_NEGATIVE ||
            replyType == FtpReply.PERMANENT_NEGATIVE) {
            throw new FtpError(getReply());
        }
    }

    //-------------------------------------------------------------------
    // Bound properties

    /** PropertyChange handler **/
    protected PropertyChangeSupport changes = new PropertyChangeSupport(this);

    /**
     * Add a PropertyChangeListener
     **/
    public void addPropertyChangeListener(PropertyChangeListener l) {
        changes.addPropertyChangeListener(l);
    }

    /**
     * Remove a PropertyChangeListener
     **/
    public void removePropertyChangeListener(PropertyChangeListener l) {
        changes.removePropertyChangeListener(l);
    }

    //-------------------------------------------------------------------
    // Event handling

    /** The FtpListeners are kept here **/
    protected Vector ftpListeners = new Vector();

    /**
     * Add an FTP listener.
     * @param listener the listener to add
     **/
    public synchronized void addFtpListener(FtpListener listener) {
        ftpListeners.addElement(listener);
    }

    /**
     * Remove an FTP listener.
     * @param listener the listener to remove
     **/
    public synchronized void removeFtpListener(FtpListener listener) {
        ftpListeners.removeElement(listener);
    }

    /**
     * Helper method to dispatch transfer started events.
     * @param evt the event to dispatch
     **/
    protected void fireTransferStartedEvent(TransferEvent evt) {
        if (ftpListeners.size() == 0) {
            return;
        }
        FtpListener[] targets;
        synchronized (this) {
            targets = new FtpListener[ftpListeners.size()];
            ftpListeners.copyInto(targets);
        }
        for (int i = 0; i < targets.length; i++) {
            targets[i].transferStarted(evt);
        }
    }

    /**
     * Helper method to dispatch transfer progress events.
     * @param evt the event to dispatch
     **/
    protected void fireTransferProgressEvent(TransferEvent evt) {
        if (ftpListeners.size() == 0) {
            return;
        }
        FtpListener[] targets;
        synchronized (this) {
            targets = new FtpListener[ftpListeners.size()];
            ftpListeners.copyInto(targets);
        }
        for (int i = 0; i < targets.length; i++) {
            targets[i].transferProgress(evt);
        }
    }

    /**
     * Helper method to dispatch transfer ended events.
     * @param evt the event to dispatch
     **/
    protected void fireTransferEndedEvent(TransferEvent evt) {
        if (ftpListeners.size() == 0) {
            return;
        }
        FtpListener[] targets;
        synchronized (this) {
            targets = new FtpListener[ftpListeners.size()];
            ftpListeners.copyInto(targets);
        }
        for (int i = 0; i < targets.length; i++) {
            targets[i].transferEnded(evt);
        }
    }

    /**
     * Helper method to dispatch reply events.
     * This method will dispatch the current FTP reply.
     **/
    protected void fireReplyEvent() {
        if (ftpListeners.size() == 0) {
            return;
        }
        ReplyEvent re = new ReplyEvent(this, getReply());
        FtpListener[] targets;
        synchronized (this) {
            targets = new FtpListener[ftpListeners.size()];
            ftpListeners.copyInto(targets);
        }
        for (int i = 0; i < targets.length; i++) {
            targets[i].ftpReply(re);
        }
    }

    //------------------------------------------------------------------
    // Helper methods

    /**
     * Receive reply from host.
     * The reply can be read through <code>getReply()</code> and
     * <code>getReplyCode()</code>
     **/
    protected void readReply() throws IOException {
        String str;
        int code = 0;
        StringBuffer sb = new StringBuffer();
        while (true) {
            str = controlIn.readLine();
            if (str == null) {
                // The connection was dropped or something -- this is really an error
                throw new IOException("EOF while reading from control connection");
            }
            // search for three-digit number followed by space
            if (str.length() > 3 &&
                Character.isDigit(str.charAt(0)) &&
                Character.isDigit(str.charAt(1)) &&
                Character.isDigit(str.charAt(2)) && 
                str.charAt(3) == ' ') {
                // extract reply code
                code = Integer.parseInt(str.substring(0,3));
                // extract message
                sb.append(str.substring(4));
                break;
            }
            sb.append(str);
            sb.append("\n");
        } 
        reply = new FtpReplyImpl(code, sb.toString());
        if (trace) {
            traceDestination.println("<--  " + reply.getCode() + " " + reply.getMessage());
        }
        fireReplyEvent();
    }

}
