// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FTP/Transfer.java,v 1.1.1.1.4.2 2001/09/24 18:39:42 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
//

package com.ibm.nzna.shared.ftp.FTP ;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

import com.ibm.nzna.shared.ftp.* ;

/**
 * A generic transfer between FTP host and client.
 *
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public abstract class Transfer {

    /** Carriage Return (=0x0D) **/
    protected static final int CR = 0x0D;
    /** Line Feed (=0x0A) **/
    protected static final int LF = 0x0A;
    static final byte[] CRLF = { CR, LF };
    /** Platform-specific line separator **/
    protected static final byte[] LINESEP = System.getProperty("line.separator", "\n").getBytes();
    /** Milliseconds between progess events **/
    protected static final int PROGRESS_INTERVAL = 1000;
    /** The FTP connection **/
    protected Ftp ftp;
    /** Data connection **/
    protected Socket dataSocket;
    /** FTP command **/
    protected String command;
    /** Aborted flag **/
    protected boolean aborted;
    /** TransferEvent describing the transfer **/
    TransferEvent state;

    /**
     * Create new Transfer.
     **/
    public Transfer(Ftp ftp, String command) {
        this.ftp = ftp;
        this.command = command;
    }

    public void start() throws FtpError, IOException {
        aborted = false;        // not aborted yet...
        // Select port for data connection and send command
        ServerSocket serverSocket = null;
        try {
            before();
            serverSocket = openDataConnection();
            ftp.command(command);

            // Open data connection
            dataSocket = serverSocket.accept();
            int timeout = ftp.getTimeout();
            if (timeout > 0) {
                dataSocket.setSoTimeout(timeout);
            }
            doIt();
            if (aborted) {
                checkAborted();
            }
            // Get reply from host
            ftp.readReply();
            int replyType = ftp.getReply().getType();
            if (replyType == FtpReply.TRANSIENT_NEGATIVE ||
                replyType == FtpReply.PERMANENT_NEGATIVE) {
                throw new FtpError(ftp.getReply());
            }
            after();
        }
        finally {
            if (dataSocket != null) dataSocket.close();
            if (serverSocket != null) serverSocket.close();
        }
    }

    public void abort() {
        aborted = true;
        try {
            // end the current transfer by closing the data connection
            // this will generate a java.net.SocketException when any
            // IO is attempted on the socket
            // the SocketException will be caught in get() or put()
            dataSocket.close();
        }
        catch (Exception e) {
            // ignored
        }
        dataSocket = null;
    }

    /**
     * Open new data connection to the host.
     * This method will create a new ServerSocket on a system-assigned port,
     * send a PORT command to the host to inform it about the port.
     * @return the newly created ServerSocket which the host will connect to.
     **/
    protected ServerSocket openDataConnection() throws FtpError, IOException {
        // Create a server socket on a free port
        ServerSocket socket = new ServerSocket(0);
        int timeout = ftp.getTimeout();
        if (timeout > 0) {
            socket.setSoTimeout(timeout);
        }
        // Get IP address and port number
        String addr = ftp.controlSocket.getLocalAddress().getHostAddress();
        int port = socket.getLocalPort();
        // Send PORT command
        ftp.command("PORT " + addr.replace('.', ',') + "," +
            ((port & 0xff00) >> 8) + "," + (port & 0x00ff));
        return socket;
    }

    /**
     * Called before transfer
     **/
    protected void before() throws FtpError, IOException {
    }

    /**
     * Called after transfer
     **/
    protected void after() throws FtpError, IOException {
    }

    /**
     * Make the actual transfer
     **/
    protected abstract void doIt() throws IOException;

    /**
     * See if current transfer has been aborted.
     * If it has, then send an "ABOR" command to the host and
     * process the reply
     **/
    protected void checkAborted() throws FtpError, IOException {
        if (aborted) {
            try {
                ftp.command("ABOR");
            }
            catch (FtpError e) {
            }
//             switch (ftp.getReply().getCode()) {
//               case 226:
//                 // 226: "transfer complete."
//                 // get result of ABORT command
//                 ftp.readReply();
//                 break;
//               case 426:
//                 // 426: "Connection closed; transfer aborted."
//                 // get result of ABORT command
//                 ftp.readReply();
//                 break;
//             }
        }
    }

}
