package com.ibm.nzna.shared.ftp ;
/*-----------------------------------------------------------*/
/*   FTPFile                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPFile                             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.pom.* ;
import java.io.* ;
/**
 * This record represents a file on an FTP site and could represent the
 * same file on the local platform. This record keeps track of size, location
 * on both local and remote and the FTP status.<br><br>
 *
 *
 * Please note, when dealing with remoteFilenames and remoteDirectories, the forward
 * slash will <b>always</b> be used since FTP accepts it no matter what type of
 * FTP server something is. The localFilename and localDirectory will default to the
 * host platform that this class is instantiated on.<br><br>
 *
 * Please note the new REC_XXX types defined in this file for more advanced
 * ftp manipulations. All new REC_XXX that are TO_FTP should be taken as REC_INSERT, and
 * all others as REC_NOCHANGE.
 *
 * @version 1.0, 7/24/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FTPFile                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPFile extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Public Members                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int PENDING        = 0 ;
   public static final int COMPLETE       = 1 ;
   public static final int ERROR          = 2 ;      

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Move Types                                              */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /** Move only, deletes local file once FTP is complete */
   public static final int REC_MOVE_TO_FTP      = 10   ;  

   /** Move only, deletes remote file once FTP is complete */
   public static final int REC_MOVE_FROM_FTP     = 11 ;

   /** Copy only, no files are deleted */
   public static final int REC_COPY_TO_FTP        = 12 ;

   /** Copy only, no files are deleted */
   public static final int REC_COPY_FROM_FTP      = 13 ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String  remoteFilename       = null ;       // Stores the remote filename, with no leading file separator
   private String  localFilename        = null ;       // Stores the local filename, with no leading file separator
   private String  remoteDirectory      = null ;       // Stores the remote directory, does not contain the trailing path separator
   private String  localDirectory       = null ;       // Stores the local directory, does not contain the trailing path separator
   private int     ftpStatus            = PENDING ;
   private long    size                 = 0 ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method creates a blank FTPFile
    *  record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FTPFile () {
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the FTP
    *  file record to the parameters passed in
    *  
    *
    *  @param remoteFilename The Remote filename of the file on the FTP Server
    *  @param remoteDirectory The Remote Directory of the file on the FTP Server
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FTPFile ( String remoteFilename, String remoteDirectory ) {
      setRemoteFilename  ( remoteFilename ) ;
      setRemoteDirectory ( remoteDirectory ) ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create an FTPFile
    *  with the information form the File record passed in,
    *  please keep in mind that this sets the localFilename and localDirectory, nothing remote!
    *  Also, please keep in mind that the localDirectory is parsed
    *  from the getAbsolutePath() method in the File object passed.
    *
    *  @param file The java.io.File object to create with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FTPFile ( File file ) {
      try {
         if ( !file.isDirectory() ) {
            String directory = file.getAbsolutePath() ;

            if ( directory.indexOf ( File.separator ) >= 0 )
               directory = directory.substring ( 0, directory.lastIndexOf ( File.separator) ) ;

            setLocalFilename  ( file.getName() ) ;
            setLocalDirectory ( directory ) ;
            setSize           ( file.length() ) ;

         } else
            throw ( new IllegalArgumentException ( "Cannot FTP a directory" ) ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }






   /*------------------------------------------------------------*/
   /* setRemoteDirectory
    */
   /**
    *    This method will change the remote directory
    *  that this file is placed in. Please keep in mind that this
    *  doesn't touch any FTP server, and will also set the ftpStatus property to PENDING
    *
    *  @param remoteDirectory The remote directory to change to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRemoteDirectory ( String remoteDirectory ) {
      this.remoteDirectory = remoteDirectory ;
      this.ftpStatus       = PENDING ;
   }




   /*------------------------------------------------------------*/
   /* setRemoteFilename
    */
   /**
     *    This method will change the remote filename
     *  that this file is placed in. Please keep in mind that this
     *  doesn't touch any FTP server, and will also set the ftpStatus property to PENDING
     *
     *  @param remoteFilename The remote filename to change to
     *  @param None
     *  @param None
     *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRemoteFilename ( String remoteFilename ) {
      this.remoteFilename = remoteFilename ;
      this.ftpStatus      = PENDING ;
   }








   /*------------------------------------------------------------*/
   /* setLocalDirectory
    */
   /**
    *    This method will change the local directory
    *  that this file is placed in. Please keep in mind that this
    *  doesn't touch any FTP server, and will also set the ftpStatus property to PENDING
    *
    *  @param localDirectory The local directory to change to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocalDirectory ( String localDirectory ) {
      this.localDirectory = localDirectory ;
      this.ftpStatus       = PENDING ;
   }




   /*------------------------------------------------------------*/
   /* setLocalFilename
    */
   /**
     *    This method will change the local filename
     *  that this file is placed in. Please keep in mind that this
     *  doesn't touch any FTP server, and will also set the ftpStatus property to PENDING
     *
     *  @param localFilename The local filename to change to
     *  @param None
     *  @param None
     *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocalFilename ( String localFilename ) {
      this.localFilename = localFilename ;
      this.ftpStatus      = PENDING ;
   }




   /*------------------------------------------------------------*/
   /* setSize
    */
   /**
    *    This method will set the size of our
    *  file. Normally when working with local only
    *  files, this should be handled through the constructor
    *
    *  @param size The size of the file
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSize ( long size ) {
      this.size      = size ;
      this.ftpStatus = PENDING ;
   }





   /*------------------------------------------------------------*/
   /* getRemoteFilename
    */
   /**
    *    This method will return the name of the
    *  remote filename. In other words, the name of
    *  this file on the FTP Server
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getRemoteFilename () {
      return( remoteFilename ) ;
   }




   /*------------------------------------------------------------*/
   /* getRemoteDirectory
    */
   /**
    *    This method will return the remote
    *  directory that this file is being placed in or
    *  received from
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getRemoteDirectory () {
      return( remoteDirectory ) ;
   }





   /*------------------------------------------------------------*/
   /* getLocalFilename
    */
   /**
    *    This method will return the name of the local
    *  file being represented. This local file should
    *  be located on the local host.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getLocalFilename () {
      return( localFilename ) ;
   }




   /*------------------------------------------------------------*/
   /* getLocalDirectory
    */
   /**
    *    This method will return the local directory
    *  that the file is stored in. This directory should
    *  be on the local host
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getLocalDirectory () {
      return( localDirectory ) ;
   }





   /*------------------------------------------------------------*/
   /* getSize
    */
   /**
    *    This method will return the size of this
    *  file. Please keep in mind if this file is not
    *  located on the local file system and has not been FTPed, this value will be zero
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public long getSize () {
      return( size ) ;
   }





   /*------------------------------------------------------------*/
   /* getAbsoluteRemote
    */
   /**
    *    This method will return the entire
    *  remote directory and remote filename as
    *  one string
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getAbsoluteRemote () {
      return( getRemoteDirectory()+"/"+getRemoteFilename() ) ;
   }





   /*------------------------------------------------------------*/
   /* getAbsoluteLocal
    */
   /**
    *    This method will return the entire local directory
    *  and local filename, joined with the host's
    *  path separator
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getAbsoluteLocal () {
      return( getLocalDirectory()+File.separator+getLocalFilename() ) ;
   }





   /*------------------------------------------------------------*/
   /* validForFTP
    */
   /**
    *    This method will return TRUE if this record
    *  is valid for an FTP attempt. The only reason
    *  that this record would not be valid is if the remoteFilename
    *  or remoteDirectory is null.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public boolean validForFTP () {
      return( ( remoteFilename != null ) && ( remoteDirectory != null ) ) ;
   }

} /* FTPFile */
