package com.ibm.nzna.shared.ftp ;  
/*-----------------------------------------------------------*/
/*   FTPManager                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPManager                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.net.* ;
import java.io.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The FTPManager is a class to help with FTPing files. The manager 
 * allows for moving of FTPFile objects from or to an FTP Site. The manager
 * also uses the current LogSystem if one has been created. To turn off logging,
 * call the method 'setLogging(null)'. 
 * <br><br><br>
 *
 * Possible getLastError() values returned:<br><pre>
 * 
 * Value  Description<br>
 * 200    Command okay.<br>
 * 500    Syntax error, command unrecognized.<br>
 *        This may include errors such as command line too long.<br>
 * 501    Syntax error in parameters or arguments.<br>
 * 202    Command not implemented, superfluous at this site.<br>
 * 502    Command not implemented.<br>
 * 503    Bad sequence of commands.<br>
 * 504    Command not implemented for that parameter.<br>
 * 1      An IOException was thrown, check the log files<br>
 * 2      An Exception was thrown, check the log files<br>
 * </pre><br><br>
 *
 * Please use the static defined error messages in this class instead of the numbers above.
 *
 *
 * @version 1.0, 7/25/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FTPManager                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPManager extends Object {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Static Members                                          */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static final int ERROR_NONE                                = 200 ;
    public static final int ERROR_COMMAND_UNRECOGNIZED                = 500 ;
    public static final int ERROR_SYNTAX_ERROR                        = 501 ;
    public static final int ERROR_COMMAND_NOT_IMPLEMENTED_SUPERFLUOUS = 202 ;
    public static final int ERROR_COMMAND_NOT_IMPLEMENTED             = 502 ;
    public static final int ERROR_BAD_SEQUENCE_OF_COMMANDS            = 503 ;
    public static final int ERROR_COMMAND_NOT_IMPLEMENTED_FOR_PARAM   = 504 ;
    public static final int ERROR_IOEXCEPTION                         = 1 ;
    public static final int ERROR_EXCEPTION                           = 2 ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private Ftp       ftpBean        = null ;
    private String    host           = null ;
    private String    userId         = null ;
    private String    password       = null ;
    private LogSystem logSystem      = null ;
    private int       lastError      = ERROR_NONE ;




    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This constructor creates a blank FTPManager
     *  record, with no host, user id or password
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public FTPManager() {
    }





    /*------------------------------------------------------------*/
    /* FTPManager
     */
    /**
     *    This method will create an FTPManager
     *  Object initialized to the values passed in
     *  
     *
     *  @param host The host FTP Site to connect to
     *  @param user The User Id to connect to the FTP Site with
     *  @param password The password of the user id
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public FTPManager ( String host, String userId, String password ) {
        setHost     ( host ) ;
        setUserId   ( userId ) ;
        setPassword ( password ) ;
    }





    /*------------------------------------------------------------*/
    /* setHost
     */
    /**
     *    This method will set the current host. Please note
     *  that if this FTPManager is currently connected,
     *  that connection will be dropped.
     *
     *  @param host The new host
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setHost ( String host ) {
        this.host = host ;
        if ( isConnected() ) {
            close() ;
        }
    } /* setHost  */


    /*------------------------------------------------------------*/
    /* setUserId
     */
    /**
     *    This method will set the current userId. Please note
     *  that if this FTPManager is currently connected,
     *  that connection will be dropped.
     *
     *  @param userId The new userId
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setUserId ( String userId ) {
        this.userId = userId ;
        if ( isConnected() ) {
            close() ;
        }
    } /* setUserId  */



    /*------------------------------------------------------------*/
    /* setPassword
     */
    /**
     *    This method will set the current password. Please note
     *  that if this FTPManager is currently connected,
     *  that connection will be dropped.
     *
     *  @param password The new password
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setPassword ( String password ) {
        this.password = password ;
        if ( isConnected() ) {
            close() ;
        }
    } /* setPassword  */





    /*------------------------------------------------------------*/
    /* isConnected
     */
    /**
     *    This method will return TRUE if this FTPManager
     *  is currently connected to an FTP Site
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean isConnected () {
        return( ftpBean != null ) ;
    }




    /*------------------------------------------------------------*/
    /* close
     */
    /**
     *    This method will stop all current transfers
     *  and disconnect from the host. Please be sure
     *  that you are not FTPing a file when this is called. The
     *  results from an operation like that would be unknown
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void close () {
        if ( isConnected() ) {

            try {
                ftpBean.disconnect() ;
            } catch ( Exception e ) {
                if ( getLogging() ) {
                    logSystem.log ( 1, e, false ) ;
                    lastError = ERROR_EXCEPTION ;
                }
            }
            ftpBean = null ;
        }
    }





    /*------------------------------------------------------------*/
    /* getLogging
     */
    /**
     *    This method will return TRUE if this FTPManager
     *  is currently logging to the default LogSystem
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean getLogging() {
        return( logSystem != null ) ;
    }





    /*------------------------------------------------------------*/
    /* setLogging
     */
    /**
     *    This method will set logging on or off. If you
     *  pass a NULL, that will turn logging off
     *  
     *
     *  @param logSystem The LogSystem to use for logging
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setLogging ( LogSystem logSystem ) {
        this.logSystem = logSystem ;
    }





    /*------------------------------------------------------------*/
    /* open
     */
    /**
     *    This method will attempt to connect to the
     *  host FTP Site 
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return TRUE if connected, FALSE if otherwise
     *//*---------------------------------------------------------*/
    public boolean open () {
        try {

            if ( getLogging() )
                LogSystem.log ( 1, "Connecting to FTP Site: "+host ) ;

            ftpBean = new Ftp() ;
            ftpBean.connect ( host ) ;
            ftpBean.login ( userId, password ) ;

            if ( getLogging() )
                LogSystem.log ( 1, "Connected to "+host ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Catch the Unknown Host Exception                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        catch ( UnknownHostException uhe ) {

            if ( getLogging() ) {
                LogSystem.log ( 1, "Unknown Host. Cannot connect to FTP site :"+host ) ;
                LogSystem.log ( 1, "-----------------------------------------------------------" ) ;
                LogSystem.log ( 1, uhe, false ) ;
                lastError = ERROR_EXCEPTION ;
            }

        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Catch All other exceptions                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        catch ( Exception e ) {
            if ( getLogging() ) {
                logSystem.log ( 1, e, false ) ;
                lastError = ERROR_EXCEPTION ;
            }
            ftpBean = null ;
        }

        return( isConnected() ) ;
    }





    /*------------------------------------------------------------*/
    /* processFile
     */
    /**
     *    This method will process a single FTPFile
     *  object
     *  
     *
     *  @param ftpFile The FTPFile object to process
     *  @param None
     *  @param None
     *
     *  @return boolean TRUE = process went ok, FALSE if failed. Please
     *  use the 'getLastError' method to determine why a failure happened
     *//*---------------------------------------------------------*/
    public boolean processFile ( FTPFile ftpFile ) {
        boolean rc = false ;

        if ( ( ftpFile != null ) &&
             ( ftpFile.validForFTP() ) ) {
            String remoteFile = ftpFile.getAbsoluteRemote() ;
            String localFile  = ftpFile.getAbsoluteLocal() ;


            switch ( ftpFile.getRecStatus() ) {
                
                case ftpFile.REC_INSERT :
                case ftpFile.REC_UPDATE :
                case ftpFile.REC_COPY_TO_FTP :
                    copyToFtp ( ftpFile ) ;
                    break ;

                case ftpFile.REC_COPY_FROM_FTP :
                    break ;

                case ftpFile.REC_MOVE_FROM_FTP :
                    break ;

                case ftpFile.REC_MOVE_TO_FTP :
                    break ;

                case ftpFile.REC_DELETE :
                    break ;


            }

        } else
            rc = true ;

        return( rc ) ;

    }






    /*------------------------------------------------------------*/
    /* copyToFtp
     */
    /**
     *    This method will copy a file to the FTP site only.
     *  
     *  
     *
     *  @param ftpFile The ftpFile to copy
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean copyToFtp ( FTPFile ftpFile ) {
        boolean rc = false ;

        try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are connected, continue                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( isConnected() ) {

                if ( getLogging() ) 
                    LogSystem.log ( 1, "Changing to remote directory: "+ftpFile.getRemoteDirectory() ) ;
                ftpBean.cd ( ftpFile.getRemoteDirectory() ) ;

                if ( getLogging() ) 
                    LogSystem.log ( 1, "Putting File: "+ftpFile.getAbsoluteLocal() ) ;

                ftpBean.put ( ftpFile.getAbsoluteLocal(), ftpFile.getRemoteFilename(), false ) ;

                if ( getLogging() ) 
                    LogSystem.log ( 1, "File Put" ) ;
                rc = true ;

            } else
                throw new FTPSiteNotConnectedException ( ftpFile ) ;       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Catch FTP Errors                                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
        } catch ( FtpError ftpError ) {
            lastError = ftpError.getReplyCode() ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Catch IO Exceptions                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        catch ( IOException ioEx ) {

            if ( getLogging() ) {
                logSystem.log ( 1, ioEx, false ) ;
                lastError = ERROR_IOEXCEPTION ;
            }

        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Catch any other exceptions                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        catch ( Exception e ) {

            if ( getLogging() ) {
                logSystem.log ( 1, e, false ) ;
                lastError = ERROR_EXCEPTION ;
            }

        }

        return( rc ) ;
    }





    /*------------------------------------------------------------*/
    /* moveToFtp
     */
    /**
     *    This method will put a local file from the
     *  client to the ftp site, removing the client file
     *  when it is done
     *
     *  @param ftpFile The FTPFile to move
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private boolean moveToFtp ( FTPFile ftpFile ) {
        boolean rc = false ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First, copy the FTP file to the FTP server              */                                 
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( copyToFtp ( ftpFile ) ) {

            try {

                FileUtil.deleteFile ( ftpFile.getAbsoluteLocal() ) ;
                rc = true ;
            } catch ( Exception e ) {

                if ( getLogging() ) {
                    logSystem.log ( 1, e, false ) ;
                    lastError = ERROR_EXCEPTION ;
                } /* if  */

            } /* catch  */

        }

        return( rc ) ;
    }




    /*------------------------------------------------------------*/
    /* getLastError
     */
    /**
     *    This method will return the last FTP Error
     *  Code
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int
     *//*---------------------------------------------------------*/
    public int getLastError () {
        return( lastError ) ;
    }

} /* FTPManager */
