// 
//   $Header: /home/cvs/cvsroot/javaroot/com/ibm/nzna/shared/ftp/FtpClient.java,v 1.1.1.1.4.2 2001/09/24 18:39:43 dlentz Exp $
// 
//   Copyright (c) 1997 Eastfork Object Space (EOS)
//   All Rights Reserved
//   No part of this program may be copied, used or delivered to
//   anyone without the express written consent of EOS
//

package com.ibm.nzna.shared.ftp ;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Date;

/**
 * This interface describes an FTP client. It contains methods
 * which match those found in a command-line FTP client application.
 * @author Morten Hindsholm (mhi@eos.dk)
 * @version $Revision: 1.1.1.1.4.2 $
 **/
public interface FtpClient {

    /**
     * Connect to remote host.
     * Normally, a connect must be followed by a login.
     * @param host name of host to connect to
     * @see FtpClient#login
     **/
    void connect(String host) throws UnknownHostException, IOException;

    /**
     * Check if connected.
     * @return <b>true</b> if connected, false otherwise
     **/
    boolean isConnected();

    /**
     * Disconnect from host.
     **/
    void disconnect() throws IOException;

    /**
     * Login to host using specified username and password.
     **/
    void login(String username, String password) throws FtpError, IOException;

    /**
     * Login to host using specified username, password and account.
     **/
    void login(String userName, String password, String account)
        throws FtpError, IOException;
    
    /**
     * Log out from host.
     **/
    void logout() throws FtpError, IOException;

    /**
     * Get a verbose directory listing of the current remote directory.
     * The format of the listing is host-specific.
     **/
    String[] dir() throws FtpError, IOException;

    /**
     * Get a verbose listing of the specified remote path.
     * The format of the listing is host-specific.
     * @param path directory and/or name of file(s) to list.
     *        May include wildcards if the host supports it.
     **/
    String[] dir(String path) throws FtpError, IOException;
    
    /**
     * Get a list of file names in the current remote directory.
     * @param path directory and/or name of file(s) to list.
     *        May include wildcards if the host supports it.
     **/
    String[] list() throws FtpError, IOException;

    /**
     * Get a list of file names in the specified remote path.
     * @param path directory and/or name of file(s) to list.
     *        May include wildcards if the host supports it.
     **/
    String[] list(String path) throws FtpError, IOException;

    /**
     * Get size of remote file.
     * @param file name of remote file
     * @return the number of bytes needed to transfer in order to
     *         get the remote file, or -1 if the size could not be obtained.
     **/
    public long size(String file) throws FtpError, IOException;

    /**
     * Get modification time of remote file.
     * @param file name of remote file
     * @return modification time of <i>file</i> expressed as UTC (GMT)
     *         or null if the time could not be obtained.
     **/
    public Date time(String file) throws FtpError, IOException;
    
    /**
     * Change remote directory.
     **/
    void cd(String dir) throws FtpError, IOException;

    /**
     * Get current remote directory.
     **/
    String pwd() throws FtpError, IOException;

    /** Transfer type <code>ASCII</code> **/
    char ASCII = 'A';

    /** Transfer type <code>IMAGE</code> (binary) **/
    char IMAGE = 'I';

    /** Transfer type <code>IMAGE</code> (binary) **/
    char BINARY = 'I';

    /**
     * Set transfer type.
     * @param transferType desired transfer type, must be <code>ASCII</code>
     * or <code>IMAGE</code>.
     **/
    void setType(char transferType) throws FtpError, IOException;

    /**
     * Get the active transfer type.
     * @return transfer type, <code>ASCII</code> or <code>IMAGE</code>.
     **/
    char getType();

    /**
     * Get remote file. The local file will have the same name as the remote
     * file and be created in the current directory.
     * @param remoteFile name of remote file to get
     **/
    void get(String remoteFile) throws FtpError, IOException;

    /**
     * Get remote file.
     * @param remoteFile name of remote file to get
     * @param localFile name of local file to create.
     *        This may be a relative or absolute path.
     **/
    void get(String remoteFile, String localFile) throws FtpError, IOException;

    /**
     * Put remote file. If the file already exists, it will be overwritten.
     * @param localFile name of local file to transfer
     **/
    void put(String localFile) throws FtpError, IOException;

    /**
     * Put remote file and possibly append.
     * If the file already exists, it will be overwritten.
     * @param localFile name of local file to transfer
     * @param append if true, localFile will be appended to
     *        the host file (if it exists)
     **/
    void put(String localFile, boolean append) throws FtpError, IOException;
    
    /**
     * Check if current transfer is restartable.
     * @return true if restart can be attempted
     **/
    boolean isRestartable();

    /**
     * Restart current (failed) transfer (if possible).
     **/
    void restart() throws IOException, FtpError;

    /**
     * Rename remote file.
     * @param from old name
     * @param to new name
     **/
    void rename(String from, String to) throws FtpError, IOException;

    /**
     * Delete remote file.
     * @param filename name of file to delete
     **/
    void delete(String filename) throws FtpError, IOException;

    /**
     * Create remote directory.
     * @param directory name of directory to create
     **/
    void mkdir(String directory) throws FtpError, IOException;
    
    /**
     * Remove remote directory.
     * @param directory name of directory to remove
     **/
    void rmdir(String directory) throws FtpError, IOException;

    /**
     * Abort current command.
     * This command will request the host to abort the current transfer.
     * If the transfer is still active, it will be terminated, and a
     * new command can be sent.
     **/
    void abort() throws FtpError, IOException;

    /**
     * Get the reply to the last command.
     **/
    FtpReply getReply();

}

