package com.ibm.nzna.shared.gui ;  
/*-----------------------------------------------------------*/
/*   ValueLabel                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ValueLabel                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
/**
 * This ValueLabel is a combination of a JLabel and a Compoenent
 * set side by side. The JLabel 
 * on the left side is the description of the value. The Component
 * on the right side is the value itself. This control is useful when
 * displaying information like 'Indicator: 123' where you want 123
 * in a different color than Indicator.<br><br>
 *
 * This control is also very useful if you want to display an 
 * entry field and a description of what the entry field is, for
 * example: "Name: ________"
 *
 *
 * @version 1.0, 5/1/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ValueLabel                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ValueLabel extends DPanel {

   /**
     *  This Dimension holds the preferred and minimum size
     **/
   private Dimension prefSize = new Dimension ( 100, 18 ) ;

   /**
     *  This is the JLabel which holds the description
     **/
   private JLabel description = null ;

   /**
     *  This is the component that is being displayed next
     * to the description
     **/
   private Component component = null ;

   /**
     *  This is the length of the Description field in
     * pixels
     **/
   private int labelLength = 50 ;


   /**
     *  This Constructor creates a ValueLabel with
     * nothing but the description of the value
     **/
   public ValueLabel ( String descriptionText ) {
      description = new JLabel ( descriptionText ) ;

      setLayout ( null ) ;
      add ( description ) ;
   }


   /**
     *  This constructor creates a ValueLabel with
     * the specified description and the component
     * passed in
     **/
   public ValueLabel ( String description, Component component ) {
      this ( description ) ;
      setComponent ( component ) ;
   }

   /**
     *  This constructor creates a ValueLabel with
     * the specified description and the component
     * passed in with the width of the description
     * JLabel in pixels set
     **/
   public ValueLabel ( String description, Component component, int descriptionWidth ) {
      this ( description, component ) ;
      setDescriptionWidth ( descriptionWidth ) ;
   }


   /**
     *  This method will set what component is currently
     * being displayed next to the description
     **/
   public void setComponent ( Component component ) {
      if ( this.component != null )
         remove ( component ) ;
      this.component = component ;
      add ( component ) ;
      revalidate() ;
   }

   /**
     *  This method will set the description that is shown
     **/
   public void setText ( String text ) {
      description.setText ( text ) ;
   }

   /**
     *  This method will set the font that the description
     * is drawn in
     **/
   public void setFont ( Font font ) {
      if ( description != null )
         description.setFont ( font ) ;
   }

   /**
     *  This method will set the color of the font that the
     * description is drawn in
     **/
   public void setForeground ( Color color ) {
      if ( description != null )
         description.setForeground ( color ) ;
   }

   /**
     *  This method will set the background color of the description
     **/
   public void setBackground ( Color color ) {
      super.setBackground ( color ) ;
      if ( description != null )
         description.setBackground ( color ) ;
   }

   /**
     *  This method will set how wide the description JLabel will be
     * in pixels
     **/
   public void setDescriptionWidth ( int descriptionWidth ) {
      this.labelLength = descriptionWidth ;
   }


   /**
     *  This method will lay out and position all of our components
     **/
   public void doLayout () {
      Dimension size = getSize() ;
      super.doLayout ();

      description.setBounds ( 0, 0, labelLength, size.height ) ;

      if ( component != null ) {
         int width = 0;//component.getPreferredSize().width ;

         //if ( ( width+labelLength ) > size.width )
         width = size.width-labelLength ;

         component.setBounds ( labelLength, 0, width, size.height ) ;
      }
   }

   /**
     *  This method will return the component that we are displaying
     * to the right of the description
     **/
   public Component getComponent () {
      return( component ) ;
   }

   /**
     *  This method will return our preferred Size
     **/
   public Dimension getPreferredSize () {
      if ( component == null )
         return( description.getPreferredSize() ) ;

      else if ( component instanceof JComboBox )
         return( new Dimension ( component.getPreferredSize().width, 18 ) ) ;

      return( component.getPreferredSize() ) ;
   }

   /**
     *  This method will return the minimum size
     **/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }

   /**
     *  this method will return the JLabel that is being
     * used to display the description
     **/
   public JLabel getJLabel () {
      return( description ) ;
   }


   /**
     *  This method will only work if the Component
     * used to display the value responds to the 'setText'
     * method. However, this method basically calls 'setText ( text )'
     * on the component displaying the value. Currently this
     * method only sets text on a JLabel, JTextArea, JTextField or
     * Abstract Button
     **/
   public void setValue ( String text ) {
      if ( component != null ) {
         if ( component instanceof JTextField )
            ((JTextField)component).setText ( text ) ;
         else if ( component instanceof JTextArea )
            ((JTextArea)component).setText ( text ) ;
         else if ( component instanceof JLabel )
            ((JLabel)component).setText ( text ) ;
         else if ( component instanceof JButton )
            ((JButton)component).setText ( text ) ;
      }
   }


   /**
     *  This method will return the data in the control we are displaying. 
     **/
   public Object getValue () {
      Object retObject = null ;

      if ( component != null ) {

         if ( component instanceof JComboBox ) {
            retObject = ((JComboBox)component).getSelectedItem() ;
         } 
         
         else if ( component instanceof JTextArea ) {
            retObject = ((JTextArea)component).getText() ;
         }

         else if ( component instanceof JTextField ) {
            retObject = ((JTextField)component).getText() ;
         } 
         else if ( component instanceof JList ) {
            retObject = ((JList)component).getSelectedValue() ;
         } 

      }
      return( retObject ) ;
   }
} /* ValueLabel */
