package com.ibm.nzna.shared.servlet ;   

import javax.servlet.* ;

import javax.servlet.http.* ;

import java.io.* ;

import java.net.* ;

import java.util.* ;

import com.ibm.nzna.shared.util.* ;

import com.ibm.nzna.shared.servlet.* ;

import com.ibm.nzna.shared.mq.* ;

import com.ibm.nzna.shared.lang.* ;

   /*********************************************************

   * Provides the logic behind session states.  This class

   * interprets what data is to be stored from each request.

   * This class is integrates the session manager with the

   * session state.  The purpose of this class is to keep the

   * session state object lightweight since it will be passed

   * between many services and applications.

   *

   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.

   * Created: 3/28/100  7:35pm

   * @author Chad Meadows

   * @version 1.0

   *********************************************************/

public class SessionTracker {

   private SessionManager  sessionManager       = null ;

   private String          sessionTrackingKey   = null ;

   private String          sessionSequenceKey   = null ;

   private boolean         sessionCookieEnabled = false ;

   /*********************************************************

   * Constructor for the session Tracker.

   *

   * Author:  Chad Meadows

   * Created: 4/25/100  7:11pm

   * @param

   * @return

   *********************************************************/

   public SessionTracker ( SessionManager sessionManager , String sessionTrackingKey, boolean sessionCookieEnabled) {

      this.sessionManager        = sessionManager ;

      this.sessionTrackingKey    = sessionTrackingKey ;

      this.sessionCookieEnabled  = sessionCookieEnabled ;

   }

   /*********************************************************

   * Stores the data for this request from a servlet into the

   * session state object.

   *

   * Author:  Chad Meadows

   * Created: 4/25/100  7:11pm

   * @param

   * @return

   *********************************************************/

   private void setRequest( SessionState sessionState, HttpServletRequest req  ) {

      /* Get the request object from the session state */

      HttpRequest request = sessionState.getRequest() ;

      /* Set the http headers for the sessionState object */

      Hashtable headersHash = new Hashtable() ;

      /* set default sequence number for new sessions */

      int       sequence    = 1 ;



      /* Set the current http request information */

      request.setServletRequest( req ) ;



      /* Set the sequence number of this request */

      String sequenceNum = getSequenceId( req ) ;

      if ( sequenceNum != null ) {

         /* Get the integer value */

         sequence    = Integer.parseInt( sequenceNum ) ;

         /* Increment the sequence number */

         sequence++ ;

      }

      /* Set the sequence number in the session state */

      sessionState.setSequence ( sequence ) ;

   }

   /*********************************************************

   * Gets the session state for this servlet.  The session

   * state is stored by the session manager assigned to this

   * servlet in the properties file.  If cookie tracking is

   * enabled the session id key will be stored by cookie value.

   * Otherwise the cookie id will be stored in the URL.  

   * This method also sets the http headers in the session

   * state object.

   *

   * Author:  Chad Meadows

   * Created: 3/29/100  2:39pm

   * @param

   * @return

   *********************************************************/

   public SessionState getSessionState( HttpServletRequest req, HttpServletResponse resp ) throws StandardException {

      Cookie         cookie      = null ;

      String         keyValue    = null ;

      SessionState   sessionState= null ;

      /* Use a cookie id if enabled */

      if ( sessionCookieEnabled ) {

         /* Get the array of cookies */

         Cookie cookies[] = req.getCookies() ;

         /* Search array for our cookie value */

         for ( int cookieCount = 0 ; cookieCount < cookies.length; cookieCount ++ ) {

            /* Does the name match our key name */

            if (cookies[ cookieCount ].getName().equals( sessionTrackingKey )) {

               /* We found the cookie value */

               cookie = cookies[ cookieCount ] ;

               keyValue = cookie.getValue() ;

               break ;

            }

         }

      }

      /* Check the URL for the session id */

      else {

         if ( sessionTrackingKey != null) {

            keyValue = getSessionId( req ) ;

         }

         else {

            System.out.println( "SessionTracker:  sessionTrackingKey not defined" ) ;

         }

      }



      /* Get the sessionState object from the session manager */

      if ( keyValue != null ) {

         /* Make 3 attempts to get the message */

         for ( int attempt = 0; attempt < 3 ; attempt ++) {

            try {

               sessionState = (SessionState) sessionManager.get( keyValue ) ;

               if ( sessionState != null ) {

                  /* Session state found */

                  break ;

               }

            }

            catch ( Exception e ) {

               e.printStackTrace() ;

            }

         }

         /* IF we were unable to get the session state throw an exception */

         if ( sessionState == null ) {

            throw ( new StandardException ("Unable to get session state", -2 )) ;

         }

      }

      /* This must be the first transaction of the session */

      /* Create a new session state object for this session*/

      /* Or the above was unable to get the last session   */

      else {

         /* This new session state must be assigned an Id */

         /* used to track the session.  The id must be    */

         /* stored using a cookie or passed in the URL    */

         sessionState = new SessionState( getUniqueKey(), req ) ;

      }

      /* Store the request data in the session state */

      setRequest( sessionState , req) ;



      /* Store the response object in the session state */

      sessionState.getRequest().setServletResponse( resp ) ;



      return ( sessionState ) ;

   }

   /*********************************************************

   * Store the session state object in the session manager

   * using the key specified in the session state object.

   * 

   *

   * Author:  Chad Meadows

   * Created: 5/2/100  1:25pm

   * @param sessionState A SessionState objet.

   * @return

   *********************************************************/

   public void putSessionState ( SessionState sessionState ) {

      sessionManager.put ( sessionState.getId() , sessionState ) ;

   }

   /*********************************************************

   * Generates a unique key identifier for the session object.

   * 

   *

   * Author:  

   * Created: 5/2/100  3:35pm

   * @param

   * @return

   *********************************************************/

   private String getUniqueKey() {

      /* Generate a random key value */

      Random random = new Random() ;

      /* Return a string of the long value */

      return ( new Long(random.nextLong()).toString() ) ;

   }

   /*********************************************************

   * Gets the session id from the request parameter

   * 

   *

   * Author:  

   * Created: 5/24/100  9:48pm

   * @param

   * @return

   *********************************************************/

   private String getSessionId( HttpServletRequest req ) {

      String sessionId = null ;

      /* Get the session id from the request */

      String keyValue = req.getParameter(sessionTrackingKey) ;

      if ( keyValue != null ) {

         /* Separate the session id from the sequence number */

         StringTokenizer token = new StringTokenizer ( keyValue, "," ) ;

         /* Get the session id */

         sessionId = (String) token.nextElement() ;

      }

      return ( sessionId ) ;

   }

   /*********************************************************

   * Gets the sequence id from the request parameter

   * 

   *

   * Author:  

   * Created: 5/24/100  9:48pm

   * @param

   * @return

   *********************************************************/

   private String getSequenceId( HttpServletRequest req ) {

      String sequenceId = null ;

      /* Get the session id from the request */

      String keyValue = req.getParameter(sessionTrackingKey) ;

      if ( keyValue != null) {

         /* Separate the session id from the sequence number */

         StringTokenizer token = new StringTokenizer ( keyValue, "," ) ;

         /* move past the session id */

         token.nextElement() ;

         /* Check to see if the id is here */

         if ( token.hasMoreElements() ) {

            sequenceId = (String) token.nextElement() ;

         }

      }

      return ( sequenceId ) ;

   }

}

