package com.ibm.nzna.shared.servlet.filter ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.servlet.* ;
import java.util.* ;
   /*********************************************************
   * Allow filter chaining using the ViewPortFilter class.
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/19/100  3:53pm
   * @author 
   * @version
   *********************************************************/
public class ContentFilterManager {
   /* Logging module */
   LogModule             log                    = null ;

   private Vector filterChain = new Vector() ;
   /* The basekey name of the filters */
   final static String   localBasePropertyName  = "local.servlet.ViewPortFilter.classname" ;
   final static String   globalBasePropertyName = "global.servlet.ViewPortFilter.classname" ;
   final static String   sFilterManagerEnabled  = "servlet.filtermanager.enabled" ;

   /*********************************************************
   * The constructor takes a property object and builds a chain
   * based on the property values.  The properies contain class
   * names of the filters to load.  The filters and then loaded
   * and placed into a chain.
   *
   * Author:  
   * Created: 4/19/100  3:54pm
   * @param properties The property object used for initialization.
   * @return
   *********************************************************/
   public ContentFilterManager ( Properties properties ) {
      /* Get the logging module */
      log = LogManager.getLogModule ( properties ) ;

      /* We must find the filter references in this property object */
      /* We first add any local filters to the filter chain         */
      addFiltersToChain( properties, localBasePropertyName ) ;
      /* We must find the filter references in this property object */
      /* Next, we add the global filters to the end of the chain    */
      addFiltersToChain( properties, globalBasePropertyName ) ;

   }
   /*********************************************************
   * Loads filter classes from the base key name specified.
   * The properties should be in consecutive sequence of
   * the order you wish them to execute 
   * <PRE>
   * local.servlet.ViewPortFilter.class=com.ibm.nzna.shared.servlet.filters.body    \
   *                                    com.ibm.nzna.shared.servlet.filters.top     \
   *                                    com.ibm.nzna.shared.servlet.filters.bottom
   * </PRE>
   *
   * Global properties are specified in the Global.properties file.
   * the global filters are added to the end of any existing filter chain
   *
   * This will create a filter chain which is invoked by the order
   * of the numeral in the key of the property.  All numeral must
   * be in consecutive order.
   *
   * Author:  
   * Created: 4/26/100  5:38pm
   * @param properties The property object to get the key values from 
   *                   for the filter chains
   * @param baseName   The base name of the key by which the chain is
                       grouped by.
   * @return
   *********************************************************/
   private void addFiltersToChain ( Properties properties , String baseName) {
      /* Flag for finding more filters */
      boolean  moreFilters          = true ;
      /* The numerical number at the end of the key name */
      int      filterCount          = 1 ;
      /* Flag for filter manager enabled */
      boolean  filterManagerEnabled = false ;

      /* If this property is enabled the filter manager        */
      /* is active, otherwise the filter manager is disabled   */
      String value = properties.getProperty( sFilterManagerEnabled , "false" ) ;
      if (value.equalsIgnoreCase("true")) {
         filterManagerEnabled = true ;            
      }

      if ( filterManagerEnabled ) {
         /* Get the classnames of this filter from the properties */
         String classnames = (String) properties.get( baseName ) ;
         /* Determine if the property was found */
         if ( classnames != null ) {
            /* Load each of the classes listed in this property */
            StringTokenizer tokenizer = new StringTokenizer( classnames ) ;
            while ( tokenizer.hasMoreElements() ) {
               /* Get the classname from the string */
               String classname = tokenizer.nextToken() ;
               try {
                  /* Load the class for this filter */
                  Class contentFilterClass = Class.forName( classname ) ;
                  /* Create a new instance of this class */
                  ContentFilter contentFilter = (ContentFilter) contentFilterClass.newInstance() ;
                  log.writeLog("ContentFilterManager:  Loaded class: "+classname ) ;
                  /* Initialize the filter interface */
                  contentFilter.init( properties ) ;
                  /* Add the filter to the filter chain */
                  filterChain.addElement( contentFilter ) ;
               }
               catch ( Exception e ) {
                  log.writeLog( "ContentFilterManager:  Unable to Load class: "+classname) ;
                  e.printStackTrace() ;
               }
            }
         }
      }
   }
   /*********************************************************
   * Invokes the filter chain.  All filters in the vector are
   * called in order with the output of each feeding the input
   * of the next and the final result being returned.
   * 
   *
   * Author:  
   * Created: 4/19/100  4:54pm
   * @param html The input to be filtered.
   * @param sessionState The sessionState object for the servlet.
   * @return
   *********************************************************/
   public String filter ( String html , SessionState sessionState ) {
      /* Get the enumeration of the filter chain vector */
      Enumeration enum = filterChain.elements() ;
      /* Filter the html through each filter in the chain */
      while ( enum.hasMoreElements() ) {
         /* Get each filter object */
         ContentFilter contentFilter = (ContentFilter) enum.nextElement() ;
         /* Log the start of the filter */
         log.writeLog( "ContentFilterManager: executing filter - " + contentFilter , 10) ;
         /* Filter the data through this filter object */
         html = contentFilter.filter( html, sessionState ) ;
      }
      /* return the filtered output */
      return ( html ) ;
   }
}
